#    DamBASE - Dam Behaviour Analysis Software Environment
#    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
#    Vetsch D.F., 2015
#    http://www.dambase.ethz.ch
#
#    This file is part of DamBASE
#
#    DamBASE is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    DamBASE is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

#
dambase.T1 <-
  function(x, container) {
    x_n <- normalizeInputForCheby(x, container)
    return(x_n) #DO NOT USE THE POLYNOMS FROM chebyshev.s.polynomials of package orthopolynom. They seem to be not appropriate for this context
  }

dambase.T2 <-
  function(x, container) {
    x_n <- normalizeInputForCheby(x, container)
    return(2*x_n^2 - 1) #DO NOT USE THE POLYNOMS FROM chebyshev.s.polynomials of package orthopolynom. They seem to be not appropriate for this context
  }

dambase.T3 <-
  function(x, container) {
    x_n <- normalizeInputForCheby(x, container)
    return(4*x_n^3 - 3*x_n) #DO NOT USE THE POLYNOMS FROM chebyshev.s.polynomials of package orthopolynom. They seem to be not appropriate for this context
  }

dambase.T4 <-
  function(x, container) {
    x_n <- normalizeInputForCheby(x, container)
    return(8*x_n^4 - 8*x_n^2 + 1) #DO NOT USE THE POLYNOMS FROM chebyshev.s.polynomials of package orthopolynom. They seem to be not appropriate for this context
  }

dambase.T5 <-
  function(x, container) {
    x_n <- normalizeInputForCheby(x, container)
    return(16*x_n^5 - 20*x_n^3 + 5*x_n) #DO NOT USE THE POLYNOMS FROM chebyshev.s.polynomials of package orthopolynom. They seem to be not appropriate for this context
  }

# normalize input for chebyshev polynoms as described by Benedikt Weber's statistics paper for dam behavior prediction
# formula 5.4
normalizeInputForCheby <-
  function(x, container) {
    min_x <- min(x)
    max_x <- max(x)
    
    if(exists("phase",envir=baseenv()) && get(x="phase", envir=baseenv()) == "p") {
      min_x <- get(x="min", envir=container)
      max_x <- get(x="max", envir=container)
      message("phase: prediction min_x(", min_x, "), max_x(", max_x, ")")
    }else if(exists("phase",envir=baseenv()) && get(x="phase", envir=baseenv()) == "c"){
      message("phase: calibration min_x(", min_x, "), max_x(", max_x, ")")
      assign("min", value=min_x, envir=container)
      assign("max", value=max_x, envir=container)
    }
    
    w <- (2*x - max_x - min_x)/(max_x - min_x)
    return(w)
}