#    DamBASE - Dam Behaviour Analysis Software Environment
#    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
#    Vetsch D.F., 2015
#    http://www.dambase.ethz.ch
#
#    This file is part of DamBASE
#
#    DamBASE is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    DamBASE is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

#
# regrValues is expected to be values out of a water level pre-calculation such as T1, h^2 and so on
dambase.creep <- function (wlevelPreCalculated, dateColumn, creepConstant, convolutionTime) {
  days <- c(dateColumn[1:length(dateColumn)] - dateColumn[1])
  t <- days

  # first, kernel needs to be calculated for every day
  if (convolutionTime == 0) kernel.length <- t[length(wlevelPreCalculated)]-t[1]+1 else
                            kernel.length <- convolutionTime + 1;
  
  kernel <- rep(0, kernel.length)
  
  for(i in 1:kernel.length) {
    kernel[i] <- dambase.creep.kernel(i-1, creepConstant)
  }

  
  # second, calculate convolution with pre calculated water level, days and the kernel
  return(dambase.convolution(wlevelPreCalculated,t,kernel))
}

dambase.creep.kernel <- function(t, creepConstant) {
  if (t>0) return(exp(-creepConstant*t)) else
           return(0)
}