#    DamBASE - Dam Behaviour Analysis Software Environment
#    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
#    Vetsch D.F., 2015
#    http://www.dambase.ethz.ch
#
#    This file is part of DamBASE
#
#    DamBASE is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    DamBASE is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

#
dambase.delay <- 
function(column, timesteps) {
  if(timesteps < 0) {
    stop("dambase.delay: negative timesteps are not allowed")
  }
  if(timesteps > length(column)) {
    stop("dambase.delay: number of timesteps is higher than the length of the column");
  }
  
  delayed <- column
  n <- timesteps
  
  #first, fill vector with first value of the column n times
  fill <- rep(column[1], n)
  delayed <- c(fill, column)
  delayed <- delayed[1:length(column)]
  
  return(delayed)
}



#this code has been derive from damreg written by Dr.Benedikt Weber
dambase.average <-
function(dateColumn, column, averageTimesteps) {
  if(averageTimesteps == 0) {
    return(column)
  }
  x <- column
  days <- c(dateColumn[1:length(dateColumn)] - dateColumn[1])
  

  n <- length(x)
  tmp <- rep(0, n)
  tmp[1] = x[1]
  
  for(i in 1:(n-1)) {
    time <- 0
    sum <- 0
    f1 <- 0
    f2 <- x[i+1]
    for(k in i:1) {
      time <- days[i+1] - days[k-1+1]
      f1 <- f2
      f2 <- x[k-1+1]
      if(is.na(time)) {
        stop("time = NA ", i)
      }
      if(time >= averageTimesteps) {
        # use interpolation
        dt <- days[k+1]-days[k-1+1]
        dt1 <- averageTimesteps - time + dt
        f2 <- f1 + (f2-f1)/dt*dt1
        sum <- sum + (f1+f2)* 0.5 * dt1
        time <- averageTimesteps
        break
      }
      sum <- sum + (f1+f2)* 0.5 * (days[k+1]-days[k-1+1])
    }
    tmp[i+1]= sum/time
  }
  
  return(tmp)
}
