#    DamBASE - Dam Behaviour Analysis Software Environment
#    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
#    Vetsch D.F., 2015
#    http://www.dambase.ethz.ch
#
#    This file is part of DamBASE
#
#    DamBASE is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    DamBASE is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

#
dambase.seasonal.sin <-
  function(dateColumn, factor=1) {
    w <- 2*pi*yday(dateColumn)/365.25
    z <- sin(factor*w)
    
    return(z)
  }

dambase.seasonal.cos <-
  function(dateColumn, factor=1) {
    w <- 2*pi*yday(dateColumn)/365.25
    z <- cos(factor*w)
    
    return(z)
  }


#this code has been derive from dambase 1.0 written by Dr.Benedikt Weber
dambase.temp.thermal.kernel <-
  function(t, depth, thermalDiffusivity) {
    xa <- depth/sqrt(thermalDiffusivity)
    if(t>0) {
      return(xa/sqrt(4*pi*t*t*t)*exp(-xa*xa/4/t))
    }
    return(0)
}

#this code has been derive from dambase 1.0 written by Dr.Benedikt Weber
dambase.temp.thermal.oned <-
 function(dateColumn, column, depth, thermalDiffusivity, convolutionTime) {
    if(depth < 0) {
      stop("depth must be larger or equal to 0")
    }
    
    if(depth == 0) {
      return(column)
    }
    
    days <- c(dateColumn[1:length(dateColumn)] - dateColumn[1])
    
    t <- days
    temperatureMean <- mean(column)
    nker = NULL
    if(convolutionTime == 0) nker <- t[length(t) - 1]-t[1]+1
    else nker <- convolutionTime+1;
    
    kernel = NULL
    i <- 0;
    while(i < nker) {
      kernel[i+1] <- dambase.temp.thermal.kernel(i, depth, thermalDiffusivity)
      i <- i+1
    }

    #TODO: do checkResolution
    return(dambase.convolution(column-temperatureMean, t, kernel))
}




# dambase.temp.thermal.oned <-
#   function(dateColumn, column, depth, thermalDiffusivity, convolution) {
#     dateColumn <- as.Date(dateColumn, "%d.%m.%Y")
#     days <- c(dateColumn[1:length(dateColumn)] - dateColumn[1])
#     
#     message("number of days: ", length(days))
#     
#     t <- days[length(days)] #last timestep
#     
#     calculatedTemps = NULL
#     for(i in 1:length(days)-1) { #integrate from 0 to t
#       tau <- days[i]
#       temp <- column[i]
#       
#       currentValue <- temp *
#         dambase.temp.thermal.oned.g(depth, t - tau, thermalDiffusivity)
#       
#       if(i == 1) {
#         calculatedTemps[i] = currentValue
#       }else{
#         calculatedTemps[i] = calculatedTemps[i-1] + currentValue
#       }
#     }
#     
#     calculatedTemps[length(calculatedTemps) + 1] = 0
#     
#     return(calculatedTemps)
# }

# dambase.temp.thermal.oned.g <-
#   function(depth, timestep, thermalDiffusivity) {
#     t <- timestep
#     z <- depth
#     a <- thermalDiffusivity
#     value <- (z/(2*sqrt(pi*a))) * 
#       (exp(1)^((-1*(z^2)/(4*a*t))))/t^(3/2)
#     
#     return(value)
# }

# dambase.temp.thermal.oned.T <-
#   function(temp, depth, timestep, thermalDiffusivity, convolution) {
#     t <- timestep
#     z <- depth
#     omega <- 2*pi/convolution
#     d <- sqrt(2*thermalDiffusivity/omega)
#     value <- temp * exp((-1*z)/d)*sin(omega*t - z/d)
# 
#     return(value)
# }