#test.c2f <- function() {
#  checkEquals(c2f(0), 32)
#  checkEquals(c2f(10), 50)
#  checkException(c2f("xx"))
#}

.setUp <-
  function() {
    e <- parent.env(environment())
    dataModel <- read.csv(paste(file.path("test_cases"), "/Schlegeis_corr.csv", sep=""), sep=";")
    dataModel <- applyDateFormat(dataFrame=dataModel, dateColumnNr=1, dateFormat="%d.%m.%Y")
    
    dataModelMissingW <- read.csv(paste(file.path("test_cases"), "/Schlegeis_corr_missing_w.csv", sep=""), sep=";")
    dataModelMissingW <- applyDateFormat(dataFrame=dataModelMissingW, dateColumnNr=1, dateFormat="%d.%m.%Y")
    #print(head(dataModel))
    
    formula <- PENDULUM ~ I(scale(x=W_LEVEL, scale=FALSE)) + I(scale(x=W_LEVEL, scale=FALSE)^2)
    
    regressionFit.lm <- lm(formula, dataModel, x=TRUE)
    regressionFit.missingw.lm <- lm(formula, dataModelMissingW, x=TRUE)
    regressionFit.pwi <- loadRegressionFitLmPwi(dataFrame=dataModel, formula=formula)
    regressionFit.ri <- loadRegressionFitLmRidge(dataFrame=dataModel, formula=formula, k=0.06)
    
    e$containerT1 <- new.env()
    e$containerT2 <- new.env()
    e$containerT3 <- new.env()
    e$containerT4 <- new.env()

    formula <- PENDULUM ~ I(dambase.T1(W_LEVEL, e$containerT1)) + I(dambase.T2(W_LEVEL, e$containerT2)) + I(dambase.T3(W_LEVEL, e$containerT3)) + I(dambase.T4(W_LEVEL, e$containerT4)) + I(dambase.direct(T_H12_UP)) + I(dambase.direct(T_H12_MI_corr)) + I(dambase.direct(T_H12_DO)) + I(dambase.direct(T_H15_UP)) + I(dambase.direct(T_H15_MI)) + I(dambase.direct(T_H15_DO))
    regressionFit.pcr <- loadRegressionFitLmPcr(dataFrame=dataModel, formula=formula, r=10)
    
    
    
    e$dataModel <- dataModel
    e$dataModelMissingW <- dataModelMissingW
    e$regressionFit.lm <- regressionFit.lm
    e$regressionFit.missingw.lm <- regressionFit.missingw.lm
    e$regressionFit.pwi <- regressionFit.pwi
    e$regressionFit.ri <- regressionFit.ri
    e$regressionFit.pcr <- regressionFit.pcr
}

test.predictNewValues.lm <- function() {
  checkEqualsNumeric(c(53.88383, 53.85108, 53.81181, 53.70073), predictNewValues(regressionFit.lm, dataModel)[1:4], tolerance=0.001)
}

test.predictNewValues.missingw.lm <- function() {
  values <- predictNewValues(regressionFit.missingw.lm, dataModelMissingW)
#   cat("test.predictNewValues.missingw.lm : ", values[1:4])
}

test.predictNewValues.Pwi <- function() {
  DEACTIVATED()
  #cat("predictNewValues.pwi : ", predictNewValues(regressionFit.pwi, dataModel)[1:4], "\n")
  checkEqualsNumeric(c(58.68916, 58.6477, 58.59799, 58.45738), predictNewValues(regressionFit.pwi, dataModel)[1:4], tolerance=0.001)
}

test.predictNewValues.Ridge <- function() {
  #cat("predictNewValues.ridge : ", predictNewValues(regressionFit.ri, dataModel)[1:4], "\n")
  checkEqualsNumeric(c(53.28362, 53.2551, 53.22089, 53.12412), predictNewValues(regressionFit.ri, dataModel)[1:4], tolerance=0.001)
}

test.predictNewValues.Pcr <- function() {
  #cat("predictNewValues.pcr : ", predictNewValues(regressionFit.pcr, dataModel)[1:4], "\n")
  checkEqualsNumeric(c(64.93644, 65.00195, 65.13717, 64.74956), predictNewValues(regressionFit.pcr, dataModel)[1:4], tolerance=0.001)
}

test.loadPraisWinsten <- function() {
  regressionFit.pwi <- loadRegressionFitLmPwi(dataFrame=dataModel, formula=PENDULUM ~ I(dambase.T1(W_LEVEL, containerT1)) + I(dambase.T2(W_LEVEL, containerT2)) + I(dambase.T3(W_LEVEL, containerT3)) + I(dambase.T4(W_LEVEL, containerT4)) + I(dambase.direct(T_H12_UP)) + I(dambase.direct(T_H12_MI_corr)) + I(dambase.direct(T_H12_DO)) + I(dambase.direct(T_H15_UP)) + I(dambase.direct(T_H15_MI)) + I(dambase.direct(T_H15_DO)))
#   print(regressionFit.pwi$coefficients)
#   message("rho: ", regressionFit.pwi$rho)
#   message("ssres: ", regressionFit.pwi$ssres)
#   message("msres: ", regressionFit.pwi$msres)
#   message("ssres.rho: ", regressionFit.pwi$ssres.rho)
#   message("msres.rho: ", regressionFit.pwi$msres.rho)
#   cat("t_values:", regressionFit.pwi$t_values, "\n")
#   cat("p_values: ", regressionFit.pwi$p_values, "\n")
#   cat("vifs: ", regressionFit.pwi$vifs, "\n")
#   message("adj.r.squared: ", regressionFit.pwi$adj.r.squared)
#   cat("std_errors: ", regressionFit.pwi$std_errors, "\n")
#   cat("std_coefficients: ", regressionFit.pwi$std_coefficients, "\n")
}

test.loadPraisWinstenTwoIterations <- function() {
  regressionFit.pwi <- loadRegressionFitLmPwi(iterations=2, dataFrame=dataModel, formula=PENDULUM ~ I(dambase.T1(W_LEVEL, containerT1)) + I(dambase.T2(W_LEVEL, containerT2)) + I(dambase.T3(W_LEVEL, containerT3)) + I(dambase.T4(W_LEVEL, containerT4)) + I(dambase.direct(T_H12_UP)) + I(dambase.direct(T_H12_MI_corr)) + I(dambase.direct(T_H12_DO)) + I(dambase.direct(T_H15_UP)) + I(dambase.direct(T_H15_MI)) + I(dambase.direct(T_H15_DO)))
  #print(regressionFit.pwi$coefficients)
  #message("rho: ", regressionFit.pwi$rho)
  #   message("ssres: ", regressionFit.pwi$ssres)
  #   message("msres: ", regressionFit.pwi$msres)
  #   message("ssres.rho: ", regressionFit.pwi$ssres.rho)
  #   message("msres.rho: ", regressionFit.pwi$msres.rho)
  #   cat("t_values:", regressionFit.pwi$t_values, "\n")
  #   cat("p_values: ", regressionFit.pwi$p_values, "\n")
  #   cat("vifs: ", regressionFit.pwi$vifs, "\n")
  #   message("adj.r.squared: ", regressionFit.pwi$adj.r.squared)
  #   cat("std_errors: ", regressionFit.pwi$std_errors, "\n")
  #   cat("std_coefficients: ", regressionFit.pwi$std_coefficients, "\n")
}


# Ridge regression
test.loadRidgeRegression <- function() {
  dimnames.results.withoutintercept <- c("I(scale(x = W_LEVEL, scale = FALSE))", "I(scale(x = W_LEVEL, scale = FALSE)^2)")
  dimnames.results.withintercept <- c("(Intercept)", dimnames.results.withoutintercept)
  
  coefficients.expected <- c(43.714337603, 0.433300964, 0.003602382)
  names(coefficients.expected) <- dimnames.results.withintercept
  
  vifs.expected <- c(1.334436, 1.334436)
  names(vifs.expected) <- dimnames.results.withoutintercept
  
  tvalues.expected <- c(NaN, 62.34664, 15.73612)
  names(tvalues.expected) <- dimnames.results.withintercept
  
  pvalues.expected <- c(NaN, 0.000000e+00, 2.522285e-53)
  names(pvalues.expected) <- dimnames.results.withintercept
  
  checkEquals(coefficients.expected, coefficients(regressionFit.ri), checkNames=TRUE, tolerance=0.001)
  checkEquals(vifs.expected, regressionFit.ri$vifs, checkNames=TRUE, tolerance=0.001)
  checkEquals(tvalues.expected, regressionFit.ri$t_values, checkNames=TRUE, tolerance=0.001)
  checkEquals(pvalues.expected, regressionFit.ri$p_values, checkNames=TRUE, tolerance=0.001)
  checkEqualsNumeric(140958.5, regressionFit.ri$ssres, tolerance=0.001)
  checkEqualsNumeric(55.19127, regressionFit.ri$msres, tolerance=0.001)
  checkEqualsNumeric(c(12.31638, 12.44490, 12.37911), regressionFit.ri$residuals[1:3], tolerance=0.001)
}

# Principal component regression
test.loadPrincipalComponentRegression <- function() {
#   print("values:")
#   print(regressionFit.pcr$residuals[1:4])
#   print(regressionFit.pcr$df.residual)
#   print(regressionFit.pcr$fitted.values[1:4])
  
  checkEqualsNumeric(c(39.85688821, 26.62990739, 8.03976627, 1.54904214, 0.56836845, -0.26168003, -1.33863064, -0.44038541, -0.02458048,  3.99711069, -1.70378112), coefficients(regressionFit.pcr), tolerance=0.001)
  checkEqualsNumeric(c(4.357509, 2.440124, 2.141246, 1.949046, 6.082299, 9.465645, 113.104263, 11.953194, 3.036636, 126.155838), regressionFit.pcr$vifs, tolerance=0.001)
  checkEqualsNumeric(c(NaN, 239.858675, 104.169472,  26.513886,  10.556545,  -8.072975, -16.225423, -6.009095,  -0.328773,  10.629435, -19.853984), regressionFit.pcr$t_values, tolerance=0.001)
  checkEqualsNumeric(c(NaN,  0.000000e+00,  0.000000e+00, 5.413350e-137,  1.583005e-25, 1.046918e-15,  2.045456e-56,  2.132694e-09,  7.423543e-01,  7.507560e-26, 1.126749e-81), regressionFit.pcr$p_values, tolerance=0.001)
  checkEqualsNumeric(0.01145608, regressionFit.pcr$adj.r.squared, tolerance=0.001)
  checkEqualsNumeric(c(1.92629177, 0.11102332, 0.07717968, 0.05842381, 0.05384039, 0.03241433, 0.08250205, 0.07328647, 0.07476431, 0.37604170, 0.08581558), regressionFit.pcr$std_errors, tolerance=0.001)
  checkEqualsNumeric(399097.2, regressionFit.pcr$sst, tolerance=0.001)
  checkEqualsNumeric(1.788767, regressionFit.pcr$msres, tolerance=0.001)
  checkEqualsNumeric(4554.2, regressionFit.pcr$ssres, tolerance=0.001)
  checkEqualsNumeric(c(0.6635643, 0.6980494, 0.4628322, 0.8504407),regressionFit.pcr$residuals[1:4], tolerance=0.001)
  checkEqualsNumeric(2546, regressionFit.pcr$df.residual, tolerance=0.001)
  checkEqualsNumeric(c(64.93644, 65.00195, 65.13717, 64.74956),regressionFit.pcr$fitted.values[1:4], tolerance=0.001)
}

#######
# loadRegressionFitResidualsVersusFitted
#######
# Lm
test.loadRegressionFitResidualsVersusFitted.Lm.NoStudentizedResiduals <- function() {
  result <- loadRegressionFitResidualsVersusFitted(regressionFit=regressionFit.lm, externallyStudentizedResiduals=FALSE)
#   print(result$parametervalues[1:4])
#   print(result$residuals[1:4])
  
  checkEquals(2557, length(result$parametervalues))
  checkEqualsNumeric(signif(c(53.88383, 53.85108, 53.81181, 53.70073), 4), signif(result$parametervalues[1:4], 4))
  checkEquals(length(result$parametervalues), length(result$residuals))
  checkEqualsNumeric(signif(c(11.71617, 11.84892, 11.78819, 11.89927), 4), signif(result$residuals[1:4], 4))
}

test.loadRegressionFitResidualsVersusFitted.missingw.Lm.NoStudentizedResiduals <- function() {
  values <- loadRegressionFitResidualsVersusFitted(regressionFit.missingw.lm, externallyStudentizedResiduals=FALSE) #dataModelMissingW)
  #cat("test.AloadRegressionFitResidualsVersusFitted.missingw.lm : ", values$parametervalues[1:4])
}

# PWI
test.loadRegressionFitResidualsVersusFitted.Pwi.NoStudentizedResiduals <- function() {
  DEACTIVATED()
  result <- loadRegressionFitResidualsVersusFitted(regressionFit=regressionFit.pwi, externallyStudentizedResiduals=FALSE)
#   print(result$parametervalues[1:4])
#   print(result$residuals[1:4])
  
  checkEquals(2557, length(result$parametervalues))
  checkEqualsNumeric(signif(c(3.52523196, 0.06451263, 0.05618501, -0.03481017), 4), signif(result$parametervalues[1:4], 4))
  checkEquals(length(result$parametervalues), length(result$residuals))
  checkEqualsNumeric(signif(c(0.41510762, 0.15393483, -0.03755699, 0.15325762), 4), signif(result$residuals[1:4], 4))
}

# PWI
test.loadRegressionFitResidualsVersusFitted.Pwi.WithStudentizedResiduals <- function() {
  DEACTIVATED()
  result <- loadRegressionFitResidualsVersusFitted(regressionFit=regressionFit.pwi, externallyStudentizedResiduals=TRUE)
#   print("test.loadRegressionFitResidualsVersusFitted.Pwi.WithStudentizedResiduals:")
#   print(result$parametervalues[1:4])
#   print(result$residuals[1:4])
  
  checkEquals(2557, length(result$parametervalues))
  checkEqualsNumeric(signif(c(3.52523196,  0.06451263,  0.05618501, -0.03481017), 4), signif(result$parametervalues[1:4], 4))
  checkEquals(length(result$parametervalues), length(result$residuals))
  checkEqualsNumeric(signif(c(1.4780600, 0.4575312, -0.1116242, 0.4555278), 4), signif(result$residuals[1:4], 4))
}

# Ridge
test.loadRegressionFitResidualsVersusFitted.Ridge.NoStudentizedResiduals <- function() {
  result <- loadRegressionFitResidualsVersusFitted(regressionFit=regressionFit.ri, externallyStudentizedResiduals=FALSE)
#   print(result$parametervalues[1:3])
#   print(result$residuals[1:3])
  
  checkEqualsNumeric(c(53.28362, 53.25510, 53.22089), result$parametervalues[1:3], tolerance=0.001)
  checkEqualsNumeric(c(12.31638, 12.44490, 12.37911), result$residuals[1:3], tolerance=0.001)
}

# Ridge
test.loadRegressionFitResidualsVersusFitted.Ridge.WithStudentizedResiduals <- function() {
  result <- loadRegressionFitResidualsVersusFitted(regressionFit=regressionFit.ri, externallyStudentizedResiduals=TRUE)
#   print(result$parametervalues[1:3])
#   print(result$residuals[1:3])
  
  checkEqualsNumeric(c(53.28362, 53.25510, 53.22089), result$parametervalues[1:3], tolerance=0.001)
  checkEqualsNumeric(c(1.658391, 1.675695, 1.666834), result$residuals[1:3], tolerance=0.001)
}

# PCR
test.loadRegressionFitResidualsVersusFitted.Pcr.NoStudentizedResiduals <- function() {
  result <- loadRegressionFitResidualsVersusFitted(regressionFit=regressionFit.pcr, externallyStudentizedResiduals=FALSE)
#   print(result$parametervalues[1:3])
#   print(result$residuals[1:3])
  
   checkEqualsNumeric(c(64.93644, 65.00195, 65.13717), result$parametervalues[1:3], tolerance=0.001)
   checkEqualsNumeric(c(0.6635643, 0.6980494, 0.4628322), result$residuals[1:3], tolerance=0.001)
}

# PCR
test.loadRegressionFitResidualsVersusFitted.Pcr.WithStudentizedResiduals <- function() {
  result <- loadRegressionFitResidualsVersusFitted(regressionFit=regressionFit.pcr, externallyStudentizedResiduals=TRUE)
#     print(result$parametervalues[1:3])
#     print(result$residuals[1:3])
  
  checkEqualsNumeric(c(64.93644, 65.00195, 65.13717), result$parametervalues[1:3], tolerance=0.001)
  checkEqualsNumeric(c(0.4978099, 0.5239292, 0.3472965), result$residuals[1:3], tolerance=0.001)
}

#######
# loadRegressionFitResidualTimeSeries
#######
test.loadRegressionFitResidualTimeSeries.Lm <- function() {
  regressionStartDate <- dataModel$DATE[1]
  result <- loadRegressionFitResidualTimeSeries(regressionFit=regressionFit.lm, regressionDataFrame=dataModel, regressionDateColumnNr=1, predictionBehaviorIndicatorColumnNr=2, predictionDataFrame=dataModel, predictionDateColumnNr=1, predictionStartDate=regressionStartDate, overallMaxStartupTime=0)
  #cat("predictionResiduals : ", result$predictionResiduals[1:4], "\n")
  #print(result$predictionDateValues[1:4])
  
  checkEquals(2557, length(result$residuals))
  checkEqualsNumeric(signif(c(11.71617, 11.84892, 11.78819, 11.89927), 4), signif(result$residuals[1:4], 4))
  checkEquals(length(result$dates), length(result$residuals))
  checkEquals(as.Date(c("1992-01-01", "1992-01-02", "1992-01-03", "1992-01-04")), result$dates[1:4],  checkNames=FALSE)
  
  #prediction part
  checkEqualsNumeric(c(11.71617, 11.84892, 11.78819, 11.89927), result$predictionResiduals[1:4], tolerance=0.001)
  checkEquals(as.Date(c("1992-01-01", "1992-01-02", "1992-01-03", "1992-01-04")), result$predictionDateValues[1:4], checkNames=FALSE)
  
}

test.loadRegressionFitResidualTimeSeries.missingw.Lm.NoStudentizedResiduals <- function() {
  regressionStartDate <- dataModelMissingW$DATE[1]
  values <- loadRegressionFitResidualTimeSeries(regressionFit.missingw.lm, regressionDataFrame=dataModelMissingW, regressionDateColumnNr=1, predictionBehaviorIndicatorColumnNr=2, predictionDataFrame=dataModelMissingW, predictionDateColumnNr=1, predictionStartDate=regressionStartDate, overallMaxStartupTime=0) #dataModelMissingW)
  #cat("test.AloadRegressionFitResidualTimeSeries.missingw.Lm.NoStudentizedResiduals : ", values$residuals[1:4])
}

test.loadRegressionFitResidualTimeSeries.Pwi <- function() {
  DEACTIVATED()
  regressionStartDate <- dataModel$DATE[1]
  result <- loadRegressionFitResidualTimeSeries(regressionFit=regressionFit.pwi, regressionDataFrame=dataModel, regressionDateColumnNr=1, predictionBehaviorIndicatorColumnNr=2, predictionDataFrame=dataModel, predictionDateColumnNr=1, predictionStartDate=regressionStartDate, overallMaxStartupTime=0)
  #cat("pwi: predictionResiduals : ", result$predictionResiduals[1:4], "\n")
  #print(result$predictionDateValues[1:4])
  
  checkEquals(2557, length(result$residuals))
  checkEqualsNumeric(signif(c(0.41510762, 0.15393483, -0.03755699, 0.15325762), 4), signif(result$residuals[1:4], 4))
  checkEquals(length(result$dates), length(result$residuals))
  checkEquals(as.Date(c("1992-01-01", "1992-01-02", "1992-01-03", "1992-01-04")), result$dates[1:4],  checkNames=FALSE)
  
  #prediction
     #in this settup, where the prediction starts at the regression start date, the predictionResiduals should correspond with the residuals. This is somehow weird!!!
  checkEqualsNumeric(c(6.910841, 7.052298, 7.002007, 7.142622), result$predictionResiduals[1:4], tolerance=0.001)
  checkEquals(as.Date(c("1992-01-01", "1992-01-02", "1992-01-03", "1992-01-04")), result$predictionDateValues[1:4],  checkNames=FALSE)
}

test.loadRegressionFitResidualTimeSeries.Ridge <- function() {
  regressionStartDate <- dataModel$DATE[1]
  result <- loadRegressionFitResidualTimeSeries(regressionFit=regressionFit.ri, regressionDataFrame=dataModel, regressionDateColumnNr=1, predictionBehaviorIndicatorColumnNr=2, predictionDataFrame=dataModel, predictionDateColumnNr=1, predictionStartDate=regressionStartDate, overallMaxStartupTime=0)
#   print(result$dates[1:4])
#   print(result$residuals[1:4])
#   print(length(result$residuals))
  
  checkEquals(2557, length(result$residuals))
  checkEqualsNumeric(c(12.31638, 12.44490, 12.37911, 12.47588), result$residuals[1:4], tolerance=0.001)
  checkEquals(as.Date(c("1992-01-01", "1992-01-02", "1992-01-03", "1992-01-04")), result$dates[1:4],  checkNames=FALSE)
  
  #prediction
  checkEqualsNumeric(c(12.31638, 12.44490, 12.37911, 12.47588), result$predictionResiduals[1:4], tolerance=0.001)
  checkEquals(as.Date(c("1992-01-01", "1992-01-02", "1992-01-03", "1992-01-04")), result$predictionDateValues[1:4],  checkNames=FALSE)
}

test.loadRegressionFitResidualTimeSeries.Pcr <- function() {
  regressionStartDate <- dataModel$DATE[1]
  result <- loadRegressionFitResidualTimeSeries(regressionFit=regressionFit.pcr, regressionDataFrame=dataModel, regressionDateColumnNr=1, predictionBehaviorIndicatorColumnNr=2, predictionDataFrame=dataModel, predictionDateColumnNr=1, predictionStartDate=regressionStartDate, overallMaxStartupTime=0)
#     print(result$dates[1:4])
#     print(result$residuals[1:4])
#     print(length(result$residuals))
  
  checkEquals(2557, length(result$residuals))
  checkEqualsNumeric(c(0.6635643, 0.6980494, 0.4628322, 0.8504407), result$residuals[1:4], tolerance=0.001)
  checkEquals(as.Date(c("1992-01-01", "1992-01-02", "1992-01-03", "1992-01-04")), result$dates[1:4],  checkNames=FALSE)
  
  #prediction
  checkEqualsNumeric(c(0.6635643, 0.6980494, 0.4628322, 0.8504407), result$predictionResiduals[1:4], tolerance=0.001)
  checkEquals(as.Date(c("1992-01-01", "1992-01-02", "1992-01-03", "1992-01-04")), result$predictionDateValues[1:4],  checkNames=FALSE)
}


#######
# loadRegressionFitACF
#######
test.loadRegressionFitACF.Lm <- function() {
  result <- loadRegressionFitACF(regressionFit=regressionFit.lm)
  #   print(length(result$acf))
  #   print(result$acf[1:4])
  #   print(result$n.used)
  
  checkEquals(366, length(result$lag))
  checkEquals(length(result$lag), length(result$acf))
  checkEqualsNumeric(signif(c(1.0000000, 0.9981944, 0.9955726, 0.9924915), 4), signif(result$acf[1:4], 4))
  checkEqualsNumeric(2557, result$n.used)
}

test.loadRegressionFitACF.Pwi <- function() {
  DEACTIVATED()
  result <- loadRegressionFitACF(regressionFit=regressionFit.pwi)
  #  print(length(result$acf))
  #  print(result$acf[1:4])
  #  print(result$n.used)
  
  checkEquals(366, length(result$lag))
  checkEquals(length(result$lag), length(result$acf))
  checkEqualsNumeric(signif(c(1.0000000, 0.3573254, 0.2286534, 0.1609477), 4), signif(result$acf[1:4], 4))
  checkEqualsNumeric(2557, result$n.used)
}

test.loadRegressionFitACF.Ridge <- function() {
  result <- loadRegressionFitACF(regressionFit=regressionFit.ri)
#   print(length(result$acf))
#   print(result$acf[1:4])
#   print(result$n.used)
  
  checkEquals(366, length(result$lag))
  checkEquals(length(result$lag), length(result$acf))
  checkEqualsNumeric(c(1.0000000, 0.9981347, 0.9953891, 0.9921529), result$acf[1:4], tolerance=0.001)
  checkEqualsNumeric(2557, result$n.used)
}

test.loadRegressionFitACF.Pcr <- function() {
  result <- loadRegressionFitACF(regressionFit=regressionFit.pcr)
#     print(length(result$acf))
#     print(result$acf[1:4])
#     print(result$n.used)
  
  checkEquals(366, length(result$lag))
  checkEquals(length(result$lag), length(result$acf))
  checkEqualsNumeric(c(1.0000000, 0.9646786, 0.9176121, 0.8689398), result$acf[1:4], tolerance=0.001)
  checkEqualsNumeric(2557, result$n.used)
}


#######
# loadOverallRegressionQuality
#######
test.loadOverallRegressionQuality.Lm <- function() {
  result <- loadOverallRegressionQuality(regressionFit=regressionFit.lm)
  #   print(result$fit.r2adj.value)
  #   print(result$autocor.dwstat.value)
  #   print(result$autocor.dwstat.degree)
  #   print(result$fit.r2adj.degree)
  
  checkEqualsNumeric(0.002212135, result$autocor.dwstat.value, tolerance=0.001)
  checkEqualsNumeric(1, result$autocor.dwstat.degree)
  checkEqualsNumeric(0.6517445, result$fit.r2adj.value, tolerance=0.001)
  checkEqualsNumeric(2, result$fit.r2adj.degree)
}

test.loadOverallRegressionQuality.Pwi <- function() {
  DEACTIVATED()
  result <- loadOverallRegressionQuality(regressionFit=regressionFit.pwi)
#   print(result$fit.r2adj.value)
#   print(result$autocor.dwstat.value)
#   print(result$autocor.dwstat.degree)
#   print(result$fit.r2adj.degree)
  
  checkEqualsNumeric(1.284426, result$autocor.dwstat.value, tolerance=0.001)
  checkEqualsNumeric(1, result$autocor.dwstat.degree)
  checkEqualsNumeric(0.5986869, result$fit.r2adj.value, tolerance=0.001)
  checkEqualsNumeric(2, result$fit.r2adj.degree)
}

test.loadOverallRegressionQuality.Ridge <- function() {
  result <- loadOverallRegressionQuality(regressionFit=regressionFit.ri)
#   print(result$fit.r2adj.value)
#   print(result$autocor.dwstat.value)
#   print(result$autocor.dwstat.degree)
#   print(result$fit.r2adj.degree)
  
  checkEqualsNumeric(0.002212135, result$autocor.dwstat.value, tolerance=0.001)
  checkEqualsNumeric(1, result$autocor.dwstat.degree)
  checkEqualsNumeric(0.35347, result$fit.r2adj.value, tolerance=0.001)
  checkEqualsNumeric(3, result$fit.r2adj.degree)
}

test.loadOverallRegressionQuality.Pcr <- function() {
  result <- loadOverallRegressionQuality(regressionFit=regressionFit.pcr)
#     print(result$fit.r2adj.value)
#     print(result$autocor.dwstat.value)
#     print(result$autocor.dwstat.degree)
#     print(result$fit.r2adj.degree)
  
  checkEqualsNumeric(0.07054534, result$autocor.dwstat.value, tolerance=0.001)
  checkEqualsNumeric(1, result$autocor.dwstat.degree)
  checkEqualsNumeric(0.01145608, result$fit.r2adj.value, tolerance=0.001)
  checkEqualsNumeric(3, result$fit.r2adj.degree)
}

#######
# loadRegressorOverview
#######
test.loadRegressorOverview.Lm <- function() {
  result <- loadRegressorOverview(regressionFit=regressionFit.lm)
  #print(result$stdcoefficients)
  
  checkEquals(c("(Intercept)", "I(scale(x = W_LEVEL, scale = FALSE))", "I(scale(x = W_LEVEL, scale = FALSE)^2)"), result$names, checkNames=FALSE)
  checkEqualsNumeric(c(43.085304778, 0.477674601, 0.004657228), result$coefficients, tolerance=0.001)
  checkEqualsNumeric(c(210.78836, 65.51033, 19.39052), result$tstats, tolerance=0.001)
  checkEqualsNumeric(c(0.000000e+00, 0.000000e+00, 2.989117e-78), result$pvalues, tolerance=0.001)
  checkEqualsNumeric(c(0.2044007819, 0.0072915918, 0.0002401806), result$stderrors, tolerance=0.001)
  checkEqualsNumeric(c(1.490884, 1.490884), result$vifs, tolerance=0.001)
  checkEqualsNumeric(c(0.9336849, 0.2763631), result$stdcoefficients, tolerance=0.001)
}

#TODO: make a temporary object for calculateVIFs.lm out of the provided regressionFit object (see calculateVIFs.lmpwi)
test.loadRegressorOverview.Pwi <- function() {
  DEACTIVATED()
  result <- loadRegressorOverview(regressionFit=regressionFit.pwi)
  print("test.loadRegressorOverview.Pwi: ")
  cat("rho: ", result$rho)
  
  checkEquals(c("(Intercept)", "I(scale(x = W_LEVEL, scale = FALSE))", "I(scale(x = W_LEVEL, scale = FALSE)^2)"), result$names, checkNames=FALSE)
  checkEqualsNumeric(c(45.000594463, 0.606693380, 0.005841876), result$coefficients, tolerance=0.001)
  checkEqualsNumeric(c(14.60863, 59.60839, 38.31469), result$tstats, tolerance=0.001)
  checkEqualsNumeric(c(1.759975e-46, 0.000000e+00, 3.658347e-254), result$pvalues, tolerance=0.001)
  checkEqualsNumeric(c(3.0804118201, 0.0101779869, 0.0001524709), result$stderrors, tolerance=0.001)
  checkEqualsNumeric(c(NaN, NaN), result$vifs)
  checkEqualsNumeric(c(NaN, NaN), result$stdcoefficients, tolerance=0.001)
}


test.loadRegressorOverview.Ridge <- function() {
  result <- loadRegressorOverview(regressionFit=regressionFit.ri)
  #print(result$stdcoefficients)
  
  dimnames.results.withoutintercept <- c("I(scale(x = W_LEVEL, scale = FALSE))", "I(scale(x = W_LEVEL, scale = FALSE)^2)")
  dimnames.results.withintercept <- c("(Intercept)", dimnames.results.withoutintercept)
  
  coefficients.expected <- c(43.714337603, 0.433300964, 0.003602382)
  names(coefficients.expected) <- dimnames.results.withintercept
  
  vifs.expected <- c(1.334436, 1.334436)
  names(vifs.expected) <- dimnames.results.withoutintercept
  
  tvalues.expected <- c(NaN, 62.34664, 15.73612)
  names(tvalues.expected) <- dimnames.results.withintercept
  
  pvalues.expected <- c(NaN, 0.000000e+00, 2.522285e-53)
  names(pvalues.expected) <- dimnames.results.withintercept
  
  checkEquals(c("(Intercept)", "I(scale(x = W_LEVEL, scale = FALSE))", "I(scale(x = W_LEVEL, scale = FALSE)^2)"), result$names, checkNames=FALSE)
  checkEquals(coefficients.expected, coefficients(regressionFit.ri), checkNames=TRUE, tolerance=0.001)
  checkEquals(vifs.expected, regressionFit.ri$vifs, checkNames=TRUE, tolerance=0.001)
  checkEquals(tvalues.expected, regressionFit.ri$t_values, checkNames=TRUE, tolerance=0.001)
  checkEquals(pvalues.expected, regressionFit.ri$p_values, checkNames=TRUE, tolerance=0.001)
  checkEqualsNumeric(c(0.2059253729, 0.0073459786, 0.0002419721), result$stderrors, tolerance=0.001)
  checkEqualsNumeric(c(1.334436, 1.334436), result$vifs, tolerance=0.001)
  checkEqualsNumeric(c(0.8469501, 0.2137678), result$stdcoefficients, tolerance=0.001)
}

test.loadRegressorOverview.Pcr <- function() {
  result <- loadRegressorOverview(regressionFit=regressionFit.pcr)
  
#   print(result$names)
  checkEquals(c("(Intercept)", "I(dambase.T1(W_LEVEL, e$containerT1))", "I(dambase.T2(W_LEVEL, e$containerT2))", "I(dambase.T3(W_LEVEL, e$containerT3))", "I(dambase.T4(W_LEVEL, e$containerT4))", "I(dambase.direct(T_H12_UP))", "I(dambase.direct(T_H12_MI_corr))", "I(dambase.direct(T_H12_DO))", "I(dambase.direct(T_H15_UP))", "I(dambase.direct(T_H15_MI))", "I(dambase.direct(T_H15_DO))"), result$names, checkNames=FALSE)
  checkEqualsNumeric(c(39.85688821, 26.62990739, 8.03976627, 1.54904214, 0.56836845, -0.26168003, -1.33863064, -0.44038541, -0.02458048,  3.99711069, -1.70378112), result$coefficients, tolerance=0.001)
  checkEqualsNumeric(c(4.357509, 2.440124, 2.141246, 1.949046, 6.082299, 9.465645, 113.104263, 11.953194, 3.036636, 126.155838), result$vifs, tolerance=0.001)
  checkEqualsNumeric(c(NaN, 239.858675, 104.169472,  26.513886,  10.556545,  -8.072975, -16.225423, -6.009095,  -0.328773,  10.629435, -19.853984), result$tstats, tolerance=0.001)
  checkEqualsNumeric(c(NaN,  0.000000e+00,  0.000000e+00, 5.413350e-137,  1.583005e-25, 1.046918e-15,  2.045456e-56,  2.132694e-09,  7.423543e-01,  7.507560e-26, 1.126749e-81), result$pvalues, tolerance=0.001)
  checkEqualsNumeric(0.01145608, regressionFit.pcr$adj.r.squared, tolerance=0.001)
  checkEqualsNumeric(c(1.92629177, 0.11102332, 0.07717968, 0.05842381, 0.05384039, 0.03241433, 0.08250205, 0.07328647, 0.07476431, 0.37604170, 0.08581558), result$stderrors, tolerance=0.001)
  checkEqualsNumeric(c(1.060015252, 0.344495835, 0.082138011, 0.031201135, -0.042150720, -0.105683856, -0.135296356, -0.002406443, 0.039214260, -0.472105299), result$stdcoefficients, tolerance=0.001)
}


#######
# loadRegressionSummary
#######
test.loadRegressionSummary.Lm <- function() {
  result <- loadRegressionSummary(regressionFit=regressionFit.lm, dataFrame=dataModel, dateColumnNr=1, behaviorIndicatorColumnNr=2, predictionStartDate=dataModel$DATE[1], observationStartDate=dataModel$DATE[1], overallMaxStartupTime=0)
  
#   print(":separation:")
#   print(result$msres)
#   print(result$msresderived)
#   print(result$r2)
#   print(result$r2adj)
#   print(result$fstat)
#   print(result$fstatpvalue)
#   print(result$dwstat)
#   print(result$nobs)
#   print(result$ssresPred)
#   print(result$msresderivedPred)
  
  
  checkEqualsNumeric(138879, result$ssres, tolerance=0.001)
  checkEqualsNumeric(54.37706, result$msres, tolerance=0.001)
  checkEqualsNumeric(54.31326, result$msresderived, tolerance=0.001)
  checkEqualsNumeric(0.652017, result$r2, tolerance=0.001)
  checkEqualsNumeric(0.6517445, result$r2, tolerance=0.001)
  checkEqualsNumeric(2392.72, result$fstat[1], tolerance=0.001)
  checkEqualsNumeric(0, result$fstatpvalue, tolerance=0.001)
  checkEqualsNumeric(0.002212135, result$dwstat, tolerance=0.001)
  checkEqualsNumeric(2557, result$nobs, tolerance=0.001)
  checkEqualsNumeric(138879, result$ssresPred, tolerance=0.001)
  checkEqualsNumeric(54.31326, result$msresderivedPred, tolerance=0.001)
}

test.loadRegressionSummary.Pwi <- function() {
  DEACTIVATED()
  result <- loadRegressionSummary(regressionFit=regressionFit.pwi, dataFrame=dataModel, dateColumnNr=1, behaviorIndicatorColumnNr=2, predictionStartDate=dataModel$DATE[1], observationStartDate=dataModel$DATE[1], overallMaxStartupTime=0)
  
#     print(":separation:")
#     print(result$ssres)
#     print(result$msres)
#     print(result$msresderived)
#     print(result$r2)
#     print(result$r2adj)
#     print(result$fstat)
#     print(result$fstatpvalue)
#     print(result$dwstat)
#     print(result$nobs)
#     print(result$ssresPred)
#     print(result$msresderivedPred)
  
  
   checkEqualsNumeric(289.0935, result$ssres, tolerance=0.001)
   checkEqualsNumeric(0.1131925, result$msres, tolerance=0.001)
   checkEqualsNumeric(0.1130597, result$msresderived, tolerance=0.001)
   checkEqualsNumeric(0.5991577, result$r2, tolerance=0.001)
   checkEqualsNumeric(0.5986869, result$r2, tolerance=0.001)
   checkEqualsNumeric(1272.528, result$fstat[1], tolerance=0.001)
   checkEqualsNumeric(0, result$fstatpvalue, tolerance=0.001)
   checkEqualsNumeric(1.284426, result$dwstat, tolerance=0.001)
   checkEqualsNumeric(2557, result$nobs, tolerance=0.001)
   checkEqualsNumeric(175688.7, result$ssresPred, tolerance=0.001)
   checkEqualsNumeric(68.70892, result$msresderivedPred, tolerance=0.001)
}

test.loadRegressionSummary.Ridge <- function() {
  result <- loadRegressionSummary(regressionFit=regressionFit.ri, dataFrame=dataModel, dateColumnNr=1, behaviorIndicatorColumnNr=2, predictionStartDate=dataModel$DATE[1], observationStartDate=dataModel$DATE[1], overallMaxStartupTime=0)
  
#   print(":separation:")
#   print(result$ssres)
#   print(result$msres)
#   print(result$msresderived)
#   print(result$r2)
#   print(result$r2adj)
#   cat("result$fstat: ", result$fstat, "\n")
#   message("result$fstatpvalue: ", result$fstatpvalue)
#   print(result$dwstat)
#   print(result$nobs)
#   print(result$ssresPred)
#   print(result$msresderivedPred)
  
  
  checkEqualsNumeric(140958.5, result$ssres, tolerance=0.001)
  checkEqualsNumeric(55.19127, result$msres, tolerance=0.001)
  checkEqualsNumeric(55.12651, result$msresderived, tolerance=0.001)
  checkEqualsNumeric(0.6468066, result$r2, tolerance=0.001)
  checkEqualsNumeric(2007.531, result$fstat[1], tolerance=0.001)
  checkEqualsNumeric(0, result$fstatpvalue, tolerance=0.001)
  checkEqualsNumeric(0.002212135, result$dwstat, tolerance=0.001)
  checkEqualsNumeric(2557, result$nobs, tolerance=0.001)
  checkEqualsNumeric(result$ssres, result$ssresPred, tolerance=0.001) #this is because the prediction starts at the beginning
  checkEqualsNumeric(result$msresderived, result$msresderivedPred, tolerance=0.001) #this is because the prediction starts at the beginning
}

test.loadRegressionSummary.Pcr <- function() {
  result <- loadRegressionSummary(regressionFit=regressionFit.pcr, dataFrame=dataModel, dateColumnNr=1, behaviorIndicatorColumnNr=2, predictionStartDate=dataModel$DATE[1], observationStartDate=dataModel$DATE[1], overallMaxStartupTime=0)
  
#   print(":separation:")
#   print(result$ssres)
#   print(result$msres)
#   print(result$msresderived)
#   print(result$r2)
#   print(result$r2adj)
#   cat("result$fstat: ", result$fstat, "\n")
#   message("result$fstatpvalue: ", result$fstatpvalue)
#   print(result$dwstat)
#   print(result$nobs)
#   print(result$ssresPred)
#   print(result$msresderivedPred)
  
  
  checkEqualsNumeric(4554.2, result$ssres, tolerance=0.001)
  checkEqualsNumeric(1.788767, result$msres, tolerance=0.001)
  checkEqualsNumeric(1.781071, result$msresderived, tolerance=0.001)
  checkEqualsNumeric(0.9885887, result$r2, tolerance=0.001)
  checkEqualsNumeric(0.01145608, result$r2adj, tolerance=0.001)
  checkEqualsNumeric(22056.7, result$fstat[1], tolerance=0.001)
  checkEqualsNumeric(0, result$fstatpvalue, tolerance=0.001)
  checkEqualsNumeric(0.07054534, result$dwstat, tolerance=0.001)
  checkEqualsNumeric(2557, result$nobs, tolerance=0.001)
  checkEqualsNumeric(result$ssres, result$ssresPred, tolerance=0.001) #this is because the prediction starts at the beginning
  checkEqualsNumeric(result$msresderived, result$msresderivedPred, tolerance=0.001) #this is because the prediction starts at the beginning
}

#######
# loadRegressionFitAnalysisPhysicalSummary
#######
test.loadRegressionFitAnalysisPhysicalSummary.Lm <- function() {
  regressionStartDate <- dataModel$DATE[1]
  regressionEndDate <- tail(dataModel$DATE, 2)[1]
  result <- loadRegressionFitAnalysisPhysicalSummary(regressionFit=regressionFit.lm, dataFrame=dataModel, dateColumnNr=1, behaviorIndicatorColumnNr=2, startDate=regressionStartDate, regressionEndDate=regressionEndDate, overallMaxStartupTime=0)
  
#   print(length(result$predictions))
#   print(result$predictions[1:3])
#   print(length(result$lowerCi))
#   print(result$lowerCi[1:3])
#   print(length(result$upperCi))
#   print(result$upperCi[1:3])
#   print(length(which(!is.nan(result$outOfLimit))))
#   print(length(result$outOfLimitDates))
  
  checkEquals(length(result$dateValues), 2557)
  checkEquals(dataModel$DATE, result$dateValues, checkNames=FALSE)
  checkEquals(dataModel$DATE, result$predictionDateValues, checkNames=FALSE)
  checkEquals(dataModel[[2]], result$behaviorIndicatorValues, checkNames=FALSE)
  checkEqualsNumeric(length(dataModel[[2]]), length(result$predictions), tolerance=0)
  checkEqualsNumeric(c(53.88383, 53.85108, 53.81181), result$predictions[1:3], tolerance=0.001)
  checkEqualsNumeric(length(result$predictions), length(result$lowerCi), tolerance=0)
  checkEqualsNumeric(c(39.41943, 39.38669, 39.34743), result$lowerCi[1:3], tolerance=0.001)
  checkEqualsNumeric(length(result$lowerCi), length(result$upperCi), tolerance=0)
  checkEqualsNumeric(c(68.34824, 68.31548, 68.27620), result$upperCi[1:3], tolerance=0.001)
  checkEquals(14, length(which(!is.nan(result$outOfLimit))))
  checkEquals(dataModel$DATE, result$outOfLimitDates)
  checkEquals(regressionStartDate, as.Date(result$regressionStartDate), checkNames=FALSE)
  checkEquals(regressionEndDate, as.Date(result$regressionEndDate), checkNames=FALSE)
  checkEqualsNumeric(0, length(result$extrapolation.values))
  checkEqualsNumeric(0, length(result$extrapolation.dates))
}

test.AloadRegressionFitAnalysisPhysicalSummary.missingw.Lm <- function() {

  regressionStartDate <- dataModel$DATE[1]
  regressionEndDate <- tail(dataModel$DATE, 2)[1]
  
  values <- loadRegressionFitAnalysisPhysicalSummary(regressionFit=regressionFit.lm, dataFrame=dataModelMissingW, dateColumnNr=1, behaviorIndicatorColumnNr=2, startDate=regressionStartDate, regressionEndDate=regressionEndDate, overallMaxStartupTime=0)
  #values <- loadRegressionFitResidualTimeSeries(regressionFit.missingw.lm, regressionDataFrame=dataModelMissingW, regressionDateColumnNr=1, predictionBehaviorIndicatorColumnNr=2, predictionDataFrame=dataModelMissingW, predictionDateColumnNr=1, predictionStartDate=regressionStartDate, overallMaxStartupTime=0) #dataModelMissingW)
  cat("test.AloadRegressionFitAnalysisPhysicalSummary.missingw.Lm : ", values$predictions[1:4])
}

test.loadRegressionFitAnalysisPhysicalSummary.Pwi <- function() {
  DEACTIVATED()
  regressionStartDate <- dataModel$DATE[1]
  regressionEndDate <- tail(dataModel$DATE, 2)[1]
  result <- loadRegressionFitAnalysisPhysicalSummary(regressionFit=regressionFit.pwi, dataFrame=dataModel, dateColumnNr=1, behaviorIndicatorColumnNr=2, startDate=regressionStartDate, regressionEndDate=regressionEndDate, overallMaxStartupTime=0)
  
#   print("separation")
#   print(result$predictions[1:3])
#   print(length(result$lowerCi))
#   print(result$lowerCi[1:3])
#   print(length(result$upperCi))
#   print(result$upperCi[1:3])
#   print(length(which(!is.nan(result$outOfLimit))))
#   print(length(result$outOfLimitDates))
   
 checkEquals(length(result$dateValues), 2557)
 checkEquals(dataModel$DATE, result$dateValues, checkNames=FALSE)
 checkEquals(dataModel$DATE, result$predictionDateValues, checkNames=FALSE)
 checkEquals(dataModel[[2]], result$behaviorIndicatorValues, checkNames=FALSE)
 checkEqualsNumeric(length(dataModel[[2]]), length(result$predictions), tolerance=0)
 checkEqualsNumeric(c(58.68916, 58.64770, 58.59799), result$predictions[1:3], tolerance=0.001)
 checkEqualsNumeric(length(result$predictions), length(result$lowerCi), tolerance=0)
 checkEqualsNumeric(c(42.41137, 42.38414, 42.33442), result$lowerCi[1:3], tolerance=0.001)
 checkEqualsNumeric(length(result$lowerCi), length(result$upperCi), tolerance=0)
 checkEqualsNumeric(c(74.96695, 74.91126, 74.86156), result$upperCi[1:3], tolerance=0.001)
 checkEquals(34, length(which(!is.nan(result$outOfLimit))))
 checkEquals(dataModel$DATE, result$outOfLimitDates)
 checkEquals(regressionStartDate, as.Date(result$regressionStartDate), checkNames=FALSE)
 checkEquals(regressionEndDate, as.Date(result$regressionEndDate), checkNames=FALSE)
 checkEqualsNumeric(0, length(result$extrapolation.values))
 checkEqualsNumeric(0, length(result$extrapolation.dates))
}


test.loadRegressionFitAnalysisPhysicalSummary.Ridge <- function() {
  regressionStartDate <- dataModel$DATE[1]
  regressionEndDate <- tail(dataModel$DATE, 2)[1]
  result <- loadRegressionFitAnalysisPhysicalSummary(regressionFit=regressionFit.ri, dataFrame=dataModel, dateColumnNr=1, behaviorIndicatorColumnNr=2, startDate=regressionStartDate, regressionEndDate=regressionEndDate, overallMaxStartupTime=0)
  
#     print("separation")
#     print(result$predictions[1:3])
#     print(length(result$lowerCi))
#     print(result$lowerCi[1:3])
#     print(length(result$upperCi))
#     print(result$upperCi[1:3])
#     print(length(which(!is.nan(result$outOfLimit))))
#     print(length(result$outOfLimitDates))
  
   checkEquals(length(result$dateValues), 2557)
   checkEquals(dataModel$DATE, result$dateValues, checkNames=FALSE)
   checkEquals(dataModel$DATE, result$predictionDateValues, checkNames=FALSE)
   checkEquals(dataModel[[2]], result$behaviorIndicatorValues, checkNames=FALSE)
   checkEqualsNumeric(length(dataModel[[2]]), length(result$predictions), tolerance=0)
   checkEqualsNumeric(c(53.28362, 53.25510, 53.22089), result$predictions[1:3], tolerance=0.001)
   checkEqualsNumeric(length(result$predictions), length(result$lowerCi), tolerance=0)
   checkEqualsNumeric(c(38.71132, 38.68281, 38.64862), result$lowerCi[1:3], tolerance=0.001)
   checkEqualsNumeric(length(result$lowerCi), length(result$upperCi), tolerance=0)
   checkEqualsNumeric(c(67.85592, 67.82738, 67.79316), result$upperCi[1:3], tolerance=0.001)
   checkEquals(47, length(which(!is.nan(result$outOfLimit))))
   checkEquals(dataModel$DATE, result$outOfLimitDates)
   checkEquals(regressionStartDate, as.Date(result$regressionStartDate), checkNames=FALSE)
   checkEquals(regressionEndDate, as.Date(result$regressionEndDate), checkNames=FALSE)
   checkEqualsNumeric(0, length(result$extrapolation.values))
   checkEqualsNumeric(0, length(result$extrapolation.dates))
}

test.loadRegressionFitAnalysisPhysicalSummary.Pcr <- function() {
  regressionStartDate <- dataModel$DATE[1]
  regressionEndDate <- tail(dataModel$DATE, 2)[1]
  result <- loadRegressionFitAnalysisPhysicalSummary(regressionFit=regressionFit.pcr, dataFrame=dataModel, dateColumnNr=1, behaviorIndicatorColumnNr=2, startDate=regressionStartDate, regressionEndDate=regressionEndDate, overallMaxStartupTime=0)
  
#       print("separation")
#       print(result$predictions[1:3])
#       print(length(result$lowerCi))
#       print(result$lowerCi[1:3])
#       print(length(result$upperCi))
#       print(result$upperCi[1:3])
#       print(length(which(!is.nan(result$outOfLimit))))
#       print(length(result$outOfLimitDates))
  
  checkEquals(length(result$dateValues), 2557)
  checkEquals(dataModel$DATE, result$dateValues, checkNames=FALSE)
  checkEquals(dataModel$DATE, result$predictionDateValues, checkNames=FALSE)
  checkEquals(dataModel[[2]], result$behaviorIndicatorValues, checkNames=FALSE)
  checkEqualsNumeric(length(dataModel[[2]]), length(result$predictions), tolerance=0)
  checkEqualsNumeric(c(64.93644, 65.00195, 65.13717), result$predictions[1:3], tolerance=0.001)
  checkEqualsNumeric(length(result$predictions), length(result$lowerCi), tolerance=0)
  checkEqualsNumeric(c(62.30508, 62.36937, 62.50524), result$lowerCi[1:3], tolerance=0.001)
  checkEqualsNumeric(length(result$lowerCi), length(result$upperCi), tolerance=0)
  checkEqualsNumeric(c(67.56779, 67.63453, 67.76910), result$upperCi[1:3], tolerance=0.001)
  checkEquals(132, length(which(!is.nan(result$outOfLimit))))
  checkEquals(dataModel$DATE, result$outOfLimitDates)
  checkEquals(regressionStartDate, as.Date(result$regressionStartDate), checkNames=FALSE)
  checkEquals(regressionEndDate, as.Date(result$regressionEndDate), checkNames=FALSE)
  checkEqualsNumeric(0, length(result$extrapolation.values))
  checkEqualsNumeric(0, length(result$extrapolation.dates))
}

#######
# loadRegressionFitResidualNormalProbability
#######
test.loadRegressionFitResidualNormalProbability.Lm <- function() {
  result <- loadRegressionFitResidualNormalProbability(regressionFit=regressionFit.lm)
  
#    print("separation - lm")
#    print(length(result$theoreticalQuantiles))
#    print(result$theoreticalQuantiles[1:3])
#    print(length(result$sampleQuantiles))
     print(result$sampleQuantiles[1:3])
#    print(result$line[[1]])
#    print(result$line[[2]])
  
  checkEquals(2557, length(result$theoreticalQuantiles), checkNames=FALSE)
  checkEqualsNumeric(c(1.584750, 1.616512, 1.602196), result$theoreticalQuantiles[1:3], tolerance=0.001)
  checkEquals(length(result$theoreticalQuantiles), length(result$sampleQuantiles), checkNames=FALSE)
  checkEqualsNumeric(c(1.589339, 1.607346, 1.599106), result$sampleQuantiles[1:3], tolerance=0.001)
  checkEqualsNumeric(c(-1.530636, -2.041849), result$line[[1]], tolerance=0.001)
  checkEqualsNumeric(c(1.616000, 2.134405), result$line[[2]], tolerance=0.001)
}

#######
# loadRegressionFitResidualNormalProbability
#######
test.loadRegressionFitResidualNormalProbability.Pwi <- function() {
  DEACTIVATED()
  result <- loadRegressionFitResidualNormalProbability(regressionFit=regressionFit.pwi)
  
#   print("separation")
#   print(length(result$theoreticalQuantiles))
#   print(result$theoreticalQuantiles[1:3])
#   print(length(result$sampleQuantiles))
#   print(result$sampleQuantiles[1:3])
#   print(result$line[[1]])
#   print(result$line[[2]])
  
  checkEquals(2557, length(result$theoreticalQuantiles), checkNames=FALSE)
  checkEqualsNumeric(c(1.4486391, 0.5764172, -0.1634548), result$theoreticalQuantiles[1:3], tolerance=0.001)
  checkEquals(length(result$theoreticalQuantiles), length(result$sampleQuantiles), checkNames=FALSE)
  checkEqualsNumeric(c(0.41510762, 0.15393483, -0.03755699), result$sampleQuantiles[1:3], tolerance=0.001)
  checkEqualsNumeric(c(-15.23533, -12.63955), result$line[[1]], tolerance=0.001)
  checkEqualsNumeric(c( 8.692721, 7.198459), result$line[[2]], tolerance=0.001)
}