/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * AnalysisToolsControllerDispatcher.cpp
 *
 *  Created on: Jan 31, 2014
 *      Author: gerberma
 */

#include <iostream>
#include <boost/function.hpp>
#include <boost/bind.hpp>
#include <boost/shared_ptr.hpp>
#include "app/GenericApplicationContext.hpp"
#include "app/types.hpp"
#include "controller/DataModelAnalysisToolsControllerDispatcher.hpp"
#include "model/DataModelOverviewStatistics.hpp"
#include "model/DataModelAnalysisInfluenceFactors.hpp"
#include "model/DataModelAnalysisTimeSeriesOverview.hpp"

namespace controller {

using namespace boost;
using namespace app::context;

/**
 * DataModel related
 */

DataModelAnalysisToolsControllerDispatcher::DataModelAnalysisToolsControllerDispatcher() {;}
DataModelAnalysisToolsControllerDispatcher::~DataModelAnalysisToolsControllerDispatcher() {;}

template<>
function<void()> DataModelAnalysisToolsControllerDispatcher::dispatch<Analysis_t, Analysis_Tool_Description_t, Analysis_Tool_Ui_t>(
		boost::shared_ptr<Analysis_t> analysis, boost::shared_ptr<Analysis_Tool_Description_t> toolDescription, function<void(Analysis_Tool_Ui_t *)> uiLocation)
{
	using namespace dambase::model;

	if(toolDescription->getId() == DataModelOverviewStatistics::getStaticToolId()) {
		return bind(&DataModel_Analysis_Overview_Summary_Controller_t::addAnalysisTool,
					FETCH_FROM_CTX(DataModel_Analysis_Overview_Summary_Controller_t),
					analysis, uiLocation
				);
	}else if(toolDescription->getId() == DataModelAnalysisInfluenceFactors::getStaticToolId()) {
		return bind(&DataModel_Analysis_Influence_Factors_Controller_t::addAnalysisTool,
					FETCH_FROM_CTX(DataModel_Analysis_Influence_Factors_Controller_t),
					analysis, uiLocation
				);
	}else if(toolDescription->getId() == DataModelAnalysisTimeSeriesOverview::getStaticToolId()) {
		return bind(&DataModelAnalysisTimeSeriesOverviewController_t::addAnalysisTool<Analysis_t>,
					FETCH_FROM_CTX(DataModelAnalysisTimeSeriesOverviewController_t),
					analysis, uiLocation
				);
	}

	return boost::function<void()>();
}


} /* namespace controller */
