/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * AnalysisToolsControllerDispatcher.cpp
 *
 *  Created on: Jan 31, 2014
 *      Author: gerberma
 */

#include <iostream>
#include <boost/function.hpp>
#include <boost/bind.hpp>
#include <boost/shared_ptr.hpp>
#include "app/GenericApplicationContext.hpp"
#include "app/types.hpp"
#include "controller/RegressionModelAnalysisToolsControllerDispatcher.hpp"
#include "model/RegressionFitAnalysisPhysicalSummary.hpp"
#include "model/RegressionFitAnalysisRegressorOverview.hpp"
#include "model/RegressionFitAnalysisRegressionSummary.hpp"
#include "model/RegressionFitAnalysisResidualNormalProbability.hpp"
#include "model/RegressionFitAnalysisResidualTimeSeries.hpp"
#include "model/RegressionFitAnalysisACF.hpp"
#include "model/RegressionFitAnalysisResidualVersusRegressionParameter.hpp"
#include "model/RegressionFitAnalysisOverallRegressionQuality.hpp"
#include "model/RegressionFitAnalysisPhysicalBehavior.hpp"
#include "model/RegressionFitAnalysisRegressorTimeseries.hpp"

namespace controller {

using namespace boost;
using namespace app::context;

/**
 * RegressionModel related
 */

RegressionModelAnalysisToolsControllerDispatcher::RegressionModelAnalysisToolsControllerDispatcher(){;}
RegressionModelAnalysisToolsControllerDispatcher::~RegressionModelAnalysisToolsControllerDispatcher(){;}

template<>
function<void()> RegressionModelAnalysisToolsControllerDispatcher::dispatch<Analysis_t, RegressionModel_t, Analysis_Tool_Description_t, Analysis_Tool_Ui_t>(
		boost::shared_ptr<Analysis_t> analysis, boost::shared_ptr<RegressionModel_t> regressionModel, boost::shared_ptr<Analysis_Tool_Description_t> toolDescription, function<void(Analysis_Tool_Ui_t *)> uiLocation)
{
	using namespace dambase::model;
	using namespace boost;

	//std::cout << __FUNCTION__ << " - not implemented yet" << std::endl;
	if(toolDescription->getId() == RegressionFitAnalysisPhysicalSummary::getStaticToolId()) {
		return bind<void>(&RegressionFitAnalysisPhysicalSummaryController_t::addAnalysisTool<Analysis_t, RegressionModel_t>,
					FETCH_FROM_CTX(RegressionFitAnalysisPhysicalSummaryController_t),
					analysis, regressionModel, uiLocation
				);
	}else if(toolDescription->getId() == RegressionFitAnalysisRegressorOverview::getStaticToolId()) {
		return bind<void>(&RegressionFitAnalysisRegressorOverviewController_t::addAnalysisTool<Analysis_t, RegressionModel_t>,
					FETCH_FROM_CTX(RegressionFitAnalysisRegressorOverviewController_t),
					analysis, regressionModel, uiLocation
				);
	}else if(toolDescription->getId() == RegressionFitAnalysisRegressionSummary::getStaticToolId()) {
		return bind<void>(&RegressionFitAnalysisRegressionSummaryController_t::addAnalysisTool<Analysis_t, RegressionModel_t>,
					FETCH_FROM_CTX(RegressionFitAnalysisRegressionSummaryController_t),
					analysis, regressionModel, uiLocation
				);
	}else if(toolDescription->getId() == RegressionFitAnalysisResidualNormalProbability::getStaticToolId()) {
		return bind<void>(&RegressionFitAnalysisResidualNormalProbabilityController_t::addAnalysisTool<Analysis_t, RegressionModel_t>,
					FETCH_FROM_CTX(RegressionFitAnalysisResidualNormalProbabilityController_t),
					analysis, regressionModel, uiLocation
				);
	}else if(toolDescription->getId() == RegressionFitAnalysisResidualTimeSeries::getStaticToolId()) {
		return bind<void>(&RegressionFitAnalysisResidualTimeSeriesController_t::addAnalysisTool<Analysis_t, RegressionModel_t>,
					FETCH_FROM_CTX(RegressionFitAnalysisResidualTimeSeriesController_t),
					analysis, regressionModel, uiLocation
				);
	}else if(toolDescription->getId() == RegressionFitAnalysisACF::getStaticToolId()) {
		return bind<void>(&RegressionFitAnalysisACFController_t::addAnalysisTool<Analysis_t, RegressionModel_t>,
					FETCH_FROM_CTX(RegressionFitAnalysisACFController_t),
					analysis, regressionModel, uiLocation
				);
	}else if(toolDescription->getId() == RegressionFitAnalysisResidualVersusRegressionParameter::getStaticToolId()) {
		return bind<void>(&RegressionFitAnalysisResidualVersusRegressionParameterController_t::addAnalysisTool<Analysis_t, RegressionModel_t>,
					FETCH_FROM_CTX(RegressionFitAnalysisResidualVersusRegressionParameterController_t),
					analysis, regressionModel, uiLocation
				);
	}else if(toolDescription->getId() == RegressionFitAnalysisOverallRegressionQuality::getStaticToolId()) {
		return bind<void>(&RegressionFitAnalysisOverallRegressionQualityController_t::addAnalysisTool<Analysis_t, RegressionModel_t>,
					FETCH_FROM_CTX(RegressionFitAnalysisOverallRegressionQualityController_t),
					analysis, regressionModel, uiLocation
				);
	}else if(toolDescription->getId() == RegressionFitAnalysisPhysicalBehavior::getStaticToolId()) {
		return bind<void>(&RegressionFitAnalysisPhysicalBehaviorController_t::addAnalysisTool<Analysis_t, RegressionModel_t>,
					FETCH_FROM_CTX(RegressionFitAnalysisPhysicalBehaviorController_t),
					analysis, regressionModel, uiLocation
				);
	}else if(toolDescription->getId() == RegressionFitAnalysisRegressorTimeseries::getStaticToolId()) {
		return bind<void>(&RegressionFitAnalysisRegressorTimeseriesController_t::addAnalysisTool<Analysis_t, RegressionModel_t>,
					FETCH_FROM_CTX(RegressionFitAnalysisRegressorTimeseriesController_t),
					analysis, regressionModel, uiLocation
				);
	}

	return boost::function<void()>();
}


} /* namespace controller */
