/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * CoreDataModelSubscriptions.cpp
 *
 *  Created on: 13.02.2015
 *      Author: gerberma
 */

#include <boost/foreach.hpp>
#include <boost/bind.hpp>
#include <boost/ref.hpp>
#include <boost/function.hpp>
#include <boost/bind/protect.hpp>
#include "app/types.hpp"
#include "app/GenericApplicationContext.hpp"
#include "communication/Signals.hpp"
#include "app/CoreDataModelSubscriptions.hpp"

namespace app {

CoreDataModelSubscriptions::CoreDataModelSubscriptions() {
}

CoreDataModelSubscriptions::~CoreDataModelSubscriptions() {
	using namespace boost;

	BOOST_FOREACH(signals2::connection &connection, subscriptionConnections) {
		connection.disconnect();
	}
}

} /* namespace app */

void app::CoreDataModelSubscriptions::subscribe() {
	using namespace context;
	using namespace boost;

	Navigation_Tree_Widget_t *navigationTreeWidget = FETCH_FROM_CTX(Navigation_Tree_Widget_t);

	DataModel_Controller_t *dataModelController = FETCH_FROM_CTX(DataModel_Controller_t);
	subscriptionConnections.push_back(navigationTreeWidget->subscribeForDataModelDoubleClicked(boost::bind(&DataModel_Controller_t::show<Analysis_t>, FETCH_FROM_CTX(DataModel_Controller_t), _1, _2)));
	subscriptionConnections.push_back(navigationTreeWidget->subscribeForCreateDataModel(boost::bind(&DataModel_Controller_t::createFor<Analysis_t, Model_Factory_t>, dataModelController, _1, FETCH_FROM_CTX(DataModel_Factory_t))));

	subscriptionConnections.push_back(COM_SUBSCRIBE_FOR(navigationTreeWidget, LoadDataModelSource,
				(bind(&DataModel_Controller_t::editCommonParameters<Analysis_t>, FETCH_FROM_CTX(DataModel_Controller_t),
						_1, _2
				))
		));

	subscriptionConnections.push_back(COM_SUBSCRIBE_FOR(navigationTreeWidget, EditDataModelTimeSeries,
				(bind(&DataModel_Controller_t::editTimeSeries<Analysis_t>, FETCH_FROM_CTX(DataModel_Controller_t),
						_1, _2
				))
		));

	subscriptionConnections.push_back(COM_SUBSCRIBE_FOR(navigationTreeWidget, EditDataModelAssignment,
				(bind(&DataModel_Controller_t::editAssignment<Analysis_t>, FETCH_FROM_CTX(DataModel_Controller_t),
						_1, _2
				))
		));
}
