/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * CoreSubscriptions.cpp
 *
 *  Created on: 13.02.2015
 *      Author: gerberma
 */

#include <boost/foreach.hpp>
#include <boost/bind.hpp>
#include <boost/ref.hpp>
#include <boost/function.hpp>
#include <boost/bind/protect.hpp>
#include "app/CoreSubscriptions.hpp"
#include "app/types.hpp"
#include "app/GenericApplicationContext.hpp"
#include "communication/Signals.hpp"

namespace app {

CoreSubscriptions::CoreSubscriptions() {
	// TODO Auto-generated constructor stub

}

CoreSubscriptions::~CoreSubscriptions() {
	using namespace boost;

	BOOST_FOREACH(signals2::connection &connection, subscriptionConnections) {
		connection.disconnect();
	}
}

void CoreSubscriptions::subscribe() {
	using namespace context;
	using namespace boost;

	Core_Window_t *coreUi = FETCH_FROM_CTX(Core_Window_t);
	Project_Controller_t *projectController = FETCH_FROM_CTX(Project_Controller_t);
	Core_Controller_t *coreController = FETCH_FROM_CTX(Core_Controller_t);

	subscriptionConnections.push_back(coreUi->subscribeForClose(boost::bind(&Core_Controller_t::close, coreController)));
	subscriptionConnections.push_back(coreUi->subscribeForCreate(boost::bind(&Project_Controller_t::create, projectController)));
	subscriptionConnections.push_back(coreUi->subscribeForSave(boost::bind(&Project_Controller_t::save, projectController)));
	subscriptionConnections.push_back(COM_SUBSCRIBE_FOR(coreUi, ModeChanged, bind(&Core_Controller_t::modeChanged, coreController, _1)));
	subscriptionConnections.push_back(coreUi->subscribeForLoad(boost::bind(&Project_Controller_t::load<Project_Loader_t>, projectController, FETCH_FROM_CTX(Project_Loader_t))));
	subscriptionConnections.push_back(COM_SUBSCRIBE_FOR(coreUi, CloseRequested, bind(&Core_Controller_t::closeRequested, coreController, _1)));


	Navigation_Tree_Widget_t *navigationTreeWidget = FETCH_FROM_CTX(Navigation_Tree_Widget_t);

	Project_Wizard_t *projectWizard = FETCH_FROM_CTX(Project_Wizard_t);
	Project_Overview_Widget_t *projectOverviewWidget = FETCH_FROM_CTX(Project_Overview_Widget_t);
	subscriptionConnections.push_back(COM_SUBSCRIBE_FOR(projectWizard, Edited, boost::bind(&Project_Controller_t::edited, projectController)));
	subscriptionConnections.push_back(COM_SUBSCRIBE_FOR(projectWizard, ModeChanged, bind(&Project_Controller_t::modeChanged, projectController, _1)));
	subscriptionConnections.push_back(projectOverviewWidget->subscribeForEdit(boost::bind(&Project_Controller_t::edit, projectController)));
	subscriptionConnections.push_back(navigationTreeWidget->subscribeForProjectDoubleClicked(boost::bind(&Project_Controller_t::show, projectController)));

/*
	Navigation_Tree_Widget_t *navigationTreeWidget = FETCH_FROM_CTX(Navigation_Tree_Widget_t);

	Analysis_Controller_t *analysisController = FETCH_FROM_CTX(Analysis_Controller_t);
	subscriptionConnections.push_back(navigationTreeWidget->subscribeForProjectDoubleClicked(boost::bind(&Project_Controller_t::show, projectController)));
	subscriptionConnections.push_back(navigationTreeWidget->subscribeForCreateAnalysis(boost::bind(&Analysis_Controller_t::createFor<Project_t, Analysis_Factory_t>, analysisController, _1, FETCH_FROM_CTX(Analysis_Factory_t))));
	subscriptionConnections.push_back(navigationTreeWidget->subscribeForAnalysisDoubleClicked(boost::bind(&Analysis_Controller_t::show, analysisController, _1)));


	subscriptionConnections.push_back(COM_SUBSCRIBE_FOR(navigationTreeWidget, RemoveAnalysis,
			(bind(&Analysis_Controller_t::remove<Project_t>, FETCH_FROM_CTX(Analysis_Controller_t),
					_1, _2
			))
	));
*/
/*
/// ->
	DataModel_Controller_t *dataModelController = FETCH_FROM_CTX(DataModel_Controller_t);
	subscriptionConnections.push_back(navigationTreeWidget->subscribeForCreateDataModel(boost::bind(&DataModel_Controller_t::createFor<Analysis_t, Model_Factory_t>, dataModelController, _1, FETCH_FROM_CTX(DataModel_Factory_t))));

	subscriptionConnections.push_back(COM_SUBSCRIBE_FOR(navigationTreeWidget, LoadDataModelSource,
				(bind(&DataModel_Controller_t::editCommonParameters<Analysis_t>, FETCH_FROM_CTX(DataModel_Controller_t),
						_1, _2
				))
		));

	subscriptionConnections.push_back(COM_SUBSCRIBE_FOR(navigationTreeWidget, EditDataModelTimeSeries,
				(bind(&DataModel_Controller_t::editTimeSeries<Analysis_t>, FETCH_FROM_CTX(DataModel_Controller_t),
						_1, _2
				))
		));

	subscriptionConnections.push_back(COM_SUBSCRIBE_FOR(navigationTreeWidget, EditDataModelAssignment,
				(bind(&DataModel_Controller_t::editAssignment<Analysis_t>, FETCH_FROM_CTX(DataModel_Controller_t),
						_1, _2
				))
		));
/// <--
/// -->
	subscriptionConnections.push_back(COM_SUBSCRIBE_FOR(navigationTreeWidget, CreateRegressionModel,
			(bind(&RegressionModel_Controller_t::createFor<Analysis_t, RegressionModel_Subscription_Manager_t>, FETCH_FROM_CTX(RegressionModel_Controller_t),
					_1,
					protect(bind((RegressionModel_t*(App_Factory_t::*)(boost::shared_ptr<DataModel_t>, boost::shared_ptr<Analysis_t>))&App_Factory_t::create<RegressionModel_t, DataModel_t, Analysis_t>, FETCH_FROM_CTX(App_Factory_t), _1, _2)), //this _1 is not the same as the 'outer' _1
					FETCH_FROM_CTX(RegressionModel_Subscription_Manager_t)
			))
	));

	subscriptionConnections.push_back(COM_SUBSCRIBE_FOR(navigationTreeWidget, RegressionModelDoubleClicked,
			(bind(&RegressionModel_Controller_t::show<Analysis_t, RegressionModel_Subscription_Manager_t>, FETCH_FROM_CTX(RegressionModel_Controller_t),
					_2, _1,
					FETCH_FROM_CTX(RegressionModel_Subscription_Manager_t)
			))
	));

	subscriptionConnections.push_back(COM_SUBSCRIBE_FOR(navigationTreeWidget, CreateReport,
			(bind(&ReportController_t::createFor<Project_t, Analysis_t, RegressionModel_t, ReportSubscriptionManager_t>, FETCH_FROM_CTX(ReportController_t),
					_1, _2, _3,
					protect(bind((Report_t *(App_Factory_t::*)(boost::shared_ptr<Project_t>, boost::shared_ptr<Analysis_t>, boost::shared_ptr<RegressionModel_t>))&App_Factory_t::create<Report_t, Project_t, Analysis_t, RegressionModel_t>, FETCH_FROM_CTX(App_Factory_t), _1, _2, _3)), //this _1 is not the same as the 'outer' _1
					FETCH_FROM_CTX(ReportSubscriptionManager_t)
			))
	));

	subscriptionConnections.push_back(COM_SUBSCRIBE_FOR(navigationTreeWidget, CreateExport,
			(bind(&RawDataExportController_t::createExportRawDataFor<Project_t, Analysis_t, RegressionModel_t>, FETCH_FROM_CTX(RawDataExportController_t),
					_1, _2, _3,
					protect(bind((RawDataCollector_t *(App_Factory_t::*)())&App_Factory_t::create<RawDataCollector_t>, FETCH_FROM_CTX(App_Factory_t)))
			)
	)));


	subscriptionConnections.push_back(COM_SUBSCRIBE_FOR(navigationTreeWidget, EditRegressionCommonParameters,
					(bind(&RegressionModel_Controller_t::editCommonParameters<RegressionModel_Subscription_Manager_t>, FETCH_FROM_CTX(RegressionModel_Controller_t),
							_1,
							FETCH_FROM_CTX(RegressionModel_Subscription_Manager_t)
					))
	));


	subscriptionConnections.push_back(COM_SUBSCRIBE_FOR(navigationTreeWidget, EditRegressionBehaviorIndicator,
					(bind(&RegressionModel_Controller_t::editBehaviorIndicator<RegressionModel_Subscription_Manager_t>, FETCH_FROM_CTX(RegressionModel_Controller_t),
							_1,
							FETCH_FROM_CTX(RegressionModel_Subscription_Manager_t)
					))
	));

	subscriptionConnections.push_back(COM_SUBSCRIBE_FOR(navigationTreeWidget, EditRegressionTimeRange,
				(bind(&RegressionModel_Controller_t::editTimeRange<RegressionModel_Subscription_Manager_t>, FETCH_FROM_CTX(RegressionModel_Controller_t),
						_1,
						FETCH_FROM_CTX(RegressionModel_Subscription_Manager_t)
				))
	));

	subscriptionConnections.push_back(COM_SUBSCRIBE_FOR(navigationTreeWidget, EditRegressionFormula,
			(bind(&RegressionModel_Controller_t::editRegressionFormula<RegressionModel_Subscription_Manager_t>, FETCH_FROM_CTX(RegressionModel_Controller_t),
					_1,
					FETCH_FROM_CTX(RegressionModel_Subscription_Manager_t)
			))
	));

	subscriptionConnections.push_back(COM_SUBSCRIBE_FOR(navigationTreeWidget, ExecuteRegression,
			(bind(&RegressionModel_Controller_t::calculateRegressionFit, FETCH_FROM_CTX(RegressionModel_Controller_t),
					_1
			))
	));

	subscriptionConnections.push_back(COM_SUBSCRIBE_FOR(navigationTreeWidget, RemoveRegression,
			(bind(&RegressionModel_Controller_t::remove<Analysis_t, RegressionModel_Subscription_Manager_t>, FETCH_FROM_CTX(RegressionModel_Controller_t),
					_1, _2,
					FETCH_FROM_CTX(RegressionModel_Subscription_Manager_t)
			))
	));
///<--
	*/
}

} /* namespace app */
