/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelApplicationContext.cpp
 *
 *  Created on: 12.02.2015
 *      Author: gerberma
 */

#include <boost/bind.hpp>
#include "app/DataModelApplicationContext.hpp"

namespace app {
namespace context {

DataModelApplicationContext::DataModelApplicationContext():
	GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(DataModel_Analysis_Settings_t, boost::bind<DataModel_Analysis_Settings_t *>(&DataModelApplicationContext::fromRegisteredFactory<DataModel_Analysis_Settings_t, App_Factory_t>, this)),
	GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(Data_Frame_Provider_t, boost::bind<Data_Frame_Provider_t *>(&DataModelApplicationContext::fromRegisteredFactory<Data_Frame_Provider_t, Model_Factory_t>, this)),
	GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(DataFrame_Overview_Statistics_Provider_t, boost::bind<DataFrame_Overview_Statistics_Provider_t *>(&DataModelApplicationContext::fromRegisteredFactory<DataFrame_Overview_Statistics_Provider_t, Model_Factory_t>, this)),
	GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(DataFrame_Influence_Factors_Provider_t, boost::bind<DataFrame_Influence_Factors_Provider_t *>(&DataModelApplicationContext::fromRegisteredFactory<DataFrame_Influence_Factors_Provider_t, Model_Factory_t>, this)),
	GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(DataModel_Ui_Manager_t, boost::bind<DataModel_Ui_Manager_t *>(&DataModelApplicationContext::fromRegisteredFactory<DataModel_Ui_Manager_t, App_Factory_t>, this)),
	GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(DataModel_Controller_Internal_Subscription_Manager_t, boost::bind<DataModel_Controller_Internal_Subscription_Manager_t *>(&DataModelApplicationContext::fromRegisteredFactory<DataModel_Controller_Internal_Subscription_Manager_t, App_Factory_t>, this)),
	GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(DataModel_Controller_Internal_Subscription_Manager_Impl_t, boost::bind<DataModel_Controller_Internal_Subscription_Manager_Impl_t *>(&DataModelApplicationContext::fromRegisteredFactory<DataModel_Controller_Internal_Subscription_Manager_Impl_t, App_Factory_t>, this)),
	GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(DataModel_Controller_t, boost::bind<DataModel_Controller_t *>(&DataModelApplicationContext::fromRegisteredFactory<DataModel_Controller_t, App_Factory_t>, this)), //dataModelControllerFactory, defaultCleanupStrategy<DataModel_Controller_t>);
	GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(DataModel_Analysis_Tools_Ui_Manager_t, boost::bind<DataModel_Analysis_Tools_Ui_Manager_t *>(&DataModelApplicationContext::fromRegisteredFactory<DataModel_Analysis_Tools_Ui_Manager_t, App_Factory_t>, this)),
	GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(DataModel_Overview_Statistics_Ui_Manager_t, boost::bind<DataModel_Overview_Statistics_Ui_Manager_t *>(&DataModelApplicationContext::fromRegisteredFactory<DataModel_Overview_Statistics_Ui_Manager_t, App_Factory_t>, this)),
	GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(DataModel_Overview_Statistics_Subscription_Manager_t, boost::bind<DataModel_Overview_Statistics_Subscription_Manager_t *>(&DataModelApplicationContext::fromRegisteredFactory<DataModel_Overview_Statistics_Subscription_Manager_t, App_Factory_t>, this)),
	GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(DataModel_Analysis_Overview_Summary_Controller_t, boost::bind<DataModel_Analysis_Overview_Summary_Controller_t *>(&DataModelApplicationContext::fromRegisteredFactory<DataModel_Analysis_Overview_Summary_Controller_t, App_Factory_t>, this)),
	GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(DataModel_Analysis_Tools_Controller_Dispatcher_t, boost::bind<DataModel_Analysis_Tools_Controller_Dispatcher_t *>(&DataModelApplicationContext::fromRegisteredFactory<DataModel_Analysis_Tools_Controller_Dispatcher_t, App_Factory_t>, this)),
	GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(DataModel_Analysis_Tools_Controller_t, boost::bind<DataModel_Analysis_Tools_Controller_t *>(&DataModelApplicationContext::fromRegisteredFactory<DataModel_Analysis_Tools_Controller_t, App_Factory_t>, this)),
	GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(DataModel_Analysis_Influence_Factors_Ui_Manager_t, boost::bind<DataModel_Analysis_Influence_Factors_Ui_Manager_t *>(&DataModelApplicationContext::fromRegisteredFactory<DataModel_Analysis_Influence_Factors_Ui_Manager_t, App_Factory_t>, this)),
	GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(DataModel_Analysis_Influence_Factors_Controller_Subscription_Manager_t, boost::bind<DataModel_Analysis_Influence_Factors_Controller_Subscription_Manager_t *>(&DataModelApplicationContext::fromRegisteredFactory<DataModel_Analysis_Influence_Factors_Controller_Subscription_Manager_t, App_Factory_t>, this)),
	GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(DataModel_Analysis_Influence_Factors_Controller_t, boost::bind<DataModel_Analysis_Influence_Factors_Controller_t *>(&DataModelApplicationContext::fromRegisteredFactory<DataModel_Analysis_Influence_Factors_Controller_t, App_Factory_t>, this)),
	GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(DataModelAnalysisTimeSeriesOverviewUiManager_t, boost::bind<DataModelAnalysisTimeSeriesOverviewUiManager_t *>(&DataModelApplicationContext::fromRegisteredFactory<DataModelAnalysisTimeSeriesOverviewUiManager_t, App_Factory_t>, this)),
	GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(DataModelAnalysisTimeSeriesOverviewController_t, boost::bind<DataModelAnalysisTimeSeriesOverviewController_t *>(&DataModelApplicationContext::fromRegisteredFactory<DataModelAnalysisTimeSeriesOverviewController_t, App_Factory_t>, this)),
	GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(DataFrameTimeSeriesOverviewProvider_t, boost::bind<DataFrameTimeSeriesOverviewProvider_t *>(&DataModelApplicationContext::fromRegisteredFactory<DataFrameTimeSeriesOverviewProvider_t, App_Factory_t>, this)),
	GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(DataModelAnalysisToolsTimeSeriesOverviewControllerSubscriptionManager_t, boost::bind<DataModelAnalysisToolsTimeSeriesOverviewControllerSubscriptionManager_t *>(&DataModelApplicationContext::fromRegisteredFactory<DataModelAnalysisToolsTimeSeriesOverviewControllerSubscriptionManager_t, App_Factory_t>, this))

{
	// TODO Auto-generated constructor stub

}

DataModelApplicationContext::~DataModelApplicationContext() {
	// TODO Auto-generated destructor stub
}

} /* namespace context */
} /* namespace app */
