/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelApplicationContext.hpp
 *
 *  Created on: 12.02.2015
 *      Author: gerberma
 */

#ifndef DATAMODELAPPLICATIONCONTEXT_HPP_
#define DATAMODELAPPLICATIONCONTEXT_HPP_

#include "app/types.hpp"
#include "app/GenericApplicationContext.hpp"

#define GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(CLASS, FACTORY) \
	scope##CLASS(registerCtxWithContextScope<GenericApplicationContext<CLASS>, CLASS>(FACTORY))
#define GAC_FULL_CTOR_INIT(CLASS, FACTORY, CLEANUP) \
	scope##CLASS(registerCtxWithContextScope<GenericApplicationContext<CLASS>, CLASS>(FACTORY, CLEANUP))

namespace app {
namespace context {

class DataModelApplicationContext {
public:
	DataModelApplicationContext();
	virtual ~DataModelApplicationContext();

	template<
		typename T,
		typename Factory
	>
	T *fromRegisteredFactory() {
		Factory *factory = FETCH_FROM_CTX(Factory);
		return factory->template create<T>();
	}

	template<
		typename T,
		typename U,
		typename Factory
	>
	T *fromRegisteredFactory(U &u) {
		Factory *factory = FETCH_FROM_CTX(Factory);
		return factory->template create<T, U>(u);
	}

	template<
		typename T,
		typename U,
		typename W,
		typename Factory
	>
	T *fromRegisteredFactory(U u, W w) {
		Factory *factory = FETCH_FROM_CTX(Factory);
		return factory->template create<T>(u, w);
	}

private:
	GAC_SCOPE(DataModel_Analysis_Settings_t);
	GAC_SCOPE(Data_Frame_Provider_t);
	GAC_SCOPE(DataFrame_Overview_Statistics_Provider_t);
	GAC_SCOPE(DataFrame_Influence_Factors_Provider_t);
	GAC_SCOPE(DataModel_Ui_Manager_t);
	GAC_SCOPE(DataModel_Controller_Internal_Subscription_Manager_t);
	GAC_SCOPE(DataModel_Controller_Internal_Subscription_Manager_Impl_t);
	GAC_SCOPE(DataModel_Controller_t);
	GAC_SCOPE(DataModel_Analysis_Tools_Ui_Manager_t);
	GAC_SCOPE(DataModel_Overview_Statistics_Ui_Manager_t);
	GAC_SCOPE(DataModel_Overview_Statistics_Subscription_Manager_t);
	GAC_SCOPE(DataModel_Analysis_Overview_Summary_Controller_t);
	GAC_SCOPE(DataModel_Analysis_Tools_Controller_Dispatcher_t);
	GAC_SCOPE(DataModel_Analysis_Tools_Controller_t);
	GAC_SCOPE(DataModel_Analysis_Influence_Factors_Ui_Manager_t);
	GAC_SCOPE(DataModel_Analysis_Influence_Factors_Controller_Subscription_Manager_t);
	GAC_SCOPE(DataModel_Analysis_Influence_Factors_Controller_t);
	GAC_SCOPE(DataModelAnalysisTimeSeriesOverviewUiManager_t);
	GAC_SCOPE(DataModelAnalysisTimeSeriesOverviewController_t);
	GAC_SCOPE(DataFrameTimeSeriesOverviewProvider_t);
	GAC_SCOPE(DataModelAnalysisToolsTimeSeriesOverviewControllerSubscriptionManager_t);
};

} /* namespace context */
} /* namespace app */
#endif /* DATAMODELAPPLICATIONCONTEXT_HPP_ */
