/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * ProjectApplicationContext.hpp
 *
 *  Created on: 13.02.2015
 *      Author: gerberma
 */

#ifndef PROJECTAPPLICATIONCONTEXT_HPP_
#define PROJECTAPPLICATIONCONTEXT_HPP_

#include "app/types.hpp"
#include "app/GenericApplicationContext.hpp"

#define GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(CLASS, FACTORY) \
	scope##CLASS(registerCtxWithContextScope<GenericApplicationContext<CLASS>, CLASS>(FACTORY))
#define GAC_FULL_CTOR_INIT(CLASS, FACTORY, CLEANUP) \
	scope##CLASS(registerCtxWithContextScope<GenericApplicationContext<CLASS>, CLASS>(FACTORY, CLEANUP))

namespace app {
namespace context {

class ProjectApplicationContext {
public:
	ProjectApplicationContext();
	virtual ~ProjectApplicationContext();

	template<
			typename T,
			typename Factory
		>
		T *fromRegisteredFactory() {
			Factory *factory = FETCH_FROM_CTX(Factory);
			return factory->template create<T>();
		}

		template<
			typename T,
			typename U,
			typename Factory
		>
		T *fromRegisteredFactory(U &u) {
			Factory *factory = FETCH_FROM_CTX(Factory);
			return factory->template create<T, U>(u);
		}

		template<
			typename T,
			typename U,
			typename W,
			typename Factory
		>
		T *fromRegisteredFactory(U u, W w) {
			Factory *factory = FETCH_FROM_CTX(Factory);
			return factory->template create<T>(u, w);
		}

private:
	GAC_SCOPE(Project_Controller_t);
	GAC_SCOPE(Project_Overview_Widget_t);
	GAC_SCOPE(Project_Wizard_t);
};

} /* namespace context */
} /* namespace app */
#endif /* PROJECTAPPLICATIONCONTEXT_HPP_ */
