/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * ApplicationContext.cpp
 *
 *  Created on: Nov 5, 2013
 *      Author: gerberma
 */

#include "app/types.hpp"
#include "app/RegressionModelApplicationContext.hpp"


namespace app {
namespace context {

RegressionModelApplicationContext::RegressionModelApplicationContext() :
				GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(RegressionModel_Analysis_Physics_Category_Provider_t, boost::bind<RegressionModel_Analysis_Physics_Category_Provider_t*>(&RegressionModelApplicationContext::fromRegisteredFactory<RegressionModel_Analysis_Physics_Category_Provider_t, App_Factory_t>, this)),
				GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(RegressionModel_Analysis_Statistics_Category_Provider_t, boost::bind<RegressionModel_Analysis_Statistics_Category_Provider_t*>(&RegressionModelApplicationContext::fromRegisteredFactory<RegressionModel_Analysis_Statistics_Category_Provider_t, App_Factory_t>, this)),
				GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(Regression_Formula_Index_Manager_t, boost::bind<Regression_Formula_Index_Manager_t *>(&RegressionModelApplicationContext::fromRegisteredFactory<Regression_Formula_Index_Manager_t, App_Factory_t>, this)),
				GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(Regression_Fit_Provider_t, boost::bind<Regression_Fit_Provider_t *>(&RegressionModelApplicationContext::fromRegisteredFactory<Regression_Fit_Provider_t, App_Factory_t>, this)),
				GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(RegressionModel_Ui_Manager_t, boost::bind<RegressionModel_Ui_Manager_t*>(&RegressionModelApplicationContext::fromRegisteredFactory<RegressionModel_Ui_Manager_t, App_Factory_t>, this)),
				GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(RegressionModel_Internal_Generic_Controller_t, boost::bind<RegressionModel_Internal_Generic_Controller_t *>(&RegressionModelApplicationContext::fromRegisteredFactory<RegressionModel_Internal_Generic_Controller_t, App_Factory_t>, this)),
				GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(RegressionModel_Controller_t, boost::bind<RegressionModel_Controller_t *>(&RegressionModelApplicationContext::fromRegisteredFactory<RegressionModel_Controller_t, App_Factory_t>, this)),
				GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(RegressionModel_Subscription_Manager_t, boost::bind<RegressionModel_Subscription_Manager_t *>(&RegressionModelApplicationContext::fromRegisteredFactory<RegressionModel_Subscription_Manager_t, App_Factory_t>, this)),
				GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(RegressionModelAnalysisToolsControllerDispatcher_t, boost::bind<RegressionModelAnalysisToolsControllerDispatcher_t *>(&RegressionModelApplicationContext::fromRegisteredFactory<RegressionModelAnalysisToolsControllerDispatcher_t, App_Factory_t>, this)),
				GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(RegressionModelAnalysisToolsController_t, boost::bind<RegressionModelAnalysisToolsController_t *>(&RegressionModelApplicationContext::fromRegisteredFactory<RegressionModelAnalysisToolsController_t, App_Factory_t>, this)),
				GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(RegressionFormulaMethodsController_t, boost::bind<RegressionFormulaMethodsController_t *>(&RegressionModelApplicationContext::fromRegisteredFactory<RegressionFormulaMethodsController_t, App_Factory_t>, this)),
				GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(RegressionFitAnalysisPhysicalSummaryUiManager_t, boost::bind<RegressionFitAnalysisPhysicalSummaryUiManager_t *>(&RegressionModelApplicationContext::fromRegisteredFactory<RegressionFitAnalysisPhysicalSummaryUiManager_t, App_Factory_t>, this)),
				GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(RegressionFitAnalysisPhysicalSummaryController_t, boost::bind<RegressionFitAnalysisPhysicalSummaryController_t *>(&RegressionModelApplicationContext::fromRegisteredFactory<RegressionFitAnalysisPhysicalSummaryController_t, App_Factory_t>, this)),
				GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(R_RegressionFitAnalysisPhysicalSummaryProvider_t, boost::bind<R_RegressionFitAnalysisPhysicalSummaryProvider_t *>(&RegressionModelApplicationContext::fromRegisteredFactory<R_RegressionFitAnalysisPhysicalSummaryProvider_t, App_Factory_t>, this)),
				GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(RegressionFitAnalysisRegressorOverviewUiManager_t, boost::bind<RegressionFitAnalysisRegressorOverviewUiManager_t *>(&RegressionModelApplicationContext::fromRegisteredFactory<RegressionFitAnalysisRegressorOverviewUiManager_t, App_Factory_t>, this)),
				GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(RegressionFitAnalysisRegressorOverviewSubscriptionManager_t, boost::bind<RegressionFitAnalysisRegressorOverviewSubscriptionManager_t *>(&RegressionModelApplicationContext::fromRegisteredFactory<RegressionFitAnalysisRegressorOverviewSubscriptionManager_t, App_Factory_t>, this)),
				GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(RegressionFitAnalysisRegressorOverviewController_t, boost::bind<RegressionFitAnalysisRegressorOverviewController_t *>(&RegressionModelApplicationContext::fromRegisteredFactory<RegressionFitAnalysisRegressorOverviewController_t, App_Factory_t>, this)),
				GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(RegressionFitAnalysisRegressorOverviewProvider_t, boost::bind<RegressionFitAnalysisRegressorOverviewProvider_t *>(&RegressionModelApplicationContext::fromRegisteredFactory<RegressionFitAnalysisRegressorOverviewProvider_t, App_Factory_t>, this)),
				GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(RegressionFitAnalysisRegressionSummaryUiManager_t, boost::bind<RegressionFitAnalysisRegressionSummaryUiManager_t *>(&RegressionModelApplicationContext::fromRegisteredFactory<RegressionFitAnalysisRegressionSummaryUiManager_t, App_Factory_t>, this)),
				GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(RegressionFitAnalysisRegressionSummaryController_t, boost::bind<RegressionFitAnalysisRegressionSummaryController_t *>(&RegressionModelApplicationContext::fromRegisteredFactory<RegressionFitAnalysisRegressionSummaryController_t, App_Factory_t>, this)),
				GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(RegressionFitAnalysisRegressionSummaryProvider_t, boost::bind<RegressionFitAnalysisRegressionSummaryProvider_t *>(&RegressionModelApplicationContext::fromRegisteredFactory<RegressionFitAnalysisRegressionSummaryProvider_t, App_Factory_t>, this)),
				GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(RegressionFitAnalysisRegressionResidualNormalProbabilityUiManager_t, boost::bind<RegressionFitAnalysisRegressionResidualNormalProbabilityUiManager_t *>(&RegressionModelApplicationContext::fromRegisteredFactory<RegressionFitAnalysisRegressionResidualNormalProbabilityUiManager_t, App_Factory_t>, this)),
				GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(RegressionFitAnalysisResidualNormalProbabilityController_t, boost::bind<RegressionFitAnalysisResidualNormalProbabilityController_t *>(&RegressionModelApplicationContext::fromRegisteredFactory<RegressionFitAnalysisResidualNormalProbabilityController_t, App_Factory_t>, this)),
				GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(RegressionFitAnalysisResidualNormalProbabilityProvider_t, boost::bind<RegressionFitAnalysisResidualNormalProbabilityProvider_t *>(&RegressionModelApplicationContext::fromRegisteredFactory<RegressionFitAnalysisResidualNormalProbabilityProvider_t, App_Factory_t>, this)),
				GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(RegressionFitAnalysisResidualTimeSeriesUiManager_t, boost::bind<RegressionFitAnalysisResidualTimeSeriesUiManager_t *>(&RegressionModelApplicationContext::fromRegisteredFactory<RegressionFitAnalysisResidualTimeSeriesUiManager_t, App_Factory_t>, this)),
				GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(RegressionFitAnalysisResidualTimeSeriesController_t, boost::bind<RegressionFitAnalysisResidualTimeSeriesController_t *>(&RegressionModelApplicationContext::fromRegisteredFactory<RegressionFitAnalysisResidualTimeSeriesController_t, App_Factory_t>, this)),
				GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(RegressionFitAnalysisResidualTimeSeriesProvider_t, boost::bind<RegressionFitAnalysisResidualTimeSeriesProvider_t *>(&RegressionModelApplicationContext::fromRegisteredFactory<RegressionFitAnalysisResidualTimeSeriesProvider_t, App_Factory_t>, this)),
				GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(RegressionFitAnalysisACFUiManager_t, boost::bind<RegressionFitAnalysisACFUiManager_t *>(&RegressionModelApplicationContext::fromRegisteredFactory<RegressionFitAnalysisACFUiManager_t, App_Factory_t>, this)),
				GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(RegressionFitAnalysisACFController_t, boost::bind<RegressionFitAnalysisACFController_t *>(&RegressionModelApplicationContext::fromRegisteredFactory<RegressionFitAnalysisACFController_t, App_Factory_t>, this)),
				GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(RegressionFitAnalysisACFProvider_t, boost::bind<RegressionFitAnalysisACFProvider_t *>(&RegressionModelApplicationContext::fromRegisteredFactory<RegressionFitAnalysisACFProvider_t, App_Factory_t>, this)),
				GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(RegressionFitAnalysisResidualVersusRegressionParameterUiManager_t, boost::bind<RegressionFitAnalysisResidualVersusRegressionParameterUiManager_t *>(&RegressionModelApplicationContext::fromRegisteredFactory<RegressionFitAnalysisResidualVersusRegressionParameterUiManager_t, App_Factory_t>, this)),
				GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(RegressionFitAnalysisResidualVersusRegressionParameterController_t, boost::bind<RegressionFitAnalysisResidualVersusRegressionParameterController_t *>(&RegressionModelApplicationContext::fromRegisteredFactory<RegressionFitAnalysisResidualVersusRegressionParameterController_t, App_Factory_t>, this)),
				GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(RegressionFitAnalysisResidualVersusRegressionParameterProvider_t, boost::bind<RegressionFitAnalysisResidualVersusRegressionParameterProvider_t *>(&RegressionModelApplicationContext::fromRegisteredFactory<RegressionFitAnalysisResidualVersusRegressionParameterProvider_t, App_Factory_t>, this)),
				GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(RegressionFitAnalysisOverallRegressionQualityUiManager_t, boost::bind<RegressionFitAnalysisOverallRegressionQualityUiManager_t *>(&RegressionModelApplicationContext::fromRegisteredFactory<RegressionFitAnalysisOverallRegressionQualityUiManager_t, App_Factory_t>, this)),
				GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(RegressionFitAnalysisOverallRegressionQualityController_t, boost::bind<RegressionFitAnalysisOverallRegressionQualityController_t *>(&RegressionModelApplicationContext::fromRegisteredFactory<RegressionFitAnalysisOverallRegressionQualityController_t, App_Factory_t>, this)),
				GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(RegressionFitAnalysisOverallRegressionQualityProvider_t, boost::bind<RegressionFitAnalysisOverallRegressionQualityProvider_t *>(&RegressionModelApplicationContext::fromRegisteredFactory<RegressionFitAnalysisOverallRegressionQualityProvider_t, App_Factory_t>, this)),
				GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(RegressionFitAnalysisPhysicalBehaviorUiManager_t, boost::bind<RegressionFitAnalysisPhysicalBehaviorUiManager_t *>(&RegressionModelApplicationContext::fromRegisteredFactory<RegressionFitAnalysisPhysicalBehaviorUiManager_t, App_Factory_t>, this)),
				GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(RegressionFitAnalysisPhysicalBehaviorSubscriptionManager_t, boost::bind<RegressionFitAnalysisPhysicalBehaviorSubscriptionManager_t *>(&RegressionModelApplicationContext::fromRegisteredFactory<RegressionFitAnalysisPhysicalBehaviorSubscriptionManager_t, App_Factory_t>, this)),
				GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(RegressionFitAnalysisPhysicalBehaviorController_t, boost::bind<RegressionFitAnalysisPhysicalBehaviorController_t *>(&RegressionModelApplicationContext::fromRegisteredFactory<RegressionFitAnalysisPhysicalBehaviorController_t, App_Factory_t>, this)),
				GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(RegressionFitAnalysisPhysicalBehaviorProvider_t, boost::bind<RegressionFitAnalysisPhysicalBehaviorProvider_t *>(&RegressionModelApplicationContext::fromRegisteredFactory<RegressionFitAnalysisPhysicalBehaviorProvider_t, App_Factory_t>, this)),
				GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(RegressionFitAnalysisRegressorTimeseriesUiManager_t, boost::bind<RegressionFitAnalysisRegressorTimeseriesUiManager_t *>(&RegressionModelApplicationContext::fromRegisteredFactory<RegressionFitAnalysisRegressorTimeseriesUiManager_t, App_Factory_t>, this)),
				GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(RegressionFitAnalysisRegressorTimeseriesController_t, boost::bind<RegressionFitAnalysisRegressorTimeseriesController_t *>(&RegressionModelApplicationContext::fromRegisteredFactory<RegressionFitAnalysisRegressorTimeseriesController_t, App_Factory_t>, this)),
				GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(RegressionFitAnalysisRegressorTimeseriesProvider_t, boost::bind<RegressionFitAnalysisRegressorTimeseriesProvider_t *>(&RegressionModelApplicationContext::fromRegisteredFactory<RegressionFitAnalysisRegressorTimeseriesProvider_t, App_Factory_t>, this)),
				GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(RegressionFitAnalysisRegressorTimeseriesSubscriptionManager_t, boost::bind<RegressionFitAnalysisRegressorTimeseriesSubscriptionManager_t *>(&RegressionModelApplicationContext::fromRegisteredFactory<RegressionFitAnalysisRegressorTimeseriesSubscriptionManager_t, App_Factory_t>, this))

			{;}


}} /* namespace app */


