/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionModelApplicationContext.hpp
 *
 *  Created on: Nov 5, 2013
 *      Author: gerberma
 */

#ifndef REGRESSIONMODELAPPLICATIONCONTEXTA_HPP_
#define REGRESSIONMODELAPPLICATIONCONTEXTA_HPP_

#include "app/types.hpp"
#include "app/GenericApplicationContext.hpp"

#define GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(CLASS, FACTORY) \
	scope##CLASS(registerCtxWithContextScope<GenericApplicationContext<CLASS>, CLASS>(FACTORY))
#define GAC_FULL_CTOR_INIT(CLASS, FACTORY, CLEANUP) \
	scope##CLASS(registerCtxWithContextScope<GenericApplicationContext<CLASS>, CLASS>(FACTORY, CLEANUP))

/*
struct RegressionModel_Analysis_Physics_Category_Provider_t;
struct RegressionModel_Analysis_Statistics_Category_Provider_t;
struct Regression_Formula_Index_Manager_t;
struct Regression_Fit_Provider_t;
struct RegressionModel_Ui_Manager_t;
struct RegressionModel_Internal_Generic_Controller_t;
struct RegressionModel_Controller_t;
struct RegressionModel_Subscription_Manager_t;
struct RegressionModelAnalysisToolsControllerDispatcher_t;
struct RegressionModelAnalysisToolsController_t;
struct RegressionFormulaMethodsController_t;
struct RegressionFitAnalysisPhysicalSummaryUiManager_t;
struct RegressionFitAnalysisPhysicalSummaryController_t;
*/

namespace app {
namespace context {

class RegressionModelApplicationContext {
public:
	RegressionModelApplicationContext();

	template<
		typename T,
		typename Factory
	>
	T *fromRegisteredFactory() {
		Factory *factory = FETCH_FROM_CTX(Factory);
		return factory->template create<T>();
	}

	template<
		typename T,
		typename U,
		typename Factory
	>
	T *fromRegisteredFactory(U &u) {
		Factory *factory = FETCH_FROM_CTX(Factory);
		return factory->template create<T, U>(u);
	}

	template<
		typename T,
		typename U,
		typename W,
		typename Factory
	>
	T *fromRegisteredFactory(U u, W w) {
		Factory *factory = FETCH_FROM_CTX(Factory);
		return factory->template create<T>(u, w);
	}

private:
	GAC_SCOPE(RegressionModel_Analysis_Physics_Category_Provider_t);
	GAC_SCOPE(RegressionModel_Analysis_Statistics_Category_Provider_t);
	GAC_SCOPE(Regression_Formula_Index_Manager_t);
	GAC_SCOPE(Regression_Fit_Provider_t);
	GAC_SCOPE(RegressionModel_Ui_Manager_t);
	GAC_SCOPE(RegressionModel_Internal_Generic_Controller_t);
	GAC_SCOPE(RegressionModel_Controller_t);
	GAC_SCOPE(RegressionModel_Subscription_Manager_t);
	GAC_SCOPE(RegressionModelAnalysisToolsControllerDispatcher_t);
	GAC_SCOPE(RegressionModelAnalysisToolsController_t);
	GAC_SCOPE(RegressionFormulaMethodsController_t);
	GAC_SCOPE(RegressionFitAnalysisPhysicalSummaryUiManager_t);
	GAC_SCOPE(RegressionFitAnalysisPhysicalSummaryController_t);
	GAC_SCOPE(R_RegressionFitAnalysisPhysicalSummaryProvider_t);
	GAC_SCOPE(RegressionFitAnalysisRegressorOverviewUiManager_t);
	GAC_SCOPE(RegressionFitAnalysisRegressorOverviewSubscriptionManager_t);
	GAC_SCOPE(RegressionFitAnalysisRegressorOverviewController_t);
	GAC_SCOPE(RegressionFitAnalysisRegressorOverviewProvider_t);
	GAC_SCOPE(RegressionFitAnalysisRegressionSummaryUiManager_t);
	GAC_SCOPE(RegressionFitAnalysisRegressionSummaryController_t);
	GAC_SCOPE(RegressionFitAnalysisRegressionSummaryProvider_t);
	GAC_SCOPE(RegressionFitAnalysisRegressionResidualNormalProbabilityUiManager_t);
	GAC_SCOPE(RegressionFitAnalysisResidualNormalProbabilityController_t);
	GAC_SCOPE(RegressionFitAnalysisResidualNormalProbabilityProvider_t);
	GAC_SCOPE(RegressionFitAnalysisResidualTimeSeriesUiManager_t);
	GAC_SCOPE(RegressionFitAnalysisResidualTimeSeriesController_t);
	GAC_SCOPE(RegressionFitAnalysisResidualTimeSeriesProvider_t);
	GAC_SCOPE(RegressionFitAnalysisACFUiManager_t);
	GAC_SCOPE(RegressionFitAnalysisACFController_t);
	GAC_SCOPE(RegressionFitAnalysisACFProvider_t);
	GAC_SCOPE(RegressionFitAnalysisResidualVersusRegressionParameterUiManager_t);
	GAC_SCOPE(RegressionFitAnalysisResidualVersusRegressionParameterController_t);
	GAC_SCOPE(RegressionFitAnalysisResidualVersusRegressionParameterProvider_t);
	GAC_SCOPE(RegressionFitAnalysisOverallRegressionQualityUiManager_t);
	GAC_SCOPE(RegressionFitAnalysisOverallRegressionQualityController_t);
	GAC_SCOPE(RegressionFitAnalysisOverallRegressionQualityProvider_t);
	GAC_SCOPE(RegressionFitAnalysisPhysicalBehaviorUiManager_t);
	GAC_SCOPE(RegressionFitAnalysisPhysicalBehaviorSubscriptionManager_t);
	GAC_SCOPE(RegressionFitAnalysisPhysicalBehaviorController_t);
	GAC_SCOPE(RegressionFitAnalysisPhysicalBehaviorProvider_t);
	GAC_SCOPE(RegressionFitAnalysisRegressorTimeseriesUiManager_t);
	GAC_SCOPE(RegressionFitAnalysisRegressorTimeseriesController_t);
	GAC_SCOPE(RegressionFitAnalysisRegressorTimeseriesProvider_t);
	GAC_SCOPE(RegressionFitAnalysisRegressorTimeseriesSubscriptionManager_t);
};

}} /* namespace app::context */
#endif /* REGRESSIONMODELAPPLICATIONCONTEXTA_HPP_ */
