/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * AnalysisUiFactory.cpp
 *
 *  Created on: Dec 3, 2013
 *      Author: gerberma
 */

#include <QObject>
#include <boost/shared_ptr.hpp>
#include "Factory.hpp"
#include "app/GenericApplicationContext.hpp"
#include "app/types.hpp"

using namespace app::context;

namespace app {

template<>
InfoDialogUi_t *Factory::create<InfoDialogUi_t, Analysis_t>(boost::shared_ptr<Analysis_t>) {
	return new InfoDialogUi_t();
}

template<>
InfoDialogUi_t *Factory::create<InfoDialogUi_t, RegressionModel_t>(boost::shared_ptr<RegressionModel_t>) {
	return new InfoDialogUi_t();
}

template<>
HourGlassProgressManager_t *Factory::create<HourGlassProgressManager_t, QApplication>(QApplication &app) {
	return new HourGlassProgressManager_t(app);
}

template<>
Analysis_Ui_Manager_t *Factory::create<Analysis_Ui_Manager_t>() {
	return new Analysis_Ui_Manager_t(FETCH_FROM_CTX(Ui_Factory_t));
}

template<>
Analysis_Overview_Ui_t* Factory::create<Analysis_Overview_Ui_t, Analysis_t>(boost::shared_ptr<Analysis_t> analysis) {
	return new Analysis_Overview_Ui_t(analysis);
}

template<>
Analysis_Wizard_t* Factory::create<Analysis_Wizard_t, Analysis_t>(boost::shared_ptr<Analysis_t> analysis) {
	return new Analysis_Wizard_t(analysis);
}

template<>
DataModel_Overview_Ui_t* Factory::create<DataModel_Overview_Ui_t, DataModel_t>(boost::shared_ptr<DataModel_t>dataModel) {
	return new DataModel_Overview_Ui_t(dataModel, FETCH_FROM_CTX(Model_t)->getDataModelAnalysisToolsDescription());
}

template<>
DataModel_Wizard_t* Factory::create<DataModel_Wizard_t, DataModel_t>(boost::shared_ptr<DataModel_t> dataModel) {
	return new DataModel_Wizard_t(dataModel);
}


/**
 * to partial specialize this, we can introduce a helper which is of a template class which then can get partial specialized
 */
template<>
gui::FileDialog *Factory::create<gui::FileDialog, DataModel_t>(boost::shared_ptr<DataModel_t>) {
	return new gui::FileDialog;
}

template<>
DataModel_Analysis_Overview_Summary_Display_Widget_t *Factory::create<DataModel_Analysis_Overview_Summary_Display_Widget_t, DataModel_Overview_Statistics_t>(boost::shared_ptr<DataModel_Overview_Statistics_t> overviewStatistics) {
	return new DataModel_Analysis_Overview_Summary_Display_Widget_t(overviewStatistics);
}

template<>
DataModel_Analysis_Influence_Factors_Display_Widget_t *Factory::create<DataModel_Analysis_Influence_Factors_Display_Widget_t, DataModel_Influence_Factors_t>(boost::shared_ptr<DataModel_Influence_Factors_t> influenceFactors) {
	return new DataModel_Analysis_Influence_Factors_Display_Widget_t(influenceFactors);
}

template<>
DataModelAnalysisTimeSeriesOverviewConfigurationWidget_t *Factory::create<DataModelAnalysisTimeSeriesOverviewConfigurationWidget_t, DataModelAnalysisTimeSeriesOverview_t>(boost::shared_ptr<DataModelAnalysisTimeSeriesOverview_t> overview) {
	return new DataModelAnalysisTimeSeriesOverviewConfigurationWidget_t(overview);
}

template<>
DataModelAnalysisTimeSeriesOverviewDisplayWidget_t *Factory::create<DataModelAnalysisTimeSeriesOverviewDisplayWidget_t, DataModelAnalysisTimeSeriesOverview_t>(boost::shared_ptr<DataModelAnalysisTimeSeriesOverview_t> overview) {
	return new DataModelAnalysisTimeSeriesOverviewDisplayWidget_t(overview);
}

template<>
DataModelAnalysisTimeSeriesOverviewUiManager_t *Factory::create<DataModelAnalysisTimeSeriesOverviewUiManager_t>() {
	return new DataModelAnalysisTimeSeriesOverviewUiManager_t(FETCH_FROM_CTX(Ui_Factory_t));
}

} /* namespace app */


