/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * AnalysisUiFactory.cpp
 *
 *  Created on: Dec 3, 2013
 *      Author: gerberma
 */

#include <QObject>
#include <boost/shared_ptr.hpp>
#include "Factory.hpp"
#include "app/GenericApplicationContext.hpp"
#include "app/types.hpp"

using namespace app::context;

namespace app {

template<>
AnalysisToolRegressionFitAnalysisPhysicalSummaryDisplayWidget_t *Factory::create<AnalysisToolRegressionFitAnalysisPhysicalSummaryDisplayWidget_t, RegressionFitAnalysisPhysicalSummary_t>(boost::shared_ptr<RegressionFitAnalysisPhysicalSummary_t>) {
	return new AnalysisToolRegressionFitAnalysisPhysicalSummaryDisplayWidget_t();
}

template<>
RegressionFitAnalysisPhysicalSummaryDisplayWidget_t *Factory::create<RegressionFitAnalysisPhysicalSummaryDisplayWidget_t, RegressionFitAnalysisPhysicalSummary_t>(boost::shared_ptr<RegressionFitAnalysisPhysicalSummary_t> summary) {
	return new RegressionFitAnalysisPhysicalSummaryDisplayWidget_t(summary);
}

template<>
AnalysisToolRegressionFitAnalysisRegressorOverviewEditWidget_t *Factory::create<AnalysisToolRegressionFitAnalysisRegressorOverviewEditWidget_t, RegressionFitAnalysisRegressorOverview_t>(boost::shared_ptr<RegressionFitAnalysisRegressorOverview_t>) {
	return new AnalysisToolRegressionFitAnalysisRegressorOverviewEditWidget_t();
}

template<>
RegressionFitAnalysisRegressorOverviewEditWidget_t *Factory::create<RegressionFitAnalysisRegressorOverviewEditWidget_t, RegressionFitAnalysisRegressorOverview_t>(boost::shared_ptr<RegressionFitAnalysisRegressorOverview_t> overview) {
	return new RegressionFitAnalysisRegressorOverviewEditWidget_t(overview);
}

template<>
RegressionFitAnalysisRegressorOverviewUiManager_t *Factory::create<RegressionFitAnalysisRegressorOverviewUiManager_t>() {
	return new RegressionFitAnalysisRegressorOverviewUiManager_t(FETCH_FROM_CTX(Ui_Factory_t));
}

template<>
AnalysisToolRegressionFitAnalysisRegressionSummaryDisplayWidget_t *Factory::create<AnalysisToolRegressionFitAnalysisRegressionSummaryDisplayWidget_t, RegressionFitAnalysisRegressionSummary_t>(boost::shared_ptr<RegressionFitAnalysisRegressionSummary_t>) {
	return new AnalysisToolRegressionFitAnalysisRegressionSummaryDisplayWidget_t();
}

template<>
RegressionFitAnalysisRegressionSummaryDisplayWidget_t *Factory::create<RegressionFitAnalysisRegressionSummaryDisplayWidget_t, RegressionFitAnalysisRegressionSummary_t>(boost::shared_ptr<RegressionFitAnalysisRegressionSummary_t> summary) {
	return new RegressionFitAnalysisRegressionSummaryDisplayWidget_t(summary);
}

template<>
RegressionFitAnalysisRegressionSummaryUiManager_t *Factory::create<RegressionFitAnalysisRegressionSummaryUiManager_t>() {
	return new RegressionFitAnalysisRegressionSummaryUiManager_t(FETCH_FROM_CTX(Ui_Factory_t));
}



template<>
AnalysisToolRegressionFitAnalysisResidualNormalProbabilityDisplayWidget_t *Factory::create<AnalysisToolRegressionFitAnalysisResidualNormalProbabilityDisplayWidget_t, RegressionFitAnalysisResidualNormalProbability_t>(boost::shared_ptr<RegressionFitAnalysisResidualNormalProbability_t>) {
	return new AnalysisToolRegressionFitAnalysisResidualNormalProbabilityDisplayWidget_t();
}

template<>
RegressionFitAnalysisResidualNormalProbabilityDisplayWidget_t *Factory::create<RegressionFitAnalysisResidualNormalProbabilityDisplayWidget_t, RegressionFitAnalysisResidualNormalProbability_t>(boost::shared_ptr<RegressionFitAnalysisResidualNormalProbability_t> normalProbability) {
	return new RegressionFitAnalysisResidualNormalProbabilityDisplayWidget_t(normalProbability);
}

template<>
RegressionFitAnalysisRegressionResidualNormalProbabilityUiManager_t *Factory::create<RegressionFitAnalysisRegressionResidualNormalProbabilityUiManager_t>() {
	return new RegressionFitAnalysisRegressionResidualNormalProbabilityUiManager_t(FETCH_FROM_CTX(Ui_Factory_t));
}



template<>
AnalysisToolRegressionFitAnalysisResidualTimeSeriesDisplayWidget_t *Factory::create<AnalysisToolRegressionFitAnalysisResidualTimeSeriesDisplayWidget_t, RegressionFitAnalysisResidualTimeSeries_t>(boost::shared_ptr<RegressionFitAnalysisResidualTimeSeries_t> ) {
	return new AnalysisToolRegressionFitAnalysisResidualTimeSeriesDisplayWidget_t();
}

template<>
RegressionFitAnalysisResidualTimeSeriesDisplayWidget_t *Factory::create<RegressionFitAnalysisResidualTimeSeriesDisplayWidget_t, RegressionFitAnalysisResidualTimeSeries_t>(boost::shared_ptr<RegressionFitAnalysisResidualTimeSeries_t> residualTimeSeries) {
	return new RegressionFitAnalysisResidualTimeSeriesDisplayWidget_t(residualTimeSeries);
}

template<>
RegressionFitAnalysisResidualTimeSeriesUiManager_t *Factory::create<RegressionFitAnalysisResidualTimeSeriesUiManager_t>() {
	return new RegressionFitAnalysisResidualTimeSeriesUiManager_t(FETCH_FROM_CTX(Ui_Factory_t));
}



template<>
AnalysisToolRegressionFitAnalysisACFDisplayWidget_t *Factory::create<AnalysisToolRegressionFitAnalysisACFDisplayWidget_t, RegressionFitAnalysisACF_t>(boost::shared_ptr<RegressionFitAnalysisACF_t>) {
	return new AnalysisToolRegressionFitAnalysisACFDisplayWidget_t();
}

template<>
RegressionFitAnalysisACFDisplayWidget_t *Factory::create<RegressionFitAnalysisACFDisplayWidget_t, RegressionFitAnalysisACF_t>(boost::shared_ptr<RegressionFitAnalysisACF_t> acf) {
	return new RegressionFitAnalysisACFDisplayWidget_t(acf);
}

template<>
RegressionFitAnalysisACFUiManager_t *Factory::create<RegressionFitAnalysisACFUiManager_t>() {
	return new RegressionFitAnalysisACFUiManager_t(FETCH_FROM_CTX(Ui_Factory_t));
}



template<>
AnalysisToolRegressionFitAnalysisResidualVersusRegressionParameterWidget_t *Factory::create<AnalysisToolRegressionFitAnalysisResidualVersusRegressionParameterWidget_t, RegressionFitAnalysisResidualVersusRegressionParameter_t>(boost::shared_ptr<RegressionFitAnalysisResidualVersusRegressionParameter_t>) {
	return new AnalysisToolRegressionFitAnalysisResidualVersusRegressionParameterWidget_t();
}

template<>
RegressionFitAnalysisResidualVersusRegressionParameterDisplayWidget_t *Factory::create<RegressionFitAnalysisResidualVersusRegressionParameterDisplayWidget_t, RegressionFitAnalysisResidualVersusRegressionParameter_t>(boost::shared_ptr<RegressionFitAnalysisResidualVersusRegressionParameter_t> acf) {
	return new RegressionFitAnalysisResidualVersusRegressionParameterDisplayWidget_t(acf);
}

template<>
RegressionFitAnalysisResidualVersusRegressionParameterUiManager_t *Factory::create<RegressionFitAnalysisResidualVersusRegressionParameterUiManager_t>() {
	return new RegressionFitAnalysisResidualVersusRegressionParameterUiManager_t(FETCH_FROM_CTX(Ui_Factory_t));
}

} /* namespace app */


