/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * AnalysisUiFactory.cpp
 *
 *  Created on: Dec 3, 2013
 *      Author: gerberma
 */

#include <QObject>
#include <boost/shared_ptr.hpp>
#include "Factory.hpp"
#include "app/GenericApplicationContext.hpp"
#include "app/types.hpp"

using namespace app::context;

namespace app {


/****************************
 * Regression model related *
 ****************************/

template<>
RegressionModel_Behavior_Indicator_Display_Widget_t *Factory::create<RegressionModel_Behavior_Indicator_Display_Widget_t, RegressionModel_t>(boost::shared_ptr<RegressionModel_t> regressionModel) {
	return new RegressionModel_Behavior_Indicator_Display_Widget_t(regressionModel);
}

template<>
RegressionModel_Regression_Formula_Display_Widget_t *Factory::create<RegressionModel_Regression_Formula_Display_Widget_t, RegressionModel_t>(boost::shared_ptr<RegressionModel_t> regressionModel) {
	return new RegressionModel_Regression_Formula_Display_Widget_t(regressionModel);
}

template<>
RegressionModel_Algorithm_Display_Widget_t *Factory::create<RegressionModel_Algorithm_Display_Widget_t, RegressionModel_t>(boost::shared_ptr<RegressionModel_t> regressionModel) {
	return new RegressionModel_Algorithm_Display_Widget_t(regressionModel);
}

template<>
RegressionModelExecuteRegressionWidget_t *Factory::create<RegressionModelExecuteRegressionWidget_t, RegressionModel_t>(boost::shared_ptr<RegressionModel_t>regressionModel) {
	return new RegressionModelExecuteRegressionWidget_t(regressionModel);
}


template<>
RegressionModel_Data_Overview_Widget_t *Factory::create<RegressionModel_Data_Overview_Widget_t>() {
	return new RegressionModel_Data_Overview_Widget_t();
}


template<>
RegressionModel_Overview_Widget_t *Factory::create<RegressionModel_Overview_Widget_t>() {
	return new RegressionModel_Overview_Widget_t(QObject::tr("Regression"), QObject::tr("Analysis"));
}

template<>
RegressionModel_Analysis_Widget_t *Factory::create<RegressionModel_Analysis_Widget_t>() {
	return new RegressionModel_Analysis_Widget_t();
}

template<>
RegressionModel_Analysis_Physics_Category_Provider_t *Factory::create<RegressionModel_Analysis_Physics_Category_Provider_t>() {
	return new RegressionModel_Analysis_Physics_Category_Provider_t(FETCH_FROM_CTX(Model_t)->getRegressionModelAnalysisToolsDescription());
}

template<>
RegressionModel_Analysis_Statistics_Category_Provider_t *Factory::create<RegressionModel_Analysis_Statistics_Category_Provider_t>() {
	return new RegressionModel_Analysis_Statistics_Category_Provider_t(FETCH_FROM_CTX(Model_t)->getRegressionModelAnalysisToolsDescription());
}

template<>
RegressionModel_Physics_Category_Widget_t *Factory::create<RegressionModel_Physics_Category_Widget_t>() {
	return new RegressionModel_Physics_Category_Widget_t(FETCH_FROM_CTX(RegressionModel_Analysis_Physics_Category_Provider_t));
}

template<>
RegressionModel_Statistics_Category_Widget_t *Factory::create<RegressionModel_Statistics_Category_Widget_t>() {
	return new RegressionModel_Statistics_Category_Widget_t(FETCH_FROM_CTX(RegressionModel_Analysis_Statistics_Category_Provider_t));
}

} /* namespace app */


