/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * AnalysisUiFactory.cpp
 *
 *  Created on: Dec 3, 2013
 *      Author: gerberma
 */

#include <QObject>
#include <boost/shared_ptr.hpp>
#include "Factory.hpp"
#include "app/GenericApplicationContext.hpp"
#include "app/types.hpp"

using namespace app::context;

namespace app {


/******************
 * Report related *
 ******************/
template<>
ReportUiManager_t *Factory::create<ReportUiManager_t>() {
	return new ReportUiManager_t(FETCH_FROM_CTX(Ui_Factory_t));
}

template<>
ReportWidget_t *Factory::create<ReportWidget_t, Report_t>(boost::shared_ptr<Report_t> report) {
	return new ReportWidget_t(report);
}

template<>
ReportSaveDialog_t *Factory::create<ReportSaveDialog_t, Report_t>(boost::shared_ptr<Report_t> report) {
	return new ReportSaveDialog_t(
				QObject::tr("Report"),
				report->getSupportedFileTypesAsString()
			);
}

template<>
ReportListTitleElementDynamicContentWidget_t *Factory::create<ReportListTitleElementDynamicContentWidget_t, TitleReportListElement_t>(boost::shared_ptr<TitleReportListElement_t> titleElement) {
	return new ReportListTitleElementDynamicContentWidget_t(titleElement);
}

template<>
ReportListTitleElementWidget_t *Factory::create<ReportListTitleElementWidget_t, TitleReportListElement_t>(boost::shared_ptr<TitleReportListElement_t> titleElement) {
	return new ReportListTitleElementWidget_t(titleElement);
}

template<>
ReportListAnalysisToolElementDynamicContentWidget_t *Factory::create<ReportListAnalysisToolElementDynamicContentWidget_t, AnalysisToolReportListElement_t>(boost::shared_ptr<AnalysisToolReportListElement_t> analysisToolElement) {
	return new ReportListAnalysisToolElementDynamicContentWidget_t(analysisToolElement);
}

template<>
ReportListAnalysisToolElementWidget_t *Factory::create<ReportListAnalysisToolElementWidget_t, AnalysisToolReportListElement_t>(boost::shared_ptr<AnalysisToolReportListElement_t> analysisToolElement) {
	return new ReportListAnalysisToolElementWidget_t(analysisToolElement);
}

template<>
ReportListDescriptionElementDynamicContentWidget_t *Factory::create<ReportListDescriptionElementDynamicContentWidget_t, DescriptionReportListElement_t>(boost::shared_ptr<DescriptionReportListElement_t> element) {
	return new ReportListDescriptionElementDynamicContentWidget_t(element);
}

template<>
ReportListDescriptionElementWidget_t *Factory::create<ReportListDescriptionElementWidget_t, DescriptionReportListElement_t>(boost::shared_ptr<DescriptionReportListElement_t> element) {
	return new ReportListDescriptionElementWidget_t(element);
}


template<>
ReportListProjectToolElementDynamicContentWidget_t *Factory::create<ReportListProjectToolElementDynamicContentWidget_t, ProjectReportListElement_t>(boost::shared_ptr<ProjectReportListElement_t> element) {
	return new ReportListProjectToolElementDynamicContentWidget_t(element);
}

template<>
ReportListProjectElementWidget_t *Factory::create<ReportListProjectElementWidget_t, ProjectReportListElement_t>(boost::shared_ptr<ProjectReportListElement_t> element) {
	return new ReportListProjectElementWidget_t(element);
}

template<>
ReportListAlgorithmElementDynamicContentWidget_t *Factory::create<ReportListAlgorithmElementDynamicContentWidget_t, AlgorithmReportListElement_t>(boost::shared_ptr<AlgorithmReportListElement_t> element) {
	return new ReportListAlgorithmElementDynamicContentWidget_t(element);
}

template<>
ReportListAlgorithmElementWidget_t *Factory::create<ReportListAlgorithmElementWidget_t, AlgorithmReportListElement_t>(boost::shared_ptr<AlgorithmReportListElement_t> element) {
	return new ReportListAlgorithmElementWidget_t(element);
}

template<>
RawDataExportUiManager_t *Factory::create<RawDataExportUiManager_t>() {
	return new RawDataExportUiManager_t(FETCH_FROM_CTX(Ui_Factory_t));
}

template<>
RawPlotCollectorSaveDialog_t *Factory::create<RawPlotCollectorSaveDialog_t, RawDataCollector_t>(boost::shared_ptr<RawDataCollector_t> collector) {
	return new RawPlotCollectorSaveDialog_t(
				QObject::tr("Export Plotdata"),
				collector->getSupportedFileTypesAsString()
			);
}

} /* namespace app */


