/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include <QObject>
#include <QDebug>
#include <QProcess>
#include <RInside.h>
#include <boost/bind.hpp>
#include <boost/ref.hpp>
#include <boost/function.hpp>
#include <boost/bind/protect.hpp>
#include <boost/foreach.hpp>
#include <boost/shared_ptr.hpp>
#include "ExceptionNotificationQApplication.hpp"
#include "communication/Signals.hpp"
#include "GenericApplicationContext.hpp"
#include "model/r/RManager.hpp"
#include "types.hpp"
#include "RegressionModelApplicationContext.hpp"
#include "DataModelApplicationContext.hpp"
#include "CoreApplicationContext.hpp"
#include "ProjectApplicationContext.hpp"
#include "AnalysisApplicationContext.hpp"
#include "ReportApplicationContext.hpp"
#include "ExportApplicationContext.hpp"
#include "SerializationApplicationContext.hpp"
#include "CoreSubscriptions.hpp"
#include "CoreDataModelSubscriptions.hpp"
#include "CoreAnalysisSubscriptions.hpp"
#include "CoreRegressionModelSubscriptions.hpp"
#include "CoreRegressionModelSubscriptions2.hpp"
#include "CoreRegressionModelSubscriptions3.hpp"
#include "CoreRegressionModelSubscriptions4.hpp"
#include <boost/config.hpp>
#if defined(BOOST_NO_STDC_NAMESPACE)
namespace std {
	using ::remove;
}
#endif


using namespace app::context;


typedef dambase::model::r::RManager R_Manager_t;

template<
	typename T,
	typename Factory
>
T *fromRegisteredFactory() {
	Factory *factory = FETCH_FROM_CTX(Factory);
	return factory->template create<T>();
}

template<
	typename T,
	typename U,
	typename Factory
>
T *fromRegisteredFactory(U &u) {
	Factory *factory = FETCH_FROM_CTX(Factory);
	return factory->template create<T, U>(u);
}

template<
	typename T,
	typename U,
	typename W,
	typename Factory
>
T *fromRegisteredFactory(U u, W w) {
	Factory *factory = FETCH_FROM_CTX(Factory);
	return factory->template create<T>(u, w);
}


void preSetup() {
	FETCH_FROM_CTX(DirectoryManager_t)->cleanupTemporaryDirectory(); //remove temporary directory
	FETCH_FROM_CTX(R_Manager_t);
}




int
main(int argc, char *argv[]) {
	using namespace boost;

	std::vector<boost::signals2::connection> subscriptionConnections;

	qDebug() << "Path environment: " << QProcessEnvironment::systemEnvironment().value("PATH");

	app::ExceptionNotificationQApplication app(argc, argv);
	qDebug() << "A.... ";
	//QApplication::setStyle("plastique");

	//Core related
	app::context::CoreApplicationContext coreApplicationContext;

	REGISTER_FULL_CTX_WITH_CTX_SCOPE(HourGlassProgressManager_t, boost::bind<HourGlassProgressManager_t *>(fromRegisteredFactory<HourGlassProgressManager_t, QApplication, Ui_Factory_t>, boost::ref(app)), defaultCleanupStrategy<HourGlassProgressManager_t>);
	REGISTER_FULL_CTX_WITH_CTX_SCOPE(R_Manager_t, boost::bind<R_Manager_t *>(fromRegisteredFactory<R_Manager_t, int, char **, Model_Factory_t>, argc, argv), defaultCleanupStrategy<R_Manager_t>);


	//Project related
	app::context::ProjectApplicationContext projectApplicationContext;

	//Analysis related
	app::context::AnalysisApplicationContext analysisApplicationContext;

	//DataModel related
	app::context::DataModelApplicationContext dataModelAppContext;

	app::context::RegressionModelApplicationContext regressionModelAppContext;

	//Report related
	app::context::ReportApplicationContext reportApplicationContext;

	//Export related
	app::context::ExportApplicationContext exportApplicationContext;

	//register loaders
	app::context::SerializationApplicationContext serializationApplicationContext;

	qDebug() << "B.... ";
	preSetup();

	qDebug() << "C.... ";
	Core_Controller_t *coreController = FETCH_FROM_CTX(Core_Controller_t);

	qDebug() << "D.... ";
	app::CoreSubscriptions coreSubscriptions;
	app::CoreDataModelSubscriptions coreDataModelSubscriptions;
	app::CoreAnalysisSubscriptions coreAnalysisSubscriptions;
	app::CoreRegressionModelSubscriptions coreRegressionModelSubscriptions;
	app::CoreRegressionModelSubscriptions2 coreRegressionModelSubscriptions2;
	app::CoreRegressionModelSubscriptions3 coreRegressionModelSubscriptions3;
	app::CoreRegressionModelSubscriptions4 coreRegressionModelSubscriptions4;
	coreSubscriptions.subscribe();
	coreDataModelSubscriptions.subscribe();
	coreAnalysisSubscriptions.subscribe();
	coreRegressionModelSubscriptions.subscribe();
	coreRegressionModelSubscriptions2.subscribe();
	coreRegressionModelSubscriptions3.subscribe();
	coreRegressionModelSubscriptions4.subscribe();

	qDebug() << "1...";
	coreController->start();

	qDebug() << "2...";

	int ret = app.exec();

	qDebug() << "terminated...";

	return ret;
}
