/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelFactoryImpl.cpp
 *
 *  Created on: Dec 17, 2013
 *      Author: gerberma
 */


#include <boost/shared_ptr.hpp>
#include <iostream>
#include <map>
#include <stdexcept>
#include "app/GenericApplicationContext.hpp"
#include "app/Factory.hpp"
#include "app/types.hpp"
#include "model/DataModel.hpp"
#include "model/r/DataFrameProvider.hpp"
#include "model/r/DataFrameOverviewStatisticsProvider.hpp"
#include "model/r/DataFrameInfluenceFactorsProvider.hpp"

namespace app {
template<>
DataModel_t* Factory::create<DataModel_t>() {
	using namespace app::context;
	using namespace dambase::model;

	std::map<QString, QString> availableCSVDelimiters;
	availableCSVDelimiters["Tab"] = "\t";
	availableCSVDelimiters["Comma"] = ",";
	availableCSVDelimiters["Semicolon"] = ";";
	availableCSVDelimiters["Space"] = " ";

	DataModel_t *dataModel = new DataModel(
			FETCH_FROM_CTX(Data_Frame_Provider_t),
			availableCSVDelimiters,
			",",
			"%d.%m.%Y");

	return dataModel;
}

template<>
DataModel_Controller_t *Factory::create<DataModel_Controller_t>(){
	//TODO: provide necessary objects to the constructor
	return new DataModel_Controller_t(
			FETCH_FROM_CTX(Model_t)->getActivityTracker(),
			FETCH_FROM_CTX(DataModel_Analysis_Tools_Controller_t),
			FETCH_FROM_CTX(DataModel_Analysis_Settings_t),
			FETCH_FROM_CTX(DataModel_Ui_Manager_t),
			FETCH_FROM_CTX(DataModel_Controller_Internal_Subscription_Manager_t)
			);
}

template<>
Data_Frame_Provider_t *Factory::create<Data_Frame_Provider_t>() {
	using namespace dambase::model::r;

	return new DataFrameProvider;
}

template<>
DataFrame_Overview_Statistics_Provider_t *Factory::create<DataFrame_Overview_Statistics_Provider_t>() {
	using namespace dambase::model::r;

	return new DataFrameOverviewStatisticsProvider;
}

template<>
DataFrame_Influence_Factors_Provider_t *Factory::create<DataFrame_Influence_Factors_Provider_t>() {
	using namespace dambase::model::r;

	return new DataFrameInfluenceFactorsProvider;
}

}/* namespace app */


