/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressorFormulaBuilderFactorImpl.cpp
 *
 *  Created on: Jun 4, 2014
 *      Author: gerberma
 */

#include <boost/foreach.hpp>
#include <boost/shared_ptr.hpp>
#include <map>
#include <string>
#include "stdexcept"
#include "app/types.hpp"

#include "model/r/MinMaxOpLevelRegressorFormulaBuilderPreprocessor.hpp"
#include "model/r/ExpressionRegressorFormulaBuilderUserParameterType.hpp"
#include "model/r/ExpressionRegressorFormulaBuilder.hpp"



namespace app {


template<>
MinMaxOpLevelRegressorFormulaBuilderPreprocessor_t *Factory::create<MinMaxOpLevelRegressorFormulaBuilderPreprocessor_t, WaterLevelScalingMinMaxOpLevel_t>(boost::shared_ptr<WaterLevelScalingMinMaxOpLevel_t> waterLevelScalingMinMaxOpLevel, QString name, QString formula) {
	using namespace dambase::model::r;

	return new MinMaxOpLevelRegressorFormulaBuilderPreprocessor(waterLevelScalingMinMaxOpLevel, name, formula);
}

template<>
MinMaxOpLevelRegressorFormulaBuilderPreprocessor_t *Factory::create<MinMaxOpLevelRegressorFormulaBuilderPreprocessor_t, MinMaxOpLevelRegressorFormulaBuilderPreprocessor_t>(boost::shared_ptr<MinMaxOpLevelRegressorFormulaBuilderPreprocessor_t> original) {
	using namespace dambase::model::r;

	boost::shared_ptr<MinMaxOpLevelRegressorFormulaBuilderPreprocessor> castedOriginal = boost::static_pointer_cast<MinMaxOpLevelRegressorFormulaBuilderPreprocessor>(original);

	if(castedOriginal == NULL) {
		std::string errorMessage = "Unknown MinMaxOpLevelRegressorFormulaBuilderPreprocessor type in ";
		errorMessage += __FUNCTION__;
		throw std::logic_error(errorMessage);
	}
	return new MinMaxOpLevelRegressorFormulaBuilderPreprocessor(castedOriginal);
}

template<>
ExpressionRegressorFormulaBuilderUserParameterType_t *Factory::create<ExpressionRegressorFormulaBuilderUserParameterType_t>() {
	using namespace dambase::model::r;

	return new ExpressionRegressorFormulaBuilderUserParameterType();
}

template<>
ExpressionRegressorFormulaBuilderUserParameterType_t *Factory::create<ExpressionRegressorFormulaBuilderUserParameterType_t>(boost::shared_ptr<ExpressionRegressorFormulaBuilderUserParameterType_t> original) {
	using namespace dambase::model::r;

	return new ExpressionRegressorFormulaBuilderUserParameterType(original->getValue(), original->isMinimumLeadTime());
}

/**
 * This is for the setup of the ExpressionRegressorFormulaBuilderUserParameterType in genericExpressionRegressorFormulaBuilderFactory
 */
template<>
ExpressionRegressorFormulaBuilderUserParameterType_t *Factory::create<ExpressionRegressorFormulaBuilderUserParameterType_t>(ExpressionRegressorFormulaBuilderUserParameterType_t& original) {
	using namespace dambase::model::r;

	return new ExpressionRegressorFormulaBuilderUserParameterType(original.getValue(), original.isMinimumLeadTime());
}

/**
 * This is for the setup of the ExpressionRegressorFormulaBuilderUserParameterType in genericExpressionRegressorFormulaBuilderFactory
 */
template<>
ExpressionRegressorFormulaBuilderUserParameterTypeImpl_t *Factory::create<ExpressionRegressorFormulaBuilderUserParameterTypeImpl_t, ExpressionRegressorFormulaBuilderUserParameterTypeImpl_t>(boost::shared_ptr<ExpressionRegressorFormulaBuilderUserParameterTypeImpl_t> original) {
	using namespace dambase::model::r;

	return new ExpressionRegressorFormulaBuilderUserParameterType(original->getValue(), original->isMinimumLeadTime());
}

template<>
SeasonalFormulaBuilderDefinition_t *Factory::create<SeasonalFormulaBuilderDefinition_t>(bool isSin, int factor) {
	using namespace dambase::model::r;

	return new SeasonalFormulaBuilderDefinition_t(isSin, factor);
}

template<>
PowerFormulaBuilderDefinition_t *Factory::create<PowerFormulaBuilderDefinition_t>(int degree) {
	using namespace dambase::model::r;

	return new PowerFormulaBuilderDefinition_t(degree);
}

template<>
ChebyshevFormulaBuilderDefinition_t *Factory::create<ChebyshevFormulaBuilderDefinition_t>(int factor) {
	using namespace dambase::model::r;

	return new ChebyshevFormulaBuilderDefinition_t(factor);
}

template<>
ExponentialBuilderDefinition_t *Factory::create<ExponentialBuilderDefinition_t>(int divisor) {
	using namespace dambase::model::r;

	return new ExponentialBuilderDefinition_t(divisor);
}

template<>
SquareRootBuilderDefinition_t *Factory::create<SquareRootBuilderDefinition_t>() {
	using namespace dambase::model::r;

	return new SquareRootBuilderDefinition_t();
}

template<>
LogarithmusNaturalisBuilderDefinition_t *Factory::create<LogarithmusNaturalisBuilderDefinition_t>() {
	using namespace dambase::model::r;

	return new LogarithmusNaturalisBuilderDefinition_t();
}


template<>
DriftPowerBuilderDefinition_t *Factory::create<DriftPowerBuilderDefinition_t>(int power) {
	using namespace dambase::model::r;

	return new DriftPowerBuilderDefinition_t(power);
}

template<>
CreepChebyshevFormulaBuilderDefinition_t *Factory::create<CreepChebyshevFormulaBuilderDefinition_t, ChebyshevFormulaBuilderDefinition_t>(boost::shared_ptr<ChebyshevFormulaBuilderDefinition_t> cheby) {
	using namespace dambase::model::r;

	return new CreepChebyshevFormulaBuilderDefinition_t(cheby, 0.01, 365, 180);
}

template<>
CreepPowerFormulaBuilderDefinition_t *Factory::create<CreepPowerFormulaBuilderDefinition_t, PowerFormulaBuilderDefinition_t>(boost::shared_ptr<PowerFormulaBuilderDefinition_t> power) {
	using namespace dambase::model::r;

	return new CreepPowerFormulaBuilderDefinition_t(power, 0.01, 365, 180);
}

template<>
ExtraSqrtRampBuilderDefinition_t *Factory::create<ExtraSqrtRampBuilderDefinition_t>(int subtrahend) {
	using namespace dambase::model::r;

	return new ExtraSqrtRampBuilderDefinition_t(subtrahend);
}

template<>
ExtraColumnTimesStepBuilderDefinition_t *Factory::create<ExtraColumnTimesStepBuilderDefinition_t>(int subtrahend) {
	using namespace dambase::model::r;

	return new ExtraColumnTimesStepBuilderDefinition_t(subtrahend);
}

template<>
SplineExpressionFormulaBuilderDefinition_t *Factory::create<SplineExpressionFormulaBuilderDefinition_t>(std::vector<double> x, std::vector<double> y) {
	using namespace dambase::model::r;

	return new SplineExpressionFormulaBuilderDefinition_t(x, y);
}

template<>
TemperatureOneDimensionalHeatEquation_t *Factory::create<TemperatureOneDimensionalHeatEquation_t>(int depth) {
	using namespace dambase::model::r;

	return new TemperatureOneDimensionalHeatEquation_t(depth, 0.1, 365, 180);
}

template<>
AverageExpressionFormulaBuilderDefinition_t *Factory::create<AverageExpressionFormulaBuilderDefinition_t>(int averageTimesteps) {
	using namespace dambase::model::r;

	return new AverageExpressionFormulaBuilderDefinition_t(averageTimesteps);
}

template<>
DelayExpressionFormulaBuilderDefinition_t *Factory::create<DelayExpressionFormulaBuilderDefinition_t>(int delayTimesteps) {
	using namespace dambase::model::r;

	return new DelayExpressionFormulaBuilderDefinition_t(delayTimesteps);
}

template<>
DirectExpressionFormulaBuilderDefinition_t *Factory::create<DirectExpressionFormulaBuilderDefinition_t>(QString identifierSuffix, QString formulaSuffix) {
	using namespace dambase::model::r;

	return new DirectExpressionFormulaBuilderDefinition_t(identifierSuffix, formulaSuffix);
}

template<>
ExpressionRegressorFormulaBuilder_t *Factory::create<ExpressionRegressorFormulaBuilder_t>(boost::shared_ptr<ExpressionRegressorFormulaBuilder_t> original, QString newIdentifier, QString newFormula) {
	using namespace dambase::model::r;

	ExpressionRegressorFormulaBuilder_t *builder = new ExpressionRegressorFormulaBuilder(original, newIdentifier, newFormula);

	BOOST_FOREACH(typename ExpressionRegressorFormulaBuilder_t::UserParameter_t userParameter, original->getUserParameters()) {
		boost::shared_ptr<ExpressionRegressorFormulaBuilderUserParameterType_t> cloneUserParameterType(FETCH_FROM_CTX(Model_Factory_t)->create<ExpressionRegressorFormulaBuilderUserParameterType_t>(userParameter.second));
		builder->addUserParameter(userParameter.first, cloneUserParameterType);
	}

	std::vector<boost::shared_ptr<typename ExpressionRegressorFormulaBuilder_t::RegressorFormulaBuilderPreprocessor_t> > newPreprocessors;
	BOOST_FOREACH(boost::shared_ptr<typename ExpressionRegressorFormulaBuilder_t::RegressorFormulaBuilderPreprocessor_t> preprocessor, original->getPreprocessors()) {
		boost::shared_ptr<typename ExpressionRegressorFormulaBuilder_t::RegressorFormulaBuilderPreprocessor_t> castedPreprocessor = boost::static_pointer_cast<MinMaxOpLevelRegressorFormulaBuilderPreprocessor_t>(preprocessor);
		if(castedPreprocessor) {
			boost::shared_ptr<typename ExpressionRegressorFormulaBuilder_t::RegressorFormulaBuilderPreprocessor_t> newPreprocessor(FETCH_FROM_CTX(Model_Factory_t)->create<MinMaxOpLevelRegressorFormulaBuilderPreprocessor_t>(boost::static_pointer_cast<MinMaxOpLevelRegressorFormulaBuilderPreprocessor_t>(preprocessor)));
			newPreprocessors.push_back(newPreprocessor);
		}
	}

	builder->setPreprocessors(newPreprocessors);

	return builder;
}

template<typename Definition>
ExpressionRegressorFormulaBuilder_t *genericExpressionRegressorFormulaBuilderFactory(boost::shared_ptr<Definition> definition, boost::shared_ptr<AssignmentCategory_t> assignmentCategory, std::vector<boost::shared_ptr<AssignmentCategory_t> > assignmentCategories,std::vector<boost::shared_ptr<RegressorFormulaBuilderPreprocessor_t> > preprocessors) {
	using namespace dambase::model::r;

	ExpressionRegressorFormulaBuilder_t *builder = new ExpressionRegressorFormulaBuilder(definition->getIdentifier(), assignmentCategories, assignmentCategory, preprocessors);

	typedef std::pair<QString, boost::shared_ptr<typename Definition::UserParameterType_t> > pair_t;
	BOOST_FOREACH(pair_t userParameter, definition->getUserParameters()) {
		boost::shared_ptr<typename Definition::UserParameterType_t> userParameterType(FETCH_FROM_CTX(Model_Factory_t)->create<typename Definition::UserParameterType_t>(userParameter.second));
		builder->addUserParameter(userParameter.first, userParameterType);

		//builder->addUserParameter(userParameter.first, FETCH_FROM_CTX(Model_Factory_t)->create<typename Definition::UserParameterType_t>(&userParameter.second));
	}

	builder->setFormula(definition->getFormula());
	builder->setFormulaDefinition(definition->getFormulaDefinition());

 return builder;
}

template<typename Definition>
ExpressionRegressorFormulaBuilder_t *genericExpressionRegressorFormulaBuilderFactory(boost::shared_ptr<Definition> definition, boost::shared_ptr<AssignmentCategory_t> assignmentCategory, std::vector<boost::shared_ptr<AssignmentCategory_t> > assignmentCategories) {
	std::vector<boost::shared_ptr<RegressorFormulaBuilderPreprocessor_t> > preprocessors;

	return genericExpressionRegressorFormulaBuilderFactory(definition, assignmentCategory, assignmentCategories, preprocessors);
}

template<>
ExpressionRegressorFormulaBuilder_t *Factory::create<ExpressionRegressorFormulaBuilder_t, SeasonalFormulaBuilderDefinition_t, AssignmentCategory_t, std::vector<boost::shared_ptr<AssignmentCategory_t> > >(boost::shared_ptr<SeasonalFormulaBuilderDefinition_t> definition, boost::shared_ptr<AssignmentCategory_t> assignmentCategory, std::vector<boost::shared_ptr<AssignmentCategory_t> > assignmentCategories) {
	return genericExpressionRegressorFormulaBuilderFactory(definition, assignmentCategory, assignmentCategories);
}

template<>
ExpressionRegressorFormulaBuilder_t *Factory::create<ExpressionRegressorFormulaBuilder_t, ChebyshevFormulaBuilderDefinition_t, AssignmentCategory_t, std::vector<boost::shared_ptr<AssignmentCategory_t > > >(boost::shared_ptr<ChebyshevFormulaBuilderDefinition_t> definition, boost::shared_ptr<AssignmentCategory_t> assignmentCategory, std::vector<boost::shared_ptr<AssignmentCategory_t> > assignmentCategories) {
	return genericExpressionRegressorFormulaBuilderFactory(definition, assignmentCategory, assignmentCategories);
}

template<>
ExpressionRegressorFormulaBuilder_t *Factory::create<ExpressionRegressorFormulaBuilder_t, PowerFormulaBuilderDefinition_t, AssignmentCategory_t, std::vector<boost::shared_ptr<AssignmentCategory_t > >, std::vector<boost::shared_ptr<RegressorFormulaBuilderPreprocessor_t > > >(boost::shared_ptr<PowerFormulaBuilderDefinition_t> definition, boost::shared_ptr<AssignmentCategory_t> assignmentCategory, std::vector<boost::shared_ptr<AssignmentCategory_t> > assignmentCategories, std::vector<boost::shared_ptr<RegressorFormulaBuilderPreprocessor_t> > preprocessors) {
	return genericExpressionRegressorFormulaBuilderFactory(definition, assignmentCategory, assignmentCategories, preprocessors);
}

template<>
ExpressionRegressorFormulaBuilder_t *Factory::create<ExpressionRegressorFormulaBuilder_t, ExponentialBuilderDefinition_t, AssignmentCategory_t, std::vector<boost::shared_ptr<AssignmentCategory_t> > >(boost::shared_ptr<ExponentialBuilderDefinition_t> definition, boost::shared_ptr<AssignmentCategory_t> assignmentCategory, std::vector<boost::shared_ptr<AssignmentCategory_t> > assignmentCategories) {
	return genericExpressionRegressorFormulaBuilderFactory(definition, assignmentCategory, assignmentCategories);
}

template<>
ExpressionRegressorFormulaBuilder_t *Factory::create<ExpressionRegressorFormulaBuilder_t, SquareRootBuilderDefinition_t, AssignmentCategory_t, std::vector<boost::shared_ptr<AssignmentCategory_t> > >(boost::shared_ptr<SquareRootBuilderDefinition_t> definition, boost::shared_ptr<AssignmentCategory_t> assignmentCategory, std::vector<boost::shared_ptr<AssignmentCategory_t> > assignmentCategories) {
	return genericExpressionRegressorFormulaBuilderFactory(definition, assignmentCategory, assignmentCategories);
}


template<>
ExpressionRegressorFormulaBuilder_t *Factory::create<ExpressionRegressorFormulaBuilder_t, LogarithmusNaturalisBuilderDefinition_t, AssignmentCategory_t, std::vector<boost::shared_ptr<AssignmentCategory_t > > >(boost::shared_ptr<LogarithmusNaturalisBuilderDefinition_t> definition, boost::shared_ptr<AssignmentCategory_t> assignmentCategory, std::vector<boost::shared_ptr<AssignmentCategory_t> > assignmentCategories) {
	return genericExpressionRegressorFormulaBuilderFactory(definition, assignmentCategory, assignmentCategories);
}


template<>
ExpressionRegressorFormulaBuilder_t *Factory::create<ExpressionRegressorFormulaBuilder_t, DriftPowerBuilderDefinition_t, AssignmentCategory_t, std::vector<boost::shared_ptr<AssignmentCategory_t> > >(boost::shared_ptr<DriftPowerBuilderDefinition_t> definition, boost::shared_ptr<AssignmentCategory_t> assignmentCategory, std::vector<boost::shared_ptr<AssignmentCategory_t> > assignmentCategories) {
	return genericExpressionRegressorFormulaBuilderFactory(definition, assignmentCategory, assignmentCategories);
}

template<>
ExpressionRegressorFormulaBuilder_t *Factory::create<ExpressionRegressorFormulaBuilder_t, CreepChebyshevFormulaBuilderDefinition_t,  AssignmentCategory_t, std::vector<boost::shared_ptr<AssignmentCategory_t > > >(boost::shared_ptr<CreepChebyshevFormulaBuilderDefinition_t> definition, boost::shared_ptr<AssignmentCategory_t> assignmentCategory, std::vector<boost::shared_ptr<AssignmentCategory_t > > assignmentCategories) {
	return genericExpressionRegressorFormulaBuilderFactory(definition, assignmentCategory, assignmentCategories);
}

template<>
ExpressionRegressorFormulaBuilder_t *Factory::create<ExpressionRegressorFormulaBuilder_t, CreepPowerFormulaBuilderDefinition_t,  AssignmentCategory_t, std::vector<boost::shared_ptr<AssignmentCategory_t > >, std::vector<boost::shared_ptr<RegressorFormulaBuilderPreprocessor_t> > >(boost::shared_ptr<CreepPowerFormulaBuilderDefinition_t> definition, boost::shared_ptr<AssignmentCategory_t> assignmentCategory, std::vector<boost::shared_ptr<AssignmentCategory_t> > assignmentCategories, std::vector<boost::shared_ptr<RegressorFormulaBuilderPreprocessor_t > > preprocessors) {
	return genericExpressionRegressorFormulaBuilderFactory(definition, assignmentCategory, assignmentCategories, preprocessors);
}

template<>
ExpressionRegressorFormulaBuilder_t *Factory::create<ExpressionRegressorFormulaBuilder_t, ExtraSqrtRampBuilderDefinition_t, AssignmentCategory_t, std::vector<boost::shared_ptr<AssignmentCategory_t > > >(boost::shared_ptr<ExtraSqrtRampBuilderDefinition_t> definition, boost::shared_ptr<AssignmentCategory_t> assignmentCategory, std::vector<boost::shared_ptr<AssignmentCategory_t> > assignmentCategories) {
	return genericExpressionRegressorFormulaBuilderFactory(definition, assignmentCategory, assignmentCategories);
}


template<>
ExpressionRegressorFormulaBuilder_t *Factory::create<ExpressionRegressorFormulaBuilder_t, ExtraColumnTimesStepBuilderDefinition_t, AssignmentCategory_t, std::vector<boost::shared_ptr<AssignmentCategory_t> > >(boost::shared_ptr<ExtraColumnTimesStepBuilderDefinition_t> definition, boost::shared_ptr<AssignmentCategory_t> assignmentCategory, std::vector<boost::shared_ptr<AssignmentCategory_t > > assignmentCategories) {
	return genericExpressionRegressorFormulaBuilderFactory(definition, assignmentCategory, assignmentCategories);
}


template<>
ExpressionRegressorFormulaBuilder_t *Factory::create<ExpressionRegressorFormulaBuilder_t, SplineExpressionFormulaBuilderDefinition_t, AssignmentCategory_t, std::vector<boost::shared_ptr<AssignmentCategory_t > > >(boost::shared_ptr<SplineExpressionFormulaBuilderDefinition_t> definition, boost::shared_ptr<AssignmentCategory_t> assignmentCategory, std::vector<boost::shared_ptr<AssignmentCategory_t> > assignmentCategories) {
	return genericExpressionRegressorFormulaBuilderFactory(definition, assignmentCategory, assignmentCategories);
}

template<>
ExpressionRegressorFormulaBuilder_t *Factory::create<ExpressionRegressorFormulaBuilder_t, TemperatureOneDimensionalHeatEquation_t, AssignmentCategory_t, std::vector<boost::shared_ptr<AssignmentCategory_t > > >(boost::shared_ptr<TemperatureOneDimensionalHeatEquation_t> definition, boost::shared_ptr<AssignmentCategory_t> assignmentCategory, std::vector<boost::shared_ptr<AssignmentCategory_t> > assignmentCategories) {
	return genericExpressionRegressorFormulaBuilderFactory(definition, assignmentCategory, assignmentCategories);
}

template<>
ExpressionRegressorFormulaBuilder_t *Factory::create<ExpressionRegressorFormulaBuilder_t, AverageExpressionFormulaBuilderDefinition_t, AssignmentCategory_t, std::vector<boost::shared_ptr<AssignmentCategory_t> > >(boost::shared_ptr<AverageExpressionFormulaBuilderDefinition_t> definition, boost::shared_ptr<AssignmentCategory_t> assignmentCategory, std::vector<boost::shared_ptr<AssignmentCategory_t> > assignmentCategories) {
	return genericExpressionRegressorFormulaBuilderFactory(definition, assignmentCategory, assignmentCategories);
}

template<>
ExpressionRegressorFormulaBuilder_t *Factory::create<ExpressionRegressorFormulaBuilder_t, DelayExpressionFormulaBuilderDefinition_t, AssignmentCategory_t, std::vector<boost::shared_ptr<AssignmentCategory_t> > >(boost::shared_ptr<DelayExpressionFormulaBuilderDefinition_t> definition, boost::shared_ptr<AssignmentCategory_t> assignmentCategory, std::vector<boost::shared_ptr<AssignmentCategory_t> > assignmentCategories) {
	return genericExpressionRegressorFormulaBuilderFactory(definition, assignmentCategory, assignmentCategories);
}

template<>
ExpressionRegressorFormulaBuilder_t *Factory::create<ExpressionRegressorFormulaBuilder_t, DirectExpressionFormulaBuilderDefinition_t, AssignmentCategory_t, std::vector<boost::shared_ptr<AssignmentCategory_t > > >(boost::shared_ptr<DirectExpressionFormulaBuilderDefinition_t> definition, boost::shared_ptr<AssignmentCategory_t> assignmentCategory, std::vector<boost::shared_ptr<AssignmentCategory_t> > assignmentCategories) {
	return genericExpressionRegressorFormulaBuilderFactory(definition, assignmentCategory, assignmentCategories);
}

}  // namespace app
