/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * ReportListElementFactoryImpl.cpp
 *
 *  Created on: Jul 17, 2014
 *      Author: gerberma
 */


#include <QObject>
#include <boost/shared_ptr.hpp>
#include "app/Factory.hpp"
#include "app/types.hpp"

#include "model/TitleReportListElement.hpp"
#include "model/DescriptionReportListElement.hpp"
#include "model/ProjectReportListElement.hpp"
#include "model/AlgorithmReportListElement.hpp"
#include "model/AnalysisToolReportListElement.hpp"

namespace app {

template<>
TitleReportListElement_t *Factory::create<TitleReportListElement_t>(int index) {
	using namespace dambase::model;

	return new TitleReportListElement(QObject::tr("Report"), QObject::tr("This report has been generated by DamBASE"), index);
}

template<>
DescriptionReportListElement_t *Factory::create<DescriptionReportListElement_t>(int index) {
	using namespace dambase::model;

	return new DescriptionReportListElement(QObject::tr("some description comes here"), index);
}

template<>
ProjectReportListElement_t *Factory::create<ProjectReportListElement_t, Project_t, Analysis_t, RegressionModel_t>(boost::shared_ptr<Project_t> project, boost::shared_ptr<Analysis_t> analysis, boost::shared_ptr<RegressionModel_t> regressionModel, int index) {
	using namespace dambase::model;

	return new ProjectReportListElement(project, analysis, regressionModel, index);
}

template<>
AlgorithmReportListElement_t *Factory::create<AlgorithmReportListElement_t, RegressionModel_t>(boost::shared_ptr<RegressionModel_t> regressionModel, int index) {
	using namespace dambase::model;

	return new AlgorithmReportListElement(regressionModel, index);
}

template<>
AnalysisToolReportListElement_t *Factory::create<AnalysisToolReportListElement_t, Analysis_Tool_t>(boost::shared_ptr<Analysis_Tool_t> analysisTool, int index) {
	using namespace dambase::model;

	return new AnalysisToolReportListElement(analysisTool, index);
}

}  // namespace app
