/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * types.hpp
 *
 *  Created on: Dec 17, 2013
 *      Author: gerberma
 */

#include "controller/CoreController.hpp"
#include "controller/ProjectController.hpp"
#include "controller/AnalysisController.hpp"
#include "controller/DataModelController.hpp"
#include "controller/DataModelControllerInternalSubscriptionManager.hpp"
#include "controller/DataModelControllerInternalSubscriptionManagerImpl.hpp"
#include "controller/DataModelAnalysisToolsControllerInternalSubscriptionManager.hpp"
#include "controller/DataModelAnalysisOverviewSummaryController.hpp"
#include "controller/DataModelAnalysisOverviewSummaryControllerSubscriptionManager.hpp"
#include "controller/DataModelAnalysisInfluenceFactorsController.hpp"
#include "controller/DataModelAnalysisInfluenceFactorsControllerSubscriptionManager.hpp"
#include "controller/DataModelAnalysisTimeSeriesOverviewController.hpp"
#include "controller/DataModelAnalysisTimeSeriesOverviewController.hpp"
#include "controller/DataModelAnalysisToolsController.hpp"
#include "controller/DataModelAnalysisToolsControllerDispatcher.hpp"
#include "controller/RegressionModelController.hpp"
#include "controller/RegressionModelAnalysisToolsController.hpp"
#include "controller/RegressionModelAnalysisToolsControllerDispatcher.hpp"
#include "controller/RegressionFormulaMethodsController.hpp"
#include "controller/RegressionFitAnalysisRegressorOverviewController.hpp"
#include "controller/RegressionFitAnalysisRegressorTimeseriesController.hpp"
#include "controller/RegressionFitAnalysisPhysicalBehaviorController.hpp"
#include "controller/ReportController.hpp"
#include "controller/RawDataExportController.hpp"
#include "controller/GenericAnalysisToolController.hpp"
#include "subscription/RegressionModelSubscriptionManager.hpp"
#include "subscription/DataModelAnalysisToolsTimeSeriesOverviewControllerSubscriptionManager.hpp"
#include "subscription/RegressionFitAnalysisRegressorOverviewSubscriptionManager.hpp"
#include "subscription/RegressionFitAnalysisRegressorTimeseriesSubscriptionManager.hpp"
#include "subscription/RegressionFitAnalysisPhysicalBehaviorSubscriptionManager.hpp"
#include "subscription/AnalysisToolsControllerSubscriptionManager.hpp"
#include "subscription/ReportSubscriptionManager.hpp"
#include "gui/InputDialog.hpp"
#include "gui/HourGlassProgressManager.hpp"
#include "gui/TwoSideNamedWidget.hpp"
#include "gui/DataOverviewWidget.hpp"
#include "gui/AnalysisCategoryGenericWidget.hpp"
#include "gui/FileDialog.hpp"
#include "gui/InfoDialog.hpp"
#include "gui/SaveDialog.hpp"
#include "gui/SVGPlotDisplayWidget.hpp"
#include "gui/CoreWindow.hpp"
#include "gui/ProjectWizard.hpp"
#include "gui/ProjectOverviewWidget.hpp"
#include "gui/NavigationTreeWidget.hpp"
#include "gui/DataModelOverviewWidget.hpp"
#include "gui/DataModelWizard.hpp"
#include "gui/AnalysisWizard.hpp"
#include "gui/AnalysisToolWidget.hpp"
#include "gui/AnalysisOverviewWidget.hpp"
#include "gui/DataModelAnalysisOverviewSummaryDisplayWidget.hpp"
#include "gui/DataModelAnalysisInfluenceFactorsDisplayWidget.hpp"
#include "gui/DataModelAnalysisTimeSeriesOverviewConfigurationWidget.hpp"
#include "gui/RegressorInfluenceFactorSelectionWidget.hpp"
#include "gui/RegressorInfluenceFactorInstantiationMessageDialog.hpp"
#include "gui/RegressionModelWizard.hpp"
#include "gui/RegressionModelDefinitionEditWidget.hpp"
#include "gui/RegressorFormulaBuilderEditWidget.hpp"
#include "gui/RegressionModelCommonParameterDisplayWidget.hpp"
#include "gui/RegressionModelTimeRangeDisplayWidget.hpp"
#include "gui/RegressionModelBehaviorIndicatorDisplayWidget.hpp"
#include "gui/RegressionModelRegressionFormulaDisplayWidget.hpp"
#include "gui/RegressionModelAlgorithmDisplayWidget.hpp"
#include "gui/RegressionModelWaterLevelRegressionFormulaEditWidget.hpp"
#include "gui/RegressionModelExecuteRegressionWidget.hpp"
#include "gui/RegressionModelAnalysisWidget.hpp"
#include "gui/RegressionModelAnalysisPhysicsCategoryProvider.hpp"
#include "gui/RegressionModelAnalysisStatisticsCategoryProvider.hpp"
#include "gui/DataModelAnalysisTimeSeriesOverviewConfigurationWidget.hpp"
#include "gui/DataModelAnalysisTimeSeriesOverviewDisplayWidget.hpp"
#include "gui/RegressionFitAnalysisPhysicalSummaryDisplayWidget.hpp"
#include "gui/RegressionFitAnalysisRegressorOverviewEditWidget.hpp"
#include "gui/RegressionFitAnalysisRegressionSummaryDisplayWidget.hpp"
#include "gui/RegressionFitAnalysisResidualNormalProbabilityDisplayWidget.hpp"
#include "gui/RegressionFitAnalysisResidualTimeSeriesDisplayWidget.hpp"
#include "gui/RegressionFitAnalysisACFDisplayWidget.hpp"
#include "gui/RegressionFitAnalysisResidualVersusRegressionParameterDisplayWidget.hpp"
#include "gui/RegressionFitAnalysisOverallRegressionQualityDisplayWidget.hpp"
#include "gui/RegressionFitAnalysisPhysicalBehaviorDisplayWidget.hpp"
#include "gui/RegressionFitAnalysisRegressorTimeseriesDisplayWidget.hpp"
#include "gui/GenericAnalysisToolWidget.hpp"
#include "gui/GenericPushButton.hpp"
#include "gui/ReportWidget.hpp"
#include "gui/ReportListElementWidget.hpp"
#include "gui/ReportTextEditWidget.hpp"
#include "gui/ReportPlaceholderWidget.hpp"
#include "gui/EnhancedQTextEdit.hpp"
#include "ui/AnalysisUiManager.hpp"
#include "ui/DataModelUiManager.hpp"
#include "ui/DataModelAnalysisToolsUiManager.hpp"
#include "ui/DataModelOverviewStatisticsUiManager.hpp"
#include "ui/DataModelAnalysisInfluenceFactorsUiManager.hpp"
#include "ui/DataModelAnalysisTimeSeriesOverviewUiManager.hpp"
#include "ui/RegressionModelUiManager.hpp"
#include "ui/ReportUiManager.hpp"
#include "ui/RawDataExportUiManager.hpp"
#include "ui/GenericAnalysisToolUiManager.hpp"
#include "app/AnalysisToolsFactory.hpp"
#include "model/api/IDirectoryManager.hpp"
#include "model/api/IActivityTracker.hpp"
#include "model/api/IModel.hpp"
#include "model/api/IProject.hpp"
#include "model/api/IWaterLevelScalingMinMaxOpLevel.hpp"
#include "model/AnalysisActionRules.hpp"
#include "model/api/IAnalysis.hpp"
#include "model/r/api/IAssignmentCategory.hpp"
#include "model/api/IDataModel.hpp"
#include "model/api/IDataModelOverviewStatistics.hpp"
#include "model/api/IDataModelAnalysisInfluenceFactors.hpp"
#include "model/api/IDataModelAnalysisTimeSeriesOverview.hpp"
#include "model/r/api/IDataFrameProvider.hpp"
#include "model/r/api/IDataFrameOverviewStatisticsProvider.hpp"
#include "model/r/api/IDataFrameInfluenceFactors.hpp"
#include "model/r/api/IDataFrameInfluenceFactorsProvider.hpp"
#include "model/r/api/IDataFrameTimeSeriesOverviewProvider.hpp"
#include "model/api/IAnalysisToolsDescriptions.hpp"
#include "model/api/IAnalysisToolDescription.hpp"
#include "model/api/IDataModelAnalysisSettings.hpp"
#include "model/api/IDataModelAnalysisToolsDescription.hpp"
#include "model/api/IRegressionModelAnalysisToolsDescription.hpp"
#include "model/api/IRegressionModel.hpp"
#include "model/ReportContexts.hpp"
#include "model/api/IReport.hpp"
#include "model/ReportPolicies.hpp"
#include "model/api/IReportListElement.hpp"
#include "model/api/ITitleReportListElement.hpp"
#include "model/api/IDescriptionReportListElement.hpp"
#include "model/api/IProjectReportListElement.hpp"
#include "model/api/IAlgorithmReportListElement.hpp"
#include "model/api/IAnalysisToolReportListElement.hpp"
#include "model/api/IRawDataCollector.hpp"
#include "model/RawDataCollectionPolicies.hpp"
#include "model/r/api/ITimeRange.hpp"
#include "model/r/api/IIncreasingRegressionFormulaIndexManager.hpp"
#include "model/r/api/IRegressor.hpp"
#include "model/r/api/IRegressionFormula.hpp"
#include "model/r/api/IRegressionModelTemplate.hpp"
#include "model/r/api/IRegressionFitProvider.hpp"
#include "model/api/IRegressionFitAnalysisPhysicalSummary.hpp"
#include "model/r/api/IRRegressionFitAnalysisPhysicalSummary.hpp"
#include "model/r/api/IRegressionFitAnalysisPhysicalSummaryProvider.hpp"
#include "model/r/api/IRRegressionFitAnalysisRegressorOverview.hpp"
#include "model/r/api/IRegressionFitAnalysisRegressorOverviewProvider.hpp"
#include "model/api/IRegressionFitAnalysisRegressorOverview.hpp"
#include "model/api/IRegressionFitAnalysisRegressionSummary.hpp"
#include "model/r/api/IRRegressionFitAnalysisRegressionSummary.hpp"
#include "model/r/api/IRegressionFitAnalysisRegressionSummaryProvider.hpp"
#include "model/r/api/IRRegressionFitAnalysisResidualNormalProbability.hpp"
#include "model/r/api/IRegressionFitAnalysisResidualNormalProbabilityProvider.hpp"
#include "model/api/IRegressionFitAnalysisResidualNormalProbability.hpp"
#include "model/r/api/IRRegressionFitAnalysisResidualTimeSeries.hpp"
#include "model/r/RegressionFitAnalysisResidualTimeSeriesProvider.hpp"
#include "model/api/IRegressionFitAnalysisACF.hpp"
#include "model/r/api/IRRegressionFitAnalysisACF.hpp"
#include "model/r/api/IRegressionFitAnalysisACFProvider.hpp"
#include "model/api/IRegressionFitAnalysisResidualVersusRegressionParameter.hpp"
#include "model/r/api/IRegressionFitAnalysisResidualVersusRegressionParameterProvider.hpp"
#include "model/r/api/IRRegressionFitAnalysisResidualVersusRegressionParameter.hpp"
#include "model/r/api/IRegressionFitAnalysisOverallRegressionQualityProvider.hpp"
#include "model/r/api/IRRegressionFitAnalysisOverallRegressionQuality.hpp"
#include "model/api/IRegressionFitAnalysisOverallRegressionQuality.hpp"
#include "model/api/IRegressionFitAnalysisResidualTimeSeries.hpp"

#include "model/api/IRegressionFitAnalysisPhysicalBehavior.hpp"
#include "model/r/api/IRRegressionFitAnalysisPhysicalBehavior.hpp"
#include "model/r/api/IRegressionFitAnalysisPhysicalBehaviorProvider.hpp"

#include "model/api/IRegressionFitAnalysisRegressorTimeseries.hpp"
#include "model/r/api/IRRegressionFitAnalysisRegressorTimeseries.hpp"
#include "model/r/api/IRegressionFitAnalysisRegressorTimeseriesProvider.hpp"

#include "model/r/api/IRegressorFormulaBuilderPreprocessor.hpp"
#include "model/r/api/IRegressorFormulaBuilderPreprocessor.hpp"
#include "model/r/api/IExpressionRegressorFormulaBuilder.hpp"
#include "model/r/ExpressionRegressorFormulaBuilderUserParameterType.hpp"
#include "model/r/formulabuilder/ExpressionFormulaBuilderDefinitions.hpp"
#include "model/api/IModeHandler.hpp"
#include "model/api/IAnalysisToolModeDescription.hpp"
#include "model/api/IAnalysisToolsModeDescription.hpp"
#include "model/api/IModeSettings.hpp"

#include "Factory.hpp"
#include <QLineEdit>
#include <QTextEdit>

#include "persistence/XmlSerializationPolicies.hpp"
#include "persistence/Loader.hpp"

#include "exceptioning/ExceptionPolicies.hpp"
#include "ExceptionPoliciesImpl.hpp"
#include "memory/OwnerhshipPolicies.hpp"
#include "comparator.hpp"

typedef gui::InfoDialog InfoDialogUi_t;
typedef dambase::model::api::IDirectoryManager DirectoryManager_t;

typedef app::Factory App_Factory_t;
typedef app::Factory Model_Factory_t;
typedef app::Factory Ui_Factory_t;


typedef dambase::model::api::IReportListElement::ReportContext_t ReportContext_t;
typedef dambase::model::api::IAnalysisToolDescription Analysis_Tool_Description_t;
typedef dambase::model::api::IAnalysisTool Analysis_Tool_t;


/**
 * Mode Settings
 */
typedef dambase::model::api::IModeHandler ModeHandler_t;
typedef dambase::model::api::IAnalysisToolModeDescription AnalysisToolModeDescription_t;
typedef dambase::model::api::IAnalysisToolsModeDescription AnalysisToolsModeDescription_t;
typedef dambase::model::api::IModeSettings ModeSettings_t;
/**
 * /Mode Settings
 */

namespace app {
	class AnalysisToolsFactory;
}
typedef app::AnalysisToolsFactory Analysis_Tools_Factory_t;
typedef gui::AnalysisToolWidget Analysis_Tool_Ui_t;
typedef dambase::model::api::IAnalysisToolDescription Analysis_Tool_Description_t;
typedef dambase::model::api::IDataModelAnalysisToolsDescription DataModel_Analysis_Tools_Description_t;
typedef subscription::AnalysisToolsControllerSubscriptionManager AnalysisToolsControllerSubscriptionManager_t;


typedef dambase::model::r::api::IAssignmentCategory AssignmentCategory_t;
/*
typedef typename DataModel_t::Column_t Influence_Factor_t;
typedef typename DataModel_t::DateColumn_t DateColumn_t;
*/
typedef dambase::model::api::IDataModel DataModel_t;
typedef Model_Factory_t DataModel_Factory_t;


typedef dambase::model::r::api::IDataFrameProvider Data_Frame_Provider_t;
typedef gui::DataModelWizard<DataModel_t> DataModel_Wizard_t;
typedef gui::DataModelOverviewWidget<DataModel_t, DataModel_Analysis_Tools_Description_t> DataModel_Overview_Ui_t;
typedef gui::FileDialog DataModel_File_Dialog_t;

typedef dambase::model::api::IAnalysisTool Analysis_Tool_t;

typedef ui::DataModelAnalysisToolsUiManager<Analysis_Tool_t, Analysis_Tool_Ui_t, Ui_Factory_t, memory::CustomOwnershipPolicy> DataModel_Analysis_Tools_Ui_Manager_t;

typedef dambase::model::r::api::ITimeRange TimeRange_t;
typedef dambase::model::r::api::IIncreasingRegressionFormulaIndexManager Regression_Formula_Index_Manager_t;


typedef dambase::model::api::IWaterLevelScalingMinMaxOpLevel WaterLevelScalingMinMaxOpLevel_t;
typedef dambase::model::r::api::IRegressorFormulaBuilderPreprocessor RegressorFormulaBuilderPreprocessor_t;
typedef dambase::model::r::api::IRegressorFormulaBuilderPreprocessor MinMaxOpLevelRegressorFormulaBuilderPreprocessor_t;

typedef dambase::model::r::api::IExpressionRegressorFormulaBuilderUserParameterType ExpressionRegressorFormulaBuilderUserParameterType_t;
typedef dambase::model::r::ExpressionRegressorFormulaBuilderUserParameterType ExpressionRegressorFormulaBuilderUserParameterTypeImpl_t;
typedef dambase::model::r::api::IExpressionRegressorFormulaBuilder ExpressionRegressorFormulaBuilder_t;
typedef dambase::model::r::api::IRegressor Regressor_t;

typedef dambase::model::r::SeasonalFormulaBuilderDefinition<ExpressionRegressorFormulaBuilderUserParameterTypeImpl_t> SeasonalFormulaBuilderDefinition_t;
typedef dambase::model::r::ChebyshevFormulaBuilderDefinition<ExpressionRegressorFormulaBuilderUserParameterTypeImpl_t> ChebyshevFormulaBuilderDefinition_t;
typedef dambase::model::r::PowerFormulaBuilderDefinition<ExpressionRegressorFormulaBuilderUserParameterTypeImpl_t> PowerFormulaBuilderDefinition_t;
typedef dambase::model::r::ExponentialBuilderDefinition<ExpressionRegressorFormulaBuilderUserParameterTypeImpl_t> ExponentialBuilderDefinition_t;
typedef dambase::model::r::SquareRootBuilderDefinition<ExpressionRegressorFormulaBuilderUserParameterTypeImpl_t> SquareRootBuilderDefinition_t;
typedef dambase::model::r::LogarithmusNaturalisBuilderDefinition<ExpressionRegressorFormulaBuilderUserParameterTypeImpl_t> LogarithmusNaturalisBuilderDefinition_t;
typedef dambase::model::r::DriftPowerBuilderDefinition<ExpressionRegressorFormulaBuilderUserParameterTypeImpl_t> DriftPowerBuilderDefinition_t;
typedef dambase::model::r::CreepFormulaBuilderDefinition<ExpressionRegressorFormulaBuilderUserParameterTypeImpl_t, ChebyshevFormulaBuilderDefinition_t> CreepChebyshevFormulaBuilderDefinition_t;
typedef dambase::model::r::CreepFormulaBuilderDefinition<ExpressionRegressorFormulaBuilderUserParameterTypeImpl_t, PowerFormulaBuilderDefinition_t> CreepPowerFormulaBuilderDefinition_t;
typedef dambase::model::r::ExtraSqrtRampBuilderDefinition<ExpressionRegressorFormulaBuilderUserParameterTypeImpl_t> ExtraSqrtRampBuilderDefinition_t;
typedef dambase::model::r::ExtraColumnTimesStepBuilderDefinition<ExpressionRegressorFormulaBuilderUserParameterTypeImpl_t> ExtraColumnTimesStepBuilderDefinition_t;
typedef dambase::model::r::TemperatureOneDimensionalHeatEquation<ExpressionRegressorFormulaBuilderUserParameterTypeImpl_t> TemperatureOneDimensionalHeatEquation_t;
typedef dambase::model::r::SplineExpressionFormulaBuilderDefinition<ExpressionRegressorFormulaBuilderUserParameterTypeImpl_t> SplineExpressionFormulaBuilderDefinition_t;
typedef dambase::model::r::AverageExpressionFormulaBuilderDefinition<ExpressionRegressorFormulaBuilderUserParameterTypeImpl_t> AverageExpressionFormulaBuilderDefinition_t;
typedef dambase::model::r::DelayExpressionFormulaBuilderDefinition<ExpressionRegressorFormulaBuilderUserParameterTypeImpl_t> DelayExpressionFormulaBuilderDefinition_t;
typedef dambase::model::r::DirectExpressionFormulaBuilderDefinition<ExpressionRegressorFormulaBuilderUserParameterTypeImpl_t> DirectExpressionFormulaBuilderDefinition_t;

typedef dambase::model::r::api::IRegressionModelDefinition RegressionModelDefinition_t;

typedef dambase::model::r::api::IRegressionModelTemplate RegressionModelTemplate_t;
typedef dambase::model::r::api::IRegressionFormula Regression_Formula_t;
typedef dambase::model::r::api::IRegressionFitProvider Regression_Fit_Provider_t;
typedef dambase::model::r::api::IRegressionFit RegressionFit_t;
typedef dambase::model::api::IRegressionModel RegressionModel_t;

typedef gui::RegressionModelCommonParameterDisplayWidget<RegressionModel_t> RegressionModel_Common_Parameter_Display_Widget_t;
typedef gui::RegressionModelTimeRangeDisplayWidget<RegressionModel_t> RegressionModel_Time_Range_Display_Widget_t;
typedef gui::RegressionModelBehaviorIndicatorDisplayWidget<RegressionModel_t> RegressionModel_Behavior_Indicator_Display_Widget_t;
typedef gui::RegressionModelRegressionFormulaDisplayWidget<RegressionModel_t> RegressionModel_Regression_Formula_Display_Widget_t;
typedef gui::RegressionModelAlgorithmDisplayWidget<RegressionModel_t> RegressionModel_Algorithm_Display_Widget_t;
typedef gui::RegressionModelExecuteRegressionWidget<RegressionModel_t> RegressionModelExecuteRegressionWidget_t;

typedef dambase::model::api::IRegressionModelAnalysisToolsDescription RegressionModel_Analysis_Tools_Description_t;
typedef gui::RegressionModelAnalysisPhysicsCategoryProvider<RegressionModel_Analysis_Tools_Description_t> RegressionModel_Analysis_Physics_Category_Provider_t;
typedef gui::RegressionModelAnalysisStatisticsCategoryProvider<RegressionModel_Analysis_Tools_Description_t> RegressionModel_Analysis_Statistics_Category_Provider_t;
typedef gui::AnalysisCategoryGenericWidget<RegressionModel_Analysis_Physics_Category_Provider_t> RegressionModel_Physics_Category_Widget_t;
typedef gui::AnalysisCategoryGenericWidget<RegressionModel_Analysis_Statistics_Category_Provider_t> RegressionModel_Statistics_Category_Widget_t;
typedef gui::RegressionModelAnalysisWidget<RegressionModel_Physics_Category_Widget_t, RegressionModel_Statistics_Category_Widget_t> RegressionModel_Analysis_Widget_t;


typedef gui::DataOverviewWidget<1> RegressionModel_Data_Overview_Widget_t;
typedef gui::TwoSideNamedWidget<RegressionModel_Data_Overview_Widget_t, RegressionModel_Analysis_Widget_t> RegressionModel_Overview_Widget_t;
typedef gui::RegressionModelDefinitionEditWidget<Regression_Formula_t, RegressionModelDefinition_t> RegressionModelDefinitionEditWidget_t;
typedef gui::RegressorFormulaBuilderEditWidget<ExpressionRegressorFormulaBuilder_t> RegressorFormulaBuilderEditWidget_t;
typedef gui::RegressionModelWizard<RegressionModel_t> RegressionModel_Wizard_t;
typedef gui::RegressorInfluenceFactorInstantiationMessageDialog<Regressor_t> RegressorInfluenceFactorInstantiationMessageDialog_t;
typedef gui::RegressorInfluenceFactorSelectionWidget<Regressor_t> RegressorInfluenceFactorSelectionWidget_t;



typedef dambase::model::api::IActivityTracker ActivityTracker_t;
typedef dambase::model::api::IAnalysis Analysis_t;
typedef dambase::model::api::IProject Project_t;
typedef dambase::model::api::IAnalysisToolsDescriptions Analysis_Tools_Descriptions_t;
typedef dambase::model::api::IModel Model_t;
typedef Model_Factory_t Project_Factory_t;
typedef gui::CoreWindow<Model_t> Core_Window_t;
typedef gui::ProjectWizard<Project_t> Project_Wizard_t;
typedef gui::ProjectOverviewWidget<Project_t> Project_Overview_Widget_t;
typedef gui::NavigationTreeWidget<Project_t> Navigation_Tree_Widget_t;
typedef controller::ProjectController<
		Project_t,
		Project_Factory_t,
		Project_Wizard_t,
		Model_t,
		Core_Window_t,
		Project_Overview_Widget_t,
		Navigation_Tree_Widget_t,
		gui::FileDialog,
		persistence::XmlSerializationPolicy,
		exceptionpolicies::ExceptionPolicy
> Project_Controller_t;
typedef persistence::ProjectLoader<Project_t> Project_Loader_t;

typedef controller::CoreController<Model_t, Core_Window_t, gui::InfoDialog, Project_Controller_t > Core_Controller_t;



typedef Model_Factory_t Analysis_Factory_t;
typedef gui::AnalysisOverviewWidget<Analysis_t> Analysis_Overview_Ui_t;
typedef gui::AnalysisWizard<Analysis_t> Analysis_Wizard_t;
typedef ui::AnalysisUiManager<Analysis_t, Analysis_Overview_Ui_t, Analysis_Wizard_t, InfoDialogUi_t, Ui_Factory_t, memory::CustomOwnershipPolicy> Analysis_Ui_Manager_t;
typedef controller::AnalysisController<Analysis_t, Core_Window_t, Analysis_Ui_Manager_t, ActivityTracker_t> Analysis_Controller_t;
typedef persistence::Loader<Analysis_t> Analysis_Loader_t;


typedef ui::DataModelUiManager<DataModel_t, DataModel_Overview_Ui_t, DataModel_Wizard_t, DataModel_File_Dialog_t, Core_Window_t, Ui_Factory_t, memory::CustomOwnershipPolicy> DataModel_Ui_Manager_t;

typedef dambase::model::r::api::IDataFrameOverviewStatisticsProvider DataFrame_Overview_Statistics_Provider_t;
typedef dambase::model::api::IDataModelOverviewStatistics DataModel_Overview_Statistics_t;

// NEW APPROACH
typedef controller::DataModelAnalysisToolsControllerDispatcher DataModel_Analysis_Tools_Controller_Dispatcher_t;
typedef controller::DataModelAnalysisToolsController<Analysis_t, Analysis_Tool_Description_t, Analysis_Tool_Ui_t, DataModel_Analysis_Tools_Controller_Dispatcher_t> DataModel_Analysis_Tools_Controller_t;
typedef gui::DataModelAnalysisOverviewSummaryDisplayWidget<DataModel_Overview_Statistics_t> DataModel_Analysis_Overview_Summary_Display_Widget_t;
typedef ui::DataModelOverviewStatisticsUiManager<DataModel_Overview_Statistics_t, DataModel_Analysis_Overview_Summary_Display_Widget_t, Ui_Factory_t, memory::CustomOwnershipPolicy> DataModel_Overview_Statistics_Ui_Manager_t;
typedef controller::DataModelAnalysisOverviewSummaryControllerSubscriptionManager DataModel_Overview_Statistics_Subscription_Manager_t;
typedef controller::DataModelAnalysisOverviewSummaryController<Analysis_t, DataModel_Overview_Statistics_t, Model_Factory_t, DataModel_Overview_Statistics_Ui_Manager_t, DataModel_Overview_Statistics_Subscription_Manager_t> DataModel_Analysis_Overview_Summary_Controller_t;

//influence factors
typedef dambase::model::r::api::IDataFrameInfluenceFactorsProvider DataFrame_Influence_Factors_Provider_t;
typedef dambase::model::api::IDataModelAnalysisInfluenceFactors DataModel_Influence_Factors_t;
typedef gui::DataModelAnalysisInfluenceFactorsDisplayWidget<DataModel_Influence_Factors_t> DataModel_Analysis_Influence_Factors_Display_Widget_t;
typedef ui::DataModelInfluenceFactorsUiManager<DataModel_Influence_Factors_t, DataModel_Analysis_Influence_Factors_Display_Widget_t, Ui_Factory_t, memory::CustomOwnershipPolicy> DataModel_Analysis_Influence_Factors_Ui_Manager_t;
typedef controller::DataModelAnalysisInfluenceFactorsControllerSubscriptionManager DataModel_Analysis_Influence_Factors_Controller_Subscription_Manager_t;
typedef controller::DataModelAnalysisInfluenceFactorsController<Analysis_t, DataModel_Influence_Factors_t, Model_Factory_t, DataModel_Analysis_Influence_Factors_Ui_Manager_t, DataModel_Analysis_Influence_Factors_Controller_Subscription_Manager_t> DataModel_Analysis_Influence_Factors_Controller_t;

//timeseries overview
typedef dambase::model::r::api::IDataFrameTimeSeriesOverviewProvider DataFrameTimeSeriesOverviewProvider_t;
typedef dambase::model::api::IDataModelAnalysisTimeSeriesOverview DataModelAnalysisTimeSeriesOverview_t;
//typedef model::DataModelAnalysisTimeSeriesOverview<DataModel_t> DataModelAnalysisTimeSeriesOverview_t;
typedef gui::DataModelAnalysisTimeSeriesOverviewConfigurationWidget<DataModelAnalysisTimeSeriesOverview_t> DataModelAnalysisTimeSeriesOverviewConfigurationWidget_t;
typedef gui::DataModelAnalysisTimeSeriesOverviewDisplayWidget<DataModelAnalysisTimeSeriesOverview_t, DataModelAnalysisTimeSeriesOverviewConfigurationWidget_t> DataModelAnalysisTimeSeriesOverviewDisplayWidget_t;
typedef ui::DataModelAnalysisTimeSeriesOverviewUiManager<
		DataModelAnalysisTimeSeriesOverview_t,
		DataModelAnalysisTimeSeriesOverviewDisplayWidget_t,
		Ui_Factory_t,
		memory::CustomOwnershipPolicy> DataModelAnalysisTimeSeriesOverviewUiManager_t;
typedef subscription::DataModelAnalysisToolsTimeSeriesOverviewControllerSubscriptionManager<DataModelAnalysisTimeSeriesOverviewDisplayWidget_t> DataModelAnalysisToolsTimeSeriesOverviewControllerSubscriptionManager_t;
typedef controller::DataModelAnalysisTimeSeriesOverviewController<
				DataModelAnalysisTimeSeriesOverview_t,
				Model_Factory_t,
				DataModelAnalysisTimeSeriesOverviewUiManager_t,
				DataModelAnalysisToolsTimeSeriesOverviewControllerSubscriptionManager_t
			> DataModelAnalysisTimeSeriesOverviewController_t;

// /NEW APPROACH
typedef controller::DataModelControllerInternalSubscriptionManagerImpl DataModel_Controller_Internal_Subscription_Manager_Impl_t;
typedef controller::DataModelControllerInternalSubscriptionManager<DataModel_Analysis_Tools_Controller_t> DataModel_Controller_Internal_Subscription_Manager_t;

typedef dambase::model::api::IDataModelAnalysisSettings DataModel_Analysis_Settings_t;
typedef controller::DataModelController<
			DataModel_t,
			ActivityTracker_t,
			DataModel_Analysis_Tools_Controller_t,
			DataModel_Analysis_Settings_t,
			DataModel_Ui_Manager_t,
			DataModel_Controller_Internal_Subscription_Manager_t
		> DataModel_Controller_t;


typedef dambase::model::api::IRawDataCollector RawDataCollector_t;


typedef gui::HourGlassProgressManager HourGlassProgressManager_t;
typedef gui::InputDialog<0> UserParameterNameWidget_t;
typedef ui::RegressionModelUiManager<
		RegressionModel_t,
		RegressionModel_Overview_Widget_t,
		RegressionModel_Common_Parameter_Display_Widget_t,
		RegressionModel_Time_Range_Display_Widget_t,
		RegressionModel_Behavior_Indicator_Display_Widget_t,
		RegressionModel_Regression_Formula_Display_Widget_t,
		RegressionModel_Algorithm_Display_Widget_t,
		RegressionModelExecuteRegressionWidget_t,
		RegressionModel_Wizard_t,
		InfoDialogUi_t,
		RegressorInfluenceFactorInstantiationMessageDialog_t,
		RegressorInfluenceFactorSelectionWidget_t,
		RegressionModelDefinitionEditWidget_t,
		RegressorFormulaBuilderEditWidget_t,
		UserParameterNameWidget_t,
		Core_Window_t,
		Ui_Factory_t,
		HourGlassProgressManager_t,
		memory::CustomOwnershipPolicy> RegressionModel_Ui_Manager_t;

typedef controller::RegressionModelAnalysisToolsControllerDispatcher RegressionModelAnalysisToolsControllerDispatcher_t;
typedef controller::RegressionModelAnalysisToolsController<Analysis_t, RegressionModel_t, Analysis_Tool_Description_t, Analysis_Tool_Ui_t, RegressionModelAnalysisToolsControllerDispatcher_t> RegressionModelAnalysisToolsController_t;
typedef controller::GenericController<RegressionModel_t, RegressionModel_Ui_Manager_t> RegressionModel_Internal_Generic_Controller_t;
typedef controller::RegressionModelController<
			RegressionModel_t,
			RegressionModel_Analysis_Tools_Description_t,
			ActivityTracker_t,
			RegressionModelAnalysisToolsController_t,
			RegressionModel_Ui_Manager_t
		> RegressionModel_Controller_t;
typedef controller::RegressionFormulaMethodsController<RegressionModel_t> RegressionFormulaMethodsController_t;
typedef subscription::RegressionModelSubscriptionManager<
		RegressionModel_Overview_Widget_t,
		RegressionModel_Common_Parameter_Display_Widget_t,
		RegressionModel_Behavior_Indicator_Display_Widget_t,
		RegressionModel_Regression_Formula_Display_Widget_t,
		RegressionModel_Algorithm_Display_Widget_t,
		RegressionModel_Time_Range_Display_Widget_t,
		RegressionModelExecuteRegressionWidget_t,
		RegressionModel_Wizard_t,
		RegressorInfluenceFactorSelectionWidget_t,
		RegressionModelDefinitionEditWidget_t,
		RegressorFormulaBuilderEditWidget_t,
		Model_Factory_t,
		RegressionModel_Controller_t,
		RegressionModelAnalysisToolsController_t,
		RegressionFormulaMethodsController_t> RegressionModel_Subscription_Manager_t;


typedef dambase::model::r::api::IRegressionFitAnalysisPhysicalSummaryProvider R_RegressionFitAnalysisPhysicalSummaryProvider_t;

typedef dambase::model::api::IRegressionFitAnalysisPhysicalSummary RegressionFitAnalysisPhysicalSummary_t;


typedef gui::RegressionFitAnalysisPhysicalSummaryDisplayWidget<RegressionFitAnalysisPhysicalSummary_t> RegressionFitAnalysisPhysicalSummaryDisplayWidget_t;
typedef gui::GenericAnalysisToolWidget<RegressionFitAnalysisPhysicalSummaryDisplayWidget_t> AnalysisToolRegressionFitAnalysisPhysicalSummaryDisplayWidget_t;

typedef ui::GenericAnalysisToolUiManager<
			RegressionFitAnalysisPhysicalSummary_t,
			AnalysisToolRegressionFitAnalysisPhysicalSummaryDisplayWidget_t,
			Ui_Factory_t,
			memory::CustomOwnershipPolicy
		> RegressionFitAnalysisPhysicalSummaryUiManager_t;

typedef controller::GenericAnalysisToolController<
		RegressionFitAnalysisPhysicalSummary_t,
		Model_Factory_t,
		RegressionFitAnalysisPhysicalSummaryUiManager_t,
		AnalysisToolsControllerSubscriptionManager_t> RegressionFitAnalysisPhysicalSummaryController_t;

typedef dambase::model::r::api::IRRegressionFitAnalysisRegressorOverview R_RegressionFitAnalysisRegressorOverview_t;
typedef dambase::model::r::api::IRegressionFitAnalysisRegressorOverviewProvider RegressionFitAnalysisRegressorOverviewProvider_t;
typedef dambase::model::api::IRegressionFitAnalysisRegressorOverview RegressionFitAnalysisRegressorOverview_t;

typedef gui::RegressionFitAnalysisRegressorOverviewEditWidget<RegressionFitAnalysisRegressorOverview_t> RegressionFitAnalysisRegressorOverviewEditWidget_t;
typedef gui::GenericAnalysisToolWidget<RegressionFitAnalysisRegressorOverviewEditWidget_t> AnalysisToolRegressionFitAnalysisRegressorOverviewEditWidget_t;
typedef ui::GenericAnalysisToolUiManager<
			RegressionFitAnalysisRegressorOverview_t,
			AnalysisToolRegressionFitAnalysisRegressorOverviewEditWidget_t,
			Ui_Factory_t,
			memory::CustomOwnershipPolicy> RegressionFitAnalysisRegressorOverviewUiManager_t;
typedef subscription::RegressionFitAnalysisRegressorOverviewSubscriptionManager<AnalysisToolsControllerSubscriptionManager_t> RegressionFitAnalysisRegressorOverviewSubscriptionManager_t;
typedef controller::RegressionFitAnalysisRegressorOverviewController<
		RegressionFitAnalysisRegressorOverview_t,
		Model_Factory_t,
		RegressionFitAnalysisRegressorOverviewUiManager_t,
		RegressionFitAnalysisRegressorOverviewSubscriptionManager_t> RegressionFitAnalysisRegressorOverviewController_t;

typedef dambase::model::r::api::IRegressionFitAnalysisRegressionSummaryProvider RegressionFitAnalysisRegressionSummaryProvider_t;

typedef dambase::model::api::IRegressionFitAnalysisRegressionSummary RegressionFitAnalysisRegressionSummary_t;

typedef gui::RegressionFitAnalysisRegressionSummaryDisplayWidget<RegressionFitAnalysisRegressionSummary_t> RegressionFitAnalysisRegressionSummaryDisplayWidget_t;
typedef gui::GenericAnalysisToolWidget<RegressionFitAnalysisRegressionSummaryDisplayWidget_t> AnalysisToolRegressionFitAnalysisRegressionSummaryDisplayWidget_t;
typedef ui::GenericAnalysisToolUiManager<
			RegressionFitAnalysisRegressionSummary_t,
			AnalysisToolRegressionFitAnalysisRegressionSummaryDisplayWidget_t,
			Ui_Factory_t,
			memory::CustomOwnershipPolicy
		> RegressionFitAnalysisRegressionSummaryUiManager_t;
typedef controller::GenericAnalysisToolController<
			RegressionFitAnalysisRegressionSummary_t,
			Model_Factory_t,
			RegressionFitAnalysisRegressionSummaryUiManager_t,
			AnalysisToolsControllerSubscriptionManager_t
		> RegressionFitAnalysisRegressionSummaryController_t;


typedef dambase::model::r::api::IRegressionFitAnalysisResidualNormalProbabilityProvider RegressionFitAnalysisResidualNormalProbabilityProvider_t;

typedef dambase::model::api::IRegressionFitAnalysisResidualNormalProbability RegressionFitAnalysisResidualNormalProbability_t;

typedef gui::RegressionFitAnalysisResidualNormalProbabilityDisplayWidget<RegressionFitAnalysisResidualNormalProbability_t> RegressionFitAnalysisResidualNormalProbabilityDisplayWidget_t;
typedef gui::GenericAnalysisToolWidget<RegressionFitAnalysisResidualNormalProbabilityDisplayWidget_t> AnalysisToolRegressionFitAnalysisResidualNormalProbabilityDisplayWidget_t;
typedef ui::GenericAnalysisToolUiManager<
			RegressionFitAnalysisResidualNormalProbability_t,
			AnalysisToolRegressionFitAnalysisResidualNormalProbabilityDisplayWidget_t,
			Ui_Factory_t,
			memory::CustomOwnershipPolicy
		> RegressionFitAnalysisRegressionResidualNormalProbabilityUiManager_t;
typedef controller::GenericAnalysisToolController<
			RegressionFitAnalysisResidualNormalProbability_t,
			Model_Factory_t,
			RegressionFitAnalysisRegressionResidualNormalProbabilityUiManager_t,
			AnalysisToolsControllerSubscriptionManager_t
		> RegressionFitAnalysisResidualNormalProbabilityController_t;


typedef dambase::model::r::api::IRRegressionFitAnalysisResidualTimeSeries R_RegressionFitAnalysisResidualTimeSeries_t;
typedef dambase::model::r::api::IRegressionFitAnalysisResidualTimeSeriesProvider RegressionFitAnalysisResidualTimeSeriesProvider_t;

typedef dambase::model::api::IRegressionFitAnalysisResidualTimeSeries RegressionFitAnalysisResidualTimeSeries_t;

typedef gui::RegressionFitAnalysisResidualTimeSeriesDisplayWidget<RegressionFitAnalysisResidualTimeSeries_t> RegressionFitAnalysisResidualTimeSeriesDisplayWidget_t;
typedef gui::GenericAnalysisToolWidget<RegressionFitAnalysisResidualTimeSeriesDisplayWidget_t> AnalysisToolRegressionFitAnalysisResidualTimeSeriesDisplayWidget_t;
typedef ui::GenericAnalysisToolUiManager<
			RegressionFitAnalysisResidualTimeSeries_t,
			AnalysisToolRegressionFitAnalysisResidualTimeSeriesDisplayWidget_t,
			Ui_Factory_t,
			memory::CustomOwnershipPolicy
		> RegressionFitAnalysisResidualTimeSeriesUiManager_t;
typedef controller::GenericAnalysisToolController<
			RegressionFitAnalysisResidualTimeSeries_t,
			Model_Factory_t,
			RegressionFitAnalysisResidualTimeSeriesUiManager_t,
			AnalysisToolsControllerSubscriptionManager_t
		> RegressionFitAnalysisResidualTimeSeriesController_t;


typedef dambase::model::r::api::IRegressionFitAnalysisACFProvider RegressionFitAnalysisACFProvider_t;

typedef dambase::model::api::IRegressionFitAnalysisACF RegressionFitAnalysisACF_t;

typedef gui::RegressionFitAnalysisACFDisplayWidget<RegressionFitAnalysisACF_t> RegressionFitAnalysisACFDisplayWidget_t;
typedef gui::GenericAnalysisToolWidget<RegressionFitAnalysisACFDisplayWidget_t> AnalysisToolRegressionFitAnalysisACFDisplayWidget_t;
typedef ui::GenericAnalysisToolUiManager<
			RegressionFitAnalysisACF_t,
			AnalysisToolRegressionFitAnalysisACFDisplayWidget_t,
			Ui_Factory_t,
			memory::CustomOwnershipPolicy
		> RegressionFitAnalysisACFUiManager_t;
typedef controller::GenericAnalysisToolController<
			RegressionFitAnalysisACF_t,
			Model_Factory_t,
			RegressionFitAnalysisACFUiManager_t,
			AnalysisToolsControllerSubscriptionManager_t
		> RegressionFitAnalysisACFController_t;



typedef dambase::model::r::api::IRegressionFitAnalysisResidualVersusRegressionParameterProvider RegressionFitAnalysisResidualVersusRegressionParameterProvider_t;
typedef dambase::model::api::IRegressionFitAnalysisResidualVersusRegressionParameter RegressionFitAnalysisResidualVersusRegressionParameter_t;

typedef gui::RegressionFitAnalysisResidualVersusRegressionParameterDisplayWidget<RegressionFitAnalysisResidualVersusRegressionParameter_t> RegressionFitAnalysisResidualVersusRegressionParameterDisplayWidget_t;
typedef gui::GenericAnalysisToolWidget<RegressionFitAnalysisResidualVersusRegressionParameterDisplayWidget_t> AnalysisToolRegressionFitAnalysisResidualVersusRegressionParameterWidget_t;
typedef ui::GenericAnalysisToolUiManager<
			RegressionFitAnalysisResidualVersusRegressionParameter_t,
			AnalysisToolRegressionFitAnalysisResidualVersusRegressionParameterWidget_t,
			Ui_Factory_t,
			memory::CustomOwnershipPolicy
		> RegressionFitAnalysisResidualVersusRegressionParameterUiManager_t;
typedef controller::GenericAnalysisToolController<
			RegressionFitAnalysisResidualVersusRegressionParameter_t,
			Model_Factory_t,
			RegressionFitAnalysisResidualVersusRegressionParameterUiManager_t,
			AnalysisToolsControllerSubscriptionManager_t
		> RegressionFitAnalysisResidualVersusRegressionParameterController_t;


typedef dambase::model::r::api::IRRegressionFitAnalysisOverallRegressionQuality R_RegressionFitAnalysisOverallRegressionQuality_t;
typedef dambase::model::r::api::IRegressionFitAnalysisOverallRegressionQualityProvider RegressionFitAnalysisOverallRegressionQualityProvider_t;

typedef dambase::model::api::IRegressionFitAnalysisOverallRegressionQuality RegressionFitAnalysisOverallRegressionQuality_t;


typedef gui::RegressionFitAnalysisOverallRegressionQualityDisplayWidget<RegressionFitAnalysisOverallRegressionQuality_t> RegressionFitAnalysisOverallRegressionQualityDisplayWidget_t;
typedef gui::GenericAnalysisToolWidget<RegressionFitAnalysisOverallRegressionQualityDisplayWidget_t> AnalysisToolRegressionFitAnalysisOverallRegressionQualityWidget_t;
typedef ui::GenericAnalysisToolUiManager<
			RegressionFitAnalysisOverallRegressionQuality_t,
			AnalysisToolRegressionFitAnalysisOverallRegressionQualityWidget_t,
			Ui_Factory_t,
			memory::CustomOwnershipPolicy
		> RegressionFitAnalysisOverallRegressionQualityUiManager_t;
typedef controller::GenericAnalysisToolController<
			RegressionFitAnalysisOverallRegressionQuality_t,
			Model_Factory_t,
			RegressionFitAnalysisOverallRegressionQualityUiManager_t,
			AnalysisToolsControllerSubscriptionManager_t
		> RegressionFitAnalysisOverallRegressionQualityController_t;


typedef dambase::model::r::api::IRegressionFitAnalysisPhysicalBehaviorProvider RegressionFitAnalysisPhysicalBehaviorProvider_t;
typedef dambase::model::api::IRegressionFitAnalysisPhysicalBehavior RegressionFitAnalysisPhysicalBehavior_t;
typedef gui::RegressionFitAnalysisPhysicalBehaviorDisplayWidget<RegressionFitAnalysisPhysicalBehavior_t> RegressionFitAnalysisPhysicalBehaviorDisplayWidget_t;
typedef gui::GenericAnalysisToolWidget<RegressionFitAnalysisPhysicalBehaviorDisplayWidget_t> AnalysisToolRegressionFitAnalysisPhysicalBehaviorWidget_t;
typedef ui::GenericAnalysisToolUiManager<
		RegressionFitAnalysisPhysicalBehavior_t,
		AnalysisToolRegressionFitAnalysisPhysicalBehaviorWidget_t,
			Ui_Factory_t,
			memory::CustomOwnershipPolicy
		> RegressionFitAnalysisPhysicalBehaviorUiManager_t;

typedef subscription::RegressionFitAnalysisPhysicalBehaviorSubscriptionManager<AnalysisToolsControllerSubscriptionManager_t> RegressionFitAnalysisPhysicalBehaviorSubscriptionManager_t;
typedef controller::GenericAnalysisToolController<
		RegressionFitAnalysisPhysicalBehavior_t,
			Model_Factory_t,
			RegressionFitAnalysisPhysicalBehaviorUiManager_t,
			RegressionFitAnalysisPhysicalBehaviorSubscriptionManager_t
		> RegressionFitAnalysisPhysicalBehaviorGenericController_t;
typedef controller::RegressionFitAnalysisPhysicalBehaviorController<
			RegressionFitAnalysisPhysicalBehavior_t,
			RegressionFitAnalysisPhysicalBehaviorGenericController_t
		> RegressionFitAnalysisPhysicalBehaviorController_t;




typedef dambase::model::r::api::IRegressionFitAnalysisRegressorTimeseriesProvider RegressionFitAnalysisRegressorTimeseriesProvider_t;
typedef dambase::model::api::IRegressionFitAnalysisRegressorTimeseries RegressionFitAnalysisRegressorTimeseries_t;
typedef gui::RegressionFitAnalysisRegressorTimeseriesDisplayWidget<RegressionFitAnalysisRegressorTimeseries_t> RegressionFitAnalysisRegressorTimeseriesDisplayWidget_t;
typedef gui::GenericAnalysisToolWidget<RegressionFitAnalysisRegressorTimeseriesDisplayWidget_t> AnalysisToolRegressionFitAnalysisRegressorTimeseriesWidget_t;
typedef ui::GenericAnalysisToolUiManager<
		RegressionFitAnalysisRegressorTimeseries_t,
		AnalysisToolRegressionFitAnalysisRegressorTimeseriesWidget_t,
			Ui_Factory_t,
			memory::CustomOwnershipPolicy
		> RegressionFitAnalysisRegressorTimeseriesUiManager_t;
typedef subscription::RegressionFitAnalysisRegressorTimeseriesSubscriptionManager<AnalysisToolsControllerSubscriptionManager_t> RegressionFitAnalysisRegressorTimeseriesSubscriptionManager_t;
typedef controller::GenericAnalysisToolController<
			RegressionFitAnalysisRegressorTimeseries_t,
			Model_Factory_t,
			RegressionFitAnalysisRegressorTimeseriesUiManager_t,
			RegressionFitAnalysisRegressorTimeseriesSubscriptionManager_t
		> RegressionFitAnalysisRegressorTimeseriesGenericController_t;
typedef controller::RegressionFitAnalysisRegressorTimeseriesController<
			RegressionFitAnalysisRegressorTimeseries_t,
			RegressionFitAnalysisRegressorTimeseriesGenericController_t
		> RegressionFitAnalysisRegressorTimeseriesController_t;



//REPORT RELATED
// note, ReportContext_t has been defined above as it is needed by the AnalysisTools
typedef dambase::model::api::IReportListElement ReportListElement_t;
typedef dambase::model::api::ITitleReportListElement TitleReportListElement_t;
typedef dambase::model::api::IDescriptionReportListElement DescriptionReportListElement_t;
typedef dambase::model::api::IProjectReportListElement ProjectReportListElement_t;
typedef dambase::model::api::IAlgorithmReportListElement AlgorithmReportListElement_t;
typedef dambase::model::api::IAnalysisToolReportListElement AnalysisToolReportListElement_t;


typedef dambase::model::api::IReport Report_t;

typedef gui::ReportWidget<Report_t> ReportWidget_t;
typedef gui::SaveDialog<Report_t> ReportSaveDialog_t;
typedef gui::ReportTextEditWidget<TitleReportListElement_t, QLineEdit> ReportListTitleElementDynamicContentWidget_t;
typedef gui::ReportListElementWidget<TitleReportListElement_t, ReportListTitleElementDynamicContentWidget_t> ReportListTitleElementWidget_t;
typedef gui::ReportPlaceholderWidget<AnalysisToolReportListElement_t> ReportListAnalysisToolElementDynamicContentWidget_t;
typedef gui::ReportListElementWidget<AnalysisToolReportListElement_t, ReportListAnalysisToolElementDynamicContentWidget_t> ReportListAnalysisToolElementWidget_t;
typedef gui::ReportTextEditWidget<DescriptionReportListElement_t, gui::EnhancedQTextEdit> ReportListDescriptionElementDynamicContentWidget_t;
typedef gui::ReportListElementWidget<DescriptionReportListElement_t, ReportListDescriptionElementDynamicContentWidget_t> ReportListDescriptionElementWidget_t;

typedef gui::ReportPlaceholderWidget<ProjectReportListElement_t> ReportListProjectToolElementDynamicContentWidget_t;
typedef gui::ReportListElementWidget<ProjectReportListElement_t, ReportListProjectToolElementDynamicContentWidget_t> ReportListProjectElementWidget_t;

typedef gui::ReportPlaceholderWidget<AlgorithmReportListElement_t> ReportListAlgorithmElementDynamicContentWidget_t;
typedef gui::ReportListElementWidget<AlgorithmReportListElement_t, ReportListAlgorithmElementDynamicContentWidget_t> ReportListAlgorithmElementWidget_t;

typedef ui::ReportUiManager<
			Report_t,
			ReportWidget_t,
			ReportListTitleElementWidget_t,
			ReportListDescriptionElementWidget_t,
			ReportListProjectElementWidget_t,
			ReportListAlgorithmElementWidget_t,
			ReportListAnalysisToolElementWidget_t,
			ReportSaveDialog_t,
			Ui_Factory_t,
			memory::CustomOwnershipPolicy
		> ReportUiManager_t;
typedef controller::GenericController<Report_t, ReportUiManager_t> ReportInternalGenericController_t;
typedef controller::ReportController<
			Report_t,
			ReportUiManager_t,
			App_Factory_t,
			ReportInternalGenericController_t
		> ReportController_t;
typedef subscription::ReportSubscriptionManager<
			ReportWidget_t,
			ReportListTitleElementWidget_t,
			ReportListDescriptionElementWidget_t,
			ReportListProjectElementWidget_t,
			ReportListAlgorithmElementWidget_t,
			ReportListAnalysisToolElementWidget_t,
			ReportController_t
		> ReportSubscriptionManager_t;


typedef gui::SaveDialog<RawDataCollector_t> RawPlotCollectorSaveDialog_t;
typedef ui::RawDataExportUiManager<
			RawDataCollector_t,
			RawPlotCollectorSaveDialog_t,
			Ui_Factory_t,
			memory::CustomOwnershipPolicy
		> RawDataExportUiManager_t;
typedef controller::RawDataExportController<
			RawDataCollector_t, RawDataExportUiManager_t
		> RawDataExportController_t;
