/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * scopes.hpp
 *
 *  Created on: Dec 4, 2013
 *      Author: gerberma
 */

#ifndef SCOPES_HPP_
#define SCOPES_HPP_

#include <boost/bind.hpp>
#include <boost/function.hpp>
#include <boost/utility.hpp>

namespace app {
namespace context {

namespace cleanup {
template<typename T>
void defaultCleanupStrategy(T *t) {
	delete t;
}
} /* namespace cleanup */

template<typename T>
class LocalScope : public boost::noncopyable {
public:
	typedef boost::function<void(T *)> DestroyFunction_t; /* note that you can set _1 or anything on bind */
	typedef boost::function<void(T *)> DelegateFunction_t; /* note that you can set _1 or anything on bind */

	LocalScope(T *reference)
		:reference(reference), destroyFunction(boost::bind(cleanup::defaultCleanupStrategy<T>, _1))
	{
	}

	LocalScope(T *reference, DestroyFunction_t destroyFunction)
		:reference(reference), destroyFunction(destroyFunction)
	{}

	virtual ~LocalScope() {
		destroyFunction(reference);
	}

	T *operator->() {
		return get();
	}

	T *get() {
		return reference;
	}

	void delegate(DelegateFunction_t delegate){
		destroyFunction = delegate;
	}

	void release() {
		using namespace boost;
		delegate(bind(&LocalScope<T>::nullFunction, this, _1));
	}

private:
	void nullFunction(T *){}

private:
	T *reference;
	DestroyFunction_t destroyFunction;
};

}} /* namespace app::context */


#endif /* SCOPES_HPP_ */
