/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * Signals.hpp
 *
 *  Created on: Jan 9, 2014
 *      Author: gerberma
 */

#ifndef SIGNALS_HPP_
#define SIGNALS_HPP_

#include <boost/signals2/signal.hpp>

namespace communication {

class Signals {
public:
	typedef boost::signals2::connection connection_t;
};

template
<
	typename Signature
>
class Signal : public boost::signals2::signal<Signature> {
public:
	typedef boost::signals2::signal<Signature> signal_t;
	typedef typename signal_t::slot_type subscriber_t;
};

class SimpleChangedSignal : public Signal<void()>
{
};

template
<
	typename Signature
>
class IndexChangedSignal : public Signal<void(Signature, int)>
{
};

class CleanupConnection {
public:
	CleanupConnection & operator=(const boost::signals2::connection &connection) {
		this->connection = connection;
		return *this;
	}
	virtual ~CleanupConnection() {
		connection.disconnect();
	}

private:
	boost::signals2::connection connection;
};

} /* namespace communication */




#define COM_CONNECTION communication::CleanupConnection

#define COM_SIGNAL(NAME, SIGNAL_CLASS) \
	SIGNAL_CLASS NAME##Signal

#define COM_SUBSCRIPTION_FUNCTION(NAME, SIGNAL_CLASS) \
	typedef SIGNAL_CLASS __##NAME##_signal_t; \
	COM_SIGNAL(NAME, SIGNAL_CLASS);  \
	boost::signals2::connection subscribeFor##NAME(const typename SIGNAL_CLASS::slot_type &subscriber) { \
		return NAME##Signal.connect(subscriber); \
	}

#define COM_SUBSCRIPTION_FUNCTION_TYPENAMED(NAME, SIGNAL_CLASS) \
	typedef typename SIGNAL_CLASS __##NAME##_signal_t; \
	COM_SIGNAL(NAME, typename SIGNAL_CLASS);  \
	boost::signals2::connection subscribeFor##NAME(const typename SIGNAL_CLASS::slot_type &subscriber) { \
		return NAME##Signal.connect(subscriber); \
	}

#define COM_SUBSCRIPTION_DELEGATE_FUNCTION(DELEGATE_CLASS, DELEGATE, NAME) \
	typedef typename DELEGATE_CLASS::__##NAME##_signal_t __##NAME##_signal_t; \
	boost::signals2::connection subscribeFor##NAME(const typename DELEGATE_CLASS::__##NAME##_signal_t::slot_type &subscriber) { \
		return DELEGATE->subscribeFor##NAME(subscriber); \
	}

/*
 * COM_SUBSCRIPTION_DELEGATE_NAMED_FUNCTION(DataModelCommonParameterDisplayWidget<DataModel>, commonParameterDisplayWidget, Edit, EditCommonParameters);
 * FUNCTIONNAME is the name of the subscribeFor function and NAME the name of the Signal on the delegate
 */
#define COM_SUBSCRIPTION_DELEGATE_NAMED_FUNCTION(DELEGATE_CLASS, DELEGATE, NAME, FUNCTIONNAME) \
	typedef typename DELEGATE_CLASS::__##NAME##_signal_t __##FUNCTIONNAME##_signal_t; \
	boost::signals2::connection subscribeFor##FUNCTIONNAME(const typename DELEGATE_CLASS::__##NAME##_signal_t::slot_type &subscriber) { \
		return DELEGATE->subscribeFor##NAME(subscriber); \
	}

#define COM_SUBSCRIBE_FOR(OBJECT, NAME, FUNCTION) \
		OBJECT->subscribeFor##NAME(FUNCTION)

#define COM_SUBSCRIBE_FOR_REFERENCE(OBJECT, NAME, FUNCTION) \
		OBJECT.subscribeFor##NAME(FUNCTION)

#endif /* SIGNALS_HPP_ */
