/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * CoreController.hpp
 *
 *  Created on: Nov 1, 2013
 *      Author: gerberma
 */

#ifndef CORECONTROLLER_HPP_
#define CORECONTROLLER_HPP_

#include <QObject>
#include <QDebug>
#include <QCloseEvent>
#include <boost/signals2/signal.hpp>
#include <iostream>

namespace controller {

/**
 * CU = CoreUi
 */
template<typename Model, typename CU, typename InfoDialog, typename ProjectController>
class CoreController {
public:
	void start() {
		qDebug() << __FUNCTION__;
		coreUi->start();
	}

	void setCoreUi(CU *coreUi) {
		this->coreUi = coreUi;
	}

	CoreController(Model *model, CU *coreUi, ProjectController *projectController)
		:model(model), coreUi(coreUi), projectController(projectController)
	{
		bool isNull = projectController == NULL;
		qDebug() <<  __FUNCTION__ << " projectController: " << isNull;
	}

	virtual ~CoreController(){
		qDebug() << __FUNCTION__;
	}

	void close() {
		coreUi->close();
	}

	void closeRequested(QCloseEvent *closeEvent) {
		if(model->getActivityTracker()->hasChanged() == true) {
			bool answer = InfoDialog::askYesNo("Project has been changed. Do you want to save it?");

			if(answer == true) {
				projectController->save();
			}
		}

		closeEvent->accept();
	}

	void save();

	void modeChanged(int newMode) {
		model->getModeSettings()->getModeHandler()->setMode(newMode);
	}

private:
	Model *model;
	CU *coreUi;
	ProjectController *projectController;
};



} /* namespace controller */

#endif /* CORECONTROLLER_HPP_ */
