/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelAnalysisInfluenceFactorsController.hpp
 *
 *  Created on: Feb 4, 2014
 *      Author: gerberma
 */

#ifndef DATAMODELANALYSISINFLUENCEFACTORSCONTROLLER_HPP_
#define DATAMODELANALYSISINFLUENCEFACTORSCONTROLLER_HPP_

#include <boost/function.hpp>
#include <boost/shared_ptr.hpp>
#include <iostream>

namespace controller{

template<typename Analysis, typename InfluenceFactors, typename InfluenceFactorsFactory, typename UiManager, typename SubscriptionManager>
class DataModelAnalysisInfluenceFactorsController {
public:
	DataModelAnalysisInfluenceFactorsController(InfluenceFactorsFactory *factory, UiManager *uiManager, SubscriptionManager *subscriptionManager);
	virtual ~DataModelAnalysisInfluenceFactorsController();


	void addAnalysisTool(boost::shared_ptr<Analysis> analysis, boost::function<void(typename UiManager::base_ui_type *)> uiLocation);
	void removeAnalysisTool(boost::shared_ptr<Analysis> analysis, boost::shared_ptr<InfluenceFactors> influenceFactors);

	void executeAnalysis(boost::shared_ptr<InfluenceFactors> influenceFactors);

	void behaviorIndicatorChanged(boost::shared_ptr<InfluenceFactors> influenceFactors, int index);
	void influenceFactorChanged(boost::shared_ptr<InfluenceFactors> influenceFactors, int index);

private:
	InfluenceFactorsFactory *factory;
	UiManager *uiManager;
	SubscriptionManager *subscriptionManager;
};


template
<
	typename Analysis, typename InfluenceFactors,
	typename InfluenceFactorsFactory,
	typename UiManager,
	typename SubscriptionManager
>
inline DataModelAnalysisInfluenceFactorsController<Analysis, InfluenceFactors, InfluenceFactorsFactory, UiManager, SubscriptionManager>::DataModelAnalysisInfluenceFactorsController(InfluenceFactorsFactory *factory, UiManager *uiManager, SubscriptionManager *subscriptionManager)
	:factory(factory), uiManager(uiManager), subscriptionManager(subscriptionManager)
{
}

template
<
	typename Analysis, typename InfluenceFactors,
	typename InfluenceFactorsFactory,
	typename UiManager,
	typename SubscriptionManager
>
inline DataModelAnalysisInfluenceFactorsController<Analysis, InfluenceFactors, InfluenceFactorsFactory, UiManager, SubscriptionManager>::~DataModelAnalysisInfluenceFactorsController() {
}

template
<
	typename Analysis, typename InfluenceFactors,
	typename InfluenceFactorsFactory,
	typename UiManager,
	typename SubscriptionManager
>
inline void DataModelAnalysisInfluenceFactorsController<Analysis, InfluenceFactors, InfluenceFactorsFactory, UiManager, SubscriptionManager>::addAnalysisTool(
		boost::shared_ptr<Analysis> analysis, boost::function<void(typename UiManager::base_ui_type *)> uiLocation)
{
	boost::shared_ptr<InfluenceFactors> influenceFactors(factory->template create<InfluenceFactors>(analysis->getDataModel()));

	influenceFactors->estimateCurrentBehaviorIndicator();
	influenceFactors->estimateCurrentInfluenceFactor();

	influenceFactors->execute();
	analysis->getDataModel()->addDataModelAnalysis(influenceFactors);


	typename UiManager::overview_ui_type *overviewUi = uiManager->getOverviewUiFor(influenceFactors);
	subscriptionManager->subscribeOverviewUi(overviewUi, this, analysis);

	//TODO: keep track of overviewUi

	uiLocation(overviewUi);
}


template
<
	typename Analysis, typename InfluenceFactors,
	typename InfluenceFactorsFactory,
	typename UiManager,
	typename SubscriptionManager
>
inline void controller::DataModelAnalysisInfluenceFactorsController<Analysis, InfluenceFactors, InfluenceFactorsFactory, UiManager, SubscriptionManager>::behaviorIndicatorChanged(boost::shared_ptr<InfluenceFactors> influenceFactors, int index) {
	influenceFactors->setCurrentBehaviorIndicator(index);
	executeAnalysis(influenceFactors);
}

template
<
	typename Analysis, typename InfluenceFactors,
	typename InfluenceFactorsFactory,
	typename UiManager,
	typename SubscriptionManager
>
inline void DataModelAnalysisInfluenceFactorsController<Analysis, InfluenceFactors, InfluenceFactorsFactory, UiManager, SubscriptionManager>::removeAnalysisTool(boost::shared_ptr<Analysis> analysis, boost::shared_ptr<InfluenceFactors> influenceFactors) {
	uiManager->destroyOverviewUiFor(influenceFactors);
	analysis->getDataModel()->removeDataModelAnalysis(influenceFactors);
}

template
<
	typename Analysis, typename InfluenceFactors,
	typename InfluenceFactorsFactory,
	typename UiManager,
	typename SubscriptionManager
>
inline void controller::DataModelAnalysisInfluenceFactorsController<Analysis, InfluenceFactors, InfluenceFactorsFactory, UiManager, SubscriptionManager>::influenceFactorChanged(boost::shared_ptr<InfluenceFactors> influenceFactors, int index) {
	influenceFactors->setCurrentInfluenceFactor(index);
	executeAnalysis(influenceFactors);
}


template<typename Analysis, typename InfluenceFactors, typename InfluenceFactorsFactory, typename UiManager, typename SubscriptionManager>
inline void controller::DataModelAnalysisInfluenceFactorsController<Analysis, InfluenceFactors, InfluenceFactorsFactory, UiManager, SubscriptionManager>::executeAnalysis(boost::shared_ptr<InfluenceFactors> influenceFactors) {
	influenceFactors->execute();
}


} /* namespace controller */



#endif /* DATAMODELANALYSISINFLUENCEFACTORSCONTROLLER_HPP_ */
