/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelAnalysisOverviewSummaryController.hpp
 *
 *  Created on: Jan 22, 2014
 *      Author: gerberma
 */

#ifndef DATAMODELANALYSISOVERVIEWSUMMARYCONTROLLER_HPP_
#define DATAMODELANALYSISOVERVIEWSUMMARYCONTROLLER_HPP_

#include <boost/function.hpp>
#include <boost/shared_ptr.hpp>
#include <iostream>

namespace controller{

template<typename Analysis, typename SummaryOverview, typename SummaryOverviewFactory, typename UiManager, typename SubscriptionManager>
class DataModelAnalysisOverviewSummaryController {
public:
	DataModelAnalysisOverviewSummaryController(SummaryOverviewFactory *factory, UiManager *uiManager, SubscriptionManager *subscriptionManager);
	virtual ~DataModelAnalysisOverviewSummaryController();


	void addAnalysisTool(boost::shared_ptr<Analysis> analysis, boost::function<void(typename UiManager::base_ui_type *)> uiLocation);

	void removeAnalysisTool(boost::shared_ptr<Analysis> analysis, boost::shared_ptr<SummaryOverview> summaryOverview);

	void executeAnalysis(boost::shared_ptr<SummaryOverview> summaryOverview);

private:
	SummaryOverviewFactory *factory;
	UiManager *uiManager;
	SubscriptionManager *subscriptionManager;
};


template
<
	typename Analysis, typename SummaryOverview,
	typename SummaryOverviewFactory,
	typename UiManager,
	typename SubscriptionManager
>
inline DataModelAnalysisOverviewSummaryController<Analysis, SummaryOverview, SummaryOverviewFactory, UiManager, SubscriptionManager>::DataModelAnalysisOverviewSummaryController(SummaryOverviewFactory *factory, UiManager *uiManager, SubscriptionManager *subscriptionManager)
	:factory(factory), uiManager(uiManager), subscriptionManager(subscriptionManager)
{
}

template
<
	typename Analysis, typename SummaryOverview,
	typename SummaryOverviewFactory,
	typename UiManager,
	typename SubscriptionManager
>
inline DataModelAnalysisOverviewSummaryController<Analysis, SummaryOverview, SummaryOverviewFactory, UiManager, SubscriptionManager>::~DataModelAnalysisOverviewSummaryController() {
}

template
<
	typename Analysis, typename SummaryOverview,
	typename SummaryOverviewFactory,
	typename UiManager,
	typename SubscriptionManager
>
inline void DataModelAnalysisOverviewSummaryController<Analysis, SummaryOverview, SummaryOverviewFactory, UiManager, SubscriptionManager>::addAnalysisTool(
		boost::shared_ptr<Analysis> analysis, boost::function<void(typename UiManager::base_ui_type *)> uiLocation)
{
	boost::shared_ptr<SummaryOverview> analysisTool(factory->template create<SummaryOverview>(analysis->getDataModel()));


	analysisTool->execute();
	analysis->getDataModel()->addDataModelAnalysis(analysisTool);


	typename UiManager::overview_ui_type *overviewUi = uiManager->getOverviewUiFor(analysisTool);
	subscriptionManager->subscribeOverviewUi(overviewUi, this, analysis);

	//TODO: keep track of overviewUi

	uiLocation(overviewUi);
}

template
<
	typename Analysis, typename SummaryOverview,
	typename SummaryOverviewFactory,
	typename UiManager,
	typename SubscriptionManager
>
inline void DataModelAnalysisOverviewSummaryController<Analysis, SummaryOverview, SummaryOverviewFactory, UiManager, SubscriptionManager>::removeAnalysisTool(boost::shared_ptr<Analysis> analysis, boost::shared_ptr<SummaryOverview> summaryOverview) {
	//remove analysis tool from analysis list

	uiManager->destroyOverviewUiFor(summaryOverview);
	analysis->getDataModel()->removeDataModelAnalysis(summaryOverview);
}

template
<
	typename Analysis, typename SummaryOverview,
	typename SummaryOverviewFactory,
	typename UiManager,
	typename SubscriptionManager
>
inline void controller::DataModelAnalysisOverviewSummaryController<Analysis, SummaryOverview, SummaryOverviewFactory, UiManager, SubscriptionManager>::executeAnalysis(boost::shared_ptr<SummaryOverview> summaryOverview) {
	summaryOverview->execute();
}

} /* namespace controller */


#endif /* DATAMODELANALYSISOVERVIEWSUMMARYCONTROLLER_HPP_ */
