/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelAnalysisTimeSeriesOverviewController.hpp
 *
 *  Created on: Mar 11, 2014
 *      Author: gerberma
 */

#ifndef DATAMODELANALYSISTIMESERIESOVERVIEWCONTROLLER_HPP_
#define DATAMODELANALYSISTIMESERIESOVERVIEWCONTROLLER_HPP_

#include <QDebug>
#include <boost/shared_ptr.hpp>

namespace controller {

template
<
	typename TimeSeriesOverview,
	typename Factory,
	typename UiManager,
	typename SubscriptionManager
>
class DataModelAnalysisTimeSeriesOverviewController {
public:
	DataModelAnalysisTimeSeriesOverviewController(Factory *factory, UiManager *uiManager, SubscriptionManager *subscriptionManager)
		:factory(factory), uiManager(uiManager), subscriptionManager(subscriptionManager)
	{;}

	template<typename Analysis>
	void addAnalysisTool(boost::shared_ptr<Analysis> analysis, boost::function<void(typename UiManager::base_ui_type *)> uiLocation) {
		boost::shared_ptr<TimeSeriesOverview> timeSeriesOverview(factory->template create<TimeSeriesOverview>(analysis->getDataModel()));

		timeSeriesOverview->execute();
		analysis->getDataModel()->addDataModelAnalysis(timeSeriesOverview);

		typename UiManager::overview_ui_t *overviewUi = uiManager->getOverviewUiFor(timeSeriesOverview);
		subscriptionManager->subscribe(this, overviewUi, analysis);
		subscriptionManager->subscribe(this, uiManager->getConfigurationUiFor(timeSeriesOverview));

		uiLocation(overviewUi);
	}

	template<typename Analysis>
	void removeAnalysisTool(boost::shared_ptr<Analysis> analysis, boost::shared_ptr<TimeSeriesOverview> timeSeriesOverview) {
		uiManager->destroyOverviewUiFor(timeSeriesOverview);
		analysis->getDataModel()->removeDataModelAnalysis(timeSeriesOverview);
	}

	void executeAnalysis(boost::shared_ptr<TimeSeriesOverview> timeSeriesOverview) {
		timeSeriesOverview->execute();
	}

	void valueColumnChanged(boost::shared_ptr<TimeSeriesOverview> timeSeriesOverview, int index) {
		qDebug() << __FUNCTION__ << __LINE__ ;
		timeSeriesOverview->setValueColumn(index);
		executeAnalysis(timeSeriesOverview);
	}

private:
	Factory *factory;
	UiManager *uiManager;
	SubscriptionManager *subscriptionManager;
};

}  // namespace controller


#endif /* DATAMODELANALYSISTIMESERIESOVERVIEWCONTROLLER_HPP_ */
