/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * NewDataModelAnalysisToolsController.hpp
 *
 *  Created on: Jan 31, 2014
 *      Author: gerberma
 */

#ifndef NEWDATAMODELANALYSISTOOLSCONTROLLER_HPP_
#define NEWDATAMODELANALYSISTOOLSCONTROLLER_HPP_

#include <boost/function.hpp>
#include <boost/shared_ptr.hpp>

namespace controller {

template<typename Analysis, typename AnalysisToolDescription, typename BaseUiType, typename Dispatcher>
class DataModelAnalysisToolsController {
public:
	typedef BaseUiType base_ui_t;
	typedef AnalysisToolDescription analysis_tool_description_t;
public:
	DataModelAnalysisToolsController(Dispatcher *dispatcher);
	virtual ~DataModelAnalysisToolsController();

	void addAnalysisTool(boost::shared_ptr<Analysis> analysis, boost::shared_ptr<AnalysisToolDescription> toolDescription, boost::function<void(BaseUiType *)> uiLocation);

private:
	Dispatcher *dispatcher;
};

template
<
	typename Analysis, typename AnalysisToolDescription,
	typename BaseUiType, typename Dispatcher
>
inline DataModelAnalysisToolsController<Analysis, AnalysisToolDescription, BaseUiType, Dispatcher>::DataModelAnalysisToolsController(Dispatcher *dispatcher)
	:dispatcher(dispatcher)
{
}

template
<
	typename Analysis, typename AnalysisToolDescription,
	typename BaseUiType, typename Dispatcher
>
inline DataModelAnalysisToolsController<Analysis, AnalysisToolDescription, BaseUiType, Dispatcher>::~DataModelAnalysisToolsController() {
}

template
<
	typename Analysis, typename AnalysisToolDescription,
	typename BaseUiType, typename Dispatcher
>
inline void DataModelAnalysisToolsController<Analysis, AnalysisToolDescription, BaseUiType, Dispatcher>::addAnalysisTool(
		boost::shared_ptr<Analysis> analysis, boost::shared_ptr<AnalysisToolDescription> toolDescription, boost::function<void(BaseUiType*)> uiLocation)
{
	dispatcher->dispatch(analysis, toolDescription, uiLocation)();
}

} /* namespace controller */

#endif /* NEWDATAMODELANALYSISTOOLSCONTROLLER_HPP_ */
