/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelControllerInternalSubscriptionManager.hpp
 *
 *  Created on: Jan 20, 2014
 *      Author: gerberma
 */

#ifndef DATAMODELCONTROLLERINTERNALSUBSCRIPTIONMANAGER_HPP_
#define DATAMODELCONTROLLERINTERNALSUBSCRIPTIONMANAGER_HPP_

#include <boost/shared_ptr.hpp>
#include "DataModelControllerInternalSubscriptionManagerImpl.hpp"

namespace controller {

template
<
	typename ToolsController,
	typename DelegateSubscriptionManager = DataModelControllerInternalSubscriptionManagerImpl
>
class DataModelControllerInternalSubscriptionManager {
public:
	DataModelControllerInternalSubscriptionManager(DelegateSubscriptionManager *delegateSubscriptionManager, ToolsController *toolsController);
	virtual ~DataModelControllerInternalSubscriptionManager();

	template<typename Wizard, typename Controller>
	void subscribeWizard(Wizard *wizard, Controller *controller);

	template<typename OverviewUi, typename Analysis, typename Controller>
	void subscribeOverviewUi(OverviewUi *overviewUi, boost::shared_ptr<Analysis> analysis, Controller *controller);

	template<typename OverviewUi, typename Analysis>
	void subscribeOverviewUiDelegates(OverviewUi *overviewUi, boost::shared_ptr<Analysis> analysis);

private:
	DelegateSubscriptionManager *delegateSubscriptionManager;
	ToolsController *toolsController;
};


template
<
	typename ToolsController,
	typename DelegateSubscriptionManager
>
inline DataModelControllerInternalSubscriptionManager<ToolsController, DelegateSubscriptionManager>::DataModelControllerInternalSubscriptionManager(DelegateSubscriptionManager *delegateSubscriptionManager, ToolsController *toolsController)
	:delegateSubscriptionManager(delegateSubscriptionManager), toolsController(toolsController)
{
}

template
<
	typename ToolsController,
	typename DelegateSubscriptionManager
>
inline DataModelControllerInternalSubscriptionManager<ToolsController, DelegateSubscriptionManager>::~DataModelControllerInternalSubscriptionManager() {
}

template
<
	typename ToolsController,
	typename DelegateSubscriptionManager
>
template
<
	typename Wizard,
	typename Controller
>
inline void DataModelControllerInternalSubscriptionManager<ToolsController, DelegateSubscriptionManager>::subscribeWizard(Wizard* wizard, Controller* controller) {
	delegateSubscriptionManager->subscribeWizard(wizard, controller);
}


template
<
	typename ToolsController,
	typename DelegateSubscriptionManager
>
template
<
	typename OverviewUi,
	typename Analysis,
	typename Controller
>
inline void DataModelControllerInternalSubscriptionManager<ToolsController, DelegateSubscriptionManager>::subscribeOverviewUi(OverviewUi* overviewUi, boost::shared_ptr<Analysis> analysis, Controller* controller) {
	delegateSubscriptionManager->subscribeOverviewUi(overviewUi, analysis, controller);
}

template
<
	typename ToolsController,
	typename DelegateSubscriptionManager
>
template
<
	typename OverviewUi,
	typename Analysis
>
inline void DataModelControllerInternalSubscriptionManager<ToolsController, DelegateSubscriptionManager>::subscribeOverviewUiDelegates(OverviewUi* overviewUi, boost::shared_ptr<Analysis> analysis) {
	delegateSubscriptionManager->subscribeOverviewUiDelegates(overviewUi, analysis, toolsController);
}

} /* namespace controller */


#endif /* DATAMODELCONTROLLERINTERNALSUBSCRIPTIONMANAGER_HPP_ */
