/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelControllerInternalSubscriptionManagerImpl.hpp
 *
 *  Created on: Jan 21, 2014
 *      Author: gerberma
 */

#ifndef DATAMODELCONTROLLERINTERNALSUBSCRIPTIONMANAGERIMPL_HPP_
#define DATAMODELCONTROLLERINTERNALSUBSCRIPTIONMANAGERIMPL_HPP_

#include "communication/Signals.hpp"
#include <boost/bind.hpp>
#include <boost/function.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/any.hpp>

namespace controller {


class DataModelControllerInternalSubscriptionManagerImpl {
public:
	DataModelControllerInternalSubscriptionManagerImpl();
	virtual ~DataModelControllerInternalSubscriptionManagerImpl();

	template<typename Wizard, typename Controller>
	void subscribeWizard(Wizard *wizard, Controller *controller);

	template<typename OverviewUi, typename Analysis, typename Controller>
	void subscribeOverviewUi(OverviewUi *overviewUi, boost::shared_ptr<Analysis> analysis, Controller *controller);

	template<typename OverviewUi, typename Analysis, typename ToolsController>
	void subscribeOverviewUiDelegates(OverviewUi *overviewUi, boost::shared_ptr<Analysis> analysis, ToolsController *toolsController);
};


inline DataModelControllerInternalSubscriptionManagerImpl::DataModelControllerInternalSubscriptionManagerImpl() {
}

inline DataModelControllerInternalSubscriptionManagerImpl::~DataModelControllerInternalSubscriptionManagerImpl() {
}

template
<
	typename Wizard,
	typename Controller
>
inline void DataModelControllerInternalSubscriptionManagerImpl::subscribeWizard(Wizard* wizard, Controller* controller) {
	COM_SUBSCRIBE_FOR(wizard, RequestSourceUri, (boost::bind(&Controller::requestSourceUri, controller, _1)));
	COM_SUBSCRIBE_FOR(wizard, CSVDelimiterChanged, (boost::bind(&Controller::setCSVDelimiter, controller, _1, _2)));
	COM_SUBSCRIBE_FOR(wizard, DateColumnChanged, (boost::bind(&Controller::dateColumnChange, controller, _1, _2)));
	COM_SUBSCRIBE_FOR(wizard, ApplyDateFormat, (boost::bind(&Controller::applyDateFormat, controller, _1, _2)));
	COM_SUBSCRIBE_FOR(wizard, BehaviorIndicatorAssignmentChanged, (boost::bind(&Controller::behaviorIndicatorAssignmentChanged, controller, _1, _2, _3)));
	COM_SUBSCRIBE_FOR(wizard, WaterLevelAssignmentChanged, (boost::bind(&Controller::waterLevelAssignmentChanged, controller, _1, _2, _3)));
	COM_SUBSCRIBE_FOR(wizard, TemperatureAssignmentChanged, (boost::bind(&Controller::temperatureAssignmentChanged, controller, _1, _2, _3)));
}

template
<
	typename OverviewUi,
	typename Analysis,
	typename Controller
>
inline void DataModelControllerInternalSubscriptionManagerImpl::subscribeOverviewUi(OverviewUi* overviewUi, boost::shared_ptr<Analysis> analysis, Controller* controller) {
	COM_SUBSCRIBE_FOR(overviewUi, EditCommonParameters, (boost::bind(&Controller::template editCommonParameters<Analysis>, controller, _1, analysis)));
	COM_SUBSCRIBE_FOR(overviewUi, EditTimeSeries, (boost::bind(&Controller::template editTimeSeries<Analysis>, controller, _1, analysis)));
	COM_SUBSCRIBE_FOR(overviewUi, EditAssignment, (boost::bind(&Controller::template editAssignment<Analysis>, controller, _1, analysis)));
}

template
<
	typename OverviewUi,
	typename Analysis,
	typename ToolsController
>
inline void DataModelControllerInternalSubscriptionManagerImpl::subscribeOverviewUiDelegates(OverviewUi* overviewUi, boost::shared_ptr<Analysis> analysis, ToolsController* toolsController) {
	using namespace boost;

	COM_SUBSCRIBE_FOR(overviewUi, AddAnalysisTool, (bind((void(ToolsController::*)(boost::shared_ptr<Analysis>, boost::shared_ptr<typename ToolsController::analysis_tool_description_t>, boost::function<void(typename ToolsController::base_ui_t *)>))&ToolsController::addAnalysisTool, toolsController, analysis, _1, _2)));
	COM_SUBSCRIBE_FOR(overviewUi, AddLeftSideAnalysisTool, (bind((void(ToolsController::*)(boost::shared_ptr<Analysis>, boost::shared_ptr<typename ToolsController::analysis_tool_description_t>, boost::function<void(typename ToolsController::base_ui_t *)>))&ToolsController::addAnalysisTool, toolsController, analysis, _1, _2)));

//	COM_SUBSCRIBE_FOR(overviewUi, AddAnalysisTool, (bind((void(ToolsController::*)(Analysis *, typename ToolsController::data_model_analysis_tool_description_t *, boost::function<void(typename ToolsController::ui_manager_t::analysis_tool_ui_type *)>))&ToolsController::addAnalysisTool, toolsController, analysis, _1, _2)));
}

}/* namespace controller */

#endif /* DATAMODELCONTROLLERINTERNALSUBSCRIPTIONMANAGERCONTAINERIMPL_HPP_ */
