/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * GenericAnalysisToolController.hpp
 *
 *  Created on: Mar 28, 2014
 *      Author: gerberma
 */

#ifndef GENERICANALYSISTOOLCONTROLLER_HPP_
#define GENERICANALYSISTOOLCONTROLLER_HPP_

#include <stdexcept>
#include <boost/bind.hpp>
#include <boost/function.hpp>
#include <boost/shared_ptr.hpp>
#include <QDebug>

namespace controller {

template
<
	typename T,
	typename Factory,
	typename UiManager,
	typename SubscriptionManager
>
class GenericAnalysisToolController {
public:
	typedef UiManager UiManager_t;
public:
	GenericAnalysisToolController(Factory *factory, UiManager *uiManager, SubscriptionManager *subscriptionManager)
		:factory(factory), uiManager(uiManager), subscriptionManager(subscriptionManager)
	{;}

	template<typename Analysis, typename RegressionModel>
	void addAnalysisTool(boost::shared_ptr<Analysis> analysis, boost::shared_ptr<RegressionModel> regressionModel, boost::function<void(typename UiManager::base_ui_type *)> uiLocation) {
		if(regressionModel->getRegressionFit() == NULL) {
			throw std::logic_error("calculate regression first before adding this analysis tool");
		}

		boost::shared_ptr<T> t(factory->template create<T>(regressionModel));

		t->execute();
		regressionModel->addRegressionModelAnalysis(t);

		typename UiManager::overview_ui_t *overviewUi = uiManager->getOverviewUiFor(t);
		subscriptionManager->subscribe(this, overviewUi, regressionModel);
		uiLocation(overviewUi);
	}

	template<typename Controller, typename Analysis, typename RegressionModel>
	void addAnalysisTool(Controller *controller, boost::shared_ptr<Analysis> analysis, boost::shared_ptr<RegressionModel> regressionModel, boost::function<void(typename UiManager::base_ui_type *)> uiLocation) {
		if(regressionModel->getRegressionFit() == NULL) {
			throw std::logic_error("calculate regression first before adding this analysis tool");
		}

		boost::shared_ptr<T> t(factory->template create<T>(regressionModel));

		t->execute();
		regressionModel->addRegressionModelAnalysis(t);

		typename UiManager::overview_ui_t *overviewUi = uiManager->getOverviewUiFor(t);
		subscriptionManager->subscribe(controller, overviewUi, regressionModel);
		uiLocation(overviewUi);
	}

	template<typename RegressionModel>
	void removeAnalysisTool(boost::shared_ptr<RegressionModel> regressionModel, boost::shared_ptr<T> t) {
		uiManager->destroyOverviewUiFor(t);
		regressionModel->removeRegressionModelAnalysis(t);
	}

	void executeAnalysis(boost::shared_ptr<T> t) {
		t->execute();
	}

private:
	Factory *factory;
	UiManager *uiManager;
	SubscriptionManager *subscriptionManager;
};

}  // namespace controller


#endif /* GENERICANALYSISTOOLCONTROLLER_HPP_ */
