/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RawDataExportController.hpp
 *
 *  Created on: 07.10.2014
 *      Author: gerberma
 */

#ifndef RAWDATAEXPORTCONTROLLER_HPP_
#define RAWDATAEXPORTCONTROLLER_HPP_

#include <boost/function.hpp>
#include <boost/foreach.hpp>
#include <boost/bind.hpp>
#include <boost/shared_ptr.hpp>
#include <QFile>
#include <QDebug>
#include "applicationcontext/scopes.hpp"

namespace controller {

template<
	typename RawDataCollector,
	typename UiManager
>
class RawDataExportController {
public:
	RawDataExportController(UiManager *uiManager):uiManager(uiManager){;}
	~RawDataExportController(){;}

	template<
		typename Project,
		typename Analysis,
		typename RegressionModel
	>
	void createExportRawDataFor(boost::shared_ptr<Project> project, boost::shared_ptr<Analysis> analysis, boost::shared_ptr<RegressionModel> regressionModel, boost::function<RawDataCollector *()> factory) {
		using namespace boost;
		using namespace app::context;

		boost::shared_ptr<RawDataCollector> rawDataCollector(factory());


		BOOST_FOREACH(boost::shared_ptr<typename RegressionModel::AnalysisTool_t> analysisTool, regressionModel->getAnalysisTools()) {
			if(analysisTool->supportRawDataExport() == true) {
				analysisTool->build(rawDataCollector);
			}
		}

		typedef typename UiManager::save_dialog_ui_t save_dialog_ui_t;
		LocalScope<save_dialog_ui_t> saveDialog(uiManager->getSaveDialogUiFor(rawDataCollector), bind<void>(&UiManager::destroySaveDialogUiFor, uiManager, rawDataCollector));
		QString exportFileLocation = saveDialog->getSaveLocation();

		if(exportFileLocation.isEmpty()) {
				return;
			}

		QFile file(exportFileLocation);
		file.open(QIODevice::WriteOnly | QIODevice::Text);

		rawDataCollector->writeHeader(&file, project, analysis, regressionModel);
		rawDataCollector->write(&file);

		file.close();

		qDebug() << "export done";
	}

private:
	UiManager *uiManager;
};

}  // namespace controller


#endif /* RAWDATAEXPORTCONTROLLER_HPP_ */
