/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitAnalysisPhysicalBehvaiorController.hpp
 *
 *  Created on: 26.11.2014
 *      Author: gerberma
 */

#ifndef REGRESSIONFITANALYSISPHYSICALBEHAVIORCONTROLLER_HPP_
#define REGRESSIONFITANALYSISPHYSICALBEHAVIORCONTROLLER_HPP_

#include <boost/function.hpp>
#include <boost/shared_ptr.hpp>
#include "memory/OwnerhshipPolicies.hpp"

namespace controller {

template
<
	typename PhysicalBehavior,
	typename GenericAnalysisToolController,
	typename GenericAnalysisToolControllerOwnershipPolicy = memory::CustomOwnershipPolicy<GenericAnalysisToolController>
>
class RegressionFitAnalysisPhysicalBehaviorController : GenericAnalysisToolControllerOwnershipPolicy {
public:
	RegressionFitAnalysisPhysicalBehaviorController(GenericAnalysisToolController *genericAnalysisToolController)
		:genericAnalysisToolController(genericAnalysisToolController)
	{
		GenericAnalysisToolControllerOwnershipPolicy::overtakeOwnership(genericAnalysisToolController);
	}

	template<typename Analysis, typename RegressionModel>
	void addAnalysisTool(boost::shared_ptr<Analysis> analysis, boost::shared_ptr<RegressionModel> regressionModel, boost::function<void(typename GenericAnalysisToolController::UiManager_t::base_ui_type *)> uiLocation) {
		genericAnalysisToolController->addAnalysisTool(this, analysis, regressionModel, uiLocation);
	}

	template<typename RegressionModel>
	void removeAnalysisTool(boost::shared_ptr<RegressionModel> regressionModel, boost::shared_ptr<PhysicalBehavior> pyhsicalBehavior) {
		genericAnalysisToolController->removeAnalysisTool(regressionModel, pyhsicalBehavior);
	}

	void executeAnalysis(boost::shared_ptr<PhysicalBehavior> pyhsicalBehavior) {
		genericAnalysisToolController->executeAnalysis(pyhsicalBehavior);
	}

	void setAssignmentCategory(boost::shared_ptr<PhysicalBehavior> physicalBehavior, QString assignmentCategoryIdentifier) {
		physicalBehavior->setAssignmentCategory(physicalBehavior->findAssignmentCategoryByIdentifier(assignmentCategoryIdentifier));
		executeAnalysis(physicalBehavior);
	}

private:
	GenericAnalysisToolController *genericAnalysisToolController;
};

}  // namespace controller



#endif /* REGRESSIONFITANALYSISPHYSICALBEHAVIORCONTROLLER_HPP_ */
