/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitAnalysisRegressorOverviewController.hpp
 *
 *  Created on: Mar 26, 2014
 *      Author: gerberma
 */

#ifndef REGRESSIONFITANALYSISREGRESSOROVERVIEWCONTROLLER_HPP_
#define REGRESSIONFITANALYSISREGRESSOROVERVIEWCONTROLLER_HPP_

#include <boost/bind.hpp>
#include <boost/function.hpp>
#include <boost/shared_ptr.hpp>
#include <QDebug>

namespace controller {

template
<
	typename RegressorOverview,
	typename Factory,
	typename UiManager,
	typename SubscriptionManager
>
class RegressionFitAnalysisRegressorOverviewController {
public:
	RegressionFitAnalysisRegressorOverviewController(Factory *factory, UiManager *uiManager, SubscriptionManager *subscriptionManager)
		:factory(factory), uiManager(uiManager), subscriptionManager(subscriptionManager)
	{;}

	template<typename Analysis, typename RegressionModel>
	void addAnalysisTool(boost::shared_ptr<Analysis> analysis, boost::shared_ptr<RegressionModel> regressionModel, boost::function<void(typename UiManager::base_ui_type *)> uiLocation) {
		boost::shared_ptr<RegressorOverview> overview(factory->template create<RegressorOverview>(regressionModel));

		overview->execute();
		regressionModel->addRegressionModelAnalysis(overview);

		typename UiManager::overview_ui_t *overviewUi = uiManager->getOverviewUiFor(overview);
		subscriptionManager->subscribe(this, overviewUi, regressionModel);
		subscriptionManager->subscribeEditUi(this, uiManager->getTargetAnalysisToolUiFor(overview));
		uiLocation(overviewUi);
	}

	template<typename RegressionModel>
	void removeAnalysisTool(boost::shared_ptr<RegressionModel> regressionModel, boost::shared_ptr<RegressorOverview> overview) {
		uiManager->destroyOverviewUiFor(overview);
		regressionModel->removeRegressionModelAnalysis(overview);
	}

	void executeAnalysis(boost::shared_ptr<RegressorOverview> overview) {
		overview->execute();
	}

	void setRegressorActive(boost::shared_ptr<RegressorOverview> overview, int regressorIndex, bool isActive) {
		overview->setRegressorActive(regressorIndex, isActive);
	}

private:
	Factory *factory;
	UiManager *uiManager;
	SubscriptionManager *subscriptionManager;
};

}  // namespace controller


#endif /* REGRESSIONFITANALYSISREGRESSOROVERVIEWCONTROLLER_HPP_ */
