/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * converter.hpp
 *
 *  Created on: Dec 6, 2013
 *      Author: gerberma
 */

#ifndef CONVERTER_HPP_
#define CONVERTER_HPP_

#include <boost/function.hpp>

namespace conversion {


template<class From, class To>
struct Converter;

template<class From, class To>
To convert(From from) { return Converter<From, To>::convert(from); }

template<class From, class To>
struct Converter
{
  static To convert(From from);
};



/* factory based conversion */
template<class From, class To>
class FactoryBasedConverter {
public:
	typedef boost::function<To*(From*)> Factory_Function_t;
public:
	FactoryBasedConverter(Factory_Function_t factory);

	To *convert(From *from);

private:
	Factory_Function_t factory;
};

template<class From, class To>
inline conversion::FactoryBasedConverter<From, To>::FactoryBasedConverter(Factory_Function_t factory)
	:factory(factory)
{}


template<class From, class To>
inline To* conversion::FactoryBasedConverter<From, To>::convert(From* from) {
	To *to = factory(from);

	return to;
}


} /* namespace conversion */

#endif /* CONVERTER_HPP_ */
