/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * AnalysisCommonParameterWindow.hpp
 *
 *  Created on: Nov 28, 2013
 *      Author: gerberma
 */

#ifndef ANALYSISCOMMONPARAMETERWINDOW_HPP_
#define ANALYSISCOMMONPARAMETERWINDOW_HPP_

#include <QWidget>
#include <QWizardPage>
#include <QLabel>
#include <QLineEdit>
#include <QTextEdit>
#include <QGridLayout>
#include <QRegExpValidator>
#include <iostream>
#include <boost/shared_ptr.hpp>

namespace gui {

template<typename A>
class AnalysisCommonParameterWindow : public QWizardPage {
public:
	AnalysisCommonParameterWindow(QWidget *parent, boost::shared_ptr<A> analysis);
	virtual ~AnalysisCommonParameterWindow();

	bool isComplete() const;

private:
	void setupLayout();
	void prepare();

private:
	boost::shared_ptr<A> analysis;
	QLabel *nameLabel;
	QLineEdit *nameLineEdit;
	QLabel *descriptionLabel;
	QTextEdit *descriptionTextEdit;
};


template<typename A>
inline AnalysisCommonParameterWindow<A>::AnalysisCommonParameterWindow(QWidget *parent, boost::shared_ptr<A> analysis)
	:QWizardPage(parent), analysis(analysis)
{
	setupLayout();
	prepare();
}

template<typename A>
inline AnalysisCommonParameterWindow<A>::~AnalysisCommonParameterWindow() {
}

template<typename A>
inline bool AnalysisCommonParameterWindow<A>::isComplete() const {
	std::cout << __FUNCTION__ << std::endl;
	QString nameText(nameLineEdit->text());
	int nameLength = nameLineEdit->text().length();
	if(nameLineEdit->validator()->validate(nameText, nameLength) == QValidator::Acceptable) {
		return true;
	}

	return false;
}

template<typename A>
inline void AnalysisCommonParameterWindow<A>::prepare() {
	nameLineEdit->setText(analysis->getName());
	descriptionTextEdit->setText(analysis->getDescription());
}

template<typename A>
inline void AnalysisCommonParameterWindow<A>::setupLayout() {
	setTitle(tr("Analysis information"));

	QRegExp rx("^(?!\\s*$).+");
	QValidator *validator = new QRegExpValidator(rx, this);
	nameLabel = new QLabel(tr("Name"));
	nameLineEdit = new QLineEdit;
	nameLineEdit->setValidator(validator);
	connect(nameLineEdit, SIGNAL(textChanged(const QString &)), this, SIGNAL(completeChanged()));

	nameLabel->setBuddy(nameLineEdit);
	descriptionLabel = new QLabel(tr("Description"));
	descriptionTextEdit = new QTextEdit;
	descriptionLabel->setBuddy(descriptionTextEdit);

	registerField("name", nameLineEdit);
	registerField("description", descriptionTextEdit, "plainText");

	QGridLayout *layout = new QGridLayout;
	layout->addWidget(nameLabel, 0, 0);
	layout->addWidget(nameLineEdit, 0, 1);
	layout->addWidget(descriptionLabel, 1, 0, Qt::AlignTop);
	layout->addWidget(descriptionTextEdit, 1, 1);

	setLayout(layout);
}


} /* namespace gui */

#endif /* ANALYSISCOMMONPARAMETERWINDOW_HPP_ */
