/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * AnalysisToolToolBar.hpp
 *
 *  Created on: Jan 22, 2014
 *      Author: gerberma
 */

#ifndef ANALYSISTOOLTOOLBAR_HPP_
#define ANALYSISTOOLTOOLBAR_HPP_

#include <QWidget>
#include <QHBoxLayout>
#include <QSpacerItem>
#include "MoveToolbarButton.hpp"
#include "communication/Signals.hpp"
#include "Delegates.hpp"

namespace gui {

class AnalysisToolToolBar : public QWidget {
public:
	AnalysisToolToolBar(QWidget *parent = 0);
	virtual ~AnalysisToolToolBar();

	void setMoveUpButtonEnabled(bool state);
	void setMoveDownButtonEnabled(bool state);

	COM_SUBSCRIPTION_FUNCTION(RemoveButtonClicked, communication::Signal<void()>);
	COM_SUBSCRIPTION_FUNCTION(MoveUpButtonClicked, communication::Signal<void()>);
	COM_SUBSCRIPTION_FUNCTION(MoveDownButtonClicked, communication::Signal<void()>);

private:
	void setupLayout();
	void connectLocal();

private:
	QHBoxLayout *hBoxLayout;
	MoveToolbarButton *removeButton;
	MoveToolbarButton *moveUpButton;
	MoveToolbarButton *moveDownButton;
};

inline AnalysisToolToolBar::AnalysisToolToolBar(QWidget *parent)
	:QWidget(parent)
{
	setupLayout();
	connectLocal();
}

inline AnalysisToolToolBar::~AnalysisToolToolBar() {
}

inline void AnalysisToolToolBar::setMoveUpButtonEnabled(bool state) {
	moveUpButton->setEnabled(state);
}

inline void AnalysisToolToolBar::setMoveDownButtonEnabled(bool state) {
	moveDownButton->setEnabled(state);
}

inline void AnalysisToolToolBar::setupLayout() {
	hBoxLayout = new QHBoxLayout(this);

	removeButton = new MoveToolbarButton("-", this);
	moveUpButton = new MoveToolbarButton("^", this);
	moveDownButton = new MoveToolbarButton("v", this);

	QSpacerItem *spacer = new QSpacerItem(1,1,QSizePolicy::Expanding, QSizePolicy::Ignored);
	hBoxLayout->addSpacerItem(spacer);

	hBoxLayout->addWidget(removeButton);
	hBoxLayout->addWidget(moveUpButton);
	hBoxLayout->addWidget(moveDownButton);

	setLayout(hBoxLayout);
}

inline void AnalysisToolToolBar::connectLocal() {
	using namespace communication;
	using namespace boost;

	QtBoostDelegate *removeButtonDelegate = new QtBoostDelegate(bind(ref(RemoveButtonClickedSignal)), this);
	connect(removeButton, SIGNAL(clicked()), removeButtonDelegate, SLOT(execute()));

	QtBoostDelegate *moveUpButtonDelegate = new QtBoostDelegate(bind(ref(MoveUpButtonClickedSignal)), this);
	connect(moveUpButton, SIGNAL(clicked()), moveUpButtonDelegate, SLOT(execute()));

	QtBoostDelegate *moveDownButtonDelegate = new QtBoostDelegate(bind(ref(MoveDownButtonClickedSignal)), this);
	connect(moveDownButton, SIGNAL(clicked()), moveDownButtonDelegate, SLOT(execute()));
}

} /* namespace gui */

#endif /* ANALYSISTOOLTOOLBAR_HPP_ */
