/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * AnalysisWizard.hpp
 *
 *  Created on: Nov 28, 2013
 *      Author: gerberma
 */

#ifndef ANALYSISWIZARD_HPP_
#define ANALYSISWIZARD_HPP_

#include <QObject>
#include <QWizard>
#include <boost/signals2/signal.hpp>
#include <boost/shared_ptr.hpp>
#include <iostream>
#include "BaseWizard.hpp"
#include "AnalysisCommonParameterWindow.hpp"
#include "AnalysisWaterLevelScalingWindow.hpp"


namespace gui {

template<typename A>
class AnalysisWizard : public BaseWizard {
public:
	typedef boost::signals2::signal<void()> created_signal_t;
	typedef boost::signals2::signal<void()> edited_signal_t;

public:
	AnalysisWizard(boost::shared_ptr<A> analysis);
	virtual ~AnalysisWizard();

	void start();

	void editWaterLevelScaling();

	boost::signals2::connection subscribeForCreated(const created_signal_t::slot_type &subscriber);

	boost::signals2::connection subscribeForEdited(const edited_signal_t::slot_type &subscriber);

	bool accepted();

protected:
	void accept();

private:
	enum Mode {Create, Edit};

private:
	boost::shared_ptr<A> analysis;
	AnalysisCommonParameterWindow<A> *commonParameterPage;
	AnalysisWaterLevelScalingWindow<A> *waterLevelScalingPage;
	Mode mode;
	created_signal_t createdSignal;
	edited_signal_t editedSignal;
};


template<typename A>
inline AnalysisWizard<A>::AnalysisWizard(boost::shared_ptr<A> analysis)
	:analysis(analysis), mode(Create)
{
	setWizardStyle(ModernStyle);
}

template<typename A>
inline AnalysisWizard<A>::~AnalysisWizard() {
	std::cout << __FUNCTION__ << std::endl;
}

template<typename A>
inline void AnalysisWizard<A>::start() {
	mode = Create;

	commonParameterPage = new AnalysisCommonParameterWindow<A>(this, analysis);
	//waterLevelScalingPage = new AnalysisWaterLevelScalingWindow<A>(this, analysis);

	removePage(0);
	//removePage(1);
	setPage(0, commonParameterPage);
	//setPage(1, waterLevelScalingPage);

	exec(); //execute synchronously
}

template<typename A>
inline void AnalysisWizard<A>::editWaterLevelScaling() {
	mode = Edit;

	waterLevelScalingPage = new AnalysisWaterLevelScalingWindow<A>(this, analysis);

	removePage(0);
	removePage(1);
	setPage(0, waterLevelScalingPage);

	exec(); //execute synchronously
}

template<typename A>
inline boost::signals2::connection AnalysisWizard<A>::subscribeForCreated(const created_signal_t::slot_type& subscriber) {
	return createdSignal.connect(subscriber);
}

template<typename A>
inline boost::signals2::connection AnalysisWizard<A>::subscribeForEdited(const edited_signal_t::slot_type& subscriber) {
	return editedSignal.connect(subscriber);
}

template<typename A>
inline bool AnalysisWizard<A>::accepted() {
	return result() == Accepted;
}

template<typename A>
inline void AnalysisWizard<A>::accept() {
	if(mode != Edit) {
		analysis->setName(field("name").toString());
		analysis->setDescription(field("description").toString());
	}
	analysis->getWaterLevelScalingStrategy()->setFullSupplyLevel(field("fullsupplylevel").toDouble(NULL));
	analysis->getWaterLevelScalingStrategy()->setMinimumOperatingLevel(field("minoplevel").toDouble(NULL));
	analysis->getWaterLevelScalingStrategy()->setIsEnabled(field("isenabled").toBool());

	QDialog::accept();
	if(mode == Create) {
		createdSignal();
	}else if(mode == Edit) {
		editedSignal();
	}
}

} /* namespace gui */
#endif /* ANALYSISWIZARD_HPP_ */
