/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * ComboBox.hpp
 *
 *  Created on: Jan 8, 2014
 *      Author: gerberma
 */

#ifndef COMBOBOX_HPP_
#define COMBOBOX_HPP_

#include <QWidget>
#include <QComboBox>
#include <QStandardItemModel>
#include <algorithm>

namespace gui {

class ComboBox : public QComboBox {
public:
	ComboBox(QWidget *parent = 0);
	virtual ~ComboBox();

	void addItem(const QString &name);
	void insertItem(int index, const QString &name);

	template<typename Value>
	void addItem(const QString &name, Value value);

	template<typename Value>
	void insertItem(int index, const QString &name, Value value);

	template<typename Container, typename T>
	void setCurrent(const Container &container, T *item);

	template<typename Value>
	void setCurrentByValue(Value value);

	void disableSelectionFor(int index) {
		QStandardItemModel *selectRegressorSelectionModel = qobject_cast<QStandardItemModel *>(this->model());
		QModelIndex modelIndex = selectRegressorSelectionModel->index(index, this->modelColumn(), this->rootModelIndex());
		selectRegressorSelectionModel->itemFromIndex(modelIndex)->setSelectable(false);
	}
};



inline ComboBox::ComboBox(QWidget *parent)
	:QComboBox(parent)
{
}

inline ComboBox::~ComboBox() {
}

inline void ComboBox::addItem(const QString &name) {
	QComboBox::addItem(name);
}

inline void ComboBox::insertItem(int index, const QString &name) {
	QComboBox::insertItem(index, name);
}

template<typename Value>
inline void ComboBox::addItem(const QString &name, Value value) {
	QComboBox::addItem(name, value);
}

template<typename Value>
inline void ComboBox::insertItem(int index, const QString &name, Value value) {
	QComboBox::insertItem(index, name, value);
}

template<typename Value>
inline void ComboBox::setCurrentByValue(Value value) {
	int index = findData(value);
	setCurrentIndex(index);
}

template<typename Container, typename T>
inline void ComboBox::setCurrent(const Container &container, T *item) {
	typedef typename Container::const_iterator it_t;

	for(it_t iter = container.begin(); iter != container.end(); ++iter) {
		if(*iter == item) {
			size_t containerIndex = std::distance(container.begin(), iter);


			setCurrentIndex(containerIndex);
		}
	}
}

} /* namespace gui */

#endif /* COMBOBOX_HPP_ */
