/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelAnalysisOverviewSummaryDisplayWidget.hpp
 *
 *  Created on: Jan 22, 2014
 *      Author: gerberma
 */

#ifndef DATAMODELANALYSISOVERVIEWSUMMARYDISPLAYWIDGET_HPP_
#define DATAMODELANALYSISOVERVIEWSUMMARYDISPLAYWIDGET_HPP_

#include <QWidget>
#include <QGroupBox>
#include <QVBoxLayout>
#include <QGridLayout>
#include <QSpacerItem>
#include <boost/shared_ptr.hpp>
#include "communication/Signals.hpp"
#include "AnalysisToolToolBar.hpp"
#include "KeyValueLabel.hpp"
#include "AnalysisToolWidget.hpp"


namespace gui {

/**
 * Extend the AnalysisToolWidget with the AnalysisToolToolBar as well as implement move methods on the AnalysisToolWidget
 * to update tool bar buttons and call these methods from the analysis tools container
 */
template<typename Statistics>
class DataModelAnalysisOverviewSummaryDisplayWidget : public AnalysisToolWidget {
public:
	typedef AnalysisToolWidget base_ui_type;
public:
	DataModelAnalysisOverviewSummaryDisplayWidget(boost::shared_ptr<Statistics> statistics, QWidget *parent = 0);
	virtual ~DataModelAnalysisOverviewSummaryDisplayWidget();

	COM_SUBSCRIPTION_FUNCTION(RemoveAnalysisTool, communication::Signal<void(boost::shared_ptr<Statistics>)>);

private:
	void setupLayout();
	void updateValues();
	void connectLocal();

private:
	KeyValueLabel *totalNumberOfDataPointsKeyValueLabel;
	KeyValueLabel *numberOfNotAvailableDataPointsKeyValueLabel;
	KeyValueLabel *numberOfAvailableDataPointsKeyValueLabel;
	boost::shared_ptr<Statistics> statistics;
	COM_CONNECTION loadedConnection;
	COM_CONNECTION analysisToolToolBarRemoveButtonClickedConnection;
};

template
<
	typename Statistics
>
inline DataModelAnalysisOverviewSummaryDisplayWidget<Statistics>::DataModelAnalysisOverviewSummaryDisplayWidget(boost::shared_ptr<Statistics> statistics, QWidget *parent)
	:AnalysisToolWidget(parent), statistics(statistics)
{
	setupLayout();
	updateValues();
	connectLocal();
}

template
<
	typename Statistics
>
inline DataModelAnalysisOverviewSummaryDisplayWidget<Statistics>::~DataModelAnalysisOverviewSummaryDisplayWidget() {
}

template
<
	typename Statistics
>
inline void DataModelAnalysisOverviewSummaryDisplayWidget<Statistics>::setupLayout() {
	QVBoxLayout *mainLayout = new QVBoxLayout;
	QVBoxLayout *contentLayout = new QVBoxLayout;


	//fill elements
	QGridLayout *dataLayout = new QGridLayout(this);
	QWidget *dataWidget = new QWidget(this);
	dataWidget->setLayout(dataLayout);

	totalNumberOfDataPointsKeyValueLabel = new KeyValueLabel(QObject::tr("Total number of values"), "", dataWidget, 120);
	numberOfAvailableDataPointsKeyValueLabel = new KeyValueLabel(QObject::tr("Number of available values"), "", dataWidget, 120);
	numberOfNotAvailableDataPointsKeyValueLabel = new KeyValueLabel(QObject::tr("Number of missing values"), "", dataWidget, 120);


	contentLayout->addWidget(AnalysisToolWidget::getToolBar(), 0, Qt::AlignRight | Qt::AlignTop);
	contentLayout->addWidget(dataWidget);


	dataLayout->addWidget(numberOfNotAvailableDataPointsKeyValueLabel->getKeyLabel(), 0, 0, Qt::AlignBottom);
	dataLayout->addWidget(numberOfNotAvailableDataPointsKeyValueLabel->getValueLabel(), 0, 1, Qt::AlignTop);
	dataLayout->addWidget(numberOfAvailableDataPointsKeyValueLabel->getKeyLabel(), 1, 0, Qt::AlignBottom);
	dataLayout->addWidget(numberOfAvailableDataPointsKeyValueLabel->getValueLabel(), 1, 1, Qt::AlignTop);
	dataLayout->addWidget(totalNumberOfDataPointsKeyValueLabel->getKeyLabel(), 2, 0, Qt::AlignBottom);
	dataLayout->addWidget(totalNumberOfDataPointsKeyValueLabel->getValueLabel(), 2, 1, Qt::AlignTop);
	// /fill elements


	QGroupBox *groupBox = new QGroupBox(QObject::tr("Summary"), this);
	groupBox->setLayout(contentLayout);
	groupBox->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);

	QSpacerItem *spacer = new QSpacerItem(1,1,QSizePolicy::Ignored, QSizePolicy::Expanding);
	mainLayout->addWidget(groupBox);
	mainLayout->addSpacerItem(spacer);

	setLayout(mainLayout);
}


template
<
	typename Statistics
>
inline void DataModelAnalysisOverviewSummaryDisplayWidget<Statistics>::updateValues() {
	totalNumberOfDataPointsKeyValueLabel->updateValueLabel(QString::number(statistics->getTotalNumberOfDataPoints()));
	numberOfNotAvailableDataPointsKeyValueLabel->updateValueLabel(QString::number(statistics->getNumberOfNotAvailableDataPoints()));
	numberOfAvailableDataPointsKeyValueLabel->updateValueLabel(QString::number(statistics->getNumberOfAvailableDataPoints()));
}

template
<
	typename Statistics
>
inline void gui::DataModelAnalysisOverviewSummaryDisplayWidget<Statistics>::connectLocal() {
	using namespace boost;

	loadedConnection = COM_SUBSCRIBE_FOR(statistics, Loaded, bind(&DataModelAnalysisOverviewSummaryDisplayWidget<Statistics>::updateValues, this));

	analysisToolToolBarRemoveButtonClickedConnection = COM_SUBSCRIBE_FOR(AnalysisToolWidget::getToolBar(), RemoveButtonClicked, (boost::bind(boost::ref(RemoveAnalysisToolSignal), statistics)));
}


} /* namespace gui */


#endif /* DATAMODELANALYSISOVERVIEWSUMMARYDISPLAYWIDGET_HPP_ */
