/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelAnalysisWidget.hpp
 *
 *  Created on: Jan 16, 2014
 *      Author: gerberma
 */

#ifndef DATAMODELANALYSISWIDGET_HPP_
#define DATAMODELANALYSISWIDGET_HPP_

#include <QWidget>
#include <QTabWidget>
#include <QGridLayout>
#include <boost/function.hpp>
#include <boost/bind.hpp>
#include <boost/shared_ptr.hpp>
#include "communication/Signals.hpp"
#include "AnalysisCategoryGenericWidget.hpp"
#include "DataModelAnalysisOverviewCategoryProvider.hpp"
#include "DataModelAnalysisDistributionCategoryProvider.hpp"
#include "AnalysisToolWidget.hpp"

namespace gui {

template<typename AnalysisToolsDescription>
class DataModelAnalysisWidget : public QWidget {
public:
	typedef AnalysisCategoryGenericWidget<DataModelAnalysisOverviewCategoryProvider<AnalysisToolsDescription> > OverviewCategoryWidget_t;
public:
	DataModelAnalysisWidget(boost::shared_ptr<AnalysisToolsDescription> analysisToolsDescription, QWidget *parent = 0);
	virtual ~DataModelAnalysisWidget();

	COM_SUBSCRIPTION_FUNCTION(AddAnalysisTool, communication::Signal<
				void(boost::shared_ptr<typename AnalysisToolsDescription::tool_description_t>, boost::function<void(AnalysisToolWidget *)>)
			>);

	AnalysisCategoryGenericWidget<DataModelAnalysisOverviewCategoryProvider<AnalysisToolsDescription> > *getOverviewCategoryWidget() {
		return dataModelAnalysisOverviewCategoryWidget;
	}

private:
	void setupLayout();
	void connectLocal();

private:
	QTabWidget *tabWidget;
	DataModelAnalysisOverviewCategoryProvider<AnalysisToolsDescription> *overviewCategoryProvider;
	AnalysisCategoryGenericWidget<DataModelAnalysisOverviewCategoryProvider<AnalysisToolsDescription> > *dataModelAnalysisOverviewCategoryWidget;
	/*
	DataModelAnalysisDistributionCategoryProvider<AnalysisToolsDescription> *distributionCategoryProvider;
	AnalysisCategoryGenericWidget<DataModelAnalysisDistributionCategoryProvider<AnalysisToolsDescription> > *dataModelAnalysisDistributionCategoryWidget;
	*/
	boost::shared_ptr<AnalysisToolsDescription> analysisToolsDescription;
	COM_CONNECTION overviewATAddedConnection;
};

template
<
	typename AnalysisToolsDescription
>
inline DataModelAnalysisWidget<AnalysisToolsDescription>::DataModelAnalysisWidget(boost::shared_ptr<AnalysisToolsDescription> analysisToolsDescription, QWidget* parent)
	:QWidget(parent), analysisToolsDescription(analysisToolsDescription)
{
	setupLayout();
	connectLocal();
}

template
<
	typename AnalysisToolsDescription
>
inline DataModelAnalysisWidget<AnalysisToolsDescription>::~DataModelAnalysisWidget() {
}

template
<
	typename AnalysisToolsDescription
>
inline void DataModelAnalysisWidget<AnalysisToolsDescription>::setupLayout() {
	tabWidget = new QTabWidget(this);

	overviewCategoryProvider = new DataModelAnalysisOverviewCategoryProvider<AnalysisToolsDescription>(analysisToolsDescription, this);
	dataModelAnalysisOverviewCategoryWidget = new AnalysisCategoryGenericWidget<DataModelAnalysisOverviewCategoryProvider<AnalysisToolsDescription> >(overviewCategoryProvider, this);
	tabWidget->addTab(dataModelAnalysisOverviewCategoryWidget, overviewCategoryProvider->getCategoryName());

	/*
	distributionCategoryProvider = new DataModelAnalysisDistributionCategoryProvider<AnalysisToolsDescription>(analysisToolsDescription, this);
	dataModelAnalysisDistributionCategoryWidget = new AnalysisCategoryGenericWidget<DataModelAnalysisDistributionCategoryProvider<AnalysisToolsDescription> >(distributionCategoryProvider, this);
	tabWidget->addTab(dataModelAnalysisDistributionCategoryWidget, distributionCategoryProvider->getCategoryName());
	*/

	QGridLayout *gridLayout = new QGridLayout;
	gridLayout->addWidget(tabWidget, 0, 0);

	setLayout(gridLayout);
}

template
<
	typename AnalysisToolsDescription
>
inline void DataModelAnalysisWidget<AnalysisToolsDescription>::connectLocal() {
	//COM_SUBSCRIBE_FOR(dataModelAnalysisDistributionCategoryWidget, AddAnalysisTool, (boost::bind(boost::ref(AddAnalysisToolSignal), _1, _2)));
	overviewATAddedConnection = COM_SUBSCRIBE_FOR(dataModelAnalysisOverviewCategoryWidget, AddAnalysisTool, (boost::bind(boost::ref(AddAnalysisToolSignal), _1, _2)));
}



} /* namespace gui */

#endif /* DATAMODELANALYSISWIDGET_HPP_ */
