/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelAssignmentDisplayWidget.hpp
 *
 *  Created on: Jan 15, 2014
 *      Author: gerberma
 */

#ifndef DATAMODELASSIGNMENTDISPLAYWIDGET_HPP_
#define DATAMODELASSIGNMENTDISPLAYWIDGET_HPP_

#include <QWidget>
#include <QTabWidget>
#include "DataModelAssignmentGenericDisplayWidget.hpp"
#include "DataModelBehaviorIndicatorAssignmentDisplayProvider.hpp"
#include "DataModelWaterLevelAssignmentDisplayProvider.hpp"
#include "DataModelTemperatureAssignmentDisplayProvider.hpp"
#include "OverviewEditButton.hpp"
#include "communication/Signals.hpp"
#include "Delegates.hpp"
#include <boost/bind.hpp>
#include <boost/ref.hpp>
#include <boost/shared_ptr.hpp>

namespace gui {

template<typename DataModel>
class DataModelAssignmentDisplayWidgetSignals {
public:
	COM_SUBSCRIPTION_FUNCTION(Edit, communication::Signal<void(boost::shared_ptr<DataModel>)>);
};

template<typename DataModel>
class DataModelAssignmentDisplayWidget : public QWidget, public DataModelAssignmentDisplayWidgetSignals<DataModel> {
public:
	DataModelAssignmentDisplayWidget(boost::shared_ptr<DataModel> dataModel, QWidget *parent=0);
	virtual ~DataModelAssignmentDisplayWidget();

private:
	void setupLayout();
	void connectLocal();
private:
	boost::shared_ptr<DataModel> dataModel;
	QTabWidget *tabWidget;
	DataModelAssignmentGenericDisplayWidget<DataModelBehaviorIndicatorAssignmentDisplayProvider<DataModel> > *behaviorIndicatorDisplayWidget;
	DataModelAssignmentGenericDisplayWidget<DataModelWaterLevelAssignmentDisplayProvider<DataModel> > *waterLevelDisplayWidget;
	DataModelAssignmentGenericDisplayWidget<DataModelTemperatureAssignmentDisplayProvider<DataModel> > *temperatureDisplayWidget;
	OverviewEditButton *editButton;
};



template<typename DataModel>
inline DataModelAssignmentDisplayWidget<DataModel>::DataModelAssignmentDisplayWidget(boost::shared_ptr<DataModel> dataModel, QWidget* parent)
	:QWidget(parent), dataModel(dataModel)
{
	setupLayout();
	connectLocal();
}

template<typename DataModel>
inline DataModelAssignmentDisplayWidget<DataModel>::~DataModelAssignmentDisplayWidget() {
}

template<typename DataModel>
inline void DataModelAssignmentDisplayWidget<DataModel>::setupLayout() {
	tabWidget = new QTabWidget(this);

	//behaviorIndicatorDisplayWidget = new DataModelBehaviorIndicatorAssignmentDisplayWidget<DataModel>(dataModel, tabWidget);
	behaviorIndicatorDisplayWidget = new DataModelAssignmentGenericDisplayWidget<
				DataModelBehaviorIndicatorAssignmentDisplayProvider<DataModel> >(new DataModelBehaviorIndicatorAssignmentDisplayProvider<DataModel>(dataModel, this), this);
	waterLevelDisplayWidget = new DataModelAssignmentGenericDisplayWidget<
			DataModelWaterLevelAssignmentDisplayProvider<DataModel> >(new DataModelWaterLevelAssignmentDisplayProvider<DataModel>(dataModel, this), this);
	temperatureDisplayWidget = new DataModelAssignmentGenericDisplayWidget<
			DataModelTemperatureAssignmentDisplayProvider<DataModel> >(new DataModelTemperatureAssignmentDisplayProvider<DataModel>(dataModel, this), this);

	tabWidget->addTab(behaviorIndicatorDisplayWidget, tr("Behavior Indicators"));
	tabWidget->addTab(waterLevelDisplayWidget, tr("Water Level"));
	tabWidget->addTab(temperatureDisplayWidget, tr("Temperature"));
	tabWidget->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Maximum);

	editButton = new OverviewEditButton(this);

	QGridLayout *gridLayout = new QGridLayout;
	gridLayout->addWidget(tabWidget, 0, 0);
	gridLayout->addWidget(editButton, 0, 1, Qt::AlignRight | Qt::AlignBottom);
	setLayout(gridLayout);

}

template<typename DataModel>
inline void DataModelAssignmentDisplayWidget<DataModel>::connectLocal() {
	using namespace communication;
	using namespace boost;
	QtBoostDelegate *editClickedDelegate = new QtBoostDelegate(bind(ref(DataModelAssignmentDisplayWidgetSignals<DataModel>::EditSignal), dataModel), this);
	connect(editButton, SIGNAL(clicked()), editClickedDelegate, SLOT(execute()));
}

} /* namespace gui */

#endif /* DATAMODELASSIGNMENTDISPLAYWIDGET_HPP_ */
