/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelAssignmentEditWidget.hpp
 *
 *  Created on: Jan 10, 2014
 *      Author: gerberma
 */

#ifndef DATAMODELASSIGNMENTEDITWIDGET_HPP_
#define DATAMODELASSIGNMENTEDITWIDGET_HPP_

#include <QObject>
#include <QVBoxLayout>
#include <QGridLayout>
#include <QWidget>
#include <QLabel>
#include <QListWidget>
#include <QGroupBox>
#include <QTabWidget>
#include <boost/bind.hpp>
#include <boost/foreach.hpp>
#include <boost/signals2/signal.hpp>
#include <boost/ref.hpp>
#include <boost/shared_ptr.hpp>
#include "SplittedLayoutBuilder.hpp"
#include "GroupBoxLayoutBuilder.hpp"
#include "communication/Signals.hpp"
#include "Delegates.hpp"
#include "DataModelRawDataDisplayWidget.hpp"
#include "DataModelBehaviorIndicatorAssignmentEditWidget.hpp"
#include "DataModelWaterLevelAssignmentEditWidget.hpp"
#include "DataModelTemperatureAssignmentEditWidget.hpp"

namespace gui {



template<typename DataModel>
class DataModelAssignmentEditWidget : public QWidget {
public:
	DataModelAssignmentEditWidget(QWidget *parent, boost::shared_ptr<DataModel> dataModel);
	virtual ~DataModelAssignmentEditWidget();

	COM_SUBSCRIPTION_DELEGATE_FUNCTION(DataModelBehaviorIndicatorAssignmentEditWidget<DataModel>, behaviorIndicatorEditWidget, BehaviorIndicatorAssignmentChanged);
	COM_SUBSCRIPTION_DELEGATE_FUNCTION(DataModelWaterLevelAssignmentEditWidget<DataModel>, waterLevelEditWidget, WaterLevelAssignmentChanged);
	COM_SUBSCRIPTION_DELEGATE_FUNCTION(DataModelTemperatureAssignmentEditWidget<DataModel>, temperatureEditWidget, TemperatureAssignmentChanged);

private:
	void setupLayout();
	void connectLocal();

private:
	boost::shared_ptr<DataModel> dataModel;
	QTabWidget *tabWidget;
	DataModelBehaviorIndicatorAssignmentEditWidget<DataModel> *behaviorIndicatorEditWidget;
	DataModelWaterLevelAssignmentEditWidget<DataModel> *waterLevelEditWidget;
	DataModelTemperatureAssignmentEditWidget<DataModel> *temperatureEditWidget;
	DataModelRawDataDisplayWidget<DataModel> *rawDataWidget;
	//COM_CONNECTION dataModelLoadedConnection;
};


template<typename DataModel>
inline gui::DataModelAssignmentEditWidget<DataModel>::DataModelAssignmentEditWidget(QWidget* parent, boost::shared_ptr<DataModel> dataModel)
	:QWidget(parent), dataModel(dataModel)
{
	setupLayout();
	connectLocal();
}

template<typename DataModel>
inline DataModelAssignmentEditWidget<DataModel>::~DataModelAssignmentEditWidget() {
}

template<typename DataModel>
inline void DataModelAssignmentEditWidget<DataModel>::setupLayout() {
	tabWidget = new QTabWidget(this);

	behaviorIndicatorEditWidget = new DataModelBehaviorIndicatorAssignmentEditWidget<DataModel>(tabWidget, dataModel);
	waterLevelEditWidget = new DataModelWaterLevelAssignmentEditWidget<DataModel>(tabWidget, dataModel);
	temperatureEditWidget = new DataModelTemperatureAssignmentEditWidget<DataModel>(tabWidget, dataModel);

	tabWidget->addTab(behaviorIndicatorEditWidget, tr("Behavior Indicators"));
	tabWidget->addTab(waterLevelEditWidget, tr("Water Level"));
	tabWidget->addTab(temperatureEditWidget, tr("Temperature"));

	rawDataWidget = new DataModelRawDataDisplayWidget<DataModel>(dataModel, this);

	GroupBoxLayoutBuilder dataModelLayoutBuilder(tr("Data assignment"));
	dataModelLayoutBuilder.addWidget(tabWidget);
	//dataModelLayoutBuilder.addWidget(rawDataWidget);
	QWidget *dataModelWidget = new QWidget(this);
	QVBoxLayout *dataModelWidgetLayout = dataModelLayoutBuilder.build(this, QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed), false);
	dataModelWidgetLayout->setAlignment(Qt::AlignTop);
	dataModelWidget->setLayout(dataModelWidgetLayout);

	GroupBoxLayoutBuilder rawDataLayoutBuilder(tr("Data"));
	rawDataLayoutBuilder.addWidget(rawDataWidget, 0, 0, 3, 2);
	QWidget *rawDataWidget = new QWidget(this);
	QVBoxLayout *rawDataWidgetLayout = rawDataLayoutBuilder.build(this, QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding), false);
	rawDataWidgetLayout->setAlignment(Qt::AlignTop);
	rawDataWidget->setLayout(rawDataWidgetLayout);

	QVBoxLayout *mainLayout = new QVBoxLayout(this);
	mainLayout->addWidget(dataModelWidget);
	mainLayout->addWidget(rawDataWidget);
	mainLayout->setAlignment(Qt::AlignTop);

	setLayout(mainLayout);
}


template<typename DataModel>
inline void DataModelAssignmentEditWidget<DataModel>::connectLocal() {
}


}/* namespace gui */

#endif /* DATAMODELASSIGNMENTEDITWIDGET_HPP_ */
