/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelAssignmentGenericDisplayWidget.hpp
 *
 *  Created on: Jan 15, 2014
 *      Author: gerberma
 */

#ifndef DATAMODELASSIGNMENTGENERICDISPLAYWIDGET_HPP_
#define DATAMODELASSIGNMENTGENERICDISPLAYWIDGET_HPP_

#include <boost/foreach.hpp>
#include <boost/shared_ptr.hpp>
#include <QListWidget>
#include <QListWidgetItem>
#include "GroupBoxLayoutBuilder.hpp"
#include "communication/Signals.hpp"

namespace gui {

template
<
	typename AssignmentProvider
>
class DataModelAssignmentGenericDisplayWidget : public QWidget {
public:
	DataModelAssignmentGenericDisplayWidget(AssignmentProvider *assignmentProvider, QWidget *parent);
	virtual ~DataModelAssignmentGenericDisplayWidget();

private:
	void setupLayout();
	void updateValues();
	void connectLocal();

private:
	AssignmentProvider *assignmentProvider;
	QListWidget *listWidget;
	COM_CONNECTION dataModelChangedConnection;
};


template
<
	typename AssignmentProvider
>
inline gui::DataModelAssignmentGenericDisplayWidget<AssignmentProvider>::DataModelAssignmentGenericDisplayWidget(AssignmentProvider *assignmentProvider, QWidget* parent)
	:QWidget(parent), assignmentProvider(assignmentProvider)
{
	setupLayout();
	updateValues();
	connectLocal();
}

template
<
	typename AssignmentProvider
>
inline gui::DataModelAssignmentGenericDisplayWidget<AssignmentProvider>::~DataModelAssignmentGenericDisplayWidget() {
}

template
<
	typename AssignmentProvider
>
inline void gui::DataModelAssignmentGenericDisplayWidget<AssignmentProvider>::setupLayout() {
	listWidget = new QListWidget(this);

	GroupBoxLayoutBuilder layoutBuilder(tr(""));
	layoutBuilder.addWidget(listWidget);

	setLayout(layoutBuilder.build(this));
}

template
<
	typename AssignmentProvider
>
inline void gui::DataModelAssignmentGenericDisplayWidget<AssignmentProvider>::updateValues() {
	typedef typename AssignmentProvider::type_t assignment_t;

	listWidget->clear();

	BOOST_FOREACH(boost::shared_ptr<assignment_t> t, assignmentProvider->getAssignments()) {
		QListWidgetItem *item = new QListWidgetItem(t->getName(), listWidget);
		listWidget->addItem(item);
	}
}

template
<
	typename AssignmentProvider
>
inline void gui::DataModelAssignmentGenericDisplayWidget<AssignmentProvider>::connectLocal() {
	dataModelChangedConnection = COM_SUBSCRIBE_FOR(assignmentProvider->getDataModel(), Changes, boost::bind(&DataModelAssignmentGenericDisplayWidget<AssignmentProvider>::updateValues, this));
}

} /* namespace gui */


#endif /* DATAMODELASSIGNMENTGENERICDISPLAYWIDGET_HPP_ */
