/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelAssignementWizardPage.hpp
 *
 *  Created on: Jan 10, 2014
 *      Author: gerberma
 */

#ifndef DATAMODELASSIGNEMENTWIZARDPAGE_HPP_
#define DATAMODELASSIGNEMENTWIZARDPAGE_HPP_

#include <QObject>
#include <QVBoxLayout>
#include <QGridLayout>
#include <QWidget>
#include <QLabel>
#include <QGroupBox>
#include <QWizardPage>
#include <iostream>
#include <algorithm>
#include <boost/bind.hpp>
#include <boost/foreach.hpp>
#include <boost/signals2/signal.hpp>
#include <boost/ref.hpp>
#include <boost/shared_ptr.hpp>
#include "communication/Signals.hpp"
#include "DataModelWizardPages.hpp"
#include "WizardPage.hpp"
#include "SplittedLayoutBuilder.hpp"
#include "DataModelAssignmentEditWidget.hpp"
#include "Delegates.hpp"

namespace gui {

template
<
	typename DataModel,
	typename NextIdStrategy
>
class DataModelAssignmentWizardPage : public WizardPage<NextIdStrategy>
{
public:
	DataModelAssignmentWizardPage(QWidget *parent, boost::shared_ptr<DataModel> dataModel, NextIdStrategy *nextIdStrategy, typename NextIdStrategy::type_t myId);
	virtual ~DataModelAssignmentWizardPage();

	COM_SUBSCRIPTION_DELEGATE_FUNCTION(DataModelAssignmentEditWidget<DataModel>, dataModelAssignmentEditWidget, BehaviorIndicatorAssignmentChanged);
	COM_SUBSCRIPTION_DELEGATE_FUNCTION(DataModelAssignmentEditWidget<DataModel>, dataModelAssignmentEditWidget, WaterLevelAssignmentChanged);
	COM_SUBSCRIPTION_DELEGATE_FUNCTION(DataModelAssignmentEditWidget<DataModel>, dataModelAssignmentEditWidget, TemperatureAssignmentChanged);

private:
	void setupLayout();
	void connectLocal();

private:
	boost::shared_ptr<DataModel> dataModel;
	SplittedLayoutBuilder splittedLayoutBuilder;
	DataModelAssignmentEditWidget<DataModel> *dataModelAssignmentEditWidget;
};


template<typename DataModel, typename NextIdStrategy>
inline gui::DataModelAssignmentWizardPage<DataModel, NextIdStrategy>::DataModelAssignmentWizardPage(
		QWidget* parent,
		boost::shared_ptr<DataModel> dataModel,
		NextIdStrategy* nextIdStrategy,
		typename NextIdStrategy::type_t myId)
	:WizardPage<NextIdStrategy>(parent, nextIdStrategy, myId), dataModel(dataModel)
{
	setupLayout();
}

template<typename DataModel, typename NextIdStrategy>
inline gui::DataModelAssignmentWizardPage<DataModel, NextIdStrategy>::~DataModelAssignmentWizardPage() {
}


template<typename DataModel, typename NextIdStrategy>
inline void gui::DataModelAssignmentWizardPage<DataModel, NextIdStrategy>::setupLayout() {
	dataModelAssignmentEditWidget = new DataModelAssignmentEditWidget<DataModel>(this, dataModel);

	WizardPage<NextIdStrategy>::setLayout(splittedLayoutBuilder.build(this, dataModelAssignmentEditWidget, NULL));
}

template<typename DataModel, typename NextIdStrategy>
inline void gui::DataModelAssignmentWizardPage<DataModel, NextIdStrategy>::connectLocal() {
}


} /* namespace gui */

#endif /* DATAMODELASSIGNEMENTWIZARDPAGE_HPP_ */
