/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelBehaviorIndicatorAssignmentEditWidget.hpp
 *
 *  Created on: Jan 10, 2014
 *      Author: gerberma
 */

#ifndef DATAMODELBEHAVIORINDICATORASSIGNMENTEDITWIDGET_HPP_
#define DATAMODELBEHAVIORINDICATORASSIGNMENTEDITWIDGET_HPP_

#include <QObject>
#include <QObject>
#include <QVBoxLayout>
#include <QGridLayout>
#include <QWidget>
#include <QLabel>
#include <QListWidget>
#include <QGroupBox>
#include <QTabWidget>
#include <boost/bind.hpp>
#include <boost/foreach.hpp>
#include <boost/ref.hpp>
#include <boost/shared_ptr.hpp>
#include <iostream>
//#include "ComboBox.hpp"
#include "SplittedLayoutBuilder.hpp"
#include "GroupBoxLayoutBuilder.hpp"
#include "communication/Signals.hpp"
#include "DataModelWizardPages.hpp"
#include "Delegates.hpp"


namespace gui {

namespace priv {
class DataModelBehaviorIndicatorAssignmentEditWidgetSignalSlots : public QWidget {
public:
	DataModelBehaviorIndicatorAssignmentEditWidgetSignalSlots(QWidget *parent)
		:QWidget(parent)
	{}

	virtual ~DataModelBehaviorIndicatorAssignmentEditWidgetSignalSlots()
	{}
};

class DataModelBehaviorIndicatorAssignmentEditWidgetQtSignalsSlots : public DataModelBehaviorIndicatorAssignmentEditWidgetSignalSlots {
	Q_OBJECT
public:
	DataModelBehaviorIndicatorAssignmentEditWidgetQtSignalsSlots(QWidget *parent)
		:DataModelBehaviorIndicatorAssignmentEditWidgetSignalSlots(parent)
	{}

	virtual ~DataModelBehaviorIndicatorAssignmentEditWidgetQtSignalsSlots()
	{}

public slots:
	virtual void itemChanged(QListWidgetItem *item) = 0;
	//virtual void timeSeriesIndexChanged(int index) = 0;
};
} /* namespace priv */

template<typename DataModel>
class DataModelBehaviorAssignmentEditWidgetSignals {
public:
	COM_SUBSCRIPTION_FUNCTION(BehaviorIndicatorAssignmentChanged, communication::Signal<void(boost::shared_ptr<DataModel>, int, bool)>);
};

template<typename DataModel>
class DataModelBehaviorIndicatorAssignmentEditWidget  : public priv::DataModelBehaviorIndicatorAssignmentEditWidgetQtSignalsSlots, public DataModelBehaviorAssignmentEditWidgetSignals<DataModel>{
public:
	DataModelBehaviorIndicatorAssignmentEditWidget(QWidget *parent, boost::shared_ptr<DataModel> dataModel);
	virtual ~DataModelBehaviorIndicatorAssignmentEditWidget();

private:
	void setupLayout();
	void updateValues();
	void connectLocal();

	void behaviorIndicatorIndexChanged(int index);

	void itemChanged(QListWidgetItem *item);

private:
//	QLabel *behaviorIndicatorLabel;
	QListWidget *behaviorIndicatorListWidget;
	boost::shared_ptr<DataModel> dataModel;
	COM_CONNECTION dataModelLoadedConnection;
	COM_CONNECTION dateColumnChangedConnection;
	COM_CONNECTION waterLevelAssignmentChangedConnection;
	COM_CONNECTION temperatureAssignmentChangedConnection;
};

template<typename DataModel>
inline gui::DataModelBehaviorIndicatorAssignmentEditWidget<DataModel>::DataModelBehaviorIndicatorAssignmentEditWidget(QWidget* parent, boost::shared_ptr<DataModel> dataModel)
	:priv::DataModelBehaviorIndicatorAssignmentEditWidgetQtSignalsSlots(parent), dataModel(dataModel)
{
	setupLayout();
	updateValues();
	connectLocal();
}

template<typename DataModel>
inline gui::DataModelBehaviorIndicatorAssignmentEditWidget<DataModel>::~DataModelBehaviorIndicatorAssignmentEditWidget() {
}

template<typename DataModel>
inline void gui::DataModelBehaviorIndicatorAssignmentEditWidget<DataModel>::setupLayout() {
	behaviorIndicatorListWidget = new QListWidget(this);

	GroupBoxLayoutBuilder layoutBuilder(tr(""));
	layoutBuilder.addWidget(behaviorIndicatorListWidget);

	setLayout(layoutBuilder.build(this));
}

template<typename DataModel>
inline void gui::DataModelBehaviorIndicatorAssignmentEditWidget<DataModel>::updateValues() {
	typedef typename DataModel::Header_t Header_t;

	disconnect(behaviorIndicatorListWidget, SIGNAL(itemChanged(QListWidgetItem *)), this, SLOT(itemChanged(QListWidgetItem *)));

	behaviorIndicatorListWidget->clear();

	BOOST_FOREACH(Header_t const &header, dataModel->getAssignableBehaviorIndicatorHeaders()) {
		std::cout << "#BehaviorIndicator : " << header.getIndex() << std::endl;
		QListWidgetItem *item = new QListWidgetItem(header.getName(), behaviorIndicatorListWidget);
		//QVariant index(header.getIndex());
		item->setData(Qt::UserRole, header.getIndex());
		item->setFlags(item->flags() | Qt::ItemIsUserCheckable);
		if(dataModel->isBehaviorIndicatorColumnAssignedFor(header.getIndex())) {
			item->setCheckState(Qt::Checked);
		}else{
			item->setCheckState(Qt::Unchecked);
		}
		behaviorIndicatorListWidget->addItem(item);
	}

	connect(behaviorIndicatorListWidget, SIGNAL(itemChanged(QListWidgetItem *)), this, SLOT(itemChanged(QListWidgetItem *)));
}

template<typename DataModel>
inline void gui::DataModelBehaviorIndicatorAssignmentEditWidget<DataModel>::connectLocal() {
	connect(behaviorIndicatorListWidget, SIGNAL(itemChanged(QListWidgetItem *)), this, SLOT(itemChanged(QListWidgetItem *)));
	dataModelLoadedConnection = COM_SUBSCRIBE_FOR(dataModel, DataModelLoaded, (boost::bind(&DataModelBehaviorIndicatorAssignmentEditWidget<DataModel>::updateValues, this)));
	dateColumnChangedConnection = COM_SUBSCRIBE_FOR(dataModel, DateColumnChanged, (boost::bind(&DataModelBehaviorIndicatorAssignmentEditWidget<DataModel>::updateValues, this)));
	waterLevelAssignmentChangedConnection = COM_SUBSCRIBE_FOR(dataModel, WaterLevelAssignmentChanged, (boost::bind(&DataModelBehaviorIndicatorAssignmentEditWidget<DataModel>::updateValues, this)));
	temperatureAssignmentChangedConnection = COM_SUBSCRIBE_FOR(dataModel, TemperatureAssignmentChanged, (boost::bind(&DataModelBehaviorIndicatorAssignmentEditWidget<DataModel>::updateValues, this)));
}

template<typename DataModel>
inline void gui::DataModelBehaviorIndicatorAssignmentEditWidget<DataModel>::itemChanged(QListWidgetItem *item) {
	std::cout << item->checkState() << std::endl;
	bool assignmentState;
	if(item->checkState() == Qt::Checked) {
		assignmentState = true;
	}else if(item->checkState() == Qt::Unchecked){
		assignmentState = false;
	}else {
		return;
	}

	DataModelBehaviorAssignmentEditWidgetSignals<DataModel>::BehaviorIndicatorAssignmentChangedSignal(dataModel, item->data(Qt::UserRole).toString().toInt(), assignmentState);
}

template<typename DataModel>
inline void gui::DataModelBehaviorIndicatorAssignmentEditWidget<DataModel>::behaviorIndicatorIndexChanged(int index) {
}


} /* namespace gui */

#endif /* DATAMODELBEHAVIORINDICATORASSIGNMENTEDITWIDGET_HPP_ */
