/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelCommonParameterDisplayWidget.hpp
 *
 *  Created on: Jan 14, 2014
 *      Author: gerberma
 */

#ifndef DATAMODELCOMMONPARAMETERDISPLAYWIDGET_HPP_
#define DATAMODELCOMMONPARAMETERDISPLAYWIDGET_HPP_

#include <QWidget>
#include <boost/shared_ptr.hpp>
#include "communication/Signals.hpp"
#include "OverviewKeyValueGridLayout.hpp"
#include "OverviewEditButton.hpp"
#include "OverviewKeyValueDisplayWidget.hpp"
#include "KeyValueLabel.hpp"
#include "conversion/converter.hpp"
#include "gui/guiconverter.hpp"
#include "communication/Signals.hpp"

namespace gui {

template<typename DataModel>
class DataModelCommonParameterDisplayWidget : public OverviewKeyValueDisplayWidget<DataModel>
{
public:
	DataModelCommonParameterDisplayWidget(boost::shared_ptr<DataModel> dataModel, QWidget *parent = 0);
	virtual ~DataModelCommonParameterDisplayWidget();


private:
	void setupLayout();
	void updateValues();
	void connectLocal();

private:
	boost::shared_ptr<DataModel> dataModel;
	KeyValueLabel *dataSourceLabels;
	COM_CONNECTION dataModelChangedConnection;
};


template<typename DataModel>
inline DataModelCommonParameterDisplayWidget<DataModel>::DataModelCommonParameterDisplayWidget(boost::shared_ptr<DataModel> dataModel, QWidget *parent)
	:OverviewKeyValueDisplayWidget<DataModel>(dataModel, parent), dataModel(dataModel)
{
	setupLayout();
	updateValues();
	connectLocal();
}

template<typename DataModel>
inline DataModelCommonParameterDisplayWidget<DataModel>::~DataModelCommonParameterDisplayWidget() {
}

template<typename DataModel>
inline void DataModelCommonParameterDisplayWidget<DataModel>::setupLayout() {
	dataSourceLabels = new KeyValueLabel(QObject::tr("Data source"), "", this);

	OverviewKeyValueDisplayWidget<DataModel>::addKeyValueLabel(dataSourceLabels, 0);
	OverviewKeyValueDisplayWidget<DataModel>::setEditButton(0);

	OverviewKeyValueDisplayWidget<DataModel>::finishLayout();
}

template<typename DataModel>
inline void DataModelCommonParameterDisplayWidget<DataModel>::updateValues() {
	dataSourceLabels->updateValueLabel(dataModel->getSourceUri().toString());
}

template<typename DataModel>
inline void DataModelCommonParameterDisplayWidget<DataModel>::connectLocal() {
	dataModelChangedConnection = COM_SUBSCRIBE_FOR(dataModel, Changes, boost::bind(&DataModelCommonParameterDisplayWidget<DataModel>::updateValues, this));
}

} /* namespace gui */

#endif /* DATAMODELCOMMONPARAMETERDISPLAYWIDGET_HPP_ */
