/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelRawDataDisplayWidget.hpp
 *
 *  Created on: 09.09.2014
 *      Author: gerberma
 */

#ifndef DATAMODELRAWDATADISPLAYWIDGET_HPP_
#define DATAMODELRAWDATADISPLAYWIDGET_HPP_

#include <QTableWidget>
#include <QWidget>
#include <QVBoxLayout>
#include <QDebug>
#include <boost/bind.hpp>
#include <boost/shared_ptr.hpp>
#include "communication/Signals.hpp"

namespace gui {

template<typename DataModel>
class DataModelRawDataDisplayWidget : public QWidget {
public:
	DataModelRawDataDisplayWidget(boost::shared_ptr<DataModel> dataModel, QWidget *parent = 0)
		:QWidget(parent), dataModel(dataModel)
	{
		setupLayout();
		connectLocal();
		updateValues();
	}

private:
	void setupLayout() {
		QVBoxLayout *mainLayout = new QVBoxLayout(this);
			table = new QTableWidget(this);
			table->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Maximum);
			mainLayout->addWidget(table);
		setLayout(mainLayout);
	}

	void connectLocal() {
		using namespace boost;
		dataModelLoadedConnection = COM_SUBSCRIBE_FOR(dataModel, DataModelLoaded, bind(&DataModelRawDataDisplayWidget<DataModel>::updateValues, this));
	}

	void updateValues() {
		table->clear();
		QStringList headers;
		typedef std::pair<QString, std::vector<QString> > value_pair_t;
		if(dataModel->getDataFrame() == NULL) {
			return;
		}

		BOOST_FOREACH(typename DataModel::Header_t header, dataModel->getDataFrame()->getHeaders()) {
			headers.push_back(header.getName());
		}

		table->setColumnCount(headers.size());
		if(dataModel->getDataFrame()->getHeaders().size() == 0) {
			return;
		}

		table->setRowCount(dataModel->getDataFrame()->getSomeRawValuesFor(dataModel->getDataFrame()->getHeaders().front().getName()).size());
		table->setHorizontalHeaderLabels(headers);

		int column = 0;
		BOOST_FOREACH(typename DataModel::Header_t header, dataModel->getDataFrame()->getHeaders()) {
			std::vector<QString> rawValues = dataModel->getDataFrame()->getSomeRawValuesFor(header.getName());
			int row = 0;
			BOOST_FOREACH(QString value, rawValues) {
				table->setItem(row, column, new QTableWidgetItem(value));
				//qDebug() << "row : " << QString::number(row) << " column: " << QString::number(column) << " value: " << value;
				++row;
			}
			++column;
		}
	}



private:
	boost::shared_ptr<DataModel> dataModel;
	QTableWidget *table;
	COM_CONNECTION dataModelLoadedConnection;
};

}  // namespace gui


#endif /* DATAMODELRAWDATADISPLAYWIDGET_HPP_ */
