/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelTemperatureAssignmentEditWidget.hpp
 *
 *  Created on: Jan 14, 2014
 *      Author: gerberma
 */

#ifndef DATAMODELTEMPERATUREASSIGNMENTEDITWIDGET_HPP_
#define DATAMODELTEMPERATUREASSIGNMENTEDITWIDGET_HPP_

#include <QObject>
#include <QObject>
#include <QVBoxLayout>
#include <QGridLayout>
#include <QWidget>
#include <QLabel>
#include <QListWidget>
#include <QGroupBox>
#include <QTabWidget>
#include <boost/bind.hpp>
#include <boost/foreach.hpp>
#include <boost/ref.hpp>
#include <boost/shared_ptr.hpp>
#include <iostream>
//#include "ComboBox.hpp"
#include "SplittedLayoutBuilder.hpp"
#include "GroupBoxLayoutBuilder.hpp"
#include "communication/Signals.hpp"
#include "DataModelWizardPages.hpp"
#include "Delegates.hpp"


namespace gui {

namespace priv {
class DataModelTemperatureAssignmentEditWidgetSignalSlots : public QWidget {
public:
	DataModelTemperatureAssignmentEditWidgetSignalSlots(QWidget *parent)
		:QWidget(parent)
	{}

	virtual ~DataModelTemperatureAssignmentEditWidgetSignalSlots()
	{}
};

class DataModelTemperatureAssignmentEditWidgetQtSignalsSlots : public DataModelTemperatureAssignmentEditWidgetSignalSlots {
	Q_OBJECT
public:
	DataModelTemperatureAssignmentEditWidgetQtSignalsSlots(QWidget *parent)
		:DataModelTemperatureAssignmentEditWidgetSignalSlots(parent)
	{}

	virtual ~DataModelTemperatureAssignmentEditWidgetQtSignalsSlots()
	{}

public slots:
	virtual void itemChanged(QListWidgetItem *item) = 0;
	//virtual void timeSeriesIndexChanged(int index) = 0;
};
} /* namespace priv */

template<typename DataModel>
class DataModelTemperatureAssignmentEditWidgetSignals {
public:
	COM_SUBSCRIPTION_FUNCTION(TemperatureAssignmentChanged, communication::Signal<void(boost::shared_ptr<DataModel>, int, bool)>);
};

template<typename DataModel>
class DataModelTemperatureAssignmentEditWidget  : public priv::DataModelTemperatureAssignmentEditWidgetQtSignalsSlots, public DataModelTemperatureAssignmentEditWidgetSignals<DataModel>{
public:
	DataModelTemperatureAssignmentEditWidget(QWidget *parent, boost::shared_ptr<DataModel> dataModel);
	virtual ~DataModelTemperatureAssignmentEditWidget();

private:
	void setupLayout();
	void updateValues();
	void connectLocal();

	void temperatureIndexChanged(int index);

	void itemChanged(QListWidgetItem *item);

private:
//	QLabel *temperatureLabel;
	QListWidget *temperatureListWidget;
	boost::shared_ptr<DataModel> dataModel;
	COM_CONNECTION dataModelLoadedConnection;
	COM_CONNECTION dateColumnChangedConnection;
	COM_CONNECTION behaviorIndicatorAssignmentChangedConnection;
	COM_CONNECTION waterLevelAssignmentChangedConnection;
};

template<typename DataModel>
inline gui::DataModelTemperatureAssignmentEditWidget<DataModel>::DataModelTemperatureAssignmentEditWidget(QWidget* parent, boost::shared_ptr<DataModel> dataModel)
	:priv::DataModelTemperatureAssignmentEditWidgetQtSignalsSlots(parent), dataModel(dataModel)
{
	setupLayout();
	updateValues();
	connectLocal();
}

template<typename DataModel>
inline gui::DataModelTemperatureAssignmentEditWidget<DataModel>::~DataModelTemperatureAssignmentEditWidget() {
}

template<typename DataModel>
inline void gui::DataModelTemperatureAssignmentEditWidget<DataModel>::setupLayout() {
	temperatureListWidget = new QListWidget(this);

	GroupBoxLayoutBuilder layoutBuilder(tr(""));
	layoutBuilder.addWidget(temperatureListWidget);

	setLayout(layoutBuilder.build(this));
}

template<typename DataModel>
inline void gui::DataModelTemperatureAssignmentEditWidget<DataModel>::updateValues() {
	typedef typename DataModel::Header_t Header_t;

	disconnect(temperatureListWidget, SIGNAL(itemChanged(QListWidgetItem *)), this, SLOT(itemChanged(QListWidgetItem *)));

	temperatureListWidget->clear();

	BOOST_FOREACH(Header_t const &header, dataModel->getAssignableTemperatureHeaders()) {
		std::cout << "#Temperature : " << header.getIndex() << std::endl;
		QListWidgetItem *item = new QListWidgetItem(header.getName(), temperatureListWidget);
		//QVariant index(header.getIndex());
		item->setData(Qt::UserRole, header.getIndex());
		item->setFlags(item->flags() | Qt::ItemIsUserCheckable);

		if(dataModel->isTemperatureColumnAssignedFor(header.getIndex())) {
			item->setCheckState(Qt::Checked);
		}else{
			item->setCheckState(Qt::Unchecked);
		}
		temperatureListWidget->addItem(item);
	}
	connect(temperatureListWidget, SIGNAL(itemChanged(QListWidgetItem *)), this, SLOT(itemChanged(QListWidgetItem *)));
}

template<typename DataModel>
inline void gui::DataModelTemperatureAssignmentEditWidget<DataModel>::connectLocal() {
	connect(temperatureListWidget, SIGNAL(itemChanged(QListWidgetItem *)), this, SLOT(itemChanged(QListWidgetItem *)));
	dataModelLoadedConnection = COM_SUBSCRIBE_FOR(dataModel, DataModelLoaded, (boost::bind(&DataModelTemperatureAssignmentEditWidget<DataModel>::updateValues, this)));
	dateColumnChangedConnection = COM_SUBSCRIBE_FOR(dataModel, DateColumnChanged, (boost::bind(&DataModelTemperatureAssignmentEditWidget<DataModel>::updateValues, this)));
	behaviorIndicatorAssignmentChangedConnection = COM_SUBSCRIBE_FOR(dataModel, BehaviorIndicatorAssignmentChanged, (boost::bind(&DataModelTemperatureAssignmentEditWidget<DataModel>::updateValues, this)));
	waterLevelAssignmentChangedConnection = COM_SUBSCRIBE_FOR(dataModel, WaterLevelAssignmentChanged, (boost::bind(&DataModelTemperatureAssignmentEditWidget<DataModel>::updateValues, this)));
}

template<typename DataModel>
inline void gui::DataModelTemperatureAssignmentEditWidget<DataModel>::itemChanged(QListWidgetItem *item) {
	bool assignmentState;
	if(item->checkState() == Qt::Checked) {
		assignmentState = true;
	}else if(item->checkState() == Qt::Unchecked){
		assignmentState = false;
	}else {
		return;
	}

	DataModelTemperatureAssignmentEditWidgetSignals<DataModel>::TemperatureAssignmentChangedSignal(dataModel, item->data(Qt::UserRole).toString().toInt(), assignmentState);
}

template<typename DataModel>
inline void gui::DataModelTemperatureAssignmentEditWidget<DataModel>::temperatureIndexChanged(int index) {
}


} /* namespace gui */


#endif /* DATAMODELTEMPERATUREASSIGNMENTEDITWIDGET_HPP_ */
