/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelTimeSeriesWidget.hpp
 *
 *  Created on: Jan 7, 2014
 *      Author: gerberma
 */

#ifndef DATAMODELTIMESERIESWIDGET_HPP_
#define DATAMODELTIMESERIESWIDGET_HPP_

#include <QObject>
#include <QVBoxLayout>
#include <QGridLayout>
#include <QWidget>
#include <QLabel>
#include <QGroupBox>
#include <QWizardPage>
#include <iostream>
#include <algorithm>
#include <boost/bind.hpp>
#include <boost/foreach.hpp>
#include <boost/ref.hpp>
#include <boost/shared_ptr.hpp>
#include "TimeSeriesEditWidget.hpp"
#include "communication/Signals.hpp"
#include "DataModelWizardPages.hpp"
#include "Delegates.hpp"
#include "communication/Signals.hpp"

namespace gui {

template
<
	typename DataModel,
	typename NextIdStrategy
>
class DataModelTimeSeriesWidget : public QWizardPage
{
public:
	DataModelTimeSeriesWidget(QWidget *parent, boost::shared_ptr<DataModel> dataModel, NextIdStrategy *nextIdStrategy, typename NextIdStrategy::type_t myId);
	virtual ~DataModelTimeSeriesWidget();

	COM_SUBSCRIPTION_DELEGATE_FUNCTION(TimeSeriesEditWidget<DataModel>, timeSeriesEditWidget, DateColumnChanged);
	COM_SUBSCRIPTION_DELEGATE_FUNCTION(TimeSeriesEditWidget<DataModel>, timeSeriesEditWidget, ApplyDateFormat);

	void setNextId(DataModelWizardPages page);
	int nextId() const;
	bool isComplete() const;

private:
	void setupLayout(boost::shared_ptr<DataModel> dataModel);
	void connectLocal();
	void completeStateHasBeenChanged();

private:
	boost::shared_ptr<DataModel> dataModel;
	SplittedLayoutBuilder splittedWizardPage;
	TimeSeriesEditWidget<DataModel> *timeSeriesEditWidget;
	NextIdStrategy *nextIdStrategy;
	typename NextIdStrategy::type_t myId;
	COM_CONNECTION dataModelChangedConnection;
};


template<typename DataModel, typename NextIdStrategy>
inline gui::DataModelTimeSeriesWidget<DataModel, NextIdStrategy>::DataModelTimeSeriesWidget(
		QWidget* parent,
		boost::shared_ptr<DataModel> dataModel,
		NextIdStrategy *nextIdStrategy,
		typename NextIdStrategy::type_t myId)
	:QWizardPage(parent), dataModel(dataModel), nextIdStrategy(nextIdStrategy), myId(myId)
{
	setupLayout(dataModel);
	connectLocal();
}

template<typename DataModel, typename NextIdStrategy>
inline gui::DataModelTimeSeriesWidget<DataModel, NextIdStrategy>::~DataModelTimeSeriesWidget() {
}

template<typename DataModel, typename NextIdStrategy>
inline void gui::DataModelTimeSeriesWidget<DataModel, NextIdStrategy>::setupLayout(boost::shared_ptr<DataModel> dataModel) {
	timeSeriesEditWidget = new TimeSeriesEditWidget<DataModel>(this, dataModel);

	setLayout(splittedWizardPage.build(this, timeSeriesEditWidget, NULL));
}


template<typename DataModel, typename NextIdStrategy>
inline int DataModelTimeSeriesWidget<DataModel, NextIdStrategy>::nextId() const {
	return nextIdStrategy->nextFrom(myId);
}

template<typename DataModel, typename NextIdStrategy>
inline void DataModelTimeSeriesWidget<DataModel, NextIdStrategy>::completeStateHasBeenChanged() {
	emit QWizardPage::completeChanged();
}

template<typename DataModel, typename NextIdStrategy>
inline bool DataModelTimeSeriesWidget<DataModel, NextIdStrategy>::isComplete() const {
	return dataModel->isDateColumnAssignmentComplete();
}

template<typename DataModel, typename NextIdStrategy>
inline void gui::DataModelTimeSeriesWidget<DataModel, NextIdStrategy>::connectLocal() {
	using namespace communication;
	using namespace boost;

	dataModelChangedConnection = COM_SUBSCRIBE_FOR(dataModel, Changed, bind(&gui::DataModelTimeSeriesWidget<DataModel, NextIdStrategy>::completeStateHasBeenChanged, this));
}

} /* namespace gui */

#endif /* DATAMODELTIMESERIESWIDGET_HPP_ */
