/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelTimeseriesDisplayWidget.hpp
 *
 *  Created on: Jan 14, 2014
 *      Author: gerberma
 */

#ifndef DATAMODELTIMESERIESDISPLAYWIDGET_HPP_
#define DATAMODELTIMESERIESDISPLAYWIDGET_HPP_

#include <boost/shared_ptr.hpp>
#include "OverviewKeyValueDisplayWidget.hpp"
#include "KeyValueLabel.hpp"

namespace gui {

template<typename DataModel>
class DataModelTimeseriesDisplayWidget : public OverviewKeyValueDisplayWidget<DataModel> {
public:
	DataModelTimeseriesDisplayWidget(boost::shared_ptr<DataModel> dataModel, QWidget *parent = 0);
	virtual ~DataModelTimeseriesDisplayWidget();

private:
	void setupLayout();
	void updateValues();
	void connectLocal();

private:
	boost::shared_ptr<DataModel> dataModel;
	KeyValueLabel *dateColumnLabels;
	COM_CONNECTION dataModelChangedConnection;
};

template<typename DataModel>
inline gui::DataModelTimeseriesDisplayWidget<DataModel>::DataModelTimeseriesDisplayWidget(boost::shared_ptr<DataModel> dataModel, QWidget* parent)
	:OverviewKeyValueDisplayWidget<DataModel>(dataModel, parent), dataModel(dataModel)
{
	setupLayout();
	updateValues();
	connectLocal();
}

template<typename DataModel>
inline gui::DataModelTimeseriesDisplayWidget<DataModel>::~DataModelTimeseriesDisplayWidget() {
}

template<typename DataModel>
inline void gui::DataModelTimeseriesDisplayWidget<DataModel>::setupLayout() {
	dateColumnLabels = new KeyValueLabel(QObject::tr("Date column"), "", this);
	OverviewKeyValueDisplayWidget<DataModel>::addKeyValueLabel(dateColumnLabels, 0);
	OverviewKeyValueDisplayWidget<DataModel>::setEditButton(0);

	OverviewKeyValueDisplayWidget<DataModel>::finishLayout();
}

template<typename DataModel>
inline void gui::DataModelTimeseriesDisplayWidget<DataModel>::updateValues() {
	if(dataModel->getDateColumn() != NULL) {
		dateColumnLabels->updateValueLabel(dataModel->getDateColumn()->getHeader().getName());
	}
}

template<typename DataModel>
inline void gui::DataModelTimeseriesDisplayWidget<DataModel>::connectLocal() {
	dataModelChangedConnection = COM_SUBSCRIBE_FOR(dataModel, Changes, boost::bind(&DataModelTimeseriesDisplayWidget<DataModel>::updateValues, this));
}

} /* namespace */

#endif /* DATAMODELTIMESERIESDISPLAYWIDGET_HPP_ */
