/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelWaterLevelAssignmentEditWidget.hpp
 *
 *  Created on: Jan 13, 2014
 *      Author: gerberma
 */

#ifndef DATAMODELWATERLEVELASSIGNMENTEDITWIDGET_HPP_
#define DATAMODELWATERLEVELASSIGNMENTEDITWIDGET_HPP_

#include <QObject>
#include <QObject>
#include <QVBoxLayout>
#include <QGridLayout>
#include <QWidget>
#include <QLabel>
#include <QListWidget>
#include <QGroupBox>
#include <QTabWidget>
#include <boost/bind.hpp>
#include <boost/foreach.hpp>
#include <boost/ref.hpp>
#include <boost/shared_ptr.hpp>
#include <iostream>
//#include "ComboBox.hpp"
#include "SplittedLayoutBuilder.hpp"
#include "GroupBoxLayoutBuilder.hpp"
#include "communication/Signals.hpp"
#include "DataModelWizardPages.hpp"
#include "Delegates.hpp"


namespace gui {

namespace priv {
class DataModelWaterLevelAssignmentEditWidgetSignalSlots : public QWidget {
public:
	DataModelWaterLevelAssignmentEditWidgetSignalSlots(QWidget *parent)
		:QWidget(parent)
	{}

	virtual ~DataModelWaterLevelAssignmentEditWidgetSignalSlots()
	{}
};

class DataModelWaterLevelAssignmentEditWidgetQtSignalsSlots : public DataModelWaterLevelAssignmentEditWidgetSignalSlots {
	Q_OBJECT
public:
	DataModelWaterLevelAssignmentEditWidgetQtSignalsSlots(QWidget *parent)
		:DataModelWaterLevelAssignmentEditWidgetSignalSlots(parent)
	{}

	virtual ~DataModelWaterLevelAssignmentEditWidgetQtSignalsSlots()
	{}

public slots:
	virtual void itemChanged(QListWidgetItem *item) = 0;
	//virtual void timeSeriesIndexChanged(int index) = 0;
};
} /* namespace priv */

template<typename DataModel>
class DataModelWaterLevelAssignmentEditWidgetSignals {
public:
	COM_SUBSCRIPTION_FUNCTION(WaterLevelAssignmentChanged, communication::Signal<void(boost::shared_ptr<DataModel>, int, bool)>);
};

template<typename DataModel>
class DataModelWaterLevelAssignmentEditWidget  : public priv::DataModelWaterLevelAssignmentEditWidgetQtSignalsSlots, public DataModelWaterLevelAssignmentEditWidgetSignals<DataModel>{
public:
	DataModelWaterLevelAssignmentEditWidget(QWidget *parent, boost::shared_ptr<DataModel> dataModel);
	virtual ~DataModelWaterLevelAssignmentEditWidget();

private:
	void setupLayout();
	void updateValues();
	void connectLocal();

	void waterLevelIndexChanged(int index);

	void itemChanged(QListWidgetItem *item);

private:
//	QLabel *waterLevelLabel;
	QListWidget *waterLevelListWidget;
	boost::shared_ptr<DataModel> dataModel;
	COM_CONNECTION dataModelLoadedConnection;
	COM_CONNECTION dateColumnChangedConnection;
	COM_CONNECTION behaviorIndicatorAssignmentChangedConnection;
	COM_CONNECTION temperatureAssignmentChangedConnection;
};

template<typename DataModel>
inline gui::DataModelWaterLevelAssignmentEditWidget<DataModel>::DataModelWaterLevelAssignmentEditWidget(QWidget* parent, boost::shared_ptr<DataModel> dataModel)
	:priv::DataModelWaterLevelAssignmentEditWidgetQtSignalsSlots(parent), dataModel(dataModel)
{
	setupLayout();
	updateValues();
	connectLocal();
}

template<typename DataModel>
inline gui::DataModelWaterLevelAssignmentEditWidget<DataModel>::~DataModelWaterLevelAssignmentEditWidget() {
}

template<typename DataModel>
inline void gui::DataModelWaterLevelAssignmentEditWidget<DataModel>::setupLayout() {
	waterLevelListWidget = new QListWidget(this);

	GroupBoxLayoutBuilder layoutBuilder(tr(""));
	layoutBuilder.addWidget(waterLevelListWidget);

	setLayout(layoutBuilder.build(this));
}

template<typename DataModel>
inline void gui::DataModelWaterLevelAssignmentEditWidget<DataModel>::updateValues() {
	typedef typename DataModel::Header_t Header_t;

	disconnect(waterLevelListWidget, SIGNAL(itemChanged(QListWidgetItem *)), this, SLOT(itemChanged(QListWidgetItem *)));

	waterLevelListWidget->clear();

	BOOST_FOREACH(Header_t const &header, dataModel->getAssignableWaterLevelHeaders()) {
		std::cout << "#WaterLevel : " << header.getIndex() << std::endl;
		QListWidgetItem *item = new QListWidgetItem(header.getName(), waterLevelListWidget);
		//QVariant index(header.getIndex());
		item->setData(Qt::UserRole, header.getIndex());
		item->setFlags(item->flags() | Qt::ItemIsUserCheckable);

		if(dataModel->isWaterLevelColumnAssignedFor(header.getIndex())) {
			item->setCheckState(Qt::Checked);
		}else{
			item->setCheckState(Qt::Unchecked);
		}
		waterLevelListWidget->addItem(item);
	}
	connect(waterLevelListWidget, SIGNAL(itemChanged(QListWidgetItem *)), this, SLOT(itemChanged(QListWidgetItem *)));
}

template<typename DataModel>
inline void gui::DataModelWaterLevelAssignmentEditWidget<DataModel>::connectLocal() {
	connect(waterLevelListWidget, SIGNAL(itemChanged(QListWidgetItem *)), this, SLOT(itemChanged(QListWidgetItem *)));
	dataModelLoadedConnection = COM_SUBSCRIBE_FOR(dataModel, DataModelLoaded, (boost::bind(&DataModelWaterLevelAssignmentEditWidget<DataModel>::updateValues, this)));
	dateColumnChangedConnection = COM_SUBSCRIBE_FOR(dataModel, DateColumnChanged, (boost::bind(&DataModelWaterLevelAssignmentEditWidget<DataModel>::updateValues, this)));
	behaviorIndicatorAssignmentChangedConnection = COM_SUBSCRIBE_FOR(dataModel, BehaviorIndicatorAssignmentChanged, (boost::bind(&DataModelWaterLevelAssignmentEditWidget<DataModel>::updateValues, this)));
	temperatureAssignmentChangedConnection = COM_SUBSCRIBE_FOR(dataModel,TemperatureAssignmentChanged, (boost::bind(&DataModelWaterLevelAssignmentEditWidget<DataModel>::updateValues, this)));
}

template<typename DataModel>
inline void gui::DataModelWaterLevelAssignmentEditWidget<DataModel>::itemChanged(QListWidgetItem *item) {
	bool assignmentState;
	if(item->checkState() == Qt::Checked) {
		assignmentState = true;
	}else if(item->checkState() == Qt::Unchecked){
		assignmentState = false;
	}else {
		return;
	}

	DataModelWaterLevelAssignmentEditWidgetSignals<DataModel>::WaterLevelAssignmentChangedSignal(dataModel, item->data(Qt::UserRole).toString().toInt(), assignmentState);
}

template<typename DataModel>
inline void gui::DataModelWaterLevelAssignmentEditWidget<DataModel>::waterLevelIndexChanged(int index) {
}


} /* namespace gui */


#endif /* DATAMODELWATERLEVELASSIGNMENTEDITWIDGET_HPP_ */
