/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionModelDataOverviewWidget.hpp
 *
 *  Created on: Mar 5, 2014
 *      Author: gerberma
 */

#ifndef REGRESSIONMODELDATAOVERVIEWWIDGET_HPP_
#define REGRESSIONMODELDATAOVERVIEWWIDGET_HPP_

#include <QWidget>
#include <QVBoxLayout>
#include <vector>
#include <boost/foreach.hpp>

namespace gui {

template<int Separator>
class DataOverviewWidget : public QWidget {
public:
	DataOverviewWidget(bool addSpacerAtTheEnd = false, QWidget *parent = 0):addSpacerAtTheEnd(addSpacerAtTheEnd){;}
	DataOverviewWidget(std::vector<QWidget *> widgets, bool addSpacerAtTheEnd = false, QWidget *parent = 0):addSpacerAtTheEnd(addSpacerAtTheEnd), widgets(widgets){;}


	void setWidgets(const std::vector<QWidget*>& widgets) {
		this->widgets = widgets;
	}


	void init() {
		setupLayout();
	}

private:
	void setupLayout() {
		QVBoxLayout *widgetsLayout = new QVBoxLayout(this);

		BOOST_FOREACH(QWidget *widget, widgets) {
			widgetsLayout->addWidget(widget);
			widgetsLayout->addWidget(widget);
		}

		if(addSpacerAtTheEnd == true) {
			widgetsLayout->addSpacerItem(new QSpacerItem(1,1,QSizePolicy::Ignored, QSizePolicy::Expanding));
		}

		setLayout(widgetsLayout);
	}

private:
	bool addSpacerAtTheEnd;
	std::vector<QWidget *> widgets;
};

} /* namespace gui */


#endif /* REGRESSIONMODELDATAOVERVIEWWIDGET_HPP_ */
