/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * Delegates.hpp
 *
 *  Created on: Dec 13, 2013
 *      Author: gerberma
 */

#ifndef DELEGATES_HPP_
#define DELEGATES_HPP_

#include <QObject>
#include <QPoint>
#include <boost/function.hpp>
#include "qcustomplot.h"

namespace communication {

class QtBoostDelegate : public QObject {
	Q_OBJECT
public:
	typedef boost::function<void()> DelegateFunction_t;
public:
	QtBoostDelegate(DelegateFunction_t delegateFunction, QObject *parent = 0)
		:QObject(parent), delegateFunction(delegateFunction)
	{}

	virtual ~QtBoostDelegate() {
	}

public slots:
	void execute() {
		delegateFunction();
	}

private:
	DelegateFunction_t delegateFunction;
};

class QtBoostIndexDelegate : public QObject {
	Q_OBJECT
public:
	typedef boost::function<void(int index)> DelegateFunction_t;
public:
	QtBoostIndexDelegate(DelegateFunction_t delegateFunction, QObject *parent = 0)
		:QObject(parent), delegateFunction(delegateFunction)
	{}

	virtual ~QtBoostIndexDelegate() {
	}

public slots:
	void execute(int index) {
		delegateFunction(index);
	}

private:
	DelegateFunction_t delegateFunction;
};

class QtBoostContextMenuDelegate : public QObject {
	Q_OBJECT
public:
	typedef boost::function<void(const QPoint &)> DelegateFunction_t;
public:
	QtBoostContextMenuDelegate(DelegateFunction_t delegateFunction, QObject *parent = 0)
		:QObject(parent), delegateFunction(delegateFunction)
	{}

	virtual ~QtBoostContextMenuDelegate() {
	}

public slots:
	void execute(const QPoint &point) {
		delegateFunction(point);
	}

private:
	DelegateFunction_t delegateFunction;
};

class QtBoostQCustomPlotContextMenuDelegate : public QObject {
	Q_OBJECT
public:
	typedef boost::function<void(QCPLegend *, QCPAbstractLegendItem *, QMouseEvent *)> DelegateFunction_t;
public:
	QtBoostQCustomPlotContextMenuDelegate(DelegateFunction_t delegateFunction, QObject *parent = 0)
		:QObject(parent), delegateFunction(delegateFunction)
	{}

	virtual ~QtBoostQCustomPlotContextMenuDelegate() {
	}

public slots:
	void execute(QCPLegend *legend, QCPAbstractLegendItem *item, QMouseEvent *mouseEvent) {
		delegateFunction(legend, item, mouseEvent);
	}

private:
	DelegateFunction_t delegateFunction;
};

/*
class QtBoostQCustomPlotLegendItemDelegate : public QObject {
	Q_OBJECT
public:
	typedef boost::function<void(QCPPlottableLegendItem *, QCPGraph *)> DelegateFunction_t;
public:
	QtBoostQCustomPlotLegendItemDelegate(DelegateFunction_t delegateFunction, QObject *parent = 0)
		:QObject(parent), delegateFunction(delegateFunction)
	{}

	virtual ~QtBoostQCustomPlotLegendItemDelegate() {
	}

public slots:
	void execute(QCPPlottableLegendItem *item, QCPGraph *graph) {
		delegateFunction(item, graph);
	}

private:
	DelegateFunction_t delegateFunction;
};
*/

class QtBoostBooleanDelegate : public QObject {
	Q_OBJECT
public:
	typedef boost::function<void(bool)> DelegateFunction_t;
public:
	QtBoostBooleanDelegate(DelegateFunction_t delegateFunction, QObject *parent = 0)
		:QObject(parent), delegateFunction(delegateFunction)
	{}

	virtual ~QtBoostBooleanDelegate() {
	}

public slots:
	void execute(int state) {
		execute((bool) state);
	}
	void execute(bool state) {
		delegateFunction(state);
	}

private:
	DelegateFunction_t delegateFunction;
};

class QtBoostStringDelegate : public QObject {
	Q_OBJECT
public:
	typedef boost::function<void(QString)> DelegateFunction_t;
public:
	QtBoostStringDelegate(DelegateFunction_t delegateFunction, QObject *parent = 0)
		:QObject(parent), delegateFunction(delegateFunction)
	{}

	virtual ~QtBoostStringDelegate() {
	}

public slots:
	void execute(QString value) {
		delegateFunction(value);
	}

private:
	DelegateFunction_t delegateFunction;
};


class BoostQtSimpleDelegate : public QObject {
	Q_OBJECT
public:
	BoostQtSimpleDelegate(const QObject *receiver, const char* delegateFunction, QObject *parent)
		:QObject(parent)
	{
		connect(this, SIGNAL(runSignal()), receiver, delegateFunction);
	}

	void execute() {
		emit runSignal();
	}

signals:
	void runSignal();
};

} /* namespace communication */


#endif /* DELEGATES_HPP_ */
