/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * GenericAnalysisToolWidget.hpp
 *
 *  Created on: Mar 27, 2014
 *      Author: gerberma
 */

#ifndef GENERICANALYSISTOOLWIDGET_HPP_
#define GENERICANALYSISTOOLWIDGET_HPP_

#include <boost/bind.hpp>
#include <boost/ref.hpp>
#include <boost/function.hpp>
#include <boost/shared_ptr.hpp>
#include <QVBoxLayout>
#include <QGridLayout>
#include "AnalysisToolWidget.hpp"
#include "communication/Signals.hpp"

namespace gui {

template<typename TargetAnalysisToolWidget>
class GenericAnalysisToolWidget : public AnalysisToolWidget {
public:
	typedef AnalysisToolWidget base_ui_type;
	typedef TargetAnalysisToolWidget TargetAnalysisToolWidget_t;
	typedef typename TargetAnalysisToolWidget::type_t type_t;

	COM_SUBSCRIPTION_FUNCTION(RemoveAnalysisTool, communication::Signal<void(boost::shared_ptr<type_t>)>);

public:
	GenericAnalysisToolWidget(Qt::Alignment targetAnalysisToolWidgetAlignment = 0, QWidget *parent = 0)
		:AnalysisToolWidget(parent), targetAnalysisToolWidgetAlignment(targetAnalysisToolWidgetAlignment), targetAnalysisToolWidget(NULL)
	{
	}

	void setTargetAnalysisToolWidget(TargetAnalysisToolWidget *targetAnalysisToolWidget) {
		this->targetAnalysisToolWidget = targetAnalysisToolWidget;
	}

	TargetAnalysisToolWidget *getTargetAnalysisToolWidget() {
		return targetAnalysisToolWidget;
	}

	void init() {
		if(targetAnalysisToolWidget == NULL) {
			throw std::logic_error("no target analysis widget defined");
		}

		setupLayout();
		connectLocal();
	}

private:
	void setupLayout() {
		QVBoxLayout *mainLayout = new QVBoxLayout;
			QVBoxLayout *contentLayout = new QVBoxLayout;
			contentLayout->addWidget(AnalysisToolWidget::getToolBar(), 0, Qt::AlignRight | Qt::AlignTop);
			contentLayout->addWidget(targetAnalysisToolWidget, 0, targetAnalysisToolWidgetAlignment);

			QGroupBox *groupBox = new QGroupBox(targetAnalysisToolWidget->getName(), this);
			groupBox->setLayout(contentLayout);
			groupBox->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);

			QSpacerItem *verticalExpandingSpacer = new QSpacerItem(1,1,QSizePolicy::Ignored, QSizePolicy::Expanding);
			mainLayout->addWidget(groupBox);
			mainLayout->addSpacerItem(verticalExpandingSpacer);

		setLayout(mainLayout);
	}

protected:
	void connectLocal() {
		using namespace boost;
		analysisToolToolBarRemoveButtonClickedConnection = COM_SUBSCRIBE_FOR(AnalysisToolWidget::getToolBar(), RemoveButtonClicked, (bind(ref(RemoveAnalysisToolSignal), targetAnalysisToolWidget->get())));
		loadedConnection = COM_SUBSCRIBE_FOR(targetAnalysisToolWidget->get(), Loaded, (bind(&TargetAnalysisToolWidget::updateValues, targetAnalysisToolWidget)));

		//targetAnalysisToolWidget->connectLocal();
	}

private:
	COM_CONNECTION analysisToolToolBarRemoveButtonClickedConnection;
	COM_CONNECTION loadedConnection;
	TargetAnalysisToolWidget *targetAnalysisToolWidget;
	Qt::Alignment targetAnalysisToolWidgetAlignment;
};

}  // namespace gui



#endif /* GENERICANALYSISTOOLWIDGET_HPP_ */
