/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * GenericPushButton.hpp
 *
 *  Created on: Apr 10, 2014
 *      Author: gerberma
 */

#ifndef GENERICPUSHBUTTON_HPP_
#define GENERICPUSHBUTTON_HPP_

#include <boost/bind.hpp>
#include <boost/ref.hpp>
#include <boost/shared_ptr.hpp>
#include <QPushButton>
#include "communication/Signals.hpp"
#include "Delegates.hpp"

namespace gui {

template<typename T>
class GenericPushButton : public QPushButton {
public:
	COM_SUBSCRIPTION_FUNCTION(Clicked, communication::Signal<void(boost::shared_ptr<T>)>);
public:
	GenericPushButton(boost::shared_ptr<T>t, QWidget *parent = 0)
		:QPushButton(parent), t(t)
	{
		setupLayout();
		connectLocal();
	}

private:
	void setupLayout() {
		setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
	}
	void connectLocal() {
		communication::QtBoostDelegate *clickedDelegate = new communication::QtBoostDelegate(boost::bind(boost::ref(GenericPushButton<T>::ClickedSignal), t), this);
		connect(this, SIGNAL(clicked()), clickedDelegate, SLOT(execute()));
	}

private:
	boost::shared_ptr<T> t;
};

}  // namespace gui


#endif /* GENERICPUSHBUTTON_HPP_ */
