/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * InfoDialog.hpp
 *
 *  Created on: Nov 22, 2013
 *      Author: gerberma
 */

#ifndef INFODIALOG_HPP_
#define INFODIALOG_HPP_

#include <QMessageBox>
#include <QString>
#include <QObject>
#include <boost/shared_ptr.hpp>

namespace gui {

class InfoDialog {
public:
	InfoDialog() {;}
	virtual ~InfoDialog() {;}

	static void showException(QString &exception){
		QMessageBox::question(0, QObject::tr("Error"), exception,
					QMessageBox::Ok);
	}

	static void showError(QString &error){
		InfoDialog::showException(error);
	}

	void showWarning(QString &warning) {
		QMessageBox::question(0, QObject::tr("Warning"), warning,
					QMessageBox::Ok);
	}

	static bool askYesNo(QString questionToAsk) {
		QMessageBox::StandardButton answerButton = QMessageBox::question( 0, QObject::tr("Confirmation required"),
																	questionToAsk,
																	QMessageBox::No | QMessageBox::Yes,
																	QMessageBox::Yes);
		return answerButton == QMessageBox::Yes;
	}

	template<typename T>
	bool confirmRemoveQuestionFor(boost::shared_ptr<T> t) {
		QString questionToAsk = QObject::tr("Are you sure you want to remove '") + t->getName() + QString("' ?");
		return InfoDialog::askYesNo(questionToAsk);
	}
};

} /* namespace gui */

#endif /* INFODIALOG_HPP_ */
