/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * KeyValueLabel.hpp
 *
 *  Created on: Jan 14, 2014
 *      Author: gerberma
 */

#ifndef KEYVALUELABEL_HPP_
#define KEYVALUELABEL_HPP_

#include <QObject>
#include <QString>
#include <QLabel>
#include <QSizePolicy>

namespace gui {

class KeyValueLabel : public QObject {
public:
	KeyValueLabel(QString key, QString value = "", QObject *parent = 0, int fixedWidth = 80)
		:QObject(parent)
	{
		keyLabel = new QLabel(key);
		valueLabel = new QLabel(value);

		keyLabel->setWordWrap(true);
		keyLabel->setFixedWidth(fixedWidth);
		valueLabel->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
		keyLabel->setBuddy(valueLabel);
	}

	QLabel *getKeyLabel() {
		return keyLabel;
	}

	QLabel *getValueLabel() {
		return valueLabel;
	}

	void hide() {
		valueLabel->hide();
		keyLabel->hide();
	}

	void show() {
		valueLabel->show();
		keyLabel->show();
	}

	void updateValueLabel(QString value) {
		valueLabel->setText(value);
	}

private:
	QLabel *keyLabel;
	QLabel *valueLabel;
};

} /* namespace gui */



#endif /* KEYVALUELABEL_HPP_ */
