/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * OverviewKeyValueDisplayWidget.hpp
 *
 *  Created on: Jan 14, 2014
 *      Author: gerberma
 */

#ifndef OVERVIEWKEYVALUEDISPLAYWIDGET_HPP_
#define OVERVIEWKEYVALUEDISPLAYWIDGET_HPP_

#include <QWidget>
#include <boost/bind.hpp>
#include <boost/ref.hpp>
#include <boost/shared_ptr.hpp>
#include "Delegates.hpp"
#include "communication/Signals.hpp"
#include "OverviewEditButton.hpp"
#include "OverviewKeyValueGridLayout.hpp"

namespace gui {

template<typename DataModel>
class OverviewKeyValueDisplayWidgetSignals {
public:
public:
	COM_SUBSCRIPTION_FUNCTION(Edit, communication::Signal<void(boost::shared_ptr<DataModel>)>);
};

template
<
	typename DataModel,
	typename EditButton = OverviewEditButton
>
class OverviewKeyValueDisplayWidget : public QWidget, public OverviewKeyValueDisplayWidgetSignals<DataModel> {
public:
	OverviewKeyValueDisplayWidget(boost::shared_ptr<DataModel> dataModel, QWidget *parent = 0);
	virtual ~OverviewKeyValueDisplayWidget();

	template<typename KeyValueLabel>
	void addKeyValueLabel(KeyValueLabel *labels, int lineIndex);

	void setEditButton(int lineIndex);
	void setButtonText(QString text);

	void finishLayout();

private:
	boost::shared_ptr<DataModel> dataModel;
	EditButton *editButton;
	OverviewKeyValueGridLayout *gridLayout;
};


template
<
	typename DataModel,
	typename EditButton
>
inline gui::OverviewKeyValueDisplayWidget<DataModel, EditButton>::OverviewKeyValueDisplayWidget(boost::shared_ptr<DataModel> dataModel, QWidget* parent)
	:QWidget(parent), dataModel(dataModel)
{
	editButton = new EditButton(this);
	gridLayout = new OverviewKeyValueGridLayout;
	communication::QtBoostDelegate *editDelegate = new communication::QtBoostDelegate(boost::bind(boost::ref(OverviewKeyValueDisplayWidgetSignals<DataModel>::EditSignal), dataModel), this);
	connect(editButton, SIGNAL(clicked()), editDelegate, SLOT(execute()));
}

template
<
	typename DataModel,
	typename EditButton
>
inline gui::OverviewKeyValueDisplayWidget<DataModel, EditButton>::~OverviewKeyValueDisplayWidget() {
}

template
<
	typename DataModel,
	typename EditButton
>
template<typename KeyValueLabel>
inline void gui::OverviewKeyValueDisplayWidget<DataModel, EditButton>::addKeyValueLabel(KeyValueLabel* labels, int lineIndex) {
	gridLayout->addKeyValueLabels(labels, lineIndex);
}

template
<
	typename DataModel,
	typename EditButton
>
inline void gui::OverviewKeyValueDisplayWidget<DataModel, EditButton>::setEditButton(int lineIndex) {
	gridLayout->setEditButton(editButton, lineIndex);
}

template
<
	typename DataModel,
	typename EditButton
>
inline void gui::OverviewKeyValueDisplayWidget<DataModel, EditButton>::setButtonText(QString text) {
	editButton->setText(text);
}

template
<
	typename DataModel,
	typename EditButton
>
inline void gui::OverviewKeyValueDisplayWidget<DataModel, EditButton>::finishLayout() {
	setLayout(gridLayout);
}

} /* namespace gui */

#endif /* OVERVIEWKEYVALUEDISPLAYWIDGET_HPP_ */
