/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * ProjectCommonParameterWindow.hpp
 *
 *  Created on: Nov 6, 2013
 *      Author: gerberma
 */

#ifndef PROJECTCOMMONPARAMETERWINDOW_HPP_
#define PROJECTCOMMONPARAMETERWINDOW_HPP_

#include <QObject>
#include <QWizardPage>
#include <QLabel>
#include <QLineEdit>
#include <QTextEdit>
#include <QGridLayout>
#include <QRegExpValidator>
#include <boost/shared_ptr.hpp>

namespace gui {

template<typename P>
class ProjectCommonParameterWindow: public QWizardPage {
public:
	ProjectCommonParameterWindow(QWidget *parent = 0);
	virtual ~ProjectCommonParameterWindow();

	void prepare(boost::shared_ptr<P> project);

	bool isComplete() const;

private:
	void setupLayout();

private:
	QLabel *nameLabel;
	QLineEdit *nameLineEdit;
	QLabel *descriptionLabel;
	QTextEdit *descriptionTextEdit;
	QLabel *damOperatorLabel;
	QLineEdit *damOperatorLineEdit;
};

} /* namespace gui */

template<typename P>
inline gui::ProjectCommonParameterWindow<P>::ProjectCommonParameterWindow(QWidget *parent)
	:QWizardPage(parent)
{
	setupLayout();
}

template<typename P>
inline gui::ProjectCommonParameterWindow<P>::~ProjectCommonParameterWindow() {
}

template<typename P>
inline void gui::ProjectCommonParameterWindow<P>::prepare(boost::shared_ptr<P> project) {
	nameLineEdit->setText(project->getName());
	descriptionTextEdit->setText(project->getDescription());
	damOperatorLineEdit->setText(project->getDamOperator());
}

template<typename P>
inline bool gui::ProjectCommonParameterWindow<P>::isComplete() const {
	QString nameText(nameLineEdit->text());
	int nameLength = nameText.length();
	if(nameLineEdit->validator()->validate(nameText, nameLength) == QValidator::Acceptable) {
		return true;
	}

	return false;
}

template<typename P>
inline void gui::ProjectCommonParameterWindow<P>::setupLayout() {
	setTitle(tr("Project information"));

	QRegExp rx("^(?!\\s*$).+");
	QValidator *validator = new QRegExpValidator(rx, this);
	nameLabel = new QLabel(tr("Name"));
	nameLineEdit = new QLineEdit;
	nameLineEdit->setValidator(validator);
	connect(nameLineEdit, SIGNAL(textChanged(const QString &)), this, SIGNAL(completeChanged()));
	nameLabel->setBuddy(nameLineEdit);
	descriptionLabel = new QLabel(tr("Description"));
	descriptionTextEdit = new QTextEdit;
	descriptionLabel->setBuddy(descriptionTextEdit);
	damOperatorLabel = new QLabel(tr("Dam operator"));
	damOperatorLineEdit = new QLineEdit;
	damOperatorLabel->setBuddy(damOperatorLineEdit);

	registerField("name", nameLineEdit);
	registerField("description", descriptionTextEdit, "plainText");
	registerField("damoperator", damOperatorLineEdit);

	QGridLayout *layout = new QGridLayout;
	layout->addWidget(nameLabel, 0, 0);
	layout->addWidget(nameLineEdit, 0, 1);
	layout->addWidget(descriptionLabel, 1, 0, Qt::AlignTop);
	layout->addWidget(descriptionTextEdit, 1, 1);
	layout->addWidget(damOperatorLabel, 2, 0);
	layout->addWidget(damOperatorLineEdit, 2, 1);

	setLayout(layout);
}
#endif /* PROJECTCOMMONPARAMETERWINDOW_HPP_ */
