/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * ProjectModeWindow.hpp
 *
 *  Created on: Nov 7, 2013
 *      Author: gerberma
 */

#ifndef PROJECTMODEWINDOW_HPP_
#define PROJECTMODEWINDOW_HPP_

#include <QWizardPage>
#include <QLabel>
#include <QRadioButton>
#include <QButtonGroup>
#include <QLineEdit>
#include <QTextEdit>
#include <QVBoxLayout>
#include <boost/bind.hpp>
#include <boost/shared_ptr.hpp>
#include "communication/Signals.hpp"
#include "Delegates.hpp"
//#include <QGridLayout>


namespace gui {

template<typename P>
class ProjectModeWindow: public QWizardPage {
public:
	COM_SUBSCRIPTION_FUNCTION(ModeChanged, communication::Signal<void(int)>);
public:
	ProjectModeWindow(QWidget *parent = 0);
	virtual ~ProjectModeWindow();

	void prepare(boost::shared_ptr<P> project);

private:
	void setupLayout();
	void modeChangedHandler();

private:
	QLabel *basicModeLabel;
	QRadioButton *basicModeRadioButton;
	QRadioButton *scientificModeRadioButton;
	QButtonGroup *modeRadioButtonGroup;
	boost::shared_ptr<P> project;
};



template<typename P>
inline ProjectModeWindow<P>::ProjectModeWindow(QWidget* parent)
	:QWizardPage(parent)
{
	setupLayout();
}

template<typename P>
inline ProjectModeWindow<P>::~ProjectModeWindow() {
}

template<typename P>
inline void ProjectModeWindow<P>::prepare(boost::shared_ptr<P> project) {
	using namespace boost;
	using namespace communication;

	this->project = project;

	if(project->isScientific()) {
		scientificModeRadioButton->setChecked(true);
		basicModeRadioButton->setChecked(false);
	}else{
		scientificModeRadioButton->setChecked(false);
		basicModeRadioButton->setChecked(true);
	}

	QtBoostDelegate *scientificModeRadioButtonDelegate = new QtBoostDelegate(bind(&ProjectModeWindow<P>::modeChangedHandler, this), this);
	connect(scientificModeRadioButton, SIGNAL(toggled(bool)), scientificModeRadioButtonDelegate, SLOT(execute()));
	QtBoostDelegate *basicModeRadioButtonDelegate = new QtBoostDelegate(bind(&ProjectModeWindow<P>::modeChangedHandler, this), this);
	connect(basicModeRadioButton, SIGNAL(toggled(bool)), basicModeRadioButtonDelegate, SLOT(execute()));
}

template<typename P>
inline void ProjectModeWindow<P>::modeChangedHandler() {
	if(scientificModeRadioButton->isChecked()) {
		ModeChangedSignal(1);
	}else if(basicModeRadioButton->isChecked()) {
		ModeChangedSignal(0);
	}
}

template<typename P>
inline void ProjectModeWindow<P>::setupLayout() {
	setTitle(tr("Choose mode"));
	setSubTitle(tr("Please choose the mode in which you want to run the software. You can switch between the modes later by using the 'Mode' menu"));

	//QGridLayout *layout = new QGridLayout;
	QVBoxLayout *mainLayout = new QVBoxLayout(this);

	basicModeRadioButton = new QRadioButton(tr("Basic mode"), this);
	scientificModeRadioButton = new QRadioButton(tr("Scientific mode"), this);

	modeRadioButtonGroup = new QButtonGroup(this);
	modeRadioButtonGroup->addButton(basicModeRadioButton);
	modeRadioButtonGroup->addButton(scientificModeRadioButton);

	mainLayout->addWidget(basicModeRadioButton);
	mainLayout->addWidget(scientificModeRadioButton);


	setLayout(mainLayout);
}


} /* namespace gui */

#endif /* PROJECTMODEWINDOW_HPP_ */
