/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * ProjectWizard.hpp
 *
 *  Created on: Nov 7, 2013
 *      Author: gerberma
 */

#ifndef PROJECTWIZARD_HPP_
#define PROJECTWIZARD_HPP_


#include <QObject>
#include <QWizard>
#include <boost/signals2/signal.hpp>
#include <boost/shared_ptr.hpp>
#include "BaseWizard.hpp"
#include "ProjectCommonParameterWindow.hpp"
#include "ProjectModeWindow.hpp"
#include "communication/Signals.hpp"

namespace gui {

template<typename P>
class ProjectWizard : public BaseWizard {
public:
	COM_SUBSCRIPTION_DELEGATE_FUNCTION(ProjectModeWindow<P>, modePage, ModeChanged);
public:
	//COM_SUBSCRIPTION_FUNCTION(Created, communication::Signal<void(P *)>);
	COM_SUBSCRIPTION_FUNCTION(Edited, communication::Signal<void()>);
public:
	ProjectWizard();
	virtual ~ProjectWizard();

	void start(boost::shared_ptr<P> project);
	void editCommonProperties(boost::shared_ptr<P> project);

	bool accepted();
	void resetProject();

protected:
	void accept();

private:
	enum Mode {Create, Edit};

private:
	boost::shared_ptr<P> project;
	ProjectCommonParameterWindow<P> *commonParameterPage;
	ProjectModeWindow<P> *modePage;
	Mode mode;
};

template<typename P>
inline ProjectWizard<P>::ProjectWizard()
	:mode(Create)
{
	commonParameterPage = new ProjectCommonParameterWindow<P>(this);
	modePage = new ProjectModeWindow<P>(this);

	setWizardStyle(ModernStyle);
}

template<typename P>
inline ProjectWizard<P>::~ProjectWizard() {
}

template<typename P>
inline void ProjectWizard<P>::start(boost::shared_ptr<P> project) {
	mode = Create;
	this->project = project;

	removePage(0);
	removePage(1);

	setPage(0, commonParameterPage);
	setPage(1, modePage);

	commonParameterPage->prepare(project);
	modePage->prepare(project);
	exec();
}

template<typename P>
inline void ProjectWizard<P>::editCommonProperties(boost::shared_ptr<P> project) {
	mode = Edit;
	this->project = project;

	restart();

	removePage(0);
	removePage(1);

	setPage(0, commonParameterPage);

	commonParameterPage->prepare(project);
	show();
}

template<typename P>
inline void ProjectWizard<P>::resetProject() {
	this->project = boost::shared_ptr<P>();
}

template<typename P>
inline bool ProjectWizard<P>::accepted() {
	return result() == Accepted;
}


template<typename P>
inline void ProjectWizard<P>::accept() {
	project->setName(field("name").toString());
	project->setDescription(field("description").toString());
	project->setDamOperator(field("damoperator").toString());

	QDialog::accept();
	if(mode == Create) {
		//CreatedSignal(project); //emit createdSignal();
	}else if(mode == Edit) {
		EditedSignal();//emit editedSignal();
	}
}

}/* namespace gui */
#endif /* PROJECTWIZARD_HPP_ */
