/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitAnalysisACFDisplayWidget.hpp
 *
 *  Created on: Apr 8, 2014
 *      Author: gerberma
 */

#ifndef REGRESSIONFITANALYSISACFDISPLAYWIDGET_HPP_
#define REGRESSIONFITANALYSISACFDISPLAYWIDGET_HPP_

#include <boost/foreach.hpp>
#include <boost/shared_ptr.hpp>
#include <QWidget>
#include <QVBoxLayout>
#include <QVector>
#include <QDebug>
#include "qcustomplot.h"
#include "Plottable.hpp"

namespace gui {

template<typename ACF>
class RegressionFitAnalysisACFDisplayWidget : public QWidget {
public:
	typedef ACF type_t;
public:
	RegressionFitAnalysisACFDisplayWidget(boost::shared_ptr<ACF> acf, QWidget *parent = 0)
		:QWidget(parent), acf(acf)
	{
		setupLayout();
		updateValues();
		plottable->rescaleAxes();
	}

	boost::shared_ptr<ACF> get() {
		return acf;
	}

	QString getName() {
		return "Residual ACF plot";
	}

	void setupLayout() {
		QVBoxLayout *mainLayout = new QVBoxLayout(this);
		plottable = new Plottable(this);
		mainLayout->addWidget(plottable->getPlot());
		setLayout(mainLayout);
	}

	void updateValues() {
		using namespace boost;

		QVector<double> acfs;
		BOOST_FOREACH(double value, acf->getAcfs()) {
			acfs.push_back(value);
		}

		QVector<double> lags;
		BOOST_FOREACH(unsigned value, acf->getLags()) {
			lags.push_back(value);
		}

		/**
		 * Confidence interval
		 */
		QVector<double> ciX;
		QVector<double> lowerCiY;
		QVector<double> upperCiY;

		ciX << 0 << acf->getLags().back();
		lowerCiY << acf->getLowerCi() << acf->getLowerCi();
		upperCiY << acf->getUpperCi() << acf->getUpperCi();


		plottable->preUpdate();
		QCustomPlot *plot = plottable->getPlot();

		QCPBars *acfBar = new QCPBars(plot->xAxis, plot->yAxis);
		plot->addPlottable(acfBar);
		acfBar->setName(tr("ACF"));

		QPen pen;
		pen.setWidth(0.01);
		pen.setColor(QColor(255, 131, 0));
		acfBar->setPen(pen);
		acfBar->setBrush(QColor(255, 131, 0, 50));
		acfBar->setData(lags, acfs);
		acfBar->setWidth(0.01);


		plot->addGraph();
		plot->graph(0)->setData(ciX, lowerCiY);
		plot->graph(0)->setName(tr("95% Confidence"));
		plottable->addGraphToLastLegendItem(plot->graph(0));
		plot->addGraph();
		plot->graph(1)->setData(ciX, upperCiY);
		plot->legend->removeItem(plot->legend->itemCount()-1);
		plottable->addGraphToLastLegendItem(plot->graph(1));

		/**
		 * General
		 */
		plot->xAxis->setAutoTickStep(false);
		plot->xAxis->setAutoSubTicks(false);
		plot->xAxis->setTickStep(20);
		plot->xAxis->setSubTickCount(19);
		plot->xAxis->setLabel(tr("lag (days)"));
		plot->yAxis->setLabel(tr("ACF"));
		plot->setMinimumHeight(200);

		plottable->postUpdate();
	}

	QCustomPlot *getPlot() {
		return plottable->getPlot();
	}

private:
	boost::shared_ptr<ACF> acf;
	Plottable *plottable;
};

} // namespace gui


#endif /* REGRESSIONFITANALYSISACFDISPLAYWIDGET_HPP_ */
