/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitAnalysisOverallRegressionQualityDisplayWidget.hpp
 *
 *  Created on: Apr 16, 2014
 *      Author: gerberma
 */

#ifndef REGRESSIONFITANALYSISOVERALLREGRESSIONQUALITYDISPLAYWIDGET_HPP_
#define REGRESSIONFITANALYSISOVERALLREGRESSIONQUALITYDISPLAYWIDGET_HPP_

#include <QWidget>
#include <QLabel>
#include <boost/shared_ptr.hpp>
//#include <QGridLayout>
#include "GroupBoxLayoutBuilder.hpp"
#include <map>

namespace gui {

template<typename Quality>
class RegressionFitAnalysisOverallRegressionQualityDisplayWidget : public QWidget {
public:
	typedef Quality type_t;
public:
	RegressionFitAnalysisOverallRegressionQualityDisplayWidget(boost::shared_ptr<Quality> quality, QWidget *parent = 0)
		:QWidget(parent), quality(quality)
	{
		setupLayout();
		connectLocal();
		updateValues();
	}

	QString getName() {
		return QString("Regression overall quality");
	}

	boost::shared_ptr<Quality> get() {
		return quality;
	}


	void setupLayout() {
		//QGridLayout *mainLayout = new QGridLayout(this);
		GroupBoxLayoutBuilder layoutBuilder(tr("Overall regression quality"));

		autocorrMapping[-1] = tr("Negative autocorrelation");
		autocorrMapping[0] = tr("No autocorrelation");
		autocorrMapping[1] = tr("Positive autocorrelation");

		autocorrLabel = new QLabel(tr("Autocorrelation (Durbin-Watson):"), this);
		autocorrLabel->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Ignored);
		autocorrDegree = new QLabel(this);
		autocorrValue = new QLabel(this);


		fitMapping[0] = tr("Perfect fit");
		fitMapping[1] = tr("Good fit");
		fitMapping[2] = tr("Bad fit");
		fitMapping[3] = tr("Very bad fit");

		fitLabel = new QLabel(tr("Overall fit (R2adj) : "), this);
		fitLabel->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Ignored);
		fitDegree = new QLabel(this);
		fitValue = new QLabel(this);


		layoutBuilder.addWidget(autocorrLabel, 0, 0, Qt::AlignLeft);
		layoutBuilder.addWidget(autocorrDegree, 0, 1, Qt::AlignLeft);
		layoutBuilder.addWidget(autocorrValue, 0, 2, Qt::AlignLeft);

		layoutBuilder.addWidget(fitLabel, 1, 0, Qt::AlignLeft);
		layoutBuilder.addWidget(fitDegree, 1, 1, Qt::AlignLeft);
		layoutBuilder.addWidget(fitValue, 1, 2, Qt::AlignLeft);

		setLayout(layoutBuilder.build(this));
	}

	void updateValues() {
		autocorrDegree->setText(autocorrMapping[quality->getAutocorrelationDegree()]);
		autocorrValue->setText(tr("value: ") + QString::number(quality->getAutocorrelationValue()));

		fitDegree->setText(fitMapping[quality->getFitDegree()]);
		fitValue->setText(tr("value: ") + QString::number(quality->getFitValue()));
	}

	void connectLocal() {

	}

private:
	boost::shared_ptr<Quality> quality;
	std::map<int, QString> autocorrMapping;
	std::map<double, QString> fitMapping;
	QLabel *autocorrLabel;
	QLabel *autocorrDegree;
	QLabel *autocorrValue;
	QLabel *fitLabel;
	QLabel *fitDegree;
	QLabel *fitValue;
};

}  // namespace gui


#endif /* REGRESSIONFITANALYSISOVERALLREGRESSIONQUALITYDISPLAYWIDGET_HPP_ */
