/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitAnalysisRegressionSummaryDisplayWidget.hpp
 *
 *  Created on: Mar 28, 2014
 *      Author: gerberma
 */

#ifndef REGRESSIONFITANALYSISREGRESSIONSUMMARYDISPLAYWIDGET_HPP_
#define REGRESSIONFITANALYSISREGRESSIONSUMMARYDISPLAYWIDGET_HPP_

#include <QDebug>
#include <QWidget>
#include <QLabel>
#include <QFrame>
#include <QVBoxLayout>
#include <boost/bind.hpp>
#include <boost/ref.hpp>
#include <boost/shared_ptr.hpp>
#include "KeyValueLabel.hpp"
#include "communication/Signals.hpp"
#include "Delegates.hpp"

namespace gui {

template
<
	typename Summary
>
class RegressionFitAnalysisRegressionSummaryDisplayWidget : public QWidget {
public:
	typedef Summary type_t;
public:
	RegressionFitAnalysisRegressionSummaryDisplayWidget(boost::shared_ptr<Summary> summary, QWidget *parent = 0)
		:QWidget(parent), summary(summary)
	{
		setupLayout();
		connectLocal();
		updateValues();
	}

	virtual ~RegressionFitAnalysisRegressionSummaryDisplayWidget(){;}

	QString getName() {
		return QString("Regression summary");
	}

	boost::shared_ptr<Summary> get() {
		return summary;
	}

	void setupLayout(){
		QVBoxLayout *mainLayout = new QVBoxLayout;

		//fill elements
		QGridLayout *dataLayout = new QGridLayout(this);
		QWidget *dataWidget = new QWidget(this);
		dataWidget->setLayout(dataLayout);

		ssResKeyValueLabel = new KeyValueLabel(QObject::tr("SS Res"), "", dataWidget, 120);
		msResKeyValueLabel = new KeyValueLabel(QObject::tr("MS Res"), "", dataWidget, 120);
		msResDerivedKeyValueLabel = new KeyValueLabel(QObject::tr("MS'Res"), "", dataWidget, 120);
		r2KeyValueLabel = new KeyValueLabel(QObject::tr("R2"), "", dataWidget, 120);
		r2adjKeyValueLabel = new KeyValueLabel(QObject::tr("R2adj"), "", dataWidget, 120);
		fstatKeyValueLabel = new KeyValueLabel(QObject::tr("F-stat"), "", dataWidget, 120);
		fstatPvalueKeyValueLabel = new KeyValueLabel(QObject::tr("signif F"), "", dataWidget, 120);
		dwStatKeyValueLabel = new KeyValueLabel(QObject::tr("DW-stat"), "", dataWidget, 120);
		nobsKeyValueLabel = new KeyValueLabel(QObject::tr("Observ"), "", dataWidget, 120);
		ssResPredKeyValueLabel = new KeyValueLabel(QObject::tr("SS Res (prediction)"), "", dataWidget, 120);
		msResDerivedPredKeyValueLabel = new KeyValueLabel(QObject::tr("MS'Res (prediction)"), "", dataWidget, 120);


		dataLayout->addWidget(ssResKeyValueLabel->getKeyLabel(), 0, 0, Qt::AlignBottom);
		dataLayout->addWidget(ssResKeyValueLabel->getValueLabel(), 0, 1, Qt::AlignTop);
		dataLayout->addWidget(msResKeyValueLabel->getKeyLabel(), 1, 0, Qt::AlignBottom);
		dataLayout->addWidget(msResKeyValueLabel->getValueLabel(), 1, 1, Qt::AlignTop);
		dataLayout->addWidget(msResDerivedKeyValueLabel->getKeyLabel(), 2, 0, Qt::AlignBottom);
		dataLayout->addWidget(msResDerivedKeyValueLabel->getValueLabel(), 2, 1, Qt::AlignTop);
		QFrame* lineResFrame = new QFrame(this);
		lineResFrame->setFrameShape(QFrame::HLine);
		dataLayout->addWidget(lineResFrame, 3, 0, 1, 2);

		dataLayout->addWidget(r2KeyValueLabel->getKeyLabel(), 4, 0, Qt::AlignBottom);
		dataLayout->addWidget(r2KeyValueLabel->getValueLabel(), 4, 1, Qt::AlignTop);
		dataLayout->addWidget(r2adjKeyValueLabel->getKeyLabel(), 5, 0, Qt::AlignBottom);
		dataLayout->addWidget(r2adjKeyValueLabel->getValueLabel(), 5, 1, Qt::AlignTop);
		QFrame* lineRFrame = new QFrame(this);
		lineRFrame->setFrameShape(QFrame::HLine);
		dataLayout->addWidget(lineRFrame, 6, 0, 1, 2);

		dataLayout->addWidget(fstatKeyValueLabel->getKeyLabel(), 7, 0, Qt::AlignBottom);
		dataLayout->addWidget(fstatKeyValueLabel->getValueLabel(), 7, 1, Qt::AlignTop);
		dataLayout->addWidget(fstatPvalueKeyValueLabel->getKeyLabel(), 8, 0, Qt::AlignBottom);
		dataLayout->addWidget(fstatPvalueKeyValueLabel->getValueLabel(), 8, 1, Qt::AlignTop);
		lineFStatFrame = new QFrame(this);
		lineFStatFrame->setFrameShape(QFrame::HLine);
		dataLayout->addWidget(lineFStatFrame, 9, 0, 1, 2);

		dataLayout->addWidget(dwStatKeyValueLabel->getKeyLabel(), 10, 0, Qt::AlignBottom);
		dataLayout->addWidget(dwStatKeyValueLabel->getValueLabel(), 10, 1, Qt::AlignTop);
		dataLayout->addWidget(nobsKeyValueLabel->getKeyLabel(), 11, 0, Qt::AlignBottom);
		dataLayout->addWidget(nobsKeyValueLabel->getValueLabel(), 11, 1, Qt::AlignTop);
		QFrame* linePredictionFrame = new QFrame(this);
		linePredictionFrame->setFrameShape(QFrame::HLine);
		dataLayout->addWidget(linePredictionFrame, 12, 0, 1, 2);

		dataLayout->addWidget(ssResPredKeyValueLabel->getKeyLabel(), 13, 0, Qt::AlignBottom);
		dataLayout->addWidget(ssResPredKeyValueLabel->getValueLabel(), 13, 1, Qt::AlignTop);
		dataLayout->addWidget(msResDerivedPredKeyValueLabel->getKeyLabel(), 14, 0, Qt::AlignBottom);
		dataLayout->addWidget(msResDerivedPredKeyValueLabel->getValueLabel(), 14, 1, Qt::AlignTop);

		mainLayout->addWidget(dataWidget);

		setLayout(mainLayout);
	}

	void updateValues(){
		ssResKeyValueLabel->updateValueLabel(formatValue(summary->getSSRes(), summary->getSSRes(true)));
		msResKeyValueLabel->updateValueLabel(formatValue(summary->getMSRes(), summary->getMSRes(true)));
		msResDerivedKeyValueLabel->updateValueLabel(formatValue(summary->getMSResDerived(), summary->getMSResDerived(true)));
		r2KeyValueLabel->updateValueLabel(formatValue(summary->getR2(), summary->getR2(true)));
		r2adjKeyValueLabel->updateValueLabel(formatValue(summary->getR2Adj(), summary->getR2Adj(true)));
		fstatKeyValueLabel->updateValueLabel(formatValue(summary->getFStat(), summary->getFStat(true)));
		fstatPvalueKeyValueLabel->updateValueLabel(formatValue(summary->getFStatPValue(), summary->getFStatPValue(true)));
		dwStatKeyValueLabel->updateValueLabel(formatValue(summary->getDwStat(), summary->getDwStat(true)));
		nobsKeyValueLabel->updateValueLabel(formatValue(summary->getNobs(), summary->getNobs(true)));
		ssResPredKeyValueLabel->updateValueLabel(formatValue(summary->getSSResPred(), summary->getSSResPred(true)));
		msResDerivedPredKeyValueLabel->updateValueLabel(formatValue(summary->getMSResDerivedPred(), summary->getMSResDerivedPred(true)));

		if(summary->isScientific()) {
			ssResKeyValueLabel->show();
			msResKeyValueLabel->show();
			msResDerivedKeyValueLabel->show();
			r2KeyValueLabel->show();
			r2adjKeyValueLabel->show();
			fstatKeyValueLabel->show();
			fstatPvalueKeyValueLabel->show();
			dwStatKeyValueLabel->show();
			nobsKeyValueLabel->show();
			ssResPredKeyValueLabel->show();
			msResDerivedPredKeyValueLabel->show();
			lineFStatFrame->show();
		}else{
			ssResKeyValueLabel->hide();
			msResDerivedKeyValueLabel->hide();
			r2KeyValueLabel->hide();
			fstatKeyValueLabel->hide();
			fstatPvalueKeyValueLabel->hide();
			dwStatKeyValueLabel->hide();
			ssResPredKeyValueLabel->hide();
			lineFStatFrame->hide();
		}
	}

	QString formatValue(double current, double previous) {
		//return QString("%1(%2)").arg(QString::number(current), -30, ' ').arg(QString::number(current - previous), 10, ' ');
		//return QString("%1 (%2)").arg(QString::number(current)).leftJustified(20, ' ', true).arg(QString::number(current - previous));
		return QString("%1 (%2)").arg(QString::number(current)).arg(QString::number(current - previous));
	}

	void connectLocal(){
			modeHandlerChangedConnection = COM_SUBSCRIBE_FOR(summary->getModeHandler(), ModeChanged, boost::bind(&RegressionFitAnalysisRegressionSummaryDisplayWidget<Summary>::updateValues, this));
	}

private:
	boost::shared_ptr<Summary> summary;
	KeyValueLabel *ssResKeyValueLabel;
	KeyValueLabel *msResKeyValueLabel;
	KeyValueLabel *msResDerivedKeyValueLabel;
	KeyValueLabel *r2KeyValueLabel;
	KeyValueLabel *r2adjKeyValueLabel;
	KeyValueLabel *fstatKeyValueLabel;
	KeyValueLabel *fstatPvalueKeyValueLabel;
	KeyValueLabel *dwStatKeyValueLabel;
	KeyValueLabel *nobsKeyValueLabel;
	KeyValueLabel *ssResPredKeyValueLabel;
	KeyValueLabel *msResDerivedPredKeyValueLabel;
	QFrame* lineFStatFrame;
	COM_CONNECTION modeHandlerChangedConnection;
};

}  // namespace gui


#endif /* REGRESSIONFITANALYSISREGRESSIONSUMMARYDISPLAYWIDGET_HPP_ */
