/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitAnalysisRegressorOverviewEditWidget.hpp
 *
 *  Created on: Mar 26, 2014
 *      Author: gerberma
 */

#ifndef REGRESSIONFITANALYSISREGRESSOROVERVIEWEDITWIDGET_HPP_
#define REGRESSIONFITANALYSISREGRESSOROVERVIEWEDITWIDGET_HPP_

#include <QDebug>
#include <QObject>
#include <QWidget>
#include <QLabel>
#include <QGroupBox>
#include <QVBoxLayout>
#include <QGridLayout>
#include <QSpacerItem>
#include <QTableWidget>
#include <QHeaderView>
#include <QScrollBar>
#include <QCheckBox>
#include <boost/bind.hpp>
#include <boost/ref.hpp>
#include <boost/foreach.hpp>
#include <boost/shared_ptr.hpp>
#include "communication/Signals.hpp"
#include "Delegates.hpp"

namespace gui {

namespace priv {
	class RegressionFitAnalysisRegressorOverviewEditWidgetBase : public QWidget {
	public:
		RegressionFitAnalysisRegressorOverviewEditWidgetBase(QWidget *parent)
			:QWidget(parent)
		{;}

		virtual ~RegressionFitAnalysisRegressorOverviewEditWidgetBase(){;}
	};

	class RegressionFitAnalysisRegressorOverviewEditWidgetQtSlots : public RegressionFitAnalysisRegressorOverviewEditWidgetBase {
		Q_OBJECT
	public:
		RegressionFitAnalysisRegressorOverviewEditWidgetQtSlots(QWidget *parent)
			:RegressionFitAnalysisRegressorOverviewEditWidgetBase(parent){;}
		virtual ~RegressionFitAnalysisRegressorOverviewEditWidgetQtSlots(){;}

	public slots:
		virtual void tableWidgetItemClicked(QTableWidgetItem *tableWidgetItem) = 0;
	};
} //namespace priv

template
<
	typename RegressionFitAnalysisRegressorOverview
>
class RegressionFitAnalysisRegressorOverviewEditWidget : public priv::RegressionFitAnalysisRegressorOverviewEditWidgetQtSlots {
public:
	typedef RegressionFitAnalysisRegressorOverview type_t;

	COM_SUBSCRIPTION_FUNCTION(RegressorActiveStateChanged, communication::Signal<void(boost::shared_ptr<RegressionFitAnalysisRegressorOverview>, int, bool)>);

public:
	RegressionFitAnalysisRegressorOverviewEditWidget(boost::shared_ptr<RegressionFitAnalysisRegressorOverview> overview, QWidget *parent = 0)
		:priv::RegressionFitAnalysisRegressorOverviewEditWidgetQtSlots(parent), overview(overview), table(NULL)
	{
		setupLayout();
		connectLocal();
		updateValues();
	}

	virtual ~RegressionFitAnalysisRegressorOverviewEditWidget()
	{;}

	QString getName() {
		return tr("Regressor overview");
	}

	boost::shared_ptr<RegressionFitAnalysisRegressorOverview> get() {
		return overview;
	}

	void setupLayout() {
		QVBoxLayout *mainLayout = new QVBoxLayout;
			enableSortingCheckBox = new QCheckBox(tr("enable sorting"), this);
			enableSortingCheckBox->setChecked(false);
			table = new QTableWidget(this);
			table->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
			//table->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
			mainLayout->addWidget(enableSortingCheckBox, 1, Qt::AlignLeft);
			mainLayout->addWidget(table);
		setLayout(mainLayout);
	}

	void updateValues() {
		table->setSortingEnabled(false);
		table->clear();
		QStringList headers;
		headers.push_back(tr("Regressor"));
		if(overview->isScientific()) {
			headers.push_back(tr("t-stat"));
			headers.push_back(tr("p-value"));
			headers.push_back(tr("VIF"));
		}
		headers.push_back(tr("Coefficient"));
		if(overview->isScientific()) {
			headers.push_back(tr("StdError"));
			headers.push_back(tr("Std Coefficient"));
		}
		table->setColumnCount(headers.size());
		table->setRowCount(overview->getRegressorItems().size() + 1);
		table->setHorizontalHeaderLabels(headers);

		addInterceptTableRow(overview->getInterceptItem(), 0);
		addRegressorTableRows(overview->getRegressorItems(), 1);

		table->resizeRowsToContents();
		table->resizeColumnsToContents();


		//from: http://www.qtcentre.org/threads/12307-Adjust-QTableWidget-to-contents
		//Width of the table
		int iWidth = 0;
		for (int i=0; i<table->columnCount(); i++)
		{
		iWidth += table->horizontalHeader()->sectionSize(i);
		}
		iWidth += table->verticalHeader()->width();
		table->setMinimumWidth(iWidth);

		//Height of the table
		int iHeight = 0;
		for (int i=0; i<table->rowCount(); i++)
		{
		iHeight += table->verticalHeader()->sectionSize(i);
		}
		iHeight += table->horizontalHeader()->height();
		table->setFixedHeight(iHeight + table->horizontalScrollBar()->height());

		table->setSortingEnabled(enableSortingCheckBox->isChecked());
	}

	void connectLocal() {
		using namespace boost;
		using namespace communication;

		connect(table, SIGNAL(itemClicked(QTableWidgetItem *)), this, SLOT(tableWidgetItemClicked(QTableWidgetItem *)));

		modeHandlerChangedConnection = COM_SUBSCRIBE_FOR(overview->getModeHandler(), ModeChanged, bind(&RegressionFitAnalysisRegressorOverviewEditWidget<RegressionFitAnalysisRegressorOverview>::updateValues, this));

		communication::QtBoostDelegate *sortCheckBoxDelegate = new QtBoostDelegate(bind(&RegressionFitAnalysisRegressorOverviewEditWidget<RegressionFitAnalysisRegressorOverview>::updateValues, this), this);
		connect(enableSortingCheckBox, SIGNAL(clicked(bool)), sortCheckBoxDelegate, SLOT(execute()));
	}

	template<typename InterceptItem>
	void addInterceptTableRow(boost::shared_ptr<InterceptItem> interceptItem, int index) {
		if(interceptItem == NULL) {
			return;
		}

		QTableWidgetItem *item = new QTableWidgetItem(interceptItem->getName());
		table->setItem(index, 0, item);
		addGeneralInformation(interceptItem, index);
	}

	template<typename Item>
	void addRegressorTableRows(std::vector<boost::shared_ptr<Item> > items, int startIndex) {
		int i = startIndex;
		BOOST_FOREACH(boost::shared_ptr<Item> regressorItem, items) {
			addRegressorTableRow(regressorItem, i);
			++i;
		}
	}

	template<typename Item>
	void addRegressorTableRow(boost::shared_ptr<Item> regressorOverview, int index) {
		if(regressorOverview == NULL) {
			return;
		}

		QTableWidgetItem *item = new QTableWidgetItem(regressorOverview->getName());
		if(regressorOverview->isEditable()) {
			item->setCheckState(regressorOverview->isActive() ? Qt::Checked : Qt::Unchecked);
		}
		item->setData(Qt::UserRole+1, regressorOverview->getRegressorIndex());
		table->setItem(index, 0, item);
		addGeneralInformation(regressorOverview, index);
	}


	template<typename Item>
	void addGeneralInformation(boost::shared_ptr<Item> item, int index) {
		int position = 1;
		if(overview->isScientific()) {
			table->setItem(index, position++, new QTableWidgetItem(QString::number(item->getTStat())));
			table->setItem(index, position++, new QTableWidgetItem(QString::number(item->getPValue())));
			table->setItem(index, position++, new QTableWidgetItem(QString::number(item->getVIF())));
		}

		table->setItem(index, position++, new QTableWidgetItem(QString::number(item->getCoefficient())));

		if(overview->isScientific()) {
			table->setItem(index, position++, new QTableWidgetItem(QString::number(item->getStdError())));
			table->setItem(index, position++, new QTableWidgetItem(QString::number(item->getStdCoefficient())));
		}
	}

private:
	void tableWidgetItemClicked(QTableWidgetItem *tableWidgetItem){
		QVariant isActiveData = tableWidgetItem->data(Qt::UserRole+1);
		if(isActiveData.isValid()) {
			int regressorIndex = isActiveData.toString().toInt();
			RegressorActiveStateChangedSignal(overview, regressorIndex, tableWidgetItem->checkState() == Qt::Checked ? true : false);
		}
	}

private:
	boost::shared_ptr<RegressionFitAnalysisRegressorOverview> overview;
	QTableWidget *table;
	QCheckBox *enableSortingCheckBox;
	COM_CONNECTION modeHandlerChangedConnection;
};


}  // namespace gui


#endif /* REGRESSIONFITANALYSISREGRESSOROVERVIEWEDITWIDGET_HPP_ */
