/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitAnalysisResidualNormalProbabilityDisplayWidget.hpp
 *
 *  Created on: Apr 2, 2014
 *      Author: gerberma
 */

#ifndef REGRESSIONFITANALYSISRESIDUALNORMALPROBABILITYDISPLAYWIDGET_HPP_
#define REGRESSIONFITANALYSISRESIDUALNORMALPROBABILITYDISPLAYWIDGET_HPP_

#include <QDebug>
#include <QWidget>
#include <QVBoxLayout>
#include <QVector>
#include <boost/foreach.hpp>
#include <boost/shared_ptr.hpp>
#include "qcustomplot.h"
#include "Plottable.hpp"

namespace gui {

template<typename NormalProbability>
class RegressionFitAnalysisResidualNormalProbabilityDisplayWidget : public QWidget {
public:
	typedef NormalProbability type_t;
public:
	RegressionFitAnalysisResidualNormalProbabilityDisplayWidget(boost::shared_ptr<NormalProbability> normalProbability, QWidget *parent = 0)
		:QWidget(parent), normalProbability(normalProbability)
	{
		setupLayout();
		updateValues();
		plottable->rescaleAxes();
	}

	boost::shared_ptr<NormalProbability> get() {
		return normalProbability;
	}

	QString getName() {
		return "Residual normal probability";
	}

	void setupLayout() {
		QVBoxLayout *mainLayout = new QVBoxLayout(this);
		plottable = new Plottable(this);
		mainLayout->addWidget(plottable->getPlot());
		setLayout(mainLayout);
	}

	void updateValues() {

		QVector<double> theoreticalQuantiles;
		BOOST_FOREACH(double value, normalProbability->getTheoreticalQuantiles()) {
			theoreticalQuantiles.push_back(value);
		}

		QVector<double> sampleQuantiles;
		BOOST_FOREACH(double value, normalProbability->getSampleQuantiles()) {
			sampleQuantiles.push_back(value);
		}

		plottable->preUpdate();
		QCustomPlot *plot = plottable->getPlot();

		plot->addGraph();
		plot->graph(0)->setData(theoreticalQuantiles, sampleQuantiles);
		plot->graph(0)->setLineStyle(QCPGraph::lsNone);
		plot->graph(0)->setScatterStyle(QCPScatterStyle(QCPScatterStyle::ssCircle, Qt::black, Qt::white, 5));
		plot->graph(0)->setName(tr("Residuals"));
		plottable->addGraphToLastLegendItem(plot->graph(0));

		//line
		plot->addGraph();
		QVector<double> lineX;
		QVector<double> lineY;

		lineX << normalProbability->getNormalLinePointOne().getX() << normalProbability->getNormalLinePointTwo().getX();
		lineY << normalProbability->getNormalLinePointOne().getY() << normalProbability->getNormalLinePointTwo().getY();
		plot->graph(1)->setData(lineX, lineY);
		plot->graph(1)->setName(tr("Normal Distribution"));
		plottable->addGraphToLastLegendItem(plot->graph(1));


		plot->xAxis->setLabel(tr("Probability (Standard Deviation)"));
		plot->yAxis->setLabel(tr("Standardized Residuals"));

		plot->setMinimumHeight(200);

		plottable->postUpdate();
	}

	QCustomPlot *getPlot() {
		return plottable->getPlot();
	}

private:
	boost::shared_ptr<NormalProbability> normalProbability;
	Plottable *plottable;
};

}  // namespace gui


#endif /* REGRESSIONFITANALYSISRESIDUALNORMALPROBABILITYDISPLAYWIDGET_HPP_ */
