/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitAnalysisResidualVersusRegressionParameterDisplayWidget.hpp
 *
 *  Created on: Apr 10, 2014
 *      Author: gerberma
 */

#ifndef REGRESSIONFITANALYSISRESIDUALVERSUSREGRESSIONPARAMETERDISPLAYWIDGET_HPP_
#define REGRESSIONFITANALYSISRESIDUALVERSUSREGRESSIONPARAMETERDISPLAYWIDGET_HPP_

#include <QWidget>
#include <QVBoxLayout>
#include <QVector>
#include <boost/foreach.hpp>
#include <boost/shared_ptr.hpp>
#include "qcustomplot.h"

namespace gui {

template<typename Versus>
class RegressionFitAnalysisResidualVersusRegressionParameterDisplayWidget : public QWidget {
public:
	typedef Versus type_t;
public:
	RegressionFitAnalysisResidualVersusRegressionParameterDisplayWidget(boost::shared_ptr<Versus> versus, QWidget *parent = 0)
		:QWidget(parent), versus(versus), plot(NULL)
	{
		setupLayout();
		updateValues();
		plot->rescaleAxes();
	}

	boost::shared_ptr<Versus> get() {
		return versus;
	}

	QString getName() {
		return "Tukey-Anscombe plot (residual vs. fitted)";
	}

	void setupLayout() {
		QVBoxLayout *mainLayout = new QVBoxLayout(this);
		plot = new QCustomPlot(this);
		plot->setInteractions(QCP::iRangeDrag | QCP::iRangeZoom);

		mainLayout->addWidget(plot);
		setLayout(mainLayout);
	}

	void updateValues() {
		using namespace boost;

		QVector<double> residuals;
		BOOST_FOREACH(double value, versus->getResiduals()) {
			residuals.push_back(value);
		}

		QVector<double> parameterValues;
		BOOST_FOREACH(double value, versus->getParameterValues()) {
			parameterValues.push_back(value);
		}

		plot->clearGraphs();
		plot->addGraph();
		plot->graph(0)->setData(parameterValues, residuals);
		plot->graph(0)->setLineStyle(QCPGraph::lsNone);
		plot->graph(0)->setScatterStyle(QCPScatterStyle(QCPScatterStyle::ssCircle, Qt::black, Qt::white, 5));

		plot->xAxis->setLabel(tr("Fitted"));
		plot->yAxis->setLabel(tr("Studentized Residuals"));

		plot->setMinimumHeight(200);
		plot->replot();
	}

	QCustomPlot *getPlot() {
		return plot;
	}

private:
	boost::shared_ptr<Versus> versus;
	QCustomPlot *plot;
};

} // namespace gui

#endif /* REGRESSIONFITANALYSISRESIDUALVERSUSREGRESSIONPARAMETERDISPLAYWIDGET_HPP_ */
