/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionModelAlgorithmDisplayWidget.hpp
 *
 *  Created on: 27.01.2015
 *      Author: gerberma
 */

#ifndef REGRESSIONMODELALGORITHMDISPLAYWIDGET_HPP_
#define REGRESSIONMODELALGORITHMDISPLAYWIDGET_HPP_

#include <boost/bind.hpp>
#include <boost/shared_ptr.hpp>
#include <QDebug>
#include "OverviewKeyValueDisplayWidget.hpp"
#include "KeyValueLabel.hpp"
#include "communication/Signals.hpp"

namespace gui {

template<typename RegressionModel>
class RegressionModelAlgorithmDisplayWidget : public OverviewKeyValueDisplayWidget<RegressionModel>
{
public:
	RegressionModelAlgorithmDisplayWidget(boost::shared_ptr<RegressionModel> regressionModel, QWidget *parent = 0)
		:OverviewKeyValueDisplayWidget<RegressionModel>(regressionModel, parent), regressionModel(regressionModel)
	{
		setupLayout();
		updateValues();
		connectLocal();
	}

	virtual ~RegressionModelAlgorithmDisplayWidget(){;}

private:
	void setupLayout() {
		algorithmLabels = new KeyValueLabel(QObject::tr("Algorithm"), "", this);

		OverviewKeyValueDisplayWidget<RegressionModel>::addKeyValueLabel(algorithmLabels, 0);
		OverviewKeyValueDisplayWidget<RegressionModel>::setEditButton(0);
		OverviewKeyValueDisplayWidget<RegressionModel>::finishLayout();
	}

	void updateValues() {
		qDebug() << "RegressionModelAlgorithmDisplayWidget::" << __FUNCTION__ << " called";
		if(regressionModel->isStandardAlgorithm()) {
			algorithmLabels->updateValueLabel(QObject::tr("Ordinare least square"));
		}else if(regressionModel->isPraisWinstenAlgorithm()) {
			algorithmLabels->updateValueLabel(QObject::tr("Prais-Winsten with rho = ") + QString::number(regressionModel->getPraisWinstenRho()));
		}else if(regressionModel->isRidgeAlgorithm()) {
			algorithmLabels->updateValueLabel(QObject::tr("Ridge with k = ") + QString::number(regressionModel->getRidgeKValue()));
		}else if(regressionModel->isPCRAlgorithm()) {
			algorithmLabels->updateValueLabel(QObject::tr("Principal component regression with r = ") + QString::number(regressionModel->getPCRrValue()));
		}
	}

	void connectLocal() {
		using namespace boost;
		algorithmChangedConnection = COM_SUBSCRIBE_FOR(regressionModel, AlgorithmChanged,
				bind(&RegressionModelAlgorithmDisplayWidget<RegressionModel>::updateValues, this));

		calculatedConnection = COM_SUBSCRIBE_FOR(regressionModel, Calculated,
				bind(&RegressionModelAlgorithmDisplayWidget<RegressionModel>::updateValues, this));
	}

private:
	boost::shared_ptr<RegressionModel> regressionModel;
	KeyValueLabel *algorithmLabels;
	COM_CONNECTION algorithmChangedConnection;
	COM_CONNECTION calculatedConnection;
};

}  // namespace gui


#endif /* REGRESSIONMODELALGORITHMDISPLAYWIDGET_HPP_ */
