/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionModelAnalysisWidget.hpp
 *
 *  Created on: Mar 7, 2014
 *      Author: gerberma
 */

#ifndef REGRESSIONMODELANALYSISWIDGET_HPP_
#define REGRESSIONMODELANALYSISWIDGET_HPP_

#include <QWidget>
#include <QTabWidget>
#include <QGridLayout>
#include <stdexcept>
#include "AnalysisCategoryGenericWidget.hpp"

namespace gui {

template
<
	typename PhysicsCategoryWidget,
	typename StatisticsCategoryWidget
>
class RegressionModelAnalysisWidget : public QWidget {
public:
	typedef PhysicsCategoryWidget Physics_Category_Widget_t;
	typedef StatisticsCategoryWidget Statistics_Category_Widget_t;
public:
	RegressionModelAnalysisWidget()
		:physicsCategoryWidget(NULL), statisticsCategoryWidget(NULL), tabWidget(NULL)
	{
	}

	RegressionModelAnalysisWidget(PhysicsCategoryWidget *physicsCategoryWidget, StatisticsCategoryWidget *statisticsCategoryWidget)
		:physicsCategoryWidget(physicsCategoryWidget), statisticsCategoryWidget(statisticsCategoryWidget), tabWidget(NULL)
	{
	}

	void setPhysicsCategoryWidget(PhysicsCategoryWidget* physicsCategoryWidget) {
		this->physicsCategoryWidget = physicsCategoryWidget;
	}

	void setStatisticsCategoryWidget(StatisticsCategoryWidget* statisticsCategoryWidget) {
		this->statisticsCategoryWidget = statisticsCategoryWidget;
	}

	void init() {
		if(physicsCategoryWidget == NULL) {
			throw std::logic_error("missing regression model's physics category widget");
		}else if(statisticsCategoryWidget == NULL) {
			throw std::logic_error("missing regression model's statistics category widget");
		}

		setupLayout();
	}

private:
	void setupLayout() {
		tabWidget = new QTabWidget(this);

		tabWidget->addTab(physicsCategoryWidget, physicsCategoryWidget->getCategoryName());
		tabWidget->addTab(statisticsCategoryWidget, statisticsCategoryWidget->getCategoryName());

		QGridLayout *gridLayout = new QGridLayout;
		gridLayout->addWidget(tabWidget, 0, 0);

		setLayout(gridLayout);
	}

private:
	PhysicsCategoryWidget *physicsCategoryWidget;
	StatisticsCategoryWidget *statisticsCategoryWidget;
	QTabWidget *tabWidget;
};

}  // namespace gui


#endif /* REGRESSIONMODELANALYSISWIDGET_HPP_ */
