/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionModelBehaviorIndicatorDisplayWidget.hpp
 *
 *  Created on: Mar 6, 2014
 *      Author: gerberma
 */

#ifndef REGRESSIONMODELBEHAVIORINDICATORDISPLAYWIDGET_HPP_
#define REGRESSIONMODELBEHAVIORINDICATORDISPLAYWIDGET_HPP_

#include <boost/bind.hpp>
#include <boost/shared_ptr.hpp>
#include "OverviewKeyValueDisplayWidget.hpp"
#include "KeyValueLabel.hpp"
#include "communication/Signals.hpp"


namespace gui {

template<typename RegressionModel>
class RegressionModelBehaviorIndicatorDisplayWidget : public OverviewKeyValueDisplayWidget<RegressionModel> {
public:
	RegressionModelBehaviorIndicatorDisplayWidget(boost::shared_ptr<RegressionModel> regressionModel, QWidget *parent = 0);
	virtual ~RegressionModelBehaviorIndicatorDisplayWidget();

private:
	void setupLayout();
	void updateValues();
	void connectLocal();

private:
	boost::shared_ptr<RegressionModel> regressionModel;
	KeyValueLabel *behaviorIndicatorLabels;
	COM_CONNECTION behaviorIndicatorChangedConnection;
};

template<typename RegressionModel>
inline RegressionModelBehaviorIndicatorDisplayWidget<RegressionModel>::RegressionModelBehaviorIndicatorDisplayWidget(boost::shared_ptr<RegressionModel> regressionModel, QWidget *parent)
	:OverviewKeyValueDisplayWidget<RegressionModel>(regressionModel, parent), regressionModel(regressionModel)
{
	setupLayout();
	updateValues();
	connectLocal();
}

template<typename RegressionModel>
inline RegressionModelBehaviorIndicatorDisplayWidget<RegressionModel>::~RegressionModelBehaviorIndicatorDisplayWidget() {
}

template<typename RegressionModel>
inline void RegressionModelBehaviorIndicatorDisplayWidget<RegressionModel>::setupLayout() {
	behaviorIndicatorLabels = new KeyValueLabel(QObject::tr("Behavior indicator"), "", this);

	OverviewKeyValueDisplayWidget<RegressionModel>::addKeyValueLabel(behaviorIndicatorLabels, 0);
	OverviewKeyValueDisplayWidget<RegressionModel>::setEditButton(0);
	OverviewKeyValueDisplayWidget<RegressionModel>::finishLayout();
}

template<typename RegressionModel>
inline void RegressionModelBehaviorIndicatorDisplayWidget<RegressionModel>::updateValues() {
	behaviorIndicatorLabels->updateValueLabel(regressionModel->getBehaviorIndicatorColumn()->getName());
}

template<typename RegressionModel>
inline void RegressionModelBehaviorIndicatorDisplayWidget<RegressionModel>::connectLocal() {
	using namespace boost;

	behaviorIndicatorChangedConnection = COM_SUBSCRIBE_FOR(regressionModel, BehaviorIndicatorChanged, bind(&RegressionModelBehaviorIndicatorDisplayWidget<RegressionModel>::updateValues, this));
}

}  // namespace gui



#endif /* REGRESSIONMODELBEHAVIORINDICATORDISPLAYWIDGET_HPP_ */
