/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionModelBehaviorIndicatorWizardPage.hpp
 *
 *  Created on: Feb 18, 2014
 *      Author: gerberma
 */

#ifndef REGRESSIONMODELBEHAVIORINDICATORWIZARDPAGE_HPP_
#define REGRESSIONMODELBEHAVIORINDICATORWIZARDPAGE_HPP_

#include <QDebug>
#include <QWizardPage>
#include <QLabel>
#include <QVBoxLayout>
#include <QGroupBox>
#include <QSpacerItem>
#include <boost/foreach.hpp>
#include <boost/bind.hpp>
#include <boost/ref.hpp>
#include <boost/shared_ptr.hpp>
#include "ComboBox.hpp"
#include "communication/Signals.hpp"
#include "Delegates.hpp"


namespace gui {

template
<
	typename RegressionModel,
	typename NextIdStrategy
>
class RegressionModelBehaviorIndicatorWizardPage : public QWizardPage
{
public:
	RegressionModelBehaviorIndicatorWizardPage(QWidget *parent, boost::shared_ptr<RegressionModel> regressionModel, NextIdStrategy *nextIdStrategy, typename NextIdStrategy::type_t myId);
	virtual ~RegressionModelBehaviorIndicatorWizardPage();

	COM_SUBSCRIPTION_FUNCTION(BehaviorIndicatorChanged, communication::IndexChangedSignal<boost::shared_ptr<RegressionModel> >);

private:
	void setupLayout();
	void connectLocal();
	void updateValues();

	void indexChanged(int index);

private:
	QLabel *behaviorIndicatorLabel;
	ComboBox *behaviorIndicatorComboBox;
	boost::shared_ptr<RegressionModel> regressionModel;
	NextIdStrategy* nextIdStrategy;
	typename NextIdStrategy::type_t myId;
	communication::QtBoostIndexDelegate *behaviorIndicatorChangedDelegate;
};

template<typename RegressionModel, typename NextIdStrategy>
inline RegressionModelBehaviorIndicatorWizardPage<RegressionModel, NextIdStrategy>::RegressionModelBehaviorIndicatorWizardPage(
			QWidget* parent, boost::shared_ptr<RegressionModel> regressionModel,
			NextIdStrategy* nextIdStrategy, typename NextIdStrategy::type_t myId)
		:QWizardPage(parent), regressionModel(regressionModel), nextIdStrategy(nextIdStrategy), myId(myId)
{
	setupLayout();
	connectLocal();
	updateValues();
}

template<typename RegressionModel, typename NextIdStrategy>
inline RegressionModelBehaviorIndicatorWizardPage<RegressionModel, NextIdStrategy>::~RegressionModelBehaviorIndicatorWizardPage() {
}

template<typename RegressionModel, typename NextIdStrategy>
inline void RegressionModelBehaviorIndicatorWizardPage<RegressionModel, NextIdStrategy>::setupLayout() {
	QVBoxLayout *_mainLayout = new QVBoxLayout();


	QGroupBox *groupBox = new QGroupBox(tr("Behavior indicator"), this);

	QVBoxLayout *groupBoxLayout = new QVBoxLayout();

	behaviorIndicatorLabel = new QLabel(tr("Behavior indicator"), this);
	behaviorIndicatorLabel->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Ignored);
	behaviorIndicatorLabel->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Ignored);

	behaviorIndicatorComboBox = new ComboBox(this);

	behaviorIndicatorLabel->setBuddy(behaviorIndicatorComboBox);

	QWidget *fromToWidget = new QWidget(this);
		QGridLayout *fromToWidgetLayout = new QGridLayout(fromToWidget);
		fromToWidgetLayout->addWidget(behaviorIndicatorLabel, 0, 0);
		fromToWidgetLayout->addWidget(behaviorIndicatorComboBox, 0, 1, Qt::AlignLeft);
	fromToWidget->setLayout(fromToWidgetLayout);

	groupBoxLayout->addWidget(fromToWidget);

	groupBox->setLayout(groupBoxLayout);
	groupBox->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);

	_mainLayout->addWidget(groupBox);
	_mainLayout->addSpacerItem(new QSpacerItem(1,1,QSizePolicy::Ignored, QSizePolicy::Expanding));

	setLayout(_mainLayout);
}

template<typename RegressionModel, typename NextIdStrategy>
inline void RegressionModelBehaviorIndicatorWizardPage<RegressionModel, NextIdStrategy>::connectLocal() {
	using namespace communication;
	using namespace boost;

	behaviorIndicatorChangedDelegate = new QtBoostIndexDelegate(bind(&RegressionModelBehaviorIndicatorWizardPage<RegressionModel, NextIdStrategy>::indexChanged, this, _1), this);
	connect(behaviorIndicatorComboBox, SIGNAL(currentIndexChanged(int)), behaviorIndicatorChangedDelegate, SLOT(execute(int)));

}

template<typename RegressionModel, typename NextIdStrategy>
inline void RegressionModelBehaviorIndicatorWizardPage<RegressionModel, NextIdStrategy>::updateValues() {
	behaviorIndicatorComboBox->clear();

	BOOST_FOREACH(boost::shared_ptr<typename RegressionModel::DataModel_t::Behavior_Indicator_Column_t> behaviorIndicatorColumn, regressionModel->getDataModel()->getBehaviorIndicatorColumns()) {
		behaviorIndicatorComboBox->addItem(behaviorIndicatorColumn->getName(), behaviorIndicatorColumn->getIndex());
	}

	if(regressionModel->getBehaviorIndicatorColumn() != NULL) {
		behaviorIndicatorComboBox->setCurrentByValue(regressionModel->getBehaviorIndicatorColumn()->getIndex());
	}
}

template<typename RegressionModel, typename NextIdStrategy>
inline void RegressionModelBehaviorIndicatorWizardPage<RegressionModel, NextIdStrategy>::indexChanged(int index) {
	int behaviorIndicatorColumnNr = behaviorIndicatorComboBox->itemData(index, Qt::UserRole).toString().toInt();

	qDebug() << __FUNCTION__ << " : " << behaviorIndicatorColumnNr;
	BehaviorIndicatorChangedSignal(regressionModel, behaviorIndicatorColumnNr);
}

} /* namespace gui */


#endif /* REGRESSIONMODELBEHAVIORINDICATORWIZARDPAGE_HPP_ */
