/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionModelCommonParameterDisplayWidget.hpp
 *
 *  Created on: Mar 5, 2014
 *      Author: gerberma
 */

#ifndef REGRESSIONMODELCOMMONPARAMETERDISPLAYWIDGET_HPP_
#define REGRESSIONMODELCOMMONPARAMETERDISPLAYWIDGET_HPP_

#include <boost/bind.hpp>
#include <boost/shared_ptr.hpp>
#include "OverviewKeyValueDisplayWidget.hpp"
#include "KeyValueLabel.hpp"
#include "communication/Signals.hpp"

namespace gui {

template<typename RegressionModel>
class RegressionModelCommonParameterDisplayWidget : public OverviewKeyValueDisplayWidget<RegressionModel>
{
public:
	RegressionModelCommonParameterDisplayWidget(boost::shared_ptr<RegressionModel> regressionModel, QWidget *parent = 0);
	virtual ~RegressionModelCommonParameterDisplayWidget();

private:
	void setupLayout();
	void updateValues();
	void connectLocal();

private:
	boost::shared_ptr<RegressionModel> regressionModel;
	KeyValueLabel *nameLabels;
	KeyValueLabel *descriptionLabels;
	COM_CONNECTION commonParametersChangedConnection;
};

template<typename RegressionModel>
inline RegressionModelCommonParameterDisplayWidget<RegressionModel>::RegressionModelCommonParameterDisplayWidget(boost::shared_ptr<RegressionModel> regressionModel, QWidget* parent)
	:OverviewKeyValueDisplayWidget<RegressionModel>(regressionModel, parent), regressionModel(regressionModel)
{
	setupLayout();
	updateValues();
	connectLocal();
}

template<typename RegressionModel>
inline RegressionModelCommonParameterDisplayWidget<RegressionModel>::~RegressionModelCommonParameterDisplayWidget() {
}

template<typename RegressionModel>
inline void RegressionModelCommonParameterDisplayWidget<RegressionModel>::setupLayout() {
	nameLabels = new KeyValueLabel(QObject::tr("Name"), "", this);
	descriptionLabels = new KeyValueLabel(QObject::tr("Description"), "", this);

	nameLabels->getValueLabel()->setWordWrap(true);
	descriptionLabels->getValueLabel()->setWordWrap(true);

	OverviewKeyValueDisplayWidget<RegressionModel>::addKeyValueLabel(nameLabels, 0);
	OverviewKeyValueDisplayWidget<RegressionModel>::addKeyValueLabel(descriptionLabels, 1);
	OverviewKeyValueDisplayWidget<RegressionModel>::setEditButton(1);
	OverviewKeyValueDisplayWidget<RegressionModel>::finishLayout();
}

template<typename RegressionModel>
inline void RegressionModelCommonParameterDisplayWidget<RegressionModel>::updateValues() {
	nameLabels->updateValueLabel(regressionModel->getName());
	descriptionLabels->updateValueLabel(regressionModel->getDescription());
}

template<typename RegressionModel>
inline void RegressionModelCommonParameterDisplayWidget<RegressionModel>::connectLocal() {
	using namespace boost;

	commonParametersChangedConnection = COM_SUBSCRIBE_FOR(regressionModel, CommonParametersChanged, bind(
				&RegressionModelCommonParameterDisplayWidget<RegressionModel>::updateValues, this
			));
}

} /* namespace gui */

#endif /* REGRESSIONMODELCOMMONPARAMETERDISPLAYWIDGET_HPP_ */
