/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionModelMethodSelectionWidget.hpp
 *
 *  Created on: May 28, 2014
 *      Author: gerberma
 */

#ifndef REGRESSIONMODELMETHODDESCRIPTIONWIDGET_HPP_
#define REGRESSIONMODELMETHODDESCRIPTIONWIDGET_HPP_

#include <QDebug>
#include <QWidget>
#include <QCheckBox>
#include <QLabel>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QGroupBox>
#include <boost/bind.hpp>
#include <boost/ref.hpp>
#include <boost/shared_ptr.hpp>
#include "MoveToolbarButton.hpp"
#include "communication/Signals.hpp"
#include "Delegates.hpp"

namespace gui {

template<typename MethodDescription>
class RegressionModelMethodDescriptionWidget : public QWidget
{
public:
	COM_SUBSCRIPTION_FUNCTION(AddInstance, communication::Signal<void(boost::shared_ptr<MethodDescription>)>);
public:
	RegressionModelMethodDescriptionWidget(boost::shared_ptr<MethodDescription> methodDescription, QWidget *parent=0)
		:QWidget(parent), methodDescription(methodDescription)
	{
		setupLayout();
		updateValues();
		connectLocal();
	}

private:
	void setupLayout() {
		groupBox = new QGroupBox(this);

			QVBoxLayout *groupBoxLayout = new QVBoxLayout(this);
			nameLabel = new QLabel(this);
			nameLabel->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Ignored);
			authorLabel = new QLabel(this);
			descriptionLabel = new QLabel(this);
			addInstanceButton = new MoveToolbarButton(tr("+"), this);

			QHBoxLayout *nameAndAddInstanceLayout = new QHBoxLayout();
			QWidget *nameAndInstanceWidget = new QWidget(this);
			nameAndAddInstanceLayout->addWidget(nameLabel);
			//nameAndAddInstanceLayout->addWi(new QSpacerItem(1,1,QSizePolicy::Expanding, QSizePolicy::Ignored));
			nameAndAddInstanceLayout->addWidget(addInstanceButton, 1, Qt::AlignRight);
			nameAndInstanceWidget->setLayout(nameAndAddInstanceLayout);

			groupBoxLayout->addWidget(nameAndInstanceWidget, 1, Qt::AlignLeft);
			groupBoxLayout->addWidget(authorLabel, 1, Qt::AlignLeft);
			groupBoxLayout->addWidget(descriptionLabel, 1, Qt::AlignLeft);


		groupBox->setLayout(groupBoxLayout);

		QVBoxLayout *mainLayout = new QVBoxLayout(this);
		mainLayout->addWidget(groupBox);

		setLayout(mainLayout);
	}

	void updateValues() {
		nameLabel->setText(methodDescription->getName());
		authorLabel->setText(methodDescription->getAuthor());
		descriptionLabel->setText(methodDescription->getDescription());
	}

	void connectLocal() {
		using namespace communication;
		using namespace boost;

		addInstanceDelegate = new QtBoostDelegate(bind(ref(AddInstanceSignal), methodDescription), this);
		connect(addInstanceButton, SIGNAL(clicked()), addInstanceDelegate, SLOT(execute()));
	}

private:
	boost::shared_ptr<MethodDescription> methodDescription;
	QLabel *nameLabel;
	QLabel *authorLabel;
	QLabel *descriptionLabel;
	QGroupBox *groupBox;
	MoveToolbarButton *addInstanceButton;
	communication::QtBoostDelegate *addInstanceDelegate;
};

}  // namespace gui


#endif /* REGRESSIONMODELMETHODDESCRIPTIONWIDGET_HPP_ */
