/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionModelMethodsSelectionEditWidget.hpp
 *
 *  Created on: May 28, 2014
 *      Author: gerberma
 */

#ifndef REGRESSIONMODELMETHODSSELECTIONEDITWIDGET_HPP_
#define REGRESSIONMODELMETHODSSELECTIONEDITWIDGET_HPP_

#include <QDebug>
#include <QWidget>
#include <QScrollArea>
#include <QVBoxLayout>
#include <boost/foreach.hpp>
#include <boost/bind.hpp>
#include <boost/ref.hpp>
#include <boost/shared_ptr.hpp>
#include <vector>
#include "RegressionModelMethodDescriptionWidget.hpp"
#include "communication/Signals.hpp"

namespace gui {

template<typename RegressionModel>
class RegressionModelMethodsSelectionEditWidget : public QWidget {
public:
	COM_SUBSCRIPTION_FUNCTION(AddRegressionFormulaMethodInstance, communication::Signal<void(boost::shared_ptr<RegressionModel> regressionModel, boost::shared_ptr<typename RegressionModel::Regression_Formula_t::RegressionModelTemplate_t>)>);

public:
	RegressionModelMethodsSelectionEditWidget(boost::shared_ptr<RegressionModel> regressionModel, QWidget *parent=0)
		:QWidget(parent), regressionModel(regressionModel)
	{
		setupLayout();
		updateValues();
	}

	virtual ~RegressionModelMethodsSelectionEditWidget() {
		BOOST_FOREACH(boost::signals2::connection con, methodDescriptionConnections) {
			con.disconnect();
		}
	}

private:
	void setupLayout() {
		QVBoxLayout *mainLayout = new QVBoxLayout(this);

		methodDescriptionsContainerWidgetLayout = new QVBoxLayout();
		methodDescriptionsContainerWidgetLayout->setSizeConstraint(QLayout::SetMinimumSize);
		methodDescriptionsContainerWidgetLayout->addSpacerItem(new QSpacerItem(1,1,QSizePolicy::Ignored, QSizePolicy::Expanding));

		methodDescriptionsContainerWidget = new QWidget(this);
		methodDescriptionsContainerWidget->setLayout(methodDescriptionsContainerWidgetLayout);

		scrollArea = new QScrollArea(this);
		scrollArea->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
		scrollArea->setWidget(methodDescriptionsContainerWidget);
		scrollArea->setWidgetResizable(true);

		mainLayout->addWidget(scrollArea);

		setLayout(mainLayout);
	}

	void updateValues() {
		using namespace boost;
		BOOST_FOREACH(boost::shared_ptr<typename RegressionModel::Regression_Formula_t::RegressionModelTemplate_t> methodDescription, regressionModel->getRegressionFormula()->getMethodDescriptions()) {
			RegressionModelMethodDescriptionWidget<typename RegressionModel::Regression_Formula_t::RegressionModelTemplate_t> *methodDescriptionWidget = new RegressionModelMethodDescriptionWidget<typename RegressionModel::Regression_Formula_t::RegressionModelTemplate_t>(methodDescription, this);
			methodDescriptionWidget->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Maximum);
			methodDescriptionsContainerWidgetLayout->addWidget(methodDescriptionWidget, 1, Qt::AlignTop);
			methodDescriptionConnections.push_back(
						COM_SUBSCRIBE_FOR(methodDescriptionWidget, AddInstance, bind(ref(AddRegressionFormulaMethodInstanceSignal), regressionModel, _1)) //todo: instead of providing _1 which is the methodDescriptionWidget anyway, we can provide it from here
					);
		}
	}

private:
	boost::shared_ptr<RegressionModel> regressionModel;
	QScrollArea *scrollArea;
	QVBoxLayout *methodDescriptionsContainerWidgetLayout;
	QWidget *methodDescriptionsContainerWidget;
	std::vector<boost::signals2::connection> methodDescriptionConnections;
};

}  // namespace gui


#endif /* REGRESSIONMODELMETHODSSELECTIONEDITWIDGET_HPP_ */
