/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionModelRegressionFormulaDisplayWidget.hpp
 *
 *  Created on: Mar 6, 2014
 *      Author: gerberma
 */

#ifndef REGRESSIONMODELREGRESSIONFORMULADISPLAYWIDGET_HPP_
#define REGRESSIONMODELREGRESSIONFORMULADISPLAYWIDGET_HPP_

#include <boost/bind.hpp>
#include <QDebug>
#include <boost/shared_ptr.hpp>
#include "OverviewKeyValueDisplayWidget.hpp"
#include "KeyValueLabel.hpp"
#include "communication/Signals.hpp"

namespace gui {

template<typename RegressionModel>
class RegressionModelRegressionFormulaDisplayWidget : public OverviewKeyValueDisplayWidget<RegressionModel>
{
public:
	RegressionModelRegressionFormulaDisplayWidget(boost::shared_ptr<RegressionModel> regressionModel, QWidget *parent = 0);
	virtual ~RegressionModelRegressionFormulaDisplayWidget();

private:
	void setupLayout();
	void updateValues();
	void connectLocal();

private:
	boost::shared_ptr<RegressionModel> regressionModel;
	KeyValueLabel *regressionFormulaLabels;
	COM_CONNECTION regressionFormulaChangedConnection;
};

template<typename RegressionModel>
inline RegressionModelRegressionFormulaDisplayWidget<RegressionModel>::RegressionModelRegressionFormulaDisplayWidget(boost::shared_ptr<RegressionModel> regressionModel, QWidget* parent)
	:OverviewKeyValueDisplayWidget<RegressionModel>(regressionModel, parent), regressionModel(regressionModel)
{
	setupLayout();
	updateValues();
	connectLocal();
}

template<typename RegressionModel>
inline RegressionModelRegressionFormulaDisplayWidget<RegressionModel>::~RegressionModelRegressionFormulaDisplayWidget() {
}

template<typename RegressionModel>
inline void RegressionModelRegressionFormulaDisplayWidget<RegressionModel>::setupLayout() {
	regressionFormulaLabels = new KeyValueLabel(QObject::tr("Regression formula"), "", this);
	regressionFormulaLabels->getValueLabel()->setWordWrap(true);

	OverviewKeyValueDisplayWidget<RegressionModel>::addKeyValueLabel(regressionFormulaLabels, 0);
	OverviewKeyValueDisplayWidget<RegressionModel>::setEditButton(0);
	OverviewKeyValueDisplayWidget<RegressionModel>::finishLayout();
}

template<typename RegressionModel>
inline void RegressionModelRegressionFormulaDisplayWidget<RegressionModel>::updateValues() {
	regressionFormulaLabels->updateValueLabel(regressionModel->getRegressionFormula()->toString());
}

template<typename RegressionModel>
inline void RegressionModelRegressionFormulaDisplayWidget<RegressionModel>::connectLocal() {
	using namespace boost;
	regressionFormulaChangedConnection = COM_SUBSCRIBE_FOR(regressionModel->getRegressionFormula(), Changed,
			bind(&RegressionModelRegressionFormulaDisplayWidget<RegressionModel>::updateValues, this));
}

} /* namespace gui */


#endif /* REGRESSIONMODELREGRESSIONFORMULADISPLAYWIDGET_HPP_ */
