/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionModelRegressionFormulaEditWidgetWithRegressorList.hpp
 *
 *  Created on: Feb 20, 2014
 *      Author: gerberma
 */

#ifndef REGRESSIONMODELREGRESSIONFORMULAEDITWIDGET_HPP_
#define REGRESSIONMODELREGRESSIONFORMULAEDITWIDGET_HPP_

#include <QDebug>
#include <QWidget>
#include <QScrollArea>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QSpacerItem>
#include <boost/foreach.hpp>
#include <boost/bind.hpp>
#include <boost/ref.hpp>
#include <boost/function.hpp>
#include <boost/shared_ptr.hpp>
#include <map>
#include <vector>
#include "communication/Signals.hpp"
#include "Delegates.hpp"
#include "ComboBox.hpp"
#include "MoveToolbarButton.hpp"
#include "RegressorEditWidget.hpp"

namespace gui {

template<typename RegressionModel, typename SubscriptionSignature>
class RegressionModelRegressionFormulaEditWidgetWithRegressorList : public QWidget {
public:
	typedef communication::Signal<void(boost::shared_ptr<RegressionModel>, int)> RemoveRegressorSignal_t;
public:
	COM_SUBSCRIPTION_FUNCTION(CreateRegressor, communication::Signal<void(boost::shared_ptr<RegressionModel>)>);
	COM_SUBSCRIPTION_FUNCTION(RegressorFormulaEdited, communication::Signal<void(boost::shared_ptr<RegressionModel>, int, QString)>); //int: regressor index, QString formula
	COM_SUBSCRIPTION_FUNCTION(RemoveRegressor, RemoveRegressorSignal_t);

private:
	typedef typename RegressionModel::Regression_Formula_t::Regressor_t Regressor_t;

public:
	RegressionModelRegressionFormulaEditWidgetWithRegressorList(boost::shared_ptr<RegressionModel> regressionModel, boost::function<std::vector<boost::shared_ptr<Regressor_t> >()> regressorFetchFunction,  boost::function<SubscriptionSignature> subscriptionFunction, QWidget *parent=0)
		:QWidget(parent), regressionModel(regressionModel), regressorFetchFunction(regressorFetchFunction), subscriptionFunction(subscriptionFunction)
	{
		setupLayout();
		updateRegressorList();
		connectLocal();
		updateValues();
	}

	virtual ~RegressionModelRegressionFormulaEditWidgetWithRegressorList() {
		//clean up all connections registered within the connectionMapping

		typedef std::pair<RegressorEditWidget<Regressor_t> *, std::vector<boost::signals2::connection> > pair_t;
		BOOST_FOREACH(pair_t pair, connectionMapping) {
			BOOST_FOREACH(boost::signals2::connection con, pair.second) {
				con.disconnect();
			}
		}
	}

private:
	void setupLayout() {
		QVBoxLayout *mainLayout = new QVBoxLayout(this);

		QWidget *chooseRegressorWidget = new QWidget(this);
			QHBoxLayout *chooseRegressorWidgetLayout = new QHBoxLayout(chooseRegressorWidget);
			createRegressorButton = new MoveToolbarButton("+", chooseRegressorWidget);
			chooseRegressorWidgetLayout->addWidget(createRegressorButton);
			chooseRegressorWidget->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Minimum);

			regressorContainerWidgetLayout = new QVBoxLayout; //this is the layout new regressor edit widgets can be added
			regressorContainerWidgetLayout->setSizeConstraint(QLayout::SetMinimumSize);
			regressorContainerWidgetLayout->addSpacerItem(new QSpacerItem(1,1,QSizePolicy::Ignored, QSizePolicy::Expanding));
			regressorContainerWidget = new QWidget(this);
			regressorContainerWidget->setLayout(regressorContainerWidgetLayout);
			scrollArea = new QScrollArea(this);
			scrollArea->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
			scrollArea->setWidget(regressorContainerWidget);
			scrollArea->setWidgetResizable(true);

		mainLayout->addWidget(chooseRegressorWidget);
		mainLayout->addWidget(scrollArea);

		setLayout(mainLayout);
	}

	void updateRegressorList() {
		using namespace boost;

		BOOST_FOREACH(boost::shared_ptr<Regressor_t> regressor, regressorFetchFunction()) {
			RegressorEditWidget<Regressor_t> *regressorEditWidget = new RegressorEditWidget<Regressor_t>(regressor, this);
			registerRegressorEditWidget(regressorEditWidget);
			regressorContainerWidgetLayout->addWidget(regressorEditWidget, 1, Qt::AlignTop|Qt::AlignLeft);
		}
	}

	void registerRegressorEditWidget(RegressorEditWidget<Regressor_t> *regressorEditWidget) {
		using namespace boost;

		connectionMapping[regressorEditWidget].push_back(COM_SUBSCRIBE_FOR(regressorEditWidget, SetRegressionFormula, (
			bind(ref(RegressorFormulaEditedSignal), regressionModel, _1, _2)
		)));

		connectionMapping[regressorEditWidget].push_back(COM_SUBSCRIBE_FOR(regressorEditWidget, RemoveRegressor, (
			bind(ref(RemoveRegressorSignal), regressionModel, _1)
		)));
	}

	void releaseRegressorEditWidget(RegressorEditWidget<Regressor_t> *regressorEditWidget) {
		//typedef std::pair<RegressorEditWidget<Regressor_t> *, std::vector<boost::signals2::connection> > pair_t;
		std::vector<boost::signals2::connection> connections = connectionMapping[regressorEditWidget];
		BOOST_FOREACH(boost::signals2::connection con, connections) {
			con.disconnect();
		}
		connectionMapping.erase(regressorEditWidget);
		delete regressorEditWidget;
	}

	void updateValues() {
		//update influenceFactorsComboBox
		/*
		influenceFactorsComboBox->clear();
		BOOST_FOREACH(typename RegressionModel::DataModel_t::Water_Level_Column_t *waterLevelColumn, regressionModel->getDataModel()->getWaterLevelColumns()) {
			influenceFactorsComboBox->addItem(waterLevelColumn->getName(), waterLevelColumn->getIndex());
		}

		QLayoutItem *item;
		while((item = regressorContainerWidgetLayout->takeAt(0))) {
			delete item;
		}

		typedef typename RegressionModel::Regression_Formula_t::Regressor_t Regressor_t;
		BOOST_FOREACH(Regressor_t *regressor, regressionModel->getRegressionFormula()->getWaterLevelRegressors()) {
			RegressorEditWidget<Regressor_t> *regressorEditWidget = new RegressorEditWidget<Regressor_t>(regressor, this);
			regressorContainerWidgetLayout->addWidget(regressorEditWidget, 1, Qt::AlignTop|Qt::AlignLeft);
		}
		*/
	}

	void regressorAddedWith(int index) {
		qDebug() << __FUNCTION__ << " index: " << index;

		typedef typename RegressionModel::Regression_Formula_t::Regressor_t Regressor_t;
		boost::shared_ptr<Regressor_t> regressor = regressionModel->getRegressionFormula()->findRegressorByIndex(index);
		if(regressor != NULL) {
			RegressorEditWidget<Regressor_t> *regressorEditWidget = new RegressorEditWidget<Regressor_t>(regressor, this);
			registerRegressorEditWidget(regressorEditWidget);
			regressorContainerWidgetLayout->addWidget(regressorEditWidget, 1, Qt::AlignTop|Qt::AlignLeft);
		}
	}

	void regressorRemovedWith(int index) {
		qDebug() << __FUNCTION__ << " index: " << index;

		//find RegressorEditWidget which is responsible for a regressor of the given index
		typedef std::pair<RegressorEditWidget<Regressor_t> *, std::vector<boost::signals2::connection> > pair_t;
		BOOST_FOREACH(pair_t pair, connectionMapping) {
			if(pair.first->getRegressor()->getIndex() == index) {
				releaseRegressorEditWidget((RegressorEditWidget<Regressor_t> *)pair.first);
			}
		}

		return;
	}

	void connectLocal() {
		using namespace communication;
		using namespace boost;

		createWaterLevelRegressorDelegate = new QtBoostDelegate(bind(ref(CreateRegressorSignal), regressionModel), this);
		connect(createRegressorButton, SIGNAL(clicked()), createWaterLevelRegressorDelegate, SLOT(execute()));

		regressorAddedButtonConnection = subscriptionFunction((bind(
				&RegressionModelRegressionFormulaEditWidgetWithRegressorList<RegressionModel, SubscriptionSignature>::regressorAddedWith, this, _1
		)));

		regressorRemovedConnection = COM_SUBSCRIBE_FOR(regressionModel->getRegressionFormula(), RegressorRemoved, (bind(
					&RegressionModelRegressionFormulaEditWidgetWithRegressorList<RegressionModel, SubscriptionSignature>::regressorRemovedWith, this, _1
				)));
	}

private:
	boost::shared_ptr<RegressionModel> regressionModel;
	MoveToolbarButton *createRegressorButton;
	QVBoxLayout *regressorContainerWidgetLayout;
	QWidget *regressorContainerWidget;
	QScrollArea *scrollArea;
	communication::QtBoostDelegate *createWaterLevelRegressorDelegate;
	std::map<RegressorEditWidget<Regressor_t> *, std::vector<boost::signals2::connection> > connectionMapping;
	boost::function<SubscriptionSignature> subscriptionFunction;
	boost::function<std::vector<Regressor_t *>()> regressorFetchFunction;
	COM_CONNECTION regressorAddedButtonConnection;
	COM_CONNECTION regressorRemovedConnection;
};

} /* namespace gui */

#endif /* REGRESSIONMODELREGRESSIONFORMULAEDITWIDGET_HPP_ */
