/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionModelRegressionFormulaWizardPage.hpp
 *
 *  Created on: Feb 20, 2014
 *      Author: gerberma
 */

#ifndef REGRESSIONMODELREGRESSIONFORMULAWIZARDPAGE_HPP_
#define REGRESSIONMODELREGRESSIONFORMULAWIZARDPAGE_HPP_


#include <QWidget>
#include <QWizardPage>
#include <QGroupBox>
#include <boost/bind.hpp>
#include <boost/ref.hpp>
#include <boost/shared_ptr.hpp>
#include "communication/Signals.hpp"
#include "GroupBoxLayoutBuilder.hpp"
#include "KeyValueLabel.hpp"
#include "RegressionModelMethodsSelectionWidget.hpp"
#include "RegressionModelsDefinitionEditWidget.hpp"
#include "RegressorFormulaBuildersEditWidget.hpp"
#include "RegressionModelAlgorithmSelectionEditWidget.hpp"

namespace gui {

template
<
	typename RegressionModel,
	typename NextIdStrategy
>
class RegressionModelRegressionFormulaWizardPage : public QWizardPage
{
public:

	typedef RegressionModelsDefinitionEditWidget<typename RegressionModel::Regression_Formula_t> _regression_model_definitions_widget_t;
	typedef RegressorFormulaBuildersEditWidget<typename RegressionModel::Regression_Formula_t> _regressor_formula_builders_widget_t;
	typedef RegressionModelAlgorithmSelectionEditWidget<RegressionModel> _regression_model_algorithm_selection_widget_t;
public:
	RegressionModelRegressionFormulaWizardPage(QWidget *parent, boost::shared_ptr<RegressionModel> regressionModel, NextIdStrategy *nextIdStrategy, typename NextIdStrategy::type_t myId);
	virtual ~RegressionModelRegressionFormulaWizardPage();

	COM_SUBSCRIPTION_DELEGATE_FUNCTION(RegressionModelMethodsSelectionEditWidget<RegressionModel>, methodsSelectionEditWidget, AddRegressionFormulaMethodInstance);
	COM_SUBSCRIPTION_DELEGATE_FUNCTION(RegressionModelAlgorithmSelectionEditWidget<RegressionModel>, regressionModelAlgorithmSelectionWidget, AlgorithmChanged);
	COM_SUBSCRIPTION_DELEGATE_FUNCTION(RegressionModelAlgorithmSelectionEditWidget<RegressionModel>, regressionModelAlgorithmSelectionWidget, PraisWinstenIterationsChanged);
	COM_SUBSCRIPTION_DELEGATE_FUNCTION(RegressionModelAlgorithmSelectionEditWidget<RegressionModel>, regressionModelAlgorithmSelectionWidget, RidgeKValueChanged);
	COM_SUBSCRIPTION_DELEGATE_FUNCTION(RegressionModelAlgorithmSelectionEditWidget<RegressionModel>, regressionModelAlgorithmSelectionWidget, PCRrValueChanged);

	_regression_model_definitions_widget_t *getRegressionModelDefinitionWidget() {
		return regressionModelDefinitionsWidget;
	}

	_regressor_formula_builders_widget_t *getRegressorFormulaBuildersWidget() {
		return regressorFormulaBuildersWidget;
	}

	_regression_model_algorithm_selection_widget_t *getRegressionModelAlgorithmSelectionWidget() {
		return regressionModelAlgorithmSelectionWidget;
	}

private:
	void setupLayout();
	void connectLocal();
	void updateValues();

private:
	boost::shared_ptr<RegressionModel> regressionModel;
	NextIdStrategy* nextIdStrategy;
	typename NextIdStrategy::type_t myId;
	RegressionModelMethodsSelectionEditWidget<RegressionModel> *methodsSelectionEditWidget;
	_regression_model_definitions_widget_t *regressionModelDefinitionsWidget;
	_regressor_formula_builders_widget_t *regressorFormulaBuildersWidget;
	_regression_model_algorithm_selection_widget_t *regressionModelAlgorithmSelectionWidget;
	KeyValueLabel *formulaKeyValueLabel;
	QWidget *formulaWidget;
	QHBoxLayout *formulaLayout;
	COM_CONNECTION formulaChangedConnection;
};

template
<
	typename RegressionModel,
	typename NextIdStrategy
>
inline RegressionModelRegressionFormulaWizardPage<RegressionModel,NextIdStrategy>::RegressionModelRegressionFormulaWizardPage(
			QWidget* parent, boost::shared_ptr<RegressionModel> regressionModel,
			NextIdStrategy* nextIdStrategy, typename NextIdStrategy::type_t myId)
		:QWizardPage(parent), regressionModel(regressionModel), nextIdStrategy(nextIdStrategy), myId(myId)
{
	setupLayout();
	connectLocal();
	updateValues();
}

template
<
	typename RegressionModel,
	typename NextIdStrategy
>
inline RegressionModelRegressionFormulaWizardPage<RegressionModel, NextIdStrategy>::~RegressionModelRegressionFormulaWizardPage() {
}

template
<
	typename RegressionModel,
	typename NextIdStrategy
>
inline void RegressionModelRegressionFormulaWizardPage<RegressionModel, NextIdStrategy>::setupLayout() {
	QTabWidget *tabWidget = new QTabWidget(this);

	typedef typename RegressionModel::Regression_Formula_t RegressionFormula_t;

	methodsSelectionEditWidget = new RegressionModelMethodsSelectionEditWidget<RegressionModel>(regressionModel, this);
	regressionModelDefinitionsWidget = new _regression_model_definitions_widget_t(regressionModel->getRegressionFormula(), this);
	regressorFormulaBuildersWidget = new _regressor_formula_builders_widget_t(regressionModel->getRegressionFormula(), this);
	regressionModelAlgorithmSelectionWidget = new _regression_model_algorithm_selection_widget_t(regressionModel, this);

	tabWidget->addTab(methodsSelectionEditWidget, tr("Model Selection"));
	tabWidget->addTab(regressionModelDefinitionsWidget, tr("Model Edit"));
	tabWidget->addTab(regressorFormulaBuildersWidget, tr("Regressor Formula Definitions"));
	tabWidget->addTab(regressionModelAlgorithmSelectionWidget, tr("Algorithm Selection"));

	formulaWidget = new QWidget(this);
	formulaKeyValueLabel = new KeyValueLabel(tr("Regression formula"), "", this);
	formulaKeyValueLabel->getValueLabel()->setWordWrap(true);
	formulaLayout = new QHBoxLayout(this);
	formulaWidget->setLayout(formulaLayout);
	formulaLayout->addWidget(formulaKeyValueLabel->getKeyLabel());
	formulaLayout->addWidget(formulaKeyValueLabel->getValueLabel());

	QVBoxLayout *mainLayout = new QVBoxLayout(this);
	mainLayout->addWidget(tabWidget);
	mainLayout->addWidget(formulaWidget);
	setLayout(mainLayout);
}

template
<
	typename RegressionModel,
	typename NextIdStrategy
>
inline void RegressionModelRegressionFormulaWizardPage<RegressionModel, NextIdStrategy>::connectLocal() {
	using namespace boost;

	formulaChangedConnection = COM_SUBSCRIBE_FOR(regressionModel->getRegressionFormula(), Changed,
				bind(&RegressionModelRegressionFormulaWizardPage<RegressionModel, NextIdStrategy>::updateValues, this)
			);
	//formulaKeyValueLabel->getValueLabel()->setText(regressionModel->getRegressionFormula()->toString());
}

template
<
	typename RegressionModel,
	typename NextIdStrategy
>
inline void RegressionModelRegressionFormulaWizardPage<RegressionModel, NextIdStrategy>::updateValues() {
	formulaKeyValueLabel->updateValueLabel(regressionModel->getRegressionFormula()->toString());
}


} /* namespace gui */

#endif /* REGRESSIONMODELREGRESSIONFORMULAWIZARDPAGE_HPP_ */
