/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionModelTimeRangeDisplayWidget.hpp
 *
 *  Created on: Mar 5, 2014
 *      Author: gerberma
 */

#ifndef REGRESSIONMODELTIMERANGEDISPLAYWIDGET_HPP_
#define REGRESSIONMODELTIMERANGEDISPLAYWIDGET_HPP_

#include <boost/bind.hpp>
#include <boost/shared_ptr.hpp>
#include "OverviewKeyValueDisplayWidget.hpp"
#include "KeyValueLabel.hpp"
#include "communication/Signals.hpp"

namespace gui {

template<typename RegressionModel>
class RegressionModelTimeRangeDisplayWidget : public OverviewKeyValueDisplayWidget<RegressionModel> {
public:
	RegressionModelTimeRangeDisplayWidget(boost::shared_ptr<RegressionModel> regressionModel, QWidget *parent = 0)
		:OverviewKeyValueDisplayWidget<RegressionModel>(regressionModel, parent), regressionModel(regressionModel)
	{
		setupLayout();
		connectLocal();
		updateValues();
	}


private:
	void setupLayout() {
		fromDateLabels = new KeyValueLabel(QObject::tr("From date"), "", this);
		toDateLabels = new KeyValueLabel(QObject::tr("To date"), "", this);

		OverviewKeyValueDisplayWidget<RegressionModel>::addKeyValueLabel(fromDateLabels, 0);
		OverviewKeyValueDisplayWidget<RegressionModel>::addKeyValueLabel(toDateLabels, 1);
		OverviewKeyValueDisplayWidget<RegressionModel>::setEditButton(1);
		OverviewKeyValueDisplayWidget<RegressionModel>::finishLayout();
	}

	void connectLocal() {
		using namespace boost;
		regressionModelTimeValuesChanged = COM_SUBSCRIBE_FOR(regressionModel, TimeValuesChanged, bind(&RegressionModelTimeRangeDisplayWidget<RegressionModel>::updateValues, this));
	}

	void updateValues() {
		fromDateLabels->updateValueLabel(regressionModel->getFromDate().getDate());
		toDateLabels->updateValueLabel(regressionModel->getToDate().getDate());
	}

private:
	boost::shared_ptr<RegressionModel> regressionModel;
	KeyValueLabel *fromDateLabels;
	KeyValueLabel *toDateLabels;
	COM_CONNECTION regressionModelTimeValuesChanged;
};


}  // namespace gui


#endif /* REGRESSIONMODELTIMERANGEDISPLAYWIDGET_HPP_ */
