/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionModelWaterLevelRegressionFormulaEditWidget.hpp
 *
 *  Created on: Feb 20, 2014
 *      Author: gerberma
 */

#ifndef REGRESSIONMODELWATERLEVELREGRESSIONFORMULAEDITWIDGET_HPP_
#define REGRESSIONMODELWATERLEVELREGRESSIONFORMULAEDITWIDGET_HPP_

#include <QWidget>
#include <QLabel>
#include <QLineEdit>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QSpacerItem>
#include <boost/foreach.hpp>
#include <boost/bind.hpp>
#include <boost/ref.hpp>
#include <boost/shared_ptr.hpp>
#include "Delegates.hpp"
#include "ComboBox.hpp"
#include "MoveToolbarButton.hpp"
#include "RegressorEditWidget.hpp"

namespace gui {

template<typename RegressionModel>
class RegressionModelWaterLevelRegressionFormulaEditWidget : public QWidget {
public:
	RegressionModelWaterLevelRegressionFormulaEditWidget(boost::shared_ptr<RegressionModel> regressionModel, QWidget *parent=0);
	virtual ~RegressionModelWaterLevelRegressionFormulaEditWidget();

	COM_SUBSCRIPTION_FUNCTION(SetWaterLevelRegressionFormula, communication::Signal<void(boost::shared_ptr<RegressionModel>, QString)>);

private:
	void setupLayout();
	void updateValues();
	void connectLocal();
	void formulaEditFinishedHandle();

private:
	boost::shared_ptr<RegressionModel> regressionModel;
	QVBoxLayout *regressorContainerWidgetLayout;
	QLineEdit *formulaEdit;
	communication::QtBoostDelegate *regressionFormulaEditedDelegate;
};

template<typename RegressionModel>
inline RegressionModelWaterLevelRegressionFormulaEditWidget<RegressionModel>::RegressionModelWaterLevelRegressionFormulaEditWidget(boost::shared_ptr<RegressionModel> regressionModel, QWidget* parent)
	:QWidget(parent), regressionModel(regressionModel)
{
	setupLayout();
	connectLocal();
	updateValues();
}

template<typename RegressionModel>
inline RegressionModelWaterLevelRegressionFormulaEditWidget<RegressionModel>::~RegressionModelWaterLevelRegressionFormulaEditWidget() {
}

template<typename RegressionModel>
inline void RegressionModelWaterLevelRegressionFormulaEditWidget<RegressionModel>::setupLayout() {
	QVBoxLayout *mainLayout = new QVBoxLayout(this);

	QLabel *formulaLabel = new QLabel(tr("Water level formula"), this);
	formulaLabel->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Fixed);
	formulaEdit = new QLineEdit(this);
	formulaEdit->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
	formulaLabel->setBuddy(formulaEdit);

	QWidget *formulaEditWidget = new QWidget(this);
	QHBoxLayout *formulaEditWidgetLayout = new QHBoxLayout(formulaEditWidget);
	formulaEditWidgetLayout->addWidget(formulaLabel, Qt::AlignLeft);
	formulaEditWidgetLayout->addWidget(formulaEdit, Qt::AlignLeft);
	formulaEditWidgetLayout->addSpacerItem(new QSpacerItem(1,1,QSizePolicy::Expanding, QSizePolicy::Ignored));


	mainLayout->addWidget(formulaEditWidget);
	mainLayout->addSpacerItem(new QSpacerItem(1,1,QSizePolicy::Ignored, QSizePolicy::Expanding));

	setLayout(mainLayout);
}

template<typename RegressionModel>
inline void RegressionModelWaterLevelRegressionFormulaEditWidget<RegressionModel>::updateValues() {
	QString formula = regressionModel->getRegressionFormula()->getWaterLevelFormula();
	formulaEdit->setText(formula);
}

template<typename RegressionModel>
inline void RegressionModelWaterLevelRegressionFormulaEditWidget<RegressionModel>::formulaEditFinishedHandle() {
	SetWaterLevelRegressionFormulaSignal(regressionModel, formulaEdit->text());
}

template<typename RegressionModel>
inline void RegressionModelWaterLevelRegressionFormulaEditWidget<RegressionModel>::connectLocal() {
	using namespace communication;
	using namespace boost;

	regressionFormulaEditedDelegate = new QtBoostDelegate(bind(&RegressionModelWaterLevelRegressionFormulaEditWidget<RegressionModel>::formulaEditFinishedHandle, this));
	connect(formulaEdit, SIGNAL(editingFinished()), regressionFormulaEditedDelegate, SLOT(execute()));

}

} /* namespace gui */

#endif /* REGRESSIONMODELWATERLEVELREGRESSIONFORMULAEDITWIDGET_HPP_ */
