/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressorEditWidget.hpp
 *
 *  Created on: Feb 21, 2014
 *      Author: gerberma
 */

#ifndef REGRESSOREDITWIDGET_HPP_
#define REGRESSOREDITWIDGET_HPP_

#include <QWidget>
#include <QLabel>
#include <QLineEdit>
#include <QHBoxLayout>
#include <boost/shared_ptr.hpp>
#include "MoveToolbarButton.hpp"
#include "communication/Signals.hpp"
#include "Delegates.hpp"

namespace gui {

template<typename Regressor>
class RegressorEditWidget : public QWidget {
public:
	COM_SUBSCRIPTION_FUNCTION(RemoveRegressor, communication::Signal<void(boost::shared_ptr<Regressor>)>);
public:
	RegressorEditWidget(boost::shared_ptr<Regressor> regressor, QWidget *parent = 0);
	virtual ~RegressorEditWidget();

	boost::shared_ptr<Regressor> getRegressor();

private:
	void setupLayout();
	void connectLocal();
	void updateValues();

private:
	boost::shared_ptr<Regressor> regressor;
	QLabel *nameLabel;
	MoveToolbarButton *removeRegressorButton;
	communication::QtBoostDelegate *regressionFormulaEditedDelegate;
	communication::QtBoostDelegate *regressorRemoveButtonDelegate;
	COM_CONNECTION regressorChangedConnection;
};

template<typename Regressor>
inline RegressorEditWidget<Regressor>::RegressorEditWidget(boost::shared_ptr<Regressor> regressor, QWidget* parent)
	:QWidget(parent), regressor(regressor)
{
	setupLayout();
	connectLocal();
	updateValues();
}

template<typename Regressor>
inline RegressorEditWidget<Regressor>::~RegressorEditWidget() {
}

template<typename Regressor>
inline void RegressorEditWidget<Regressor>::setupLayout() {
	QHBoxLayout *mainLayout = new QHBoxLayout(this);

	nameLabel = new QLabel(this);
	nameLabel->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Maximum);

	removeRegressorButton = new MoveToolbarButton(tr("-"), this);


	mainLayout->addWidget(nameLabel, 1, Qt::AlignLeft);
	mainLayout->addWidget(removeRegressorButton, 1, Qt::AlignLeft);

	setLayout(mainLayout);
}


template<typename Regressor>
inline void RegressorEditWidget<Regressor>::updateValues() {
	nameLabel->setText(regressor->getName());
	nameLabel->setEnabled(regressor->isActive());

	if(regressor->isEditable()) {
		removeRegressorButton->show();
	}else{
		removeRegressorButton->hide();
	}
}

template<typename Regressor>
inline void RegressorEditWidget<Regressor>::connectLocal() {
	using namespace communication;
	using namespace boost;

	regressorRemoveButtonDelegate = new QtBoostDelegate(bind(ref(RemoveRegressorSignal), regressor));
	connect(removeRegressorButton, SIGNAL(clicked()), regressorRemoveButtonDelegate, SLOT(execute()));

//	regressorChangedConnection = COM_SUBSCRIBE_FOR(regressor, Changed, bind(&RegressorEditWidget<Regressor>::updateValues, this));
}

template<typename Regressor>
inline boost::shared_ptr<Regressor> RegressorEditWidget<Regressor>::getRegressor() {
	return this->regressor;
}


} /* namespace gui */

#endif /* REGRESSOREDITWIDGET_HPP_ */
