/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressorFormulaBuilderEditWidget.hpp
 *
 *  Created on: 13.08.2014
 *      Author: gerberma
 */

#ifndef REGRESSORFORMULABUILDEREDITWIDGET_HPP_
#define REGRESSORFORMULABUILDEREDITWIDGET_HPP_

#include <QWidget>
#include <QLabel>
#include <QLineEdit>
#include <QTextEdit>
#include <QSizePolicy>
#include <QGridLayout>
#include <QVBoxLayout>
#include <QGroupBox>
#include <boost/foreach.hpp>
#include <boost/bind.hpp>
#include <boost/shared_ptr.hpp>
#include "RegressorFormulaBuilderUserParametersEditWidget.hpp"
#include "RegressorFormulaBuilderPreprocessorsEditWidget.hpp"
#include "EnhancedQTextEdit.hpp"
#include "ComboBox.hpp"
#include "MoveToolbarButton.hpp"
#include "communication/Signals.hpp"
#include "Delegates.hpp"

namespace gui {

template<typename RegressorFormulaBuilder>
class RegressorFormulaBuilderEditWidget : public QWidget {
public:
	typedef RegressorFormulaBuilder RegressorFormulaBuilder_t;
public:
	COM_SUBSCRIPTION_FUNCTION(SetIdentifier, communication::Signal<void(boost::shared_ptr<RegressorFormulaBuilder>, QString)>);
	COM_SUBSCRIPTION_FUNCTION(SetFormula, communication::Signal<void(boost::shared_ptr<RegressorFormulaBuilder>, QString)>);
	COM_SUBSCRIPTION_FUNCTION(SetAssignmentCategory, communication::Signal<void(boost::shared_ptr<RegressorFormulaBuilder>, QString)>);
	COM_SUBSCRIPTION_FUNCTION(SetFormulaDefinition, communication::Signal<void(boost::shared_ptr<RegressorFormulaBuilder>, QString)>);
	COM_SUBSCRIPTION_DELEGATE_FUNCTION(RegressorFormulaBuilderPreprocessorsEditWidget<RegressorFormulaBuilder>, preprocessorsEditWidget, PreprocessorStateChanged);
	COM_SUBSCRIPTION_DELEGATE_FUNCTION(RegressorFormulaBuilderUserParametersEditWidget<RegressorFormulaBuilder>, userParametersEditWidget, UserParameterSetValue);
	COM_SUBSCRIPTION_DELEGATE_FUNCTION(RegressorFormulaBuilderUserParametersEditWidget<RegressorFormulaBuilder>, userParametersEditWidget, UserParameterSetIsMinimumLeadTime);
	COM_SUBSCRIPTION_DELEGATE_FUNCTION(RegressorFormulaBuilderUserParametersEditWidget<RegressorFormulaBuilder>, userParametersEditWidget, CreateUserParameter);
	COM_SUBSCRIPTION_DELEGATE_FUNCTION(RegressorFormulaBuilderUserParametersEditWidget<RegressorFormulaBuilder>, userParametersEditWidget, RemoveUserParameter);
	COM_SUBSCRIPTION_FUNCTION(Derive, communication::Signal<void(boost::shared_ptr<RegressorFormulaBuilder>)>);
public:
	RegressorFormulaBuilderEditWidget(boost::shared_ptr<RegressorFormulaBuilder> regressorFormulaBuilder, QWidget *parent = 0)
		:QWidget(parent), regressorFormulaBuilder(regressorFormulaBuilder)
	{
		setupLayout();
		connectLocal();
		updateValues();
	}

private:
	void setupLayout() {
		groupBox = new QGroupBox(this);

			QGridLayout *gridLayout = new QGridLayout();

			deriveButton = new MoveToolbarButton(tr("d"), this);
			deriveButton->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Fixed);

			identifierLabel = new QLabel(tr("Identifier"), this);
			identifierLabel->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Fixed);
			identifierLineEdit = new QLineEdit(this);
			//identifierLineEdit->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
			formulaLabel = new QLabel(tr("Formula"), this);
			formulaLabel->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Fixed);
			formulaLineEdit = new QLineEdit(this);
			formulaLineEdit->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);

			categoryLabel = new QLabel(tr("Assignment category"), this);
			categoryLabel->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Fixed);
			categoryComboBox = new ComboBox(this);
			categoryComboBox->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);

			preprocessorsLabel = new QLabel(tr("Behavior"), this);
			preprocessorsGroupBox = new QGroupBox(this);
				QVBoxLayout *preprocessorsGroupBoxLayout = new QVBoxLayout;
				preprocessorsGroupBox->setLayout(preprocessorsGroupBoxLayout);
				preprocessorsEditWidget = new RegressorFormulaBuilderPreprocessorsEditWidget<RegressorFormulaBuilder>(regressorFormulaBuilder, this);
				preprocessorsGroupBoxLayout->addWidget(preprocessorsEditWidget);

			userParametersLabel = new QLabel(tr("Parameters"), this);
			userParametersLabel->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Fixed);

			QGroupBox *userParametersGroupBox = new QGroupBox(this);
				QVBoxLayout *userParametersGroupBoxLayout = new QVBoxLayout;
				userParametersGroupBox->setLayout(userParametersGroupBoxLayout);
				userParametersEditWidget = new RegressorFormulaBuilderUserParametersEditWidget<RegressorFormulaBuilder>(regressorFormulaBuilder, this);
				userParametersGroupBoxLayout->addWidget(userParametersEditWidget);

			definitionLabel = new QLabel(tr("Definition"), this);
			definitionLabel->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Fixed);
			definitionTextEdit = new EnhancedQTextEdit(this);
			definitionTextEdit->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);

			int rowIndex = 0;
			gridLayout->addWidget(deriveButton, rowIndex, 3, 1, 1, Qt::AlignRight);
			gridLayout->addWidget(identifierLabel, rowIndex, 0);
			gridLayout->addWidget(identifierLineEdit, rowIndex, 1, 1, 2);
			gridLayout->addWidget(formulaLabel, ++rowIndex, 0);
			gridLayout->addWidget(formulaLineEdit, rowIndex, 1, 1, 2);
			gridLayout->addWidget(categoryLabel, ++rowIndex, 0);
			gridLayout->addWidget(categoryComboBox, rowIndex, 1, 1, 2, Qt::AlignLeft);

			gridLayout->addWidget(preprocessorsLabel, ++rowIndex, 0, 1, 1, Qt::AlignLeft|Qt::AlignTop);
			gridLayout->addWidget(preprocessorsGroupBox, rowIndex, 1, 1, 3);

			gridLayout->addWidget(userParametersLabel, ++rowIndex, 0, 1, 1, Qt::AlignLeft|Qt::AlignTop);
			gridLayout->addWidget(userParametersGroupBox, rowIndex, 1, 1, 3);
			gridLayout->addWidget(definitionLabel, ++rowIndex, 0, 1, 3, Qt::AlignLeft|Qt::AlignTop);
			gridLayout->addWidget(definitionTextEdit, rowIndex, 1, 1, 3);

		groupBox->setLayout(gridLayout);
		QVBoxLayout *mainLayout = new QVBoxLayout(this);
		mainLayout->addWidget(groupBox);

		setLayout(mainLayout);
	}


	void updateValues() {
		identifierLineEdit->setText(regressorFormulaBuilder->getIdentifier());
		identifierLineEdit->setEnabled(regressorFormulaBuilder->isEditable());
		formulaLineEdit->setText(regressorFormulaBuilder->getFormula());
		formulaLineEdit->setEnabled(regressorFormulaBuilder->isEditable());
		definitionTextEdit->setText(regressorFormulaBuilder->getFormulaDefinition());
		definitionTextEdit->setEnabled(regressorFormulaBuilder->isEditable());

		BOOST_FOREACH(boost::shared_ptr<typename RegressorFormulaBuilder::AssignmentCategory_t> assignmentCategory, regressorFormulaBuilder->getAssignmentCategories()) {
			categoryComboBox->addItem(assignmentCategory->getName(), assignmentCategory->getIdentifier());
		}
		categoryComboBox->setCurrentByValue(regressorFormulaBuilder->getAssignmentCategory()->getIdentifier());
		categoryComboBox->setEnabled(regressorFormulaBuilder->isEditable());

		/**
		 * Do not show an empty behavior area if there are not preprocessors available on this regressorFormulaBuilder
		 */
		if(regressorFormulaBuilder->hasPreprocessors()) {
			preprocessorsGroupBox->hide();
			preprocessorsLabel->hide();
		}
		preprocessorsGroupBox->setEnabled(regressorFormulaBuilder->isEditable());
	}

	void categorySelectionChanged(int index) {
		SetAssignmentCategorySignal(regressorFormulaBuilder, categoryComboBox->itemData(index, Qt::UserRole).toString());
	}

	void connectLocal() {
		using namespace communication;
		using namespace boost;

		identifierDelegate = new QtBoostDelegate(bind(ref(SetIdentifierSignal), regressorFormulaBuilder,
				bind<QString>(&QLineEdit::text, ref(identifierLineEdit))
			), this);
		connect(identifierLineEdit, SIGNAL(editingFinished()), identifierDelegate, SLOT(execute()));

		formulaDelegate = new QtBoostDelegate(bind(ref(SetFormulaSignal), regressorFormulaBuilder,
				bind<QString>(&QLineEdit::text, ref(formulaLineEdit))
			), this);
		connect(formulaLineEdit, SIGNAL(editingFinished()), formulaDelegate, SLOT(execute()));

		formulaDefinitionDelegate = new QtBoostDelegate(bind(ref(SetFormulaDefinitionSignal), regressorFormulaBuilder,
				bind<QString>(&EnhancedQTextEdit::text, ref(definitionTextEdit))
			), this);
		connect(definitionTextEdit, SIGNAL(editingFinished()), formulaDefinitionDelegate, SLOT(execute()));

		categoryDelegate = new QtBoostIndexDelegate(
				bind(&RegressorFormulaBuilderEditWidget<RegressorFormulaBuilder>::categorySelectionChanged, this, _1)
			, this);
		connect(categoryComboBox, SIGNAL(currentIndexChanged(int)), categoryDelegate, SLOT(execute(int)));

		deriveDelegate = new QtBoostDelegate(bind(ref(DeriveSignal), regressorFormulaBuilder), this);
		connect(deriveButton, SIGNAL(clicked()), deriveDelegate, SLOT(execute()));
	}

private:
	boost::shared_ptr<RegressorFormulaBuilder> regressorFormulaBuilder;
	QGroupBox *groupBox;
	QLabel *identifierLabel;
	QLineEdit *identifierLineEdit;
	QLabel *formulaLabel;
	QLineEdit *formulaLineEdit;
	QLabel *categoryLabel;
	ComboBox *categoryComboBox;
	QLabel *definitionLabel;
	EnhancedQTextEdit *definitionTextEdit;
	MoveToolbarButton *deriveButton;
	QGroupBox *preprocessorsGroupBox;
	QLabel *userParametersLabel;
	RegressorFormulaBuilderUserParametersEditWidget<RegressorFormulaBuilder> *userParametersEditWidget;
	QLabel *preprocessorsLabel;
	RegressorFormulaBuilderPreprocessorsEditWidget<RegressorFormulaBuilder> *preprocessorsEditWidget;
	communication::QtBoostDelegate *identifierDelegate;
	communication::QtBoostDelegate *formulaDelegate;
	communication::QtBoostIndexDelegate *categoryDelegate;
	communication::QtBoostDelegate *formulaDefinitionDelegate;
	communication::QtBoostDelegate *deriveDelegate;
};

}  // namespace gui


#endif /* REGRESSORFORMULABUILDEREDITWIDGET_HPP_ */
