/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressorFormulaBuilderPreprocessorsEditWidget.hpp
 *
 *  Created on: 17.11.2014
 *      Author: gerberma
 */

#ifndef REGRESSORFORMULABUILDERPREPROCESSORSEDITWIDGET_HPP_
#define REGRESSORFORMULABUILDERPREPROCESSORSEDITWIDGET_HPP_

#include <QDebug>
#include <QWidget>
#include <QLabel>
#include <QCheckBox>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <boost/foreach.hpp>
#include <boost/bind.hpp>
#include <boost/ref.hpp>
#include <boost/shared_ptr.hpp>
#include <map>
#include "communication/Signals.hpp"
#include "Delegates.hpp"

namespace gui {

template<typename RegressorFormulaBuilder>
class RegressorFormulaBuilderPreprocessorsEditWidget : public QWidget {
public:
	typedef typename RegressorFormulaBuilder::RegressorFormulaBuilderPreprocessor_t Preprocessor_t;
public:
	COM_SUBSCRIPTION_FUNCTION(PreprocessorStateChanged, communication::Signal<void(boost::shared_ptr<Preprocessor_t>, bool)>);
public:
	RegressorFormulaBuilderPreprocessorsEditWidget(boost::shared_ptr<RegressorFormulaBuilder> regressorFormulaBuilder, QWidget *parent = 0)
		:QWidget(parent), regressorFormulaBuilder(regressorFormulaBuilder)
	{
		setupLayout();
		updateValues();
	}

	virtual ~RegressorFormulaBuilderPreprocessorsEditWidget()
	{;}

private:
	void setupLayout() {
		mainLayout = new QGridLayout(this);
		setLayout(mainLayout);
	}

	void updateValues() {
		using namespace communication;
		using namespace boost;

		int rowIndex = 0;
		BOOST_FOREACH(boost::shared_ptr<Preprocessor_t> preprocessor, regressorFormulaBuilder->getPreprocessors()) {
			QLabel *nameLabel = new QLabel(tr("Use ") + preprocessor->getName(), this);
			QCheckBox *isActiveCheckBox = new QCheckBox(this);

			isActiveCheckBox->setChecked(preprocessor->isActive());
			isActiveCheckBox->setEnabled(preprocessor->isEnabled());

			QtBoostBooleanDelegate *delegate = new QtBoostBooleanDelegate(bind(ref(PreprocessorStateChangedSignal), preprocessor, _1), this);
			connect(isActiveCheckBox, SIGNAL(clicked(bool)), delegate, SLOT(execute(bool)));

			mainLayout->addWidget(nameLabel, rowIndex, 0, Qt::AlignLeft);
			mainLayout->addWidget(isActiveCheckBox, rowIndex, 1, Qt::AlignLeft);


			++rowIndex;
		}
	}

private:
	boost::shared_ptr<RegressorFormulaBuilder> regressorFormulaBuilder;
	QGridLayout *mainLayout;
};

}  // namespace gui



#endif /* REGRESSORFORMULABUILDERPREPROCESSORSEDITWIDGET_HPP_ */
