/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressorFormulaBuildersEditWidget.hpp
 *
 *  Created on: Aug 13, 2014
 *      Author: gerberma
 */

#ifndef REGRESSORFORMULABUILDERSEDITWIDGET_HPP_
#define REGRESSORFORMULABUILDERSEDITWIDGET_HPP_

#include <QWidget>
#include <QVBoxLayout>
#include <QScrollArea>
#include <boost/bind.hpp>
#include <boost/function.hpp>
#include <boost/shared_ptr.hpp>
#include "communication/Signals.hpp"

namespace gui {

/**
 * Widget containing a list of model definitions
 */
template<typename RegressionFormula>
class RegressorFormulaBuildersEditWidget : public QWidget {
public:
	typedef typename RegressionFormula::RegressorFormulaBuilder_t RegressorFormulaBuilder_t;
public:
	RegressorFormulaBuildersEditWidget(boost::shared_ptr<RegressionFormula> regressionFormula, QWidget *parent = 0)
		:QWidget(parent), regressionFormula(regressionFormula), containerWidgetLayout(NULL), elementContainerWidget(NULL), elementScrollArea(NULL)
	{;}

	/*
	 * The UiManager itself can define a function acting as factory or a boost function which delegates to the factory owned by the UiManager
	 * A destroy function is not necessary as this widget will be owned by a parent and destroyed by qt's memory management. Well, this is an
	 * assumption (and therefor a suggestion), but the UiManager can handle this as he wants.
	 */
	void init(boost::function<QWidget*(boost::shared_ptr<RegressorFormulaBuilder_t>)> widgetFactory) {
		setupLayout();
		updateValues(widgetFactory);
		connectLocal(widgetFactory);
	}

private:
	void setupLayout() {
		QVBoxLayout *mainLayout = new QVBoxLayout;

		containerWidgetLayout = new QVBoxLayout;
		containerWidgetLayout->setSizeConstraint(QLayout::SetMinimumSize);

		elementContainerWidget = new QWidget(this);
		elementContainerWidget->setLayout(containerWidgetLayout);

		elementScrollArea = new QScrollArea(this);
		elementScrollArea->setWidget(elementContainerWidget);
		elementScrollArea->setWidgetResizable(true);
		elementScrollArea->setFrameShape(QFrame::NoFrame);

		mainLayout->addWidget(elementScrollArea);

		setLayout(mainLayout);
	}

	void updateValues(boost::function<QWidget*(boost::shared_ptr<RegressorFormulaBuilder_t>)> widgetFactory) {
		BOOST_FOREACH(boost::shared_ptr<RegressorFormulaBuilder_t> regressorFormulaBuilder, regressionFormula->getRegressorFormulaBuilders()) {
			QWidget *widget = widgetFactory(regressorFormulaBuilder);
			containerWidgetLayout->addWidget(widget, Qt::AlignLeft);
		}
	}

	void regressorFormulaBuilderAdded(boost::function<QWidget*(boost::shared_ptr<RegressorFormulaBuilder_t>)> widgetFactory, boost::shared_ptr<RegressorFormulaBuilder_t> regressorFromulaBuilder) {
		QWidget *widget = widgetFactory(regressorFromulaBuilder);
		containerWidgetLayout->addWidget(widget, Qt::AlignLeft);
	}

	void connectLocal(boost::function<QWidget*(boost::shared_ptr<RegressorFormulaBuilder_t>)> widgetFactory) {
		using namespace boost;

		regressorFormulaBuilderAddedConnection = COM_SUBSCRIBE_FOR(
					regressionFormula,
					RegressorFormulaBuilderAdded,
					bind(&RegressorFormulaBuildersEditWidget<RegressionFormula>::regressorFormulaBuilderAdded, this, widgetFactory, _1)
				);
	}

private:
	boost::shared_ptr<RegressionFormula> regressionFormula;
	QVBoxLayout *containerWidgetLayout;
	QWidget *elementContainerWidget;
	QScrollArea *elementScrollArea;
	COM_CONNECTION regressorFormulaBuilderAddedConnection;
};

}  // namespace gui


#endif /* REGRESSORFORMULABUILDERSEDITWIDGET_HPP_ */
