/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressorInfluenceFactorInstantiationMessageDialog.hpp
 *
 *  Created on: 17.11.2014
 *      Author: gerberma
 */

#ifndef REGRESSORINFLUENCEFACTORINSTANTIATIONMESSAGEDIALOG_HPP_
#define REGRESSORINFLUENCEFACTORINSTANTIATIONMESSAGEDIALOG_HPP_

#include <QDebug>
#include <QDialog>
#include <QDialogButtonBox>
#include <QLabel>
#include <QVBoxLayout>
#include <boost/shared_ptr.hpp>

namespace gui {

template<typename Regressor>
class RegressorInfluenceFactorInstantiationMessageDialog : public QDialog {
public:
	typedef Regressor Regressor_t;
public:
	RegressorInfluenceFactorInstantiationMessageDialog(boost::shared_ptr<Regressor_t> regressor, QWidget *parent = 0)
		:QDialog(parent), regressor(regressor)
	{
		setupLayout();
	}

	void execWithMessage(QString message) {
		qDebug() << "##### " << __FUNCTION__;
		messageLabel->setText(message);

		exec();
	}

private:
	void setupLayout() {
		QVBoxLayout *mainLayout = new QVBoxLayout;

		infoLabel = new QLabel(tr("Note for regressor ") + regressor->getRegressorFormulaBuilder()->resolveExtendedIdentifier() + " :", this);
		messageLabel = new QLabel(this);
		buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok);

		mainLayout->addWidget(infoLabel, Qt::AlignTop);
		mainLayout->addWidget(messageLabel, Qt::AlignTop);
		mainLayout->addWidget(buttonBox, Qt::AlignBottom);


		connect(buttonBox, SIGNAL(accepted()), this, SLOT(accept()));
		setLayout(mainLayout);
	}

private:
	boost::shared_ptr<Regressor_t> regressor;
	QLabel *infoLabel;
	QLabel *messageLabel;
	QDialogButtonBox *buttonBox;
};

} // namespace gui

#endif /* REGRESSORINFLUENCEFACTORINSTANTIATIONMESSAGEDIALOG_HPP_ */
