/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * ReportListTextElementWidget.hpp
 *
 *  Created on: Jul 23, 2014
 *      Author: gerberma
 */

#ifndef REPORTLISTTEXTELEMENTWIDGET_HPP_
#define REPORTLISTTEXTELEMENTWIDGET_HPP_

#include <QWidget>
#include <QGridLayout>
#include <QLabel>
#include <QCheckBox>
#include <boost/bind.hpp>
#include <boost/ref.hpp>
#include <boost/shared_ptr.hpp>
#include "Delegates.hpp"
#include "communication/Signals.hpp"
#include "ReportListElementToolBar.hpp"

namespace gui {

template<typename ReportListElement, typename DynamicContentWidget>
class ReportListElementWidget : public QWidget
{
public:
	COM_SUBSCRIPTION_FUNCTION(SelectChanged, communication::Signal<void(boost::shared_ptr<ReportListElement>, bool)>);
	COM_SUBSCRIPTION_FUNCTION(MoveUp, communication::Signal<void(boost::shared_ptr<ReportListElement>)>);
	COM_SUBSCRIPTION_FUNCTION(MoveDown, communication::Signal<void(boost::shared_ptr<ReportListElement>)>);
public:
	typedef ReportListElement ReportListElement_t;
	typedef DynamicContentWidget DynamicContentWidget_t;
public:
	//provide somehow a boost function which reads the text from the given text element
	ReportListElementWidget(boost::shared_ptr<ReportListElement>reportListElement, QWidget *parent = 0)
		:QWidget(parent), reportListElement(reportListElement), dynamicContentWidget(NULL)
	{
	}

	void setDynamicContentWidget(DynamicContentWidget *dynamicContentWidget) {
		this->dynamicContentWidget = dynamicContentWidget;
	}

	void init() {
		setupLayout();
		connectLocal();
		updateValues();
	}

private:
	void setupLayout() {
		QGridLayout *mainLayout = new QGridLayout;

		nameLabel = new QLabel(this);
		nameLabel->setFixedWidth(100);
		nameLabel->setWordWrap(true);
		//dynamincContentWidget = new DynamicContentWidget(reportListElement, this);
		dynamicContentWidget->setMinimumWidth(200);
		checkBox = new QCheckBox(this);
		toolBar = new ReportListElementToolBar(this);

		QWidget *checkBoxAndNameLabelWidget = new QWidget(this);
		QHBoxLayout *checkBoxAndNameLabelLayout = new QHBoxLayout(checkBoxAndNameLabelWidget);
		checkBoxAndNameLabelLayout->addWidget(checkBox);
		checkBoxAndNameLabelLayout->addWidget(nameLabel);

		mainLayout->addWidget(checkBoxAndNameLabelWidget, 0, 0, Qt::AlignLeft);
		mainLayout->addWidget(dynamicContentWidget, 0, 1, Qt::AlignLeft);
		mainLayout->addWidget(toolBar, 0, 3, Qt::AlignRight);

		setLayout(mainLayout);
	}

	void updateValues() {
		nameLabel->setText(reportListElement->getName());
		checkBox->setCheckState(reportListElement->isSelected() ? Qt::Checked : Qt::Unchecked);
	}

	void connectLocal() {
		using namespace communication;
		using namespace boost;

		checkBoxDelegate = new QtBoostBooleanDelegate(bind(ref(SelectChangedSignal), reportListElement, _1), this);
		connect(checkBox, SIGNAL(stateChanged(int)), checkBoxDelegate, SLOT(execute(int)));

		COM_SUBSCRIBE_FOR(toolBar, MoveUpButtonClicked, bind(ref(MoveUpSignal), reportListElement));
		COM_SUBSCRIBE_FOR(toolBar, MoveDownButtonClicked, bind(ref(MoveDownSignal), reportListElement));
	}

private:
	QLabel *nameLabel;
	QCheckBox *checkBox;
	ReportListElementToolBar *toolBar;
	boost::shared_ptr<ReportListElement> reportListElement;
	DynamicContentWidget *dynamicContentWidget;
	communication::QtBoostBooleanDelegate *checkBoxDelegate;
};

}  // namespace gui


#endif /* REPORTLISTTEXTELEMENTWIDGET_HPP_ */
