/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * ReportTextEditWidget.hpp
 *
 *  Created on: Jul 24, 2014
 *      Author: gerberma
 */

#ifndef REPORTTEXTEDITWIDGET_HPP_
#define REPORTTEXTEDITWIDGET_HPP_

#include <QWidget>
#include <boost/bind.hpp>
#include <boost/shared_ptr.hpp>
#include "Delegates.hpp"
#include "communication/Signals.hpp"

namespace gui {

template<typename ReportListElement, typename TextEditFieldType>
class ReportTextEditWidget : public TextEditFieldType
{
public:
	typedef ReportListElement ReportListElement_t;
public:
	COM_SUBSCRIPTION_FUNCTION(TextChanged, communication::Signal<void(boost::shared_ptr<ReportListElement>, QString)>);
public:
	ReportTextEditWidget(boost::shared_ptr<ReportListElement> reportListElement, QWidget *parent = 0)
		:TextEditFieldType(parent), reportListElement(reportListElement)
	{
		//TODO: connect to reportListElement changes signal
		connectLocal();
		updateValues();
	}

private:
	void updateValues() {
		TextEditFieldType::setText(reportListElement->getText());
	}

	void connectLocal() {
		using namespace boost;
		using namespace communication;

		textChangedDelegate = new QtBoostDelegate(bind(&ReportTextEditWidget<ReportListElement, TextEditFieldType>::textEditFinishedHandle, this));
		TextEditFieldType::connect(this, SIGNAL(editingFinished()), textChangedDelegate, SLOT(execute()));
	}

	void textEditFinishedHandle() {
		TextChangedSignal(reportListElement, TextEditFieldType::text());
	}

private:
	boost::shared_ptr<ReportListElement> reportListElement;
	communication::QtBoostDelegate *textChangedDelegate;
};

}  // namespace gui


#endif /* REPORTTEXTEDITWIDGET_HPP_ */
