/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * SvgPlotWidget.hpp
 *
 *  Created on: Jan 30, 2014
 *      Author: gerberma
 */

#ifndef SVGPLOTWIDGET_HPP_
#define SVGPLOTWIDGET_HPP_

#include <QSvgWidget>
#include <boost/bind.hpp>
#include "communication/Signals.hpp"

namespace gui {

template<typename SvgPlot>
class SvgPlotWidget : public QSvgWidget {
public:
	SvgPlotWidget(SvgPlot *svgPlot, QWidget *parent = 0);
	virtual ~SvgPlotWidget();

private:
	void setupLayout();
	void updateValues();
	void connectLocal();

private:
	SvgPlot *svgPlot;
	COM_CONNECTION svgPlotLoadedConnection;
};

template<typename SvgPlot>
inline SvgPlotWidget<SvgPlot>::SvgPlotWidget(SvgPlot *svgPlot, QWidget* parent)
	:svgPlot(svgPlot)
{
	setupLayout();
	updateValues();
	connectLocal();
}

template<typename SvgPlot>
inline SvgPlotWidget<SvgPlot>::~SvgPlotWidget() {
}

template<typename SvgPlot>
inline void SvgPlotWidget<SvgPlot>::setupLayout() {
	setFixedWidth(svgPlot->getWidthInPixel());
	setFixedHeight(svgPlot->getHeightInPixel());
}

template<typename SvgPlot>
inline void SvgPlotWidget<SvgPlot>::updateValues() {
	load(svgPlot->getPlotLocation());
}

template<typename SvgPlot>
inline void SvgPlotWidget<SvgPlot>::connectLocal() {
	svgPlotLoadedConnection = COM_SUBSCRIBE_FOR(svgPlot, Loaded, boost::bind(&SvgPlotWidget<SvgPlot>::updateValues, this));
}

} /* namespace gui */


#endif /* SVGPLOTWIDGET_HPP_ */
