/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * TwoSideNamedWidget.hpp
 *
 *  Created on: Mar 5, 2014
 *      Author: gerberma
 */

#ifndef TWOSIDENAMEDWIDGET_HPP_
#define TWOSIDENAMEDWIDGET_HPP_

#include <stdexcept>
#include <QWidget>
#include <QSizePolicy>
#include "GroupBoxLayoutBuilder.hpp"
#include "SplittedLayoutBuilder.hpp"

namespace gui {

template
<
	typename LeftWidget,
	typename RightWidget
>
class TwoSideNamedWidget : public QWidget {
public:
	typedef LeftWidget left_widget_t;
	typedef RightWidget right_widget_t;
public:
	TwoSideNamedWidget(LeftWidget *leftWidget, QString leftWidgetName, RightWidget *rightWidget, QString rightWidgetName, QWidget *parent = 0)
		:QWidget(parent), leftWidget(leftWidget), leftWidgetName(leftWidgetName), rightWidget(rightWidget), rightWidgetName(rightWidgetName)
	{;}

	TwoSideNamedWidget(QString leftWidgetName, QString rightWidgetName, QWidget *parent = 0)
		:QWidget(parent), leftWidget(NULL), leftWidgetName(leftWidgetName), rightWidget(NULL), rightWidgetName(rightWidgetName)
	{;}

	void setLeftWidget(LeftWidget* leftWidget) {
		this->leftWidget = leftWidget;
	}

	void setLeftWidgetName(const QString& leftWidgetName) {
		this->leftWidgetName = leftWidgetName;
	}

	void setRightWidget(RightWidget* rightWidget) {
		this->rightWidget = rightWidget;
	}

	void setRightWidgetName(const QString& rightWidgetName) {
		this->rightWidgetName = rightWidgetName;
	}

	void init() {
		if(leftWidget == NULL || rightWidget == NULL) {
			throw std::logic_error("leftWidget or rightWidget not set");
		}
		setupLayout();
	}

private:
	void setupLayout() {
		GroupBoxLayoutBuilder leftGroupBoxLayoutBuilder(leftWidgetName);
		GroupBoxLayoutBuilder rightGroupBoxLayoutBuilder(rightWidgetName);

		leftGroupBoxLayoutBuilder.addWidget(leftWidget);
		rightGroupBoxLayoutBuilder.addWidget(rightWidget);

		QWidget *leftWidgetContainer = new QWidget(this);
		QWidget *rightWidgetContainer = new QWidget(this);

		leftWidgetContainer->setLayout(leftGroupBoxLayoutBuilder.build(this, QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding), false));
		rightWidgetContainer->setLayout(rightGroupBoxLayoutBuilder.build(this, QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding), false));

		SplittedLayoutBuilder splittedLayoutBuilder;
		setLayout(splittedLayoutBuilder.build(this, leftWidgetContainer, rightWidgetContainer));
	}

private:
	LeftWidget *leftWidget;
	QString leftWidgetName;
	RightWidget *rightWidget;
	QString rightWidgetName;
};

} /* namespace gui */

#endif /* TWOSIDENAMEDWIDGET_HPP_ */
