/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * AlgorithmReportListElement.cpp
 *
 *  Created on: 25.02.2015
 *      Author: gerberma
 */

#include <stdexcept>
#include "AlgorithmReportListElement.hpp"

namespace dambase {
namespace model {


AlgorithmReportListElement::AlgorithmReportListElement(boost::shared_ptr<api::IRegressionModel> regressionModel, int index)
	:index(index), isselected(true), regressionModel(regressionModel)
{
}

AlgorithmReportListElement::~AlgorithmReportListElement() {
	// TODO Auto-generated destructor stub
}

QString AlgorithmReportListElement::getName() {
	return "Algorithm";
}

int AlgorithmReportListElement::getIndex() {
	return index;
}

void AlgorithmReportListElement::build(api::IAlgorithmReportListElement::ReportContext_t& reportContext) {
	reportContext << "## Algorithm" << endl << endl;

	if(regressionModel->isStandardAlgorithm()) {
		reportContext << reportContext.escape("Ordinary least square") << endl << endl;
	}else if(regressionModel->isPraisWinstenAlgorithm()) {
		reportContext << reportContext.escape("Prais Winsten with rho = ") << QString::number(regressionModel->getPraisWinstenRho()) << endl << endl;
	}else if(regressionModel->isRidgeAlgorithm()) {
		reportContext << reportContext.escape("Ridge Regression with k = ") << QString::number(regressionModel->getRidgeKValue()) << endl << endl;
	}else if(regressionModel->isPCRAlgorithm()) {
		reportContext << reportContext.escape("Principal Component Regression with r = ") << QString::number(regressionModel->getPCRrValue()) << endl << endl;
	}else{
		throw std::logic_error("No known algorithm");
	}
}

bool AlgorithmReportListElement::isSelected() {
	return isselected;
}

void AlgorithmReportListElement::setIsSelected(bool state) {
	this->isselected = state;
}

} /* namespace model */
} /* namespace dambase */
