/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * Analysis.cpp
 *
 *  Created on: 17.02.2015
 *      Author: gerberma
 */

#include <algorithm>
#include <iostream>
#include "Analysis.hpp"

namespace dambase {
namespace model {

Analysis::Analysis()
{;}

Analysis::~Analysis(){std::cout << __FUNCTION__ << std::endl;}

const QString&
Analysis::getName() const {
	return name;
}

void
Analysis::setName(QString name) {
	this->name = name;
	api::IAnalysis::fire();
}

const QString&
Analysis::getDescription() const {
	return description;
}

void
Analysis::setDescription(QString description) {
	this->description = description;
	api::IAnalysis::fire();
}

const boost::shared_ptr<api::IAnalysis::DataModel_t> &
Analysis::getDataModel() const {
	return dataModel;
}

void
Analysis::setDataModel(boost::shared_ptr<api::IAnalysis::DataModel_t> dataModel) {
	this->dataModel = dataModel;
	api::IAnalysis::fireDataModelSet();
}

bool
Analysis::isNewDataModelAllowed() {
	return AnalysisActionRules::isNewDataModelAllowedFor(this);
}

bool
Analysis::isNewRegressionModelAllowed() {
	return AnalysisActionRules::isNewRegressionModelAllowedFor(this);
}

/**
 * This is primarly used by the deserialization process
 */
void
Analysis::setRegressionModels(std::vector<boost::shared_ptr<api::IAnalysis::RegressionModel_t> > &regressionModels) {
	this->regressionModels = regressionModels;
}

void
Analysis::addRegressionModel(boost::shared_ptr<api::IAnalysis::RegressionModel_t> regressionModel) {
	regressionModels.push_back(regressionModel);
	api::IAnalysis::fireRegressionModelAdded(regressionModel);
}

const api::IAnalysis::RegressionModelContainer_t &
Analysis::getRegressionModels() const {
	return regressionModels;
}

void
Analysis::removeRegressionModel(boost::shared_ptr<api::IAnalysis::RegressionModel_t> regressionModel) {
	//then, cleanup and delete by ownership policy
	regressionModels.erase(std::remove(regressionModels.begin(), regressionModels.end(), regressionModel));
	api::IAnalysis::fireRemoveRegressionModel(regressionModel);
}

const boost::shared_ptr<api::IAnalysis::WaterLevelScalingStrategy_t> &
Analysis::getWaterLevelScalingStrategy() const {
	return waterLevelScalingStrategy;
}

void
Analysis::setWaterLevelScalingStrategy(boost::shared_ptr<api::IAnalysis::WaterLevelScalingStrategy_t> waterLevelScalingStrategy) {
	this->waterLevelScalingStrategy = waterLevelScalingStrategy;
}

} //namespace model
} //namespace dambase
