/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * Analysis.hpp
 *
 *  Created on: Nov 27, 2013
 *      Author: gerberma
 */

#ifndef ANALYSIS_HPP_
#define ANALYSIS_HPP_

#include <boost/shared_ptr.hpp>
#include <QString>
#include "api/IAnalysis.hpp"
#include "AnalysisActionRules.hpp"

namespace dambase {
namespace model {

class Analysis : public api::IAnalysis, AnalysisActionRules
{
public:
	Analysis();

	virtual ~Analysis();

	const QString& getName() const;

	void setName(QString name);

	const QString& getDescription() const;

	void setDescription(QString description);

	const boost::shared_ptr<api::IAnalysis::DataModel_t> &getDataModel() const;

	void setDataModel(boost::shared_ptr<api::IAnalysis::DataModel_t> dataModel);

	bool isNewDataModelAllowed();

	bool isNewRegressionModelAllowed();

	/**
	 * This is primarly used by the deserialization process
	 */
	void setRegressionModels(std::vector<boost::shared_ptr<api::IAnalysis::RegressionModel_t> > &regressionModels);

	void addRegressionModel(boost::shared_ptr<api::IAnalysis::RegressionModel_t> regressionModel);

	const api::IAnalysis::RegressionModelContainer_t &getRegressionModels() const;

	void removeRegressionModel(boost::shared_ptr<api::IAnalysis::RegressionModel_t> regressionModel);

	const boost::shared_ptr<api::IAnalysis::WaterLevelScalingStrategy_t> &getWaterLevelScalingStrategy() const;

	void setWaterLevelScalingStrategy(boost::shared_ptr<api::IAnalysis::WaterLevelScalingStrategy_t> waterLevelScalingStrategy);

private:
	QString name;
	QString description;
	boost::shared_ptr<api::IAnalysis::DataModel_t> dataModel;
	boost::shared_ptr<api::IAnalysis::WaterLevelScalingStrategy_t> waterLevelScalingStrategy;
	api::IAnalysis::RegressionModelContainer_t regressionModels;
};

}} /* namespace dambase::model */
#endif /* ANALYSIS_HPP_ */
