/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * AnalysisToolModeDescription.cpp
 *
 *  Created on: 16.02.2015
 *      Author: gerberma
 */

#include "AnalysisToolModeDescription.hpp"

namespace dambase {
namespace model {


AnalysisToolModeDescription::AnalysisToolModeDescription(boost::shared_ptr<api::IAnalysisToolModeDescription::AnalysisToolDescription_t> analysisToolDescription)
	:analysisToolDescription(analysisToolDescription),
	 flagAppearLeft(false),
	 flagAppearEverywhere(true),
	 tabNumber(0),
	 flagAppearAutomatically(false),
	 appearForMode(1)
{;}

AnalysisToolModeDescription::AnalysisToolModeDescription(
		boost::shared_ptr<api::IAnalysisToolModeDescription::AnalysisToolDescription_t> analysisToolDescription,
		bool appearLeft,
		bool appearEverywhere,
		int tabNumber,
		bool appearAutomatically,
		int appearForMode
)
	:analysisToolDescription(analysisToolDescription),
	 flagAppearLeft(appearLeft),
	 flagAppearEverywhere(appearEverywhere),
	 tabNumber(tabNumber),
	 flagAppearAutomatically(appearAutomatically),
	 appearForMode(appearForMode)
{;}


AnalysisToolModeDescription::~AnalysisToolModeDescription(){;}

/**
 * Together with setAppearAutomatically, this flag defines on which side the AT should appear automatically if AppearAutomatically is set to true
 */
void
AnalysisToolModeDescription::setAppearLeft(bool flag) {
	flagAppearLeft = flag;
}

bool
AnalysisToolModeDescription::appearLeft(bool question) {
	return flagAppearLeft == question;
}

void
AnalysisToolModeDescription::setAppearEverywhere(bool flag) {
	flagAppearEverywhere = flag;
}

bool
AnalysisToolModeDescription::appearEverywhere(bool question) {
	return flagAppearEverywhere == question;
}

void
AnalysisToolModeDescription::setForTabNumber(int number) {
	tabNumber = number;
}

bool
AnalysisToolModeDescription::forTabNumber(int question) {
	return tabNumber == question;
}

void
AnalysisToolModeDescription::setAppearAutomatically(bool flag) {
	flagAppearAutomatically = flag;
}

bool
AnalysisToolModeDescription::appearAutomatically(bool question) {
	return flagAppearAutomatically == question;
}

void
AnalysisToolModeDescription::setForMode(int mode) {
	appearForMode = mode;
}

bool
AnalysisToolModeDescription::forMode(boost::shared_ptr<api::IAnalysisToolModeDescription::ModeHandler_t> question) {
	return question->getMode() == appearForMode;
}

boost::shared_ptr<api::IAnalysisToolModeDescription::AnalysisToolDescription_t>
AnalysisToolModeDescription::getAnalysisToolDescription() {
	return analysisToolDescription;
}

} //namespace model
} //namespace dambase
