/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * AnalysisToolsModeDescription.cpp
 *
 *  Created on: 16.02.2015
 *      Author: gerberma
 */


#include <boost/foreach.hpp>
#include "AnalysisToolsModeDescription.hpp"

namespace dambase {
namespace model {

AnalysisToolsModeDescription::AnalysisToolsModeDescription(boost::shared_ptr<api::IAnalysisToolsModeDescription::ModeHandler_t> modeHandler)
	:modeHandler(modeHandler)
{;}

void
AnalysisToolsModeDescription::addAnalysisToolModeDescription(boost::shared_ptr<api::IAnalysisToolsModeDescription::AnalysisToolModeDescription_t> analysisToolModeDescription) {
	analysisToolsModeDescriptions.push_back(analysisToolModeDescription);
}


std::vector<boost::shared_ptr<api::IAnalysisToolsModeDescription::AnalysisToolDescription_t> >
AnalysisToolsModeDescription::filterForAutomaticallyAppear(bool left, int tabNumber, bool automatically) {
	std::vector<boost::shared_ptr<api::IAnalysisToolsModeDescription::AnalysisToolDescription_t> > result;

	BOOST_FOREACH(boost::shared_ptr<api::IAnalysisToolsModeDescription::AnalysisToolModeDescription_t> analysisToolModeDescription, analysisToolsModeDescriptions) {
		if(analysisToolModeDescription->appearLeft(left)
				&& analysisToolModeDescription->forTabNumber(tabNumber)
				&& analysisToolModeDescription->appearAutomatically(automatically)
				&& analysisToolModeDescription->forMode(modeHandler)
		) {
			result.push_back(analysisToolModeDescription->getAnalysisToolDescription());
		}
	}

	return result;
}

std::vector<boost::shared_ptr<api::IAnalysisToolsModeDescription::AnalysisToolDescription_t> >
AnalysisToolsModeDescription::filterForListingEverywhere() {
	std::vector<boost::shared_ptr<api::IAnalysisToolsModeDescription::AnalysisToolDescription_t> > result;

	BOOST_FOREACH(boost::shared_ptr<api::IAnalysisToolsModeDescription::AnalysisToolModeDescription_t> analysisToolModeDescription, analysisToolsModeDescriptions) {
		if(analysisToolModeDescription->appearEverywhere(true)
				&& analysisToolModeDescription->forMode(modeHandler)
		) {
			result.push_back(analysisToolModeDescription->getAnalysisToolDescription());
		}
	}

	return result;
}

std::vector<boost::shared_ptr<api::IAnalysisToolsModeDescription::AnalysisToolDescription_t> >
AnalysisToolsModeDescription::filterForListing(int tabNumber) {
	std::vector<boost::shared_ptr<api::IAnalysisToolsModeDescription::AnalysisToolDescription_t> > result;

	BOOST_FOREACH(boost::shared_ptr<api::IAnalysisToolsModeDescription::AnalysisToolModeDescription_t> analysisToolModeDescription, analysisToolsModeDescriptions) {
		if(analysisToolModeDescription->forTabNumber(tabNumber)
				&& analysisToolModeDescription->forMode(modeHandler)
		) {
			result.push_back(analysisToolModeDescription->getAnalysisToolDescription());
		}
	}

	return result;
}

std::vector<boost::shared_ptr<api::IAnalysisToolsModeDescription::AnalysisToolDescription_t> >
AnalysisToolsModeDescription::filterForListing(bool left, int tabNumber) {
	std::vector<boost::shared_ptr<api::IAnalysisToolsModeDescription::AnalysisToolDescription_t> > result;

	BOOST_FOREACH(boost::shared_ptr<api::IAnalysisToolsModeDescription::AnalysisToolModeDescription_t> analysisToolModeDescription, analysisToolsModeDescriptions) {
		if(analysisToolModeDescription->appearLeft(left)
				&& analysisToolModeDescription->forTabNumber(tabNumber)
				&& analysisToolModeDescription->forMode(modeHandler)
		) {
			result.push_back(analysisToolModeDescription->getAnalysisToolDescription());
		}
	}

	return result;
}

} //namespace model
} //namespace dambase

