/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModel.hpp
 *
 *  Created on: Dec 6, 2013
 *      Author: gerberma
 */

#ifndef DATAMODEL_HPP_
#define DATAMODEL_HPP_

#include <QUrl>
#include <QString>
#include <boost/shared_ptr.hpp>
#include <vector>
#include "DataModelActionRules.hpp"
#include "SignalPolicies.hpp"
#include "api/IDataModel.hpp"
#include "r/api/IDataFrame.hpp"
#include "r/DataFrameProvider.hpp"

namespace dambase {
namespace model {
//using namespace communication;

class DataModel : public api::IDataModel {
public:
	void setAvailableCSVDelimiters(std::map<QString, QString> availableCSVDelimiters);

	const std::map<QString, QString> &getAvailableCSVDelimiters() const;

	const QString &getCSVDelimiter() const;

	void setCSVDelimiter(QString csvDelimiter);

	const api::IDataModel::source_uri_t &getSourceUri() const;

	void setSourceUri(const api::IDataModel::source_uri_t &sourceUri);


	void setDataFrameProvider(api::IDataModel::DataFrameProvider_t *dataFrameProvider);


	void loadDataFrame();

	/**
	 * This is used by loadDataFrame as well as the deserialization process
	 */
	void setDataFrame(boost::shared_ptr<api::IDataModel::DataFrame_t> newDataFrame);


	boost::shared_ptr<api::IDataModel::DateColumn_t> getDateColumn();

	boost::shared_ptr<api::IDataModel::DateColumn_t> getDateColumn(QString dateFormat);

	void setDateColumn(int index);

	void estimateDateColumn();

	bool isColumnAssignedWithIndex(int index);

	bool assignBehaviorIndicatorColumn(int index);

	bool removeBehaviorIndicatorColumn(int index);

	bool isBehaviorIndicatorColumnAssignedFor(int index);

	bool assignWaterLevelColumn(int index);

	bool removeWaterLevelColumn(int index);

	bool isWaterLevelColumnAssignedFor(int index);


	bool assignTemperatureColumn(int index);

	bool removeTemperatureColumn(int index);

	bool isTemperatureColumnAssignedFor(int index);

	Header_Container_t& getHeaders();


	boost::shared_ptr<api::IDataModel::Column_t> getColumnByIndex(int index);

	api::IDataModel::Assignable_Header_Container_t getAssignableDateColumnHeaders();

	api::IDataModel::Assignable_Header_Container_t getAssignableBehaviorIndicatorHeaders();

	api::IDataModel::Assignable_Header_Container_t getAssignableWaterLevelHeaders();

	api::IDataModel::Assignable_Header_Container_t getAssignableTemperatureHeaders();

	const api::IDataModel::Behavior_Indicator_Column_Container_t& getBehaviorIndicatorColumns();

	boost::shared_ptr<api::IDataModel::Behavior_Indicator_Column_t> getBehaviorIndicatorColumnWith(int index);

	const api::IDataModel::Water_Level_Column_Container_t& getWaterLevelColumns();

	const api::IDataModel::Temperature_Column_Container_t& getTemperatureColumns();

	boost::shared_ptr<api::IDataModel::Column_t> getAnyAssignedColumn();

	const boost::shared_ptr<api::IDataModel::DataFrame_t> &getDataFrame() const;

	void applyDateFormat(QString dateFormat);

	bool isDateColumnEditable();

	bool isDateColumnAssignmentComplete();

	const QString &getDateFormat() const;

	void setDateFormat(QString dateFormat);

	void addDataModelAnalysis(boost::shared_ptr<api::IDataModel::AnalysisTool_t> dataModelAnalysis);

	void removeDataModelAnalysis(boost::shared_ptr<api::IDataModel::AnalysisTool_t> dataModelAnalysis);

	DataModel();

	DataModel(
			api::IDataModel::DataFrameProvider_t *dataFrameProvider,
			std::map<QString, QString> availableCSVDelimiters,
			QString csvDelimiter,
			QString dateFormat = QString("%d.%m.%Y"));

	virtual ~DataModel();

private:
	void updateAnalysisTools();

private:
	source_uri_t sourceURI;
	std::map<QString, QString> availableCSVDelimiters;
	QString csvDelimiter;
	QString dateFormat;
	bool dateFormatApplied;
	api::IDataModel::DataFrameProvider_t *dataFrameProvider;
	boost::shared_ptr<api::IDataModel::DataFrame_t> dataFrame;
	Header_Container_t nullHeaders;
	Behavior_Indicator_Column_Container_t nullBehaviorIndicatorColumns;
	Water_Level_Column_Container_t nullWaterLevelColumns;
	Temperature_Column_Container_t nullTemperatureColumns;
	std::vector<boost::shared_ptr<api::IDataModel::AnalysisTool_t> > dataModelAnalysesTools;
};




}} /* namespace dambase::model */

#endif /* DATAMODEL_HPP_ */
