/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelAnalysisInfluenceFactors.hpp
 *
 *  Created on: Jan 31, 2014
 *      Author: gerberma
 */

#ifndef DATAMODELANALYSISINFLUENCEFACTORS_HPP_
#define DATAMODELANALYSISINFLUENCEFACTORS_HPP_

#include <boost/bind.hpp>
#include <boost/foreach.hpp>
#include <boost/shared_ptr.hpp>
#include "SignalPolicies.hpp"
#include "communication/Signals.hpp"
#include "api/IDataModelAnalysisInfluenceFactors.hpp"
#include "api/IDataModel.hpp"
#include "r/api/IDataFrameInfluenceFactorsProvider.hpp"
#include <QString>
#include <QDebug>

namespace dambase {
namespace model {

class DataModelAnalysisInfluenceFactors : public virtual api::IDataModelAnalysisInfluenceFactors {
public:
	typedef api::IDataModel DataModel_t;
	typedef r::api::IDataFrameInfluenceFactorsProvider Provider_t;
public:
	DataModelAnalysisInfluenceFactors(boost::shared_ptr<DataModel_t> dataModel, Provider_t *provider);

	virtual ~DataModelAnalysisInfluenceFactors();

	static QString getStaticToolId() {
		return QString("ffcb67a0-bc2a-45ee-8e4a-b3eace7de57c");
	}

	void execute();

	bool supportsReporting();

	bool supportRawDataExport();

	void configurationUpdateRequested();

	int getAnEstimatedBehaviorIndicator();

	void estimateCurrentBehaviorIndicator();

	int getAnEstimatedInfluenceFactor();

	void estimateCurrentInfluenceFactor();

	void setCurrentBehaviorIndicator(int index);

	boost::shared_ptr<api::IDataModelAnalysisInfluenceFactors::Column_t> getCurrentBehaviorIndicatorColumn();

	void setCurrentInfluenceFactor(int index);

	boost::shared_ptr<api::IDataModelAnalysisInfluenceFactors::Column_t> getCurrentInfluenceFactorColumn();

	api::IDataModelAnalysisInfluenceFactors::ValueContainer_t getBehaviorIndicatorColumnValues();

	api::IDataModelAnalysisInfluenceFactors::ValueContainer_t getInfluenceFactorColumnValues();

	const api::IDataModelAnalysisInfluenceFactors::Behavior_Indicator_Column_Container_t& getBehaviorIndicatorColumns();

	const DataModel_t::Water_Level_Column_Container_t& getWaterLevelColumns();

	const DataModel_t::Temperature_Column_Container_t& getTemperatureColumns();

private:
	boost::shared_ptr<DataModel_t> dataModel;
	Provider_t *provider;
	boost::shared_ptr<r::api::IDataFrameInfluenceFactors> rDataFrameInfluenceFactors;
	int currentBehaviorIndicatorColumnIndex;
	int currentInfluenceIndicatorColumnIndex;
	COM_CONNECTION dataModelBehaviorIndicatorsChangedConnection;
	COM_CONNECTION dataModelWaterLevelAssignmentChangedConnection;
	COM_CONNECTION dataModelTemperatureAssignmentChangedConnection;
	api::IDataModelAnalysisInfluenceFactors::ValueContainer_t nullBehaviorIndicatorColumnValues;
	api::IDataModelAnalysisInfluenceFactors::ValueContainer_t nullInfluenceFactorColumnValues;
};


}} /* namespace dambase::model */


#endif /* DATAMODELANALYSISINFLUENCEFACTORS_HPP_ */
