/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelAnalysisTimeSeriesOverview.cpp
 *
 *  Created on: 18.02.2015
 *      Author: gerberma
 */

#include "DataModelAnalysisTimeSeriesOverview.hpp"

namespace dambase {
namespace model {


DataModelAnalysisTimeSeriesOverview::DataModelAnalysisTimeSeriesOverview(boost::shared_ptr<api::IDataModelAnalysisTimeSeriesOverview::DataModel_t> dataModel, Provider_t *provider)
	:dataModel(dataModel), provider(provider), valueColumnIndex(0)
{
	if(dataModel->getBehaviorIndicatorColumns().size() != 0) {
		valueColumnIndex = (*dataModel->getBehaviorIndicatorColumns().begin())->getIndex();
	}
}

DataModelAnalysisTimeSeriesOverview::~DataModelAnalysisTimeSeriesOverview(){;}

void
DataModelAnalysisTimeSeriesOverview::setValueColumn(int index) {
	valueColumnIndex = index;
	api::IDataModelAnalysisTimeSeriesOverview::fire();
}

boost::shared_ptr<api::IDataModelAnalysisTimeSeriesOverview::Column_t>
DataModelAnalysisTimeSeriesOverview::getValueColumn() {
	return dataModel->getColumnByIndex(valueColumnIndex);
}

QString
DataModelAnalysisTimeSeriesOverview::getValueColumnName() {
	return getValueColumn()->getName();
}

boost::shared_ptr<api::IDataModelAnalysisTimeSeriesOverview::DataModel_t>
DataModelAnalysisTimeSeriesOverview::getDataModel() {
	return dataModel;
}

api::IDataModelAnalysisTimeSeriesOverview::DateContainer_t &
DataModelAnalysisTimeSeriesOverview::getDates() {
	if(overview == NULL) {
		return nullDateContainer;
	}

	return overview->getDates();
}

api::IDataModelAnalysisTimeSeriesOverview::ValueContainer_t &
DataModelAnalysisTimeSeriesOverview::getValues() {
	if(overview == NULL) {
		return nullValueContainer;
	}

	return overview->getValues();
}

void
DataModelAnalysisTimeSeriesOverview::configurationUpdateRequested() {
	//test that valueColumnIndex is still a valid assigned column
	if(dataModel->isColumnAssignedWithIndex(valueColumnIndex)) {
		return;
	}


	//if not, select a new value column (which should fire a signal the gui subscribes for and calls updateValues on its own)
	boost::shared_ptr<api::IDataModelAnalysisTimeSeriesOverview::Column_t> anyAssignedColumn = dataModel->getAnyAssignedColumn();
	if(anyAssignedColumn != NULL) {
		setValueColumn(anyAssignedColumn->getIndex());
	}else{
		valueColumnIndex = -1;
	}
}

void
DataModelAnalysisTimeSeriesOverview::execute() {
	boost::shared_ptr<api::IDataModelAnalysisTimeSeriesOverview::Column_t> valueColumn = getValueColumn();
	if(valueColumn == NULL) {
		return;
	}

	overview = provider->load(dataModel->getDataFrame(), valueColumn);
	api::IDataModelAnalysisTimeSeriesOverview::fireLoaded();
}

bool
DataModelAnalysisTimeSeriesOverview::supportsReporting() {
	return false;
}

bool
DataModelAnalysisTimeSeriesOverview::supportRawDataExport() {
	return false;
}

} //namespace model
} //namespace dambase
