/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelAnalysisTimeSeriesOverview.hpp
 *
 *  Created on: Mar 11, 2014
 *      Author: gerberma
 */

#ifndef DATAMODELANALYSISTIMESERIESOVERVIEW_HPP_
#define DATAMODELANALYSISTIMESERIESOVERVIEW_HPP_

#include <boost/bind.hpp>
#include <boost/foreach.hpp>
#include <boost/shared_ptr.hpp>
#include "api/IDataModelAnalysisTimeSeriesOverview.hpp"
#include "r/api/IDataFrameTimeSeriesOverviewProvider.hpp"
#include <QString>
#include <QDebug>

namespace dambase {
namespace model {

class DataModelAnalysisTimeSeriesOverview : public api::IDataModelAnalysisTimeSeriesOverview
{
public:
	typedef r::api::IDataFrameTimeSeriesOverviewProvider Provider_t;
public:
	DataModelAnalysisTimeSeriesOverview(boost::shared_ptr<api::IDataModelAnalysisTimeSeriesOverview::DataModel_t> dataModel, Provider_t *provider);

	virtual ~DataModelAnalysisTimeSeriesOverview();

	static QString getStaticToolId() {
		return QString("2b64267d-e417-4525-9e6a-a49ba245f490");
	}

	void setValueColumn(int index);

	boost::shared_ptr<api::IDataModelAnalysisTimeSeriesOverview::Column_t> getValueColumn();

	QString getValueColumnName();

	boost::shared_ptr<api::IDataModelAnalysisTimeSeriesOverview::DataModel_t> getDataModel();

	api::IDataModelAnalysisTimeSeriesOverview::DateContainer_t &getDates();

	api::IDataModelAnalysisTimeSeriesOverview::ValueContainer_t &getValues();

	void configurationUpdateRequested();

	void execute();

	bool supportsReporting();

	bool supportRawDataExport();

private:
	boost::shared_ptr<api::IDataModelAnalysisTimeSeriesOverview::DataModel_t> dataModel;
	Provider_t *provider;
	boost::shared_ptr<r::api::IDataFrameTimeSeriesOverview> overview;
	api::IDataModelAnalysisTimeSeriesOverview::DateContainer_t nullDateContainer;
	api::IDataModelAnalysisTimeSeriesOverview::ValueContainer_t nullValueContainer;
	int valueColumnIndex;
};

}  // namespace model
}  // namespace dambase

#endif /* DATAMODELANALYSISTIMESERIESOVERVIEW_HPP_ */
