/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelAnalysisToolsDescription.cpp
 *
 *  Created on: 18.02.2015
 *      Author: gerberma
 */

#include <boost/foreach.hpp>
#include "DataModelAnalysisToolsDescription.hpp"

namespace dambase {
namespace model {

DataModelAnalysisToolsDescription::DataModelAnalysisToolsDescription(std::vector<boost::shared_ptr<api::IDataModelAnalysisToolsDescription::tool_description_t> > &overviewToolsDescription, std::vector<boost::shared_ptr<api::IDataModelAnalysisToolsDescription::tool_description_t> > &distributionToolsDescription, boost::shared_ptr<api::IDataModelAnalysisToolsDescription::ModeHandler_t> modeHandler)
	:overviewToolsDescription(overviewToolsDescription), distributionToolsDescription(distributionToolsDescription), modeHandler(modeHandler)
{
}

DataModelAnalysisToolsDescription::~DataModelAnalysisToolsDescription(){}

std::vector<boost::shared_ptr<api::IDataModelAnalysisToolsDescription::tool_description_t> > &
DataModelAnalysisToolsDescription::getOverviewAnalysisToolsDescription() {
	return overviewToolsDescription;
}

std::vector<boost::shared_ptr<api::IDataModelAnalysisToolsDescription::tool_description_t> > &
DataModelAnalysisToolsDescription::getDistributionAnalysisToolsDescription() {
	return distributionToolsDescription;
}

boost::shared_ptr<api::IDataModelAnalysisToolsDescription::tool_description_t>
DataModelAnalysisToolsDescription::findOverviewAnalysisToolDescriptionById(QString id){
	return findById(id, overviewToolsDescription);
}

boost::shared_ptr<api::IDataModelAnalysisToolsDescription::ModeHandler_t>
DataModelAnalysisToolsDescription::getModeHandler() {
	return modeHandler;
}

boost::shared_ptr<api::IDataModelAnalysisToolsDescription::tool_description_t>
DataModelAnalysisToolsDescription::findById(QString id, std::vector<boost::shared_ptr<api::IDataModelAnalysisToolsDescription::tool_description_t> > &descriptions){
	BOOST_FOREACH(boost::shared_ptr<api::IDataModelAnalysisToolsDescription::tool_description_t> analysisToolDescription, descriptions) {
		if(analysisToolDescription->getId() == id) {
			return analysisToolDescription;
		}
	}

	boost::shared_ptr<api::IDataModelAnalysisToolsDescription::tool_description_t> nullReturn;
	return nullReturn;
}

} //namespace model
} //namespace dambase

