/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelOverviewStatistics.cpp
 *
 *  Created on: 18.02.2015
 *      Author: gerberma
 */


#include <QDebug>
#include "DataModelOverviewStatistics.hpp"

namespace dambase {
namespace model {

DataModelOverviewStatistics::DataModelOverviewStatistics(boost::shared_ptr<DataModel_t> dataModel, Provider_t *overviewStatisticsProvider)
	:dataModel(dataModel), overviewStatisticsProvider(overviewStatisticsProvider)
{
}

DataModelOverviewStatistics::~DataModelOverviewStatistics() {
}

int
DataModelOverviewStatistics::getTotalNumberOfDataPoints() {
	if(rDataFrameOverviewStatistics == NULL) {
		return 0;
	}

	return rDataFrameOverviewStatistics->getTotalNumberOfDataPoints();
}

int
DataModelOverviewStatistics::getNumberOfNotAvailableDataPoints() {
	if(rDataFrameOverviewStatistics != NULL) {
		return rDataFrameOverviewStatistics->getNumberOfNotAvailableDataPoints();
	}else{
		return 0;
	}
}

int
DataModelOverviewStatistics::getNumberOfAvailableDataPoints() {
	if(rDataFrameOverviewStatistics != NULL) {
		return rDataFrameOverviewStatistics->getNumberOfAvailableDataPoints();
	}else{
		return 0;
	}
}

void
DataModelOverviewStatistics::configurationUpdateRequested() {
	qDebug() << "DataModelOverviewStatistics::" << __FUNCTION__;
}

void
DataModelOverviewStatistics::execute() {
	rDataFrameOverviewStatistics = overviewStatisticsProvider->load(dataModel->getDataFrame());
	api::IDataModelOverviewStatistics::fireLoaded();
}

bool
DataModelOverviewStatistics::supportsReporting() {
	return false;
}

bool
DataModelOverviewStatistics::supportRawDataExport() {
	return false;
}

} //namespace model
} //namespace dambase
