/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelOverviewStatistics.hpp
 *
 *  Created on: Jan 22, 2014
 *      Author: gerberma
 */

#ifndef DATAMODELOVERVIEWSTATISTICS_HPP_
#define DATAMODELOVERVIEWSTATISTICS_HPP_

#include <boost/bind.hpp>
#include <boost/shared_ptr.hpp>
#include "api/IDataModel.hpp"
#include "api/IDataModelOverviewStatistics.hpp"
#include "r/api/IDataFrameOverviewStatistics.hpp"
#include "r/api/IDataFrameOverviewStatisticsProvider.hpp"
#include <QString>
#include <QDebug>

namespace dambase {
namespace model {

class DataModelOverviewStatistics : public api::IDataModelOverviewStatistics {
public:
	typedef r::api::IDataFrameOverviewStatistics r_data_frame_overview_statistics_t;
	typedef r::api::IDataFrameOverviewStatisticsProvider Provider_t;
	typedef api::IDataModel DataModel_t;
public:
	DataModelOverviewStatistics(boost::shared_ptr<DataModel_t> dataModel, Provider_t *overviewStatisticsProvider);
	virtual ~DataModelOverviewStatistics();

	static QString getStaticToolId() {
		return QString("bfe6bbd1-be03-4f45-9480-95e81d458daa");
	}

	int getTotalNumberOfDataPoints();

	int getNumberOfNotAvailableDataPoints();

	// = getTotalNumberOfDataPoints() - getNumberOfNotAvailableDataPoints()
	int getNumberOfAvailableDataPoints();

	void configurationUpdateRequested();

	void execute();

	bool supportsReporting();

	bool supportRawDataExport();

private:
	boost::shared_ptr<DataModel_t> dataModel;
	Provider_t *overviewStatisticsProvider;
	boost::shared_ptr<r_data_frame_overview_statistics_t> rDataFrameOverviewStatistics;
};


}} /* namespace dambase::model */


#endif /* DATAMODELOVERVIEWSTATISTICS_HPP_ */
