/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DirectoryManager.cpp
 *
 *  Created on: 17.02.2015
 *      Author: gerberma
 */

#include <QDir>
#include <QStringList>
#include <boost/foreach.hpp>
#include "DirectoryManager.hpp"

namespace dambase {
namespace model {

DirectoryManager::DirectoryManager(QString temporaryDirectory)
	: temporaryDirectory(temporaryDirectory)
{;}

DirectoryManager::~DirectoryManager(){;}

QString
DirectoryManager::getTemporaryDirectory() {
	return temporaryDirectory;
}

void
DirectoryManager::cleanupTemporaryDirectory() {
	QDir dir;
	if(dir.exists(temporaryDirectory)) {
		removeRecursively(temporaryDirectory, false);
	}else{
		dir.mkpath(temporaryDirectory);
	}
}

bool
DirectoryManager::removeRecursively(QString directory, bool alsoRoot) {
	bool status = true;
	QDir dir(directory);

	BOOST_FOREACH(QFileInfo fileInfo, dir.entryInfoList(QDir::AllDirs | QDir::Hidden  | QDir::System | QDir::Files | QDir::NoDotAndDotDot, QDir::DirsFirst)) {
		if (fileInfo.isDir()) {
			status = removeRecursively(fileInfo.absoluteFilePath(), true);
		}
		else {
			status = QFile::remove(fileInfo.absoluteFilePath());
		}

		if (!status) {
			return status;
		}
	}

	if(alsoRoot) {
		status = dir.rmdir(directory);
	}

	return status;
}

} //namespace model
} //namespace dambase

