/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * GenericAnalysisTool.hpp
 *
 *  Created on: Jul 21, 2014
 *      Author: gerberma
 */

#ifndef GENERICANALYSISTOOL_HPP_
#define GENERICANALYSISTOOL_HPP_

#include <boost/shared_ptr.hpp>
#include <boost/enable_shared_from_this.hpp>
#include <QTextStream>
#include "ConfigurationUpdatePolicies.hpp"

namespace model {

template<
	typename T,
	typename ReportContext,
	typename ReportPolicy,
	typename RawPlotDataCollector,
	typename RawDataCollectionPolicy,
	typename ConfigurationUpdatePolicy = IgnoreConfigurationUpdatePolicy
> //, typename ReportPolicy>
class GenericAnalysisTool
		: public boost::enable_shared_from_this<GenericAnalysisTool<T, ReportContext, ReportPolicy, RawPlotDataCollector, RawDataCollectionPolicy, ConfigurationUpdatePolicy> >,
		  public T, ReportPolicy, RawDataCollectionPolicy, ConfigurationUpdatePolicy
{
public:
	GenericAnalysisTool(){;}
	virtual ~GenericAnalysisTool(){;}

	void build(ReportContext &rc) {
		//ReportPolicy::build(this, rc);
		//rc << "### GenericAnalysisTool" << endl << endl; //this is only temporarly until we delegate to a ReportPolicy here - also remove the include
		//do something
		ReportPolicy::build(boost::enable_shared_from_this<GenericAnalysisTool<T, ReportContext, ReportPolicy, RawPlotDataCollector, RawDataCollectionPolicy, ConfigurationUpdatePolicy> >::shared_from_this(), rc);
	}

	void build(boost::shared_ptr<RawPlotDataCollector> collector) {
		RawDataCollectionPolicy::build(boost::enable_shared_from_this<GenericAnalysisTool<T, ReportContext, ReportPolicy, RawPlotDataCollector, RawDataCollectionPolicy, ConfigurationUpdatePolicy> >::shared_from_this(), collector);
	}

	void setName(QString name) {
		this->name = name;
	}

	QString getName() {
		return name;
	}

	void configurationUpdateRequested() {
		ConfigurationUpdatePolicy::run(boost::enable_shared_from_this<GenericAnalysisTool<T, ReportContext, ReportPolicy, RawPlotDataCollector, RawDataCollectionPolicy, ConfigurationUpdatePolicy> >::shared_from_this());
	}

	bool supportsReporting() {
		return ReportPolicy::supportsReporting();
	}

	bool supportRawDataExport() {
		return RawDataCollectionPolicy::supportRawDataExport();
	}

private:
	QString name;
};

}  // namespace model


#endif /* GENERICANALYSISTOOL_HPP_ */
