/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * Model.cpp
 *
 *  Created on: 18.02.2015
 *      Author: gerberma
 */

#include <boost/foreach.hpp>
#include <stdexcept>
#include "Model.hpp"

namespace dambase {
namespace model {

Model::Model(boost::shared_ptr<api::IModel::ActivityTracker_t> activityTracker, boost::shared_ptr<api::IModel::AnalysisToolsDescriptions_t> analysisToolsDescriptions, std::vector<boost::shared_ptr<api::IModel::AssignmentCategory_t> > assignmentCategories, boost::shared_ptr<api::IModel::ModeSettings_t> modeSettings)
	:activityTracker(activityTracker), assignmentCategories(assignmentCategories), modeSettings(modeSettings), analysisToolsDescriptions(analysisToolsDescriptions)
{
}

Model::~Model(){std::cout << __FUNCTION__ << std::endl;}

boost::shared_ptr<api::IModel::Project_t>
Model::getProject() {
	return project;
}

void
Model::setProject(boost::shared_ptr<api::IModel::Project_t> project) {
	this->project = project;
	api::IModel::fire();
}

boost::shared_ptr<api::IModel::AnalysisToolsDescriptions_t::dataModel_analysis_tools_description_t>
Model::getDataModelAnalysisToolsDescription() {
	return analysisToolsDescriptions->getDataModelAnalysisToolsDescription();
}

boost::shared_ptr<api::IModel::AnalysisToolsDescriptions_t::regressionModel_analysis_tools_description_t>
Model::getRegressionModelAnalysisToolsDescription() {
	return analysisToolsDescriptions->getRegressionModelAnalysisToolsDescription();
}

boost::shared_ptr<api::IModel::ModeSettings_t>
Model::getModeSettings() {
	return modeSettings;
}

boost::shared_ptr<api::IModel::ActivityTracker_t>
Model::getActivityTracker() {
	return activityTracker;
}

std::vector<boost::shared_ptr<api::IModel::AssignmentCategory_t> > &
Model::getAssignmentCategories() {
	return assignmentCategories;
}

/**
 * This is primarly used by the deserialization process to keep constness
 */
const std::vector<boost::shared_ptr<api::IModel::AssignmentCategory_t> > &
Model::getAssignmentCategoriesConst() const{
	return assignmentCategories;
}

boost::shared_ptr<api::IModel::AssignmentCategory_t>
Model::findAssignmentCategoryByIdentifier(QString identifier) {
	BOOST_FOREACH(boost::shared_ptr<api::IModel::AssignmentCategory_t> assignmentCategory, assignmentCategories) {
		if(assignmentCategory->getIdentifier() == identifier) {
			return assignmentCategory;
		}
	}

	throw std::logic_error("Model: no assignment category found with identifier " + identifier.toStdString());
}

bool
Model::isLoadingProjectAllowed() {
	return project == NULL;
}

bool
Model::isCreatingNewProjectAllowed() {
	return project == NULL;
}

} //namespace model
} //namespace dambase
