/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * Model.hpp
 *
 *  Created on: Nov 21, 2013
 *      Author: gerberma
 */

#ifndef MODEL_HPP_
#define MODEL_HPP_

#include <boost/shared_ptr.hpp>
#include <iostream>
#include <vector>
#include "api/IModel.hpp"

namespace dambase {
namespace model {

class Model : public api::IModel {
public:
	Model(boost::shared_ptr<api::IModel::ActivityTracker_t> activityTracker, boost::shared_ptr<api::IModel::AnalysisToolsDescriptions_t> analysisToolsDescriptions, std::vector<boost::shared_ptr<api::IModel::AssignmentCategory_t> > assignmentCategories, boost::shared_ptr<api::IModel::ModeSettings_t> modeSettings);

	virtual ~Model();

	boost::shared_ptr<api::IModel::Project_t> getProject();

	void setProject(boost::shared_ptr<api::IModel::Project_t> project);

	boost::shared_ptr<api::IModel::AnalysisToolsDescriptions_t::dataModel_analysis_tools_description_t> getDataModelAnalysisToolsDescription();

	boost::shared_ptr<api::IModel::AnalysisToolsDescriptions_t::regressionModel_analysis_tools_description_t> getRegressionModelAnalysisToolsDescription();

	boost::shared_ptr<api::IModel::ModeSettings_t> getModeSettings();

	boost::shared_ptr<api::IModel::ActivityTracker_t> getActivityTracker();

	std::vector<boost::shared_ptr<api::IModel::AssignmentCategory_t> > &getAssignmentCategories();

	/**
	 * This is primarly used by the deserialization process to keep constness
	 */
	const std::vector<boost::shared_ptr<api::IModel::AssignmentCategory_t> > &getAssignmentCategoriesConst() const;

	boost::shared_ptr<api::IModel::AssignmentCategory_t> findAssignmentCategoryByIdentifier(QString identifier);

	bool isLoadingProjectAllowed();

	bool isCreatingNewProjectAllowed();

private:
	boost::shared_ptr<api::IModel::ActivityTracker_t> activityTracker;
	boost::shared_ptr<api::IModel::Project_t> project;
	boost::shared_ptr<api::IModel::AnalysisToolsDescriptions_t> analysisToolsDescriptions;
	std::vector<boost::shared_ptr<api::IModel::AssignmentCategory_t> > assignmentCategories;
	boost::shared_ptr<api::IModel::ModeSettings_t> modeSettings;
};

}} /* namespace dambase::model */


#endif /* MODEL_HPP_ */
