/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * Project.cpp
 *
 *  Created on: 17.02.2015
 *      Author: gerberma
 */

#include <iostream>
#include <algorithm>
#include <boost/bind.hpp>
#include <boost/foreach.hpp>
#include "Project.hpp"

namespace dambase {
namespace model {


Project::Project(){;}

Project::Project(boost::shared_ptr<api::IProject::ModeHandler_t> modeHandler)
	:modeHandler(modeHandler)
{
}

Project::~Project() {std::cout << __FUNCTION__ << std::endl;}

void
Project::setName(QString name) {
	this->name = name;
	api::IProject::fire();
}

const QString &
Project::getName() const {
	return name;
}

void
Project::setDescription(QString description) {
	this->description = description;
	api::IProject::fire();
}

const QString &
Project::getDescription() const {
	return description;
}

void
Project::setDamOperator(QString damOperator) {
	this->damOperator = damOperator;
	api::IProject::fire();
}

const QString &
Project::getDamOperator() const {
	return damOperator;
}

void
Project::setAnalyses(const std::vector<boost::shared_ptr<api::IProject::Analysis_t> > &newAnalyses) {
	BOOST_FOREACH(boost::shared_ptr<api::IProject::Analysis_t> analysis, newAnalyses) {
		addAnalysis(analysis);
	}
}

void
Project::addAnalysis(boost::shared_ptr<api::IProject::Analysis_t> analysis) {
	analyses.push_back(analysis);
	api::IProject::fireAnalysisAdded(analysis);
}

const std::vector<boost::shared_ptr<api::IProject::Analysis_t> > &
Project::getAnalyses() const {
	return analyses;
}

void
Project::removeAnalysis(boost::shared_ptr<api::IProject::Analysis_t> analysis) {
	analyses.erase(std::remove(analyses.begin(), analyses.end(), analysis));
	api::IProject::fireRemoveAnalysis(analysis);
}

bool
Project::isNewAnalysisAllowed() {
	return ProjectActionRules::isNewAnalysisAllowedFor(this);
}

void
Project::setModeHandler(boost::shared_ptr<api::IProject::ModeHandler_t> modeHandler){
	this->modeHandler = modeHandler;
}

bool
Project::isScientific() {
	return modeHandler->isScientific();
}

} //namespace model
} //namespace dambase

