/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * Project.hpp
 *
 *  Created on: Nov 5, 2013
 *      Author: gerberma
 */

#ifndef PROJECT_HPP_
#define PROJECT_HPP_

#include <vector>
#include <boost/shared_ptr.hpp>
#include "ProjectActionRules.hpp"
#include "api/IProject.hpp"
#include "SignalPolicies.hpp"
#include "communication/Signals.hpp"

namespace dambase {
namespace model {

class Project : public api::IProject , ProjectActionRules {
public:
	Project();

	Project(boost::shared_ptr<api::IProject::ModeHandler_t> modeHandler);

	virtual ~Project();

	void setName(QString name);

	const QString &getName() const;

	void setDescription(QString description);

	const QString &getDescription() const;

	void setDamOperator(QString damOperator);

	const QString &getDamOperator() const;

	void setAnalyses(const std::vector<boost::shared_ptr<api::IProject::Analysis_t> > &newAnalyses);

	void addAnalysis(boost::shared_ptr<api::IProject::Analysis_t> analysis);

	const std::vector<boost::shared_ptr<api::IProject::Analysis_t> >& getAnalyses() const;

	void removeAnalysis(boost::shared_ptr<api::IProject::Analysis_t> analysis);

	bool isNewAnalysisAllowed();

	void setModeHandler(boost::shared_ptr<api::IProject::ModeHandler_t> modeHandler);

	bool isScientific();


private:
	QString name;
	QString description;
	QString damOperator;
	std::vector<boost::shared_ptr<api::IProject::Analysis_t> > analyses;
	boost::shared_ptr<api::IProject::ModeHandler_t> modeHandler;
};

}} /* namespace dambase::model */
#endif /* PROJECT_HPP_ */
