/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RawDataCollector.hpp
 *
 *  Created on: 07.10.2014
 *      Author: gerberma
 */

#ifndef RAWDATACOLLECTOR_HPP_
#define RAWDATACOLLECTOR_HPP_

#include <map>
#include <vector>
#include <string>
#include <algorithm>
#include <cmath>
#include <climits>
#include <boost/foreach.hpp>
#include <boost/shared_ptr.hpp>
#include <QTextStream>
#include <QFile>
#include <QObject>
#include <QDebug>
#include <QDateTime>

#include "api/IRawDataCollector.hpp"
#include "api/IProject.hpp"
#include "api/IAnalysis.hpp"
#include "api/IRegressionModel.hpp"

namespace dambase {
namespace model {

class RawDataCollector : public virtual api::IRawDataCollector {
private:

public:
	RawDataCollector();
	virtual ~RawDataCollector();

	void addStatisticsValueColumn(QString name, std::vector<std::string> dates, std::vector<double> values);

	void writeHeader(QFile *outputFile, boost::shared_ptr<IRawDataCollector::Project_t> project, boost::shared_ptr<IRawDataCollector::Analysis_t> analysis, boost::shared_ptr<IRawDataCollector::RegressionModel_t> regressionModel);

	void write(QFile *outputFile);

	QString getSupportedFileTypesAsString();

	boost::shared_ptr<RawDataCollector::Collection_t> createNewCollection(QString name, QString keyName);


private:
	std::map<QString, std::vector<DataRow> > collection; //key: date; values DataRow
	std::vector<boost::shared_ptr<Collection_t> > individualCollections;
};

}  // namespace model
}  // namespace dambase


#endif /* RAWDATACOLLECTOR_HPP_ */
