/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitAnalysisACF.cpp
 *
 *  Created on: 17.02.2015
 *      Author: gerberma
 */


#include "RegressionFitAnalysisACF.hpp"

namespace dambase {
namespace model {

RegressionFitAnalysisACF::RegressionFitAnalysisACF():provider(NULL){;}

RegressionFitAnalysisACF::RegressionFitAnalysisACF(boost::shared_ptr<RegressionModel_t> regressionModel, Provider_t *provider)
	:regressionModel(regressionModel), provider(provider)
{;}


api::IRegressionFitAnalysisACF::AcfContainer_t &
RegressionFitAnalysisACF::getAcfs() {
	if(acf == NULL) {
		return nullAcfContainer;
	}

	return acf->getAcfs();
}

api::IRegressionFitAnalysisACF::LagContainer_t &
RegressionFitAnalysisACF::getLags() {
	if(acf == NULL) {
		return nullLagContainer;
	}

	return acf->getLags();
}

double
RegressionFitAnalysisACF::getLowerCi() {
	if(acf == NULL) {
		return 0.0;
	}

	return acf->getLowerCi();
}

double
RegressionFitAnalysisACF::getUpperCi() {
	if(acf == NULL) {
		return 0.0;
	}

	return acf->getUpperCi();
}

void
RegressionFitAnalysisACF::execute() {
	acf = provider->load(regressionModel->getRegressionFit());
	api::IRegressionFitAnalysisACF::fireLoaded();
}

void
RegressionFitAnalysisACF::setRegressionModel(boost::shared_ptr<RegressionModel_t> regressionModel) {
	this->regressionModel = regressionModel;
}

void
RegressionFitAnalysisACF::setProvider(Provider_t *provider) {
	this->provider = provider;
}

void
RegressionFitAnalysisACF::build(api::IRegressionFitAnalysisACF::ReportContext_t &rc) {
	//ReportPolicy::build(this, rc);
	//rc << "### GenericAnalysisTool" << endl << endl; //this is only temporarly until we delegate to a ReportPolicy here - also remove the include
	//do something

	//ExternalReportPolicy::build(boost::enable_shared_from_this<api::IRegressionFitAnalysisACF>::shared_from_this(), rc);
	ExternalReportPolicy::build(boost::enable_shared_from_this<api::IRegressionFitAnalysisACF>::shared_from_this(), rc);
}

void
RegressionFitAnalysisACF::build(boost::shared_ptr<api::IRegressionFitAnalysisACF::RawDataCollector_t> collector) {
	ExternalRawDataCollectionPolicy::build(boost::enable_shared_from_this<api::IRegressionFitAnalysisACF>::shared_from_this(), collector);
}

void
RegressionFitAnalysisACF::setName(QString name) {
	this->name = name;
}

QString
RegressionFitAnalysisACF::getName() {
	return this->name;
}

void
RegressionFitAnalysisACF::configurationUpdateRequested() {
	//ConfigurationUpdatePolicy::run(boost::enable_shared_from_this<GenericAnalysisTool<T, ReportContext, ReportPolicy, RawPlotDataCollector, RawDataCollectionPolicy, ConfigurationUpdatePolicy> >::shared_from_this());
}

bool
RegressionFitAnalysisACF::supportsReporting() {
	return ExternalReportPolicy::supportsReporting();
}

bool
RegressionFitAnalysisACF::supportRawDataExport() {
	return ExternalRawDataCollectionPolicy::supportRawDataExport();
}

} //namespace model
} //namespace dambase


