/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitAnalysisACF.hpp
 *
 *  Created on: Apr 9, 2014
 *      Author: gerberma
 */

#ifndef REGRESSIONFITANALYSISACF_HPP_
#define REGRESSIONFITANALYSISACF_HPP_

#include <boost/shared_ptr.hpp>
#include "ReportPolicies.hpp"
#include "RawDataCollectionPolicies.hpp"
#include "SignalPolicies.hpp"
#include "r/api/IRegressionFitAnalysisACFProvider.hpp"
#include "r/api/IRRegressionFitAnalysisACF.hpp"
#include "api/IRegressionFitAnalysisACF.hpp"
#include "api/IRegressionModel.hpp"

namespace dambase {
namespace model {

class RegressionFitAnalysisACF : public api::IRegressionFitAnalysisACF, ExternalReportPolicy, ExternalRawDataCollectionPolicy
{
public:
	typedef r::api::IRegressionFitAnalysisACFProvider Provider_t;
	typedef api::IRegressionModel RegressionModel_t;
public:
	RegressionFitAnalysisACF();
	RegressionFitAnalysisACF(boost::shared_ptr<RegressionModel_t> regressionModel, Provider_t *provider);

	static QString getStaticToolId() {
		return QString("72840b98-a45a-4aee-ab95-3289cfcdbaee");
	}

	api::IRegressionFitAnalysisACF::AcfContainer_t &getAcfs();

	api::IRegressionFitAnalysisACF::LagContainer_t &getLags();

	double getLowerCi();

	double getUpperCi();

	void execute();

	void setRegressionModel(boost::shared_ptr<RegressionModel_t> regressionModel);

	void setProvider(Provider_t *provider);

	void build(api::IRegressionFitAnalysisACF::ReportContext_t &rc);

	void build(boost::shared_ptr<api::IRegressionFitAnalysisACF::RawDataCollector_t> collector);

	void setName(QString name);

	QString getName();

	void configurationUpdateRequested();

	bool supportsReporting();

	bool supportRawDataExport();

private:
	QString name;
	boost::shared_ptr<RegressionModel_t> regressionModel;
	Provider_t *provider;
	boost::shared_ptr<r::api::IRRegressionFitAnalysisACF> acf;
	api::IRegressionFitAnalysisACF::AcfContainer_t nullAcfContainer;
	api::IRegressionFitAnalysisACF::LagContainer_t nullLagContainer;
};

}  // namespace model
}  // namespace dambase


#endif /* REGRESSIONFITANALYSISACF_HPP_ */
