/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitAnalysisOverallRegressionQuality.cpp
 *
 *  Created on: 17.02.2015
 *      Author: gerberma
 */

#include "RegressionFitAnalysisOverallRegressionQuality.hpp"

namespace dambase {
namespace model {

RegressionFitAnalysisOverallRegressionQuality::RegressionFitAnalysisOverallRegressionQuality() : provider(NULL) {;}
RegressionFitAnalysisOverallRegressionQuality::RegressionFitAnalysisOverallRegressionQuality(boost::shared_ptr<api::IRegressionFitAnalysisOverallRegressionQuality::RegressionModel_t> regressionModel, Provider_t *provider)
	:regressionModel(regressionModel), provider(provider)
{;}

RegressionFitAnalysisOverallRegressionQuality::~RegressionFitAnalysisOverallRegressionQuality(){;}


void
RegressionFitAnalysisOverallRegressionQuality::execute() {
	t = provider->load(regressionModel->getRegressionFit());
	api::IRegressionFitAnalysisOverallRegressionQuality::fireLoaded();
}

int
RegressionFitAnalysisOverallRegressionQuality::getAutocorrelationDegree() {
	if(t == NULL) {
		return nan(NULL);
	}

	return t->getAutocorDwStatDegree();
}

double
RegressionFitAnalysisOverallRegressionQuality::getAutocorrelationValue() {
	if(t == NULL) {
		return nan(NULL);
	}

	return t->getAutocorDwStatValue();
}

int
RegressionFitAnalysisOverallRegressionQuality::getFitDegree() {
	if(t == NULL) {
		return nan(NULL);
	}

	return t->getFitR2adjDegree();
}

double
RegressionFitAnalysisOverallRegressionQuality::getFitValue() {
	if(t == NULL) {
		return nan(NULL);
	}

	return t->getFitR2adjValue();
}

void
RegressionFitAnalysisOverallRegressionQuality::setProvider(Provider_t *provider) {
	this->provider = provider;
}

void
RegressionFitAnalysisOverallRegressionQuality::setRegressionModel(boost::shared_ptr<api::IRegressionFitAnalysisOverallRegressionQuality::RegressionModel_t> regressionModel) {
	this->regressionModel = regressionModel;
}

void
RegressionFitAnalysisOverallRegressionQuality::build(api::IRegressionFitAnalysisOverallRegressionQuality::ReportContext_t &rc) {
	//ReportPolicy::build(this, rc);
	//rc << "### GenericAnalysisTool" << endl << endl; //this is only temporarly until we delegate to a ReportPolicy here - also remove the include
	//do something
	ExternalReportPolicy::build(boost::enable_shared_from_this<api::IRegressionFitAnalysisOverallRegressionQuality>::shared_from_this(), rc);
}

void
RegressionFitAnalysisOverallRegressionQuality::build(boost::shared_ptr<api::IRegressionFitAnalysisOverallRegressionQuality::RawDataCollector_t> collector) {
	//ExternalRawDataCollectionPolicy::build(boost::enable_shared_from_this<api::IRegressionFitAnalysisOverallRegressionQuality>::shared_from_this(), collector);
}

void
RegressionFitAnalysisOverallRegressionQuality::setName(QString name) {
	this->name = name;
}

QString
RegressionFitAnalysisOverallRegressionQuality::getName() {
	return this->name;
}

void
RegressionFitAnalysisOverallRegressionQuality::configurationUpdateRequested() {
	//ConfigurationUpdatePolicy::run(boost::enable_shared_from_this<GenericAnalysisTool<T, ReportContext, ReportPolicy, RawPlotDataCollector, RawDataCollectionPolicy, ConfigurationUpdatePolicy> >::shared_from_this());
}

bool
RegressionFitAnalysisOverallRegressionQuality::supportsReporting() {
	return ExternalReportPolicy::supportsReporting();
}

bool
RegressionFitAnalysisOverallRegressionQuality::supportRawDataExport() {
	return ExternalRawDataCollectionPolicy::supportRawDataExport();
}


} //namespace model
} //namespace dambase
