/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitAnalysisOverallRegressionQuality.hpp
 *
 *  Created on: Apr 16, 2014
 *      Author: gerberma
 */

#ifndef REGRESSIONFITANALYSISOVERALLREGRESSIONQUALITY_HPP_
#define REGRESSIONFITANALYSISOVERALLREGRESSIONQUALITY_HPP_

#include <boost/shared_ptr.hpp>
#include <boost/enable_shared_from_this.hpp>
#include "ReportPolicies.hpp"
#include "RawDataCollectionPolicies.hpp"
#include "SignalPolicies.hpp"
#include "r/api/IRegressionFitAnalysisOverallRegressionQualityProvider.hpp"
#include "r/api/IRRegressionFitAnalysisOverallRegressionQuality.hpp"
#include "api/IRegressionFitAnalysisOverallRegressionQuality.hpp"

namespace dambase {
namespace model {

class RegressionFitAnalysisOverallRegressionQuality : public api::IRegressionFitAnalysisOverallRegressionQuality, ExternalReportPolicy, ExternalRawDataCollectionPolicy {
public:
	typedef r::api::IRegressionFitAnalysisOverallRegressionQualityProvider Provider_t;
public:
	RegressionFitAnalysisOverallRegressionQuality();
	RegressionFitAnalysisOverallRegressionQuality(boost::shared_ptr<api::IRegressionFitAnalysisOverallRegressionQuality::RegressionModel_t> regressionModel, Provider_t *provider);

	virtual ~RegressionFitAnalysisOverallRegressionQuality();

	static QString getStaticToolId() {
		return QString("42b32d6a-89e2-42b5-95fd-26fca3bdfa64");
	}

	void execute();

	int getAutocorrelationDegree();

	double getAutocorrelationValue();

	int getFitDegree();

	double getFitValue();

	void setProvider(Provider_t *provider);

	void setRegressionModel(boost::shared_ptr<api::IRegressionFitAnalysisOverallRegressionQuality::RegressionModel_t> regressionModel);

	void build(api::IRegressionFitAnalysisOverallRegressionQuality::ReportContext_t &rc);

	void build(boost::shared_ptr<api::IRegressionFitAnalysisOverallRegressionQuality::RawDataCollector_t> collector);

	void setName(QString name);

	QString getName();

	void configurationUpdateRequested();

	bool supportsReporting();

	bool supportRawDataExport();

private:
	QString name;
	boost::shared_ptr<api::IRegressionFitAnalysisOverallRegressionQuality::RegressionModel_t> regressionModel;
	Provider_t *provider;
	boost::shared_ptr<r::api::IRRegressionFitAnalysisOverallRegressionQuality> t;
};

}  // namespace model
}  // namespace dambase


#endif /* REGRESSIONFITANALYSISOVERALLREGRESSIONQUALITY_HPP_ */
