/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitAnalysisPhysicalBehavior.hpp
 *
 *  Created on: 21.10.2014
 *      Author: gerberma
 */

#ifndef REGRESSIONFITANALYSISPHYSICALBEHAVIOR_HPP_
#define REGRESSIONFITANALYSISPHYSICALBEHAVIOR_HPP_

#include <stdexcept>
#include <vector>
#include <boost/bind.hpp>
#include <boost/foreach.hpp>
#include <boost/shared_ptr.hpp>
#include "ReportPolicies.hpp"
#include "RawDataCollectionPolicies.hpp"
#include "r/api/IRRegressionFitAnalysisPhysicalBehavior.hpp"
#include "r/api/IRegressionFitAnalysisPhysicalBehaviorProvider.hpp"
#include "api/IRegressionFitAnalysisPhysicalBehavior.hpp"
#include "api/IRegressionModel.hpp"

namespace dambase {
namespace model {

class RegressionFitAnalysisPhysicalBehavior : public api::IRegressionFitAnalysisPhysicalBehavior, ExternalReportPolicy, ExternalRawDataCollectionPolicy
{
public:
	typedef api::IRegressionModel RegressionModel_t;
	typedef r::api::IRegressionFitAnalysisPhysicalBehaviorProvider Provider_t;
public:
	RegressionFitAnalysisPhysicalBehavior();

	RegressionFitAnalysisPhysicalBehavior(
			boost::shared_ptr<RegressionModel_t> regressionModel,
			std::vector<boost::shared_ptr<api::IRegressionFitAnalysisPhysicalBehavior::AssignmentCategory_t> > assignmentCategories,
			boost::shared_ptr<api::IRegressionFitAnalysisPhysicalBehavior::AssignmentCategory_t> assignmentCategory,
			Provider_t *provider);

	virtual ~RegressionFitAnalysisPhysicalBehavior();

	void setProvider(Provider_t *provider);

	void setRegressionModel(boost::shared_ptr<RegressionModel_t> regressionModel);

	static QString getStaticToolId() {
		return QString("4c29d12f-928f-4772-9d0b-f96e80316536");
	}

	std::vector<boost::shared_ptr<r::api::IRRegressionFitAnalysisPhysicalBehavior> > getItems();

	void setAssignmentCategories(std::vector<boost::shared_ptr<api::IRegressionFitAnalysisPhysicalBehavior::AssignmentCategory_t> > assignmentCategories);

	std::vector<boost::shared_ptr<api::IRegressionFitAnalysisPhysicalBehavior::AssignmentCategory_t> > getAssignmentCategories();

	void setAssignmentCategory(boost::shared_ptr<api::IRegressionFitAnalysisPhysicalBehavior::AssignmentCategory_t> assignmentCategory);

	boost::shared_ptr<api::IRegressionFitAnalysisPhysicalBehavior::AssignmentCategory_t> getAssignmentCategory();

	void execute();

	boost::shared_ptr<api::IRegressionFitAnalysisPhysicalBehavior::AssignmentCategory_t> findAssignmentCategoryByIdentifier(QString identifier);

	void build(api::IRegressionFitAnalysisPhysicalBehavior::ReportContext_t &rc);

	void build(boost::shared_ptr<api::IRegressionFitAnalysisPhysicalBehavior::RawDataCollector_t> collector);

	void setName(QString name);

	QString getName();

	void configurationUpdateRequested();

	bool supportsReporting();

	bool supportRawDataExport();

private:
	QString name;
	boost::shared_ptr<RegressionModel_t> regressionModel;
	Provider_t *provider;
	std::vector<boost::shared_ptr<r::api::IRRegressionFitAnalysisPhysicalBehavior> > items;
	boost::shared_ptr<api::IRegressionFitAnalysisPhysicalBehavior::AssignmentCategory_t> assignmentCategory;
	std::vector<boost::shared_ptr<api::IRegressionFitAnalysisPhysicalBehavior::AssignmentCategory_t> > assignmentCategories;
};

}  // namespace model
}  // namespace dambase


#endif /* REGRESSIONFITANALYSISPHYSICALBEHAVIOR_HPP_ */
