/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitAnalysisPhysicalSummary.cpp
 *
 *  Created on: 17.02.2015
 *      Author: gerberma
 */

#include "RegressionFitAnalysisPhysicalSummary.hpp"

namespace dambase {
namespace model {

RegressionFitAnalysisPhysicalSummary::RegressionFitAnalysisPhysicalSummary():provider(NULL){;}

RegressionFitAnalysisPhysicalSummary::RegressionFitAnalysisPhysicalSummary(boost::shared_ptr<api::IRegressionFitAnalysisPhysicalSummary::RegressionModel_t> regressionModel, Provider_t *regressionFitSummaryProvider)
		:regressionModel(regressionModel), provider(regressionFitSummaryProvider){;}

RegressionFitAnalysisPhysicalSummary::~RegressionFitAnalysisPhysicalSummary(){;}



api::IRegressionFitAnalysisPhysicalSummary::BehaviorIndicatorValueContainer_t
RegressionFitAnalysisPhysicalSummary::getBehaviorIndicatorValues() {
	if(summary == NULL) {
		return nullBehaviorIndicatorValueContainer;
	}

	return summary->getBehaviorIndicatorValues();
}

api::IRegressionFitAnalysisPhysicalSummary::PredictionsContainer_t
RegressionFitAnalysisPhysicalSummary::getPredictions() {
	if(summary == NULL) {
		return nullPredictionsContainer;
	}

	return summary->getPredictions();
}

api::IRegressionFitAnalysisPhysicalSummary::ConfidenceIntervalContainer_t
RegressionFitAnalysisPhysicalSummary::getUpperCi() {
	if(summary == NULL) {
		return numericContainer;
	}

	return summary->getUpperCi();
}

api::IRegressionFitAnalysisPhysicalSummary::ConfidenceIntervalContainer_t
RegressionFitAnalysisPhysicalSummary::getLowerCi() {
	if(summary == NULL) {
		return numericContainer;
	}

	return summary->getLowerCi();
}

api::IRegressionFitAnalysisPhysicalSummary::OutOfLimitContainer_t
RegressionFitAnalysisPhysicalSummary::getOutOfLimit() {
	if(summary == NULL) {
		return numericContainer;
	}

	return summary->getOutOfLimit();
}

api::IRegressionFitAnalysisPhysicalSummary::DateContainer_t &
RegressionFitAnalysisPhysicalSummary::getOutOfLimitDates() {
	if(summary == NULL) {
		return nullDateContainer;
	}

	return summary->getOutOfLimitDates();
}

api::IRegressionFitAnalysisPhysicalSummary::ExtrapolationsContainer_t
RegressionFitAnalysisPhysicalSummary::getExtrapolations() {
	if(summary == NULL) {
		return numericContainer;
	}

	return summary->getExtrapolations();
}

api::IRegressionFitAnalysisPhysicalSummary::DateContainer_t
RegressionFitAnalysisPhysicalSummary::getExtrapolationDates() {
	if(summary == NULL) {
		return nullDateContainer;
	}

	return summary->getExtrapolationDates();
}

api::IRegressionFitAnalysisPhysicalSummary::DateContainer_t
RegressionFitAnalysisPhysicalSummary::getDates() {
	if(summary == NULL) {
		return nullDateContainer;
	}

	return summary->getDates();
}

api::IRegressionFitAnalysisPhysicalSummary::DateContainer_t
RegressionFitAnalysisPhysicalSummary::getPredictionDates() {
	if(summary == NULL) {
		return nullDateContainer;
	}

	return summary->getPredictionDates();
}

QString
RegressionFitAnalysisPhysicalSummary::getBehaviorIndicatorName() {
	return regressionModel->getBehaviorIndicatorColumn()->getName();
}

api::IRegressionFitAnalysisPhysicalSummary::Date_t
RegressionFitAnalysisPhysicalSummary::getRegressionStartDate() {
	if(summary == NULL) {
		return "";
	}

	return summary->getRegressionStartDate();
}

api::IRegressionFitAnalysisPhysicalSummary::Date_t
RegressionFitAnalysisPhysicalSummary::getRegressionEndDate() {
	if(summary == NULL) {
		return "";
	}

	return summary->getRegressionEndDate();
}

void
RegressionFitAnalysisPhysicalSummary::execute() {
	if(regressionModel == NULL || regressionModel->getRegressionFit() == NULL) {
		throw std::logic_error("no regression model defined to calculate regression fit pysical summary");
	}


	summary = provider->load(regressionModel->getRegressionFit());
	api::IRegressionFitAnalysisPhysicalSummary::fireLoaded();
}

void
RegressionFitAnalysisPhysicalSummary::setProvider(Provider_t *provider) {
	this->provider = provider;
}

void
RegressionFitAnalysisPhysicalSummary::setRegressionModel(boost::shared_ptr<api::IRegressionFitAnalysisPhysicalSummary::RegressionModel_t> regressionModel) {
	this->regressionModel = regressionModel;
}

void
RegressionFitAnalysisPhysicalSummary::build(api::IRegressionFitAnalysisPhysicalSummary::ReportContext_t &rc) {
	//ReportPolicy::build(this, rc);
	//rc << "### GenericAnalysisTool" << endl << endl; //this is only temporarly until we delegate to a ReportPolicy here - also remove the include
	//do something
	ExternalReportPolicy::build(boost::enable_shared_from_this<api::IRegressionFitAnalysisPhysicalSummary>::shared_from_this(), rc);
}

void
RegressionFitAnalysisPhysicalSummary::build(boost::shared_ptr<api::IRegressionFitAnalysisPhysicalSummary::RawDataCollector_t> collector) {
	ExternalRawDataCollectionPolicy::build(boost::enable_shared_from_this<api::IRegressionFitAnalysisPhysicalSummary>::shared_from_this(), collector);
}

void
RegressionFitAnalysisPhysicalSummary::setName(QString name) {
	this->name = name;
}

QString
RegressionFitAnalysisPhysicalSummary::getName() {
	return this->name;
}

void
RegressionFitAnalysisPhysicalSummary::configurationUpdateRequested() {
	//ConfigurationUpdatePolicy::run(boost::enable_shared_from_this<GenericAnalysisTool<T, ReportContext, ReportPolicy, RawPlotDataCollector, RawDataCollectionPolicy, ConfigurationUpdatePolicy> >::shared_from_this());
}

bool
RegressionFitAnalysisPhysicalSummary::supportsReporting() {
	return ExternalReportPolicy::supportsReporting();
}

bool
RegressionFitAnalysisPhysicalSummary::supportRawDataExport() {
	return ExternalRawDataCollectionPolicy::supportRawDataExport();
}

} //namespace model
} //namespace dambase
