/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitAnalysisPhysicalSummary.hpp
 *
 *  Created on: Mar 10, 2014
 *      Author: gerberma
 */

#ifndef REGRESSIONFITANALYSISPHYSICALSUMMARY_HPP_
#define REGRESSIONFITANALYSISPHYSICALSUMMARY_HPP_

#include <stdexcept>
#include <boost/foreach.hpp>
#include <boost/bind.hpp>
#include <boost/shared_ptr.hpp>
#include "ReportPolicies.hpp"
#include "RawDataCollectionPolicies.hpp"
#include "api/IRegressionFitAnalysisPhysicalSummary.hpp"
#include "r/api/IRRegressionFitAnalysisPhysicalSummary.hpp"
#include "r/api/IRegressionFitAnalysisPhysicalSummaryProvider.hpp"
#include <QString>

namespace dambase {
namespace model {

class RegressionFitAnalysisPhysicalSummary : public api::IRegressionFitAnalysisPhysicalSummary, ExternalReportPolicy, ExternalRawDataCollectionPolicy {
public:
	typedef r::api::IRegressionFitAnalysisPhysicalSummaryProvider Provider_t;
public:
	RegressionFitAnalysisPhysicalSummary();

	RegressionFitAnalysisPhysicalSummary(boost::shared_ptr<api::IRegressionFitAnalysisPhysicalSummary::RegressionModel_t> regressionModel, Provider_t *regressionFitSummaryProvider);

	virtual ~RegressionFitAnalysisPhysicalSummary();

	static QString getStaticToolId() {
		return QString("5e6ea5c6-99b4-488f-9251-c1cf6dacf267");
	}

	api::IRegressionFitAnalysisPhysicalSummary::BehaviorIndicatorValueContainer_t getBehaviorIndicatorValues();

	api::IRegressionFitAnalysisPhysicalSummary::PredictionsContainer_t getPredictions();

	api::IRegressionFitAnalysisPhysicalSummary::ConfidenceIntervalContainer_t getUpperCi();

	api::IRegressionFitAnalysisPhysicalSummary::ConfidenceIntervalContainer_t getLowerCi();

	api::IRegressionFitAnalysisPhysicalSummary::OutOfLimitContainer_t getOutOfLimit();

	api::IRegressionFitAnalysisPhysicalSummary::DateContainer_t &getOutOfLimitDates();

	api::IRegressionFitAnalysisPhysicalSummary::ExtrapolationsContainer_t getExtrapolations();

	api::IRegressionFitAnalysisPhysicalSummary::DateContainer_t getExtrapolationDates();

	api::IRegressionFitAnalysisPhysicalSummary::DateContainer_t getDates();

	api::IRegressionFitAnalysisPhysicalSummary::DateContainer_t getPredictionDates();

	QString getBehaviorIndicatorName();

	api::IRegressionFitAnalysisPhysicalSummary::Date_t getRegressionStartDate();

	api::IRegressionFitAnalysisPhysicalSummary::Date_t getRegressionEndDate();

	void execute();

	void setProvider(Provider_t *provider);

	void setRegressionModel(boost::shared_ptr<api::IRegressionFitAnalysisPhysicalSummary::RegressionModel_t> regressionModel);

	void build(api::IRegressionFitAnalysisPhysicalSummary::ReportContext_t &rc);

	void build(boost::shared_ptr<api::IRegressionFitAnalysisPhysicalSummary::RawDataCollector_t> collector);

	void setName(QString name);

	QString getName();

	void configurationUpdateRequested();

	bool supportsReporting();

	bool supportRawDataExport();

private:
	QString name;
	boost::shared_ptr<api::IRegressionFitAnalysisPhysicalSummary::RegressionModel_t> regressionModel;
	Provider_t *provider;
	api::IRegressionFitAnalysisPhysicalSummary::BehaviorIndicatorValueContainer_t nullBehaviorIndicatorValueContainer;
	api::IRegressionFitAnalysisPhysicalSummary::PredictionsContainer_t nullPredictionsContainer;
	api::IRegressionFitAnalysisPhysicalSummary::DateContainer_t nullDateContainer;
	api::IRegressionFitAnalysisPhysicalSummary::ConfidenceIntervalContainer_t numericContainer;
	boost::shared_ptr<r::api::IRRegressionFitAnalysisPhysicalSummary> summary;
};

}  // namespace model
} // namespace dambase


#endif /* REGRESSIONFITANALYSISPHYSICALSUMMARY_HPP_ */
