/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitAnalysisRegressionSummary.cpp
 *
 *  Created on: 17.02.2015
 *      Author: gerberma
 */

#include "RegressionFitAnalysisRegressionSummary.hpp"

namespace dambase {
namespace model {

RegressionFitAnalysisRegressionSummary::RegressionFitAnalysisRegressionSummary():provider(NULL){;}
RegressionFitAnalysisRegressionSummary::RegressionFitAnalysisRegressionSummary(boost::shared_ptr<RegressionModel_t> regressionModel, Provider_t *provider, boost::shared_ptr<api::IRegressionFitAnalysisRegressionSummary::ModeHandler_t> modeHandler)
	:regressionModel(regressionModel), provider(provider), modeHandler(modeHandler)
{;}


RegressionFitAnalysisRegressionSummary::~RegressionFitAnalysisRegressionSummary(){;}


void
RegressionFitAnalysisRegressionSummary::execute() {
	if(summary.get() != NULL) {
		previousSummary = summary;
	}
	summary = provider->load(regressionModel->getRegressionFit());
	api::IRegressionFitAnalysisRegressionSummary::fireLoaded();
}

double
RegressionFitAnalysisRegressionSummary::getSSRes(bool previous) {
	if(previous) {
		if(previousSummary.get() == NULL) {
			return 0.0f;
		}else{
			return previousSummary->getSSRes();
		}
	}
	if(summary == NULL) {
		return 0.0f;
	}

	return summary->getSSRes();
}

double
RegressionFitAnalysisRegressionSummary::getMSRes(bool previous) {
	if(previous) {
		if(previousSummary.get() == NULL) {
			return 0.0f;
		}else{
			return previousSummary->getMSRes();
		}
	}
	if(summary == NULL) {
		return 0.0f;
	}

	return summary->getMSRes();
}

double
RegressionFitAnalysisRegressionSummary::getMSResDerived(bool previous) {
	if(previous) {
		if(previousSummary.get() == NULL) {
			return 0.0f;
		}else{
			return previousSummary->getMSResDerived();
		}
	}
	if(summary == NULL) {
		return 0.0f;
	}

	return summary->getMSResDerived();
}

double
RegressionFitAnalysisRegressionSummary::getR2(bool previous) {
	if(previous) {
		if(previousSummary.get() == NULL) {
			return 0.0f;
		}else{
			return previousSummary->getR2();
		}
	}

	if(summary == NULL) {
		return 0.0f;
	}

	return summary->getR2();
}

double
RegressionFitAnalysisRegressionSummary::getR2Adj(bool previous) {
	if(previous) {
		if(previousSummary.get() == NULL) {
			return 0.0f;
		}else{
			return previousSummary->getR2Adj();
		}
	}

	if(summary == NULL) {
		return 0.0f;
	}

	return summary->getR2Adj();
}

double
RegressionFitAnalysisRegressionSummary::getFStat(bool previous) {
	if(previous) {
		if(previousSummary.get() == NULL) {
			return 0.0f;
		}else{
			return previousSummary->getFStat();
		}
	}

	if(summary == NULL) {
		return 0.0f;
	}

	return summary->getFStat();
}

double
RegressionFitAnalysisRegressionSummary::getFStatPValue(bool previous) {
	if(previous) {
		if(previousSummary.get() == NULL) {
			return 0.0f;
		}else{
			return previousSummary->getFStatPValue();
		}
	}

	if(summary == NULL) {
		return 0.0f;
	}

	return summary->getFStatPValue();
}

double
RegressionFitAnalysisRegressionSummary::getDwStat(bool previous) {
	if(previous) {
		if(previousSummary.get() == NULL) {
			return 0.0f;
		}else{
			return previousSummary->getDwStat();
		}
	}

	if(summary == NULL) {
		return 0.0f;
	}

	return summary->getDwStat();
}

unsigned
RegressionFitAnalysisRegressionSummary::getNobs(bool previous) {
	if(previous) {
		if(previousSummary.get() == NULL) {
			return 0.0f;
		}else{
			return previousSummary->getNobs();
		}
	}

	if(summary == NULL) {
		return 0.0f;
	}

	return summary->getNobs();
}

double
RegressionFitAnalysisRegressionSummary::getSSResPred(bool previous) {
	if(previous) {
		if(previousSummary.get() == NULL) {
			return 0.0f;
		}else{
			return previousSummary->getSSResPred();
		}
	}
	if(summary == NULL) {
		return 0.0f;
	}
	return summary->getSSResPred();
}

double
RegressionFitAnalysisRegressionSummary::getMSResDerivedPred(bool previous) {
	if(previous) {
		if(previousSummary.get() == NULL) {
			return 0.0f;
		}else{
			return previousSummary->getMSResDerivedPred();
		}
	}

	if(summary == NULL) {
		return 0.0f;
	}
	return summary->getMSResDerivedPred();
}

void
RegressionFitAnalysisRegressionSummary::setRegressionModel(boost::shared_ptr<RegressionModel_t> regressionModel) {
	this->regressionModel = regressionModel;
}

void
RegressionFitAnalysisRegressionSummary::setProvider(Provider_t *provider) {
	this->provider = provider;
}

bool
RegressionFitAnalysisRegressionSummary::isScientific() {
	if(modeHandler == NULL) {
		throw std::logic_error("Mode handler not set in RegressionFitAnalysisRegressionSummary");
	}
	return modeHandler->isScientific();
}

void
RegressionFitAnalysisRegressionSummary::setModeHandler(boost::shared_ptr<api::IRegressionFitAnalysisRegressionSummary::ModeHandler_t> modeHandler) {
	this->modeHandler = modeHandler;
}

boost::shared_ptr<api::IRegressionFitAnalysisRegressionSummary::ModeHandler_t>
RegressionFitAnalysisRegressionSummary::getModeHandler() {
	return modeHandler;
}

void
RegressionFitAnalysisRegressionSummary::build(api::IRegressionFitAnalysisRegressionSummary::ReportContext_t &rc) {
	//ReportPolicy::build(this, rc);
	//rc << "### GenericAnalysisTool" << endl << endl; //this is only temporarly until we delegate to a ReportPolicy here - also remove the include
	//do something
	ExternalReportPolicy::build(boost::enable_shared_from_this<api::IRegressionFitAnalysisRegressionSummary>::shared_from_this(), rc);
}

void
RegressionFitAnalysisRegressionSummary::build(boost::shared_ptr<api::IRegressionFitAnalysisRegressionSummary::RawDataCollector_t> collector) {
	ExternalRawDataCollectionPolicy::build(boost::enable_shared_from_this<api::IRegressionFitAnalysisRegressionSummary>::shared_from_this(), collector);
}

void
RegressionFitAnalysisRegressionSummary::setName(QString name) {
	this->name = name;
}

QString
RegressionFitAnalysisRegressionSummary::getName() {
	return this->name;
}

void
RegressionFitAnalysisRegressionSummary::configurationUpdateRequested() {
	//ConfigurationUpdatePolicy::run(boost::enable_shared_from_this<GenericAnalysisTool<T, ReportContext, ReportPolicy, RawPlotDataCollector, RawDataCollectionPolicy, ConfigurationUpdatePolicy> >::shared_from_this());
}

bool
RegressionFitAnalysisRegressionSummary::supportsReporting() {
	return ExternalReportPolicy::supportsReporting();
}

bool
RegressionFitAnalysisRegressionSummary::supportRawDataExport() {
	return ExternalRawDataCollectionPolicy::supportRawDataExport();
}

} //namespace model
} //namespace dambase
