/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitAnalysisRegressionSummary.hpp
 *
 *  Created on: Mar 28, 2014
 *      Author: gerberma
 */

#ifndef REGRESSIONFITANALYSISREGRESSIONSUMMARY_HPP_
#define REGRESSIONFITANALYSISREGRESSIONSUMMARY_HPP_

#include <QString>
#include <boost/shared_ptr.hpp>
#include "ReportPolicies.hpp"
#include "RawDataCollectionPolicies.hpp"
#include "api/IRegressionModel.hpp"
#include "r/api/IRegressionFitAnalysisRegressionSummaryProvider.hpp"
#include "r/api/IRRegressionFitAnalysisRegressionSummary.hpp"
#include "api/IRegressionFitAnalysisRegressionSummary.hpp"

namespace dambase {
namespace model {

class RegressionFitAnalysisRegressionSummary : public api::IRegressionFitAnalysisRegressionSummary, ExternalReportPolicy, ExternalRawDataCollectionPolicy {
public:
	typedef api::IRegressionModel RegressionModel_t;
	typedef r::api::IRegressionFitAnalysisRegressionSummaryProvider Provider_t;
public:
	RegressionFitAnalysisRegressionSummary();
	RegressionFitAnalysisRegressionSummary(boost::shared_ptr<RegressionModel_t> regressionModel, Provider_t *provider, boost::shared_ptr<api::IRegressionFitAnalysisRegressionSummary::ModeHandler_t> modeHandler);


	virtual ~RegressionFitAnalysisRegressionSummary();

	static QString getStaticToolId() {
		return QString("2cdcb93f-7a7e-4fc3-be36-b3cff7d50ea5");
	}

	void execute();

	double getSSRes(bool previous);

	double getMSRes(bool previous);

	double getMSResDerived(bool previous);

	double getR2(bool previous);

	double getR2Adj(bool previous);

	double getFStat(bool previous);

	double getFStatPValue(bool previous);

	double getDwStat(bool previous);

	unsigned getNobs(bool previous);

	double getSSResPred(bool previous);

	double getMSResDerivedPred(bool previous);

	void setRegressionModel(boost::shared_ptr<RegressionModel_t> regressionModel);

	void setProvider(Provider_t *provider);

	bool isScientific();

	void setModeHandler(boost::shared_ptr<api::IRegressionFitAnalysisRegressionSummary::ModeHandler_t> modeHandler);

	boost::shared_ptr<api::IRegressionFitAnalysisRegressionSummary::ModeHandler_t> getModeHandler();

	void build(api::IRegressionFitAnalysisRegressionSummary::ReportContext_t &rc);

	void build(boost::shared_ptr<api::IRegressionFitAnalysisRegressionSummary::RawDataCollector_t> collector);

	void setName(QString name);

	QString getName();

	void configurationUpdateRequested();

	bool supportsReporting();

	bool supportRawDataExport();

private:
	QString name;
	boost::shared_ptr<RegressionModel_t> regressionModel;
	Provider_t *provider;
	boost::shared_ptr<r::api::IRRegressionFitAnalysisRegressionSummary> summary;
	boost::shared_ptr<r::api::IRRegressionFitAnalysisRegressionSummary> previousSummary;
	boost::shared_ptr<api::IRegressionFitAnalysisRegressionSummary::ModeHandler_t> modeHandler;
};

}  // namespace model
}  // namespace dambase


#endif /* REGRESSIONFITANALYSISREGRESSIONSUMMARY_HPP_ */
