/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitAnalysisRegressorOverview.cpp
 *
 *  Created on: 18.02.2015
 *      Author: gerberma
 */


#include "RegressionFitAnalysisRegressorOverview.hpp"

namespace dambase {
namespace model {


RegressionFitAnalysisRegressorOverview::RegressionFitAnalysisRegressorOverview() : provider(NULL)
{;}

RegressionFitAnalysisRegressorOverview::RegressionFitAnalysisRegressorOverview(boost::shared_ptr<RegressionModel_t> regressionModel, Provider_t *provider, boost::shared_ptr<api::IRegressionFitAnalysisRegressorOverview::ModeHandler_t> modeHandler)
	:regressionModel(regressionModel), provider(provider), modeHandler(modeHandler)
{;}

RegressionFitAnalysisRegressorOverview::~RegressionFitAnalysisRegressorOverview(){;}


void
RegressionFitAnalysisRegressorOverview::execute() {
	overview = provider->load(regressionModel->getRegressionFit());
	api::IRegressionFitAnalysisRegressorOverview::fireLoaded();
}

api::IRegressionFitAnalysisRegressorOverview::RegressorItemContainer_t
RegressionFitAnalysisRegressorOverview::getRegressorItems() {
	if(overview == NULL) {
		return nullRegressorItems;
	}

	return overview->getRegressorItems();
}

boost::shared_ptr<api::IRegressionFitAnalysisRegressorOverview::InterceptItem_t>
RegressionFitAnalysisRegressorOverview::getInterceptItem() {
	if(overview == NULL) {
		return nullInterceptItem;
	}

	return overview->getInterceptItem();
}

void
RegressionFitAnalysisRegressorOverview::setRegressorActive(int regressorIndex, bool isActive) {
	if(regressionModel->getRegressionFormula()->findRegressorByIndex(regressorIndex) == NULL) {
		throw std::logic_error("no regressor found with the given index on the formula");
	}

	regressionModel->getRegressionFormula()->findRegressorByIndex(regressorIndex)->setIsActive(isActive);
	regressionModel->updateFromFormulaChanges();
}

void
RegressionFitAnalysisRegressorOverview::setProvider(Provider_t *provider) {
	this->provider = provider;
}

void
RegressionFitAnalysisRegressorOverview::setRegressionModel(boost::shared_ptr<RegressionModel_t> regressionModel) {
	this->regressionModel = regressionModel;
}

bool
RegressionFitAnalysisRegressorOverview::isScientific() {
	if(modeHandler == NULL) {
		throw std::logic_error("Mode handler not set in RegressionFitAnalysisRegressorOverview");
	}
	return modeHandler->isScientific();
}

void
RegressionFitAnalysisRegressorOverview::setModeHandler(boost::shared_ptr<api::IRegressionFitAnalysisRegressorOverview::ModeHandler_t> modeHandler) {
	this->modeHandler = modeHandler;
}

boost::shared_ptr<api::IRegressionFitAnalysisRegressorOverview::ModeHandler_t>
RegressionFitAnalysisRegressorOverview::getModeHandler() {
	return modeHandler;
}

void
RegressionFitAnalysisRegressorOverview::build(api::IRegressionFitAnalysisRegressorOverview::ReportContext_t &rc) {
	//ReportPolicy::build(this, rc);
	//rc << "### GenericAnalysisTool" << endl << endl; //this is only temporarly until we delegate to a ReportPolicy here - also remove the include
	//do something
	ExternalReportPolicy::build(boost::enable_shared_from_this<api::IRegressionFitAnalysisRegressorOverview>::shared_from_this(), rc);
}

void
RegressionFitAnalysisRegressorOverview::build(boost::shared_ptr<api::IRegressionFitAnalysisRegressorOverview::RawDataCollector_t> collector) {
	ExternalRawDataCollectionPolicy::build(boost::enable_shared_from_this<api::IRegressionFitAnalysisRegressorOverview>::shared_from_this(), collector);
}

void
RegressionFitAnalysisRegressorOverview::setName(QString name) {
	this->name = name;
}

QString
RegressionFitAnalysisRegressorOverview::getName() {
	return this->name;
}

void
RegressionFitAnalysisRegressorOverview::configurationUpdateRequested() {
	//ConfigurationUpdatePolicy::run(boost::enable_shared_from_this<GenericAnalysisTool<T, ReportContext, ReportPolicy, RawPlotDataCollector, RawDataCollectionPolicy, ConfigurationUpdatePolicy> >::shared_from_this());
}

bool
RegressionFitAnalysisRegressorOverview::supportsReporting() {
	return ExternalReportPolicy::supportsReporting();
}

bool
RegressionFitAnalysisRegressorOverview::supportRawDataExport() {
	return ExternalRawDataCollectionPolicy::supportRawDataExport();
}

} //namespace model
} //namespace dambase
