/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitAnalysisRegressorOverview.hpp
 *
 *  Created on: Mar 26, 2014
 *      Author: gerberma
 */

#ifndef REGRESSIONFITANALYSISREGRESSOROVERVIEW_HPP_
#define REGRESSIONFITANALYSISREGRESSOROVERVIEW_HPP_

#include <QString>
#include <boost/shared_ptr.hpp>
#include "ReportPolicies.hpp"
#include "RawDataCollectionPolicies.hpp"
#include "api/IRegressionModel.hpp"
#include "api/IModeHandler.hpp"
#include "api/IRegressionFitAnalysisRegressorOverview.hpp"
#include "r/api/IRRegressionFitAnalysisRegressorOverview.hpp"
#include "r/api/IRegressionFitAnalysisRegressorOverviewProvider.hpp"

namespace dambase {
namespace model {

class RegressionFitAnalysisRegressorOverview  : public api::IRegressionFitAnalysisRegressorOverview, ExternalReportPolicy, ExternalRawDataCollectionPolicy
{
public:
	typedef api::IModeHandler ModeHandler_t;
	typedef api::IRegressionModel RegressionModel_t;
	typedef r::api::IRegressionFitAnalysisRegressorOverviewProvider Provider_t;
	typedef api::IRegressionFitAnalysisRegressorOverview::RegressorItem_t RegressorItem_t;
	typedef api::IRegressionFitAnalysisRegressorOverview::InterceptItem_t InterceptItem_t;
public:
	RegressionFitAnalysisRegressorOverview();

	RegressionFitAnalysisRegressorOverview(boost::shared_ptr<RegressionModel_t> regressionModel, Provider_t *provider, boost::shared_ptr<api::IRegressionFitAnalysisRegressorOverview::ModeHandler_t> modeHandler);

	virtual ~RegressionFitAnalysisRegressorOverview();

	static QString getStaticToolId() {
		return QString("3fa5a6aa-23b6-4685-af94-25b635db2050");
	}

	void execute();

	api::IRegressionFitAnalysisRegressorOverview::RegressorItemContainer_t getRegressorItems();

	boost::shared_ptr<api::IRegressionFitAnalysisRegressorOverview::InterceptItem_t> getInterceptItem();

	void setRegressorActive(int regressorIndex, bool isActive);

	void setProvider(Provider_t *provider);

	void setRegressionModel(boost::shared_ptr<RegressionModel_t> regressionModel);

	bool isScientific();

	void setModeHandler(boost::shared_ptr<ModeHandler_t> modeHandler);

	boost::shared_ptr<api::IRegressionFitAnalysisRegressorOverview::ModeHandler_t> getModeHandler();

	void build(api::IRegressionFitAnalysisRegressorOverview::ReportContext_t &rc);

	void build(boost::shared_ptr<api::IRegressionFitAnalysisRegressorOverview::RawDataCollector_t> collector);

	void setName(QString name);

	QString getName();

	void configurationUpdateRequested();

	bool supportsReporting();

	bool supportRawDataExport();

private:
	QString name;
	boost::shared_ptr<RegressionModel_t> regressionModel;
	Provider_t *provider;
	boost::shared_ptr<api::IRegressionFitAnalysisRegressorOverview::ModeHandler_t> modeHandler;
	boost::shared_ptr<r::api::IRRegressionFitAnalysisRegressorOverview> overview;
	RegressorItemContainer_t nullRegressorItems;
	boost::shared_ptr<InterceptItem_t> nullInterceptItem;
};

}  // namespace model
}  // namespace dambase



#endif /* REGRESSIONFITANALYSISREGRESSOROVERVIEW_HPP_ */
