/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitAnalysisRegressorTimeseries.cpp
 *
 *  Created on: 17.02.2015
 *      Author: gerberma
 */


#include "RegressionFitAnalysisRegressorTimeseries.hpp"

namespace dambase {
namespace model {

RegressionFitAnalysisRegressorTimeseries::RegressionFitAnalysisRegressorTimeseries():provider(NULL), ispolynom(false), isscaled(false){;}

RegressionFitAnalysisRegressorTimeseries::RegressionFitAnalysisRegressorTimeseries(
		boost::shared_ptr<RegressionModel_t> regressionModel,
		Provider_t *provider,
		bool ispolynom,
		bool isscaled)
	:regressionModel(regressionModel), provider(provider), ispolynom(ispolynom), isscaled(isscaled)
{;}

RegressionFitAnalysisRegressorTimeseries::~RegressionFitAnalysisRegressorTimeseries()
{;}

void
RegressionFitAnalysisRegressorTimeseries::setProvider(Provider_t *provider) {
	this->provider = provider;
}

void
RegressionFitAnalysisRegressorTimeseries::setRegressionModel(boost::shared_ptr<RegressionModel_t> regressionModel) {
	this->regressionModel = regressionModel;
}


std::vector<boost::shared_ptr<api::IRegressionFitAnalysisRegressorTimeseries::Item_t> >
RegressionFitAnalysisRegressorTimeseries::getItems() {
	return items;
}

void
RegressionFitAnalysisRegressorTimeseries::setIsPolynom(bool ispolynom) {
	this->ispolynom = ispolynom;
}

bool
RegressionFitAnalysisRegressorTimeseries::isPolynom() {
	return ispolynom;
}

void
RegressionFitAnalysisRegressorTimeseries::setIsScaled(bool isscaled) {
	this->isscaled = isscaled;
}

bool
RegressionFitAnalysisRegressorTimeseries::isScaled() {
	return isscaled;
}

void
RegressionFitAnalysisRegressorTimeseries::execute() {
	typedef api::IDataModel::Influence_Factor_t InfluenceFactor_t;
	typedef r::api::IRegressor Regressor_t;


	//keep old items as they get deleted later by the OwnershipPolicy and clear the items container
	items.clear();

	if(isPolynom()) {
		std::vector<boost::shared_ptr<api::IRegressionFitAnalysisRegressorTimeseries::AssignmentCategory_t> > assignmentCategoriesInUse = regressionModel->getRegressionFormula()->findAssignmentCategoriesFor(regressionModel->getRegressionFormula()->getActiveRegressors());
		BOOST_FOREACH(boost::shared_ptr<api::IRegressionFitAnalysisRegressorTimeseries::AssignmentCategory_t> assignmentCategoryInUse, assignmentCategoriesInUse) {
			boost::shared_ptr<api::IRegressionFitAnalysisRegressorTimeseries::Item_t> item = provider->load(
				regressionModel->getRegressionFit(),
				regressionModel->getRegressionFormula()->buildFormulaForAssignmentCategory(assignmentCategoryInUse),
				//we choose the prediction frame as the data frame of interest here but we can make this configurable
				regressionModel->getRegressionFormula()->getDataFrame()->getDataFrame(),
				regressionModel->getRegressionFormula()->getDataFrame()->getDateColumn(),
				regressionModel->getRegressionFit()->getRegressionFitDataFrame()->getDateColumn()->getFirstValue().getDate(),
				regressionModel->getRegressionFormula()->getDataFrame()->getDateColumn()->getLastValue().getDate(),
				regressionModel->getRegressionFit()->getOverallMaxStartupTime(),
				regressionModel->getRegressionFormula()->getDataFrame()->getDateColumn(),
				assignmentCategoryInUse->getName(),
				isScaled()
			);
			items.push_back(item);
		}
	}else{
		//single values
		BOOST_FOREACH(boost::shared_ptr<Regressor_t> activeRegressor, regressionModel->getRegressionFormula()->getActiveRegressors()) {
			boost::shared_ptr<api::IRegressionFitAnalysisRegressorTimeseries::Item_t> item = provider->load(
				regressionModel->getRegressionFit(),
				regressionModel->getRegressionFormula()->buildFormulaForSingleRegressor(activeRegressor),
				//we choose the prediction frame as the data frame of interest here but we can make this configurable
				regressionModel->getRegressionFormula()->getDataFrame()->getDataFrame(),
				regressionModel->getRegressionFormula()->getDataFrame()->getDateColumn(),
				regressionModel->getRegressionFit()->getRegressionFitDataFrame()->getDateColumn()->getFirstValue().getDate(),
				regressionModel->getRegressionFormula()->getDataFrame()->getDateColumn()->getLastValue().getDate(),
				regressionModel->getRegressionFit()->getOverallMaxStartupTime(),
				regressionModel->getRegressionFormula()->getDataFrame()->getDateColumn(),
				activeRegressor->getName(),
				isScaled()
			);
			items.push_back(item);
		}
	}


	api::IRegressionFitAnalysisRegressorTimeseries::fireLoaded();
}

void
RegressionFitAnalysisRegressorTimeseries::build(api::IRegressionFitAnalysisRegressorTimeseries::ReportContext_t &rc) {
	//ExternalReportPolicy::build(boost::enable_shared_from_this<api::IRegressionFitAnalysisRegressorTimeseries>::shared_from_this(), rc);
}

void
RegressionFitAnalysisRegressorTimeseries::build(boost::shared_ptr<api::IRegressionFitAnalysisRegressorTimeseries::RawDataCollector_t> collector) {
	//ExternalRawDataCollectionPolicy::build(boost::enable_shared_from_this<api::IRegressionFitAnalysisRegressorTimeseries>::shared_from_this(), collector);
}

void
RegressionFitAnalysisRegressorTimeseries::setName(QString name) {
	this->name = name;
}

QString
RegressionFitAnalysisRegressorTimeseries::getName() {
	return this->name;
}

void
RegressionFitAnalysisRegressorTimeseries::configurationUpdateRequested() {
	//ConfigurationUpdatePolicy::run(boost::enable_shared_from_this<GenericAnalysisTool<T, ReportContext, ReportPolicy, RawPlotDataCollector, RawDataCollectionPolicy, ConfigurationUpdatePolicy> >::shared_from_this());
}

bool
RegressionFitAnalysisRegressorTimeseries::supportsReporting() {
	return ExternalReportPolicy::supportsReporting();
}

bool
RegressionFitAnalysisRegressorTimeseries::supportRawDataExport() {
	return ExternalRawDataCollectionPolicy::supportRawDataExport();
}

} //namespace model
} //namespace dambase
