/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitAnalysisRegressorTimeseries.hpp
 *
 *  Created on: 28.10.2014
 *      Author: gerberma
 */

#ifndef REGRESSIONFITANALYSISREGRESSORTIMESERIES_HPP_
#define REGRESSIONFITANALYSISREGRESSORTIMESERIES_HPP_

#include <stdexcept>
#include <vector>
#include <boost/bind.hpp>
#include <boost/foreach.hpp>

#include <QString>
#include <boost/shared_ptr.hpp>
#include "ReportPolicies.hpp"
#include "RawDataCollectionPolicies.hpp"
#include "SignalPolicies.hpp"
#include "api/IRegressionModel.hpp"
#include "api/IDataModel.hpp"
#include "r/api/IRegressor.hpp"
#include "r/api/IRRegressionFitAnalysisRegressorTimeseries.hpp"
#include "r/api/IRegressionFitAnalysisRegressorTimeseriesProvider.hpp"
#include "api/IRegressionFitAnalysisRegressorTimeseries.hpp"


namespace dambase {
namespace model {

class RegressionFitAnalysisRegressorTimeseries : public api::IRegressionFitAnalysisRegressorTimeseries, ExternalReportPolicy, ExternalRawDataCollectionPolicy
{
public:
	typedef api::IRegressionModel RegressionModel_t;
	typedef r::api::IRegressionFitAnalysisRegressorTimeseriesProvider Provider_t;
public:
	RegressionFitAnalysisRegressorTimeseries();

	RegressionFitAnalysisRegressorTimeseries(
			boost::shared_ptr<RegressionModel_t> regressionModel,
			Provider_t *provider,
			bool ispolynom = false,
			bool isscaled = false);

	virtual ~RegressionFitAnalysisRegressorTimeseries();

	void setProvider(Provider_t *provider);

	void setRegressionModel(boost::shared_ptr<RegressionModel_t> regressionModel);

	static QString getStaticToolId() {
		return QString("f7131d6e-6125-4d89-9d4f-a608b09ec12f");
	}

	std::vector<boost::shared_ptr<api::IRegressionFitAnalysisRegressorTimeseries::Item_t> > getItems();

	void setIsPolynom(bool ispolynom);

	bool isPolynom();

	void setIsScaled(bool isscaled);

	bool isScaled();

	void execute();

	void build(api::IRegressionFitAnalysisRegressorTimeseries::ReportContext_t &rc);

	void build(boost::shared_ptr<api::IRegressionFitAnalysisRegressorTimeseries::RawDataCollector_t> collector);

	void setName(QString name);

	QString getName();

	void configurationUpdateRequested();

	bool supportsReporting();

	bool supportRawDataExport();

private:
	QString name;
	boost::shared_ptr<RegressionModel_t> regressionModel;
	Provider_t *provider;
	std::vector<boost::shared_ptr<api::IRegressionFitAnalysisRegressorTimeseries::Item_t> > items;
	bool ispolynom;
	bool isscaled;
};

}  // namespace model
}  // namespace dambase


#endif /* REGRESSIONFITANALYSISREGRESSORTIMESERIES_HPP_ */
