/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitAnalysisResidualNormalProbability.cpp
 *
 *  Created on: 17.02.2015
 *      Author: gerberma
 */


#include "RegressionFitAnalysisResidualNormalProbability.hpp"

namespace dambase {
namespace model {

RegressionFitAnalysisResidualNormalProbability::RegressionFitAnalysisResidualNormalProbability() : provider(NULL) {;}

RegressionFitAnalysisResidualNormalProbability::RegressionFitAnalysisResidualNormalProbability(boost::shared_ptr<RegressionModel_t> regressionModel, Provider_t *provider)
	:regressionModel(regressionModel), provider(provider)
{;}

RegressionFitAnalysisResidualNormalProbability::~RegressionFitAnalysisResidualNormalProbability(){;}


api::IRegressionFitAnalysisResidualNormalProbability::QuantileContainer_t
RegressionFitAnalysisResidualNormalProbability::getTheoreticalQuantiles() {
	if(normalProbability == NULL) {
		return nullQuantileContainer;
	}

	return normalProbability->getTheoreticalQuantiles();
}

api::IRegressionFitAnalysisResidualNormalProbability::QuantileContainer_t
RegressionFitAnalysisResidualNormalProbability::getSampleQuantiles() {
	if(normalProbability == NULL) {
		return nullQuantileContainer;
	}

	return normalProbability->getSampleQuantiles();
}


api::IRegressionFitAnalysisResidualNormalProbability::Point_t &
RegressionFitAnalysisResidualNormalProbability::getNormalLinePointOne() {
	if(normalProbability == NULL) {
		return nullPoint;
	}

	return normalProbability->getNormalLinePointOne();
}

api::IRegressionFitAnalysisResidualNormalProbability::Point_t &
RegressionFitAnalysisResidualNormalProbability::getNormalLinePointTwo() {
	if(normalProbability == NULL) {
		return nullPoint;
	}

	return normalProbability->getNormalLinePointTwo();
}

void
RegressionFitAnalysisResidualNormalProbability::execute() {
	normalProbability = provider->load(regressionModel->getRegressionFit());
	api::IRegressionFitAnalysisResidualNormalProbability::fireLoaded();
}

void
RegressionFitAnalysisResidualNormalProbability::setRegressionModel(boost::shared_ptr<RegressionModel_t> regressionModel) {
	this->regressionModel = regressionModel;
}

void
RegressionFitAnalysisResidualNormalProbability::setProvider(Provider_t *provider) {
	this->provider = provider;
}

void
RegressionFitAnalysisResidualNormalProbability::build(api::IRegressionFitAnalysisResidualNormalProbability::ReportContext_t &rc) {
	//ReportPolicy::build(this, rc);
	//rc << "### GenericAnalysisTool" << endl << endl; //this is only temporarly until we delegate to a ReportPolicy here - also remove the include
	//do something
	ExternalReportPolicy::build(boost::enable_shared_from_this<api::IRegressionFitAnalysisResidualNormalProbability>::shared_from_this(), rc);
}

void
RegressionFitAnalysisResidualNormalProbability::build(boost::shared_ptr<api::IRegressionFitAnalysisResidualNormalProbability::RawDataCollector_t> collector) {
	ExternalRawDataCollectionPolicy::build(boost::enable_shared_from_this<api::IRegressionFitAnalysisResidualNormalProbability>::shared_from_this(), collector);
}

void
RegressionFitAnalysisResidualNormalProbability::setName(QString name) {
	this->name = name;
}

QString
RegressionFitAnalysisResidualNormalProbability::getName() {
	return this->name;
}

void
RegressionFitAnalysisResidualNormalProbability::configurationUpdateRequested() {
	//ConfigurationUpdatePolicy::run(boost::enable_shared_from_this<GenericAnalysisTool<T, ReportContext, ReportPolicy, RawPlotDataCollector, RawDataCollectionPolicy, ConfigurationUpdatePolicy> >::shared_from_this());
}

bool
RegressionFitAnalysisResidualNormalProbability::supportsReporting() {
	return ExternalReportPolicy::supportsReporting();
}

bool
RegressionFitAnalysisResidualNormalProbability::supportRawDataExport() {
	return ExternalRawDataCollectionPolicy::supportRawDataExport();
}

} //namespace model
} //namespace dambase

