/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitAnalysisResidualNormalProbability.hpp
 *
 *  Created on: Apr 1, 2014
 *      Author: gerberma
 */

#ifndef REGRESSIONFITANALYSISRESIDUALNORMALPROBABILITY_HPP_
#define REGRESSIONFITANALYSISRESIDUALNORMALPROBABILITY_HPP_

#include <stdexcept>
#include <boost/bind.hpp>

#include <QString>
#include <boost/shared_ptr.hpp>
#include "ReportPolicies.hpp"
#include "RawDataCollectionPolicies.hpp"
#include "api/IRegressionModel.hpp"
#include "r/api/IRegressionFitAnalysisResidualNormalProbabilityProvider.hpp"
#include "r/api/IRRegressionFitAnalysisResidualNormalProbability.hpp"
#include "api/IRegressionFitAnalysisResidualNormalProbability.hpp"

namespace dambase {
namespace model {

class RegressionFitAnalysisResidualNormalProbability : public api::IRegressionFitAnalysisResidualNormalProbability, ExternalReportPolicy, ExternalRawDataCollectionPolicy
{
public:
	typedef r::api::IRegressionFitAnalysisResidualNormalProbabilityProvider Provider_t;
	typedef api::IRegressionModel RegressionModel_t;
public:
	RegressionFitAnalysisResidualNormalProbability();

	RegressionFitAnalysisResidualNormalProbability(boost::shared_ptr<RegressionModel_t> regressionModel, Provider_t *provider);

	virtual ~RegressionFitAnalysisResidualNormalProbability();

	static QString getStaticToolId() {
		return QString("1c5d90c0-d5a2-4e64-aef7-e8b1a813e959");
	}

	api::IRegressionFitAnalysisResidualNormalProbability::QuantileContainer_t getTheoreticalQuantiles();

	api::IRegressionFitAnalysisResidualNormalProbability::QuantileContainer_t getSampleQuantiles();

	api::IRegressionFitAnalysisResidualNormalProbability::Point_t &getNormalLinePointOne();

	api::IRegressionFitAnalysisResidualNormalProbability::Point_t &getNormalLinePointTwo();

	void execute();

	void setRegressionModel(boost::shared_ptr<RegressionModel_t> regressionModel);

	void setProvider(Provider_t *provider);

	void build(api::IRegressionFitAnalysisResidualNormalProbability::ReportContext_t &rc);

	void build(boost::shared_ptr<api::IRegressionFitAnalysisResidualNormalProbability::RawDataCollector_t> collector);

	void setName(QString name);

	QString getName();

	void configurationUpdateRequested();

	bool supportsReporting();

	bool supportRawDataExport();

private:
	QString name;
	boost::shared_ptr<RegressionModel_t> regressionModel;
	Provider_t *provider;
	boost::shared_ptr<r::api::IRRegressionFitAnalysisResidualNormalProbability> normalProbability;
	api::IRegressionFitAnalysisResidualNormalProbability::QuantileContainer_t nullQuantileContainer;
	api::IRegressionFitAnalysisResidualNormalProbability::Point_t nullPoint;
};

}  // namespace model
}  // namespace dambase

#endif /* REGRESSIONFITANALYSISRESIDUALNORMALPROBABILITY_HPP_ */
