/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitAnalysisResidualTimeSeries.cpp
 *
 *  Created on: 17.02.2015
 *      Author: gerberma
 */


#include "RegressionFitAnalysisResidualTimeSeries.hpp"

namespace dambase {
namespace model {

RegressionFitAnalysisResidualTimeSeries::RegressionFitAnalysisResidualTimeSeries():provider(NULL) {;}

RegressionFitAnalysisResidualTimeSeries::RegressionFitAnalysisResidualTimeSeries(boost::shared_ptr<RegressionModel_t> regressionModel, Provider_t *provider)
	:regressionModel(regressionModel), provider(provider)
{;}


api::IRegressionFitAnalysisResidualTimeSeries::ResidualContainer_t
RegressionFitAnalysisResidualTimeSeries::getResiduals() {
	if(residualTimeSeries == NULL) {
		return nullResidualContainer;
	}

	return residualTimeSeries->getResiduals();
}

api::IRegressionFitAnalysisResidualTimeSeries::DateContainer_t
RegressionFitAnalysisResidualTimeSeries::getDates() {
	if(residualTimeSeries == NULL) {
		return nullDateContainer;
	}

	return residualTimeSeries->getDates();
}

api::IRegressionFitAnalysisResidualTimeSeries::ResidualContainer_t
RegressionFitAnalysisResidualTimeSeries::getPredictionResiduals() {
	if(residualTimeSeries == NULL) {
		return nullResidualContainer;
	}

	return residualTimeSeries->getPredictionResiduals();
}

api::IRegressionFitAnalysisResidualTimeSeries::DateContainer_t
RegressionFitAnalysisResidualTimeSeries::getPredictionDates() {
	if(residualTimeSeries == NULL) {
		return nullDateContainer;
	}

	return residualTimeSeries->getPredictionDates();
}

void
RegressionFitAnalysisResidualTimeSeries::execute() {
	residualTimeSeries = provider->load(
			regressionModel->getRegressionFit()
			);
	api::IRegressionFitAnalysisResidualTimeSeries::fireLoaded();
}

void
RegressionFitAnalysisResidualTimeSeries::setRegressionModel(boost::shared_ptr<RegressionModel_t> regressionModel) {
	this->regressionModel = regressionModel;
}

void
RegressionFitAnalysisResidualTimeSeries::setProvider(Provider_t *provider) {
	this->provider = provider;
}

void
RegressionFitAnalysisResidualTimeSeries::build(api::IRegressionFitAnalysisResidualTimeSeries::ReportContext_t &rc) {
	//ReportPolicy::build(this, rc);
	//rc << "### GenericAnalysisTool" << endl << endl; //this is only temporarly until we delegate to a ReportPolicy here - also remove the include
	//do something
	ExternalReportPolicy::build(boost::enable_shared_from_this<api::IRegressionFitAnalysisResidualTimeSeries>::shared_from_this(), rc);
}

void
RegressionFitAnalysisResidualTimeSeries::build(boost::shared_ptr<api::IRegressionFitAnalysisResidualTimeSeries::RawDataCollector_t> collector) {
	ExternalRawDataCollectionPolicy::build(boost::enable_shared_from_this<api::IRegressionFitAnalysisResidualTimeSeries>::shared_from_this(), collector);
}

void
RegressionFitAnalysisResidualTimeSeries::setName(QString name) {
	this->name = name;
}

QString
RegressionFitAnalysisResidualTimeSeries::getName() {
	return this->name;
}

void
RegressionFitAnalysisResidualTimeSeries::configurationUpdateRequested() {
	//ConfigurationUpdatePolicy::run(boost::enable_shared_from_this<GenericAnalysisTool<T, ReportContext, ReportPolicy, RawPlotDataCollector, RawDataCollectionPolicy, ConfigurationUpdatePolicy> >::shared_from_this());
}

bool
RegressionFitAnalysisResidualTimeSeries::supportsReporting() {
	return ExternalReportPolicy::supportsReporting();
}

api::IRegressionFitAnalysisResidualTimeSeries::Date_t RegressionFitAnalysisResidualTimeSeries::getRegressionStartDate() {
	if(residualTimeSeries == NULL) {
		return "";
	}

	return residualTimeSeries->getRegressionStartDate();
}

api::IRegressionFitAnalysisResidualTimeSeries::Date_t
RegressionFitAnalysisResidualTimeSeries::getRegressionEndDate() {
	if(residualTimeSeries == NULL) {
		return "";
	}

	return residualTimeSeries->getRegressionEndDate();
}

api::IRegressionFitAnalysisResidualTimeSeries::ConfidenceIntervalContainer_t
RegressionFitAnalysisResidualTimeSeries::getUpperCi() {
	if(residualTimeSeries == NULL) {
		return numericContainer;
	}

	return residualTimeSeries->getUpperCi();
}

api::IRegressionFitAnalysisResidualTimeSeries::ConfidenceIntervalContainer_t
RegressionFitAnalysisResidualTimeSeries::getLowerCi() {
	if(residualTimeSeries == NULL) {
		return numericContainer;
	}

	return residualTimeSeries->getLowerCi();
}

api::IRegressionFitAnalysisResidualTimeSeries::OutOfLimitContainer_t
RegressionFitAnalysisResidualTimeSeries::getOutOfLimit() {
	if(residualTimeSeries == NULL) {
		return numericContainer;
	}

	return residualTimeSeries->getOutOfLimit();
}

api::IRegressionFitAnalysisResidualTimeSeries::DateContainer_t &
RegressionFitAnalysisResidualTimeSeries::getOutOfLimitDates() {
	if(residualTimeSeries == NULL) {
		return nullDateContainer;
	}

	return residualTimeSeries->getOutOfLimitDates();
}

api::IRegressionFitAnalysisResidualTimeSeries::ExtrapolationsContainer_t
RegressionFitAnalysisResidualTimeSeries::getExtrapolations() {
	if(residualTimeSeries == NULL) {
		return numericContainer;
	}

	return residualTimeSeries->getExtrapolations();
}

api::IRegressionFitAnalysisResidualTimeSeries::DateContainer_t
RegressionFitAnalysisResidualTimeSeries::getExtrapolationDates() {
	if(residualTimeSeries == NULL) {
		return nullDateContainer;
	}

	return residualTimeSeries->getExtrapolationDates();
}

bool
RegressionFitAnalysisResidualTimeSeries::supportRawDataExport() {
	return ExternalRawDataCollectionPolicy::supportRawDataExport();
}

} //namespace model
} //namespace dambase
