/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitAnalysisResidualTimeSeries.hpp
 *
 *  Created on: Apr 4, 2014
 *      Author: gerberma
 */

#ifndef REGRESSIONFITANALYSISRESIDUALTIMESERIES_HPP_
#define REGRESSIONFITANALYSISRESIDUALTIMESERIES_HPP_

#include <QString>
#include <boost/shared_ptr.hpp>
#include "ReportPolicies.hpp"
#include "RawDataCollectionPolicies.hpp"
#include "api/IRegressionModel.hpp"
#include "r/api/IRegressionFitAnalysisResidualTimeSeriesProvider.hpp"
#include "r/api/IRRegressionFitAnalysisResidualTimeSeries.hpp"
#include "api/IRegressionFitAnalysisResidualTimeSeries.hpp"

namespace dambase {
namespace model {

class RegressionFitAnalysisResidualTimeSeries : public api::IRegressionFitAnalysisResidualTimeSeries, ExternalReportPolicy, ExternalRawDataCollectionPolicy
{
public:
	typedef api::IRegressionModel RegressionModel_t;
	typedef r::api::IRegressionFitAnalysisResidualTimeSeriesProvider Provider_t;
public:
	RegressionFitAnalysisResidualTimeSeries();

	RegressionFitAnalysisResidualTimeSeries(boost::shared_ptr<RegressionModel_t> regressionModel, Provider_t *provider);

	static QString getStaticToolId() {
		return QString("9ae8b47d-cd7b-4c01-8551-e4ec5427f7eb");
	}

	api::IRegressionFitAnalysisResidualTimeSeries::Date_t getRegressionStartDate();

	api::IRegressionFitAnalysisResidualTimeSeries::Date_t getRegressionEndDate();

	api::IRegressionFitAnalysisResidualTimeSeries::ResidualContainer_t getResiduals();

	api::IRegressionFitAnalysisResidualTimeSeries::DateContainer_t getDates();

	api::IRegressionFitAnalysisResidualTimeSeries::ResidualContainer_t getPredictionResiduals();

	api::IRegressionFitAnalysisResidualTimeSeries::DateContainer_t getPredictionDates();

	api::IRegressionFitAnalysisResidualTimeSeries::ConfidenceIntervalContainer_t getUpperCi();

	api::IRegressionFitAnalysisResidualTimeSeries::ConfidenceIntervalContainer_t getLowerCi();

	api::IRegressionFitAnalysisResidualTimeSeries::OutOfLimitContainer_t getOutOfLimit();

	api::IRegressionFitAnalysisResidualTimeSeries::DateContainer_t &getOutOfLimitDates();

	api::IRegressionFitAnalysisResidualTimeSeries::ExtrapolationsContainer_t getExtrapolations();

	api::IRegressionFitAnalysisResidualTimeSeries::DateContainer_t getExtrapolationDates();

	void execute();

	void setRegressionModel(boost::shared_ptr<RegressionModel_t> regressionModel);

	void setProvider(Provider_t *provider);

	void build(api::IRegressionFitAnalysisResidualTimeSeries::ReportContext_t &rc);

	void build(boost::shared_ptr<api::IRegressionFitAnalysisResidualTimeSeries::RawDataCollector_t> collector);

	void setName(QString name);

	QString getName();

	void configurationUpdateRequested();

	bool supportsReporting();

	bool supportRawDataExport();

private:
	QString name;
	boost::shared_ptr<RegressionModel_t> regressionModel;
	Provider_t *provider;
	boost::shared_ptr<r::api::IRRegressionFitAnalysisResidualTimeSeries> residualTimeSeries;
	api::IRegressionFitAnalysisResidualTimeSeries::ResidualContainer_t nullResidualContainer;
	api::IRegressionFitAnalysisResidualTimeSeries::DateContainer_t nullDateContainer;
	api::IRegressionFitAnalysisResidualTimeSeries::ConfidenceIntervalContainer_t numericContainer;
};

}  // namespace model
}  // namespace dambase


#endif /* REGRESSIONFITANALYSISRESIDUALTIMESERIES_HPP_ */
