/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitAnalysisResidualVersusRegressionParameter.cpp
 *
 *  Created on: 17.02.2015
 *      Author: gerberma
 */


#include "RegressionFitAnalysisResidualVersusRegressionParameter.hpp"

namespace dambase {
namespace model {

RegressionFitAnalysisResidualVersusRegressionParameter::RegressionFitAnalysisResidualVersusRegressionParameter():provider(NULL) {;}

RegressionFitAnalysisResidualVersusRegressionParameter::RegressionFitAnalysisResidualVersusRegressionParameter(boost::shared_ptr<RegressionModel_t> regressionModel, Provider_t *provider)
	:regressionModel(regressionModel), provider(provider)
{;}

api::IRegressionFitAnalysisResidualVersusRegressionParameter::ResidualContainer_t
RegressionFitAnalysisResidualVersusRegressionParameter::getResiduals() {
	if(versus == NULL) {
		return nullResidualContainer;
	}

	return versus->getResiduals();
}

api::IRegressionFitAnalysisResidualVersusRegressionParameter::ParameterValueContainer_t
RegressionFitAnalysisResidualVersusRegressionParameter::getParameterValues() {
	if(versus == NULL) {
		return nullParameterValueContainer;
	}

	return versus->getParameterValues();
}

void
RegressionFitAnalysisResidualVersusRegressionParameter::execute() {
	versus = provider->loadVersusFitted(regressionModel->getRegressionFit());
	api::IRegressionFitAnalysisResidualVersusRegressionParameter::fireLoaded();
}

void
RegressionFitAnalysisResidualVersusRegressionParameter::setRegressionModel(boost::shared_ptr<RegressionModel_t> regressionModel) {
	this->regressionModel = regressionModel;
}

void
RegressionFitAnalysisResidualVersusRegressionParameter::setProvider(Provider_t *provider) {
	this->provider = provider;
}

void
RegressionFitAnalysisResidualVersusRegressionParameter::build(api::IRegressionFitAnalysisResidualVersusRegressionParameter::ReportContext_t &rc) {
	//ReportPolicy::build(this, rc);
	//rc << "### GenericAnalysisTool" << endl << endl; //this is only temporarly until we delegate to a ReportPolicy here - also remove the include
	//do something
	ExternalReportPolicy::build(boost::enable_shared_from_this<api::IRegressionFitAnalysisResidualVersusRegressionParameter>::shared_from_this(), rc);
}

void
RegressionFitAnalysisResidualVersusRegressionParameter::build(boost::shared_ptr<api::IRegressionFitAnalysisResidualVersusRegressionParameter::RawDataCollector_t> collector) {
	ExternalRawDataCollectionPolicy::build(boost::enable_shared_from_this<api::IRegressionFitAnalysisResidualVersusRegressionParameter>::shared_from_this(), collector);
}

void
RegressionFitAnalysisResidualVersusRegressionParameter::setName(QString name) {
	this->name = name;
}

QString
RegressionFitAnalysisResidualVersusRegressionParameter::getName() {
	return this->name;
}

void
RegressionFitAnalysisResidualVersusRegressionParameter::configurationUpdateRequested() {
	//ConfigurationUpdatePolicy::run(boost::enable_shared_from_this<GenericAnalysisTool<T, ReportContext, ReportPolicy, RawPlotDataCollector, RawDataCollectionPolicy, ConfigurationUpdatePolicy> >::shared_from_this());
}

bool
RegressionFitAnalysisResidualVersusRegressionParameter::supportsReporting() {
	return ExternalReportPolicy::supportsReporting();
}

bool
RegressionFitAnalysisResidualVersusRegressionParameter::supportRawDataExport() {
	return ExternalRawDataCollectionPolicy::supportRawDataExport();
}

} //namespace model
} //namespace dambase
