/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitAnalysisResidualVersusRegressionParameter.hpp
 *
 *  Created on: Apr 10, 2014
 *      Author: gerberma
 */

#ifndef REGRESSIONFITANALYSISRESIDUALVERSUSREGRESSIONPARAMETER_HPP_
#define REGRESSIONFITANALYSISRESIDUALVERSUSREGRESSIONPARAMETER_HPP_

#include <QString>
#include <boost/shared_ptr.hpp>
#include "api/IRegressionModel.hpp"
#include "ReportPolicies.hpp"
#include "RawDataCollectionPolicies.hpp"
#include "SignalPolicies.hpp"
#include "r/api/IRegressionFitAnalysisResidualVersusRegressionParameterProvider.hpp"
#include "r/api/IRRegressionFitAnalysisResidualVersusRegressionParameter.hpp"
#include "api/IRegressionFitAnalysisResidualVersusRegressionParameter.hpp"

namespace dambase {
namespace model {

class RegressionFitAnalysisResidualVersusRegressionParameter : public api::IRegressionFitAnalysisResidualVersusRegressionParameter, ExternalReportPolicy, ExternalRawDataCollectionPolicy
{
public:
	typedef api::IRegressionModel RegressionModel_t;
	typedef r::api::IRegressionFitAnalysisResidualVersusRegressionParameterProvider Provider_t;
public:
	RegressionFitAnalysisResidualVersusRegressionParameter();

	RegressionFitAnalysisResidualVersusRegressionParameter(boost::shared_ptr<RegressionModel_t> regressionModel, Provider_t *provider);

	static QString getStaticToolId() {
		return QString("3473ec9b-f1af-4267-8793-ed215bd326d1");
	}

	api::IRegressionFitAnalysisResidualVersusRegressionParameter::ResidualContainer_t getResiduals();

	api::IRegressionFitAnalysisResidualVersusRegressionParameter::ParameterValueContainer_t getParameterValues();

	void execute();

	void setRegressionModel(boost::shared_ptr<RegressionModel_t> regressionModel);

	void setProvider(Provider_t *provider);

	void build(api::IRegressionFitAnalysisResidualVersusRegressionParameter::ReportContext_t &rc);

	void build(boost::shared_ptr<api::IRegressionFitAnalysisResidualVersusRegressionParameter::RawDataCollector_t> collector);

	void setName(QString name);

	QString getName();

	void configurationUpdateRequested();

	bool supportsReporting();

	bool supportRawDataExport();

private:
	QString name;
	boost::shared_ptr<RegressionModel_t> regressionModel;
	Provider_t *provider;
	boost::shared_ptr<r::api::IRRegressionFitAnalysisResidualVersusRegressionParameter> versus;
	api::IRegressionFitAnalysisResidualVersusRegressionParameter::ResidualContainer_t nullResidualContainer;
	api::IRegressionFitAnalysisResidualVersusRegressionParameter::ParameterValueContainer_t nullParameterValueContainer;
};

}  //namespace model
}  // namespace dambase

#endif /* REGRESSIONFITANALYSISRESIDUALVERSUSREGRESSIONPARAMETER_HPP_ */
