/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionModelAnalysisToolsDescription.cpp
 *
 *  Created on: 16.02.2015
 *      Author: gerberma
 */

#include <boost/foreach.hpp>
#include "RegressionModelAnalysisToolsDescription.hpp"

namespace dambase {
namespace model {

RegressionModelAnalysisToolsDescription::RegressionModelAnalysisToolsDescription(
		boost::shared_ptr<api::IRegressionModelAnalysisToolsDescription::AnalysisToolsModeDescription_t> analysisToolsModeDescription,
		boost::shared_ptr<api::IRegressionModelAnalysisToolsDescription::ModeHandler_t> modeHandler
)
	:analysisToolsModeDescription(analysisToolsModeDescription),
	 modeHandler(modeHandler)
{;}

RegressionModelAnalysisToolsDescription::~RegressionModelAnalysisToolsDescription(){;}

api::IRegressionModelAnalysisToolsDescription::container_t
RegressionModelAnalysisToolsDescription::getPhysicsAnalysisToolsDescription() {
	return analysisToolsModeDescription->filterForListingEverywhere();
}

api::IRegressionModelAnalysisToolsDescription::container_t
RegressionModelAnalysisToolsDescription::getStatisticsAnalysisToolsDescription() {
	return analysisToolsModeDescription->filterForListingEverywhere();
}

boost::shared_ptr<api::IRegressionModelAnalysisToolsDescription::tool_description_t>
RegressionModelAnalysisToolsDescription::findStatisticsAnalysisToolDescriptionById(QString id) {
	return findAnalysisToolDescriptionById(getStatisticsAnalysisToolsDescription(), id);
}

boost::shared_ptr<api::IRegressionModelAnalysisToolsDescription::tool_description_t>
RegressionModelAnalysisToolsDescription::findPhysicsAnalysisToolDescriptionById(QString id) {
	return findAnalysisToolDescriptionById(getPhysicsAnalysisToolsDescription(), id);
}

//left side
api::IRegressionModelAnalysisToolsDescription::container_t
RegressionModelAnalysisToolsDescription::getAutomaticLeftSidePhysicalCategoryAnalysisToolDescriptions() {
	return analysisToolsModeDescription->filterForAutomaticallyAppear(true, 0, true);
}

api::IRegressionModelAnalysisToolsDescription::container_t
RegressionModelAnalysisToolsDescription::getAutomaticLeftSideStatisticsCategoryAnalysisToolDescriptions() {
	return analysisToolsModeDescription->filterForAutomaticallyAppear(true, 1, true);
}

//right side
api::IRegressionModelAnalysisToolsDescription::container_t
RegressionModelAnalysisToolsDescription::getAutomaticRightSidePhysicalCategoryAnalysisToolDescriptions() {
	return analysisToolsModeDescription->filterForAutomaticallyAppear(false, 0, true);
}

api::IRegressionModelAnalysisToolsDescription::container_t
RegressionModelAnalysisToolsDescription::getAutomaticRightSideStatisticsCategoryAnalysisToolDescriptions() {
	return analysisToolsModeDescription->filterForAutomaticallyAppear(false, 1, true);
}

boost::shared_ptr<api::IRegressionModelAnalysisToolsDescription::ModeHandler_t>
RegressionModelAnalysisToolsDescription::getModeHandler() {
	return modeHandler;
}


boost::shared_ptr<api::IRegressionModelAnalysisToolsDescription::tool_description_t>
RegressionModelAnalysisToolsDescription::findAnalysisToolDescriptionById(std::vector<boost::shared_ptr<tool_description_t> > container, QString id) {
	BOOST_FOREACH(boost::shared_ptr<api::IRegressionModelAnalysisToolsDescription::tool_description_t> analysisToolDescription, container) {
		if(analysisToolDescription->getId() == id) {
			return analysisToolDescription;
		}
	}

	boost::shared_ptr<tool_description_t> nullReturn;
	return nullReturn;
}


} //namespace model
} //namespace dambase
