/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * Report.hpp
 *
 *  Created on: Jul 10, 2014
 *      Author: gerberma
 */

#ifndef REPORT_HPP_
#define REPORT_HPP_

#include <QString>
#include <QProcess>
#include <QStringList>
#include <QDir>
#include <QFile>
#include <QDateTime>
#include <boost/shared_ptr.hpp>
#include "api/IReport.hpp"
#include "model/SignalPolicies.hpp"
#include "communication/Signals.hpp"

namespace dambase {
namespace model {



class Report : public api::IReport {
public:
	Report(boost::shared_ptr<api::IReport::Project_t> project, boost::shared_ptr<api::IReport::Analysis_t> analysis, boost::shared_ptr<api::IReport::RegressionModel_t> regressionModel,
			QString baseDirectory, QString supportedFileTypes, int plotWidth = 700, int plotHeight = 200);

	virtual ~Report();

	QString getOutputFileName();

	void setOutputFileName(QString outputFileName);

	void build();

	bool write();

	boost::shared_ptr<api::IReport::TitleReportListElement_t> getTitleReportListElement();

	void setTitleReportListElement(boost::shared_ptr<api::IReport::TitleReportListElement_t> titleReportListElement);

	boost::shared_ptr<api::IReport::DescriptionReportListElement_t> getDescriptionReportListElement();

	void setDescriptionReportListElement(boost::shared_ptr<api::IReport::DescriptionReportListElement_t> descriptionReportListElement);

	bool isDescriptionReportListElement(boost::shared_ptr<api::IReport::ReportListElement_t> element);

	boost::shared_ptr<api::IReport::ProjectReportListElement_t> getProjectReportListElement();

	void setProjectReportListElement(boost::shared_ptr<api::IReport::ProjectReportListElement_t> projectReportListElement);

	bool isProjectReportListElement(boost::shared_ptr<api::IReport::ReportListElement_t> element);

	boost::shared_ptr<api::IReport::AlgorithmReportListElement_t> getAlgorithmReportListElement();

	void setAlgorithmReportListElement(boost::shared_ptr<AlgorithmReportListElement_t> algorithmReportListElement);

	bool isAlgorithmReportListElement(boost::shared_ptr<api::IReport::ReportListElement_t> element);

	api::IReport::ReportListElementContainer_t &getReportListElements();

	void setReportListElements(api::IReport::ReportListElementContainer_t &reportListElements);

	bool isAnalysisToolListElement(boost::shared_ptr<api::IReport::ReportListElement_t> reportListElement);

	boost::shared_ptr<api::IReport::AnalysisToolReportListElement_t> getAnalysisToolListElementFor(boost::shared_ptr<api::IReport::ReportListElement_t> reportListElement);

	void setAnalysisToolReportListElements(api::IReport::AnalysisToolListElementContainer_t &analysisToolListElements);

	QString getSupportedFileTypesAsString();

	void moveUp(boost::shared_ptr<api::IReport::ReportListElement_t> reportListElement);

	void moveDown(boost::shared_ptr<api::IReport::ReportListElement_t> reportListElement);

	int findIndexOf(boost::shared_ptr<api::IReport::ReportListElement_t> reportListElement);

	void exchange(int elementIndex, int withElementIndex);

	std::vector<double> getAvailablePlotScaleFactors();

	double getPlotScaleFactor();

	void setPlotScaleFactor(double plotSize);

	std::vector<int> getAvailableFontSizes();

	int getFontSize();

	void setFontSize(int fontSize);

	void writeSuccessful();

	void writeTimedOut();

	void writeFailed();

	bool hasFailed();

	QString getFailMessage();

private:
	QString workingDirectory;
	QString markdownFileLocation;
	QString outputFileName;
	QString supportedFileTypes;
	boost::shared_ptr<api::IReport::Project_t> project;
	boost::shared_ptr<api::IReport::Analysis_t> analysis;
	boost::shared_ptr<api::IReport::RegressionModel_t> regressionModel;
	boost::shared_ptr<api::IReport::TitleReportListElement_t> titleListElement;
	boost::shared_ptr<api::IReport::DescriptionReportListElement_t> descriptionListElement;
	boost::shared_ptr<api::IReport::ProjectReportListElement_t> projectReportListElement;
	boost::shared_ptr<api::IReport::AlgorithmReportListElement_t> algorithmReportListElement;
	api::IReport::AnalysisToolListElementContainer_t analysisToolListElements;
	api::IReport::ReportListElementContainer_t reportListElements;
	std::vector<double> availablePlotScaleFactors;
	double plotScaleFactor;
	std::vector<int> availableFontSizes;
	int fontSize;
	int plotWidth;
	int plotHeight;
	QProcess writeProcess;
};

} // namespace model
} // namespace dambase


#endif /* REPORTDEFINITION_HPP_ */
