/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * ReportContexts.hpp
 *
 *  Created on: Jul 22, 2014
 *      Author: gerberma
 */

#ifndef REPORTCONTEXTS_HPP_
#define REPORTCONTEXTS_HPP_

#include <QTextStream>
#include <QFile>
#include <QString>
#include <QUuid>
#include <QDebug>
#include <boost/foreach.hpp>
#include <boost/algorithm/string.hpp>
#include <string>

namespace dambase {
namespace model {

class StreamReportContext : public QTextStream {
public:
	StreamReportContext(QFile *markdownFile, QString imageDirectory, int plotWidth = 0, int plotHeight = 0, double plotScaleFactor = 1.0, int plotPngQuality = -1)
		: QTextStream(markdownFile), imageDirectory(imageDirectory), plotWidth(plotWidth), plotHeight(plotHeight), plotScaleFactor(plotScaleFactor), plotPngQuality(plotPngQuality)
	{
		setCodec("UTF-8");
	}

	QString getRandomFileNameFor(QString extension) {
		QString randomString = QUuid::createUuid().toString();
		randomString = randomString.mid(1, randomString.length() - 2);

		return randomString + "." + extension;
	}

	QString getImageDirectory() {
		return imageDirectory;
	}

	QString getImageLocationFor(QString image) {
		return getImageDirectory() + "/" + image;
	}

	void setPlotWidth(int plotWidth) {
		this->plotWidth = plotWidth;
	}

	int getPlotWidth() {
		return plotWidth;
	}

	void setPlotHeight(int plotHeight) {
		this->plotHeight = plotHeight;
	}

	int getPlotHeight() {
		return plotHeight;
	}

	void setPlotScaleFactor(double scaleFactor) {
		plotScaleFactor = scaleFactor;
	}

	double getPlotScaleFactor() {
		return plotScaleFactor;
	}

	int getPlotPngQuality() {
		return plotPngQuality;
	}

	QString escape(QString str) {
		using namespace boost;

		std::string escapePattern = "\\`*_{}[]()>#+-.!^";
		std::string workingString = str.toStdString();

		BOOST_FOREACH(const std::string::value_type c, escapePattern) {
			std::string charAsString;
			charAsString.push_back(c);
			std::string replacement("\\");
			replacement.push_back(c);

			replace_all(workingString, charAsString, replacement);
		}


#ifdef _WIN32
		replace_all(workingString, "\n", "\n\n");
#endif

		return QString::fromStdString(workingString);
	}

private:
	QString imageDirectory;
	int plotWidth;
	int plotHeight;
	double plotScaleFactor;
	int plotPngQuality;
};

}  // namespace model
}  // namespace dambase


#endif /* REPORTCONTEXTS_HPP_ */
