/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * IAnalysis.hpp
 *
 *  Created on: 17.02.2015
 *      Author: gerberma
 */

#ifndef IANALYSIS_HPP_
#define IANALYSIS_HPP_

#include <boost/shared_ptr.hpp>
#include <QString>
#include "IRegressionModel.hpp"
#include "IDataModel.hpp"
#include "IWaterLevelScalingMinMaxOpLevel.hpp"
#include "model/SignalPolicies.hpp"

namespace dambase {
namespace model {
namespace api {

class AnalysisSignals : public communication::BoostSignalPolicy {
public:
	COM_SUBSCRIPTION_FUNCTION_TYPENAMED(DataModelSet, communication::BoostSignalPolicy::changed_signal_t)
	COM_SUBSCRIPTION_FUNCTION_TYPENAMED(RegressionModelAdded, communication::Signal<void(boost::shared_ptr<IRegressionModel>)>);
	COM_SUBSCRIPTION_FUNCTION_TYPENAMED(RemoveRegressionModel, communication::Signal<void(boost::shared_ptr<IRegressionModel>)>);

	void fire() {
		communication::BoostSignalPolicy::fire();
	}

	void fireDataModelSet() {
		DataModelSetSignal();
		communication::BoostSignalPolicy::fire();
	}

	void fireRegressionModelAdded(boost::shared_ptr<IRegressionModel> regressionModel) {
		RegressionModelAddedSignal(regressionModel);
		communication::BoostSignalPolicy::fire();
	}

	void fireRemoveRegressionModel(boost::shared_ptr<IRegressionModel> regressionModel) {
		RemoveRegressionModelSignal(regressionModel);
		communication::BoostSignalPolicy::fire();
	}
};

class IAnalysis : public AnalysisSignals {
public:
	typedef IDataModel DataModel_t;
	typedef IRegressionModel RegressionModel_t;
	typedef std::vector<boost::shared_ptr<RegressionModel_t> > RegressionModelContainer_t;
	typedef model::api::IWaterLevelScalingMinMaxOpLevel WaterLevelScalingStrategy_t;
public:
	IAnalysis(){;}
	virtual ~IAnalysis(){;}

	virtual const QString& getName() const = 0;

	virtual void setName(QString name) = 0;

	virtual const QString& getDescription() const = 0;

	virtual void setDescription(QString description) = 0;

	virtual const boost::shared_ptr<DataModel_t> &getDataModel() const = 0;

	virtual void setDataModel(boost::shared_ptr<DataModel_t> dataModel) = 0;

	virtual bool isNewDataModelAllowed() = 0;

	virtual bool isNewRegressionModelAllowed() = 0;

	/**
	 * This is primarly used by the deserialization process
	 */
	virtual void setRegressionModels(std::vector<boost::shared_ptr<RegressionModel_t> > &regressionModels) = 0;

	virtual void addRegressionModel(boost::shared_ptr<RegressionModel_t> regressionModel) = 0;

	virtual const RegressionModelContainer_t &getRegressionModels() const = 0;

	virtual void removeRegressionModel(boost::shared_ptr<RegressionModel_t> regressionModel) = 0;

	virtual const boost::shared_ptr<WaterLevelScalingStrategy_t> &getWaterLevelScalingStrategy() const = 0;

	virtual void setWaterLevelScalingStrategy(boost::shared_ptr<WaterLevelScalingStrategy_t> waterLevelScalingStrategy) = 0;
};

} //namespace api
} //namespace model
} //namespace


#endif /* IANALYSIS_HPP_ */
