/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * AnalysisTool.hpp
 *
 *  Created on: Jan 22, 2014
 *      Author: gerberma
 */

#ifndef ANALYSISTOOL_HPP_
#define ANALYSISTOOL_HPP_

#include <boost/shared_ptr.hpp>
#include <QString>
#include "GenericAnalysisToolSignalPolicy.hpp"
#include "IReportListElement.hpp"
#include "IRawDataCollector.hpp"

namespace dambase {
namespace model {
namespace api {

class IAnalysisTool : public virtual GenericAnalysisToolSignalPolicy {
public:
	typedef IReportListElement::ReportContext_t ReportContext_t;
	typedef IRawDataCollector RawDataCollector_t;
public:
	IAnalysisTool()
	{;}

	virtual ~IAnalysisTool()
	{}

	virtual QString getName() {return "<undefined>";}

	virtual void build(ReportContext_t &reportContext) {
		//do nothing in default case
	}

	virtual void build(boost::shared_ptr<RawDataCollector_t> collector) {
		//do nothing in default case
	}

	/**
	 * This method gets called before execute() in case where the model calls execute() on that AT (e.g.: regressionModel updates its ats in case of having the fit loaded)
	 */
	virtual void configurationUpdateRequested() = 0;

	virtual void execute() = 0;

	virtual bool supportsReporting() = 0;

	virtual bool supportRawDataExport() = 0;
};

}
}
} /* namespace dambase::model::api */



#endif /* ANALYSISTOOL_HPP_ */
