/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * IDataModelAnalysisInfluenceFactors.hpp
 *
 *  Created on: 18.02.2015
 *      Author: gerberma
 */

#ifndef IDATAMODELANALYSISINFLUENCEFACTORS_HPP_
#define IDATAMODELANALYSISINFLUENCEFACTORS_HPP_

#include "IAnalysisTool.hpp"
#include "IDataModel.hpp"
#include "model/r/api/IDataFrameInfluenceFactors.hpp"

namespace dambase {
namespace model {
namespace api {

namespace detail {

class DataModelAnalysisInfluenceFactorsSignals : public virtual GenericAnalysisToolSignalPolicy {
public:
	DataModelAnalysisInfluenceFactorsSignals(){}
	virtual ~DataModelAnalysisInfluenceFactorsSignals(){}

	COM_SUBSCRIPTION_FUNCTION(BehaviorIndicatorsChanged, communication::Signal<void()>);
	COM_SUBSCRIPTION_FUNCTION(AnyInfluenceFactorChanged, communication::Signal<void()>);

	void fireBehaviorIndicatorsChanged() {
		GenericAnalysisToolSignalPolicy::fire();
		BehaviorIndicatorsChangedSignal();
	}

	void fireAnyInfluenceFactorChanged() {
		GenericAnalysisToolSignalPolicy::fire();
		AnyInfluenceFactorChangedSignal();
	}
};

}  // namespace detail

class IDataModelAnalysisInfluenceFactors : public virtual IAnalysisTool, public virtual detail::DataModelAnalysisInfluenceFactorsSignals {
public:
	typedef r::api::IDataFrameInfluenceFactors::ValueContainer_t ValueContainer_t;
	typedef IDataModel::Behavior_Indicator_Column_Container_t Behavior_Indicator_Column_Container_t;
	typedef r::api::IDataFrameInfluenceFactors r_data_frame_influence_factors_t;
	typedef IDataModel DataModel_t;
	typedef IDataModel::Column_t Column_t;
public:
	IDataModelAnalysisInfluenceFactors(){;}
	virtual ~IDataModelAnalysisInfluenceFactors(){;}

	virtual void execute() = 0;

	virtual bool supportsReporting() = 0;

	virtual bool supportRawDataExport() = 0;

	virtual void configurationUpdateRequested() = 0;

	virtual int getAnEstimatedBehaviorIndicator() = 0;

	virtual void estimateCurrentBehaviorIndicator() = 0;

	virtual int getAnEstimatedInfluenceFactor() = 0;

	virtual void estimateCurrentInfluenceFactor() = 0;

	virtual void setCurrentBehaviorIndicator(int index) = 0;

	virtual boost::shared_ptr<Column_t> getCurrentBehaviorIndicatorColumn() = 0;

	virtual void setCurrentInfluenceFactor(int index) = 0;

	virtual boost::shared_ptr<api::IDataModelAnalysisInfluenceFactors::Column_t> getCurrentInfluenceFactorColumn() = 0;

	virtual api::IDataModelAnalysisInfluenceFactors::ValueContainer_t getBehaviorIndicatorColumnValues() = 0;

	virtual api::IDataModelAnalysisInfluenceFactors::ValueContainer_t getInfluenceFactorColumnValues() = 0;

	virtual const Behavior_Indicator_Column_Container_t& getBehaviorIndicatorColumns() = 0;

	virtual const DataModel_t::Water_Level_Column_Container_t& getWaterLevelColumns() = 0;

	virtual const DataModel_t::Temperature_Column_Container_t& getTemperatureColumns() = 0;
};

} //namespace api
} //namespace model
} //namespace dambase


#endif /* IDATAMODELANALYSISINFLUENCEFACTORS_HPP_ */
