/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * IModel.hpp
 *
 *  Created on: 18.02.2015
 *      Author: gerberma
 */

#ifndef IMODEL_HPP_
#define IMODEL_HPP_

#include "IActivityTracker.hpp"
#include "IAnalysisToolsDescriptions.hpp"
#include "IProject.hpp"
#include "model/r/api/IAssignmentCategory.hpp"
#include "IModeSettings.hpp"
#include "model/SignalPolicies.hpp"

namespace dambase {
namespace model {
namespace api {

class IModel : public communication::BoostSignalPolicy {
public:
	typedef IActivityTracker ActivityTracker_t;
	typedef IAnalysisToolsDescriptions AnalysisToolsDescriptions_t;
	typedef IProject Project_t;
	typedef r::api::IAssignmentCategory AssignmentCategory_t;
	typedef IModeSettings ModeSettings_t;
public:
	IModel(){;}
	virtual ~IModel(){;}

	virtual boost::shared_ptr<Project_t> getProject() = 0;

	virtual void setProject(boost::shared_ptr<Project_t> project) = 0;

	virtual boost::shared_ptr<AnalysisToolsDescriptions_t::dataModel_analysis_tools_description_t> getDataModelAnalysisToolsDescription() = 0;

	virtual boost::shared_ptr<AnalysisToolsDescriptions_t::regressionModel_analysis_tools_description_t> getRegressionModelAnalysisToolsDescription() = 0;

	virtual boost::shared_ptr<ModeSettings_t> getModeSettings() = 0;

	virtual boost::shared_ptr<ActivityTracker_t> getActivityTracker() = 0;

	virtual std::vector<boost::shared_ptr<AssignmentCategory_t> > &getAssignmentCategories() = 0;

	/**
	 * This is primarly used by the deserialization process to keep constness
	 */
	virtual const std::vector<boost::shared_ptr<AssignmentCategory_t> > &getAssignmentCategoriesConst() const = 0;

	virtual boost::shared_ptr<AssignmentCategory_t> findAssignmentCategoryByIdentifier(QString identifier) = 0;

	virtual bool isLoadingProjectAllowed() = 0;

	virtual bool isCreatingNewProjectAllowed() = 0;
};

} //namespace api
} //namespace model
} //namespace

#endif /* IMODEL_HPP_ */
