/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * IProject.hpp
 *
 *  Created on: 17.02.2015
 *      Author: gerberma
 */

#ifndef IPROJECT_HPP_
#define IPROJECT_HPP_

#include <boost/shared_ptr.hpp>
#include <QString>
#include "IModeHandler.hpp"
#include "IAnalysis.hpp"
#include "model/SignalPolicies.hpp"
#include "communication/Signals.hpp"

namespace dambase {
namespace model {
namespace api {

class ProjectSignals : public communication::BoostSignalPolicy {
public:
	typedef boost::signals2::signal<void(boost::shared_ptr<IAnalysis>)> analysis_added_signal_t;
public:
	ProjectSignals(){}
	virtual ~ProjectSignals(){}

	COM_SUBSCRIPTION_FUNCTION(RemoveAnalysis, communication::Signal<void(boost::shared_ptr<IAnalysis>)>);

	boost::signals2::connection subscribeForAnalysisAdded(const typename analysis_added_signal_t::slot_type &subscriber) {
		return analysisAddedSignal.connect(subscriber);
	}

	void fireAnalysisAdded(boost::shared_ptr<IAnalysis> analysis){
		fire();
		analysisAddedSignal(analysis);
	}

	void fireRemoveAnalysis(boost::shared_ptr<IAnalysis> analysis){
		RemoveAnalysisSignal(analysis);
		fire();
	}

	void fire() {
		communication::BoostSignalPolicy::fire();
	}

private:
	analysis_added_signal_t analysisAddedSignal;
};

class IProject : public ProjectSignals {
public:
	typedef IAnalysis Analysis_t;
	typedef IModeHandler ModeHandler_t;
public:
	IProject(){;}
	virtual ~IProject(){;}

	virtual void setName(QString name) = 0;

	virtual const QString &getName() const = 0;

	virtual void setDescription(QString description) = 0;

	virtual const QString &getDescription() const = 0;

	virtual void setDamOperator(QString damOperator) = 0;

	virtual const QString &getDamOperator() const = 0;

	virtual void setAnalyses(const std::vector<boost::shared_ptr<Analysis_t> > &newAnalyses) = 0;

	virtual void addAnalysis(boost::shared_ptr<Analysis_t> analysis) = 0;

	virtual const std::vector<boost::shared_ptr<Analysis_t> >& getAnalyses() const = 0;

	virtual void removeAnalysis(boost::shared_ptr<Analysis_t> analysis) = 0;

	virtual bool isNewAnalysisAllowed() = 0;

	virtual void setModeHandler(boost::shared_ptr<ModeHandler_t> modeHandler) = 0;

	virtual bool isScientific() = 0;
};

} //namespace api
} //namespace model
} //namespace


#endif /* IPROJECT_HPP_ */
