/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * IRawDataCollector.hpp
 *
 *  Created on: 18.02.2015
 *      Author: gerberma
 */

#ifndef IRAWDATACOLLECTOR_HPP_
#define IRAWDATACOLLECTOR_HPP_

#include <vector>
#include <boost/shared_ptr.hpp>
#include <QFile>

namespace dambase {
namespace model {
namespace api {

class IProject;
class IAnalysis;
class IRegressionModel;

class IRawDataCollector {
public:
	typedef IProject Project_t;
	typedef IAnalysis Analysis_t;
	typedef IRegressionModel RegressionModel_t;
public:
	class DataRow {
		public:
			DataRow(QString name, double value);
			virtual ~DataRow();

			double getValue();
			QString getName();
		private:
			QString name;
			double value;
		};

		class Collection {
		public:
			Collection(QString name, QString keyName);
			virtual ~Collection();

			void addValues(QString name, std::vector<std::string> keys, std::vector<double> values);

			void addValues(QString name, std::vector<QString> keys, std::vector<double> values);

			void addValues(QString name, std::vector<unsigned> keys, std::vector<double> values);

			void addValues(QString name, std::vector<double> keys, std::vector<double> values);

			void addValue(QString name, unsigned key, double value);

			QString getKeyName();

			QString getName();

			std::map<QString, std::vector<DataRow> > &getCollection();

		private:
			unsigned calculateNrOfPlaceholdersFor(unsigned u);
		private:
			QString name;
			QString keyName;
			std::map<QString, std::vector<DataRow> > collection;
		};
public:
	typedef IRawDataCollector::Collection Collection_t;
public:
	IRawDataCollector(){;}
	virtual ~IRawDataCollector(){;}

	virtual void addStatisticsValueColumn(QString name, std::vector<std::string> dates, std::vector<double> values) = 0;

	virtual void writeHeader(QFile *outputFile, boost::shared_ptr<Project_t> project, boost::shared_ptr<Analysis_t> analysis, boost::shared_ptr<RegressionModel_t> regressionModel) = 0;

	virtual void write(QFile *outputFile) = 0;

	virtual QString getSupportedFileTypesAsString() = 0;

	virtual boost::shared_ptr<Collection_t> createNewCollection(QString name, QString keyName) = 0;
};

} //namespace api
} //namespace model
} //namespace

#endif /* IRAWDATACOLLECTOR_HPP_ */
