/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * IRegressionFitAnalysisACF.hpp
 *
 *  Created on: 17.02.2015
 *      Author: gerberma
 */

#ifndef IREGRESSIONFITANALYSISACF_HPP_
#define IREGRESSIONFITANALYSISACF_HPP_

#include <boost/shared_ptr.hpp>
#include <boost/enable_shared_from_this.hpp>
#include "model/r/api/IRRegressionFitAnalysisACF.hpp"
#include "IAnalysisTool.hpp"

namespace dambase {
namespace model {
namespace api {

class IRegressionFitAnalysisACF : public virtual IAnalysisTool, public boost::enable_shared_from_this<IRegressionFitAnalysisACF>  {
public:
	typedef r::api::IRRegressionFitAnalysisACF::AcfContainer_t AcfContainer_t;
	typedef r::api::IRRegressionFitAnalysisACF::LagContainer_t LagContainer_t;
public:
	IRegressionFitAnalysisACF(){;}
	virtual ~IRegressionFitAnalysisACF(){;}

	virtual AcfContainer_t &getAcfs() = 0;

	virtual LagContainer_t &getLags() = 0;

	virtual double getLowerCi() = 0;

	virtual double getUpperCi() = 0;

	virtual void build(api::IRegressionFitAnalysisACF::ReportContext_t &rc) = 0;

	virtual void build(boost::shared_ptr<api::IAnalysisTool::RawDataCollector_t> collector) = 0;

	virtual void setName(QString name) = 0;

	virtual QString getName() = 0;

	virtual void configurationUpdateRequested() = 0;

	virtual bool supportsReporting() = 0;

	virtual bool supportRawDataExport() = 0;
};

} //namespace api
} //namespace model
} //namespace


#endif /* IREGRESSIONFITANALYSISACF_HPP_ */
