/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * IRegressionFitAnalysisPhysicalBehavior.hpp
 *
 *  Created on: 17.02.2015
 *      Author: gerberma
 */

#ifndef IREGRESSIONFITANALYSISPHYSICALBEHAVIOR_HPP_
#define IREGRESSIONFITANALYSISPHYSICALBEHAVIOR_HPP_

#include <boost/enable_shared_from_this.hpp>
#include "model/r/api/IAssignmentCategory.hpp"
#include "model/r/api/IRRegressionFitAnalysisPhysicalBehavior.hpp"
#include "IRegressionModel.hpp"
#include "IAnalysisTool.hpp"

namespace dambase {
namespace model {
namespace api {

class IRegressionFitAnalysisPhysicalBehavior : public IAnalysisTool, public boost::enable_shared_from_this<IRegressionFitAnalysisPhysicalBehavior>  {
public:
	typedef model::r::api::IAssignmentCategory AssignmentCategory_t;
	typedef r::api::IRRegressionFitAnalysisPhysicalBehavior Item_t;
public:
	IRegressionFitAnalysisPhysicalBehavior(){;}
	virtual ~IRegressionFitAnalysisPhysicalBehavior(){;}

	virtual std::vector<boost::shared_ptr<r::api::IRRegressionFitAnalysisPhysicalBehavior> > getItems() = 0;

	virtual void setAssignmentCategories(std::vector<boost::shared_ptr<AssignmentCategory_t> > assignmentCategories) = 0;

	virtual std::vector<boost::shared_ptr<AssignmentCategory_t> > getAssignmentCategories() = 0;

	virtual void setAssignmentCategory(boost::shared_ptr<AssignmentCategory_t> assignmentCategory) = 0;

	virtual boost::shared_ptr<AssignmentCategory_t> getAssignmentCategory() = 0;

	virtual void execute() = 0;

	virtual boost::shared_ptr<AssignmentCategory_t> findAssignmentCategoryByIdentifier(QString identifier) = 0;

	virtual void build(ReportContext_t &rc) = 0;

	virtual void build(boost::shared_ptr<RawDataCollector_t> collector) = 0;

	virtual void setName(QString name) = 0;

	virtual QString getName() = 0;

	virtual void configurationUpdateRequested() = 0;

	virtual bool supportsReporting() = 0;

	virtual bool supportRawDataExport() = 0;
};

} //namespace api
} //namespace model
} //namespace


#endif /* IREGRESSIONFITANALYSISPHYSICALBEHAVIOR_HPP_ */
