/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * IRegressionFitAnalysisPhysicalSummary.hpp
 *
 *  Created on: 17.02.2015
 *      Author: gerberma
 */

#ifndef IREGRESSIONFITANALYSISPHYSICALSUMMARY_HPP_
#define IREGRESSIONFITANALYSISPHYSICALSUMMARY_HPP_

#include <boost/enable_shared_from_this.hpp>
#include "model/r/api/IAssignmentCategory.hpp"
#include "IRegressionModel.hpp"
#include "model/r/api/IRRegressionFitAnalysisPhysicalSummary.hpp"
#include "IAnalysisTool.hpp"

namespace dambase {
namespace model {
namespace api {

class IRegressionFitAnalysisPhysicalSummary : public IAnalysisTool, public boost::enable_shared_from_this<IRegressionFitAnalysisPhysicalSummary> {
public:
	typedef IRegressionModel RegressionModel_t;
	typedef r::api::IRRegressionFitAnalysisPhysicalSummary::BehaviorIndicatorValueContainer_t BehaviorIndicatorValueContainer_t;
	typedef r::api::IRRegressionFitAnalysisPhysicalSummary::PredictionsContainer_t PredictionsContainer_t;
	typedef r::api::IRRegressionFitAnalysisPhysicalSummary::DateContainer_t DateContainer_t;
	typedef r::api::IRRegressionFitAnalysisPhysicalSummary::ConfidenceIntervalContainer_t ConfidenceIntervalContainer_t;
	typedef r::api::IRRegressionFitAnalysisPhysicalSummary::OutOfLimitContainer_t OutOfLimitContainer_t;
	typedef r::api::IRRegressionFitAnalysisPhysicalSummary::ExtrapolationsContainer_t ExtrapolationsContainer_t;
	typedef r::api::IRRegressionFitAnalysisPhysicalSummary::Date_t Date_t;
public:
	IRegressionFitAnalysisPhysicalSummary(){;}
	virtual ~IRegressionFitAnalysisPhysicalSummary(){;}

	virtual BehaviorIndicatorValueContainer_t getBehaviorIndicatorValues() = 0;

	virtual PredictionsContainer_t getPredictions() = 0;

	virtual ConfidenceIntervalContainer_t getUpperCi() = 0;

	virtual ConfidenceIntervalContainer_t getLowerCi() = 0;

	virtual OutOfLimitContainer_t getOutOfLimit() = 0;

	virtual DateContainer_t &getOutOfLimitDates() = 0;

	virtual ExtrapolationsContainer_t getExtrapolations() = 0;

	virtual DateContainer_t getExtrapolationDates() = 0;

	virtual DateContainer_t getDates() = 0;

	virtual DateContainer_t getPredictionDates() = 0;

	virtual QString getBehaviorIndicatorName() = 0;

	virtual Date_t getRegressionStartDate() = 0;

	virtual Date_t getRegressionEndDate() = 0;

	virtual void execute() = 0;

	virtual void build(IAnalysisTool::ReportContext_t &rc) = 0;

	virtual void build(boost::shared_ptr<IAnalysisTool::RawDataCollector_t> collector) = 0;

	virtual void setName(QString name) = 0;

	virtual QString getName() = 0;

	virtual void configurationUpdateRequested() = 0;

	virtual bool supportsReporting() = 0;

	virtual bool supportRawDataExport() = 0;
};

} //namespace api
} //namespace model
} //namespace


#endif /* IREGRESSIONFITANALYSISPHYSICALSUMMARY_HPP_ */
