/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * IRegressionFitAnalysisRegressionSummary.hpp
 *
 *  Created on: 17.02.2015
 *      Author: gerberma
 */

#ifndef IREGRESSIONFITANALYSISREGRESSIONSUMMARY_HPP_
#define IREGRESSIONFITANALYSISREGRESSIONSUMMARY_HPP_

#include <boost/enable_shared_from_this.hpp>
#include "model/SignalPolicies.hpp"
#include "IModeHandler.hpp"
#include "IAnalysisTool.hpp"

namespace dambase {
namespace model {
namespace api {

class IRegressionFitAnalysisRegressionSummary : public IAnalysisTool, public boost::enable_shared_from_this<IRegressionFitAnalysisRegressionSummary> {
public:
	typedef IModeHandler ModeHandler_t;
public:
	IRegressionFitAnalysisRegressionSummary(){;}
	virtual ~IRegressionFitAnalysisRegressionSummary(){;}

	virtual void execute() = 0;

	virtual double getSSRes(bool previous = false) = 0;

	virtual double getMSRes(bool previous = false) = 0;

	virtual double getMSResDerived(bool previous = false) = 0;

	virtual double getR2(bool previous = false) = 0;

	virtual double getR2Adj(bool previous = false) = 0;

	virtual double getFStat(bool previous = false) = 0;

	virtual double getFStatPValue(bool previous = false) = 0;

	virtual double getDwStat(bool previous = false) = 0;

	virtual unsigned getNobs(bool previous = false) = 0;

	virtual double getSSResPred(bool previous = false) = 0;

	virtual double getMSResDerivedPred(bool previous = false) = 0;

	virtual bool isScientific() = 0;

	virtual void setModeHandler(boost::shared_ptr<api::IRegressionFitAnalysisRegressionSummary::ModeHandler_t> modeHandler) = 0;

	virtual boost::shared_ptr<api::IRegressionFitAnalysisRegressionSummary::ModeHandler_t> getModeHandler() = 0;

	virtual void build(api::IRegressionFitAnalysisRegressionSummary::ReportContext_t &rc) = 0;

	virtual void build(boost::shared_ptr<api::IRegressionFitAnalysisRegressionSummary::RawDataCollector_t> collector) = 0;

	virtual void setName(QString name) = 0;

	virtual QString getName() = 0;

	virtual void configurationUpdateRequested() = 0;

	virtual bool supportsReporting() = 0;

	virtual bool supportRawDataExport() = 0;

};

} //namespace api
} //namespace model
} //namespace


#endif /* IREGRESSIONFITANALYSISREGRESSIONSUMMARY_HPP_ */
