/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * IRegressionFitAnalysisRegressorOverview.hpp
 *
 *  Created on: 18.02.2015
 *      Author: gerberma
 */

#ifndef IREGRESSIONFITANALYSISREGRESSOROVERVIEW_HPP_
#define IREGRESSIONFITANALYSISREGRESSOROVERVIEW_HPP_

#include <boost/enable_shared_from_this.hpp>
#include "model/SignalPolicies.hpp"
#include "communication/Signals.hpp"
#include "model/r/api/IRRegressionFitAnalysisRegressorOverview.hpp"
#include "IModeHandler.hpp"
#include "IAnalysisTool.hpp"

namespace dambase {
namespace model {
namespace api {


class IRegressionFitAnalysisRegressorOverview : public IAnalysisTool, public boost::enable_shared_from_this<IRegressionFitAnalysisRegressorOverview>{
public:
	typedef r::api::IRRegressionFitAnalysisRegressorOverview::RegressorItemContainer_t RegressorItemContainer_t;
	typedef r::api::IRRegressionFitAnalysisRegressorOverview::RegressorItem_t RegressorItem_t;
	typedef r::api::IRRegressionFitAnalysisRegressorOverview::InterceptItem_t InterceptItem_t;
	typedef IModeHandler ModeHandler_t;
public:
	IRegressionFitAnalysisRegressorOverview(){;}
	virtual ~IRegressionFitAnalysisRegressorOverview(){;}

	virtual void execute() = 0;

	virtual RegressorItemContainer_t getRegressorItems() = 0;

	virtual boost::shared_ptr<InterceptItem_t> getInterceptItem() = 0;

	virtual void setRegressorActive(int regressorIndex, bool isActive) = 0;

	virtual bool isScientific() = 0;

	virtual boost::shared_ptr<ModeHandler_t> getModeHandler() = 0;

	virtual void build(api::IRegressionFitAnalysisRegressorOverview::ReportContext_t &rc) = 0;

	virtual void build(boost::shared_ptr<api::IRegressionFitAnalysisRegressorOverview::RawDataCollector_t> collector) = 0;

	virtual void setName(QString name) = 0;

	virtual QString getName() = 0;

	virtual void configurationUpdateRequested() = 0;

	virtual bool supportsReporting() = 0;

	virtual bool supportRawDataExport() = 0;
};

} //namespace api
} //namespace model
} //namespace


#endif /* IREGRESSIONFITANALYSISREGRESSOROVERVIEW_HPP_ */
