/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * IRegressionFitAnalysisResidualTimeSeries.hpp
 *
 *  Created on: 17.02.2015
 *      Author: gerberma
 */

#ifndef IREGRESSIONFITANALYSISRESIDUALTIMESERIES_HPP_
#define IREGRESSIONFITANALYSISRESIDUALTIMESERIES_HPP_

#include <boost/enable_shared_from_this.hpp>
#include "r/api/IRRegressionFitAnalysisResidualTimeSeries.hpp"
#include "IAnalysisTool.hpp"

namespace dambase {
namespace model {
namespace api {

class IRegressionFitAnalysisResidualTimeSeries : public IAnalysisTool, public boost::enable_shared_from_this<IRegressionFitAnalysisResidualTimeSeries> {
public:
	typedef r::api::IRRegressionFitAnalysisResidualTimeSeries::ResidualContainer_t ResidualContainer_t;
	typedef r::api::IRRegressionFitAnalysisResidualTimeSeries::DateContainer_t DateContainer_t;
	typedef r::api::IRRegressionFitAnalysisResidualTimeSeries::ConfidenceIntervalContainer_t ConfidenceIntervalContainer_t;
	typedef r::api::IRRegressionFitAnalysisResidualTimeSeries::OutOfLimitContainer_t OutOfLimitContainer_t;
	typedef r::api::IRRegressionFitAnalysisResidualTimeSeries::Date_t Date_t;
	typedef r::api::IRRegressionFitAnalysisResidualTimeSeries::ExtrapolationsContainer_t ExtrapolationsContainer_t;
public:
	IRegressionFitAnalysisResidualTimeSeries(){;}
	virtual ~IRegressionFitAnalysisResidualTimeSeries(){;}

	virtual api::IRegressionFitAnalysisResidualTimeSeries::Date_t getRegressionStartDate() = 0;

	virtual api::IRegressionFitAnalysisResidualTimeSeries::Date_t getRegressionEndDate() = 0;

	virtual ResidualContainer_t getResiduals() = 0;

	virtual DateContainer_t getDates() = 0;

	virtual ResidualContainer_t getPredictionResiduals() = 0;

	virtual DateContainer_t getPredictionDates() = 0;

	virtual ConfidenceIntervalContainer_t getUpperCi() = 0;

	virtual ConfidenceIntervalContainer_t getLowerCi() = 0;

	virtual OutOfLimitContainer_t getOutOfLimit() = 0;

	virtual DateContainer_t &getOutOfLimitDates() = 0;

	virtual ExtrapolationsContainer_t getExtrapolations() = 0;

	virtual DateContainer_t getExtrapolationDates() = 0;

	virtual void execute() = 0;

	virtual void build(ReportContext_t &rc) = 0;

	virtual void build(boost::shared_ptr<RawDataCollector_t> collector) = 0;

	virtual void setName(QString name) = 0;

	virtual QString getName() = 0;

	virtual void configurationUpdateRequested() = 0;

	virtual bool supportsReporting() = 0;

	virtual bool supportRawDataExport() = 0;
};

} //namespace api
} //namespace model
} //namespace

#endif /* IREGRESSIONFITANALYSISRESIDUALTIMESERIES_HPP_ */
