/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * IRegressionModelAnalysisToolsDescription.hpp
 *
 *  Created on: 16.02.2015
 *      Author: gerberma
 */

#ifndef IREGRESSIONMODELANALYSISTOOLSDESCRIPTION_HPP_
#define IREGRESSIONMODELANALYSISTOOLSDESCRIPTION_HPP_

#include <boost/shared_ptr.hpp>
#include <vector>
#include "IAnalysisToolDescription.hpp"
#include "IAnalysisToolsModeDescription.hpp"
#include "IModeHandler.hpp"
#include "communication/Signals.hpp"

namespace dambase {
namespace model {
namespace api {

class IRegressionModelAnalysisToolsDescription {
public:
	typedef IAnalysisToolDescription tool_description_t;
	typedef tool_description_t AnalysisToolDescription_t;
	typedef std::vector<boost::shared_ptr<tool_description_t> > container_t;
	typedef IModeHandler ModeHandler_t;
	typedef IAnalysisToolsModeDescription AnalysisToolsModeDescription_t;
public:
	IRegressionModelAnalysisToolsDescription(){;}
	virtual ~IRegressionModelAnalysisToolsDescription(){;}

	virtual container_t getPhysicsAnalysisToolsDescription() = 0;

	virtual container_t getStatisticsAnalysisToolsDescription() = 0;

	virtual boost::shared_ptr<tool_description_t> findStatisticsAnalysisToolDescriptionById(QString id) = 0;

	virtual boost::shared_ptr<tool_description_t> findPhysicsAnalysisToolDescriptionById(QString id) = 0;

	//left side
	virtual container_t getAutomaticLeftSidePhysicalCategoryAnalysisToolDescriptions() = 0;

	virtual container_t getAutomaticLeftSideStatisticsCategoryAnalysisToolDescriptions() = 0;

	//right side
	virtual container_t getAutomaticRightSidePhysicalCategoryAnalysisToolDescriptions() = 0;

	virtual container_t getAutomaticRightSideStatisticsCategoryAnalysisToolDescriptions() = 0;

	virtual boost::shared_ptr<ModeHandler_t> getModeHandler() = 0;
};

} //namespace api
} //namespace model
} //namespace

#endif /* IREGRESSIONMODELANALYSISTOOLSDESCRIPTION_HPP_ */
