/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * IReport.hpp
 *
 *  Created on: 17.02.2015
 *      Author: gerberma
 */

#ifndef IREPORT_HPP_
#define IREPORT_HPP_

#include <boost/shared_ptr.hpp>
#include "IReportListElement.hpp"
#include "ITitleReportListElement.hpp"
#include "IDescriptionReportListElement.hpp"
#include "IProjectReportListElement.hpp"
#include "IAlgorithmReportListElement.hpp"
#include "IAnalysisToolReportListElement.hpp"
#include "IProject.hpp"
#include "IAnalysis.hpp"
#include "IRegressionModel.hpp"
#include "model/SignalPolicies.hpp"
#include "communication/Signals.hpp"

namespace dambase {
namespace model {
namespace api {

namespace detail {
class ReportSignals : public communication::BoostSignalPolicy {
public:
	ReportSignals(){;}
	virtual ~ReportSignals(){;}

	COM_SUBSCRIPTION_FUNCTION(MoveReportListElement, communication::Signal<void(int, int)>);
	COM_SUBSCRIPTION_FUNCTION(WriteSuccessful, communication::Signal<void()>);
	COM_SUBSCRIPTION_FUNCTION(WriteFailed, communication::Signal<void()>);
	COM_SUBSCRIPTION_FUNCTION(WriteTimedOut, communication::Signal<void(int)>);

	void fireMoveReportListElement(int fromIndex, int toIndex) {
		communication::BoostSignalPolicy::fire();
		MoveReportListElementSignal(fromIndex, toIndex);
	}
};
} // namespace detail


class IReport : public detail::ReportSignals {
public:
	typedef IReportListElement ReportListElement_t;
	typedef ITitleReportListElement TitleReportListElement_t;
	typedef IDescriptionReportListElement DescriptionReportListElement_t;
	typedef IProjectReportListElement ProjectReportListElement_t;
	typedef IAlgorithmReportListElement AlgorithmReportListElement_t;
	typedef IAnalysisToolReportListElement AnalysisToolReportListElement_t;
	typedef std::vector<boost::shared_ptr<AnalysisToolReportListElement_t> > AnalysisToolListElementContainer_t;
	typedef std::vector<boost::shared_ptr<IReportListElement> > ReportListElementContainer_t;
	typedef std::vector<double> AvailablePlotScaleFactorsContainer_t;
	typedef IProject Project_t;
	typedef IAnalysis Analysis_t;
	typedef IRegressionModel RegressionModel_t;
public:
	IReport(){;}
	virtual ~IReport(){;}

	virtual QString getOutputFileName() = 0;

	virtual void setOutputFileName(QString outputFileName) = 0;

	virtual void build() = 0;

	virtual bool write() = 0;

	virtual boost::shared_ptr<api::IReport::TitleReportListElement_t> getTitleReportListElement() = 0;

	virtual void setTitleReportListElement(boost::shared_ptr<TitleReportListElement_t> titleReportListElement) = 0;

	virtual boost::shared_ptr<api::IReport::DescriptionReportListElement_t> getDescriptionReportListElement() = 0;

	virtual void setDescriptionReportListElement(boost::shared_ptr<DescriptionReportListElement_t> descriptionReportListElement) = 0;

	virtual bool isDescriptionReportListElement(boost::shared_ptr<api::IReport::ReportListElement_t> descriptionReportListElement) = 0;

	virtual boost::shared_ptr<api::IReport::ProjectReportListElement_t> getProjectReportListElement() = 0;

	virtual void setProjectReportListElement(boost::shared_ptr<ProjectReportListElement_t> projectReportListElement) = 0;

	virtual bool isProjectReportListElement(boost::shared_ptr<api::IReport::ReportListElement_t> element) = 0;

	virtual boost::shared_ptr<api::IReport::AlgorithmReportListElement_t> getAlgorithmReportListElement() = 0;

	virtual void setAlgorithmReportListElement(boost::shared_ptr<AlgorithmReportListElement_t> algorithmReportListElement) = 0;

	virtual bool isAlgorithmReportListElement(boost::shared_ptr<api::IReport::ReportListElement_t> element) = 0;

	virtual api::IReport::ReportListElementContainer_t &getReportListElements() = 0;

	virtual void setReportListElements(api::IReport::ReportListElementContainer_t &reportListElements) = 0;

	virtual bool isAnalysisToolListElement(boost::shared_ptr<ReportListElement_t> reportListElement) = 0;

	virtual boost::shared_ptr<api::IReport::AnalysisToolReportListElement_t> getAnalysisToolListElementFor(boost::shared_ptr<api::IReport::ReportListElement_t> reportListElement) = 0;

	virtual void setAnalysisToolReportListElements(AnalysisToolListElementContainer_t &analysisToolListElements) = 0;

	virtual QString getSupportedFileTypesAsString() = 0;

	virtual void moveUp(boost::shared_ptr<api::IReport::ReportListElement_t> reportListElement) = 0;

	virtual void moveDown(boost::shared_ptr<api::IReport::ReportListElement_t> reportListElement) = 0;

	virtual int findIndexOf(boost::shared_ptr<api::IReport::ReportListElement_t> reportListElement) = 0;

	virtual void exchange(int elementIndex, int withElementIndex) = 0;

	virtual std::vector<double> getAvailablePlotScaleFactors() = 0;

	virtual double getPlotScaleFactor() = 0;

	virtual void setPlotScaleFactor(double plotSize) = 0;

	virtual std::vector<int> getAvailableFontSizes() = 0;

	virtual int getFontSize() = 0;

	virtual void setFontSize(int fontSize) = 0;

	virtual void writeSuccessful() = 0;

	virtual void writeTimedOut() = 0;

	virtual void writeFailed() = 0;

	virtual bool hasFailed() = 0;

	virtual QString getFailMessage() = 0;
};

} //namespace api
} //namespace model
} //namespace

#endif /* IREPORT_HPP_ */
