/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataFrame.hpp
 *
 *  Created on: Dec 18, 2013
 *      Author: gerberma
 */

#ifndef DATAFRAME_HPP_
#define DATAFRAME_HPP_

#include <QString>
#include <QDebug>
#include <Rcpp.h>
#include <vector>
#include <set>
#include <algorithm>
#include <iterator>
#include <boost/bind.hpp>
#include <boost/foreach.hpp>
#include "applicationcontext/scopes.hpp"
#include "memory/OwnerhshipPolicies.hpp"
#include "DateColumn.hpp"
#include "BehaviorIndicatorColumn.hpp"
#include "WaterLevelColumn.hpp"
#include "TemperatureColumn.hpp"
#include "DataFrameHeader.hpp"
#include "api/IDataFrame.hpp"

namespace dambase {
namespace model {
namespace r{

/**
 * Rule: clients get values from stl or Qt or our own type from here but the implementations
 * refer to Rcpp proxies (see DataColum for an example)
 */

class DataFrame : virtual public api::IDataFrame
{
public:
	DataFrame();
	DataFrame(Rcpp::DataFrame &dataFrame);
	virtual ~DataFrame();

	void updateSomeRawValues();
	void updateFromApplyDateFormat(Rcpp::DataFrame dataFrame);
	DataFrameHeader &getHeader(int at);

	Header_Container_t& getHeaders();

	std::vector<QString> getSomeRawValuesFor(QString headerName);

	const std::map<QString, std::vector<QString> > &getRawValues() const;

	boost::shared_ptr<Column_t> getColumnByIndex(int index);

	const Assignable_Header_Container_t getAssignableDateColumnHeaders();

	const Assignable_Header_Container_t getAssignableBehaviorIndicatorHeaders();

	const Assignable_Header_Container_t getAssignableWaterLevelHeaders();

	const Assignable_Header_Container_t getAssignableTemperatureHeaders();

	int getNrOfRows();

	int getNrOfColumns();

	/*
	template<typename T>
	T getRow(int at) {
		return dataFrame[at];
	}
	*/

	bool setDateColumn(int fromIndex, bool forceUpdate = false);

	/**
	 * This is called from the deserialization process
	 */
	void setDateColumn(boost::shared_ptr<DateColumn> dateColumn);

	const boost::shared_ptr<DateColumn> &getDateColumn() const;

	boost::shared_ptr<DateColumn> getDateColumn(Rcpp::DataFrame forDataFrame);

	int estimateDateColumn();

	bool assignBehaviorIndicatorColumn(int fromIndex);

	bool isBehaviorIndicatorColumnAssignedFor(int index);

	bool removeBehaviorIndicatorColumn(int fromIndex);

	const Behavior_Indicator_Column_Container_t &getBehaviorIndicatorColumns() const;

	void setBehaviorIndicatorColumns(Behavior_Indicator_Column_Container_t &columns);


	bool assignWaterLevelColumn(int fromIndex);

	bool isWaterLevelColumnAssignedFor(int index);

	bool removeWaterLevelColumn(int fromIndex);

	boost::shared_ptr<WaterLevelColumn> findWaterLevelColumnByName(QString name);

	const Water_Level_Column_Container_t &getWaterLevelColumns() const;

	void setWaterLevelColumns(Water_Level_Column_Container_t &columns);

	bool assignTemperatureColumn(int fromIndex);

	bool isTemperatureColumnAssignedFor(int index);

	bool removeTemperatureColumn(int fromIndex);

	const Temperature_Column_Container_t &getTemperatureColumns() const;

	void setTemperatureColumns(Temperature_Column_Container_t &columns);

	Influence_Factor_Container_t getAssignedInfluenceFactorsFor(boost::shared_ptr<api::IAssignmentCategory> assignmentCategory);

	void setDataFrameFromDeserialization(Rcpp::DataFrame dataFrame);

	void calculateNonPointable();

	/**
	 * Use this only for methods based on the real dataFrame for other calculations
	 */
	Rcpp::DataFrame &getDataFrame();

	/**
	 * For deserialization
	 */
	const Rcpp::DataFrame &getDataFrameConst() const;

private:
	Rcpp::DataFrame dataFrame;
	std::vector<DataFrameHeader> headers;
	std::map<QString, std::vector<QString> > someRawValues;
	boost::shared_ptr<DateColumn> dateColumn;
	Behavior_Indicator_Column_Container_t behaviorIndicatorColumns;
	Water_Level_Column_Container_t waterLevelColumns;
	Temperature_Column_Container_t temperatureColumns;
};



}}} /* namespace dambase::model::r */
#endif /* DATAFRAME_HPP_ */
