/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataFrameInfluenceFactorsProvider.cpp
 *
 *  Created on: 18.02.2015
 *      Author: gerberma
 */

#include <Rcpp.h>
#include "DataFrameInfluenceFactorsProvider.hpp"

namespace dambase {
namespace model {
namespace r {

DataFrameInfluenceFactorsProvider::DataFrameInfluenceFactorsProvider(){;}
DataFrameInfluenceFactorsProvider::~DataFrameInfluenceFactorsProvider(){;}

boost::shared_ptr<api::IDataFrameInfluenceFactors>
DataFrameInfluenceFactorsProvider::load(
		boost::shared_ptr<api::IDataFrameInfluenceFactorsProvider::DataFrame_t> dataFrame,
		boost::shared_ptr<api::IDataFrameInfluenceFactorsProvider::Column_t> behaviorIndicatorColumn,
		boost::shared_ptr<api::IDataFrameInfluenceFactorsProvider::Column_t> influenceFactorColumn)
{
	Rcpp::Environment damBaseEnv("package:DamBASE");
	Rcpp::Function loadDataFrameInfluenceFactors = damBaseEnv["loadDataFrameInfluenceFactors"];

	Rcpp::List values = loadDataFrameInfluenceFactors(
				Rcpp::Named("dataFrame", dataFrame->getDataFrame()),
				Rcpp::Named("behaviorIndicatorColumnNr", behaviorIndicatorColumn->getIndex() + 1),
				Rcpp::Named("influenceFactorColumnNr", influenceFactorColumn->getIndex() + 1)
			);

	boost::shared_ptr<api::IDataFrameInfluenceFactors> result(new DataFrameInfluenceFactors(values));

	return result;
}

} //namespace r
} //namespace model
} //namespace dambase
