/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataFrameProvider.cpp
 *
 *  Created on: 17.02.2015
 *      Author: gerberma
 */

#include "DataFrameProvider.hpp"
#include "DataFrame.hpp"

namespace dambase {
namespace model {
namespace r {

DataFrameProvider::DataFrameProvider(){;}
DataFrameProvider::~DataFrameProvider(){;}

boost::shared_ptr<api::IDataFrameProvider::data_frame_type>
DataFrameProvider::load(QUrl uri, QString csvDelimiter, unsigned missingDataStrategy) {
	Rcpp::Environment damBaseEnv("package:DamBASE");
	Rcpp::Function loadDataFrame = damBaseEnv["loadDataFrame"];

	Rcpp::DataFrame dataFrame = loadDataFrame(
				Rcpp::Named("url", uri.toString().toStdString()),
				Rcpp::Named("csvDelimiter", csvDelimiter.toStdString()),
				Rcpp::Named("missingDataStrategy", missingDataStrategy)
			);

	boost::shared_ptr<api::IDataFrameProvider::data_frame_type> result (new DataFrame(dataFrame));
	return result;
}

void
DataFrameProvider::saveDataFrame(const Rcpp::DataFrame &dataFrame, QString location, boost::shared_ptr<api::IDataFrameProvider::DateColumn_t> dateColumn) {
	Rcpp::Environment damBaseEnv("package:DamBASE");
	Rcpp::Function saveDataFrameFunction = damBaseEnv["saveDataFrame"];

	saveDataFrameFunction(
				Rcpp::Named("dataFrame", dataFrame),
				Rcpp::Named("dateColumnNr", dateColumn->getIndex() + 1),
				Rcpp::Named("location", location.toStdString())
			);
}

Rcpp::DataFrame
DataFrameProvider::readDataFrame(QString location, boost::shared_ptr<api::IDataFrameProvider::DateColumn_t> dateColumn) {
	Rcpp::Environment damBaseEnv("package:DamBASE");
	Rcpp::Function readDataFrameFunction = damBaseEnv["readDataFrame"];

	return Rcpp::as<Rcpp::DataFrame>(readDataFrameFunction(
				Rcpp::Named("location", location.toStdString()),
				Rcpp::Named("dateColumnNr", dateColumn->getIndex() + 1)
			));
}

Rcpp::DataFrame
DataFrameProvider::applyDateFormat(Rcpp::DataFrame &dataFrame, int dateColumnNr, QString dateFormat) {
	Rcpp::Environment damBaseEnv("package:DamBASE");
	Rcpp::Function applyDateFormat = damBaseEnv["applyDateFormat"];

	return Rcpp::as<Rcpp::DataFrame>(applyDateFormat(
				Rcpp::Named("dataFrame", dataFrame),
				Rcpp::Named("dateColumnNr", dateColumnNr + 1),
				Rcpp::Named("dateFormat", dateFormat.toStdString())
			));
}

bool
DataFrameProvider::testDateColumnValidity(Rcpp::DataFrame &dataFrame, int dateColumnNr) {
	Rcpp::Environment damBaseEnv("package:DamBASE");
	Rcpp::Function testDateColumnValidity = damBaseEnv["testDateColumnValidity"];

	return Rcpp::as<bool>(testDateColumnValidity(
				Rcpp::Named("dataFrame", dataFrame),
				Rcpp::Named("dateColumnNr", dateColumnNr + 1)
			));
}

Rcpp::DataFrame
DataFrameProvider::applyMissingDataStrategy(Rcpp::DataFrame &dataFrame, unsigned missingDataStrategy) {
	Rcpp::Environment damBaseEnv("package:DamBASE");
	Rcpp::Function applyMissingDataStrategy = damBaseEnv["applyMissingDataStrategy"];

	return Rcpp::as<Rcpp::DataFrame>(applyMissingDataStrategy(
				Rcpp::Named("dataFrame", dataFrame),
				Rcpp::Named("missingDataStrategy", missingDataStrategy)
			));
}

} //namespace r
} //namespace model
} //namespace dambase
