/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DateColumn.hpp
 *
 *  Created on: Dec 20, 2013
 *      Author: gerberma
 */

#ifndef DATECOLUMN_HPP_
#define DATECOLUMN_HPP_

#include <Rcpp.h>
#include <boost/foreach.hpp>
#include <string>
#include <QString>
#include "DataFrameHeader.hpp"
#include "DateDataPoint.hpp"
#include "DataFrameColumn.hpp"

namespace dambase {
namespace model {
namespace r {

class DateColumn : public DataFrameColumn {
public:
	typedef DateDataPoint Date_Data_Point_t;
	typedef std::vector<Date_Data_Point_t> DateContainer_t;
	typedef DataFrameColumn Base_t;
public:
	DateColumn()
		:DataFrameColumn()
	{;}

	DateColumn(DataFrameHeader &dataFrameHeader, Rcpp::CharacterVector values)
		:DataFrameColumn(dataFrameHeader)
	{
		{
			int index=0;
			BOOST_FOREACH(Rcpp::CharacterVector::value_type date, values) {
				dates.push_back(DateDataPoint(QString::fromStdString(Rcpp::as<std::string>(date)), index++));
			}
		}
	}

	void setValues(std::vector<DateDataPoint> values) {
		this->dates = values;
	}

	const std::vector<DateDataPoint> &getValues() const{
		return dates;
		//std::vector<DateDataPoint> values;

		/*
		int index = 0;
		BOOST_FOREACH(Rcpp::CharacterVector::value_type value, columnContent) {
			QString date = QString::fromStdString(Rcpp::as<std::string>(value));
			values.push_back(DateDataPoint(date, index++));
		}

		return values;
		*/
	}

	DateDataPoint getValueAt(int index) {
		/*
		Rcpp::CharacterVector::value_type value = columnContent[index];
		QString date = QString::fromStdString(Rcpp::as<std::string>(value));

		return DateDataPoint(date, index);
		*/
		if(index + 1 > dates.size()) {
			throw std::logic_error("No date available for index: " + QString::number(index).toStdString());
		}
		return dates.at(index);
	}

	DateDataPoint getValueAt(DateDataPoint &fromDateDataPoint, int relativeTimesteps) {
		return getValueAt(fromDateDataPoint.getIndex() + relativeTimesteps);
	}

	DateDataPoint getFirstValue() {
		return getValues().front();
	}

	DateDataPoint getLastValue() {
		return getValues().back();
	}

private:
	DateContainer_t dates;
};

}}} /* namespace dambase::model::r */

#endif /* DATECOLUMN_HPP_ */
