/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DateDataPoint.hpp
 *
 *  Created on: Feb 26, 2014
 *      Author: gerberma
 */

#ifndef DATEDATAPOINT_HPP_
#define DATEDATAPOINT_HPP_

#include <QString>
#include <boost/date_time/gregorian/gregorian.hpp>
#include <boost/date_time/posix_time/posix_time.hpp>

namespace dambase {
namespace model {
namespace r {

class DateDataPoint {
public:
	DateDataPoint()
		:date(""), index(0){;}

	DateDataPoint(QString date, int index) : date(date), index(index){;}

	void setDate(QString date) {
		this->date = date;
	}

	const QString &getDate() const{
		return date;
	}

	void setIndex(int index) {
		this->index = index;
	}

	const int &getIndex() const {
		return index;
	}

	bool operator==(DateDataPoint const &other) const {
		return asDate() == other.asDate();
	}

	bool operator<(DateDataPoint const &other) const {
		return asDate() < other.asDate();
	}

	bool operator>(DateDataPoint const &other) const {
		return asDate() > other.asDate();
	}

private:
	boost::gregorian::date asDate() const {
		return boost::gregorian::date(boost::gregorian::from_string(date.toStdString()));
	}

private:
	QString date;
	int index;
};


}}} /* namespace dambase::model::r */


#endif /* DATEDATAPOINT_HPP_ */
