/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * ExpressionRegressorFormulaBuilder.hpp
 *
 *  Created on: Aug 7, 2014
 *      Author: gerberma
 */

#ifndef EXPRESSIONREGRESSORFORMULABUILDER_HPP_
#define EXPRESSIONREGRESSORFORMULABUILDER_HPP_

#include "api/IExpressionRegressorFormulaBuilder.hpp"

namespace dambase {
namespace model {
namespace r {

class ExpressionRegressorFormulaBuilder : public virtual api::IExpressionRegressorFormulaBuilder {
public:
	ExpressionRegressorFormulaBuilder();

	ExpressionRegressorFormulaBuilder(QString identifier, std::vector<boost::shared_ptr<AssignmentCategory_t> > assignmentCategories, boost::shared_ptr<AssignmentCategory_t> assignmentCategory, std::vector<boost::shared_ptr<RegressorFormulaBuilderPreprocessor_t> > preprocessors, bool iseditable = false);

	ExpressionRegressorFormulaBuilder(boost::shared_ptr<api::IExpressionRegressorFormulaBuilder> original, QString newIdentifier, QString newFormula, bool iseditable = true);

	virtual ~ExpressionRegressorFormulaBuilder();

	const QString &getIdentifier() const;

	void setIdentifier(QString identifier);

	QString resolveExtendedIdentifier();

	QString resolveName(boost::shared_ptr<InfluenceFactor_t> influenceFactor);

	QString resolveName();

	void setFormula(QString formula);

	const QString &getFormula() const;

	void setFormulaDefinition(QString formulaDefinition);

	const QString &getFormulaDefinition() const;

	QString resolveFormulaDefinition(boost::shared_ptr<DataFrame_t> dataFrame, int overallMaxStartupTime);

	/**
	 * overallMaxStartupTime is the maximum startup time of all involved regressors
	 * Therefore, the dateColumn has to start at the date where the overallMaxStartupTime added points to the startup time of the regression
	 * So we calculate here a difference which tells us at what date we can start and provide this difference to the evaluateExpansion function
	 */
	QString resolveFormulaDefinition(boost::shared_ptr<DataFrame_t> dataFrame, boost::shared_ptr<InfluenceFactor_t> influenceFactor, int overallMaxStartupTime);

	bool hasPreprocessors();

	bool requiresInfluenceFactor();

	boost::shared_ptr<AssignmentCategory_t> findAssignmentCategoryByIdentifier(QString identifier);

	/**
	 * This is primarly used by the deserialization process.
	 */
	void setAssignmentCategories(std::vector<boost::shared_ptr<AssignmentCategory_t> > &assignmentCategories);

	std::vector<boost::shared_ptr<AssignmentCategory_t> > &getAssignmentCategories();

	void setAssignmentCategory(boost::shared_ptr<AssignmentCategory_t> assignmentCategory);

	boost::shared_ptr<AssignmentCategory_t> getAssignmentCategory() const;

	DataFrame_t::Influence_Factor_Container_t getAvailableInfluenceFactors(boost::shared_ptr<DataFrame_t> dataFrame);

	/**
	 * This is primarly used by the deserialization process
	 */
	void setIsEditable(bool iseditable);

	const bool &isEditable() const;

	/**
	 * This is primarly used by the deserialization process.
	 */
	void setUserParameters(UserParameters_t &userParameters);

	void addUserParameter(QString name, boost::shared_ptr<UserParameterType_t> userParameterType);

	const std::map<QString, boost::shared_ptr<UserParameterType_t> > &getUserParameters() const;

	void removeUserParameter(UserParameter_t userParameter);

	int calculateStartupTimesteps();

	const std::vector<boost::shared_ptr<RegressorFormulaBuilderPreprocessor_t> >& getPreprocessors() const;

	/**
	 * This is primarly used by the deserialization process as well as the clone process
	 */
	void setPreprocessors(std::vector<boost::shared_ptr<RegressorFormulaBuilderPreprocessor_t> >& preprocessors);

private:
	int calculateStartDiffFromBeginningOfTheRegressionFrame(int overallMaxStartupTime);

	int calculateFinalCutDiff(int overallMaxStartupTime, int startDiffFromBeginningOfTheRegressionFrame);

	int calculateFinalCutDiff(int overallMaxStartupTime);

	QString resolvePreprocessorsDefinition();

	bool hasActivePreprocessors();

	QString getEnhancedUserParameters();

	QString getEnhancedFormulaDefinition();
private:
	QString identifier;
	boost::shared_ptr<api::IAssignmentCategory> assignmentCategory;
	std::vector<boost::shared_ptr<api::IAssignmentCategory> > assignmentCategories;
	std::map<QString, boost::shared_ptr<UserParameterType_t> > userParameters;
	QString formula;
	QString formulaDefinition;
	bool iseditable;
	std::vector<boost::shared_ptr<api::IRegressorFormulaBuilderPreprocessor> > preprocessors;
};

}  // namespace r
}  // namespace model
}  // namespace dambase


#endif /* EXPRESSIONREGRESSORFORMULABUILDER_HPP_ */
