/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * MinMaxOpLevelRegressorFormulaBuilderPreprocessor.cpp
 *
 *  Created on: 16.02.2015
 *      Author: gerberma
 */

#include "MinMaxOpLevelRegressorFormulaBuilderPreprocessor.hpp"

namespace dambase {
namespace model {
namespace r {

MinMaxOpLevelRegressorFormulaBuilderPreprocessor::MinMaxOpLevelRegressorFormulaBuilderPreprocessor(boost::shared_ptr<MinMaxOpLevelRegressorFormulaBuilderPreprocessor> original)
{
	minMaxOpLevelStrategy = original->getMinMaxOpLevelStrategy();
	name = original->getName();
	functionCallName = original->getFunctionCallName();
}

MinMaxOpLevelRegressorFormulaBuilderPreprocessor::MinMaxOpLevelRegressorFormulaBuilderPreprocessor(boost::shared_ptr<WaterLevelScalingMinMaxOpLevel_t> strategy, QString name, QString functionCallName)
	:minMaxOpLevelStrategy(strategy), name(name), functionCallName(functionCallName)
{;}

MinMaxOpLevelRegressorFormulaBuilderPreprocessor::MinMaxOpLevelRegressorFormulaBuilderPreprocessor()
	: name(""), functionCallName("")
{;}

MinMaxOpLevelRegressorFormulaBuilderPreprocessor::~MinMaxOpLevelRegressorFormulaBuilderPreprocessor() {

}

/**
 * Note, this is for serialization process only
 */
const QString &
MinMaxOpLevelRegressorFormulaBuilderPreprocessor::getFunctionCallName() const {
	return functionCallName;
}

/**
 * Note, this is for deserialization process only
 */
void
MinMaxOpLevelRegressorFormulaBuilderPreprocessor::setFunctionCallName(QString functionCallName) {
	this->functionCallName = functionCallName;
}

QString
MinMaxOpLevelRegressorFormulaBuilderPreprocessor::resolveFormula() {

	QString formula = QString("{") + functionCallName + QString("(wlevel = col_")
				+ QString(", min = ") + QString::number(minMaxOpLevelStrategy->getMinimumOperatingLevel())
				+ QString(", max = ") + QString::number(minMaxOpLevelStrategy->getFullSupplyLevel())
				+ QString(")}");
	return formula;
}

/**
 * Note, this is for deserialization process only
 */
void
MinMaxOpLevelRegressorFormulaBuilderPreprocessor::setName(QString name) {
	this->name = name;
}

const QString &
MinMaxOpLevelRegressorFormulaBuilderPreprocessor::getName() const {
	return name;
}

/**
 * The active state depends on whether the min/max op level strategy has been enabled or disabled.
 * If it is disabled, then this preprocessor is not active.
 * If however, the strategy is enabled, then it depends on what the user has been set.
 */
const bool &
MinMaxOpLevelRegressorFormulaBuilderPreprocessor::isActive() const {
	if( minMaxOpLevelStrategy->isEnabled() == false) {
		return false;
	}else{
		return api::IRegressorFormulaBuilderPreprocessor::isActive();
	}
}

const bool &
MinMaxOpLevelRegressorFormulaBuilderPreprocessor::isEnabled() const {
	return minMaxOpLevelStrategy->isEnabled();
}

void
MinMaxOpLevelRegressorFormulaBuilderPreprocessor::setMinMaxOpLevelStrategy(boost::shared_ptr<MinMaxOpLevelRegressorFormulaBuilderPreprocessor::WaterLevelScalingMinMaxOpLevel_t> strategy) {
	minMaxOpLevelStrategy = strategy;
}

const boost::shared_ptr<MinMaxOpLevelRegressorFormulaBuilderPreprocessor::WaterLevelScalingMinMaxOpLevel_t> &
MinMaxOpLevelRegressorFormulaBuilderPreprocessor::getMinMaxOpLevelStrategy() const {
	return minMaxOpLevelStrategy;
}


} //namespace r
} //namespace model
} //namespace dambase
