/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressorFormulaBuilderPreprocessors.hpp
 *
 *  Created on: 12.11.2014
 *      Author: gerberma
 */

#ifndef REGRESSORFORMULABUILDERPREPROCESSORS_HPP_
#define REGRESSORFORMULABUILDERPREPROCESSORS_HPP_

#include <QString>
#include <boost/shared_ptr.hpp>
#include "api/IRegressorFormulaBuilderPreprocessor.hpp"
#include "model/api/IWaterLevelScalingMinMaxOpLevel.hpp"

namespace dambase {
namespace model {
namespace r {

class MinMaxOpLevelRegressorFormulaBuilderPreprocessor : public api::IRegressorFormulaBuilderPreprocessor {
public:
	typedef model::api::IWaterLevelScalingMinMaxOpLevel WaterLevelScalingMinMaxOpLevel_t;
public:
	MinMaxOpLevelRegressorFormulaBuilderPreprocessor(boost::shared_ptr<MinMaxOpLevelRegressorFormulaBuilderPreprocessor> original);

	MinMaxOpLevelRegressorFormulaBuilderPreprocessor(boost::shared_ptr<WaterLevelScalingMinMaxOpLevel_t> strategy, QString name, QString functionCallName);

	MinMaxOpLevelRegressorFormulaBuilderPreprocessor();
	virtual ~MinMaxOpLevelRegressorFormulaBuilderPreprocessor();

	/**
	 * Note, this is for serialization process only
	 */
	const QString &getFunctionCallName() const;

	/**
	 * Note, this is for deserialization process only
	 */
	void setFunctionCallName(QString functionCallName);

	QString resolveFormula();

	/**
	 * Note, this is for deserialization process only
	 */
	void setName(QString name);

	const QString &getName() const;

	/**
	 * The active state depends on whether the min/max op level strategy has been enabled or disabled.
	 * If it is disabled, then this preprocessor is not active.
	 * If however, the strategy is enabled, then it depends on what the user has been set.
	 */
	const bool &isActive() const;

	const bool &isEnabled() const;

	void setMinMaxOpLevelStrategy(boost::shared_ptr<WaterLevelScalingMinMaxOpLevel_t> strategy);

	const boost::shared_ptr<WaterLevelScalingMinMaxOpLevel_t> &getMinMaxOpLevelStrategy() const;

private:
	boost::shared_ptr<WaterLevelScalingMinMaxOpLevel_t> minMaxOpLevelStrategy;
	QString name;
	QString functionCallName;
};

}  // namespace r
}  // namespace model
}  // namespace dambase

#endif /* REGRESSORFORMULABUILDERPREPROCESSORS_HPP_ */
