/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RRegressionFitAnalysisACF.cpp
 *
 *  Created on: 09.02.2015
 *      Author: gerberma
 */

#include "RRegressionFitAnalysisACF.hpp"
#include <vector>
#include <boost/foreach.hpp>
#include <Rcpp.h>

namespace dambase {
namespace model {
namespace r {

RRegressionFitAnalysisACF::RRegressionFitAnalysisACF(Rcpp::List values) {
	Rcpp::NumericVector rAcfs = Rcpp::as<Rcpp::NumericVector>(values["acf"]);

	BOOST_FOREACH(double acf, rAcfs) {
		acfs.push_back(acf);
	}

	Rcpp::NumericVector rLags = Rcpp::as<Rcpp::NumericVector>(values["lag"]);
	BOOST_FOREACH(double lag, rLags) {
		lags.push_back(lag);
	}

	lowerCi = Rcpp::as<Rcpp::NumericVector>(values["ci.interval"])[0];
	upperCi = Rcpp::as<Rcpp::NumericVector>(values["ci.interval"])[1];

}

RRegressionFitAnalysisACF::~RRegressionFitAnalysisACF() {
}

api::IRRegressionFitAnalysisACF::AcfContainer_t &
RRegressionFitAnalysisACF::getAcfs() {
	return acfs;
}

api::IRRegressionFitAnalysisACF::LagContainer_t &
RRegressionFitAnalysisACF::getLags() {
	return lags;
}

double
RRegressionFitAnalysisACF::getLowerCi() {
	return lowerCi;
}

double
RRegressionFitAnalysisACF::getUpperCi() {
	return upperCi;
}

} /* namespace r */
} /* namespace model */
} /* namespace dambase */
