/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RRegressionFitAnalysisOverallRegressionQuality.cpp
 *
 *  Created on: 09.02.2015
 *      Author: gerberma
 */

#include "RRegressionFitAnalysisOverallRegressionQuality.hpp"
#include <Rcpp.h>

namespace dambase {
namespace model {
namespace r {

RRegressionFitAnalysisOverallRegressionQuality::RRegressionFitAnalysisOverallRegressionQuality(Rcpp::List values)
{
	if(values.containsElementNamed("autocor.dwstat.value") == false
			|| values.containsElementNamed("autocor.dwstat.degree") == false
			|| values.containsElementNamed("fit.r2adj.value") == false
			|| values.containsElementNamed("fit.r2adj.degree") == false)
	{
		throw std::logic_error("incomplete definition of OverallRegressionQuality");
	}

	autocorDwStatValue = Rcpp::as<double>(values["autocor.dwstat.value"]);
	autocorDwStatDegree = Rcpp::as<double>(values["autocor.dwstat.degree"]);
	fitR2adjValue = Rcpp::as<double>(values["fit.r2adj.value"]);
	fitR2adjDegree = Rcpp::as<double>(values["fit.r2adj.degree"]);
}

RRegressionFitAnalysisOverallRegressionQuality::~RRegressionFitAnalysisOverallRegressionQuality(){;}

double
RRegressionFitAnalysisOverallRegressionQuality::getAutocorDwStatDegree() const {
	return autocorDwStatDegree;
}

double
RRegressionFitAnalysisOverallRegressionQuality::getAutocorDwStatValue() const {
	return autocorDwStatValue;
}

double
RRegressionFitAnalysisOverallRegressionQuality::getFitR2adjDegree() const {
	return fitR2adjDegree;
}

double
RRegressionFitAnalysisOverallRegressionQuality::getFitR2adjValue() const {
	return fitR2adjValue;
}

} //namespace r
} //namespace model
} //namespace dambase



