/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RRegressionFitAnalysisPhysicalBehavior.cpp
 *
 *  Created on: 09.02.2015
 *      Author: gerberma
 */

#include "RRegressionFitAnalysisPhysicalBehavior.hpp"

namespace dambase {
namespace model {
namespace r {

RRegressionFitAnalysisPhysicalBehavior::RRegressionFitAnalysisPhysicalBehavior(Rcpp::List values, QString name) : name(name) {
	Rcpp::NumericVector rApplyedValues = Rcpp::as<Rcpp::NumericVector>(values["applyedValues"]);
	BOOST_FOREACH(double applyedValue, rApplyedValues) {
		applyedValues.push_back(applyedValue);
	}

	Rcpp::String rResponseType = Rcpp::as<Rcpp::String>(values["responseType"]);
	responseType = QString::fromStdString(rResponseType);

	if(responseType == "numeric") {
		Rcpp::NumericVector rResponseValues = Rcpp::as<Rcpp::NumericVector>(values["response"]);
		BOOST_FOREACH(double responseValue, rResponseValues) {
			numericResponse.push_back(responseValue);
		}
	}else if(responseType == "Date") {
		Rcpp::CharacterVector rDates = Rcpp::as<Rcpp::CharacterVector>(values["response"]);
		BOOST_FOREACH(Rcpp::CharacterVector::value_type date, rDates) {
			dateResponse.push_back(Rcpp::as<std::string>(date));
		}
	}else{
		throw std::logic_error("Unknown response type : " + responseType.toStdString());
	}
}

RRegressionFitAnalysisPhysicalBehavior::~RRegressionFitAnalysisPhysicalBehavior(){;}

api::IRRegressionFitAnalysisPhysicalBehavior::ApplyedValuesContainer_t &
RRegressionFitAnalysisPhysicalBehavior::getApplyedValues() {
	return applyedValues;
}

QString
RRegressionFitAnalysisPhysicalBehavior::getResponseType() {
	return responseType;
}

api::IRRegressionFitAnalysisPhysicalBehavior::DateResponseContainer_t &
RRegressionFitAnalysisPhysicalBehavior::getDateResponse() {
	return dateResponse;
}

api::IRRegressionFitAnalysisPhysicalBehavior::NumericResponseContainer_t
RRegressionFitAnalysisPhysicalBehavior::getNumericResponse() {
	return numericResponse;
}

QString
RRegressionFitAnalysisPhysicalBehavior::getName() {
	return name;
}

} //namespace r
} //namespace model
} //namespace dambase
