/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RRegressionFitAnalysisPhysicalSummary.cpp
 *
 *  Created on: 09.02.2015
 *      Author: gerberma
 */


#include "RRegressionFitAnalysisPhysicalSummary.hpp"

namespace dambase {
namespace model {
namespace r {

RRegressionFitAnalysisPhysicalSummary::RRegressionFitAnalysisPhysicalSummary(Rcpp::List values)
{
	{
		Rcpp::CharacterVector rDates = Rcpp::as<Rcpp::CharacterVector>(values["dateValues"]);
		BOOST_FOREACH(Rcpp::CharacterVector::value_type date, rDates) {
			dates.push_back(Rcpp::as<std::string>(date));
		}
	}

	{
		Rcpp::CharacterVector rDates = Rcpp::as<Rcpp::CharacterVector>(values["predictionDateValues"]);
		BOOST_FOREACH(Rcpp::CharacterVector::value_type date, rDates) {
			predictionDates.push_back(Rcpp::as<std::string>(date));
		}
	}

	Rcpp::NumericVector rBehaviorIndicatorValues = Rcpp::as<Rcpp::NumericVector>(values["behaviorIndicatorValues"]);
	BOOST_FOREACH(double value, rBehaviorIndicatorValues) {
		behaviorIndicatorValues.push_back(value);
	}

	Rcpp::NumericVector rPredictions = Rcpp::as<Rcpp::NumericVector>(values["predictions"]);
	BOOST_FOREACH(double value, rPredictions) {
		predictions.push_back(value);
	}

	Rcpp::NumericVector rLowerCi = Rcpp::as<Rcpp::NumericVector>(values["lowerCi"]);
	BOOST_FOREACH(double value, rLowerCi) {
		lowerCi.push_back(value);
	}

	Rcpp::NumericVector rUpperCi = Rcpp::as<Rcpp::NumericVector>(values["upperCi"]);
	BOOST_FOREACH(double value, rUpperCi) {
		upperCi.push_back(value);
	}



	Rcpp::NumericVector rOutOfLimit = Rcpp::as<Rcpp::NumericVector>(values["outOfLimit"]);
	BOOST_FOREACH(double value, rOutOfLimit) {
		outOfLimit.push_back(value);
	}

	Rcpp::CharacterVector rOutOfLimitDates = Rcpp::as<Rcpp::CharacterVector>(values["outOfLimitDates"]);
	BOOST_FOREACH(Rcpp::CharacterVector::value_type date, rOutOfLimitDates) {
		outOfLimitDates.push_back(Rcpp::as<std::string>(date));
	}


	Rcpp::NumericVector rExtrapolation = Rcpp::as<Rcpp::NumericVector>(values["extrapolation.values"]);
	BOOST_FOREACH(double value, rExtrapolation) {
		extrapolations.push_back(value);
	}

	Rcpp::CharacterVector rExtrapolationDate = Rcpp::as<Rcpp::CharacterVector>(values["extrapolation.dates"]);
	BOOST_FOREACH(Rcpp::CharacterVector::value_type date, rExtrapolationDate) {
		extrapolationDates.push_back(Rcpp::as<std::string>(date));
	}


	Rcpp::String rregressionStartDate = values["regressionStartDate"];
	regressionStartDate = rregressionStartDate;
	Rcpp::String rregressionEndDate = values["regressionEndDate"];
	regressionEndDate = rregressionEndDate;
}

RRegressionFitAnalysisPhysicalSummary::~RRegressionFitAnalysisPhysicalSummary(){;}

api::IRRegressionFitAnalysisPhysicalSummary::BehaviorIndicatorValueContainer_t &
RRegressionFitAnalysisPhysicalSummary::getBehaviorIndicatorValues() {
	return behaviorIndicatorValues;
}

api::IRRegressionFitAnalysisPhysicalSummary::PredictionsContainer_t &
RRegressionFitAnalysisPhysicalSummary::getPredictions() {
	return predictions;
}

api::IRRegressionFitAnalysisPhysicalSummary::ConfidenceIntervalContainer_t
RRegressionFitAnalysisPhysicalSummary::getUpperCi() {
	return upperCi;
}

api::IRRegressionFitAnalysisPhysicalSummary::ConfidenceIntervalContainer_t
RRegressionFitAnalysisPhysicalSummary::getLowerCi() {
	return lowerCi;
}

api::IRRegressionFitAnalysisPhysicalSummary::OutOfLimitContainer_t
RRegressionFitAnalysisPhysicalSummary::getOutOfLimit() {
	return outOfLimit;
}

api::IRRegressionFitAnalysisPhysicalSummary::DateContainer_t &
RRegressionFitAnalysisPhysicalSummary::getOutOfLimitDates() {
	return outOfLimitDates;
}

api::IRRegressionFitAnalysisPhysicalSummary::ExtrapolationsContainer_t
RRegressionFitAnalysisPhysicalSummary::getExtrapolations() {
	return extrapolations;
}

api::IRRegressionFitAnalysisPhysicalSummary::DateContainer_t
RRegressionFitAnalysisPhysicalSummary::getExtrapolationDates() {
	return extrapolationDates;
}

api::IRRegressionFitAnalysisPhysicalSummary::DateContainer_t &
RRegressionFitAnalysisPhysicalSummary::getDates() {
	return dates;
}

api::IRRegressionFitAnalysisPhysicalSummary::Date_t &
RRegressionFitAnalysisPhysicalSummary::getRegressionStartDate() {
	return regressionStartDate;
}

api::IRRegressionFitAnalysisPhysicalSummary::Date_t &
RRegressionFitAnalysisPhysicalSummary::getRegressionEndDate() {
	return regressionEndDate;
}

api::IRRegressionFitAnalysisPhysicalSummary::DateContainer_t &
RRegressionFitAnalysisPhysicalSummary::getPredictionDates() {
	return predictionDates;
}


} //namespace r
} //namespace model
} //namespace dambase

