/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RRegressionFitAnalysisPhysicalSummary.hpp
 *
 *  Created on: Mar 10, 2014
 *      Author: gerberma
 */

#ifndef RREGRESSIONFITANALYSISPHYSICALSUMMARY_HPP_
#define RREGRESSIONFITANALYSISPHYSICALSUMMARY_HPP_

#include <vector>
#include <string>
#include <boost/foreach.hpp>
#include <QString>
#include <Rcpp.h>
#include "api/IRRegressionFitAnalysisPhysicalSummary.hpp"


namespace dambase {
namespace model {
namespace r {


class RRegressionFitAnalysisPhysicalSummary : public virtual api::IRRegressionFitAnalysisPhysicalSummary {
public:
	typedef std::vector<double> BehaviorIndicatorValueContainer_t;
	typedef std::vector<double> PredictionsContainer_t;
	typedef std::vector<double> OutOfLimitContainer_t;
	typedef std::vector<double> ExtrapolationsContainer_t;
	typedef std::vector<double> ConfidenceIntervalContainer_t;
	typedef std::string Date_t;
	typedef std::vector<std::string> DateContainer_t;

public:
	RRegressionFitAnalysisPhysicalSummary(Rcpp::List values);
	~RRegressionFitAnalysisPhysicalSummary();

	BehaviorIndicatorValueContainer_t &getBehaviorIndicatorValues();
	PredictionsContainer_t &getPredictions();
	ConfidenceIntervalContainer_t getUpperCi();
	ConfidenceIntervalContainer_t getLowerCi();
	OutOfLimitContainer_t getOutOfLimit();
	DateContainer_t &getOutOfLimitDates();
	ExtrapolationsContainer_t getExtrapolations();
	DateContainer_t getExtrapolationDates();
	DateContainer_t &getDates();
	Date_t &getRegressionStartDate();
	Date_t &getRegressionEndDate();
	DateContainer_t &getPredictionDates();

private:
	BehaviorIndicatorValueContainer_t behaviorIndicatorValues;
	PredictionsContainer_t predictions;
	ConfidenceIntervalContainer_t upperCi;
	ConfidenceIntervalContainer_t lowerCi;
	OutOfLimitContainer_t outOfLimit;
	ExtrapolationsContainer_t extrapolations;
	DateContainer_t dates;
	DateContainer_t predictionDates;
	DateContainer_t outOfLimitDates;
	DateContainer_t extrapolationDates;
	Date_t regressionStartDate;
	Date_t regressionEndDate;
};

}  // namespace r
}  // namespace model
}  // namespace dambase

#endif /* RREGRESSIONFITANALYSISPHYSICALSUMMARY_HPP_ */
