/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RRegressionFitAnalysisRegressionSummary.cpp
 *
 *  Created on: 09.02.2015
 *      Author: gerberma
 */


#include "RRegressionFitAnalysisRegressionSummary.hpp"

namespace dambase {
namespace model {
namespace r {

RRegressionFitAnalysisRegressionSummary::RRegressionFitAnalysisRegressionSummary(Rcpp::List summary)
{
	if(summary.containsElementNamed("ssres") == false
			|| summary.containsElementNamed("msres") == false
			|| summary.containsElementNamed("msresderived") == false
			|| summary.containsElementNamed("r2") == false
			|| summary.containsElementNamed("r2adj") == false
			|| summary.containsElementNamed("fstat") == false
			|| summary.containsElementNamed("fstatpvalue") == false
			|| summary.containsElementNamed("dwstat") == false
			|| summary.containsElementNamed("nobs") == false
			|| summary.containsElementNamed("ssresPred") == false
			|| summary.containsElementNamed("msresderivedPred") == false
			)
	{
		throw std::logic_error("incomplete definition of RegressionSummary");
	}

	ssres = Rcpp::as<double>(summary["ssres"]);
	msresderived = Rcpp::as<double>(summary["msresderived"]);
	msres = Rcpp::as<double>(summary["msres"]);
	r2 = Rcpp::as<double>(summary["r2"]);
	r2adj = Rcpp::as<double>(summary["r2adj"]);
	fstat = Rcpp::as<Rcpp::NumericVector>(summary["fstat"])[0];
	fstatpvalue = Rcpp::as<double>(summary["fstatpvalue"]);
	dwstat = Rcpp::as<double>(summary["dwstat"]);
	nobs = Rcpp::as<double>(summary["nobs"]);
	ssresPred = Rcpp::as<double>(summary["ssresPred"]);
	msresderivedPred = Rcpp::as<double>(summary["msresderivedPred"]);
}

RRegressionFitAnalysisRegressionSummary::~RRegressionFitAnalysisRegressionSummary(){;}

double
RRegressionFitAnalysisRegressionSummary::getSSRes() {
	return ssres;
}

double
RRegressionFitAnalysisRegressionSummary::getMSRes() {
	return msres;
}

double
RRegressionFitAnalysisRegressionSummary::getMSResDerived() {
	return msresderived;
}

double
RRegressionFitAnalysisRegressionSummary::getR2() {
	return r2;
}

double
RRegressionFitAnalysisRegressionSummary::getR2Adj() {
	return r2adj;
}

double
RRegressionFitAnalysisRegressionSummary::getFStat() {
	return fstat;
}

double
RRegressionFitAnalysisRegressionSummary::getFStatPValue() {
	return fstatpvalue;
}

double
RRegressionFitAnalysisRegressionSummary::getDwStat() {
	return dwstat;
}

double
RRegressionFitAnalysisRegressionSummary::getNobs() {
	return nobs;
}

double
RRegressionFitAnalysisRegressionSummary::getSSResPred() {
	return ssresPred;
}

double
RRegressionFitAnalysisRegressionSummary::getMSResDerivedPred() {
	return msresderivedPred;
}

} //namespace r
} //namespace model
} //namespace dambase
