/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RRegressionFitAnalysisRegressorOverview.cpp
 *
 *  Created on: 18.02.2015
 *      Author: gerberma
 */

#include "RRegressionFitAnalysisRegressorOverview.hpp"

namespace dambase {
namespace model {
namespace r {


RegressionFitAnalysisRegressorOverviewItem::RegressionFitAnalysisRegressorOverviewItem(double coefficient, double tstat, double pvalue, double vif, double stderror, double stdcoefficient)
	:coefficient(coefficient), tstat(tstat), pvalue(pvalue),  vif(vif), stderror(stderror), stdcoefficient(stdcoefficient)
{;}

RegressionFitAnalysisRegressorOverviewItem::~RegressionFitAnalysisRegressorOverviewItem(){;}

double
RegressionFitAnalysisRegressorOverviewItem::getCoefficient() {
	return coefficient;
}

double
RegressionFitAnalysisRegressorOverviewItem::getTStat() {
	return tstat;
}

double
RegressionFitAnalysisRegressorOverviewItem::getPValue() {
	return pvalue;
}

double
RegressionFitAnalysisRegressorOverviewItem::getVIF() {
	return vif;
}

double
RegressionFitAnalysisRegressorOverviewItem::getStdError() {
	return stderror;
}

double
RegressionFitAnalysisRegressorOverviewItem::getStdCoefficient() {
	return stdcoefficient;
}

/***
 *
 */

RegressionFitAnalysisRegressorOverviewRegressorItem::RegressionFitAnalysisRegressorOverviewRegressorItem(boost::shared_ptr<Regressor_t> regressor, float coefficient, double tstat, double pvalue, double vif, double stderror, double stdcoefficient)
	:RegressionFitAnalysisRegressorOverviewItem(coefficient, tstat, pvalue, vif, stderror, stdcoefficient), regressor(regressor)
{}

RegressionFitAnalysisRegressorOverviewRegressorItem::~RegressionFitAnalysisRegressorOverviewRegressorItem(){;}

QString
RegressionFitAnalysisRegressorOverviewRegressorItem::getName() {
	return regressor->getFormula();
}

bool
RegressionFitAnalysisRegressorOverviewRegressorItem::isActive() {
	return regressor->isActive();
}

bool
RegressionFitAnalysisRegressorOverviewRegressorItem::isEditable() {
	return regressor->isEditable();
}

int
RegressionFitAnalysisRegressorOverviewRegressorItem::getRegressorIndex() {
	return regressor->getIndex();
}

/***
 *
 */

RegressionFitAnalysisRegressorOverviewInterceptItem::RegressionFitAnalysisRegressorOverviewInterceptItem(double coefficient, double tstat, double pvalue, double stderror)
	:RegressionFitAnalysisRegressorOverviewItem(coefficient, tstat, pvalue, nanl(NULL), stderror, nanl(NULL))
{;}

RegressionFitAnalysisRegressorOverviewInterceptItem::~RegressionFitAnalysisRegressorOverviewInterceptItem(){;}

QString
RegressionFitAnalysisRegressorOverviewInterceptItem::getName() {
	return QString("Intercept");
}

/***
 *
 */

RRegressionFitAnalysisRegressorOverview::RRegressionFitAnalysisRegressorOverview(Rcpp::List overview, boost::shared_ptr<RegressionFormula_t> regressionFormula)
	:regressionFormula(regressionFormula)
{
	Rcpp::CharacterVector names = overview["names"];
	Rcpp::NumericVector coefficients = overview["coefficients"];
	Rcpp::NumericVector tstats = overview["tstats"];
	Rcpp::NumericVector pvalues = overview["pvalues"];
	Rcpp::NumericVector stderrors = overview["stderrors"];
	Rcpp::NumericVector vifs = overview["vifs"];
	Rcpp::NumericVector stdcoefficients = overview["stdcoefficients"];

	if(names.size() != coefficients.size()) {
		throw std::logic_error("number of coefficient names is different from number of coefficients");
	}

	if(vifs.size() != names.size() - 1) {
		throw std::logic_error("incorrect number of vifs recognized");
	}

	if(stdcoefficients.size() != names.size() - 1) {
		throw std::logic_error("incorrect number of standard coefficients recognized");
	}


	std::string interceptRName("(Intercept)");
	interceptItem = boost::shared_ptr<api::IRRegressionFitAnalysisRegressorOverview::InterceptItem_t>(new RegressionFitAnalysisRegressorOverviewInterceptItem(
				getValueFor(coefficients, interceptRName),
				getValueFor(tstats, interceptRName),
				getValueFor(pvalues, interceptRName),
				getValueFor(stderrors, interceptRName)
			));

	//for each regressor
	BOOST_FOREACH(boost::shared_ptr<api::IRegressor> regressor, regressionFormula->getRegressors()) {
		boost::shared_ptr<api::IRRegressionFitAnalysisRegressorOverview::RegressorItem_t> regressorItem(new RegressionFitAnalysisRegressorOverviewRegressorItem(
					regressor,
					getValueFor(coefficients, regressor),
					getValueFor(tstats, regressor),
					getValueFor(pvalues, regressor),
					getValueFor(vifs, regressor),
					getValueFor(stderrors, regressor),
					getValueFor(stdcoefficients, regressor)
				));
		regressorItems.push_back(regressorItem);
	}
}

RRegressionFitAnalysisRegressorOverview::~RRegressionFitAnalysisRegressorOverview(){;}

api::IRRegressionFitAnalysisRegressorOverview::RegressorItemContainer_t
RRegressionFitAnalysisRegressorOverview::getRegressorItems() {
	return regressorItems;
}

boost::shared_ptr<api::IRRegressionFitAnalysisRegressorOverview::InterceptItem_t>
RRegressionFitAnalysisRegressorOverview::getInterceptItem() {
	return interceptItem;
}

} //namespace r
} //namespace model
} //namespace dambase
