/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RRegressionFitAnalysisRegressorOverview.hpp
 *
 *  Created on: Mar 26, 2014
 *      Author: gerberma
 */

#ifndef RREGRESSIONFITANALYSISREGRESSOROVERVIEW_HPP_
#define RREGRESSIONFITANALYSISREGRESSOROVERVIEW_HPP_

#include <math.h>
#include <string>
#include <vector>
#include <algorithm>
#include <stdexcept>
#include <boost/foreach.hpp>
#include <boost/shared_ptr.hpp>
#include <Rcpp.h>
#include <QString>
#include <QDebug>
#include "api/IRegressor.hpp"
#include "api/IRegressionFormula.hpp"
#include "api/IRRegressionFitAnalysisRegressorOverview.hpp"

namespace dambase {
namespace model {
namespace r {

/**
 * Contains values for a given coefficient as well as a reference to the regressor of the regression formula
 */
class RegressionFitAnalysisRegressorOverviewItem : public virtual api::IRRegressionFitAnalysisRegressorOverviewItem {
public:
	RegressionFitAnalysisRegressorOverviewItem(double coefficient, double tstat, double pvalue, double vif, double stderror, double stdcoefficient);

	virtual ~RegressionFitAnalysisRegressorOverviewItem();

	double getCoefficient();

	double getTStat();

	double getPValue();

	double getVIF();

	double getStdError();

	double getStdCoefficient();

private:
	double coefficient;
	double tstat;
	double pvalue;
	double vif;
	double stderror;
	double stdcoefficient;
};


class RegressionFitAnalysisRegressorOverviewRegressorItem : public virtual api::IRRegressionFitAnalysisRegressorOverviewRegressorItem, public virtual RegressionFitAnalysisRegressorOverviewItem {
public:
	typedef api::IRegressor Regressor_t;
public:
	RegressionFitAnalysisRegressorOverviewRegressorItem(boost::shared_ptr<Regressor_t> regressor, float coefficient, double tstat, double pvalue, double vif, double stderror, double stdcoefficient);

	virtual ~RegressionFitAnalysisRegressorOverviewRegressorItem();

	QString getName();

	bool isActive();

	bool isEditable();

	int getRegressorIndex();

	double getCoefficient(){
		return RegressionFitAnalysisRegressorOverviewItem::getCoefficient();
	}

	double getTStat() {
		return RegressionFitAnalysisRegressorOverviewItem::getTStat();
	}

	double getPValue() {
		return RegressionFitAnalysisRegressorOverviewItem::getPValue();
	}

	double getVIF() {
		return RegressionFitAnalysisRegressorOverviewItem::getVIF();
	}

	double getStdError() {
		return RegressionFitAnalysisRegressorOverviewItem::getStdError();
	}

	double getStdCoefficient() {
		return RegressionFitAnalysisRegressorOverviewItem::getStdCoefficient();
	}

private:
	boost::shared_ptr<Regressor_t> regressor;
};

class RegressionFitAnalysisRegressorOverviewInterceptItem : public virtual api::IRRegressionFitAnalysisRegressorOverviewInterceptItem, public RegressionFitAnalysisRegressorOverviewItem {
public:
	RegressionFitAnalysisRegressorOverviewInterceptItem(double coefficient, double tstat, double pvalue, double stderror);

	virtual ~RegressionFitAnalysisRegressorOverviewInterceptItem();

	QString getName();

	double getCoefficient(){
		return RegressionFitAnalysisRegressorOverviewItem::getCoefficient();
	}

	double getTStat() {
		return RegressionFitAnalysisRegressorOverviewItem::getTStat();
	}

	double getPValue() {
		return RegressionFitAnalysisRegressorOverviewItem::getPValue();
	}

	double getVIF() {
		return RegressionFitAnalysisRegressorOverviewItem::getVIF();
	}

	double getStdError() {
		return RegressionFitAnalysisRegressorOverviewItem::getStdError();
	}

	double getStdCoefficient() {
		return RegressionFitAnalysisRegressorOverviewItem::getStdCoefficient();
	}
};

class RRegressionFitAnalysisRegressorOverview : public virtual api::IRRegressionFitAnalysisRegressorOverview {
public:
	typedef api::IRegressionFormula RegressionFormula_t;
public:
	RRegressionFitAnalysisRegressorOverview(Rcpp::List overview, boost::shared_ptr<RegressionFormula_t> regressionFormula);
	virtual ~RRegressionFitAnalysisRegressorOverview();

	api::IRRegressionFitAnalysisRegressorOverview::RegressorItemContainer_t getRegressorItems();
	boost::shared_ptr<api::IRRegressionFitAnalysisRegressorOverview::InterceptItem_t> getInterceptItem();

private:
	template<typename Vector, typename Regressor>
	double getValueFor(Vector &vector, boost::shared_ptr<Regressor> regressor) {
		return getValueFor(vector, regressionFormula->resolveRFormulaFor(regressor).toStdString());
	}

	template<typename Vector>
	double getValueFor(Vector &vector, std::string vectorName) {
		Rcpp::CharacterVector vectorNames = Rcpp::as<Rcpp::CharacterVector>(vector.names());
		typename Rcpp::CharacterVector::iterator vectorNameIt = std::find(vectorNames.begin(), vectorNames.end(), vectorName.c_str());
		if(vectorNameIt != vectorNames.end()) {
			int vectorNameIndex = std::distance(vectorNames.begin(), vectorNameIt);
			return vector[vectorNameIndex];
		}

		return nan(NULL);
	}

private:
	api::IRRegressionFitAnalysisRegressorOverview::RegressorItemContainer_t regressorItems;
	boost::shared_ptr<api::IRRegressionFitAnalysisRegressorOverview::InterceptItem_t> interceptItem;

	boost::shared_ptr<RegressionFormula_t> regressionFormula;
};

}  // namespace r
}  // namespace model
}  // namespace dambase


#endif /* RREGRESSIONFITANALYSISREGRESSOROVERVIEW_HPP_ */
