/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RRegressionFitAnalysisRegressorTimeseriesProvider.hpp
 *
 *  Created on: 09.02.2015
 *      Author: gerberma
 */

#ifndef RREGRESSIONFITANALYSISREGRESSORTIMESERIESPROVIDER_HPP_
#define RREGRESSIONFITANALYSISREGRESSORTIMESERIESPROVIDER_HPP_

#include <Rcpp.h>
#include <boost/shared_ptr.hpp>
#include "RRegressionFitAnalysisRegressorTimeseries.hpp"

namespace dambase {
namespace model {
namespace r {


template<
	typename _RegressionFit
>
class RRegressionFitAnalysisRegressorTimeseriesProvider {
public:
	typedef RRegressionFitAnalysisRegressorTimeseries value_type;
public:
	RRegressionFitAnalysisRegressorTimeseriesProvider(){;}
	virtual ~RRegressionFitAnalysisRegressorTimeseriesProvider(){;}

	/**
	 * Response must be a column of the dataFrame
	 */
	template<typename DateColumn, typename ResponseColumn>
	boost::shared_ptr<api::IRRegressionFitAnalysisRegressorTimeseries> load(boost::shared_ptr<_RegressionFit> regressionFit,
			Rcpp::Formula formula,
			Rcpp::DataFrame &dataFrame,
			boost::shared_ptr<DateColumn> dateColumn,
			QString startDate,
			QString endDate,
			int overallMaxStartupTime,
			boost::shared_ptr<ResponseColumn> response,
			QString name,
			bool doApplyCoefficientsAndIntercept = true)
	{
		using namespace Rcpp;
		Environment damBaseEnv("package:DamBASE");
		Function function = damBaseEnv["loadAppliedFormula"];

		List values = function(
					Named("regressionFit", regressionFit->getFit()),
					Named("formula", formula),
					Named("originalDataFrame", dataFrame),
					Named("originalDateColumnNr", dateColumn->getIndex() + 1),
					Named("startDate", startDate.toStdString()),
					Named("endDate", endDate.toStdString()),
					Named("overallMaxStartupTime", overallMaxStartupTime),
					Named("response", response->getIndex() + 1),
					Named("doApplyCoefficientsAndIntercept", doApplyCoefficientsAndIntercept)
				);

		boost::shared_ptr<api::IRRegressionFitAnalysisRegressorTimeseries> result(new RRegressionFitAnalysisRegressorTimeseries(values, name));
		return result;
	}
};

}  // namespace r
}  // namespace model
}  // namespace dambase


#endif /* RREGRESSIONFITANALYSISREGRESSORTIMESERIESPROVIDER_HPP_ */
