/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RRegressionFitAnalysisResidualNormalProbability.cpp
 *
 *  Created on: 09.02.2015
 *      Author: gerberma
 */

#include <Rcpp.h>
#include <boost/foreach.hpp>
#include "RRegressionFitAnalysisResidualNormalProbability.hpp"

namespace dambase {
namespace model {
namespace r {

RRegressionFitAnalysisResidualNormalProbability::RRegressionFitAnalysisResidualNormalProbability(Rcpp::List values)
	:values(values)
{
	Rcpp::NumericVector p1 = Rcpp::as<Rcpp::NumericVector>(Rcpp::as<Rcpp::List>(values["line"])[0]);
	Rcpp::NumericVector p2 = Rcpp::as<Rcpp::NumericVector>(Rcpp::as<Rcpp::List>(values["line"])[1]);

	Rcpp::NumericVector rthereticalQuantile = Rcpp::as<Rcpp::NumericVector>(values["theoreticalQuantiles"]);
	BOOST_FOREACH(double value, rthereticalQuantile) {
		theoreticalQuantiles.push_back(value);
	}
	Rcpp::NumericVector rsampleQuantile = Rcpp::as<Rcpp::NumericVector>(values["sampleQuantiles"]);
	BOOST_FOREACH(double value, rsampleQuantile) {
		sampleQuantiles.push_back(value);
	}


	normalPointOne = api::IRRegressionFitAnalysisResidualNormalProbability::Point_t(p1[0], p1[1]);
	normalPointTwo = api::IRRegressionFitAnalysisResidualNormalProbability::Point_t(p2[0], p2[1]);
}

RRegressionFitAnalysisResidualNormalProbability::~RRegressionFitAnalysisResidualNormalProbability(){;}

api::IRRegressionFitAnalysisResidualNormalProbability::QuantileContainer_t &
RRegressionFitAnalysisResidualNormalProbability::getTheoreticalQuantiles() {
	return theoreticalQuantiles;
}

api::IRRegressionFitAnalysisResidualNormalProbability::QuantileContainer_t &
RRegressionFitAnalysisResidualNormalProbability::getSampleQuantiles() {
	return sampleQuantiles;
}

api::IRRegressionFitAnalysisResidualNormalProbability::Point_t &
RRegressionFitAnalysisResidualNormalProbability::getNormalLinePointOne() {
	return normalPointOne;
}

api::IRRegressionFitAnalysisResidualNormalProbability::Point_t &
RRegressionFitAnalysisResidualNormalProbability::getNormalLinePointTwo() {
	return normalPointTwo;
}


} //namespace r
} //namespace model
} //namespace dambase


