/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RRegressionFitAnalysisResidualTimeSeries.cpp
 *
 *  Created on: 16.02.2015
 *      Author: gerberma
 */

#include "RRegressionFitAnalysisResidualTimeSeries.hpp"

namespace dambase {
namespace model {
namespace r {

RRegressionFitAnalysisResidualTimeSeries::~RRegressionFitAnalysisResidualTimeSeries(){;}

RRegressionFitAnalysisResidualTimeSeries::RRegressionFitAnalysisResidualTimeSeries(Rcpp::List values)
	:values(values)
{
	Rcpp::CharacterVector rDates = Rcpp::as<Rcpp::CharacterVector>(values["dates"]);
	BOOST_FOREACH(Rcpp::CharacterVector::value_type date, rDates) {
		dates.push_back(Rcpp::as<std::string>(date));
	}

	Rcpp::NumericVector rResiduals = Rcpp::as<Rcpp::NumericVector>(values["residuals"]);
	BOOST_FOREACH(double residual, rResiduals) {
		residuals.push_back(residual);
	}

	Rcpp::CharacterVector rPredictionDates = Rcpp::as<Rcpp::CharacterVector>(values["predictionDateValues"]);
	BOOST_FOREACH(Rcpp::CharacterVector::value_type date, rPredictionDates) {
		predictionDates.push_back(Rcpp::as<std::string>(date));
	}

	Rcpp::NumericVector rPredictionResiduals = Rcpp::as<Rcpp::NumericVector>(values["predictionResiduals"]);
	BOOST_FOREACH(double residual, rPredictionResiduals) {
		predictionResiduals.push_back(residual);
	}

	Rcpp::String rregressionStartDate = values["regressionStartDate"];
	regressionStartDate = rregressionStartDate;
	Rcpp::String rregressionEndDate = values["regressionEndDate"];
	regressionEndDate = rregressionEndDate;

	Rcpp::NumericVector rLowerCi = Rcpp::as<Rcpp::NumericVector>(values["lowerCi"]);
	BOOST_FOREACH(double value, rLowerCi) {
		lowerCi.push_back(value);
	}

	Rcpp::NumericVector rUpperCi = Rcpp::as<Rcpp::NumericVector>(values["upperCi"]);
	BOOST_FOREACH(double value, rUpperCi) {
		upperCi.push_back(value);
	}

	Rcpp::NumericVector rOutOfLimit = Rcpp::as<Rcpp::NumericVector>(values["outOfLimit"]);
	BOOST_FOREACH(double value, rOutOfLimit) {
		outOfLimit.push_back(value);
	}

	Rcpp::CharacterVector rOutOfLimitDates = Rcpp::as<Rcpp::CharacterVector>(values["outOfLimitDates"]);
	BOOST_FOREACH(Rcpp::CharacterVector::value_type date, rOutOfLimitDates) {
		outOfLimitDates.push_back(Rcpp::as<std::string>(date));
	}

	Rcpp::NumericVector rExtrapolation = Rcpp::as<Rcpp::NumericVector>(values["extrapolation.values"]);
	BOOST_FOREACH(double value, rExtrapolation) {
		extrapolations.push_back(value);
	}

	Rcpp::CharacterVector rExtrapolationDate = Rcpp::as<Rcpp::CharacterVector>(values["extrapolation.dates"]);
	BOOST_FOREACH(Rcpp::CharacterVector::value_type date, rExtrapolationDate) {
		extrapolationDates.push_back(Rcpp::as<std::string>(date));
	}
}

api::IRRegressionFitAnalysisResidualTimeSeries::ResidualContainer_t
RRegressionFitAnalysisResidualTimeSeries::getResiduals() {
	return residuals;
}

api::IRRegressionFitAnalysisResidualTimeSeries::DateContainer_t
RRegressionFitAnalysisResidualTimeSeries::getDates() {
	//return Rcpp::as<DateContainer_t>(values["dates"]);
	return dates;
}

api::IRRegressionFitAnalysisResidualTimeSeries::ResidualContainer_t
RRegressionFitAnalysisResidualTimeSeries::getPredictionResiduals() {
	return predictionResiduals;
}

api::IRRegressionFitAnalysisResidualTimeSeries::Date_t
RRegressionFitAnalysisResidualTimeSeries::getRegressionStartDate() {
	return regressionStartDate;
}

api::IRRegressionFitAnalysisResidualTimeSeries::Date_t
RRegressionFitAnalysisResidualTimeSeries::getRegressionEndDate() {
	return regressionEndDate;
}

api::IRRegressionFitAnalysisResidualTimeSeries::DateContainer_t
RRegressionFitAnalysisResidualTimeSeries::getPredictionDates() {
	//return Rcpp::as<DateContainer_t>(values["dates"]);
	return predictionDates;
}

api::IRRegressionFitAnalysisResidualTimeSeries::ConfidenceIntervalContainer_t
RRegressionFitAnalysisResidualTimeSeries::getUpperCi() {
	return upperCi;
}

api::IRRegressionFitAnalysisResidualTimeSeries::ConfidenceIntervalContainer_t
RRegressionFitAnalysisResidualTimeSeries::getLowerCi() {
	return lowerCi;
}

api::IRRegressionFitAnalysisResidualTimeSeries::OutOfLimitContainer_t
RRegressionFitAnalysisResidualTimeSeries::getOutOfLimit() {
	return outOfLimit;
}

api::IRRegressionFitAnalysisResidualTimeSeries::DateContainer_t &
RRegressionFitAnalysisResidualTimeSeries::getOutOfLimitDates() {
	return outOfLimitDates;
}

api::IRRegressionFitAnalysisResidualTimeSeries::ExtrapolationsContainer_t
RRegressionFitAnalysisResidualTimeSeries::getExtrapolations() {
	return extrapolations;
}

api::IRRegressionFitAnalysisResidualTimeSeries::DateContainer_t
RRegressionFitAnalysisResidualTimeSeries::getExtrapolationDates() {
	return extrapolationDates;
}

} //namespace r
} //namespace model
} //namespace dambase


