/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RRegressionFitAnalysisResidualTimeSeries.hpp
 *
 *  Created on: Apr 3, 2014
 *      Author: gerberma
 */

#ifndef RREGRESSIONFITANALYSISRESIDUALTIMESERIES_HPP_
#define RREGRESSIONFITANALYSISRESIDUALTIMESERIES_HPP_

#include <string>
#include <vector>
#include <boost/foreach.hpp>
#include <Rcpp.h>
#include "api/IRRegressionFitAnalysisResidualTimeSeries.hpp"

namespace dambase {
namespace model {
namespace r {

class RRegressionFitAnalysisResidualTimeSeries : public virtual api::IRRegressionFitAnalysisResidualTimeSeries {
public:
	RRegressionFitAnalysisResidualTimeSeries(Rcpp::List values);

	virtual ~RRegressionFitAnalysisResidualTimeSeries();

	Date_t getRegressionStartDate();
	Date_t getRegressionEndDate();

	ResidualContainer_t getResiduals();
	DateContainer_t getDates();

	ResidualContainer_t getPredictionResiduals();
	DateContainer_t getPredictionDates();

	ConfidenceIntervalContainer_t getUpperCi();

	ConfidenceIntervalContainer_t getLowerCi();

	OutOfLimitContainer_t getOutOfLimit();

	DateContainer_t &getOutOfLimitDates();

	ExtrapolationsContainer_t getExtrapolations();

	DateContainer_t getExtrapolationDates();

private:
	Rcpp::List values;
	ResidualContainer_t residuals;
	DateContainer_t dates;
	ResidualContainer_t predictionResiduals;
	DateContainer_t predictionDates;
	ConfidenceIntervalContainer_t upperCi;
	ConfidenceIntervalContainer_t lowerCi;
	OutOfLimitContainer_t outOfLimit;
	DateContainer_t outOfLimitDates;
	Date_t regressionStartDate;
	Date_t regressionEndDate;
	ExtrapolationsContainer_t extrapolations;
	DateContainer_t extrapolationDates;
};

}  // namespace r
}  // namespace model
}  // namespace dambase

#endif /* RREGRESSIONFITANALYSISRESIDUALTIMESERIES_HPP_ */
