/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFit.cpp
 *
 *  Created on: 16.02.2015
 *      Author: gerberma
 */

#include <QDebug>
#include <boost/shared_ptr.hpp>
#include <stdexcept>
#include <strstream>
#include <Rcpp.h>
#include <cmath>
#include "RegressionFit.hpp"

namespace dambase {
namespace model {
namespace r {

RegressionFit::RegressionFit(Rcpp::List wrappedFit, boost::shared_ptr<api::IRegressionFit::RegressionFormula_t> regressionFormula, boost::shared_ptr<api::IRegressionFit::RegressionFitDataFrame_t> regressionFitDataFrame)
	: wrappedFit(wrappedFit), regressionFormula(regressionFormula), regressionFitDataFrame(regressionFitDataFrame), praisWinstenRho(nan(NULL))
{
	Rcpp::List rFit = Rcpp::as<Rcpp::List>(wrappedFit["rFit"]);
	if(rFit.containsElementNamed("rho") == true) {
		qDebug() << "contains rho";
		praisWinstenRho = Rcpp::as<double>(rFit["rho"]);
	}
}

RegressionFit::~RegressionFit() {;}

/*
double
RegressionFit::getCoefficientFor(boost::shared_ptr<api::IRegressor> regressor) {
	int index = regressionFormula->getIndexOf(regressor) + 1;
	if(index == 0) {
		std::stringstream ss;
		ss << "regressor " << regressor->getFormula().toStdString() << " for column " << regressor->getInfluenceFactor()->getName().toStdString() << " not found within formula" ;
		throw std::runtime_error(ss.str());
	}

	return Rcpp::as<Rcpp::NumericVector>(wrappedFit["coefficients"])[index];
}
*/

double
RegressionFit::getIntercept() {
	return Rcpp::as<Rcpp::NumericVector>(wrappedFit["coefficients"])[0];
}

Rcpp::List
RegressionFit::getFit() {
	return Rcpp::as<Rcpp::List>(wrappedFit["rFit"]);
}

boost::shared_ptr<api::IRegressionFit::RegressionFormula_t>
RegressionFit::getRegressionFormula() {
	return regressionFormula;
}

boost::shared_ptr<api::IRegressionFit::RegressionFitDataFrame_t>
RegressionFit::getRegressionFitDataFrame() {
	return regressionFitDataFrame;
}

int
RegressionFit::getOverallMaxStartupTime() {
	return regressionFitDataFrame->getOverallMaxStartupTime();
}

double
RegressionFit::getPraisWinstenRho() {
	return praisWinstenRho;
}

} //namespace r
} //namespace model
} //namespace dambase

