/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitAnalysisRegressionSummaryProvider.cpp
 *
 *  Created on: 16.02.2015
 *      Author: gerberma
 */

#include <Rcpp.h>
#include "RegressionFitAnalysisRegressionSummaryProvider.hpp"
#include "RRegressionFitAnalysisRegressionSummary.hpp"

namespace dambase {
namespace model {
namespace r {

RegressionFitAnalysisRegressionSummaryProvider::RegressionFitAnalysisRegressionSummaryProvider(){;}
RegressionFitAnalysisRegressionSummaryProvider::~RegressionFitAnalysisRegressionSummaryProvider(){;}

boost::shared_ptr<api::IRRegressionFitAnalysisRegressionSummary>
RegressionFitAnalysisRegressionSummaryProvider::load(boost::shared_ptr<api::IRegressionFit> regressionFit) {
		Rcpp::Environment damBaseEnv("package:DamBASE");
		Rcpp::Function function = damBaseEnv["loadRegressionSummary"];

		Rcpp::List summary = Rcpp::as<Rcpp::List>(function(
					Rcpp::Named("regressionFit", regressionFit->getFit()),
					Rcpp::Named("dataFrame", regressionFit->getRegressionFormula()->getDataFrame()->getDataFrame()),
					Rcpp::Named("dateColumnNr", regressionFit->getRegressionFormula()->getDataFrame()->getDateColumn()->getIndex() + 1),
					Rcpp::Named("behaviorIndicatorColumnNr", regressionFit->getRegressionFormula()->getBehaviorIndicatorColumn()->getIndex() + 1),
					Rcpp::Named("predictionStartDate", regressionFit->getRegressionFitDataFrame()->getDateColumn()->getFirstValue().getDate().toStdString()), //this is the start date including the dates which have to be cutted with overallMaxStartupTime at the end
					Rcpp::Named("observationStartDate", regressionFit->getRegressionFitDataFrame()->getDateColumn()->getLastValue().getDate().toStdString()),
					Rcpp::Named("overallMaxStartupTime", regressionFit->getOverallMaxStartupTime())
				));

		boost::shared_ptr<api::IRRegressionFitAnalysisRegressionSummary> result(new RRegressionFitAnalysisRegressionSummary(summary));
		return result;
	}

} //namespace r
} //namespace model
} //namespace dambase


