/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitDataFrame.cpp
 *
 *  Created on: 16.02.2015
 *      Author: gerberma
 */

#include <boost/foreach.hpp>
#include "RegressionFitDataFrame.hpp"

namespace dambase {
namespace model {
namespace r {


RegressionFitDataFrame::RegressionFitDataFrame(Rcpp::DataFrame &dataFrame, boost::shared_ptr<api::IRegressionFitDataFrame::DateColumn_t> originalDateColumn, boost::shared_ptr<api::IRegressionFitDataFrame::BehaviorIndicatorColumn_t> originalBehaviorIndicatorColumn, int overallMaxStartupTime)
	:dataFrame(dataFrame), overallMaxStartupTime(overallMaxStartupTime)
{
	Rcpp::Environment damBaseEnv("package:DamBASE");
	Rcpp::Function getHeadersFromDataFrame = damBaseEnv["getHeadersForDataFrame"];

	Rcpp::CharacterVector rheaders = Rcpp::as<Rcpp::CharacterVector >(getHeadersFromDataFrame(
			Rcpp::Named("dataFrame", dataFrame))
			);

	int index = 0;
	BOOST_FOREACH(Rcpp::CharacterVector::value_type rheader, rheaders) {
		headers.push_back(::dambase::model::r::DataFrameHeader(rheader, index++));
	}

	//create DataFrameColumns out of original assignments
	QString dateColumnHeaderName = originalDateColumn->getName();
	BOOST_FOREACH(::dambase::model::r::DataFrameHeader &header, headers) {
		if(header.getName() == originalDateColumn->getName()) {
			dateColumn = boost::shared_ptr< ::dambase::model::r::DateColumn>(new ::dambase::model::r::DateColumn(header, Rcpp::as<Rcpp::CharacterVector>(dataFrame[header.getIndex()])));
		}else if(header.getName() == originalBehaviorIndicatorColumn->getName()) {
			behaviorIndicatorColumn = boost::shared_ptr< ::dambase::model::r::BehaviorIndicatorColumn>(new ::dambase::model::r::BehaviorIndicatorColumn(header));
		}
	}
}

Rcpp::DataFrame &
RegressionFitDataFrame::getDataFrame() {
	return dataFrame;
}

boost::shared_ptr<api::IRegressionFitDataFrame::DateColumn_t>
RegressionFitDataFrame::getDateColumn() {
	return dateColumn;
}

std::vector<api::IRegressionFitDataFrame::DataFrameHeader_t> &
RegressionFitDataFrame::getHeaders() {
	return headers;
}

boost::shared_ptr<api::IRegressionFitDataFrame::BehaviorIndicatorColumn_t>
RegressionFitDataFrame::getBehaviorIndicatorColumn() {
	return behaviorIndicatorColumn;
}

int
RegressionFitDataFrame::getOverallMaxStartupTime() {
	return overallMaxStartupTime;
}


} //namespace r
} //namespace model
} //namespace dambase
