/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitDataFrame.hpp
 *
 *  Created on: 04.09.2014
 *      Author: gerberma
 */

#ifndef REGRESSIONFITDATAFRAME_HPP_
#define REGRESSIONFITDATAFRAME_HPP_

#include <boost/shared_ptr.hpp>
#include <Rcpp.h>
#include "api/IRegressionFitDataFrame.hpp"

namespace dambase {
namespace model {
namespace r {

class RegressionFitDataFrame : public virtual api::IRegressionFitDataFrame {
public:
	//Maybe we can keep a reference to the original DataFrame (model::r::DataFrame) instead of single values such as the DateColumn
	RegressionFitDataFrame(Rcpp::DataFrame &dataFrame, boost::shared_ptr<api::IRegressionFitDataFrame::DateColumn_t> originalDateColumn, boost::shared_ptr<api::IRegressionFitDataFrame::BehaviorIndicatorColumn_t> originalBehaviorIndicatorColumn, int overallMaxStartupTime);

	Rcpp::DataFrame &getDataFrame();

	boost::shared_ptr<api::IRegressionFitDataFrame::DateColumn_t> getDateColumn();

	std::vector<api::IRegressionFitDataFrame::DataFrameHeader_t> &getHeaders();

	boost::shared_ptr<api::IRegressionFitDataFrame::BehaviorIndicatorColumn_t> getBehaviorIndicatorColumn();

	int getOverallMaxStartupTime();

	//TODO: provide a findColumnByName function returning the DataCol

private:
	Rcpp::DataFrame dataFrame;
	std::vector<api::IRegressionFitDataFrame::DataFrameHeader_t> headers;
	boost::shared_ptr<api::IRegressionFitDataFrame::DateColumn_t> dateColumn;
	boost::shared_ptr<api::IRegressionFitDataFrame::BehaviorIndicatorColumn_t> behaviorIndicatorColumn;
	int overallMaxStartupTime;
};

}  // namespace r
}  // namespace model
}  // namespace dambase


#endif /* REGRESSIONFITDATAFRAME_HPP_ */
