/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFormula.hpp
 *
 *  Created on: Feb 20, 2014
 *      Author: gerberma
 */

#ifndef REGRESSIONFORMULA_HPP_
#define REGRESSIONFORMULA_HPP_

#include <boost/shared_ptr.hpp>
#include <vector>
#include <map>
#include "BehaviorIndicatorColumn.hpp"
#include "api/IRegressionFormula.hpp"

namespace dambase {
namespace model {
namespace r {


class RegressionFormula : public virtual api::IRegressionFormula {
public:
	RegressionFormula();
	RegressionFormula(
				std::vector<boost::shared_ptr<api::IRegressionFormula::RegressionModelTemplate_t> > methodDescriptions,
				std::vector<boost::shared_ptr<api::IRegressionFormula::RegressorFormulaBuilder_t> > regressorFormulaBuilders,
				boost::shared_ptr<api::IRegressionFormula::DataFrame_t> dataFrame
		);

	virtual ~RegressionFormula();

	void setDataFrame(boost::shared_ptr<api::IRegressionFormula::DataFrame_t> dataFrame);

	api::IRegressionFormula::DataFrame_t::Column_Container_t getInvolvedColumns();

	void setBehaviorIndicatorColumn(boost::shared_ptr<api::IRegressionFormula::DataFrame_t::Behavior_Indicator_Column_t> behaviorIndicatorColumn);


	const boost::shared_ptr<api::IRegressionFormula::DataFrame_t::Behavior_Indicator_Column_t> &getBehaviorIndicatorColumn() const;

	Rcpp::Formula buildFormula();

	Rcpp::Formula buildFormulaForSingleRegressor(boost::shared_ptr<api::IRegressionFormula::Regressor_t> regressor);

	Rcpp::Formula buildFormulaForInfluenceFactorWithinAssignmentCategory(boost::shared_ptr<api::IRegressionFormula::InfluenceFactor_t> influenceFactor, boost::shared_ptr<api::IRegressionFormula::AssignmentCategory_t> assignmentCategory);


	Rcpp::Formula buildFormulaForAssignmentCategory(boost::shared_ptr<api::IRegressionFormula::AssignmentCategory_t> assignmentCategory);


	int calculateOverallMaxStartupTime();

	QString resolveRFormulaFor(boost::shared_ptr<api::IRegressionFormula::Regressor_t> regressor);

	std::string buildFormulaAsRStringForSingleRegressor(boost::shared_ptr<api::IRegressionFormula::Regressor_t> regressor, QString delimiter = "+");

	std::string buildFormulaAsRString(QString delimiter = "+");


	std::string buildFormulaAsRStringForInfluenceFactorWithinAssignmentCategory(boost::shared_ptr<api::IRegressionFormula::InfluenceFactor_t> influenceFactor, boost::shared_ptr<api::IRegressionFormula::AssignmentCategory_t> assignmentCategory, QString delimiter = "+");


	std::string buildFormulaAsRStringForAssignmentCategory(boost::shared_ptr<api::IRegressionFormula::AssignmentCategory_t> assignmentCategory, QString delimiter = "+");


	QString toString(QString delimiter = "+");


	std::vector<boost::shared_ptr<api::IRegressionFormula::RegressionModelTemplate_t> > getMethodDescriptions();

	/**
	 * Primarly used by the serialization process
	 */
	const std::vector<boost::shared_ptr<api::IRegressionFormula::RegressionModelTemplate_t> > &getAllMethodDescriptionsRegardlessOfTheMode() const;

	/**
	 * Primarly used by the deserialization process
	 */
	void setMethodDescriptions(std::vector<boost::shared_ptr<api::IRegressionFormula::RegressionModelTemplate_t> > &methodDescriptions);

	/**
	 * Primarly used by the deserialization process
	 */
	void setCustomRegressionModelDefinitions(std::vector<boost::shared_ptr<api::IRegressionFormula::RegressionModelDefinition_t> > &regressionModelDefinitions);

	void addCustomRegressionModelDefinition(boost::shared_ptr<api::IRegressionFormula::RegressionModelDefinition_t> regressionModelDefinition);

	/**
	 * Primarly used by the deserialization process
	 */
	void setTemplatedRegressionModelDefinitions(std::vector<boost::shared_ptr<api::IRegressionFormula::RegressionModelDefinition_t> > &regressionModelDefinitions);

	void addTemplatedRegressionModelDefinition(boost::shared_ptr<api::IRegressionFormula::RegressionModelDefinition_t> regressionModelDefinition);

	void removeRegressionModelDefinition(boost::shared_ptr<api::IRegressionFormula::RegressionModelDefinition_t> regressionModelDefinition);

	const std::vector<boost::shared_ptr<api::IRegressionFormula::RegressionModelDefinition_t> > &getCustomRegressionModelDefinitions() const;

	const std::vector<boost::shared_ptr<api::IRegressionFormula::RegressionModelDefinition_t> > &getTemplatedRegressionModelDefinitions() const;

	std::vector<boost::shared_ptr<api::IRegressionFormula::RegressionModelDefinition_t> > getAllRegressionModelDefinitions();


	void releaseRegressionModelDefinitionForModification(boost::shared_ptr<api::IRegressionFormula::RegressionModelDefinition_t> regressionModelDefinition);


	/**
	 * This is primarly used by the deserialization process
	 */
	void setRegressorFormulaBuilders(std::vector<boost::shared_ptr<api::IRegressionFormula::RegressorFormulaBuilder_t> > &regressorFormulaBuilders);

	void addRegressorFormulaBuilder(boost::shared_ptr<api::IRegressionFormula::RegressorFormulaBuilder_t> regressorFormulaBuilder);

	const std::vector<boost::shared_ptr<api::IRegressionFormula::RegressorFormulaBuilder_t> > &getRegressorFormulaBuilders() const;

	boost::shared_ptr<api::IRegressionFormula::RegressorFormulaBuilder_t> findRegressorFormulaBuilderByIdentifier(QString identifier);

	boost::shared_ptr<api::IRegressionFormula::RegressorFormulaBuilder_t> findRegressorFormulaBuilderByFormula(QString formula);

	QString getNewIdentifierFor(boost::shared_ptr<api::IRegressionFormula::RegressorFormulaBuilder_t> regressorFormulaBuilder);

	QString getNewFormulaFor(boost::shared_ptr<api::IRegressionFormula::RegressorFormulaBuilder_t> regressorFormulaBuilder);

	std::vector<boost::shared_ptr<api::IRegressionFormula::Regressor_t> > getActiveRegressors();

	std::vector<boost::shared_ptr<api::IRegressionFormula::Regressor_t> > getRegressors();

	boost::shared_ptr<api::IRegressionFormula::Regressor_t> findRegressorByIndex(int index);

	boost::shared_ptr<api::IRegressionFormula::Regressor_t> findRegressorByFormula(QString formula);

	std::vector<boost::shared_ptr<api::IRegressionFormula::AssignmentCategory_t> > getInvolvedAssignmentCategoriesWithAnInfluenceFactor();


	std::vector<boost::shared_ptr<api::IRegressionFormula::AssignmentCategory_t> > findAssignmentCategoriesFor(std::vector<boost::shared_ptr<api::IRegressionFormula::Regressor_t> > regressors);


	/**
	 * Returns a vector containing all (unique) influence factors of all regressors being assigned to a given assignmentcategory
	 */
	std::vector<boost::shared_ptr<api::IRegressionFormula::InfluenceFactor_t> > findInfluenceFactorsOfRegressorsAssignedTo(std::vector<boost::shared_ptr<api::IRegressionFormula::Regressor_t> > regressors, boost::shared_ptr<api::IRegressionFormula::AssignmentCategory_t> assignmentCategory);

	boost::shared_ptr<api::IRegressionFormula::RegressionModelDefinition_t> getRegressionModelDefinitionForRegressor(boost::shared_ptr<api::IRegressionFormula::Regressor_t> regressor);

	bool containsRegressor(boost::shared_ptr<api::IRegressionFormula::Regressor_t> regressor);


	const boost::shared_ptr<DataFrame_t> &getDataFrame() const;

private:
	void keepRegressionModelDefinition(boost::shared_ptr<api::IRegressionFormula::RegressionModelDefinition_t> regressionModelDefinition);

	bool removeRegressionModelDefinition(boost::shared_ptr<api::IRegressionFormula::RegressionModelDefinition_t> regressionModelDefinition, std::vector<boost::shared_ptr<api::IRegressionFormula::RegressionModelDefinition_t> > &modelDefinitions);

private:
	std::vector<boost::shared_ptr<api::IRegressionFormula::RegressionModelTemplate_t> > methodDescriptions;
	boost::shared_ptr<api::IRegressionFormula::DataFrame_t> dataFrame;
	std::vector<boost::shared_ptr<api::IRegressionFormula::RegressionModelDefinition_t> > customRegressionModelDefinitions;
	std::vector<boost::shared_ptr<api::IRegressionFormula::RegressionModelDefinition_t> > templatedRegressionModelDefinitions;
	std::map<boost::shared_ptr<api::IRegressionFormula::RegressionModelDefinition_t> , boost::signals2::connection> regressionModelDefinitionConnections;
	std::vector<boost::shared_ptr<api::IRegressionFormula::RegressorFormulaBuilder_t> > regressorFormulaBuilders;
	boost::shared_ptr<api::IRegressionFormula::DataFrame_t::Behavior_Indicator_Column_t> behaviorIndicatorColumn;
};

}}} /* namespace dambase::model::r */


#endif /* REGRESSIONFORMULA_HPP_ */
