/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionModelDefinition.hpp
 *
 *  Created on: Aug 6, 2014
 *      Author: gerberma
 */

#ifndef REGRESSIONMODELDEFINITION_HPP_
#define REGRESSIONMODELDEFINITION_HPP_

#include <boost/foreach.hpp>
#include <boost/shared_ptr.hpp>
#include <vector>
#include <map>
#include <iterator>
#include <algorithm>
#include "api/IRegressionModelDefinition.hpp"

namespace dambase {
namespace model {
namespace r {


class RegressionModelDefinition : public virtual api::IRegressionModelDefinition {
public:
	RegressionModelDefinition();

	RegressionModelDefinition(QString name, boost::shared_ptr<api::IRegressionModelDefinition::DataFrame_t> dataFrame, bool iseditable=true);

	virtual ~RegressionModelDefinition();

	/**
	 * This is primarly used by the deserialization process
	 */
	void setDataFrame(boost::shared_ptr<api::IRegressionModelDefinition::DataFrame_t> dataFrame);

	/**
	 * This is primarly used by the deserialization process. For other purposes, use #freezeModification
	 */
	void setIsEditable(bool iseditable);

	const bool &isEditable() const;

	void freezeModification();

	void releaseForModification(std::vector<boost::shared_ptr<api::IRegressionModelDefinition::RegressorFormulaBuilder_t> > additionalFormulaBuilders);

	void setName(QString name);

	const QString &getName() const;

	const std::vector<boost::shared_ptr<api::IRegressionModelDefinition::Regressor_t> > &getRegressors() const;

	/**
	 * This is primarly used by deserialization process. For other purposes, use #addRegressorFormulaBuilders
	 */
	void setRegressorFormulaBuilders(std::vector<boost::shared_ptr<api::IRegressionModelDefinition::RegressorFormulaBuilder_t> > &formulaBuilders);

	void addRegressorFormulaBuilders(std::vector<boost::shared_ptr<api::IRegressionModelDefinition::RegressorFormulaBuilder_t> > formulaBuilders);

	void addRegressorFormulaBuilder(boost::shared_ptr<api::IRegressionModelDefinition::RegressorFormulaBuilder_t> formulaBuilder);

	void removeRegressorFormulaBuilder(boost::shared_ptr<api::IRegressionModelDefinition::RegressorFormulaBuilder_t> formulaBuilder);

	const std::vector<boost::shared_ptr<api::IRegressionModelDefinition::RegressorFormulaBuilder_t> > &getRegressorFormulaBuilders() const;

	boost::shared_ptr<api::IRegressionModelDefinition::RegressorFormulaBuilder_t> findRegressorFormulaBuilderByIdentifier(QString identifier);

	/**
	 * This is primary used by the deserialization process
	 */
	void setRegressors(std::vector<boost::shared_ptr<api::IRegressionModelDefinition::Regressor_t> > regressors);

	void addRegressor(boost::shared_ptr<api::IRegressionModelDefinition::Regressor_t> regressor);

	bool containsRegressor(boost::shared_ptr<api::IRegressionModelDefinition::Regressor_t> regressor);

	void removeRegressor(boost::shared_ptr<api::IRegressionModelDefinition::Regressor_t> regressor);

	const boost::shared_ptr<api::IRegressionModelDefinition::DataFrame_t> &getDataFrame() const;

private:
	QString name;
	boost::shared_ptr<api::IRegressionModelDefinition::DataFrame_t> dataFrame;
	bool iseditable;
	std::vector<boost::shared_ptr<api::IRegressionModelDefinition::Regressor_t> > regressors;
	std::vector<boost::shared_ptr<api::IRegressionModelDefinition::RegressorFormulaBuilder_t> > regressorFormulaBuilders;
	std::map<boost::shared_ptr<api::IRegressionModelDefinition::RegressorFormulaBuilder_t>, boost::signals2::connection> regressorFormulaBuilderConnections;
	std::map<boost::shared_ptr<api::IRegressionModelDefinition::Regressor_t>, boost::signals2::connection> regressorConnections;
};

}  // namespace r
}  // namespace model
}  // namespace dambase


#endif /* REGRESSIONMODELDEFINITION_HPP_ */
