/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionModelTemplate.cpp
 *
 *  Created on: 16.02.2015
 *      Author: gerberma
 */



#include "RegressionModelTemplate.hpp"

namespace dambase {
namespace model {
namespace r {

RegressionModelTemplate::~RegressionModelTemplate(){;}

RegressionModelTemplate::RegressionModelTemplate()
	:appearForBasic(false), appearForScientific(true)
{;}

RegressionModelTemplate::RegressionModelTemplate(boost::shared_ptr<api::IRegressionModelTemplate::ModeHandler_t> modeHandler, QString methodId, QString name, QString author, QString description, bool appearForScientific, bool appearForBasic)
	:modeHandler(modeHandler), methodId(methodId), name(name), author(author), description(description), appearForBasic(appearForBasic), appearForScientific(appearForScientific)
{;}

/**
 * This is primarly used by the deserialization process
 */
void
RegressionModelTemplate::setAuthor(QString &author) {
	this->author = author;
}

const QString &
RegressionModelTemplate::getAuthor() const {
	return author;
}

/**
 * This is primarly used by the deserialization process
 */
void
RegressionModelTemplate::setDescription(QString &description) {
	this->description = description;
}

const QString &
RegressionModelTemplate::getDescription() const {
	return description;
}

/**
 * This is primarly used by the deserialization process
 */
void
RegressionModelTemplate::setName(QString &name) {
	this->name = name;
}

const QString &
RegressionModelTemplate::getName() const {
	return name;
}

/**
 * This is primarly used by the deserialization process
 */
void
RegressionModelTemplate::setMethodId(QString &methodId) {
	this->methodId = methodId;
}

const QString &
RegressionModelTemplate::getMethodId() const {
	return methodId;
}

/**
 * This is primarly used by the deserialization process
 */
void
RegressionModelTemplate::setModeHandler(boost::shared_ptr<api::IRegressionModelTemplate::ModeHandler_t> modeHandler) {
	this->modeHandler = modeHandler;
}

/**
 * This is primarly used by the deserialization process. Use #availableInCurrentMode for other purposes
 */
const bool &
RegressionModelTemplate::getAppearForBasic() const {
	return appearForBasic;
}

/**
 * This is primarly used by the deserialization process
 */
void
RegressionModelTemplate::setAppearForBasic(bool appearForBasic) {
	this->appearForBasic = appearForBasic;
}

/**
 * This is primarly used by the deserialization process. Use #availableInCurrentMode for other purposes
 */
const bool &
RegressionModelTemplate::getAppearForScientific() const {
	return appearForScientific;
}

/**
 * This is primarly used by the deserialization process
 */
void
RegressionModelTemplate::setAppearForScientific(bool appearForScientific) {
	this->appearForScientific = appearForScientific;
}


/**
 * This is primarly used by the deserialization process
 */
void
RegressionModelTemplate::setRegressorFormulaBuilders(std::vector<boost::shared_ptr<api::IRegressionModelTemplate::RegressorFormulaBuilder_t> > &regressorFormulaBuilders) {
	this->regressorFormulaBuilders = regressorFormulaBuilders;
}

void
RegressionModelTemplate::addRegressorFormulaBuilder(boost::shared_ptr<api::IRegressionModelTemplate::RegressorFormulaBuilder_t> formulaBuilder) {
	regressorFormulaBuilders.push_back(formulaBuilder);
}


void
RegressionModelTemplate::addRegressorFormulaBuilder(boost::shared_ptr<api::IRegressionModelTemplate::RegressorFormulaBuilder_t> formulaBuilder, QString instantiationMessage) {
	addRegressorFormulaBuilder(formulaBuilder);
	instantiationMessages[formulaBuilder] = instantiationMessage;
}

const std::vector<boost::shared_ptr<api::IRegressionModelTemplate::RegressorFormulaBuilder_t> > &
RegressionModelTemplate::getRegressorFormulaBuilders() const {
	return regressorFormulaBuilders;
}

bool
RegressionModelTemplate::hasInstantiationMessageFor(boost::shared_ptr<api::IRegressionModelTemplate::RegressorFormulaBuilder_t> formulaBuilder) {
	return instantiationMessages.count(formulaBuilder) == 1;
}

QString
RegressionModelTemplate::resolveInstantiationMessageFor(boost::shared_ptr<api::IRegressionModelTemplate::RegressorFormulaBuilder_t> formulaBuilder) {
	return instantiationMessages[formulaBuilder];
}

std::vector<QString>
RegressionModelTemplate::resolveInstantiationMessages() {
	std::vector<QString> messages;

	BOOST_FOREACH(boost::shared_ptr<api::IRegressionModelTemplate::RegressorFormulaBuilder_t> regressorFormulaBuilder, regressorFormulaBuilders) {
		if(instantiationMessages.count(regressorFormulaBuilder) == 1) {
			messages.push_back(resolveInstantiationMessageFor(regressorFormulaBuilder));
		}
	}

	return messages;
}


bool
RegressionModelTemplate::availableInCurrentMode() {
	if(modeHandler->getMode() == 1 && appearForScientific) {
		return true;
	}
	if(modeHandler->getMode() == 0 && appearForBasic) {
		return true;
	}

	return false;
}

} //namespace r
} //namespace model
} //namespace dambase
