/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionModelTemplate.hpp
 *
 *  Created on: May 28, 2014
 *      Author: gerberma
 */

#ifndef RegressionModelTemplate_HPP_
#define RegressionModelTemplate_HPP_

#include "api/IRegressionModelTemplate.hpp"

namespace dambase {
namespace model {
namespace r {

class RegressionModelTemplate : public virtual api::IRegressionModelTemplate {
public:
	RegressionModelTemplate();

	RegressionModelTemplate(boost::shared_ptr<api::IRegressionModelTemplate::ModeHandler_t> modeHandler, QString methodId, QString name, QString author, QString description, bool appearForScientific = true, bool appearForBasic = true);

	virtual ~RegressionModelTemplate();

	/**
	 * This is primarly used by the deserialization process
	 */
	void setAuthor(QString &author);

	const QString &getAuthor() const;

	/**
	 * This is primarly used by the deserialization process
	 */
	void setDescription(QString &description);

	const QString &getDescription() const;

	/**
	 * This is primarly used by the deserialization process
	 */
	void setName(QString &name);

	const QString &getName() const;

	/**
	 * This is primarly used by the deserialization process
	 */
	void setMethodId(QString &methodId);

	const QString &getMethodId() const;

	/**
	 * This is primarly used by the deserialization process
	 */
	void setModeHandler(boost::shared_ptr<api::IRegressionModelTemplate::ModeHandler_t> modeHandler);

	/**
	 * This is primarly used by the deserialization process. Use #availableInCurrentMode for other purposes
	 */
	const bool &getAppearForBasic() const;

	/**
	 * This is primarly used by the deserialization process
	 */
	void setAppearForBasic(bool appearForBasic);

	/**
	 * This is primarly used by the deserialization process. Use #availableInCurrentMode for other purposes
	 */
	const bool &getAppearForScientific() const;

	/**
	 * This is primarly used by the deserialization process
	 */
	void setAppearForScientific(bool appearForScientific);


	/**
	 * This is primarly used by the deserialization process
	 */
	void setRegressorFormulaBuilders(std::vector<boost::shared_ptr<api::IRegressionModelTemplate::RegressorFormulaBuilder_t> > &regressorFormulaBuilders);

	void addRegressorFormulaBuilder(boost::shared_ptr<api::IRegressionModelTemplate::RegressorFormulaBuilder_t> formulaBuilder);


	void addRegressorFormulaBuilder(boost::shared_ptr<api::IRegressionModelTemplate::RegressorFormulaBuilder_t> formulaBuilder, QString instantiationMessage);

	const std::vector<boost::shared_ptr<api::IRegressionModelTemplate::RegressorFormulaBuilder_t> > &getRegressorFormulaBuilders() const;

	bool hasInstantiationMessageFor(boost::shared_ptr<api::IRegressionModelTemplate::RegressorFormulaBuilder_t> formulaBuilder);

	QString resolveInstantiationMessageFor(boost::shared_ptr<api::IRegressionModelTemplate::RegressorFormulaBuilder_t> formulaBuilder);

	std::vector<QString> resolveInstantiationMessages();


	bool availableInCurrentMode();

private:
	boost::shared_ptr<api::IRegressionModelTemplate::ModeHandler_t> modeHandler;
	QString methodId;
	QString name;
	QString author;
	QString description;
	std::vector<boost::shared_ptr<api::IRegressionModelTemplate::RegressorFormulaBuilder_t> > regressorFormulaBuilders;
	std::map<boost::shared_ptr<api::IRegressionModelTemplate::RegressorFormulaBuilder_t>, QString> instantiationMessages;
	bool appearForScientific;
	bool appearForBasic;
};

}  // namespace r
}  // namespace model
}  // namespace dambase


#endif /* RegressionModelTemplate_HPP_ */
