/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * Regressor.hpp
 *
 *  Created on: Feb 20, 2014
 *      Author: gerberma
 */

#ifndef REGRESSOR_HPP_
#define REGRESSOR_HPP_

#include <QString>
#include <string>
#include <vector>
#include <sstream>
#include <stdexcept>
#include <boost/algorithm/string.hpp>
#include <boost/foreach.hpp>
#include <boost/shared_ptr.hpp>
#include "DataFrameColumn.hpp"
#include "api/IRegressor.hpp"

namespace dambase {
namespace model {
namespace r {

class Regressor : public api::IRegressor {
public:
	Regressor();

	Regressor(boost::shared_ptr<api::IRegressor::RegressorFormulaBuilder_t> regressorFormulaBuilder, boost::shared_ptr<api::IRegressor::DataFrame_t> dataFrame, int index = 0, bool isactive = true, bool iseditable = true);

	virtual ~Regressor();

	/**
	 * This is primarly used by the serialization process
	 */
	const boost::shared_ptr<DataFrame_t> &getDataFrame() const;

	/**
	 * This is primarly used by the deserialization process
	 */
	void setDataFrame(boost::shared_ptr<api::IRegressor::DataFrame_t> dataFrame);

	//mostly for gui use only as the R formula for example should be resolved internally
	const boost::shared_ptr<Influence_Factor_t> &getInfluenceFactor() const;

	/**
	 * This is primarly used by the deserialization process.
	 */
	void setRegressorFormulaBuilder(boost::shared_ptr<api::IRegressor::RegressorFormulaBuilder_t> regressorFormulaBuilder);

	const boost::shared_ptr<api::IRegressor::RegressorFormulaBuilder_t> &getRegressorFormulaBuilder() const;

	boost::shared_ptr<api::IRegressor::AssignmentCategory_t> getAssignmentCategory() const;

	QString getRFormula(int overallMaxStartupTime);

	QString getFormula();

	QString getName();

	api::IRegressor::DataFrame_t::Influence_Factor_Container_t getAvailableInfluenceFactors();

	void setInfluenceFactor(int influenceFactorIndex);

	void setInfluenceFactor(boost::shared_ptr<api::IRegressor::Influence_Factor_t> influenceFactorToBeSet);

	/**
	 * This is primary used by the deserialization process
	 */
	void restoreInfluenceFactor(boost::shared_ptr<api::IRegressor::Influence_Factor_t> influenceFactor);

	void freezeModification();

	void releaseForModification();

	const bool &isEditable() const;

	/**
	 * This is primary used by the deserialization process. Use either #releaseForModification or #freezeModification for other purposes
	 */
	void setIsEditable(bool isEditable);

	void setIsActive(bool isactive);

	const bool &isActive() const;

	/**
	 * This is primary used by the deserialization process
	 */
	void setIndex(int index);

	const int &getIndex() const;

private:
	boost::shared_ptr<api::IRegressor::RegressorFormulaBuilder_t> regressorFormulaBuilder;
	boost::shared_ptr<api::IRegressor::DataFrame_t> dataFrame;
	bool isactive;
	bool iseditable;
	int index;
	boost::shared_ptr<api::IRegressor::Influence_Factor_t> influenceFactor;
};

}}} /* namespace dambase::model::r */


#endif /* REGRESSOR_HPP_ */
