/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * IDataFrameProvider.hpp
 *
 *  Created on: 17.02.2015
 *      Author: gerberma
 */

#ifndef IDATAFRAMEPROVIDER_HPP_
#define IDATAFRAMEPROVIDER_HPP_

#include <Rcpp.h>
#include <QUrl>
#include <boost/shared_ptr.hpp>
#include "IDataFrame.hpp"

namespace dambase {
namespace model {
namespace r {
namespace api {

class IDataFrameProvider {
public:
	typedef IDataFrame data_frame_type;
	typedef data_frame_type::DateColumn_t DateColumn_t;
public:
	IDataFrameProvider(){;}
	virtual ~IDataFrameProvider(){;}

	virtual boost::shared_ptr<data_frame_type> load(QUrl uri, QString csvDelimiter = ",", unsigned missingDataStrategy = 0) = 0;

	virtual void saveDataFrame(const Rcpp::DataFrame &dataFrame, QString location, boost::shared_ptr<DateColumn_t> dateColumn) = 0;

	virtual Rcpp::DataFrame readDataFrame(QString location, boost::shared_ptr<DateColumn_t> dateColumn) = 0;

	virtual Rcpp::DataFrame applyDateFormat(Rcpp::DataFrame &dataFrame, int dateColumnNr, QString dateFormat) = 0;

	virtual bool testDateColumnValidity(Rcpp::DataFrame &dataFrame, int dateColumnNr) = 0;

	virtual Rcpp::DataFrame applyMissingDataStrategy(Rcpp::DataFrame &dataFrame, unsigned missingDataStrategy) = 0;
};

} //namespace api
} //namespace r
} //namespace model
} //namespace

#endif /* IDATAFRAMEPROVIDER_HPP_ */
