/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * IRRegressionFitAnalysisResidualTimeSeries.hpp
 *
 *  Created on: 16.02.2015
 *      Author: gerberma
 */

#ifndef IRREGRESSIONFITANALYSISRESIDUALTIMESERIES_HPP_
#define IRREGRESSIONFITANALYSISRESIDUALTIMESERIES_HPP_

#include <vector>
#include <string>

namespace dambase {
namespace model {
namespace r {
namespace api {

class IRRegressionFitAnalysisResidualTimeSeries {
public:
	typedef std::vector<double> ResidualContainer_t;
	typedef std::vector<std::string> DateContainer_t;
	typedef std::vector<double> OutOfLimitContainer_t;
	typedef std::vector<double> ExtrapolationsContainer_t;
	typedef std::vector<double> ConfidenceIntervalContainer_t;
	typedef std::string Date_t;
public:
	IRRegressionFitAnalysisResidualTimeSeries(){;}
	virtual ~IRRegressionFitAnalysisResidualTimeSeries(){;}

	virtual Date_t getRegressionStartDate() = 0;
	virtual Date_t getRegressionEndDate() = 0;

	virtual ResidualContainer_t getResiduals() = 0;
	virtual DateContainer_t getDates() = 0;

	virtual ResidualContainer_t getPredictionResiduals() = 0;
	virtual DateContainer_t getPredictionDates() = 0;

	virtual ConfidenceIntervalContainer_t getUpperCi() = 0;

	virtual ConfidenceIntervalContainer_t getLowerCi() = 0;

	virtual OutOfLimitContainer_t getOutOfLimit() = 0;

	virtual DateContainer_t &getOutOfLimitDates() = 0;

	virtual ExtrapolationsContainer_t getExtrapolations() = 0;
	virtual DateContainer_t getExtrapolationDates() = 0;
};

} //namespace api
} //namespace r
} //namespace model
} //namespace


#endif /* IRREGRESSIONFITANALYSISRESIDUALTIMESERIES_HPP_ */
