/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * IRegressionFitProvider.hpp
 *
 *  Created on: 17.02.2015
 *      Author: gerberma
 */

#ifndef IREGRESSIONFITPROVIDER_HPP_
#define IREGRESSIONFITPROVIDER_HPP_

#include <boost/shared_ptr.hpp>
#include "IRegressionFit.hpp"
#include "IRegressionFitDataFrame.hpp"
#include "IRegressionFormula.hpp"
#include "IDataFrame.hpp"

namespace dambase {
namespace model {
namespace r {
namespace api {

class IRegressionFitProvider {
public:
	typedef IRegressionFit RegressionFit_t;
	typedef IRegressionFitDataFrame RegressionFitDataFrame_t;
	typedef IRegressionFormula RegressionFormula_t;
	typedef IDataFrame DataFrame_t;
public:
	IRegressionFitProvider(){;}
	virtual ~IRegressionFitProvider(){;}

	/**
	 * DataFrame is the final dataFrame on which the regression takes place.
	 * This means that the dataFrame has already been shrinked to the final data by any prior processing.
	 * Therefore, the returned RegressionFit<RegressionFormula> contains a reference to this dataFrame as it's the one used for the regression
	 */
	virtual boost::shared_ptr<RegressionFit_t> load(boost::shared_ptr<RegressionFitDataFrame_t> regressionFitDataFrame, boost::shared_ptr<RegressionFormula_t> formula) = 0;
	virtual boost::shared_ptr<RegressionFit_t> loadPraisWinsten(boost::shared_ptr<RegressionFitDataFrame_t> regressionFitDataFrame,boost::shared_ptr<RegressionFormula_t> formula, unsigned iterations) = 0;
	virtual boost::shared_ptr<RegressionFit_t> loadRidge(boost::shared_ptr<RegressionFitDataFrame_t> regressionFitDataFrame, boost::shared_ptr<RegressionFormula_t> formula, double k) = 0;
	virtual boost::shared_ptr<RegressionFit_t> loadPCR(boost::shared_ptr<RegressionFitDataFrame_t> regressionFitDataFrame, boost::shared_ptr<RegressionFormula_t> formula, unsigned r) = 0;
	virtual boost::shared_ptr<RegressionFitDataFrame_t> loadRegressionFitDataFrame(boost::shared_ptr<DataFrame_t> dataFrame, boost::shared_ptr<RegressionFormula_t> formula, QString fromDate, QString toDate) = 0;
};

} //namespace api
} //namespace r
} //namespace model
} //namespace


#endif /* IREGRESSIONFITPROVIDER_HPP_ */
