/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * IRegressionModelTemplate.hpp
 *
 *  Created on: 16.02.2015
 *      Author: gerberma
 */

#ifndef IREGRESSIONMODELTEMPLATE_HPP_
#define IREGRESSIONMODELTEMPLATE_HPP_

#include <QString>
#include <vector>
#include <map>
#include <boost/shared_ptr.hpp>
#include "IExpressionRegressorFormulaBuilder.hpp"
#include "model/api/IModeHandler.hpp"

namespace dambase {
namespace model {
namespace r {
namespace api {

class IRegressionModelTemplate {
public:
	typedef IExpressionRegressorFormulaBuilder RegressorFormulaBuilder_t;
	typedef model::api::IModeHandler ModeHandler_t;
public:
	IRegressionModelTemplate(){;}
	virtual ~IRegressionModelTemplate(){;}

	/**
	 * This is primarly used by the deserialization process
	 */
	virtual void setAuthor(QString &author) = 0;

	virtual const QString &getAuthor() const = 0;

	/**
	 * This is primarly used by the deserialization process
	 */
	virtual void setDescription(QString &description) = 0;

	virtual const QString &getDescription() const = 0;

	/**
	 * This is primarly used by the deserialization process
	 */
	virtual void setName(QString &name) = 0;

	virtual const QString &getName() const = 0;

	/**
	 * This is primarly used by the deserialization process
	 */
	virtual void setMethodId(QString &methodId) = 0;

	virtual const QString &getMethodId() const = 0;

	/**
	 * This is primarly used by the deserialization process
	 */
	virtual void setModeHandler(boost::shared_ptr<api::IRegressionModelTemplate::ModeHandler_t> modeHandler) = 0;

	/**
	 * This is primarly used by the deserialization process. Use #availableInCurrentMode for other purposes
	 */
	virtual const bool &getAppearForBasic() const = 0;

	/**
	 * This is primarly used by the deserialization process
	 */
	virtual void setAppearForBasic(bool appearForBasic) = 0;

	/**
	 * This is primarly used by the deserialization process. Use #availableInCurrentMode for other purposes
	 */
	virtual const bool &getAppearForScientific() const = 0;

	/**
	 * This is primarly used by the deserialization process
	 */
	virtual void setAppearForScientific(bool appearForScientific) = 0;

	/**
	 * This is primarly used by the deserialization process
	 */
	virtual void setRegressorFormulaBuilders(std::vector<boost::shared_ptr<api::IRegressionModelTemplate::RegressorFormulaBuilder_t> > &regressorFormulaBuilders) = 0;

	virtual void addRegressorFormulaBuilder(boost::shared_ptr<api::IRegressionModelTemplate::RegressorFormulaBuilder_t> formulaBuilder) = 0;

	virtual void addRegressorFormulaBuilder(boost::shared_ptr<api::IRegressionModelTemplate::RegressorFormulaBuilder_t> formulaBuilder, QString instantiationMessage) = 0;

	virtual const std::vector<boost::shared_ptr<api::IRegressionModelTemplate::RegressorFormulaBuilder_t> > &getRegressorFormulaBuilders() const = 0;

	virtual bool hasInstantiationMessageFor(boost::shared_ptr<api::IRegressionModelTemplate::RegressorFormulaBuilder_t> formulaBuilder) = 0;

	virtual QString resolveInstantiationMessageFor(boost::shared_ptr<api::IRegressionModelTemplate::RegressorFormulaBuilder_t> formulaBuilder) = 0;

	virtual std::vector<QString> resolveInstantiationMessages() = 0;

	virtual bool availableInCurrentMode() = 0;

};

} //namespace api
} //namespace r
} //namespace model
} //namespace


#endif /* IREGRESSIONMODELTEMPLATE_HPP_ */
