/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * IRegressor.hpp
 *
 *  Created on: 13.02.2015
 *      Author: gerberma
 */

#ifndef IREGRESSOR_HPP_
#define IREGRESSOR_HPP_

#include <boost/shared_ptr.hpp>
#include "model/SignalPolicies.hpp"
#include "IDataFrame.hpp"
#include "IAssignmentCategory.hpp"
#include "IExpressionRegressorFormulaBuilder.hpp"

namespace dambase {
namespace model {
namespace r {
namespace api {

class IRegressor : public communication::BoostSignalPolicy {
public:
	typedef ::dambase::model::r::DataFrameColumn Influence_Factor_t;
	typedef IDataFrame DataFrame_t;
	typedef IExpressionRegressorFormulaBuilder RegressorFormulaBuilder_t;
	typedef IAssignmentCategory AssignmentCategory_t;
public:
	IRegressor(){;}
	virtual ~IRegressor(){;}

	/**
	 * This is primarly used by the serialization process
	 */
	virtual const boost::shared_ptr<DataFrame_t> &getDataFrame() const = 0;

	/**
	 * This is primarly used by the deserialization process
	 */
	virtual void setDataFrame(boost::shared_ptr<DataFrame_t> dataFrame) = 0;

	//mostly for gui use only as the R formula for example should be resolved internally
	virtual const boost::shared_ptr<Influence_Factor_t> &getInfluenceFactor() const = 0;

	/**
	 * This is primarly used by the deserialization process.
	 */
	virtual void setRegressorFormulaBuilder(boost::shared_ptr<RegressorFormulaBuilder_t> regressorFormulaBuilder) = 0;

	virtual const boost::shared_ptr<RegressorFormulaBuilder_t> &getRegressorFormulaBuilder() const = 0;

	virtual boost::shared_ptr<AssignmentCategory_t> getAssignmentCategory() const = 0;

	virtual QString getRFormula(int overallMaxStartupTime) = 0;

	virtual QString getFormula() = 0;

	virtual QString getName() = 0;

	virtual DataFrame_t::Influence_Factor_Container_t getAvailableInfluenceFactors() = 0;

	virtual void setInfluenceFactor(int influenceFactorIndex) = 0;

	virtual void setInfluenceFactor(boost::shared_ptr<Influence_Factor_t> influenceFactorToBeSet) = 0;

	/**
	 * This is primary used by the deserialization process
	 */
	virtual void restoreInfluenceFactor(boost::shared_ptr<Influence_Factor_t> influenceFactor) = 0;

	virtual void freezeModification() = 0;

	virtual void releaseForModification() = 0;

	virtual const bool &isEditable() const = 0;

	/**
	 * This is primary used by the deserialization process. Use either #releaseForModification or #freezeModification for other purposes
	 */
	virtual void setIsEditable(bool isEditable) = 0;

	virtual void setIsActive(bool isactive) = 0;

	virtual const bool &isActive() const = 0;

	/**
	 * This is primary used by the deserialization process
	 */
	virtual void setIndex(int index) = 0;

	virtual const int &getIndex() const = 0;
};

} //namespace api
} //namespace r
} //namespace model
} //namespace


#endif /* IREGRESSOR_HPP_ */
