/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * Loader.hpp
 *
 *  Created on: Dec 5, 2013
 *      Author: gerberma
 */

#ifndef LOADER_HPP_
#define LOADER_HPP_

#include <algorithm>
#include <boost/bind.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/iterator/indirect_iterator.hpp>
#include "controller/AnalysisController.hpp"

namespace persistence {


template<typename T>
class Loader {
public:
	typedef boost::function<void(boost::shared_ptr<T>)> Functor_t;
public:
	Loader(Functor_t functor)
		:functor(functor){}

	void execute(boost::shared_ptr<T>);

protected:
	Functor_t functor;
};

template<typename T>
inline void persistence::Loader<T>::execute(boost::shared_ptr<T> t) {
	functor(t);
}


template<typename Project>
class ProjectLoader : public Loader<Project>
{
public:
	ProjectLoader(typename Loader<Project>::Functor_t localFunctor, Loader<typename Project::Analysis_t> *analysisLoader)
		:Loader<Project>(localFunctor), analysisLoader(analysisLoader)
	{}

	void execute(boost::shared_ptr<Project> project) {
		std::for_each(
				project->getAnalyses().begin(),
				project->getAnalyses().end(),
				boost::bind<void>(&Loader<typename Project::Analysis_t>::execute, analysisLoader, _1)
				);

		Loader<Project>::functor(project);
	}

private:
	Loader<typename Project::Analysis_t> *analysisLoader;
};

} /* namespace persistence */

#endif /* LOADER_HPP_ */
