/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitAnalysisRegressorTimeseriesSubscriptionManager.hpp
 *
 *  Created on: 28.10.2014
 *      Author: gerberma
 */

#ifndef REGRESSIONFITANALYSISREGRESSORTIMESERIESSUBSCRIPTIONMANAGER_HPP_
#define REGRESSIONFITANALYSISREGRESSORTIMESERIESSUBSCRIPTIONMANAGER_HPP_

#include <boost/bind.hpp>
#include <boost/shared_ptr.hpp>
#include "communication/Signals.hpp"

namespace subscription {

template<
	typename AnalysisToolsControllerSubscriptionManager
>
class RegressionFitAnalysisRegressorTimeseriesSubscriptionManager {
public:
	RegressionFitAnalysisRegressorTimeseriesSubscriptionManager(AnalysisToolsControllerSubscriptionManager *delegateSubscriptionManager)
		:delegateSubscriptionManager(delegateSubscriptionManager)
	{;}

	template<typename Controller, typename OverviewUi, typename RegressorTimeseries>
	void subscribe(Controller *controller, OverviewUi *overviewUi, boost::shared_ptr<RegressorTimeseries> regressorTimeseries) {
		using namespace boost;

		delegateSubscriptionManager->subscribe(controller, overviewUi, regressorTimeseries);
		typename OverviewUi::TargetAnalysisToolWidget_t *targetAnalysisToolWidget = overviewUi->getTargetAnalysisToolWidget();

		COM_SUBSCRIBE_FOR(targetAnalysisToolWidget, SetPolynomState, (bind<void>(&Controller::setPolynomState, controller, _1, _2)));
		COM_SUBSCRIBE_FOR(targetAnalysisToolWidget, SetScalingState, (bind<void>(&Controller::setScalingState, controller, _1, _2)));
	}

private:
	AnalysisToolsControllerSubscriptionManager *delegateSubscriptionManager;
};

}  // namespace subscription


#endif /* REGRESSIONFITANALYSISREGRESSORTIMESERIESSUBSCRIPTIONMANAGER_HPP_ */
