/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionModelSubscriptionManager.hpp
 *
 *  Created on: Feb 21, 2014
 *      Author: gerberma
 */

#ifndef REGRESSIONMODELSUBSCRIPTIONMANAGER_HPP_
#define REGRESSIONMODELSUBSCRIPTIONMANAGER_HPP_

#include <boost/bind.hpp>
#include <boost/function.hpp>
#include <boost/shared_ptr.hpp>
#include "communication/Signals.hpp"

namespace subscription {

template
<
	typename OverviewUi,
	typename CommonParameterUi,
	typename BehaviorIndicatorUi,
	typename RegressionFormulaUi,
	typename AlgorithmUi,
	typename TimeRangeUi,
	typename RegressionExecuteUi,
	typename WizardUi,
	typename InfluenceFactorSelectionUi,
	typename RegressionModelDefinitionEditUi,
	typename RegressorFormulaBuilderEditUi,
	typename Factory,
	typename Controller,
	typename AnalysisToolsController,
	typename RegressionFormulaMethodsController
>
class RegressionModelSubscriptionManager {
public:
	RegressionModelSubscriptionManager(Factory *factory, Controller *controller, AnalysisToolsController *analysisToolsController, RegressionFormulaMethodsController *regressionFormulaMethodsController)
		:factory(factory), controller(controller), analysisToolsController(analysisToolsController), regressionFormulaMethodsController(regressionFormulaMethodsController)
	{;}

	virtual ~RegressionModelSubscriptionManager()
	{;}

	void subscribe(OverviewUi *) {

	}

	void subscribe(WizardUi *wizardUi) {
		using namespace boost;

		COM_SUBSCRIBE_FOR(wizardUi, SetName, bind(&Controller::setName, controller, _1, _2));
		COM_SUBSCRIBE_FOR(wizardUi, SetDescription, bind(&Controller::setDescription, controller, _1, _2));
		COM_SUBSCRIBE_FOR(wizardUi, FromDateChanged, bind(&Controller::fromDateChanged, controller, _1, _2));
		COM_SUBSCRIBE_FOR(wizardUi, ToDateChanged, bind(&Controller::toDateChanged, controller, _1, _2));
		COM_SUBSCRIBE_FOR(wizardUi, BehaviorIndicatorChanged, bind(&Controller::behaviorIndicatorChanged, controller, _1, _2));
		COM_SUBSCRIBE_FOR(wizardUi, AddRegressionFormulaMethodInstance, bind(&Controller::template instantiateModelTemplate<
				Factory,
				RegressionModelSubscriptionManager<OverviewUi, CommonParameterUi, BehaviorIndicatorUi, RegressionFormulaUi, AlgorithmUi, TimeRangeUi, RegressionExecuteUi, WizardUi, InfluenceFactorSelectionUi, RegressionModelDefinitionEditUi, RegressorFormulaBuilderEditUi, Factory, Controller, AnalysisToolsController, RegressionFormulaMethodsController> >,
				controller, _1, _2, factory, this));
		COM_SUBSCRIBE_FOR(wizardUi, AlgorithmChanged, bind(&Controller::algorithmChanged, controller, _1, _2));
		COM_SUBSCRIBE_FOR(wizardUi, PraisWinstenIterationsChanged, bind(&Controller::setPraisWinstenIterations, controller, _1, _2));
		COM_SUBSCRIBE_FOR(wizardUi, RidgeKValueChanged, bind(&Controller::setRidgeKValue, controller, _1, _2));
		COM_SUBSCRIBE_FOR(wizardUi, PCRrValueChanged, bind(&Controller::setPCRrValue, controller, _1, _2));
		//subscribeDelegates(wizardUi);
	}

	template<typename RegressionModel>
	void subscribe(RegressionModelDefinitionEditUi *modelDefinitionEditUi, boost::shared_ptr<RegressionModel> regressionModel) {
		using namespace boost;

		typedef typename RegressionModelDefinitionEditUi::ModelDefinition_t ModelDefinition_t;

		COM_SUBSCRIBE_FOR(modelDefinitionEditUi, AddRegressor, bind(&Controller::template addRegressor<ModelDefinition_t, Factory, RegressionModelSubscriptionManager<OverviewUi, CommonParameterUi, BehaviorIndicatorUi, RegressionFormulaUi, AlgorithmUi, TimeRangeUi, RegressionExecuteUi, WizardUi, InfluenceFactorSelectionUi, RegressionModelDefinitionEditUi, RegressorFormulaBuilderEditUi, Factory, Controller, AnalysisToolsController, RegressionFormulaMethodsController> >, controller, regressionModel, _1, _2, _3, factory, this, QString("")));
		COM_SUBSCRIBE_FOR(modelDefinitionEditUi, RemoveRegressor, bind(&Controller::template removeRegressor<ModelDefinition_t>, controller, regressionModel, _1, _2));
		COM_SUBSCRIBE_FOR(modelDefinitionEditUi, Modify, bind(&Controller::releaseForModification, controller, _1, _2));
		COM_SUBSCRIBE_FOR(modelDefinitionEditUi, Remove, bind(&Controller::removeRegressionModelDefinition, controller, _1, _2));
	}

	template<typename RegressionModel>
	void subscribe(RegressorFormulaBuilderEditUi *formulaBuilderEditUi, boost::shared_ptr<RegressionModel> regressionModel) {
		using namespace boost;

		typedef typename RegressorFormulaBuilderEditUi::RegressorFormulaBuilder_t RegressorFormulaBuilder_t;
		typedef typename RegressorFormulaBuilder_t::UserParameter_t UserParameter_t;
		typedef typename RegressorFormulaBuilder_t::RegressorFormulaBuilderPreprocessor_t RegressorFormulaBuilderPreprocessor_t;

		COM_SUBSCRIBE_FOR(formulaBuilderEditUi, SetIdentifier, bind(&Controller::template setRegressorFormulaBuilderIdentifier<RegressorFormulaBuilder_t>, controller, _1, _2));
		COM_SUBSCRIBE_FOR(formulaBuilderEditUi, SetFormula, bind(&Controller::template setRegressorFormulaBuilderFormula<RegressorFormulaBuilder_t>, controller, _1, _2));
		COM_SUBSCRIBE_FOR(formulaBuilderEditUi, SetAssignmentCategory, bind(&Controller::template setRegressorFormulaBuilderAssignmentCategory<RegressorFormulaBuilder_t>, controller, _1, _2));
		COM_SUBSCRIBE_FOR(formulaBuilderEditUi, SetFormulaDefinition, bind(&Controller::template setRegressorFormulaBuilderFormulaDefinition<RegressorFormulaBuilder_t>, controller, _1, _2));
		COM_SUBSCRIBE_FOR(formulaBuilderEditUi, Derive, bind(&Controller::template deriveRegressorFormulaBuilder<RegressorFormulaBuilder_t, Factory>, controller, _1, regressionModel, factory));
		COM_SUBSCRIBE_FOR(formulaBuilderEditUi, CreateUserParameter, bind(&Controller::template createUserParameter<RegressorFormulaBuilder_t, Factory>, controller, regressionModel, _1, factory));
		COM_SUBSCRIBE_FOR(formulaBuilderEditUi, RemoveUserParameter, bind(&Controller::template removeUserParameter<RegressorFormulaBuilder_t>, controller, regressionModel, _1, _2));
		COM_SUBSCRIBE_FOR(formulaBuilderEditUi, UserParameterSetValue, bind(&Controller::template setUserParameterValue<UserParameter_t>, controller, regressionModel, _1, _2));
		COM_SUBSCRIBE_FOR(formulaBuilderEditUi, UserParameterSetIsMinimumLeadTime, bind(&Controller::template setUserParameterIsMinimumLeadTime<UserParameter_t>, controller, regressionModel, _1, _2));
		COM_SUBSCRIBE_FOR(formulaBuilderEditUi, PreprocessorStateChanged, bind(&Controller::template setRegressorFormulaBuilderPreprocessorIsActive<RegressorFormulaBuilderPreprocessor_t>, controller, _1, _2));
	}

	void subscribe(InfluenceFactorSelectionUi *influenceFactorSelectionUi) {
		using namespace boost;

		typedef typename InfluenceFactorSelectionUi::Regressor_t Regressor_t;
		COM_SUBSCRIBE_FOR(influenceFactorSelectionUi, SetInfluenceFactor, bind(&Controller::template setRegressorInfluenceFactor<Regressor_t>, controller, _1, _2));
	}

	void subscribe(RegressionFormulaUi *regressionFormulaUi) {
		using namespace boost;

		COM_SUBSCRIBE_FOR(regressionFormulaUi, Edit, bind<void>(&Controller::template editRegressionFormula<
				RegressionModelSubscriptionManager<OverviewUi, CommonParameterUi, BehaviorIndicatorUi, RegressionFormulaUi, AlgorithmUi, TimeRangeUi, RegressionExecuteUi, WizardUi, InfluenceFactorSelectionUi, RegressionModelDefinitionEditUi, RegressorFormulaBuilderEditUi, Factory, Controller, AnalysisToolsController, RegressionFormulaMethodsController>
				>, controller, _1, this));
	}

	void subscribe(AlgorithmUi *algorithmUi) {
		using namespace boost;

		COM_SUBSCRIBE_FOR(algorithmUi, Edit, bind<void>(&Controller::template editAlgorithm<
				RegressionModelSubscriptionManager<OverviewUi, CommonParameterUi, BehaviorIndicatorUi, RegressionFormulaUi, AlgorithmUi, TimeRangeUi, RegressionExecuteUi, WizardUi, InfluenceFactorSelectionUi, RegressionModelDefinitionEditUi, RegressorFormulaBuilderEditUi, Factory, Controller, AnalysisToolsController, RegressionFormulaMethodsController>
				>, controller, _1, this));
	}

	void subscribe(TimeRangeUi *timeRangeUi) {
		using namespace boost;

		COM_SUBSCRIBE_FOR(timeRangeUi, Edit, bind<void>(&Controller::template editTimeRange<
				RegressionModelSubscriptionManager<OverviewUi, CommonParameterUi, BehaviorIndicatorUi, RegressionFormulaUi, AlgorithmUi, TimeRangeUi, RegressionExecuteUi, WizardUi, InfluenceFactorSelectionUi, RegressionModelDefinitionEditUi, RegressorFormulaBuilderEditUi, Factory, Controller, AnalysisToolsController, RegressionFormulaMethodsController>
				>, controller, _1, this));
	}

	void subscribe(CommonParameterUi *commonParameterUi) {
		using namespace boost;

		COM_SUBSCRIBE_FOR(commonParameterUi, Edit, bind<void>(&Controller::template editCommonParameters<
				RegressionModelSubscriptionManager<OverviewUi, CommonParameterUi, BehaviorIndicatorUi, RegressionFormulaUi, AlgorithmUi, TimeRangeUi, RegressionExecuteUi, WizardUi, InfluenceFactorSelectionUi, RegressionModelDefinitionEditUi, RegressorFormulaBuilderEditUi, Factory, Controller, AnalysisToolsController, RegressionFormulaMethodsController>
				>, controller, _1, this));
	}

	void subscribe(BehaviorIndicatorUi *behaviorIndicatorUi) {
		using namespace boost;

		COM_SUBSCRIBE_FOR(behaviorIndicatorUi, Edit, bind<void>(&Controller::template editBehaviorIndicator<
				RegressionModelSubscriptionManager<OverviewUi, CommonParameterUi, BehaviorIndicatorUi, RegressionFormulaUi, AlgorithmUi, TimeRangeUi, RegressionExecuteUi, WizardUi, InfluenceFactorSelectionUi, RegressionModelDefinitionEditUi, RegressorFormulaBuilderEditUi, Factory, Controller, AnalysisToolsController, RegressionFormulaMethodsController>
				>, controller, _1, this));
	}

	void subscribe(RegressionExecuteUi *regressionExecuteUi) {
		using namespace boost;

		COM_SUBSCRIBE_FOR(regressionExecuteUi, Clicked, bind<void>(&Controller::calculateRegressionFit, controller, _1));
	}

	template<typename CategoryWidget, typename Analysis, typename RegressionModel>
	void subscribeDelegates(CategoryWidget *categoryWidget, boost::shared_ptr<Analysis> analysis, boost::shared_ptr<RegressionModel> regressionModel) {
		using namespace boost;
		COM_SUBSCRIBE_FOR(
				categoryWidget,
				AddAnalysisTool,
				(bind((void(AnalysisToolsController::*)(boost::shared_ptr<Analysis>, boost::shared_ptr<RegressionModel>, boost::shared_ptr<typename AnalysisToolsController::analysis_tool_description_t>, function<void(typename AnalysisToolsController::base_ui_t *)>))&AnalysisToolsController::addAnalysisTool,
						analysisToolsController, analysis, regressionModel, _1, _2)
				)
		);
	}

	/*
	void subscribeDelegates(WizardUi *wizardUi) {
		using namespace boost;
		COM_SUBSCRIBE_FOR(wizardUi, AddRegressionFormulaMethodInstance, bind(&RegressionFormulaMethodsController::addMethodsInstance, regressionFormulaMethodsController, _1, _2));
	}
	*/


private:
	Factory *factory;
	Controller *controller;
	AnalysisToolsController *analysisToolsController;
	RegressionFormulaMethodsController *regressionFormulaMethodsController;
};

} /* namespace subscription */

#endif /* REGRESSIONMODELSUBSCRIPTIONMANAGER_HPP_ */
