/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * ReportSubscriptionManager.hpp
 *
 *  Created on: Jul 24, 2014
 *      Author: gerberma
 */

#ifndef REPORTSUBSCRIPTIONMANAGER_HPP_
#define REPORTSUBSCRIPTIONMANAGER_HPP_

#include <QDebug>
#include <boost/bind.hpp>
#include <boost/shared_ptr.hpp>
#include "communication/Signals.hpp"

namespace subscription {

template
<
	typename WorkingDialogUi,
	typename ReportListTitleElementUi,
	typename ReportListDescriptionElementUi,
	typename ReportListProjectElementUi,
	typename ReportListAlgorithmElementUi,
	typename ReportListAnalysisToolElementUi,
	typename Controller
>
class ReportSubscriptionManager {
public:
	ReportSubscriptionManager(Controller *controller)
		:controller(controller)
	{;}

	virtual ~ReportSubscriptionManager()
	{;}

	void subscribe(WorkingDialogUi *workingDialogUi) {
		using namespace boost;

		COM_SUBSCRIBE_FOR(workingDialogUi, CreateReport, bind(&Controller::build, controller, _1));
		COM_SUBSCRIBE_FOR(workingDialogUi, PlotScaleFactorChanged, bind(&Controller::setPlotScaleFactor, controller, _1, _2));
		COM_SUBSCRIBE_FOR(workingDialogUi, FontSizeChanged, bind(&Controller::setFontSize, controller, _1, _2));
	}

	template<typename Report>
	void subscribe(boost::shared_ptr<Report> report, ReportListTitleElementUi *titleElementUi) {
		commonSubscriptionForElementUi(report, titleElementUi);
	}

	template<typename Report>
	void subscribe(boost::shared_ptr<Report> report, ReportListAnalysisToolElementUi *analysisToolElementUi) {
		commonSubscriptionForElementUi(report, analysisToolElementUi);
	}

	void subscribe(typename ReportListTitleElementUi::DynamicContentWidget_t *dynamicTitleElementUi) {
		using namespace boost;

		COM_SUBSCRIBE_FOR(dynamicTitleElementUi, TextChanged, bind<void>(&Controller::template setText<typename ReportListTitleElementUi::DynamicContentWidget_t::ReportListElement_t>, controller, _1, _2));
	}

	template<typename Report>
	void subscribe(boost::shared_ptr<Report> report, ReportListDescriptionElementUi *elementUi) {
		commonSubscriptionForElementUi(report, elementUi);
	}

	void subscribe(typename ReportListDescriptionElementUi::DynamicContentWidget_t *dynamicDescriptionElementUi) {
		using namespace boost;

		COM_SUBSCRIBE_FOR(dynamicDescriptionElementUi, TextChanged, bind<void>(&Controller::template setText<typename ReportListDescriptionElementUi::DynamicContentWidget_t::ReportListElement_t>, controller, _1, _2));
	}

	template<typename Report>
	void subscribe(boost::shared_ptr<Report> report, ReportListProjectElementUi *elementUi) {
		commonSubscriptionForElementUi(report, elementUi);
	}

	template<typename Report>
	void subscribe(boost::shared_ptr<Report> report, ReportListAlgorithmElementUi *elementUi) {
		commonSubscriptionForElementUi(report, elementUi);
	}


	template<typename Report, typename ElementUi>
	void commonSubscriptionForElementUi(boost::shared_ptr<Report> report, ElementUi *elementUi) {
		using namespace boost;

		COM_SUBSCRIBE_FOR(elementUi, SelectChanged, bind<void>(&Controller::template setSelectState<typename ElementUi::ReportListElement_t>, controller, _1, _2));
		COM_SUBSCRIBE_FOR(elementUi, MoveUp, bind<void>(&Controller::template moveUp<typename ElementUi::ReportListElement_t>, controller, report, _1));
		COM_SUBSCRIBE_FOR(elementUi, MoveDown, bind<void>(&Controller::template moveDown<typename ElementUi::ReportListElement_t>, controller, report, _1));
	}


private:
	Controller *controller;
};

}  // namespace subscription

#endif /* REPORTSUBSCRIPTIONMANAGER_HPP_ */
