/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
//#define BOOST_TEST_MAIN
#define BOOST_TEST_MODULE CoreControllerTests

#include <boost/test/unit_test.hpp>
#include <turtle/mock.hpp>
#include "controller/CoreController.hpp"
#include "app/GenericApplicationContext.hpp"
#include "tests/helper/GenericApplicationContextTestHelper.hpp"

using namespace app::context;
using namespace test::helper;


MOCK_CLASS(ClosePolicy) {
	MOCK_METHOD_EXT(close, 0, void(), close)
};

MOCK_CLASS(CoreUiMock) {
	MOCK_METHOD_EXT(start, 0, void(), start);
	MOCK_METHOD_EXT(close, 0, void(), close);
};

MOCK_CLASS(InfoDialog) {
};

MOCK_CLASS(ProjectController) {
};

MOCK_CLASS(ModelMock) {};

typedef controller::CoreController<ModelMock, CoreUiMock, InfoDialog, ProjectController> CoreController_t;



CoreController_t *coreControllerFactory() {
	using namespace app::context;
	return new CoreController_t(FETCH_FROM_CTX(ModelMock), FETCH_FROM_CTX(CoreUiMock), FETCH_FROM_CTX(ProjectController));
}



class ApplicationContext {
public:
	ApplicationContext() :
		GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(CoreController_t, coreControllerFactory),
		GAC_DEFAULT_CTOR_INIT(CoreUiMock),
		GAC_DEFAULT_CTOR_INIT(ProjectController),
		GAC_DEFAULT_CTOR_INIT(ModelMock)
	{}

private:
	GAC_SCOPE(CoreController_t)
	GAC_SCOPE(ProjectController)
	GAC_SCOPE(CoreUiMock)
	GAC_SCOPE(ModelMock)
};


BOOST_FIXTURE_TEST_CASE(TestStartCoreWindowOnStart, ApplicationContext)
{
	CoreController_t *coreController = FETCH_FROM_CTX(CoreController_t);

	MOCK_EXPECT(FETCH_FROM_CTX(CoreUiMock)->start).once();

	coreController->start();
}

BOOST_FIXTURE_TEST_CASE(TestCloseRequiresClosePolicyGettingCalled, ApplicationContext)
{
	CoreController_t *coreController = FETCH_FROM_CTX(CoreController_t);

	MOCK_EXPECT(FETCH_FROM_CTX(CoreUiMock)->close).once();

	coreController->close();
}
