/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelAnalysisInfluenceFactorsControllerTests.cpp
 *
 *  Created on: Feb 4, 2014
 *      Author: gerberma
 */

#ifndef DATAMODELANALYSISINFLUENCEFACTORSCONTROLLERTESTS_CPP_
#define DATAMODELANALYSISINFLUENCEFACTORSCONTROLLERTESTS_CPP_

#define BOOST_TEST_MODULE DataModelAnalysisInfluenceFactorsControllerTests
#include <boost/test/unit_test.hpp>
#include <turtle/mock.hpp>
#include <turtle/constraints.hpp>
#include <boost/any.hpp>
#include <boost/utility.hpp>
#include <boost/bind.hpp>
#include <string>
#include "controller/DataModelAnalysisInfluenceFactorsController.hpp"
#include "app/GenericApplicationContext.hpp"
#include "tests/helper/GenericApplicationContextTestHelper.hpp"

using namespace app::context;

MOCK_CLASS(AnyDataModelAnalysis) {
	MOCK_METHOD_EXT(execute, 0, void(), execute);
	MOCK_METHOD_EXT(estimateCurrentBehaviorIndicator, 0, void(), estimateCurrentBehaviorIndicator);
	MOCK_METHOD_EXT(estimateCurrentInfluenceFactor, 0, void(), estimateCurrentInfluenceFactor);
};

MOCK_CLASS(DataModel) {
};

MOCK_CLASS(Analysis) {
	MOCK_METHOD_EXT(getDataModel, 0, DataModel *(), getDataModel);
	MOCK_METHOD_EXT(addDataModelAnalysis, 1, void(AnyDataModelAnalysis *), addDataModelAnalysis);
};

MOCK_CLASS(DataModelAnalysisToolDescription) {
};

MOCK_CLASS(AnalysisToolFactory) {
public:
	typedef AnyDataModelAnalysis analysis_tool_t;

	template<typename T>
	T *create(DataModel *dataModel) {
		return delegateCreate(dataModel);
	}
	MOCK_METHOD_EXT(delegateCreate, 1, AnyDataModelAnalysis *(DataModel *), delegateCreate);
};

MOCK_CLASS(AnalysisToolUi) {
};

MOCK_CLASS(UiManager) {
public:
	typedef AnalysisToolUi base_ui_type;
	typedef AnalysisToolUi overview_ui_type;

	MOCK_METHOD_EXT(getOverviewUiFor, 1, AnalysisToolUi*(AnyDataModelAnalysis *), getOverviewUiFor);
};

MOCK_CLASS(SubscriptionManager) {
	MOCK_METHOD_EXT(subscribeOverviewUi, 3, void(AnalysisToolUi *, void*, Analysis *), subscribeOverviewUi);
};

class AnyAnalysisToolWidgetContainer {
public:
	void callback(AnalysisToolUi *){;}
};


typedef AnyAnalysisToolWidgetContainer AnyAnalysisToolWidgetContainer_t;
typedef DataModel DataModel_t;
typedef AnyDataModelAnalysis Any_DataModel_Analysis_t;
typedef Analysis Analysis_t;
typedef DataModelAnalysisToolDescription Analysis_Tool_Description_t;
typedef AnalysisToolFactory Analysis_Tool_Factory_t;
typedef UiManager Ui_Manager_t;
typedef AnalysisToolUi Analysis_Tool_Ui_t;
typedef SubscriptionManager Subscription_Manager_t;
typedef controller::DataModelAnalysisInfluenceFactorsController<Analysis_t, Any_DataModel_Analysis_t, Analysis_Tool_Factory_t, Ui_Manager_t, Subscription_Manager_t> DataModel_Analysis_Influence_Factors_Controller_t;



namespace test {

using namespace helper;

class ApplicationContext {
public:
	ApplicationContext():
		GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(DataModel_Analysis_Influence_Factors_Controller_t, boost::bind(&ApplicationContext::analysisToolsControllerFactory, this)),
		GAC_DEFAULT_CTOR_INIT(Analysis_Tool_Factory_t),
		GAC_DEFAULT_CTOR_INIT(Ui_Manager_t),
		GAC_DEFAULT_CTOR_INIT(Analysis_Tool_Ui_t),
		GAC_DEFAULT_CTOR_INIT(Subscription_Manager_t),
		GAC_DEFAULT_CTOR_INIT(Analysis_t),
		GAC_DEFAULT_CTOR_INIT(Analysis_Tool_Description_t),
		GAC_DEFAULT_CTOR_INIT(Any_DataModel_Analysis_t),
		GAC_DEFAULT_CTOR_INIT(DataModel_t),
		GAC_DEFAULT_CTOR_INIT(AnyAnalysisToolWidgetContainer_t)
	{}

private:

	DataModel_Analysis_Influence_Factors_Controller_t *analysisToolsControllerFactory() {
		return new DataModel_Analysis_Influence_Factors_Controller_t(
				FETCH_FROM_CTX(Analysis_Tool_Factory_t),
				FETCH_FROM_CTX(Ui_Manager_t),
				FETCH_FROM_CTX(Subscription_Manager_t)
				);
	}
private:
	GAC_SCOPE(DataModel_Analysis_Influence_Factors_Controller_t)
	GAC_SCOPE(Analysis_Tool_Factory_t)
	GAC_SCOPE(Ui_Manager_t)
	GAC_SCOPE(Analysis_Tool_Ui_t)
	GAC_SCOPE(Subscription_Manager_t)
	GAC_SCOPE(Analysis_t)
	GAC_SCOPE(Analysis_Tool_Description_t)
	GAC_SCOPE(Any_DataModel_Analysis_t)
	GAC_SCOPE(DataModel_t)
	GAC_SCOPE(AnyAnalysisToolWidgetContainer_t)
};
}


BOOST_FIXTURE_TEST_CASE(TestAddAnalysisTool, test::ApplicationContext) {
	MOCK_EXPECT(FETCH_FROM_CTX(Analysis_Tool_Factory_t)->delegateCreate).once().with(FETCH_FROM_CTX(DataModel_t)).returns(FETCH_FROM_CTX(Any_DataModel_Analysis_t));
	MOCK_EXPECT(FETCH_FROM_CTX(Analysis_t)->getDataModel).at_least(1).returns(FETCH_FROM_CTX(DataModel_t));
	MOCK_EXPECT(FETCH_FROM_CTX(Any_DataModel_Analysis_t)->estimateCurrentBehaviorIndicator).once();
	MOCK_EXPECT(FETCH_FROM_CTX(Any_DataModel_Analysis_t)->estimateCurrentInfluenceFactor).once();
	MOCK_EXPECT(FETCH_FROM_CTX(Any_DataModel_Analysis_t)->execute).once();
	MOCK_EXPECT(FETCH_FROM_CTX(Analysis_t)->addDataModelAnalysis).once().with(FETCH_FROM_CTX(Any_DataModel_Analysis_t));
	MOCK_EXPECT(FETCH_FROM_CTX(Ui_Manager_t)->getOverviewUiFor).once().with(FETCH_FROM_CTX(Any_DataModel_Analysis_t)).returns(FETCH_FROM_CTX(Analysis_Tool_Ui_t));
	MOCK_EXPECT(FETCH_FROM_CTX(Subscription_Manager_t)->subscribeOverviewUi).once().with(FETCH_FROM_CTX(Analysis_Tool_Ui_t), FETCH_FROM_CTX(DataModel_Analysis_Influence_Factors_Controller_t), FETCH_FROM_CTX(Analysis_t));
	//MOCK_EXPECT(FETCH_FROM_CTX(Subscription_Manager_t)->subscribeAnalysisToolUi).once().with(FETCH_FROM_CTX(Analysis_Tool_Ui_t), FETCH_FROM_CTX(Analysis_Tool_Description_t));


	FETCH_FROM_CTX(DataModel_Analysis_Influence_Factors_Controller_t)->addAnalysisTool(FETCH_FROM_CTX(Analysis_t), boost::bind(&AnyAnalysisToolWidgetContainer_t::callback, FETCH_FROM_CTX(AnyAnalysisToolWidgetContainer_t), _1));
}



#endif /* DATAMODELANALYSISINFLUENCEFACTORSCONTROLLERTESTS_CPP_ */
