/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelControllerInternalSubscriptionManagerImplTests.cpp
 *
 *  Created on: Jan 20, 2014
 *      Author: gerberma
 */

#define BOOST_TEST_MODULE DataModelControllerInternalSubscriptionManagerImplTests
#include <boost/test/unit_test.hpp>
#include <turtle/mock.hpp>
#include <turtle/constraints.hpp>
#include <boost/any.hpp>
#include <boost/utility.hpp>
#include <boost/bind.hpp>
#include <string>
#include "controller/DataModelControllerInternalSubscriptionManagerImpl.hpp"
#include "app/GenericApplicationContext.hpp"
#include "tests/helper/GenericApplicationContextTestHelper.hpp"

using namespace app::context;

MOCK_CLASS(Controller) {
	void missingDataStrategyChange(void *, unsigned) {
	}

	void otherMissingDataStrategyApply(void *, unsigned) {
	}

	void requestSourceUri(void *) {
	}

	void dateColumnChange(void *, unsigned) {
	}

	void behaviorIndicatorAssignmentChanged(void *, unsigned, unsigned) {
	}

	void waterLevelAssignmentChanged(void *, unsigned, unsigned) {
	}

	void temperatureAssignmentChanged(void *, unsigned, unsigned) {
	}

	void editCommonParameters(void *) {
	}

	void editTimeSeries(void *) {
	}

	void editAssignment(void *) {
	}
};

MOCK_CLASS(Analysis) {
};

MOCK_CLASS(AnalysisToolDescription) {
};

MOCK_CLASS(UiManager) {
	typedef void analysis_tool_ui_type;
	typedef void base_ui_t;
};

MOCK_CLASS(AnalysisToolsController) {
public:
	typedef UiManager ui_manager_t;
	typedef void base_ui_t;
	typedef AnalysisToolDescription analysis_tool_description_t;
	void addAnalysisTool(Analysis *, boost::any){}
};

MOCK_CLASS(Wizard) {
	MOCK_METHOD_EXT(subscribeForMissingDataStrategySelected, 1, void(boost::any), subscribeForMissingDataStrategySelected);
	MOCK_METHOD_EXT(subscribeForOtherMissingDataStrategySelected, 1, void(boost::any), subscribeForOtherMissingDataStrategySelected);
	MOCK_METHOD_EXT(subscribeForRequestSourceUri, 1, void(boost::any), subscribeForRequestSourceUri);
	MOCK_METHOD_EXT(subscribeForDateColumnChanged, 1, void(boost::any), subscribeForDateColumnChanged);
	MOCK_METHOD_EXT(subscribeForBehaviorIndicatorAssignmentChanged, 1, void(boost::any), subscribeForBehaviorIndicatorAssignmentChanged);
	MOCK_METHOD_EXT(subscribeForWaterLevelAssignmentChanged, 1, void(boost::any), subscribeForWaterLevelAssignmentChanged);
	MOCK_METHOD_EXT(subscribeForTemperatureAssignmentChanged, 1, void(boost::any), subscribeForTemperatureAssignmentChanged);
};

MOCK_CLASS(OverviewUi) {
	MOCK_METHOD_EXT(subscribeForEditCommonParameters, 1, void(boost::any), subscribeForEditCommonParameters);
	MOCK_METHOD_EXT(subscribeForEditTimeSeries, 1, void(boost::any), subscribeForEditTimeSeries);
	MOCK_METHOD_EXT(subscribeForEditAssignment, 1, void(boost::any), subscribeForEditAssignment);
	MOCK_METHOD_EXT(subscribeForAddAnalysisTool, 1, void(boost::any), subscribeForAddAnalysisTool);
	MOCK_METHOD_EXT(subscribeForAddLeftSideAnalysisTool, 1, void(boost::any), subscribeForAddLeftSideAnalysisTool);
};

typedef OverviewUi Overview_Ui_t;
typedef Controller Controller_t;
typedef Analysis Analysis_t;
typedef AnalysisToolsController Analysis_Tools_Controller_t;
typedef Wizard Wizard_t;
typedef controller::DataModelControllerInternalSubscriptionManagerImpl DataModel_Controller_Internal_Subscription_Manager_Impl_t;

namespace test{
using namespace helper;
class ApplicationContext {
public:
	ApplicationContext():
		GAC_DEFAULT_CTOR_INIT(Wizard_t),
		GAC_DEFAULT_CTOR_INIT(Overview_Ui_t),
		GAC_DEFAULT_CTOR_INIT(DataModel_Controller_Internal_Subscription_Manager_Impl_t),
		GAC_DEFAULT_CTOR_INIT(Controller_t),
		GAC_DEFAULT_CTOR_INIT(Analysis_Tools_Controller_t),
		GAC_DEFAULT_CTOR_INIT(Analysis)
	{}

private:
	GAC_SCOPE(Wizard_t)
	GAC_SCOPE(Overview_Ui_t)
	GAC_SCOPE(DataModel_Controller_Internal_Subscription_Manager_Impl_t)
	GAC_SCOPE(Controller_t)
	GAC_SCOPE(Analysis_Tools_Controller_t)
	GAC_SCOPE(Analysis)
};
} /* namespace test */


BOOST_FIXTURE_TEST_CASE(TestSubscribeForWizard, test::ApplicationContext)
{
	MOCK_EXPECT(FETCH_FROM_CTX(Wizard_t)->subscribeForMissingDataStrategySelected).once().with(mock::any);
	MOCK_EXPECT(FETCH_FROM_CTX(Wizard_t)->subscribeForOtherMissingDataStrategySelected).once().with(mock::any);
	MOCK_EXPECT(FETCH_FROM_CTX(Wizard_t)->subscribeForRequestSourceUri).once().with(mock::any);
	MOCK_EXPECT(FETCH_FROM_CTX(Wizard_t)->subscribeForDateColumnChanged).once().with(mock::any);
	MOCK_EXPECT(FETCH_FROM_CTX(Wizard_t)->subscribeForBehaviorIndicatorAssignmentChanged).once().with(mock::any);
	MOCK_EXPECT(FETCH_FROM_CTX(Wizard_t)->subscribeForWaterLevelAssignmentChanged).once().with(mock::any);
	MOCK_EXPECT(FETCH_FROM_CTX(Wizard_t)->subscribeForTemperatureAssignmentChanged).once().with(mock::any);

	FETCH_FROM_CTX(DataModel_Controller_Internal_Subscription_Manager_Impl_t)->subscribeWizard(FETCH_FROM_CTX(Wizard_t), FETCH_FROM_CTX(Controller_t));
}


BOOST_FIXTURE_TEST_CASE(TestSubscribeForOverviewUi, test::ApplicationContext)
{
	MOCK_EXPECT(FETCH_FROM_CTX(Overview_Ui_t)->subscribeForEditCommonParameters).once().with(mock::any);
	MOCK_EXPECT(FETCH_FROM_CTX(Overview_Ui_t)->subscribeForEditTimeSeries).once().with(mock::any);
	MOCK_EXPECT(FETCH_FROM_CTX(Overview_Ui_t)->subscribeForEditAssignment).once().with(mock::any);

	FETCH_FROM_CTX(DataModel_Controller_Internal_Subscription_Manager_Impl_t)->subscribeOverviewUi(FETCH_FROM_CTX(Overview_Ui_t), FETCH_FROM_CTX(Controller_t));
}


BOOST_FIXTURE_TEST_CASE(TestSubscribeForOverviewUiDelegates, test::ApplicationContext)
{
	MOCK_EXPECT(FETCH_FROM_CTX(Overview_Ui_t)->subscribeForAddAnalysisTool).once().with(mock::any);
	MOCK_EXPECT(FETCH_FROM_CTX(Overview_Ui_t)->subscribeForAddLeftSideAnalysisTool).once().with(mock::any);

	FETCH_FROM_CTX(DataModel_Controller_Internal_Subscription_Manager_Impl_t)->subscribeOverviewUiDelegates(
			FETCH_FROM_CTX(Overview_Ui_t), FETCH_FROM_CTX(Analysis_t), FETCH_FROM_CTX(Analysis_Tools_Controller_t));
}
