/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelControllerInternalSubscriptionManagerTests.cpp
 *
 *  Created on: Jan 21, 2014
 *      Author: gerberma
 */


#define BOOST_TEST_MODULE DataModelControllerInternalSubscriptionManagerTests
#include <boost/test/unit_test.hpp>
#include <turtle/mock.hpp>
#include <turtle/constraints.hpp>
#include <boost/any.hpp>
#include <boost/utility.hpp>
#include <boost/bind.hpp>
#include <string>
#include "controller/DataModelControllerInternalSubscriptionManager.hpp"
#include "app/GenericApplicationContext.hpp"
#include "tests/helper/GenericApplicationContextTestHelper.hpp"

using namespace app::context;

template<int Id>
class NullStruct {
};

typedef NullStruct<1> Controller_t;
typedef NullStruct<2> Analysis_Tools_Controller_t;
typedef NullStruct<3> Wizard_t;
typedef NullStruct<4> Overview_Ui_t;
typedef NullStruct<5> Analysis_t;

MOCK_CLASS(SubscriptionManagerImpl) {
	MOCK_METHOD_EXT(subscribeWizard, 2, void(Wizard_t *, Controller_t *), subscribeWizard);
	MOCK_METHOD_EXT(subscribeOverviewUi, 2, void(Overview_Ui_t *, Controller_t *), subscribeOverviewUi);
	MOCK_METHOD_EXT(subscribeOverviewUiDelegates, 3, void(Overview_Ui_t *, Analysis_t *, Analysis_Tools_Controller_t *), subscribeOverviewUiDelegates);
};
typedef SubscriptionManagerImpl Delegate_Subscription_Manager_t;
typedef controller::DataModelControllerInternalSubscriptionManager<Analysis_Tools_Controller_t, Delegate_Subscription_Manager_t> DataModel_Controller_Internal_Subscription_Manager_t;


namespace test{
using namespace helper;
class ApplicationContext {
public:
	ApplicationContext():
		GAC_DEFAULT_CTOR_INIT(Analysis_t),
		GAC_DEFAULT_CTOR_INIT(Wizard_t),
		GAC_DEFAULT_CTOR_INIT(Overview_Ui_t),
		GAC_DEFAULT_CTOR_INIT(Controller_t),
		GAC_DEFAULT_CTOR_INIT(Analysis_Tools_Controller_t),
		GAC_DEFAULT_CTOR_INIT(Delegate_Subscription_Manager_t),
		GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(DataModel_Controller_Internal_Subscription_Manager_t, (boost::bind(&ApplicationContext::factory, this)))
	{}

private:
	DataModel_Controller_Internal_Subscription_Manager_t *factory() {
		return new 	DataModel_Controller_Internal_Subscription_Manager_t(
				FETCH_FROM_CTX(Delegate_Subscription_Manager_t), FETCH_FROM_CTX(Analysis_Tools_Controller_t)
				);
	}

private:
	GAC_SCOPE(Analysis_t)
	GAC_SCOPE(Wizard_t)
	GAC_SCOPE(Overview_Ui_t)
	GAC_SCOPE(Controller_t)
	GAC_SCOPE(Analysis_Tools_Controller_t)
	GAC_SCOPE(Delegate_Subscription_Manager_t)
	GAC_SCOPE(DataModel_Controller_Internal_Subscription_Manager_t)
};
} /* namespace test */


BOOST_FIXTURE_TEST_CASE(TestDelegateSubscribeWizard, test::ApplicationContext) {
	Wizard_t *wizard = FETCH_FROM_CTX(Wizard_t);
	Controller_t *controller = FETCH_FROM_CTX(Controller_t);

	MOCK_EXPECT(FETCH_FROM_CTX(Delegate_Subscription_Manager_t)->subscribeWizard).once().with(wizard, controller);

	FETCH_FROM_CTX(DataModel_Controller_Internal_Subscription_Manager_t)->subscribeWizard(wizard, controller);
}

BOOST_FIXTURE_TEST_CASE(TestDelegateSubscribeOverviewUi, test::ApplicationContext) {
	Overview_Ui_t *overviewUi = FETCH_FROM_CTX(Overview_Ui_t);
	Controller_t *controller = FETCH_FROM_CTX(Controller_t);

	MOCK_EXPECT(FETCH_FROM_CTX(Delegate_Subscription_Manager_t)->subscribeOverviewUi).once().with(overviewUi, controller);

	FETCH_FROM_CTX(DataModel_Controller_Internal_Subscription_Manager_t)->subscribeOverviewUi(overviewUi, controller);
}

BOOST_FIXTURE_TEST_CASE(TestDelegateSubscribeOverviewUiDelegate, test::ApplicationContext) {
	Overview_Ui_t *overviewUi = FETCH_FROM_CTX(Overview_Ui_t);
	Analysis_Tools_Controller_t *toolsController = FETCH_FROM_CTX(Analysis_Tools_Controller_t);


	MOCK_EXPECT(FETCH_FROM_CTX(Delegate_Subscription_Manager_t)->subscribeOverviewUiDelegates).once().with(overviewUi,  FETCH_FROM_CTX(Analysis_t), toolsController);

	FETCH_FROM_CTX(DataModel_Controller_Internal_Subscription_Manager_t)->subscribeOverviewUiDelegates(overviewUi, FETCH_FROM_CTX(Analysis_t));
}
