/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelControllerTests.cpp
 *
 *  Created on: Jan 20, 2014
 *      Author: gerberma
 */

#define BOOST_TEST_MODULE DataModelControllerTests
#include <boost/test/unit_test.hpp>
#include <turtle/mock.hpp>
#include <turtle/constraints.hpp>
#include <boost/any.hpp>
#include <boost/utility.hpp>
#include <string>
#include "controller/DataModelController.hpp"
#include "app/GenericApplicationContext.hpp"
#include "tests/helper/GenericApplicationContextTestHelper.hpp"

using namespace app::context;

class DontDeleteOnDeleteChildPolicy {
public:
	template<typename T>
	void deleteChild(T *) {
	}

	template<typename T>
	void overtakeOwnership(T *){
	}
};

MOCK_CLASS(NullStruct){};

MOCK_CLASS(Uri){
	MOCK_METHOD_EXT(isEmpty, 0, bool(), isEmpty);
	bool operator==(Uri const &) const {
		return true;
	}
};

MOCK_CLASS(DataModel){
public:
	typedef Uri source_uri_t;

	MOCK_METHOD_EXT(loadDataFrame, 0, void(), loadDataFrame);
	MOCK_METHOD_EXT(estimateDateColumn, 0, void(), estimateDateColumn);
	MOCK_METHOD_EXT(setSourceUri, 1, void(Uri const &), setSourceUri);
	MOCK_METHOD_EXT(getSourceUri, 0, const Uri &(), getSourceUri);
	MOCK_METHOD_EXT(setCurrentMissingDataStrategy, 1, void(int), setCurrentMissingDataStrategy);
	MOCK_METHOD_EXT(updateDataFrameWithMissingDataStrategy, 0, void(), updateDataFrameWithMissingDataStrategy);
	MOCK_METHOD_EXT(assignBehaviorIndicatorColumn, 1, void(int &), assignBehaviorIndicatorColumn);
	MOCK_METHOD_EXT(removeBehaviorIndicatorColumn, 1, void(int &), removeBehaviorIndicatorColumn);
	MOCK_METHOD_EXT(assignWaterLevelColumn, 1, void(int &), assignWaterLevelColumn);
	MOCK_METHOD_EXT(removeWaterLevelColumn, 1, void(int &), removeWaterLevelColumn);
	MOCK_METHOD_EXT(assignTemperatureColumn, 1, void(int &), assignTemperatureColumn);
	MOCK_METHOD_EXT(removeTemperatureColumn, 1, void(int &), removeTemperatureColumn);
};

MOCK_CLASS(Analysis){
	MOCK_METHOD_EXT(setDataModel, 1, void(DataModel *), setDataModel);
};


MOCK_CLASS(DataModelFactory) {
public:
	template<typename T>
	T *create() {
		return createDelegate();
	}
	MOCK_METHOD_EXT(createDelegate, 0, DataModel*(), createDelegate)
};

MOCK_CLASS(Wizard) {
	//MOCK_METHOD_EXT(start, 0, void(), start);
	void start(){}
	MOCK_METHOD_EXT(accepted, 0, bool(), accepted);
	MOCK_METHOD_EXT(editCommonParameters, 0, void(), editCommonParameters);
	MOCK_METHOD_EXT(editTimeSeries, 0, void(), editTimeSeries);
	MOCK_METHOD_EXT(editAssignment, 0, void(), editAssignment);
};

MOCK_CLASS(OverviewUi){
	MOCK_METHOD_EXT(show, 0, void(), show);
};

MOCK_CLASS(WorkingAreaContainerUi) {
	MOCK_METHOD_EXT(setWorkingArea, 1, void(OverviewUi *), setWorkingArea);
};

MOCK_CLASS(UriDialog) {
	MOCK_METHOD_EXT(getOpenUrl, 2, const Uri(QString const &, QString const &), getOpenUrl);
};

MOCK_CLASS(UiManager){
public:
	typedef Wizard wizard_ui_type;
	typedef OverviewUi overview_ui_type;
	typedef UriDialog uri_dialog_ui_type;

	MOCK_METHOD_EXT(getOverviewUiFor, 1, OverviewUi*(DataModel*), getOverviewUiFor);
	MOCK_METHOD_EXT(getWizardFor, 1, Wizard*(DataModel*), getWizardFor);
	MOCK_METHOD_EXT(getWorkingAreaContainerUi, 0, WorkingAreaContainerUi*(), getWorkingAreaContainerUi);
	MOCK_METHOD_EXT(getUriDialogUiFor, 1, UriDialog*(DataModel *), getUriDialogUiFor);

	void destroyUriDialogUiFor(DataModel *dataModel) {
		delegateDestroyUriDialogUiFor(dataModel);
	}

	void destroyWizardFor(DataModel *dataModel) {
		delegateDestroyWizardFor(dataModel);
	}

	MOCK_METHOD_EXT(delegateDestroyWizardFor, 1, void(DataModel*), delegateDestroyWizardFor);
	MOCK_METHOD_EXT(delegateDestroyUriDialogUiFor, 1, void(DataModel*), delegateDestroyUriDialogUiFor);
};

MOCK_CLASS(SubscriptionManager) {
public:
	template<typename Wizard, typename Controller>
	void subscribeWizard(Wizard *wizard, Controller *controller) {
		subscribeWizardDelegate(wizard, controller);
	}
	MOCK_METHOD_EXT(subscribeWizardDelegate, 2, void(Wizard *, void*), subscribeWizardDelegate);

	template<typename OverviewUi, typename Controller>
	void subscribeOverviewUi(OverviewUi *overviewUi, Controller *controller) {
		delegateSubscribeOverviewUi(overviewUi, controller);
	}
	MOCK_METHOD_EXT(delegateSubscribeOverviewUi, 2, void(OverviewUi*, void*), delegateSubscribeOverviewUi);
	MOCK_METHOD_EXT(subscribeOverviewUiDelegates, 2, void(OverviewUi*, void*), subscribeOverviewUiDelegates);
};

typedef Analysis Analysis_t;
typedef DataModel DataModel_t;
typedef Wizard Wizard_t;
typedef OverviewUi Overview_Ui_t;
typedef WorkingAreaContainerUi Working_Area_Container_Ui_t;
typedef UriDialog Uri_Dialog_t;
typedef UiManager DataModel_Ui_Manager_t;
typedef DataModelFactory DataModel_Factory_t;
typedef SubscriptionManager SubscriptionManager_t;
typedef controller::DataModelController<DataModel_t, DataModel_Ui_Manager_t, SubscriptionManager, DontDeleteOnDeleteChildPolicy> DataModel_Controller_t;

namespace test {
using namespace helper;
class ApplicationContext {
public:
	ApplicationContext():
		GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(DataModel_Controller_t, boost::bind(&ApplicationContext::dataModelControllerFactory, this)),
		GAC_DEFAULT_CTOR_INIT(DataModel_Factory_t),
		GAC_DEFAULT_CTOR_INIT(DataModel_t),
		GAC_DEFAULT_CTOR_INIT(DataModel_Ui_Manager_t),
		GAC_DEFAULT_CTOR_INIT(Analysis_t),
		GAC_DEFAULT_CTOR_INIT(Wizard_t),
		GAC_DEFAULT_CTOR_INIT(Overview_Ui_t),
		GAC_DEFAULT_CTOR_INIT(Uri_Dialog_t),
		GAC_DEFAULT_CTOR_INIT(Working_Area_Container_Ui_t),
		GAC_DEFAULT_CTOR_INIT(SubscriptionManager_t)
	{}

private:
	DataModel_Controller_t *dataModelControllerFactory() {
		DataModel_Controller_t *dataModelController;

		dataModelController = new DataModel_Controller_t(FETCH_FROM_CTX(DataModel_Ui_Manager_t), FETCH_FROM_CTX(SubscriptionManager_t));

		return dataModelController;
	}

private:
	GAC_SCOPE(DataModel_Controller_t)
	GAC_SCOPE(DataModel_Factory_t)
	GAC_SCOPE(DataModel_t)
	GAC_SCOPE(DataModel_Ui_Manager_t)
	GAC_SCOPE(Analysis_t)
	GAC_SCOPE(Wizard_t)
	GAC_SCOPE(Overview_Ui_t)
	GAC_SCOPE(Uri_Dialog_t)
	GAC_SCOPE(Working_Area_Container_Ui_t)
	GAC_SCOPE(SubscriptionManager_t)
};
} /* namespace test */


void prepareWizardExpectations() {
	MOCK_EXPECT(FETCH_FROM_CTX(DataModel_Ui_Manager_t)->getWizardFor).once().returns(FETCH_FROM_CTX(Wizard_t));
	MOCK_EXPECT(FETCH_FROM_CTX(DataModel_Ui_Manager_t)->delegateDestroyWizardFor).once();
	MOCK_EXPECT(FETCH_FROM_CTX(SubscriptionManager_t)->subscribeWizardDelegate).once().with(FETCH_FROM_CTX(Wizard_t), FETCH_FROM_CTX(DataModel_Controller_t));
}

BOOST_FIXTURE_TEST_CASE(CreateForAnalysis, test::ApplicationContext)
{
	MOCK_EXPECT(FETCH_FROM_CTX(DataModel_Factory_t)->createDelegate).once().returns(FETCH_FROM_CTX(DataModel_t));
	prepareWizardExpectations();
	MOCK_EXPECT(FETCH_FROM_CTX(SubscriptionManager_t)->delegateSubscribeOverviewUi).once().with(FETCH_FROM_CTX(Overview_Ui_t), FETCH_FROM_CTX(DataModel_Controller_t));
	MOCK_EXPECT(FETCH_FROM_CTX(SubscriptionManager_t)->subscribeOverviewUiDelegates).once().with(FETCH_FROM_CTX(Overview_Ui_t), FETCH_FROM_CTX(Analysis_t));
	MOCK_EXPECT(FETCH_FROM_CTX(Wizard_t)->accepted).once().returns(true);
	MOCK_EXPECT(FETCH_FROM_CTX(Analysis_t)->setDataModel).once().with(FETCH_FROM_CTX(DataModel_t));
	MOCK_EXPECT(FETCH_FROM_CTX(DataModel_Ui_Manager_t)->getOverviewUiFor).at_least(1).with(FETCH_FROM_CTX(DataModel_t)).returns(FETCH_FROM_CTX(Overview_Ui_t));
	MOCK_EXPECT(FETCH_FROM_CTX(Overview_Ui_t)->show).once();
	MOCK_EXPECT(FETCH_FROM_CTX(DataModel_Ui_Manager_t)->getWorkingAreaContainerUi).once().returns(FETCH_FROM_CTX(Working_Area_Container_Ui_t));
	MOCK_EXPECT(FETCH_FROM_CTX(Working_Area_Container_Ui_t)->setWorkingArea).once().with(FETCH_FROM_CTX(Overview_Ui_t));

	DataModel_Controller_t *dataModelController = FETCH_FROM_CTX(DataModel_Controller_t);
	dataModelController->createFor(FETCH_FROM_CTX(Analysis_t), FETCH_FROM_CTX(DataModel_Factory_t));
}

BOOST_FIXTURE_TEST_CASE(Show, test::ApplicationContext)
{
	MOCK_EXPECT(FETCH_FROM_CTX(DataModel_Ui_Manager_t)->getOverviewUiFor).once().with(FETCH_FROM_CTX(DataModel_t)).returns(FETCH_FROM_CTX(Overview_Ui_t));
	MOCK_EXPECT(FETCH_FROM_CTX(Overview_Ui_t)->show).once();
	MOCK_EXPECT(FETCH_FROM_CTX(DataModel_Ui_Manager_t)->getWorkingAreaContainerUi).once().returns(FETCH_FROM_CTX(Working_Area_Container_Ui_t));
	MOCK_EXPECT(FETCH_FROM_CTX(Working_Area_Container_Ui_t)->setWorkingArea).once().with(FETCH_FROM_CTX(Overview_Ui_t));

	DataModel_Controller_t *dataModelController = FETCH_FROM_CTX(DataModel_Controller_t);
	dataModelController->show(FETCH_FROM_CTX(DataModel_t));
}

BOOST_FIXTURE_TEST_CASE(RequestSourceUriWithEmptyUriDoesntLoadTheDataModelReturned, test::ApplicationContext)
{
	MOCK_EXPECT(FETCH_FROM_CTX(DataModel_Ui_Manager_t)->getUriDialogUiFor).once().with(FETCH_FROM_CTX(DataModel_t)).returns(FETCH_FROM_CTX(Uri_Dialog_t));
	MOCK_EXPECT(FETCH_FROM_CTX(DataModel_Ui_Manager_t)->delegateDestroyUriDialogUiFor).once().with(FETCH_FROM_CTX(DataModel_t));

	Uri uri;
	MOCK_EXPECT(uri.isEmpty).at_least(1).returns(true);
	MOCK_EXPECT(FETCH_FROM_CTX(Uri_Dialog_t)->getOpenUrl).once().with(QString("Data model"), QString("csv")).returns(uri);

	DataModel_Controller_t *dataModelController = FETCH_FROM_CTX(DataModel_Controller_t);
	dataModelController->requestSourceUri(FETCH_FROM_CTX(DataModel_t));
}


BOOST_FIXTURE_TEST_CASE(RequestSourceUriWithANonEmptyUriLoadsTheDataFrameAndEstimatesTheDateColumn, test::ApplicationContext)
{
	MOCK_EXPECT(FETCH_FROM_CTX(DataModel_Ui_Manager_t)->getUriDialogUiFor).once().with(FETCH_FROM_CTX(DataModel_t)).returns(FETCH_FROM_CTX(Uri_Dialog_t));
	MOCK_EXPECT(FETCH_FROM_CTX(DataModel_Ui_Manager_t)->delegateDestroyUriDialogUiFor).once().with(FETCH_FROM_CTX(DataModel_t));

	Uri uri;
	MOCK_EXPECT(uri.isEmpty).at_least(1).returns(false);
	MOCK_EXPECT(FETCH_FROM_CTX(Uri_Dialog_t)->getOpenUrl).once().with(QString("Data model"), QString("csv")).returns(boost::ref(uri));

	MOCK_EXPECT(FETCH_FROM_CTX(DataModel_t)->setSourceUri).once().with(uri);
	MOCK_EXPECT(FETCH_FROM_CTX(DataModel_t)->getSourceUri).once().returns(uri);
	MOCK_EXPECT(FETCH_FROM_CTX(DataModel_t)->loadDataFrame).once();
	MOCK_EXPECT(FETCH_FROM_CTX(DataModel_t)->estimateDateColumn).once();

	FETCH_FROM_CTX(DataModel_Controller_t)->requestSourceUri(FETCH_FROM_CTX(DataModel_t));
}

BOOST_FIXTURE_TEST_CASE(SettingMissingDataStrategySetsTheStrategyAndUpdatesTheDataFrame, test::ApplicationContext) {
	MOCK_EXPECT(FETCH_FROM_CTX(DataModel_t)->setCurrentMissingDataStrategy).once().with(1);
	MOCK_EXPECT(FETCH_FROM_CTX(DataModel_t)->updateDataFrameWithMissingDataStrategy).once();

	FETCH_FROM_CTX(DataModel_Controller_t)->missingDataStrategyChange(FETCH_FROM_CTX(DataModel_t), 1);
}

BOOST_FIXTURE_TEST_CASE(TestEditCommonParameters, test::ApplicationContext)
{
	prepareWizardExpectations();

	MOCK_EXPECT(FETCH_FROM_CTX(Wizard_t)->editCommonParameters).once();

	FETCH_FROM_CTX(DataModel_Controller_t)->editCommonParameters(FETCH_FROM_CTX(DataModel_t));
}

BOOST_FIXTURE_TEST_CASE(TestEditTimeSeries, test::ApplicationContext)
{
	prepareWizardExpectations();

	MOCK_EXPECT(FETCH_FROM_CTX(Wizard_t)->editTimeSeries).once();

	FETCH_FROM_CTX(DataModel_Controller_t)->editTimeSeries(FETCH_FROM_CTX(DataModel_t));
}

BOOST_FIXTURE_TEST_CASE(TestEditAssignment, test::ApplicationContext)
{
	prepareWizardExpectations();

	MOCK_EXPECT(FETCH_FROM_CTX(Wizard_t)->editAssignment).once();

	FETCH_FROM_CTX(DataModel_Controller_t)->editAssignment(FETCH_FROM_CTX(DataModel_t));
}


BOOST_FIXTURE_TEST_CASE(TestBehaviorIndicatorAssignmentChangedDelegatesToAssignOnDataModel, test::ApplicationContext) {
	int index = 1;
	MOCK_EXPECT(FETCH_FROM_CTX(DataModel_t)->assignBehaviorIndicatorColumn).once().with(index);

	FETCH_FROM_CTX(DataModel_Controller_t)->behaviorIndicatorAssignmentChanged(FETCH_FROM_CTX(DataModel_t), index, true);
}

BOOST_FIXTURE_TEST_CASE(TestBehaviorIndicatorAssignmentChangedDelegatesToRemoveOnDataModel, test::ApplicationContext) {
	int index = 1;
	MOCK_EXPECT(FETCH_FROM_CTX(DataModel_t)->removeBehaviorIndicatorColumn).once().with(index);

	FETCH_FROM_CTX(DataModel_Controller_t)->behaviorIndicatorAssignmentChanged(FETCH_FROM_CTX(DataModel_t), index, false);
}


BOOST_FIXTURE_TEST_CASE(TestWaterLevelAssignmentChangedDelegatesToAssignOnDataModel, test::ApplicationContext) {
	int index = 1;
	MOCK_EXPECT(FETCH_FROM_CTX(DataModel_t)->assignWaterLevelColumn).once().with(index);

	FETCH_FROM_CTX(DataModel_Controller_t)->waterLevelAssignmentChanged(FETCH_FROM_CTX(DataModel_t), index, true);
}

BOOST_FIXTURE_TEST_CASE(TestWaterLevelAssignmentChangedDelegatesToRemoveOnDataModel, test::ApplicationContext) {
	int index = 1;
	MOCK_EXPECT(FETCH_FROM_CTX(DataModel_t)->removeWaterLevelColumn).once().with(index);

	FETCH_FROM_CTX(DataModel_Controller_t)->waterLevelAssignmentChanged(FETCH_FROM_CTX(DataModel_t), index, false);
}

BOOST_FIXTURE_TEST_CASE(TestTemperatureAssignmentChangedDelegatesToAssignOnDataModel, test::ApplicationContext) {
	int index = 1;
	MOCK_EXPECT(FETCH_FROM_CTX(DataModel_t)->assignTemperatureColumn).once().with(index);

	FETCH_FROM_CTX(DataModel_Controller_t)->temperatureAssignmentChanged(FETCH_FROM_CTX(DataModel_t), index, true);
}

BOOST_FIXTURE_TEST_CASE(TestTemperatureAssignmentChangedDelegatesToRemoveOnDataModel, test::ApplicationContext) {
	int index = 1;
	MOCK_EXPECT(FETCH_FROM_CTX(DataModel_t)->removeTemperatureColumn).once().with(index);

	FETCH_FROM_CTX(DataModel_Controller_t)->temperatureAssignmentChanged(FETCH_FROM_CTX(DataModel_t), index, false);
}
