/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionModelControllerTests.cpp
 *
 *  Created on: Feb 21, 2014
 *      Author: gerberma
 */

#define BOOST_TEST_MODULE RegressionModelControllerTests
#include <boost/test/unit_test.hpp>
#include <turtle/mock.hpp>
#include <turtle/constraints.hpp>
#include <boost/any.hpp>
#include <boost/utility.hpp>
#include <boost/bind.hpp>
#include <boost/ref.hpp>
#include <string>
#include <set>
#include "controller/RegressionModelController.hpp"
#include "app/GenericApplicationContext.hpp"
#include "tests/helper/GenericApplicationContextTestHelper.hpp"
#include "memory/OwnerhshipPolicies.hpp"

using namespace app::context;

MOCK_CLASS(BehaviorIndicatorColumn) {
	MOCK_METHOD_EXT(getIndex, 0, int(), getIndex);
};

MOCK_CLASS(DataFrame) {
};

MOCK_CLASS(DataModel) {
	MOCK_METHOD_EXT(getBehaviorIndicatorColumns, 0, const std::set<BehaviorIndicatorColumn *>(), getBehaviorIndicatorColumns);
};

MOCK_CLASS(RegressorFormulaBuilder) {
	MOCK_METHOD_EXT(requiresInfluenceFactor, 0, bool(), requiresInfluenceFactor);
};

MOCK_CLASS(Regressor) {
	MOCK_METHOD_EXT(setInfluenceFactor, 1, void(int), setInfluenceFactor);
};

MOCK_CLASS(RegressionModelDefinition) {
	typedef Regressor Regressor_t;
	typedef RegressorFormulaBuilder RegressorFormulaBuilder_t;

	MOCK_METHOD_EXT(findRegressorFormulaBuilderByIdentifier, 1, RegressorFormulaBuilder *(QString), findRegressorFormulaBuilderByIdentifier);
	MOCK_METHOD_EXT(addRegressor, 1, void(Regressor_t *), addRegressor);
	MOCK_METHOD_EXT(getDataFrame, 0, DataFrame *(), getDataFrame);
};

MOCK_CLASS(RegressionFormula) {
	MOCK_METHOD_EXT(setWaterLevelFormula, 1, void(QString), setWaterLevelFormula);
};

MOCK_CLASS(RegressionModel) {
	typedef BehaviorIndicatorColumn Behavior_Indicator_Column_t;
	MOCK_METHOD_EXT(setupBestFittingRegressionFormula, 0, void(), setupBestFittingRegressionFormula);
	MOCK_METHOD_EXT(setName, 1, void(QString), setName)
	MOCK_METHOD_EXT(setDescription, 1, void(QString), setDescription)
	MOCK_METHOD_EXT(setFromDate, 1, void(int), setFromDate);
	MOCK_METHOD_EXT(setToDate, 1, void(int), setToDate);
	MOCK_METHOD_EXT(addRegressor, 2, void(QString, int), addRegressor);
	MOCK_METHOD_EXT(updateRegressor, 2, void(QString, int), updateRegressor);
	MOCK_METHOD_EXT(setBehaviorIndicatorColumn, 1, void(int), setBehaviorIndicatorColumn);
	MOCK_METHOD_EXT(getBehaviorIndicatorColumn, 0, BehaviorIndicatorColumn *(), getBehaviorIndicatorColumn);
	MOCK_METHOD_EXT(getRegressionFormula, 0, RegressionFormula *(), getRegressionFormula);
	MOCK_METHOD_EXT(setWaterLevelFormula, 1, void(QString), setWaterLevelFormula);
	MOCK_METHOD_EXT(getDataModel, 0, DataModel *(), getDataModel);
	MOCK_METHOD_EXT(calculateRegressionFit, 0, void(), calculateRegressionFit);
};

MOCK_CLASS(Analysis) {
	typedef DataModel DataModel_t;

	MOCK_METHOD_EXT(addRegressionModel, 1, void(RegressionModel *),addRegressionModel);

	MOCK_METHOD_EXT(getDataModel, 0, DataModel *(), getDataModel);
};

MOCK_CLASS(Factory) {

	RegressionModel *createRegressionModel(DataModel *dataModel) {
		return delegateCreate(dataModel);
	}
	MOCK_METHOD_EXT(delegateCreate, 1, RegressionModel*(DataModel *), delegateCreate);

	template<typename T, typename U, typename V>
	T *create(U *u, V *v) {
		return delegateCreateRegressor(u, v);
	}
	MOCK_METHOD_EXT(delegateCreateRegressor, 2, Regressor*(RegressorFormulaBuilder *, DataFrame *), delegateCreateRegressor);
};

MOCK_CLASS(Wizard) {
	/*
	MOCK_METHOD_EXT(start, 0, void(), start);
	MOCK_METHOD_EXT(accepted, 0, bool(), accepted);
	*/
};

MOCK_CLASS(OverviewUi) {
	//MOCK_METHOD_EXT(show, 0, void(), show);
};

MOCK_CLASS(WorkingAreaContainerUi) {
	//MOCK_METHOD_EXT(setWorkingArea, 1, void(OverviewUi *), setWorkingArea);
};

MOCK_CLASS(CategoryWidget) {
};

MOCK_CLASS(InfluenceFactorSelectionWidget) {
	MOCK_METHOD_EXT(exec, 0, void(), exec);
};

MOCK_CLASS(SubscriptionManager) {
	MOCK_METHOD_EXT(subscribeDelegates, 3, void(CategoryWidget *, Analysis *, RegressionModel *), subscribeDelegates);
};



MOCK_CLASS(RegressionModelUiManager) {
	typedef InfluenceFactorSelectionWidget influence_factor_selection_ui_t;
	MOCK_METHOD_EXT(getPhysicsCategoryWidgetFor, 2, CategoryWidget *(RegressionModel *, SubscriptionManager *), getPhysicsCategoryWidgetFor);
	MOCK_METHOD_EXT(getStatisticsCategoryWidgetFor, 2, CategoryWidget *(RegressionModel *, SubscriptionManager *), getStatisticsCategoryWidgetFor);
	MOCK_METHOD_EXT(getLeftSidePhysicsCategoryWidgetFor, 2, CategoryWidget *(RegressionModel *, SubscriptionManager *), getLeftSidePhysicsCategoryWidgetFor);
	MOCK_METHOD_EXT(getLeftSideStatisticsCategoryWidgetFor, 2, CategoryWidget *(RegressionModel *, SubscriptionManager *), getLeftSideStatisticsCategoryWidgetFor);
	MOCK_METHOD_EXT(getInfluenceFactorSelectionUiFor, 2, InfluenceFactorSelectionWidget *(Regressor *, SubscriptionManager *), getInfluenceFactorSelectionUiFor);

	void destroyInfluenceFactorSelectionUiFor(Regressor *r) {delegateDestroyInfluenceFactorSelectionUiFor(r);}
	MOCK_METHOD_EXT(delegateDestroyInfluenceFactorSelectionUiFor, 1, void(Regressor *), delegateDestroyInfluenceFactorSelectionUiFor);
};



MOCK_CLASS(GenericController) {
	void createFor(boost::any a, boost::any b, SubscriptionManager *subscriptionManager) {
		delegateCreateFor(a, b, subscriptionManager);
	}
	MOCK_METHOD_EXT(delegateCreateFor, 3, void(boost::any, boost::any, SubscriptionManager *subscriptionManager), delegateCreateFor);

	MOCK_METHOD_EXT(show, 2, void(RegressionModel *, SubscriptionManager *), show);
};


typedef controller::RegressionModelController<RegressionModel, RegressionModelUiManager, memory::CustomOwnershipPolicy<RegressionModel, memory::IgnoreOnDeleteChildPolicy>, GenericController > RegressionModel_Controller_t;

namespace test {
using namespace helper;
class ApplicationContext {
public:
	ApplicationContext():
		GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(RegressionModel_Controller_t, boost::bind(&ApplicationContext::regressionModelControllerFactory, this))
		,GAC_DEFAULT_CTOR_INIT(Wizard)
	{;}

private:
	RegressionModel_Controller_t *regressionModelControllerFactory() {
		return new RegressionModel_Controller_t(&genericController, &uiManager);
	}

private:
	GAC_SCOPE(RegressionModel_Controller_t)
	GAC_SCOPE(Wizard)
public:
	DataFrame dataFrame;
	DataModel dataModel;
	Analysis analysis;
	Regressor regressor;
	RegressionFormula regressionFormula;
	RegressorFormulaBuilder regressorFormulaBuilder;
	RegressionModelDefinition regressionModelDefinition;
	RegressionModel regressionModel;
	RegressionModelUiManager uiManager;
	Factory factory;
	OverviewUi overviewUi;
	WorkingAreaContainerUi workingAreaContainerUi;
	InfluenceFactorSelectionWidget influenceFactorSelectionUi;
	SubscriptionManager subscriptionManager;
	GenericController genericController;
	BehaviorIndicatorColumn behaviorIndicatorColumn;
	CategoryWidget physicsCategoryWidget;
	CategoryWidget statisticsCategoryWidget;
	CategoryWidget leftSidePhysicsCategoryWidget;
	CategoryWidget leftSideStatisticsCategoryWidget;
	std::set<BehaviorIndicatorColumn *> behaviorIndicatorColumns;
};

} /* namespace test */


BOOST_FIXTURE_TEST_CASE(CreateRegressionModelDelegatesToGenericController, test::ApplicationContext) {
	using namespace boost;

	MOCK_EXPECT(analysis.getDataModel).at_least(1).returns(&dataModel);
	MOCK_EXPECT(genericController.delegateCreateFor).once().with(mock::any, mock::any, &subscriptionManager);

	//might look like this when using a custom factory with templated method called 'create' : bind((RegressionModel *(Factory::*)(typename Analysis::DataModel_t *))&Factory::template create<RegressionModel>, factory, analysis->getDataModel())
	FETCH_FROM_CTX(RegressionModel_Controller_t)->createFor<Analysis>(
			&analysis,
			bind(&Factory::createRegressionModel, ref(factory), _1),
			&subscriptionManager);
}


BOOST_FIXTURE_TEST_CASE(SetupBestFittingRegressionFormula, test::ApplicationContext) {
	using namespace boost;

	MOCK_EXPECT(regressionModel.setupBestFittingRegressionFormula).once();

	FETCH_FROM_CTX(RegressionModel_Controller_t)->setupBestFittingRegressionFormulaFor(&regressionModel);
}


BOOST_FIXTURE_TEST_CASE(SetName, test::ApplicationContext) {
	QString name("name");
	MOCK_EXPECT(regressionModel.setName).once().with(name);

	FETCH_FROM_CTX(RegressionModel_Controller_t)->setName(&regressionModel, name);
}

BOOST_FIXTURE_TEST_CASE(SetDescription, test::ApplicationContext) {
	QString description("description");
	MOCK_EXPECT(regressionModel.setDescription).once().with(description);

	FETCH_FROM_CTX(RegressionModel_Controller_t)->setDescription(&regressionModel, description);
}

BOOST_FIXTURE_TEST_CASE(FromDateChanged, test::ApplicationContext) {
	MOCK_EXPECT(regressionModel.setFromDate).once().with(2);

	FETCH_FROM_CTX(RegressionModel_Controller_t)->fromDateChanged(&regressionModel, 2);
}

BOOST_FIXTURE_TEST_CASE(ToDateChanged, test::ApplicationContext) {
	MOCK_EXPECT(regressionModel.setToDate).once().with(2);

	FETCH_FROM_CTX(RegressionModel_Controller_t)->toDateChanged(&regressionModel, 2);
}

BOOST_FIXTURE_TEST_CASE(BehaviorIndicatorChanged, test::ApplicationContext) {
	MOCK_EXPECT(regressionModel.setBehaviorIndicatorColumn).once().with(2);
	MOCK_EXPECT(regressionModel.setupBestFittingRegressionFormula).once();

	FETCH_FROM_CTX(RegressionModel_Controller_t)->behaviorIndicatorChanged(&regressionModel, 2);
}

BOOST_FIXTURE_TEST_CASE(WaterLevelRegressionFunctionChanged, test::ApplicationContext) {
	QString waterLevelFormula("MSVC1^1+MSVC2^2");

	MOCK_EXPECT(regressionModel.getRegressionFormula).at_least(1).returns(&regressionFormula);
	MOCK_EXPECT(regressionFormula.setWaterLevelFormula).once().with(waterLevelFormula);

	FETCH_FROM_CTX(RegressionModel_Controller_t)->setWaterLevelFormula(&regressionModel, waterLevelFormula);
}


BOOST_FIXTURE_TEST_CASE(TestPostAcceptedCreate, test::ApplicationContext) {
	MOCK_EXPECT(analysis.addRegressionModel).once().with(&regressionModel);
	MOCK_EXPECT(regressionModel.calculateRegressionFit).once();
	MOCK_EXPECT(uiManager.getPhysicsCategoryWidgetFor).at_least(1).with(&regressionModel, &subscriptionManager).returns(&physicsCategoryWidget);
	MOCK_EXPECT(uiManager.getStatisticsCategoryWidgetFor).at_least(1).with(&regressionModel, &subscriptionManager).returns(&statisticsCategoryWidget);



	MOCK_EXPECT(subscriptionManager.subscribeDelegates).once().with(&physicsCategoryWidget, &analysis, &regressionModel);
	MOCK_EXPECT(subscriptionManager.subscribeDelegates).once().with(&statisticsCategoryWidget, &analysis, &regressionModel);

	MOCK_EXPECT(uiManager.getLeftSidePhysicsCategoryWidgetFor).at_least(1).with(&regressionModel, &subscriptionManager).returns(&leftSidePhysicsCategoryWidget);
	MOCK_EXPECT(uiManager.getLeftSideStatisticsCategoryWidgetFor).at_least(1).with(&regressionModel, &subscriptionManager).returns(&leftSideStatisticsCategoryWidget);
	MOCK_EXPECT(subscriptionManager.subscribeDelegates).once().with(&leftSidePhysicsCategoryWidget, &analysis, &regressionModel);
	MOCK_EXPECT(subscriptionManager.subscribeDelegates).once().with(&leftSideStatisticsCategoryWidget, &analysis, &regressionModel);

	FETCH_FROM_CTX(RegressionModel_Controller_t)->postAcceptedCreate(&analysis, &regressionModel, &subscriptionManager);
}



BOOST_FIXTURE_TEST_CASE(ShowRegressionModelDelegatesToGenericController, test::ApplicationContext) {
	MOCK_EXPECT(genericController.show).once().with(&regressionModel, &subscriptionManager);

	FETCH_FROM_CTX(RegressionModel_Controller_t)->show(&regressionModel, &subscriptionManager);
}


//NEW
BOOST_FIXTURE_TEST_CASE(AddRegressorWithValidFormulaBuilderIdentifierAndNoInfluenceFactorRequired, test::ApplicationContext) {
	MOCK_EXPECT(regressionModelDefinition.findRegressorFormulaBuilderByIdentifier).once().returns(&regressorFormulaBuilder);
	MOCK_EXPECT(regressorFormulaBuilder.requiresInfluenceFactor).at_least(1).returns(false);
	MOCK_EXPECT(regressionModelDefinition.getDataFrame).once().returns(&dataFrame);
	MOCK_EXPECT(factory.delegateCreateRegressor).once().with(&regressorFormulaBuilder, &dataFrame).returns(&regressor);
	MOCK_EXPECT(regressionModelDefinition.addRegressor).once().with(&regressor);

	FETCH_FROM_CTX(RegressionModel_Controller_t)->addRegressor(&regressionModelDefinition, "id", &factory, &subscriptionManager);
}


BOOST_FIXTURE_TEST_CASE(AddRegressorWithValidFormulaBuilderAndOneInfluenceFactorRequired, test::ApplicationContext) {
	MOCK_EXPECT(regressionModelDefinition.findRegressorFormulaBuilderByIdentifier).once().returns(&regressorFormulaBuilder);
	MOCK_EXPECT(regressorFormulaBuilder.requiresInfluenceFactor).at_least(1).returns(true);
	MOCK_EXPECT(regressionModelDefinition.getDataFrame).once().returns(&dataFrame);
	MOCK_EXPECT(factory.delegateCreateRegressor).once().with(&regressorFormulaBuilder, &dataFrame).returns(&regressor);
	MOCK_EXPECT(uiManager.getInfluenceFactorSelectionUiFor).once().with(&regressor, &subscriptionManager).returns(&influenceFactorSelectionUi); //the widget provides methods such as getAssignableInfluenceFactors which the regressor is able to resolve with the help of the formulaBuilder and the dataFrame it contains
	//require: influenceFactorSelectionUi->exec() //execute modal, and subscribed to the controller for setInfluenceFactor(Regressor *regressor, influenceFactorIndex)
	MOCK_EXPECT(influenceFactorSelectionUi.exec).once();
	MOCK_EXPECT(regressionModelDefinition.addRegressor).once().with(&regressor);
	MOCK_EXPECT(uiManager.delegateDestroyInfluenceFactorSelectionUiFor).once().with(&regressor);

	FETCH_FROM_CTX(RegressionModel_Controller_t)->addRegressor(&regressionModelDefinition, "id", &factory, &subscriptionManager);
}


BOOST_FIXTURE_TEST_CASE(SetRegressorInfluenceFactor, test::ApplicationContext) {
	MOCK_EXPECT(regressor.setInfluenceFactor).once().with(123);

	FETCH_FROM_CTX(RegressionModel_Controller_t)->setRegressorInfluenceFactor(&regressor, 123);
}
