/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelAnalysisOverviewSummaryDisplayWidgetTest.cpp
 *
 *  Created on: Jan 21, 2014
 *      Author: gerberma
 */

#include <QApplication>
#include <QMainWindow>
#include <QUrl>
#include <QString>
#include <set>
#include <iostream>
#include <boost/bind.hpp>
#include "model/SignalPolicies.hpp"
#include "communication/Signals.hpp"
#include "gui/DataModelAnalysisOverviewSummaryDisplayWidget.hpp"
#include "app/GenericApplicationContext.hpp"


namespace test {
class AnalysisToolDescription {
};

class Summary {
public:
	typedef AnalysisToolDescription tool_description_t;
public:
	Summary(int totalNumberOfDataPoints, int numberOfNotAvailableDataPoints)
		:totalNumberOfDataPoints(totalNumberOfDataPoints), numberOfNotAvailableDataPoints(numberOfNotAvailableDataPoints)
	{}

	int getTotalNumberOfDataPoints() {
		return totalNumberOfDataPoints;
	}

	int getNumberOfNotAvailableDataPoints() {
		return numberOfNotAvailableDataPoints;
	}

	int getNumberOfAvailableDataPoints() {
		return totalNumberOfDataPoints - numberOfNotAvailableDataPoints;
	}

	AnalysisToolDescription *getAnalysisToolDescription() {
		return &analysisToolDescripion;
	}

	COM_SUBSCRIPTION_FUNCTION(Loaded, communication::Signal<void()>);

private:

	int totalNumberOfDataPoints;
	int numberOfNotAvailableDataPoints;
	AnalysisToolDescription analysisToolDescripion;
};

} /* namespace test */


typedef test::Summary Summary_t;
typedef gui::DataModelAnalysisOverviewSummaryDisplayWidget<Summary_t> Summary_Widget_t;


using namespace app::context;
Summary_t *summaryFactory() {
	return new Summary_t(1024, 63);
}

Summary_Widget_t *summaryWidgetFactory() {
	return new Summary_Widget_t(FETCH_FROM_CTX(Summary_t));
}


int
main(int argc, char *argv[]) {
	QApplication app(argc, argv);
	QMainWindow mainWindow;

	REGISTER_FULL_CTX_WITH_CTX_SCOPE(Summary_t, summaryFactory, defaultCleanupStrategy<Summary_t>);
	REGISTER_FULL_CTX_WITH_CTX_SCOPE(Summary_Widget_t, summaryWidgetFactory, defaultCleanupStrategy<Summary_Widget_t>);


	mainWindow.setCentralWidget(FETCH_FROM_CTX(Summary_Widget_t));
	mainWindow.show();
	return app.exec();
}
