/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * GuiAnalysisToolApplicationContext.hpp
 *
 *  Created on: Jun 24, 2014
 *      Author: gerberma
 */

#ifndef GUIANALYSISTOOLAPPLICATIONCONTEXT_HPP_
#define GUIANALYSISTOOLAPPLICATIONCONTEXT_HPP_

#include <stdexcept>
#include <boost/bind.hpp>
#include <QApplication>
#include <QMainWindow>
#include "tests/helper/GenericApplicationContextTestHelper.hpp"

namespace test {
namespace helper {

using namespace app::context;

template<typename _Widget>
struct NullInitPolicy {
	void init(_Widget *widget) {
	}
};

template<typename _Widget>
struct CallInitPolicy {
	void init(_Widget *widget) {
		if(widget == NULL) {
			throw std::logic_error("no widget set ! ");
		}
		widget->init();
	}
};

template<typename T, typename _Widget, typename InitPolicy = NullInitPolicy<_Widget> >
class GuiAnalysisToolApplicationContext : public InitPolicy {
public:
	typedef _Widget widget_t;
public:
	GuiAnalysisToolApplicationContext():
		GAC_FULL_CTOR_INIT(widget_t, boost::bind(&GuiAnalysisToolApplicationContext::widgetFactory, this), nullCleanupStrategy<widget_t>)
	{;}

	virtual ~GuiAnalysisToolApplicationContext(){
	}

	widget_t *widgetFactory() {
		return new widget_t(&t);
	}

	int run(int argc, char *argv[]) {
		QApplication app(argc, argv);
		QMainWindow mainWindow;

		widgetFactory();

		InitPolicy::init(FETCH_FROM_CTX(widget_t));
		mainWindow.setCentralWidget(FETCH_FROM_CTX(widget_t));
		mainWindow.show();

		return app.exec();
	}

	T t;
	GAC_SCOPE(widget_t);
};

} // namespace helper
} // namespace test


#endif /* GUIANALYSISTOOLAPPLICATIONCONTEXT_HPP_ */
