/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * NavigationTreeWidgetTest.cpp
 *
 *  Created on: Jan 15, 2014
 *      Author: gerberma
 */

#include <QApplication>
#include <QMainWindow>
#include <vector>
#include <iostream>
#include <boost/bind.hpp>
#include "model/SignalPolicies.hpp"
#include "communication/Signals.hpp"
#include "gui/NavigationTreeWidget.hpp"
#include "app/GenericApplicationContext.hpp"
#include "model/SignalPolicies.hpp"


namespace test {

struct DataModel {
};

struct RegressionModel : public communication::BoostSignalPolicy {
	RegressionModel():name("a regression model"){;}

	void setName(QString name) {
		this->name = name;
		communication::BoostSignalPolicy::fire();
	}

	QString getName() {
		return name;
	}

	QString name;
};

class Analysis : public communication::BoostSignalPolicy {
public:
	typedef DataModel DataModel_t;
	typedef RegressionModel RegressionModel_t;
	typedef communication::Signal<void()> data_model_set_signal_t;
public:
	Analysis():dataModel(NULL), name("analysis"){;}
	bool isNewDataModelAllowed() {
		return true;
	}

	bool isNewRegressionModelAllowed() {
		return true;
	}

	void setName(QString name) {
		this->name = name;
		communication::BoostSignalPolicy::fire();
	}

	QString getName() {
		return name;
	}

	void setDataModel(DataModel *dataModel) {
		std::cout << __FUNCTION__ << std::endl;
		this->dataModel = dataModel;
		DataModelSetSignal();
	}

	DataModel *getDataModel() {
		return dataModel;
	}

	void addRegressionModel(RegressionModel * regressionModel) {
		regressionModels.push_back(regressionModel);
		RegressionModelAddedSignal(regressionModel);
	}

	std::vector<RegressionModel *> &getRegressionModels() {
		return regressionModels;
	}

	void remove(RegressionModel *regressionModel) {
		qDebug() << "Analysis: remove : " << regressionModel->getName();
		RemoveRegressionModelSignal(regressionModel);
	}

	COM_SUBSCRIPTION_FUNCTION(DataModelSet, data_model_set_signal_t);
	COM_SUBSCRIPTION_FUNCTION(RegressionModelAdded, communication::Signal<void(RegressionModel *)>);
	COM_SUBSCRIPTION_FUNCTION(RemoveRegressionModel, communication::Signal<void(RegressionModel *)>);

private:
	std::vector<RegressionModel *> regressionModels;
	DataModel *dataModel;
	QString name;
};

class Project : public communication::BoostSignalPolicy{
public:
	typedef communication::Signal<void(Analysis *)> analysis_added_signal_t;
	typedef Analysis Analysis_t;
public:
	Project()
	{
	}

	bool isNewAnalysisAllowed() {
		return true;
	}

	void addAnalysis(Analysis *analysis) {
		analyses.push_back(analysis);
		AnalysisAddedSignal(analysis);
	}

	std::vector<Analysis *> &getAnalyses() {
		return analyses;
	}

	QString getName() {
		return "the project";
	}

	COM_SUBSCRIPTION_FUNCTION(AnalysisAdded, analysis_added_signal_t);
	COM_SUBSCRIPTION_FUNCTION(RemoveAnalysis, communication::Signal<void(Analysis *)>);

private:
	std::vector<Analysis *> analyses;
};

} /* namespace test */

typedef test::Project Project_t;
typedef test::Analysis Analysis_t;
typedef test::DataModel DataModel_t;
typedef gui::NavigationTreeWidget<Project_t> Navigation_Tree_Widget_t;

using namespace app::context;

/*
test::Analysis *analysisFactory() {
	test::DataModel dataModel;
	return new test::Analysis;
}

test::Project *projectFactory() {
	//return new test::Project(FETCH_FROM_CTX(Analysis_t));
	return new test::Project();
}
*/

void createRegressionModel(test::Analysis *analysis) {
	std::cout << __FUNCTION__ << std::endl;
}

void regressionModelDoubleClicked(test::RegressionModel *regressionModel) {
	std::cout << __FUNCTION__ << std::endl;
}

void removeRegressionModel(test::RegressionModel *regressionModel, test::Analysis *analysis) {
	qDebug() << __FUNCTION__ << " free function : " << regressionModel->getName();
	analysis->remove(regressionModel);
}

int
main(int argc, char *argv[]) {
	QApplication app(argc, argv);
	QMainWindow mainWindow;

	REGISTER_FULL_DEFAULT_CTX_WITH_CTX_SCOPE(Navigation_Tree_Widget_t);
	REGISTER_FULL_DEFAULT_CTX_WITH_CTX_SCOPE(Project_t);
	REGISTER_FULL_DEFAULT_CTX_WITH_CTX_SCOPE(Analysis_t);
	REGISTER_FULL_DEFAULT_CTX_WITH_CTX_SCOPE(DataModel_t);

	Navigation_Tree_Widget_t *navigationTreeWidget = FETCH_FROM_CTX(Navigation_Tree_Widget_t);
	COM_SUBSCRIBE_FOR(navigationTreeWidget, CreateRegressionModel, boost::bind(createRegressionModel, _1));
	COM_SUBSCRIBE_FOR(navigationTreeWidget, RegressionModelDoubleClicked, boost::bind(regressionModelDoubleClicked, _1));
	COM_SUBSCRIBE_FOR(navigationTreeWidget, RemoveRegression, boost::bind(removeRegressionModel, _1, _2));

	mainWindow.setCentralWidget(navigationTreeWidget);
	mainWindow.show();

	Project_t *project = FETCH_FROM_CTX(Project_t);
	Analysis_t *analysis = FETCH_FROM_CTX(Analysis_t);
	DataModel_t *dataModel = FETCH_FROM_CTX(DataModel_t);
	test::RegressionModel regressionModel1;
	test::RegressionModel regressionModel2;
	regressionModel2.name = "regr2";


	project->addAnalysis(analysis);
	analysis->setDataModel(dataModel);
	analysis->addRegressionModel(&regressionModel1);
	analysis->addRegressionModel(&regressionModel2);

	navigationTreeWidget->show(project);

	regressionModel2.setName("new name");
	analysis->setName("new analysis");

	/*
	project->addAnalysis(analysis);
	analysis->setDataModel(dataModel);
	analysis->addRegressionModel(&regressionModel);
	*/


	return app.exec();
}
