/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitAnalysisACFDisplayWidgetTest.cpp
 *
 *  Created on: Apr 8, 2014
 *      Author: gerberma
 */


#include <QApplication>
#include <QMainWindow>
#include <QVector>
#include "gui/RegressionFitAnalysisACFDisplayWidget.hpp"
#include "communication/Signals.hpp"
#include "app/GenericApplicationContext.hpp"

using namespace app::context;

namespace test {
struct ACF {
	QVector<double> getAcfs() {
		QVector<double> acfs;
		acfs << 1.0 << 0.2 << 0.05;

		return acfs;
	}

	QVector<unsigned> getLags() {
		QVector<unsigned> lags;
		lags << 0 << 20 << 40;

		return lags;
	}

	double getLowerCi() {
		return -0.0376523;
	}

	double getUpperCi() {
		return getLowerCi() * -1;
	}
};
} //namespace test

typedef gui::RegressionFitAnalysisACFDisplayWidget<test::ACF> Widget_t;

Widget_t *factory() {
	return new Widget_t(FETCH_FROM_CTX(test::ACF));
}


int
main(int argc, char *argv[]) {
	QApplication app(argc, argv);
	QMainWindow mainWindow;

	REGISTER_FULL_CTX(Widget_t, factory, defaultCleanupStrategy<Widget_t>);
	REGISTER_FULL_DEFAULT_CTX(test::ACF);

	Widget_t *widget = FETCH_FROM_CTX(Widget_t);
	mainWindow.setCentralWidget(widget);

	mainWindow.show();

 return app.exec();
}
