/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitAnalysisPhysicalBehaviorDisplayWidgetTest.cpp
 *
 *  Created on: 21.10.2014
 *      Author: gerberma
 */

#include <vector>
#include <string>
#include <QString>
#include <cmath>
#include "communication/Signals.hpp"
#include "gui/RegressionFitAnalysisPhysicalBehaviorDisplayWidget.hpp"
#include "GuiAnalysisToolApplicationContext.hpp"

using namespace app::context;
using namespace test::helper;

namespace test {

struct AssignmentCategory {
	AssignmentCategory(QString name, QString identifier):name(name), identifier(identifier){;}

	QString getName(){
		return name;
	}

	QString getIdentifier() {
		return identifier;
	}

	QString name;
	QString identifier;
};

struct Item {
	typedef std::vector<double> ApplyedValuesContainer_t;
	typedef std::vector<std::string> DateResponseContainer_t;
	typedef std::vector<double> NumericResponseContainer_t;

	ApplyedValuesContainer_t &getApplyedValues(){
		return applyedValues;
	}

	QString getResponseType() {
		return responseType;
	}

	DateResponseContainer_t &getDateResponse() {
		return dateResponse;
	}

	NumericResponseContainer_t &getNumericResponse() {
		return numericResponse;
	}

	QString getName() {
		return name;
	}

	ApplyedValuesContainer_t applyedValues;
	QString responseType;
	DateResponseContainer_t dateResponse;
	NumericResponseContainer_t numericResponse;
	QString name;
};

struct PhysicalBehavior {
	typedef AssignmentCategory AssignmentCategory_t;
	typedef Item Item_t;
	typedef std::vector<double> ApplyedValuesContainer_t;
	typedef std::vector<std::string> DateResponseContainer_t;
	typedef std::vector<double> NumericResponseContainer_t;

	std::vector<Item *> &getItems() {
		return items;
	}


	std::vector<AssignmentCategory *> getAssignmentCategories() {
		return assignmentCategories;
	}

	AssignmentCategory *getAssignmentCategory() {
		return assignmentCategory;
	}

	std::vector<Item *> items;
	std::vector<AssignmentCategory *> assignmentCategories;
	AssignmentCategory *assignmentCategory;
};
} /* namespace test */


void setAssignmentCategory(test::PhysicalBehavior *, QString identifier) {
	qDebug() << __FUNCTION__ << " : " << identifier;
}


int
main(int argc, char *argv[]) {
	typedef GuiAnalysisToolApplicationContext<test::PhysicalBehavior, gui::RegressionFitAnalysisPhysicalBehaviorDisplayWidget<test::PhysicalBehavior> > AppCtx_t;
	AppCtx_t appCtx;

	test::Item itemOne;

	itemOne.applyedValues.push_back(10.3);
	itemOne.applyedValues.push_back(11.0);
	itemOne.applyedValues.push_back(12.4);
	itemOne.applyedValues.push_back(13.8);

	itemOne.numericResponse.push_back(1027);
	itemOne.numericResponse.push_back(1028);
	itemOne.numericResponse.push_back(1029);
	itemOne.numericResponse.push_back(1030);
	itemOne.numericResponse.push_back(1032);
	itemOne.numericResponse.push_back(1035);

	itemOne.dateResponse.push_back("1999-08-11");
	itemOne.dateResponse.push_back("1999-08-12");
	itemOne.dateResponse.push_back("1999-08-13");
	itemOne.dateResponse.push_back("1999-08-14");
	itemOne.dateResponse.push_back("1999-08-15");
	itemOne.dateResponse.push_back("1999-08-16");

	itemOne.responseType = "numeric";
	itemOne.name = "W_LEVEL";

	test::AssignmentCategory acWlevel("Water level", "water_level");
	test::AssignmentCategory acTemperature("Temperature", "temperature");

	appCtx.t.items.push_back(&itemOne);
	appCtx.t.assignmentCategory = &acWlevel;
	appCtx.t.assignmentCategories.push_back(&acWlevel);
	appCtx.t.assignmentCategories.push_back(&acTemperature);

	using namespace boost;
	/*
	typename AppCtx_t::widget_t *widget = FETCH_FROM_CTX(typename AppCtx_t::widget_t);
	COM_SUBSCRIBE_FOR(widget, SetAssignmentCategory, bind(setAssignmentCategory, _1, _2));
	*/

	return appCtx.run(argc, argv);
}
