/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitAnalysisPhysicalSummaryDisplayWidgetTest.cpp
 *
 *  Created on: Jun 24, 2014
 *      Author: gerberma
 */

#include <vector>
#include <string>
#include "gui/RegressionFitAnalysisPhysicalSummaryDisplayWidget.hpp"
#include "GuiAnalysisToolApplicationContext.hpp"


namespace test {

struct PhysicalSummary {
	typedef std::vector<double> BehaviorIndicatorValueContainer_t;
	typedef std::vector<double> PredictionContainer_t;
	typedef std::vector<double> OutOfLimitContainer_t;
	typedef std::vector<double> ExtrapolationsContainer_t;
	typedef std::vector<std::string> DateContainer_t;
	typedef std::vector<double> ConfidenceIntervalContainer_t;

	BehaviorIndicatorValueContainer_t getBehaviorIndicatorValues() {
		return behaviorIndicators;
	}

	DateContainer_t getDates() {
		return dates;
	}

	DateContainer_t getPredictionDates() {
		return dates;
	}

	PredictionContainer_t getPredictions() {
		return predictions;
	}

	ConfidenceIntervalContainer_t getUpperCi() {
		return upperCi;
	}

	ConfidenceIntervalContainer_t getLowerCi() {
		return lowerCi;
	}

	OutOfLimitContainer_t getOutOfLimit() {
		return outOfLimit;
	}

	DateContainer_t &getOutOfLimitDates() {
		return outOfLimitDates;
	}

	ExtrapolationsContainer_t getExtrapolations() {
		return extrapolations;
	}

	DateContainer_t getExtrapolationDates() {
		return extrapolationDates;
	}

	std::string getRegressionStartDate() {
		return startDate;
	}

	std::string getRegressionEndDate() {
		return endDate;
	}

	QString getBehaviorIndicatorName() {
		return behaviorIndicatorName;
	}

	BehaviorIndicatorValueContainer_t behaviorIndicators;
	QString behaviorIndicatorName;
	PredictionContainer_t predictions;
	ConfidenceIntervalContainer_t upperCi;
	ConfidenceIntervalContainer_t lowerCi;
	OutOfLimitContainer_t outOfLimit;
	ExtrapolationsContainer_t extrapolations;
	DateContainer_t dates;
	DateContainer_t outOfLimitDates;
	DateContainer_t extrapolationDates;
	std::string startDate;
	std::string endDate;
};

} // namespace test

int
main(int argc, char *argv[]) {
	//test::helper::GuiAnalysisToolApplicationContext<test::PhysicalSummary, gui::RegressionFitAnalysisPhysicalSummaryDisplayWidget<test::PhysicalSummary> > appCtx;
	QApplication app(argc, argv);
	boost::shared_ptr<test::PhysicalSummary> summary(new test::PhysicalSummary);
	test::PhysicalSummary::BehaviorIndicatorValueContainer_t behaviorIndicatorValues;
	test::PhysicalSummary::BehaviorIndicatorValueContainer_t predictions;
	std::vector<std::string> dates;

	behaviorIndicatorValues.push_back(4);
	behaviorIndicatorValues.push_back(4.5);
	behaviorIndicatorValues.push_back(4.2);
	behaviorIndicatorValues.push_back(3.9);

	predictions.push_back(3.8);
	predictions.push_back(4.6);
	predictions.push_back(4.2);
	predictions.push_back(3.95);

	test::PhysicalSummary::ConfidenceIntervalContainer_t lowerCi;
	test::PhysicalSummary::ConfidenceIntervalContainer_t upperCi;

	lowerCi.push_back(3.0);
	upperCi.push_back(4.1);
	lowerCi.push_back(4.0);
	upperCi.push_back(5.0);
	lowerCi.push_back(3.8);
	upperCi.push_back(4.5);
	lowerCi.push_back(3.5);
	upperCi.push_back(4.3);

	test::PhysicalSummary::OutOfLimitContainer_t outOfLimit;
	test::PhysicalSummary::DateContainer_t outOfLimitDates;
	outOfLimitDates.push_back("1984-08-12");
	outOfLimitDates.push_back("1985-08-14");
	outOfLimit.push_back(4.5);
	outOfLimit.push_back(4.2);


	test::PhysicalSummary::ExtrapolationsContainer_t extrapolations;
	test::PhysicalSummary::DateContainer_t extrapolationDates;
	extrapolationDates.push_back("1983-08-11");
	extrapolationDates.push_back("1986-08-14");
	extrapolations.push_back(3.8);
	extrapolations.push_back(3.95);


	dates.push_back("1983-08-11");
	dates.push_back("1984-08-12");
	dates.push_back("1985-08-13");
	dates.push_back("1986-08-14");


	summary->startDate = "1984-08-12";
	summary->endDate = "1986-08-14";

	summary->behaviorIndicators = behaviorIndicatorValues;
	summary->behaviorIndicatorName = "PENDULUM";
	summary->predictions = predictions;
	summary->lowerCi = lowerCi;
	summary->upperCi = upperCi;
	summary->outOfLimit = outOfLimit;
	summary->extrapolations = extrapolations;
	summary->extrapolationDates = extrapolationDates;
	summary->outOfLimitDates = outOfLimitDates;
	summary->dates = dates;

	//return appCtx.run(argc, argv);

	gui::RegressionFitAnalysisPhysicalSummaryDisplayWidget<test::PhysicalSummary> *widget = new gui::RegressionFitAnalysisPhysicalSummaryDisplayWidget<test::PhysicalSummary>(summary);

	QMainWindow mainWindow;
	mainWindow.setCentralWidget(widget);
	mainWindow.show();

	return app.exec();
}
