/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitAnalysisRegressorOverviewEditWidgetTest.cpp
 *
 *  Created on: Mar 26, 2014
 *      Author: gerberma
 */


#include <QApplication>
#include <QMainWindow>
#include <boost/shared_ptr.hpp>
#include <math.h>
#include "gui/RegressionFitAnalysisRegressorOverviewEditWidget.hpp"
#include "communication/Signals.hpp"
#include "app/GenericApplicationContext.hpp"

using namespace app::context;

namespace test {

struct ModeHandler {
	COM_SUBSCRIPTION_FUNCTION(ModeChanged, communication::Signal<void()>);
};

struct Item {
	Item():coefficient(0.0f), tstat(0.0f), pvalue(0.0f), vif(0.0), stderror(0.0f), stdcoefficient(0.0f), isactive(false), iseditable(true), regressorIndex(0){;}

	QString getName() {
		return name;
	}

	int getRegressorIndex() {
		return regressorIndex;
	}

	double getCoefficient() {
		return coefficient;
	}

	double getTStat() {
		return tstat;
	}

	double getPValue() {
		return pvalue;
	}

	double getVIF() {
		return vif;
	}

	double getStdError() {
		return stderror;
	}

	double getStdCoefficient() {
		return stdcoefficient;
	}

	bool isActive() {
		return isactive;
	}

	bool isEditable() {
		return iseditable;
	}

	QString name;
	double coefficient;
	double tstat;
	double pvalue;
	double vif;
	double stderror;
	double stdcoefficient;
	bool isactive;
	bool iseditable;
	int regressorIndex;
};

struct RegressionFitAnalysisRegressorOverview{
	typedef Item RegressorItem_t;
	typedef Item InterceptItem_t;

	COM_SUBSCRIPTION_FUNCTION(Loaded, communication::Signal<void()>);

	RegressionFitAnalysisRegressorOverview() {
		regressorItemOne = boost::shared_ptr<Item>(new Item);
		regressorItemOne->regressorIndex = 0;
		regressorItemOne->name = "MSVC1";
		regressorItemOne->coefficient = 3.13;
		regressorItemOne->tstat = -7.13032e16;
		regressorItemOne->pvalue = 5.73855e-51;
		regressorItemOne->vif = 25;
		regressorItemOne->stderror = 4.56184e-17;
		regressorItemOne->stdcoefficient = 0.512;

		regressorItemTwo = boost::shared_ptr<Item>(new Item);
		regressorItemTwo->regressorIndex = 1;
		regressorItemTwo->name = "MSVC1^2";
		regressorItemTwo->coefficient = 6.4;
		regressorItemTwo->isactive = true;

		interceptItem = boost::shared_ptr<Item>(new Item);
		interceptItem->name = "Intercept";
		interceptItem->coefficient = 0.08;
		interceptItem->vif = nanl(NULL);
		interceptItem->stdcoefficient = nanl(NULL);

		modeHandler = boost::shared_ptr<ModeHandler>(new ModeHandler());
	}
	std::vector<boost::shared_ptr<Item> > getRegressorItems() {
		std::vector<boost::shared_ptr<Item> > items;
		items.push_back(regressorItemOne);
		items.push_back(regressorItemTwo);

		return items;
	}

	boost::shared_ptr<Item> getInterceptItem() {
		return interceptItem;
	}

	boost::shared_ptr<ModeHandler> getModeHandler() {
		return modeHandler;
	}

	bool isScientific() {
		return true;
	}

	boost::shared_ptr<Item> regressorItemOne;
	boost::shared_ptr<Item> regressorItemTwo;
	boost::shared_ptr<Item> interceptItem;
	boost::shared_ptr<ModeHandler> modeHandler;
};
}

typedef gui::RegressionFitAnalysisRegressorOverviewEditWidget<test::RegressionFitAnalysisRegressorOverview> Widget_t;

Widget_t *factory() {
	boost::shared_ptr<test::RegressionFitAnalysisRegressorOverview> overview(FETCH_FROM_CTX(test::RegressionFitAnalysisRegressorOverview));
	return new Widget_t(overview);
}

void activeStateChanged(boost::shared_ptr<test::RegressionFitAnalysisRegressorOverview> , int regressorIndex, bool state) {
	qDebug() << __FUNCTION__ << ": " << regressorIndex << " is " << state;
}

int
main(int argc, char *argv[]) {
	QApplication app(argc, argv);
	QMainWindow mainWindow;

	REGISTER_FULL_CTX(Widget_t, factory, defaultCleanupStrategy<Widget_t>);
	REGISTER_FULL_DEFAULT_CTX(test::RegressionFitAnalysisRegressorOverview);

	Widget_t *widget = FETCH_FROM_CTX(Widget_t);
	COM_SUBSCRIBE_FOR(widget, RegressorActiveStateChanged, boost::bind(activeStateChanged, _1, _2, _3));
	/*
	widget->setupLayout();
	widget->updateValues();
	*/
	mainWindow.setCentralWidget(widget);

	mainWindow.show();

 return app.exec();
}
