/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitAnalysisRegressorTimeseriesDisplayWidgetTests.cpp
 *
 *  Created on: 28.10.2014
 *      Author: gerberma
 */


#include <vector>
#include <string>
#include <QString>
#include <QDebug>
#include <boost/bind.hpp>
#include "communication/Signals.hpp"
#include "gui/RegressionFitAnalysisRegressorTimeseriesDisplayWidget.hpp"
#include "GuiAnalysisToolApplicationContext.hpp"

namespace test {

struct Item {
	typedef std::vector<double> ApplyedValuesContainer_t;
	typedef std::vector<std::string> DateResponseContainer_t;
	typedef std::vector<double> NumericResponseContainer_t;

	ApplyedValuesContainer_t &getApplyedValues(){
		return applyedValues;
	}


	DateResponseContainer_t &getDateResponse() {
		return dateResponse;
	}

	QString getName() {
		return name;
	}

	ApplyedValuesContainer_t applyedValues;
	DateResponseContainer_t dateResponse;
	QString name;
};

struct RegressorTimeseries {
	RegressorTimeseries():ispolynom(false), isscaled(true){;}

	typedef Item Item_t;
	typedef std::vector<double> ApplyedValuesContainer_t;
	typedef std::vector<std::string> DateResponseContainer_t;
	typedef std::vector<double> NumericResponseContainer_t;

	std::vector<Item *> &getItems() {
		return items;
	}

	bool isPolynom() {
		return ispolynom;
	}

	bool isScaled() {
		return isscaled;
	}

	std::vector<Item *> items;
	bool ispolynom;
	bool isscaled;
};
} /* namespace test */


void setPolynomState(test::RegressorTimeseries *, bool state) {
	qDebug() << __FUNCTION__ << " : " << state;
}

int
main(int argc, char *argv[]) {
	test::helper::GuiAnalysisToolApplicationContext<test::RegressorTimeseries, gui::RegressionFitAnalysisRegressorTimeseriesDisplayWidget<test::RegressorTimeseries> > appCtx;

	test::Item itemOne;
	test::Item itemTwo;

	itemOne.applyedValues.push_back(10.3);
	itemOne.applyedValues.push_back(11.0);
	itemOne.applyedValues.push_back(12.4);
	itemOne.applyedValues.push_back(13.8);

	itemOne.dateResponse.push_back("1999-08-11");
	itemOne.dateResponse.push_back("1999-08-12");
	itemOne.dateResponse.push_back("1999-08-13");
	itemOne.dateResponse.push_back("1999-08-14");

	itemOne.name = "Water Level";


	itemTwo.applyedValues.push_back(5.3);
	itemTwo.applyedValues.push_back(6.0);
	itemTwo.applyedValues.push_back(7.4);
	itemTwo.applyedValues.push_back(8.8);

	itemTwo.dateResponse.push_back("1999-08-01");
	itemTwo.dateResponse.push_back("1999-08-02");
	itemTwo.dateResponse.push_back("1999-08-03");
	itemTwo.dateResponse.push_back("1999-08-04");

	itemOne.name = "Water Level";
	itemTwo.name = "Seasonal";

	appCtx.t.items.push_back(&itemOne);
	appCtx.t.items.push_back(&itemTwo);

	return appCtx.run(argc, argv);
}

