/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitAnalysisResidualNormalProbabilityDisplayWidgetTest.cpp
 *
 *  Created on: Apr 2, 2014
 *      Author: gerberma
 */

#include <QApplication>
#include <QMainWindow>
#include <QVector>
#include "gui/RegressionFitAnalysisResidualNormalProbabilityDisplayWidget.hpp"
#include "communication/Signals.hpp"
#include "app/GenericApplicationContext.hpp"

using namespace app::context;

namespace test {

struct Point {
	Point(double x, double y):x(x), y(y){;}

	double getX() {
		return x;
	}

	double getY() {
		return y;
	}

	double x, y;
};

struct NormalProbability {
	typedef Point Point_t;

	NormalProbability() {
		theoreticalQuantiles << 0.223 << 1.544 << 3.7213;
		sampleQuantiles << 0.123 << 1.244 << 1.3213;
	}

	QVector<double> getTheoreticalQuantiles() {
		return theoreticalQuantiles;
	}

	QVector<double> getSampleQuantiles() {
		return sampleQuantiles;
	}

	Point_t getNormalLinePointOne() {
		return Point_t(1.2, 0.8);
	}

	Point_t getNormalLinePointTwo() {
		return Point_t(3.2, 2.8);
	}

	QVector<double> sampleQuantiles;
	QVector<double> theoreticalQuantiles;
};


}  // namespace test

typedef gui::RegressionFitAnalysisResidualNormalProbabilityDisplayWidget<test::NormalProbability> Widget_t;

Widget_t *factory() {
	return new Widget_t(FETCH_FROM_CTX(test::NormalProbability));
}


int
main(int argc, char *argv[]) {
	QApplication app(argc, argv);
	QMainWindow mainWindow;

	REGISTER_FULL_CTX(Widget_t, factory, defaultCleanupStrategy<Widget_t>);
	REGISTER_FULL_DEFAULT_CTX(test::NormalProbability);

	Widget_t *widget = FETCH_FROM_CTX(Widget_t);
	/*
	widget->setupLayout();
	widget->updateValues();
	*/
	mainWindow.setCentralWidget(widget);

	mainWindow.show();

 return app.exec();
}
