/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionModelAlgorithmDisplayWidgetTest.cpp
 *
 *  Created on: 27.01.2015
 *      Author: gerberma
 */



#include <QApplication>
#include <QMainWindow>
#include "gui/RegressionModelAlgorithmDisplayWidget.hpp"
#include "app/GenericApplicationContext.hpp"
#include "communication/Signals.hpp"

using namespace app::context;

namespace test {

struct RegressionModel {
	bool isStandardAlgorithm() {
		return false;
	}

	bool isPraisWinstenAlgorithm() {
		return false;
	}

	bool isRidgeAlgorithm() {
		return false;
	}

	bool isPCRAlgorithm() {
		return true;
	}

	double getPraisWinstenRho() {
		return 0.978;
	}

	double getRidgeKValue() {
		return 0.07;
	}

	unsigned getPCRrValue() {
		return 3;
	}

	void fireCalculated() {
		CalculatedSignal();
	}

	COM_SUBSCRIPTION_FUNCTION(AlgorithmChanged, communication::Signal<void()>);
	COM_SUBSCRIPTION_FUNCTION(Calculated, communication::Signal<void()>);
};

}  // namespace test


typedef gui::RegressionModelAlgorithmDisplayWidget<test::RegressionModel> Widget_t;

Widget_t *factory() {
	return new Widget_t(FETCH_FROM_CTX(test::RegressionModel));
}


int
main(int argc, char *argv[]) {
	QApplication app(argc, argv);
	QMainWindow mainWindow;


	REGISTER_FULL_CTX(Widget_t, factory, defaultCleanupStrategy<Widget_t>);
	REGISTER_FULL_DEFAULT_CTX(test::RegressionModel);

	mainWindow.setCentralWidget(FETCH_FROM_CTX(Widget_t));

	mainWindow.show();

	FETCH_FROM_CTX(test::RegressionModel)->fireCalculated();

 return app.exec();
}
