/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * ReportListElementWidgetsTest.cpp
 *
 *  Created on: Jul 24, 2014
 *      Author: gerberma
 */

#include <QApplication>
#include <QMainWindow>
#include <QLineEdit>
#include <QDebug>
#include <boost/bind.hpp>
#include "communication/Signals.hpp"
#include "gui/ReportListElementWidget.hpp"
#include "gui/ReportTextEditWidget.hpp"
#include "app/GenericApplicationContext.hpp"

using namespace app::context;


namespace test {

struct AnyTextReportListElement {
	QString getName() {
		return name;
	}

	QString getText() {
		return text;
	}

	bool isSelected() {
		return isselected;
	}

	QString text;
	QString name;
	bool isselected;
};

}  // namespace test

typedef gui::ReportTextEditWidget<test::AnyTextReportListElement, QLineEdit> TextEditWidget_t;
typedef gui::ReportListElementWidget<test::AnyTextReportListElement,  TextEditWidget_t> ReportListTextElement_t;


TextEditWidget_t *textEditWidgetFactory() {
	return new TextEditWidget_t(FETCH_FROM_CTX(test::AnyTextReportListElement));
}

ReportListTextElement_t *reportListTextElementFactory() {
	return new ReportListTextElement_t(
				FETCH_FROM_CTX(test::AnyTextReportListElement)
			);
}

void textChanged(test::AnyTextReportListElement *, QString text) {
	qDebug() << __FUNCTION__ << ":" << text;
}

int
main(int argc, char *argv[])
{
	using namespace boost;

	QApplication app(argc, argv);
	QMainWindow mainWindow;

	REGISTER_FULL_DEFAULT_CTX(test::AnyTextReportListElement);
	REGISTER_FULL_CTX(ReportListTextElement_t, reportListTextElementFactory, defaultCleanupStrategy<ReportListTextElement_t>);
	REGISTER_FULL_CTX(TextEditWidget_t, textEditWidgetFactory, defaultCleanupStrategy<TextEditWidget_t>);


	test::AnyTextReportListElement *anyTextContainer = FETCH_FROM_CTX(test::AnyTextReportListElement);
	anyTextContainer->name = "AnyTextContainer";
	anyTextContainer->text = "Any text";
	anyTextContainer->isselected = true;

	FETCH_FROM_CTX(ReportListTextElement_t)->setDynamicContentWidget(FETCH_FROM_CTX(TextEditWidget_t));
	FETCH_FROM_CTX(ReportListTextElement_t)->init();

	COM_SUBSCRIBE_FOR(FETCH_FROM_CTX(TextEditWidget_t), TextChanged, bind(textChanged, _1, _2));


	mainWindow.setCentralWidget(FETCH_FROM_CTX(ReportListTextElement_t));
	mainWindow.show();

	return app.exec();
}
