/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * SvgPlotWidgetTest.cpp
 *
 *  Created on: Jan 30, 2014
 *      Author: gerberma
 */


#include <QApplication>
#include <QMainWindow>
#include <QString>
#include "gui/SvgPlotWidget.hpp"
#include "app/GenericApplicationContext.hpp"
#include "communication/Signals.hpp"

namespace test {

class SvgPlot {
public:
	SvgPlot(QString plotLocation)
		:plotLocation(plotLocation)
	{}


	QString getPlotLocation() {
		return plotLocation;
	}

	int getWidthInPixel() {
		return 800;
	}

	int getHeightInPixel() {
		return 800;
	}

	COM_SUBSCRIPTION_FUNCTION(Loaded, communication::Signal<void()>);

private:
	QString plotLocation;
};

} /* namespace test */


using namespace app::context;

typedef test::SvgPlot Svg_Plot_t;
typedef gui::SvgPlotWidget<Svg_Plot_t> Svg_Plot_Widget_t;

Svg_Plot_t *svgPlotFactory() {
	return new Svg_Plot_t("./plot.svg");
}

Svg_Plot_Widget_t *svgPlotWidgetFactory() {
	return new Svg_Plot_Widget_t(FETCH_FROM_CTX(Svg_Plot_t));
}

int
main(int argc, char *argv[]) {
	QApplication app(argc, argv);
	QMainWindow mainWindow;

	REGISTER_FULL_CTX_WITH_CTX_SCOPE(Svg_Plot_t, svgPlotFactory, defaultCleanupStrategy<Svg_Plot_t>);
	REGISTER_FULL_CTX_WITH_CTX_SCOPE(Svg_Plot_Widget_t, svgPlotWidgetFactory, defaultCleanupStrategy<Svg_Plot_Widget_t>);

	mainWindow.setCentralWidget(FETCH_FROM_CTX(Svg_Plot_Widget_t));

	mainWindow.show();

	return app.exec();
}

