/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * dataModelCommonParameterWidgetTest.cpp
 *
 *  Created on: Nov 8, 2013
 *      Author: gerberma
 */

#include <iostream>
#include <boost/functional/value_factory.hpp>
#include "conversion/converter.hpp"
#include "gui/guiconverter.hpp"
#include <QString>
#include <QDebug>
#include <boost/bind.hpp>
#include "communication/Signals.hpp"
#include "gui/NavigationTreeWidget.hpp"
#include <QApplication>
#include <QMainWindow>
#include "gui/DataModelCommonParameterWidget.hpp"
#include "gui/DataModelWizardPages.hpp"
#include "gui/WizardNextIdStrategy.hpp"
#include "model/ReferenceContainer.hpp"
#include "model/DataModel.hpp"
#include "model/DataModelActionRules.hpp"
#include "model/MissingDataStrategy.hpp"
#include "app/GenericApplicationContext.hpp"

using namespace model;
using namespace app::context;

class OtherMissingDataStrategyFactory {
public:
	typedef model::r::MissingDataStrategy Missing_Data_Strategy_t;
	typedef model::CallbackPath<QString, Missing_Data_Strategy_t> Callback_Path_t;
	typedef typename model::r::OtherMissingDataStrategy<Callback_Path_t > T_t;

	T_t *create(Missing_Data_Strategy_t *m){
		T_t *t;
		//TODO: we need the application's model here to resolve the callback path correctly !!!
		//Note that this can also be done via a subscription to a signal which itself distributes the model when created
		//Or we subscribe to all model change signals and keep a local reference map which will be available in this factory (that would be great!)
		// - every model object factory should register this map as subscriber to the model object already created
		//We simply need a type specific representation here (e.g. for MissingDataStrategy a Representation which knows the Analyses and corresponding DataModels)

		Callback_Path_t path(boost::bind(boost::value_factory<QString>(), QString("bar - baz")), m);

		t = new T_t(path);

		return t;
	}
};



typedef model::r::MissingDataStrategy Missing_Data_Strategy_t;
typedef model::CallbackPath<QString, Missing_Data_Strategy_t> Callback_Path_t;
typedef model::r::OtherMissingDataStrategy<Callback_Path_t> Other_Missing_Data_Strategy_t;
typedef OtherMissingDataStrategyFactory Other_Missing_Data_Strategy_Factory_t;

typedef conversion::FactoryBasedConverter<Missing_Data_Strategy_t, Other_Missing_Data_Strategy_t> Converter_t;
typedef model::ReferenceContainer<
		Other_Missing_Data_Strategy_t,
		Missing_Data_Strategy_t,
		Converter_t> Reference_Container_t;
typedef model::DataModel<> DataModel_t;
typedef gui::WizardNextIdStrategy<gui::DataModelWizardPages::Pages> Wizard_Next_Id_Strategy;
typedef gui::DataModelCommonParameterWidget<DataModel_t, Wizard_Next_Id_Strategy> DataModelCommonParameterWidget_t;


namespace conversion {

template<>
inline QString conversion::Converter<typename gui::DataModelEditWidget<DataModel_t>::Missing_Data_Strategy_Gui_t, QString>::convert(gui::DataModelEditWidget<DataModel_t>::Missing_Data_Strategy_Gui_t strategy)
{
	const char* MissingDataStrategyNames[2] = { "insert zero", "skip"};

	return QString(MissingDataStrategyNames[strategy.value.get()]);
}


template<>
inline QString conversion::Converter<typename gui::OtherDataModelWidget<DataModel_t>::Other_Missing_Data_Strategy_Gui_t, QString>::convert(gui::OtherDataModelWidget<DataModel_t>::Other_Missing_Data_Strategy_Gui_t strategy)
{
	QString value(strategy.value.getPath().getCallback()());

	return value;
}

template<>
inline QString conversion::Converter<typename gui::OtherDataModelWidget<DataModel_t>::Other_Missing_Data_Strategy_Strategy_Gui_t, QString>::convert(gui::OtherDataModelWidget<DataModel_t>::Other_Missing_Data_Strategy_Strategy_Gui_t strategy)
{
	const char* OtherMissingDataStrategyNames[2] = { "insert zero", "skip"};

	QString value;

	value += QString(OtherMissingDataStrategyNames[strategy.value.get()]);

	return value;
}


} /* namespace conversion */

bool operator==(Missing_Data_Strategy_t const &missing, Other_Missing_Data_Strategy_t const &other) {
	return missing.get() == other.get();
}


Reference_Container_t *referenceContainerFactory() {
	Reference_Container_t *referenceContainer;

	Converter_t *converter = FETCH_FROM_CTX(Converter_t);
	referenceContainer = new Reference_Container_t(converter);

	return referenceContainer;
}


Converter_t *converterFactory() {
	Converter_t *converter;
	Other_Missing_Data_Strategy_Factory_t *factory = FETCH_FROM_CTX(Other_Missing_Data_Strategy_Factory_t);

	converter = new Converter_t(boost::bind(&Other_Missing_Data_Strategy_Factory_t::create, factory, _1));

	return converter;
}


DataModel_t *dataModelFactory() {
	//DataModel_t *otherDataModel = new DataModel_t;

	Missing_Data_Strategy_t *insertZeroMissingDataStrategyLeaf = new Missing_Data_Strategy_t(0);
	Callback_Path_t insertZeroCallbackPath(boost::bind(boost::value_factory<QString>(), QString("foo - bar")), insertZeroMissingDataStrategyLeaf); //foo-bar should come from otherDataModel

	Missing_Data_Strategy_t *skipMissingDataStrategyLeaf = new Missing_Data_Strategy_t(1);
	Callback_Path_t skipCallbackPath(boost::bind(boost::value_factory<QString>(), QString("bar - baz")), skipMissingDataStrategyLeaf); //foo-bar should come from otherDataModel

	DataModel_t::missing_data_strategy_container_t missingDataStrategies;

	missingDataStrategies.push_back(new model::r::MissingDataStrategy(0));
	missingDataStrategies.push_back(new model::r::MissingDataStrategy(1));

	std::map<QString, QString> availableCSVDelimiters;
	availableCSVDelimiters["Tab"] = "\t";
	availableCSVDelimiters["Comma"] = ",";
	availableCSVDelimiters["Semicolon"] = ";";
	availableCSVDelimiters["Space"] = " ";

	DataModel_t *dataModel = new DataModel_t(FETCH_FROM_CTX(Reference_Container_t));
	dataModel->setAvailableCSVDelimiters(availableCSVDelimiters);
	dataModel->setCSVDelimiter(availableCSVDelimiters["Comma"]);
	dataModel->setMissingDataStrategies(missingDataStrategies);
	dataModel->setSourceUri(QUrl("C:/damreg/datasource.csv"));
	dataModel->setCurrentMissingDataStrategy(0);

	return dataModel;
}



DataModelCommonParameterWidget_t *dataModelCommonParameterWidgetFactory() {
	return new DataModelCommonParameterWidget_t(NULL, FETCH_FROM_CTX(DataModel_t), FETCH_FROM_CTX(Wizard_Next_Id_Strategy), gui::DataModelWizardPages::Final);
}

void apply(DataModel_t *dataModel, unsigned index) {
	std::cout << "apply : " << dataModel->getOtherMissingDataStrategies().at(index)->get() << std::endl;
	dataModel->setCurrentMissingDataStrategyFromOtherMissingDataStrategy(index);
}

void missingDataStrategySelectionChanged(DataModel_t *dataModel, unsigned index) {
	std::cout << __FUNCTION__ << " " << dataModel->getMissingDataStrategies().at(index)->get() << std::endl;
}

void csvDelimiterChanged(DataModel_t *dataModel, QString csvDelimiter) {
	qDebug() << __FUNCTION__ << " " << csvDelimiter;
}

int
main(int argc, char *argv[]) {
	using namespace boost;
	QApplication app(argc, argv);
	QMainWindow mainWindow;

	REGISTER_FULL_DEFAULT_CTX_WITH_CTX_SCOPE(Other_Missing_Data_Strategy_Factory_t);
	REGISTER_FULL_CTX_WITH_CTX_SCOPE(Converter_t, converterFactory, defaultCleanupStrategy<Converter_t>);
	REGISTER_FULL_CTX_WITH_CTX_SCOPE(Reference_Container_t, referenceContainerFactory, defaultCleanupStrategy<Reference_Container_t>);
	REGISTER_FULL_CTX_WITH_CTX_SCOPE(DataModel_t, dataModelFactory, defaultCleanupStrategy<DataModel_t>);
	REGISTER_FULL_CTX_WITH_CTX_SCOPE(DataModelCommonParameterWidget_t, dataModelCommonParameterWidgetFactory, defaultCleanupStrategy<DataModelCommonParameterWidget_t>);
	REGISTER_FULL_DEFAULT_CTX(Wizard_Next_Id_Strategy);



	DataModelCommonParameterWidget_t *dataModelCommonParameterWidget = FETCH_FROM_CTX(DataModelCommonParameterWidget_t);
	dataModelCommonParameterWidget->subscribeForMissingDataStrategySelected(boost::bind(missingDataStrategySelectionChanged, _1, _2));
	dataModelCommonParameterWidget->subscribeForOtherMissingDataStrategySelected(boost::bind(apply, _1, _2));

	COM_SUBSCRIBE_FOR(dataModelCommonParameterWidget, CSVDelimiterChanged, bind(csvDelimiterChanged, _1, _2));

	mainWindow.setCentralWidget(dataModelCommonParameterWidget);


	dataModelCommonParameterWidget->show();

	mainWindow.show();

	int ret = app.exec();

	return ret;
}

