/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
//#define BOOST_TEST_MAIN
#define BOOST_TEST_MODULE AnalysisActionRuleTests

#include <boost/test/unit_test.hpp>
#include <turtle/mock.hpp>
#include <set>
#include <vector>
#include "model/AnalysisActionRules.hpp"
#include "app/GenericApplicationContext.hpp"
#include "tests/helper/GenericApplicationContextTestHelper.hpp"

using namespace app::context;
using namespace test::helper;

MOCK_CLASS(BehaviorIndicatorColumn) {
};

MOCK_CLASS(DataModel) {
	MOCK_METHOD_EXT(getBehaviorIndicatorColumns, 0, std::set<BehaviorIndicatorColumn *>(), getBehaviorIndicatorColumns);
};


MOCK_CLASS(Analysis) {
	MOCK_METHOD_EXT(getDataModel, 0, DataModel*(), getDataModel);
};

typedef dambase::model::AnalysisActionRules Analysis_Action_Rules_t;


class ApplicationContext {
public:
	ApplicationContext() :
		GAC_DEFAULT_CTOR_INIT(Analysis_Action_Rules_t)
	{}

public:
	DataModel dataModel;
	Analysis analysis;
	BehaviorIndicatorColumn behaviorIndicatorColumn;
	std::set<BehaviorIndicatorColumn *> behaviorIndicatorColumns;

private:
	GAC_SCOPE(Analysis_Action_Rules_t)
};



BOOST_FIXTURE_TEST_CASE(NewDataModelIsAllowedIfNoOneExistsAlready, ApplicationContext)
{
	MOCK_EXPECT(analysis.getDataModel).returns(NULL);
	BOOST_CHECK(FETCH_FROM_CTX(Analysis_Action_Rules_t)->isNewDataModelAllowedFor(&analysis) == true);
}

BOOST_FIXTURE_TEST_CASE(NewDataModelIsNotAllowedIfOneExistsAlready, ApplicationContext)
{
	MOCK_EXPECT(analysis.getDataModel).returns(&dataModel);
	BOOST_CHECK(FETCH_FROM_CTX(Analysis_Action_Rules_t)->isNewDataModelAllowedFor(&analysis) == false);
}

BOOST_FIXTURE_TEST_CASE(NewRegressionModelIsAllowedIfADataModelExistsAndABehaviorIndicatorHasBeenSet, ApplicationContext)
{
	behaviorIndicatorColumns.insert(&behaviorIndicatorColumn);
	MOCK_EXPECT(dataModel.getBehaviorIndicatorColumns).at_least(1).returns(behaviorIndicatorColumns);
	MOCK_EXPECT(analysis.getDataModel).returns(&dataModel);
	BOOST_CHECK(FETCH_FROM_CTX(Analysis_Action_Rules_t)->isNewRegressionModelAllowedFor(&analysis) == true);
}

BOOST_FIXTURE_TEST_CASE(NewRegressionModelIsNotAllowedIfNoDataModelExists, ApplicationContext)
{
	MOCK_EXPECT(analysis.getDataModel).returns(NULL);
	BOOST_CHECK(FETCH_FROM_CTX(Analysis_Action_Rules_t)->isNewRegressionModelAllowedFor(&analysis) == false);
}
