/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
//#define BOOST_TEST_MAIN
#define BOOST_TEST_MODULE ProjectActionRuleTests

#include <vector>
#include <boost/test/unit_test.hpp>
#include <turtle/mock.hpp>
#include "model/ProjectActionRules.hpp"
#include "app/GenericApplicationContext.hpp"
#include "tests/helper/GenericApplicationContextTestHelper.hpp"

using namespace app::context;
using namespace test::helper;

MOCK_CLASS(Analysis) {
};

MOCK_CLASS(Analysis_Container) {
	MOCK_METHOD_EXT(size, 0 , int(), size);
};

MOCK_CLASS(Project) {
	MOCK_METHOD_EXT(getAnalyses, 0, Analysis_Container(), getAnalyses);
};

typedef Project Project_t;
typedef Analysis Analysis_t;
typedef dambase::model::ProjectActionRules Project_Action_Rules_t;


class ApplicationContext {
public:
	ApplicationContext() :
		GAC_DEFAULT_CTOR_INIT(Project_t),
		GAC_DEFAULT_CTOR_INIT(Project_Action_Rules_t),
		GAC_DEFAULT_CTOR_INIT(Analysis_t)
	{}

private:
	GAC_SCOPE(Project_t)
	GAC_SCOPE(Project_Action_Rules_t)
	GAC_SCOPE(Analysis_t)
};



BOOST_FIXTURE_TEST_CASE(NewAnalysisIsAllowedIfAProjectExists, ApplicationContext)
{
	BOOST_CHECK(FETCH_FROM_CTX(Project_Action_Rules_t)->isNewAnalysisAllowedFor(FETCH_FROM_CTX(Project_t)) == true);
}


BOOST_FIXTURE_TEST_CASE(NewAnalysisIsAllowedIfOneAlreadyExists, ApplicationContext)
{
	Analysis_Container analysis_container;
	MOCK_EXPECT(analysis_container.size).returns(1);
	MOCK_EXPECT(FETCH_FROM_CTX(Project_t)->getAnalyses).returns(analysis_container);

	BOOST_CHECK(FETCH_FROM_CTX(Project_Action_Rules_t)->isNewAnalysisAllowedFor<Project_t>(FETCH_FROM_CTX(Project_t)) == true);
}


BOOST_FIXTURE_TEST_CASE(NewAnalysisIsNotAllowedIfNoProjectExists, ApplicationContext)
{
	BOOST_CHECK(FETCH_FROM_CTX(Project_Action_Rules_t)->isNewAnalysisAllowedFor<Project_t>(NULL) == false);
}
