/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * AnalysisToolsModeDescriptionTests.cpp
 *
 *  Created on: 14.10.2014
 *      Author: gerberma
 */


#define BOOST_TEST_MODULE AnalysisToolsModeDescriptionTests

#include <boost/test/unit_test.hpp>
#include <turtle/mock.hpp>
#include "model/mode/AnalysisToolsModeDescription.hpp"
#include "memory/OwnerhshipPolicies.hpp"
#include "app/GenericApplicationContext.hpp"
#include "tests/helper/GenericApplicationContextTestHelper.hpp"

using namespace app::context;

namespace test {

MOCK_CLASS(ModeHandler) {
};

MOCK_CLASS(AnalysisToolDescription) {
};

MOCK_CLASS(AnalysisToolModeDescription) {
	typedef AnalysisToolDescription AnalysisToolDescription_t;

	MOCK_METHOD_EXT(appearLeft, 1, bool(bool), appearLeft);
	MOCK_METHOD_EXT(forTabNumber, 1, bool(int), forTabNumber);
	MOCK_METHOD_EXT(appearAutomatically, 1, bool(bool), appearAutomatically);
	MOCK_METHOD_EXT(forMode, 1, bool(ModeHandler *), forMode);
	MOCK_METHOD_EXT(getAnalysisToolDescription, 0, AnalysisToolDescription *(), getAnalysisToolDescription);
};

}  // namespace test

typedef model::mode::AnalysisToolsModeDescription<
			test::AnalysisToolModeDescription,
			test::ModeHandler,
			memory::IgnoreOwnershipPolicy<test::AnalysisToolModeDescription>
		> AnalysisToolsModeDescription_t;

using namespace test::helper;
struct ApplicationContext {
	ApplicationContext():
		GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(AnalysisToolsModeDescription_t, boost::bind(&ApplicationContext::factory, this))
	{;}

	AnalysisToolsModeDescription_t *factory() {
		return new AnalysisToolsModeDescription_t(&modeHandler);
	}

	GAC_SCOPE(AnalysisToolsModeDescription_t)
	test::ModeHandler modeHandler;
};


BOOST_FIXTURE_TEST_CASE(TestFilterForAutomaticallyAppear, ApplicationContext) {
	test::AnalysisToolModeDescription matchAtModelDescription;
	test::AnalysisToolModeDescription noMatchAtModelDescription;
	test::AnalysisToolDescription atDescription;

	MOCK_EXPECT(matchAtModelDescription.appearLeft).once().with(true).returns(true);
	MOCK_EXPECT(matchAtModelDescription.forTabNumber).once().with(0).returns(true);
	MOCK_EXPECT(matchAtModelDescription.appearAutomatically).once().with(true).returns(true);
	MOCK_EXPECT(matchAtModelDescription.forMode).once().with(&modeHandler).returns(true);
	MOCK_EXPECT(matchAtModelDescription.getAnalysisToolDescription).once().returns(&atDescription);

	MOCK_EXPECT(noMatchAtModelDescription.appearLeft).once().with(true).returns(false);

	AnalysisToolsModeDescription_t *analysisToolsModeDescription = FETCH_FROM_CTX(AnalysisToolsModeDescription_t);
	analysisToolsModeDescription->addAnalysisToolModeDescription(&matchAtModelDescription);
	analysisToolsModeDescription->addAnalysisToolModeDescription(&noMatchAtModelDescription);

	analysisToolsModeDescription->filterForAutomaticallyAppear(true, 0, true);
}
