/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * ModeHandlerTests.cpp
 *
 *  Created on: 14.10.2014
 *      Author: gerberma
 */


#define BOOST_TEST_MODULE ModeHandlerTests
#include <boost/test/unit_test.hpp>
#include <turtle/mock.hpp>
#include <turtle/constraints.hpp>
#include <boost/any.hpp>
#include <boost/utility.hpp>
#include <boost/bind.hpp>
#include <boost/ref.hpp>
#include "model/mode/ModeHandler.hpp"
#include "app/GenericApplicationContext.hpp"
#include "tests/helper/GenericApplicationContextTestHelper.hpp"

using namespace app::context;

typedef model::mode::ModeHandler ModeHandler_t;

namespace test {

MOCK_CLASS(ModeChangedSignalReceiver) {
	void received(){receivedDelegate();}
	MOCK_METHOD_EXT(receivedDelegate, 0, void(), receivedDelegate);
};

using namespace helper;
struct ApplicationContext {
	ApplicationContext():
		GAC_DEFAULT_CTOR_INIT(ModeHandler_t)
	{;}

	GAC_SCOPE(ModeHandler_t)
};
}  // namespace test

BOOST_FIXTURE_TEST_CASE(TestSetModeFiresModeChangedSignal, test::ApplicationContext) {
	using namespace boost;

	test::ModeChangedSignalReceiver receiver;
	MOCK_EXPECT(receiver.receivedDelegate).once();

	ModeHandler_t *modeHandler = FETCH_FROM_CTX(ModeHandler_t);
	COM_SUBSCRIBE_FOR(modeHandler, ModeChanged, bind(&test::ModeChangedSignalReceiver::received, &receiver));
	modeHandler->setMode(1);
}

BOOST_FIXTURE_TEST_CASE(TestSetScientificMode, test::ApplicationContext) {
	ModeHandler_t *modeHandler = FETCH_FROM_CTX(ModeHandler_t);
	modeHandler->setMode(1);
	BOOST_CHECK(modeHandler->isScientific());

	modeHandler->setMode(0);
	BOOST_CHECK(!modeHandler->isScientific());
}
