/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataFrameInfluenceFactorsTests.cpp
 *
 *  Created on: Jan 30, 2014
 *      Author: gerberma
 */

#define BOOST_TEST_MODULE DataFrameInfluenceFactorsTests

#include <boost/test/unit_test.hpp>
#include <boost/bind.hpp>
#include <boost/filesystem.hpp>
#include <QString>
#include <QUrl>
#include <Rcpp.h>
#include "model/r/DataFrame.hpp"
#include "model/r/DataFrameProvider.hpp"
#include "model/r/DataFrameInfluenceFactors.hpp"
#include "model/r/DataFrameInfluenceFactorsProvider.hpp"
#include "RApplicationContext.hpp"
#include "applicationcontext/scopes.hpp"
#include "app/GenericApplicationContext.hpp"
#include "tests/helper/GenericApplicationContextTestHelper.hpp"

using namespace app::context;
using namespace test::helper;


typedef model::r::DataFrameColumn Data_Frame_Column_t;
typedef model::r::DataFrameProvider Data_Frame_Provider_t;
typedef model::r::DataFrameInfluenceFactorsProvider<> Data_Frame_Influence_Factors_Provider_t;


class ApplicationContext : public test::RApplicationContext {
public:
	ApplicationContext() :
		GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(Data_Frame_Provider_t, boost::bind(&ApplicationContext::dataFrameProviderFactory, this)),
		GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(Data_Frame_Influence_Factors_Provider_t, boost::bind(&ApplicationContext::dataFrameInflunceFactorsProviderFactory, this)),
		GAC_DEFAULT_CTOR_INIT(Data_Frame_Column_t)
	{;}

private:
	Data_Frame_Provider_t *dataFrameProviderFactory() {
		Data_Frame_Provider_t *dataFrameProvider = new Data_Frame_Provider_t;
		return dataFrameProvider;
	}

	Data_Frame_Influence_Factors_Provider_t *dataFrameInflunceFactorsProviderFactory() {
		return new Data_Frame_Influence_Factors_Provider_t();
	}

private:
	GAC_SCOPE(Data_Frame_Provider_t)
	GAC_SCOPE(Data_Frame_Influence_Factors_Provider_t)
	GAC_SCOPE(Data_Frame_Column_t)
};



BOOST_FIXTURE_TEST_CASE(TestPlotLocationContainsTheFileOfThePlot, ApplicationContext) {
	app::context::LocalScope<model::r::DataFrame> dataFrame(FETCH_FROM_CTX(Data_Frame_Provider_t)->load(QUrl("influence_factors_data.csv")));

	app::context::LocalScope<model::r::DataFrameInfluenceFactors<> > influenceFactors(FETCH_FROM_CTX(Data_Frame_Influence_Factors_Provider_t)->load(dataFrame.get(), FETCH_FROM_CTX(Data_Frame_Column_t), FETCH_FROM_CTX(Data_Frame_Column_t)));

	BOOST_CHECK(influenceFactors->getPlotLocation() != "");

	BOOST_CHECK(boost::filesystem::exists(influenceFactors->getPlotLocation().toStdString()));
}

