/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataFrameTimeSeriesOverviewTests.cpp
 *
 *  Created on: Jun 24, 2014
 *      Author: gerberma
 */

#define BOOST_TEST_MODULE DataFrameTimeSeriesOverviewTests

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>
#include <boost/bind.hpp>
#include <Rcpp.h>
#include <QDebug>
#include "RApplicationContext.hpp"
#include "model/r/DataFrame.hpp"
#include "model/r/DataFrameProvider.hpp"
#include "model/r/DataFrameTimeSeriesOverview.hpp"
#include "model/r/DataFrameTimeSeriesOverviewProvider.hpp"
#include "applicationcontext/scopes.hpp"
#include "app/GenericApplicationContext.hpp"
#include "RegressionFitApplicationContext.hpp"
#include "tests/helper/GenericApplicationContextTestHelper.hpp"


using namespace app::context;
using namespace test::helper;

typedef model::r::DataFrameColumn Data_Frame_Column_t;
typedef model::r::DataFrameProvider Data_Frame_Provider_t;
typedef model::r::DataFrameTimeSeriesOverviewProvider<> TimeSeriesOverviewProvider_t;

class ApplicationContext : public test::RApplicationContext {
public:
	ApplicationContext() :
		GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(Data_Frame_Provider_t, boost::bind(&ApplicationContext::dataFrameProviderFactory, this)),
		GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(TimeSeriesOverviewProvider_t, boost::bind(&ApplicationContext::timeSeriesOverviewProviderFactory, this))
	{;}

private:
	Data_Frame_Provider_t *dataFrameProviderFactory() {
		Data_Frame_Provider_t *dataFrameProvider = new Data_Frame_Provider_t;
		return dataFrameProvider;
	}

	TimeSeriesOverviewProvider_t *timeSeriesOverviewProviderFactory() {
		return new TimeSeriesOverviewProvider_t();
	}

	GAC_SCOPE(Data_Frame_Provider_t)
	GAC_SCOPE(TimeSeriesOverviewProvider_t)
};


BOOST_FIXTURE_TEST_CASE(IntegrationTest, ApplicationContext) {
	using namespace app::context;

	LocalScope<model::r::DataFrame> dataFrame(FETCH_FROM_CTX(Data_Frame_Provider_t)->load(QUrl("data.csv")));
	dataFrame->setDateColumn(0);

	dataFrame->assignWaterLevelColumn(1);
	LocalScope<model::r::DataFrameTimeSeriesOverview > timeSeriesOverview(FETCH_FROM_CTX(TimeSeriesOverviewProvider_t)->load(dataFrame.get(), *dataFrame->getWaterLevelColumns().begin()));

	/*
	qDebug() << timeSeriesOverview->getDates().size();
	qDebug() << QString::fromStdString(timeSeriesOverview->getDates().at(0));
	qDebug() << timeSeriesOverview->getValues().size();
	qDebug() << timeSeriesOverview->getValues().at(0);
	*/

	BOOST_REQUIRE_EQUAL(7, timeSeriesOverview->getDates().size());
	BOOST_REQUIRE_EQUAL("1983-06-01", timeSeriesOverview->getDates().at(0));
	BOOST_REQUIRE_EQUAL(7, timeSeriesOverview->getValues().size());
	BOOST_REQUIRE_CLOSE(9.34, timeSeriesOverview->getValues().at(0), 0.001);
}
