/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitAnalysisACFTests.cpp
 *
 *  Created on: Apr 8, 2014
 *      Author: gerberma
 */

#define BOOST_TEST_MODULE RegressionFitAnalysisACFTests

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>
#include <boost/bind.hpp>
#include <boost/foreach.hpp>
#include <string>
#include <Rcpp.h>
#include "RApplicationContext.hpp"
#include "RegressionFitApplicationContext.hpp"
#include "model/r/RRegressionFitAnalysisACF.hpp"
#include "model/r/RegressionFitAnalysisACFProvider.hpp"
#include "applicationcontext/scopes.hpp"
#include "app/GenericApplicationContext.hpp"
#include "tests/helper/GenericApplicationContextTestHelper.hpp"

using namespace app::context;
using namespace test::helper;


typedef typename test::RegressionFitApplicationContext::RegressionFit_t RegressionFit_t;
typedef model::r::RRegressionFitAnalysisACF Acf_t;
typedef model::r::RegressionFitAnalysisACFProvider<Acf_t, RegressionFit_t> Provider_t;

struct ApplicationContext : public test::RApplicationContext {
	ApplicationContext() :
		GAC_DEFAULT_CTOR_INIT(Provider_t),
		DOUBLE_PRECISION(0.001)
	{
		fitCtx.init();
	}

	GAC_SCOPE(Provider_t);
	test::RegressionFitApplicationContext fitCtx;
	const double DOUBLE_PRECISION;
};

BOOST_FIXTURE_TEST_CASE(IntegrationTest, ApplicationContext) {
	LocalScope<Acf_t> acf(FETCH_FROM_CTX(Provider_t)->load(fitCtx.regressionFit.get()));

	/*
	qDebug() << acf->getAcfs().at(1);
	qDebug() << acf->getLowerCi();
	*/

	BOOST_REQUIRE_EQUAL(acf->getAcfs().size(), 5);
	BOOST_REQUIRE_CLOSE(acf->getAcfs().at(0), 1, DOUBLE_PRECISION/100); //per definition 1 but we might differ a bit due to round errors (because we use doubles and not ints)
	BOOST_REQUIRE_CLOSE(acf->getAcfs().at(1), -0.556878, DOUBLE_PRECISION);

	BOOST_REQUIRE_EQUAL(acf->getLags().size(), acf->getAcfs().size()); //the need to be of same size, everything else doesn't make any sense
	BOOST_REQUIRE_EQUAL(acf->getLags().at(0), 0); //we start at 0
	BOOST_REQUIRE_EQUAL(acf->getLags().at(1), 1); //one step resolution

	BOOST_REQUIRE_CLOSE(acf->getLowerCi(), -0.876523, DOUBLE_PRECISION);
	BOOST_REQUIRE_EQUAL(acf->getUpperCi(), acf->getLowerCi() * -1.0); //symetric confidence interval
}
