/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitAnalysisOverallRegressionQualityTests.cpp
 *
 *  Created on: Apr 15, 2014
 *      Author: gerberma
 */


#define BOOST_TEST_MODULE RegressionFitAnalysisOverallRegressionQualityTests

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>
#include <boost/bind.hpp>
#include <Rcpp.h>
#include <QDebug>
#include "RApplicationContext.hpp"
#include "RegressionFitApplicationContext.hpp"
#include "model/r/RRegressionFitAnalysisOverallRegressionQuality.hpp"
#include "model/r/RegressionFitAnalysisOverallRegressionQualityProvider.hpp"
#include "applicationcontext/scopes.hpp"
#include "app/GenericApplicationContext.hpp"
#include "tests/helper/GenericApplicationContextTestHelper.hpp"

using namespace app::context;
using namespace test::helper;

typedef typename test::RegressionFitApplicationContext::RegressionFit_t RegressionFit_t;
typedef model::r::RRegressionFitAnalysisOverallRegressionQuality RegressionFitAnalysisOverallRegressionQuality_t;
typedef model::r::RegressionFitAnalysisOverallRegressionQualityProvider<RegressionFitAnalysisOverallRegressionQuality_t, RegressionFit_t> Provider_t;

struct ApplicationContext : public test::RApplicationContext {
	ApplicationContext() :
		GAC_DEFAULT_CTOR_INIT(Provider_t),
		DOUBLE_PRECISION(0.001)
	{
		fitCtx.init();
	}

	GAC_SCOPE(Provider_t);
	test::RegressionFitApplicationContext fitCtx;
	const double DOUBLE_PRECISION;
};

BOOST_FIXTURE_TEST_CASE(IntegrationTest, ApplicationContext) {
	LocalScope<RegressionFitAnalysisOverallRegressionQuality_t> overview(FETCH_FROM_CTX(Provider_t)->load(fitCtx.regressionFit.get()));

	/*
	qDebug() << overview->getAutocorDwStatDegree();
	qDebug() << overview->getAutocorDwStatValue();
	qDebug() << overview->getFitR2adjDegree();
	qDebug() << overview->getFitR2adjValue();
	*/

	BOOST_REQUIRE_EQUAL(overview->getAutocorDwStatDegree(), 0);
	BOOST_REQUIRE_CLOSE(overview->getAutocorDwStatValue(), 2.81861, DOUBLE_PRECISION);
	BOOST_REQUIRE_EQUAL(overview->getFitR2adjDegree(), 0);
	BOOST_REQUIRE_CLOSE(overview->getFitR2adjValue(), 1, DOUBLE_PRECISION);
}

