/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitAnalysisPhysicalSummaryTests.cpp
 *
 *  Created on: Mar 10, 2014
 *      Author: gerberma
 */

#define BOOST_TEST_MODULE RegressionFitAnalysisPhysicalSummaryTests

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>
#include <boost/bind.hpp>
#include <Rcpp.h>
#include <QDebug>
#include "RApplicationContext.hpp"
#include "model/r/RRegressionFitAnalysisPhysicalSummary.hpp"
#include "model/r/RegressionFitAnalysisPhysicalSummaryProvider.hpp"
#include "applicationcontext/scopes.hpp"
#include "app/GenericApplicationContext.hpp"
#include "RegressionFitApplicationContext.hpp"
#include "tests/helper/GenericApplicationContextTestHelper.hpp"

using namespace app::context;
using namespace test::helper;


typedef typename test::RegressionFitApplicationContext::RegressionFit_t RegressionFit_t;
typedef model::r::RRegressionFitAnalysisPhysicalSummary PhysicalSummary_t;
typedef model::r::RegressionFitAnalysisPhysicalSummaryProvider<PhysicalSummary_t, RegressionFit_t> Provider_t;

struct ApplicationContext : public test::RApplicationContext {
	ApplicationContext() :
		GAC_DEFAULT_CTOR_INIT(Provider_t),
		DOUBLE_PRECISION(0.001)
	{
		fitCtx.init();
	}

	GAC_SCOPE(Provider_t);
	test::RegressionFitApplicationContext fitCtx;
	const double DOUBLE_PRECISION;
};


BOOST_FIXTURE_TEST_CASE(IntegrationTest, ApplicationContext) {
	using namespace app::context;

	LocalScope<PhysicalSummary_t> physicalSummary(FETCH_FROM_CTX(Provider_t)->load(
		fitCtx.regressionFit.get()
	));



/*
	qDebug() << physicalSummary->getBehaviorIndicatorValues().size();
	qDebug() << physicalSummary->getBehaviorIndicatorValues().at(0);

	qDebug() << physicalSummary->getPredictions().size();
	qDebug() << physicalSummary->getPredictions().at(0);

	qDebug() << physicalSummary->getDates().size();
	qDebug() << QString::fromStdString(physicalSummary->getDates().at(0));
*/

	BOOST_REQUIRE_EQUAL(5, physicalSummary->getBehaviorIndicatorValues().size());
	BOOST_REQUIRE_CLOSE(1, physicalSummary->getBehaviorIndicatorValues().at(0), DOUBLE_PRECISION);

	BOOST_REQUIRE_EQUAL(5, physicalSummary->getDates().size());
	BOOST_REQUIRE_EQUAL("1983-08-11", physicalSummary->getDates().at(0));

	BOOST_REQUIRE_EQUAL(5, physicalSummary->getPredictions().size());
	BOOST_REQUIRE_CLOSE(1, physicalSummary->getPredictions().at(0), DOUBLE_PRECISION);

}
