/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitAnalysisRegressionSummaryTests.cpp
 *
 *  Created on: Mar 28, 2014
 *      Author: gerberma
 */



#define BOOST_TEST_MODULE RegressionFitAnalysisRegressionSummaryTests

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>
#include <boost/bind.hpp>
#include <Rcpp.h>
#include <QDebug>
#include "RApplicationContext.hpp"
#include "RegressionFitApplicationContext.hpp"
#include "model/r/RRegressionFitAnalysisRegressionSummary.hpp"
#include "model/r/RegressionFitAnalysisRegressionSummaryProvider.hpp"
#include "applicationcontext/scopes.hpp"
#include "app/GenericApplicationContext.hpp"
#include "tests/helper/GenericApplicationContextTestHelper.hpp"

using namespace app::context;
using namespace test::helper;

typedef typename test::RegressionFitApplicationContext::RegressionFit_t RegressionFit_t;
typedef model::r::RRegressionFitAnalysisRegressionSummary RegressionFitAnalysisRegressionSummary_t;
typedef model::r::RegressionFitAnalysisRegressionSummaryProvider<RegressionFit_t, RegressionFitAnalysisRegressionSummary_t> Provider_t;

struct ApplicationContext : public test::RApplicationContext {
	ApplicationContext() :
		GAC_DEFAULT_CTOR_INIT(Provider_t),
		DOUBLE_PRECISION(0.001)
	{
		fitCtx.init();
	}

	GAC_SCOPE(Provider_t);
	test::RegressionFitApplicationContext fitCtx;
	const double DOUBLE_PRECISION;
};

BOOST_FIXTURE_TEST_CASE(TestEverythingOfSummary, ApplicationContext) {
	LocalScope<RegressionFitAnalysisRegressionSummary_t> summary(FETCH_FROM_CTX(Provider_t)->load(fitCtx.regressionFit.get()));
	/*
	qDebug() << summary->getSSRes();
	qDebug() << summary->getMSRes();
	qDebug() << summary->getMSResDerived();
	qDebug() << summary->getR2();
	qDebug() << summary->getR2Adj();
	qDebug() << summary->getFStat();
	qDebug() << summary->getFStatPValue();
	qDebug() << summary->getDwStat();
	qDebug() << summary->getNobs();
	*/

	BOOST_REQUIRE_CLOSE(summary->getSSRes(), 7.21806e-31 , DOUBLE_PRECISION);
	BOOST_REQUIRE_CLOSE(summary->getMSRes(), 2.40602e-31, DOUBLE_PRECISION);
	BOOST_REQUIRE_CLOSE(summary->getMSResDerived(), 1.44361e-31, DOUBLE_PRECISION);
	BOOST_CHECK(summary->getR2() == 1);
	BOOST_CHECK(summary->getR2Adj() == 1);
	BOOST_REQUIRE_CLOSE(summary->getFStat(), 4.15624e+31, DOUBLE_PRECISION);
	BOOST_REQUIRE_CLOSE(summary->getFStatPValue(), 8.23038e-48, DOUBLE_PRECISION);
	BOOST_REQUIRE_CLOSE(summary->getDwStat(), 2.81861, DOUBLE_PRECISION);
	BOOST_REQUIRE_EQUAL(summary->getNobs(), 5);
}
