/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitAnalysisResidualTimeSeriesTests.cpp
 *
 *  Created on: Apr 3, 2014
 *      Author: gerberma
 */

#define BOOST_TEST_MODULE RegressionFitAnalysisResidualTimeSeriesTests

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>
#include <boost/bind.hpp>
#include <boost/foreach.hpp>
#include <string>
#include <Rcpp.h>
#include <QDebug>
#include <QString>
#include "RApplicationContext.hpp"
#include "RegressionFitApplicationContext.hpp"
#include "model/r/RRegressionFitAnalysisResidualTimeSeries.hpp"
#include "model/r/RegressionFitAnalysisResidualTimeSeriesProvider.hpp"
#include "applicationcontext/scopes.hpp"
#include "app/GenericApplicationContext.hpp"
#include "tests/helper/GenericApplicationContextTestHelper.hpp"

using namespace app::context;
using namespace test::helper;

typedef typename test::RegressionFitApplicationContext::RegressionFit_t RegressionFit_t;
typedef model::r::RRegressionFitAnalysisResidualTimeSeries ResidualTimeSeries_t;
typedef model::r::RegressionFitAnalysisResidualTimeSeriesProvider<ResidualTimeSeries_t, RegressionFit_t, typename test::RegressionFitApplicationContext::DataFrame_t> Provider_t;

struct ApplicationContext : public test::RApplicationContext {
	ApplicationContext() :
		GAC_DEFAULT_CTOR_INIT(Provider_t),
		DOUBLE_PRECISION(0.001)
	{
		fitCtx.init();
	}

	GAC_SCOPE(Provider_t);
	test::RegressionFitApplicationContext fitCtx;
	const double DOUBLE_PRECISION;
};

BOOST_FIXTURE_TEST_CASE(IntegrationTest, ApplicationContext) {

	LocalScope<ResidualTimeSeries_t> residualTimeSeries(FETCH_FROM_CTX(Provider_t)->load(
		fitCtx.regressionFit.get(),
		fitCtx.dataFrame.get()
	));

	/*
	qDebug() << residualTimeSeries->getResiduals()[0];
	*/

	BOOST_REQUIRE_EQUAL(residualTimeSeries->getResiduals().size(), 5);
	BOOST_REQUIRE_CLOSE(residualTimeSeries->getResiduals()[0], 1.46067, DOUBLE_PRECISION);
	BOOST_REQUIRE_EQUAL(residualTimeSeries->getDates()[0], "1983-08-11");
}
