/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitApplicationContext.hpp
 *
 *  Created on: Mar 28, 2014
 *      Author: gerberma
 */

#ifndef REGRESSIONFITAPPLICATIONCONTEXT_HPP_
#define REGRESSIONFITAPPLICATIONCONTEXT_HPP_

#include <boost/shared_ptr.hpp>
#include <vector>
#include <Rcpp.h>
#include "RegressionFitLoader.hpp"
#include "model/r/RegressionFit.hpp"
#include "model/r/RegressionFitProvider.hpp"
#include "model/r/IncreasingRegressionFormulaIndexManager.hpp"
#include "app/GenericApplicationContext.hpp"
#include "tests/helper/GenericApplicationContextTestHelper.hpp"

namespace test {

using namespace app::context;
using namespace test::helper;

namespace detail {
	typedef typename test::RegressionFitLoader::RegressionFit_t RegressionFit_t;
	typedef typename test::RegressionFitLoader::RegressorFactory_t RegressorFactory_t;
	typedef typename test::RegressionFitLoader::RegressorFormulaBuilderFactory_t RegressorFormulaBuilderFactory_t;
	typedef typename test::RegressionFitLoader::RegressorFormulaBuilderManager_t RegressorFormulaBuilderManager_t;
	typedef model::r::DataFrame DataFrame_t;
	typedef RegressionFit_t::RegressionFormula_t RegressionFormula_t;
	typedef model::r::IncreasingRegressionFormulaIndexManager IndexManager_t;
} //namespace detail

struct RegressionFitApplicationContext {
	typedef typename detail::RegressionFit_t::RegressionFormula_t::DataFrame_t DataFrame_t;
	typedef detail::RegressionFit_t RegressionFit_t;
	RegressionFitApplicationContext(){
		indexManager = boost::shared_ptr<detail::IndexManager_t>(new detail::IndexManager_t());
		regressorFactory = boost::shared_ptr<detail::RegressorFactory_t>(new detail::RegressorFactory_t());
		regressorFormulaBuilderFactory = boost::shared_ptr<detail::RegressorFormulaBuilderFactory_t>(new detail::RegressorFormulaBuilderFactory_t());
	}

	void init(Rcpp::DataFrame rDataFrame, QString formula) {
		dataFrame = boost::shared_ptr<detail::DataFrame_t>(RegressionFitLoader::createDataFrame(rDataFrame, 1, waterLevelColumns));
		regressorFormulaBuilderManager = boost::shared_ptr<detail::RegressorFormulaBuilderManager_t>(new detail::RegressorFormulaBuilderManager_t(regressorFormulaBuilderFactory.get(), NULL, dataFrame.get()));
		regressionFormula = boost::shared_ptr<detail::RegressionFormula_t>(test::RegressionFitLoader::createRegressionFormulaForWaterLevel(
				regressorFactory.get(),
				dataFrame.get(),
				regressorFormulaBuilderManager.get(),
				formula, indexManager.get())
		);
		regressionFit = boost::shared_ptr<detail::RegressionFit_t>(test::RegressionFitLoader::load(dataFrame->getDataFrame(), regressionFormula.get()));

		dataFrame->setDateColumn(0);
	}

	void init() {
		Rcpp::DataFrame rDataFrame = Rcpp::DataFrame::create(
					Rcpp::Named("date", Rcpp::CharacterVector::create("11.08.1983", "12.08.1983", "13.08.1983", "14.08.1983", "15.08.1983")),
					Rcpp::Named("y", Rcpp::NumericVector::create(1,2,3,4,5)),
					Rcpp::Named("x", Rcpp::NumericVector::create(2,4,6,8,10))
				);

		QString formula("x");
		waterLevelColumns.push_back(2);

		dataFrame = boost::shared_ptr<detail::DataFrame_t>(RegressionFitLoader::createDataFrame(rDataFrame, 1, waterLevelColumns));
		regressorFormulaBuilderManager = boost::shared_ptr<detail::RegressorFormulaBuilderManager_t>(new detail::RegressorFormulaBuilderManager_t(regressorFormulaBuilderFactory.get(), NULL, dataFrame.get()));
		regressionFormula = boost::shared_ptr<detail::RegressionFormula_t>(test::RegressionFitLoader::createRegressionFormulaForWaterLevel(
				regressorFactory.get(),
				dataFrame.get(),
				regressorFormulaBuilderManager.get(),
				formula, indexManager.get())
		);
		regressionFit = boost::shared_ptr<detail::RegressionFit_t>(test::RegressionFitLoader::load(dataFrame->getDataFrame(), regressionFormula.get()));

		dataFrame->setDateColumn(0);
	}


	boost::shared_ptr<detail::RegressionFit_t> regressionFit;
	boost::shared_ptr<detail::DataFrame_t> dataFrame;
	boost::shared_ptr<detail::IndexManager_t> indexManager;
	boost::shared_ptr<detail::RegressionFormula_t> regressionFormula;
	boost::shared_ptr<detail::RegressorFactory_t> regressorFactory;
	boost::shared_ptr<detail::RegressorFormulaBuilderFactory_t> regressorFormulaBuilderFactory;
	boost::shared_ptr<detail::RegressorFormulaBuilderManager_t> regressorFormulaBuilderManager;

	std::vector<int> waterLevelColumns;
};

} //namespace test


#endif /* REGRESSIONFITAPPLICATIONCONTEXT_HPP_ */
