/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitLoader.cpp
 *
 *  Created on: Mar 26, 2014
 *      Author: gerberma
 */

#include <boost/foreach.hpp>
#include <vector>
#include "tests/model/r/RegressionFitLoader.hpp"
#include "model/r/RegressionFitProvider.hpp"
#include "model/r/IncreasingRegressionFormulaIndexManager.hpp"
#include "applicationcontext/scopes.hpp"

namespace test {

RegressionFitLoader::RegressionFitLoader() {
	// TODO Auto-generated constructor stub

}

RegressionFitLoader::~RegressionFitLoader() {
	// TODO Auto-generated destructor stub
}

model::r::DataFrame* RegressionFitLoader::createDataFrame(
		Rcpp::DataFrame rDataFrame, int behaviorIndicatorColumn,
		std::vector<int> waterLevelColumns)
{
	model::r::DataFrame* dataFrame = new model::r::DataFrame(rDataFrame);
	dataFrame->assignBehaviorIndicatorColumn(behaviorIndicatorColumn);
	BOOST_FOREACH(int waterLevelColumn, waterLevelColumns) {
		dataFrame->assignWaterLevelColumn(waterLevelColumn);
	}

	return dataFrame;
}

RegressionFitLoader::RegressionFit_t::RegressionFormula_t* RegressionFitLoader::createRegressionFormulaForWaterLevel(RegressorFactory_t *regressorFactory, model::r::DataFrame* dataFrame, RegressorFormulaBuilderManager_t *regressorFormulaBuilderManager, QString formula, model::r::IncreasingRegressionFormulaIndexManager *indexManager)
{
	std::vector<typename RegressionFit_t::RegressionFormula_t::Method_Description_t *> methodDescriptions;
	RegressionFit_t::RegressionFormula_t *regressionFormula = new RegressionFit_t::RegressionFormula_t(methodDescriptions, regressorFactory, dataFrame, regressorFormulaBuilderManager, indexManager);
	regressionFormula->setBehaviorIndicatorColumn(*dataFrame->getBehaviorIndicatorColumns().begin());
	regressionFormula->addWaterLevelFormula(formula);

	return regressionFormula;
}

RegressionFitLoader::RegressionFit_t* RegressionFitLoader::load(Rcpp::DataFrame& dataFrame, typename RegressionFit_t::RegressionFormula_t *regressionFormula)
{
	using namespace app::context;

	LocalScope<model::r::RegressionFitProvider<typename RegressionFit_t::RegressionFormula_t> > regressionFitProvider(new model::r::RegressionFitProvider<typename RegressionFit_t::RegressionFormula_t>());

	return regressionFitProvider->load(dataFrame, regressionFormula);
}

} /* namespace test */
