/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitLoader.hpp
 *
 *  Created on: Mar 26, 2014
 *      Author: gerberma
 */

#ifndef REGRESSIONFITLOADER_HPP_
#define REGRESSIONFITLOADER_HPP_

#include <stdexcept>
#include <Rcpp.h>
#include "model/r/RegressionFit.hpp"
#include "model/r/RegressionFormula.hpp"
#include "model/r/DataFrame.hpp"
#include "model/r/formulabuilder/GenericRegressorFormulaBuilder.hpp"
#include "model/r/formulabuilder/IdRegressorFormulaBuilder.hpp"
#include "model/r/formulabuilder/PowerRegressorFormulaBuilder.hpp"
#include "model/r/formulabuilder/ChebyshevRegressorFormulaBuilder.hpp"
#include "model/r/formulabuilder/SinusRegressorFormulaBuilder.hpp"
#include "model/r/formulabuilder/CosinusRegressorFormulaBuilder.hpp"


namespace test {

/*
namespace detail {
struct NullStruct{};
struct RegressionFormula {
	typedef NullStruct Regressor_t;
	RegressionFormula(Rcpp::Formula formula) : formula(formula){;}

	Rcpp::Formula buildFormula(){
		return formula;
	}

	Rcpp::Formula formula;
};
} //
*/

namespace detail {

class RegressorFormulaBuilderFactory {
public:
	typedef model::r::RegressorFormulaBuilder value_type;
	typedef model::r::GenericRegressorFormulaBuilder<> GenericRegressorFormulaBuilder_t;

	template<typename T, typename InfluenceFactor>
	T *createForId(QString regressorFormula, InfluenceFactor *influenceFactor) {
		return new model::r::IdRegressorFormulaBuilder<GenericRegressorFormulaBuilder_t>(new GenericRegressorFormulaBuilder_t(regressorFormula, influenceFactor));
		//throw std::logic_error("not implemented yet");
	}

	template<typename T, typename InfluenceFactor>
	T *createForPower(QString regressorFormula, InfluenceFactor *influenceFactor, QString power) {
		return new model::r::PowerRegressorFormulaBuilder<GenericRegressorFormulaBuilder_t>(new GenericRegressorFormulaBuilder_t(regressorFormula, influenceFactor), power);
	}

	template<typename T, typename InfluenceFactor>
	T *createForChebyshev(QString regressorFormula, InfluenceFactor *influenceFactor, QString level) {
		return new model::r::ChebyshevRegressorFormulaBuilder<GenericRegressorFormulaBuilder_t>(new GenericRegressorFormulaBuilder_t(regressorFormula, influenceFactor), level);
	}

	template<typename T, typename DateColumn>
	T *createForSinus(QString regressorFormula, DateColumn *dateColumn, QString factor) {
		return new model::r::SinusRegressorFormulaBuilder<GenericRegressorFormulaBuilder_t>(new GenericRegressorFormulaBuilder_t(regressorFormula, dateColumn), factor);
	}

	template<typename T, typename DateColumn>
	T *createForCosinus(QString regressorFormula, DateColumn *dateColumn, QString factor) {
		return new model::r::CosinusRegressorFormulaBuilder<GenericRegressorFormulaBuilder_t>(new GenericRegressorFormulaBuilder_t(regressorFormula, dateColumn), factor);
	}
};

class RegressorFactory {
public:
	template<typename _Regressor>
	_Regressor *create(int index) {
		return new _Regressor(index);
	}
};

} // namespace detail


class RegressionFitLoader {
public:
	typedef detail::RegressorFactory RegressorFactory_t;
	typedef detail::RegressorFormulaBuilderFactory RegressorFormulaBuilderFactory_t;
	typedef model::r::RegressorFormulaBuilderManager<RegressorFormulaBuilderFactory_t> RegressorFormulaBuilderManager_t;
	typedef model::r::RegressionFit<
				model::r::RegressionFormula<RegressorFormulaBuilderManager_t, RegressorFactory_t>
			> RegressionFit_t;
public:
	RegressionFitLoader();
	virtual ~RegressionFitLoader();

	static model::r::DataFrame *createDataFrame(Rcpp::DataFrame dataFrame, int behaviorIndicatorColumn, std::vector<int> waterLevelColumns);

	static RegressionFit_t::RegressionFormula_t *createRegressionFormulaForWaterLevel(RegressorFactory_t *regressorFactory, model::r::DataFrame* dataFrame, RegressorFormulaBuilderManager_t *regressorFormulaBuilderManager, QString formula, model::r::IncreasingRegressionFormulaIndexManager *indexManager);

	static RegressionFit_t *load(Rcpp::DataFrame &dataFrame, typename RegressionFit_t::RegressionFormula_t *regressionFormula);
};

} /* namespace test */
#endif /* REGRESSIONFITLOADER_HPP_ */
