/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitProviderTests.cpp
 *
 *  Created on: Feb 19, 2014
 *      Author: gerberma
 */

#define BOOST_TEST_MODULE RRegressionFitProviderTests

#include <boost/test/unit_test.hpp>
#include <boost/bind.hpp>
#include <boost/foreach.hpp>
#include <Rcpp.h>
#include <QDebug>
#include <QString>
#include "RApplicationContext.hpp"
#include "model/r/RegressionFitProvider.hpp"
#include "applicationcontext/scopes.hpp"
#include "app/GenericApplicationContext.hpp"
#include "tests/helper/GenericApplicationContextTestHelper.hpp"

using namespace app::context;
using namespace test::helper;

namespace test {
struct NullStruct {};

struct RegressionFormula {
	typedef NullStruct Regressor_t;
	RegressionFormula(){;}

	std::string buildFormula() {
		return formula;
	}

	std::string formula;
};
}

typedef test::RegressionFormula Regression_Formula_t;
typedef model::r::RegressionFit<Regression_Formula_t> Regression_Fit_t;
typedef model::r::RegressionFitProvider<Regression_Formula_t> Regression_Fit_Provider_t;

class ApplicationContext : public test::RApplicationContext {
public:
	ApplicationContext() :
		GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(Regression_Fit_Provider_t, boost::bind(&ApplicationContext::regressionFitProviderFactory, this)),
		GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(Regression_Formula_t, boost::bind(&ApplicationContext::regressionFormulaFactory, this))
	{;}

private:
	Regression_Formula_t *regressionFormulaFactory() {
		return new Regression_Formula_t;
	}

	Regression_Fit_Provider_t *regressionFitProviderFactory() {
		Regression_Fit_Provider_t *regressionFitProvider = new Regression_Fit_Provider_t;

		return regressionFitProvider;
	}

private:
	GAC_SCOPE(Regression_Fit_Provider_t)
	GAC_SCOPE(Regression_Formula_t)
};



BOOST_FIXTURE_TEST_CASE(RoundTripTestCaseShowsThatTheSetupWorks, ApplicationContext) {
	Rcpp::DataFrame df = Rcpp::DataFrame::create(
			Rcpp::Named("x", Rcpp::NumericVector::create(1,2,3,4)),
			Rcpp::Named("y", Rcpp::NumericVector::create(1,4,9,16))
	);


	FETCH_FROM_CTX(Regression_Formula_t)->formula = "y ~ x";

	app::context::LocalScope<Regression_Fit_t> regressionFit(FETCH_FROM_CTX(Regression_Fit_Provider_t)->load(df, FETCH_FROM_CTX(Regression_Formula_t)));

	BOOST_CHECK(regressionFit->getIntercept() == -5);

}
