/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionFitTests.cpp
 *
 *  Created on: Feb 19, 2014
 *      Author: gerberma
 */

#define BOOST_TEST_MODULE RRegressionFitTests

#include <boost/test/unit_test.hpp>
#include <turtle/mock.hpp>
#include <stdexcept>
#include <Rcpp.h>
#include <QString>
#include "model/r/RegressionFit.hpp"
#include "RApplicationContext.hpp"
#include "app/GenericApplicationContext.hpp"
#include "tests/helper/GenericApplicationContextTestHelper.hpp"

using namespace app::context;
using namespace test::helper;

MOCK_CLASS(Column) {
	MOCK_METHOD_EXT(getName, 0, QString(), getName);
};

MOCK_CLASS(Regressor) {
	MOCK_METHOD_EXT(getFormula, 0, QString(), getFormula);
	MOCK_METHOD_EXT(getColumn, 0, Column*(), getColumn);
};

MOCK_CLASS(RegressionFormula) {
	typedef Regressor Regressor_t;
	MOCK_METHOD_EXT(getIndexOf, 1, int(Regressor *), getIndexOf);
};


typedef model::r::RManager R_Manager_t;
typedef model::r::RegressionFit<RegressionFormula> Regression_Fit_t;

class ApplicationContext : public test::RApplicationContext {
public:
	ApplicationContext() :
		GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(Regression_Fit_t, boost::bind(&ApplicationContext::regressionFitFactory, this))
	{;}

public:
	RegressionFormula regressionFormula;
	Regressor regressor;
	Column column;

private:
	Regression_Fit_t *regressionFitFactory() {
		Rcpp::List wrappedFit = Rcpp::List::create(
						Rcpp::Named("coefficients", Rcpp::NumericVector::create(1.0L,2.0L,3.0L))
					);

		return new Regression_Fit_t(wrappedFit, &regressionFormula);
	}

private:
	GAC_SCOPE(Regression_Fit_t)
};



BOOST_FIXTURE_TEST_CASE(TestRegressorExists, ApplicationContext) {
	MOCK_EXPECT(regressionFormula.getIndexOf).at_least(1).returns(0);

	BOOST_CHECK(FETCH_FROM_CTX(Regression_Fit_t)->getCoefficientFor(&regressor) == 2.0L);
}

BOOST_FIXTURE_TEST_CASE(TestRegressorDoesNotExistsThrowsExceptionWithDetailedInformation, ApplicationContext) {
	MOCK_EXPECT(regressionFormula.getIndexOf).at_least(1).returns(-1);
	MOCK_EXPECT(regressor.getFormula).at_least(1).returns(QString("formula"));
	MOCK_EXPECT(regressor.getColumn).at_least(1).returns(&column);
	MOCK_EXPECT(column.getName).at_least(1).returns(QString("column"));

	BOOST_CHECK_THROW(FETCH_FROM_CTX(Regression_Fit_t)->getCoefficientFor(&regressor), std::runtime_error);
}
