/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressionModelDefinitionTests.cpp
 *
 *  Created on: Aug 7, 2014
 *      Author: gerberma
 */

#define BOOST_TEST_MODULE RRegressionModelDefinitionTests

#include <boost/test/unit_test.hpp>
#include <turtle/mock.hpp>
#include <boost/bind.hpp>
#include <QString>
#include "model/r/RegressionModelDefinition.hpp"
#include "app/GenericApplicationContext.hpp"
#include "tests/helper/GenericApplicationContextTestHelper.hpp"

using namespace app::context;
using namespace test::helper;


namespace test {

MOCK_CLASS(Regressor) {
};

MOCK_CLASS(RegressorFormulaBuilder) {
	MOCK_METHOD_EXT(getIdentifier, 0, QString(), getIdentifier);
};

}  // namespace test

typedef model::r::RegressionModelDefinition<test::Regressor, test::RegressorFormulaBuilder> RegressionModelDefinition_t;

struct ApplicationContext {
	ApplicationContext() :
		GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(RegressionModelDefinition_t, boost::bind(&ApplicationContext::factory, this))
	{;}

	RegressionModelDefinition_t *factory() {
		return new RegressionModelDefinition_t("name");
	}

	test::Regressor regressor;
	test::RegressorFormulaBuilder regressorFormulaBuilder;
	GAC_SCOPE(RegressionModelDefinition_t);
};


BOOST_FIXTURE_TEST_CASE(TestFindRegressorFormulaBuilderByIdentifier, ApplicationContext) {
	RegressionModelDefinition_t *regressionModelDefinition = FETCH_FROM_CTX(RegressionModelDefinition_t);
	regressionModelDefinition->addRegressorFormulaBuilder(&regressorFormulaBuilder);

	MOCK_EXPECT(regressorFormulaBuilder.getIdentifier).returns("id");

	test::RegressorFormulaBuilder *foundBuilder = regressionModelDefinition->findRegressorFormulaBuilderByIdentifier("id");
	BOOST_CHECK(foundBuilder != NULL);
	BOOST_REQUIRE_EQUAL(foundBuilder->getIdentifier().toStdString(), regressorFormulaBuilder.getIdentifier().toStdString());

	BOOST_CHECK(regressionModelDefinition->findRegressorFormulaBuilderByIdentifier("unknown_id") == NULL);
}
