/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RegressorTests.cpp
 *
 *  Created on: Apr 9, 2014
 *      Author: gerberma
 */


#define BOOST_TEST_MODULE RegressorTests

#include <boost/test/unit_test.hpp>
#include <turtle/mock.hpp>
#include <boost/bind.hpp>
#include <boost/ref.hpp>
#include <boost/shared_ptr.hpp>
#include <stdexcept>
#include <QString>
#include "model/r/Regressor.hpp"
#include "communication/Signals.hpp"

/*
using namespace app::context;
*/
using namespace model::r;


namespace test {

MOCK_CLASS(AnyRegressorFormulaBuilder) {
	MOCK_METHOD_EXT(getRFormula, 0, QString(), getRFormula);
	MOCK_METHOD_EXT(getFormula, 0, QString(), getFormula);
	MOCK_METHOD_EXT(getInfluenceFactorName, 0, QString(), getInfluenceFactorName);
};

MOCK_CLASS(ChangeSignalReceiver) {
	void changeReceived() {
		delegateChangeReceived();
	}
	MOCK_METHOD_EXT(delegateChangeReceived, 0, void(), delegateChangeReceived);
};


struct SignalTestsApplicationContext {
	typedef Regressor<AnyRegressorFormulaBuilder, ::memory::IgnoreOwnershipPolicy<AnyRegressorFormulaBuilder> > regressor_t;
	SignalTestsApplicationContext()
		:formula("MSC1")
	{}

	test::AnyRegressorFormulaBuilder anyRegressorFormulaBuilder;
	test::ChangeSignalReceiver changeReceiver;
	QString formula;
};

}  // namespace test

BOOST_FIXTURE_TEST_CASE(TestSetRegressorFormulaBuilderFiresChangeSignal, test::SignalTestsApplicationContext) {
	using namespace boost;

	shared_ptr<regressor_t> regressor = shared_ptr<regressor_t>(new regressor_t(0, false));

	COM_SUBSCRIBE_FOR(regressor, Changed, (bind(&test::ChangeSignalReceiver::changeReceived, ref(changeReceiver))));
	MOCK_EXPECT(changeReceiver.delegateChangeReceived).once();
	regressor->setRegressorFormulaBuilder(&anyRegressorFormulaBuilder);
}


BOOST_FIXTURE_TEST_CASE(TestChangeIsActiveFiresChangeSignal, test::SignalTestsApplicationContext) {
	using namespace boost;

	shared_ptr<regressor_t> regressor = shared_ptr<regressor_t>(new regressor_t(0, false));
	COM_SUBSCRIBE_FOR(regressor, Changed, (bind(&test::ChangeSignalReceiver::changeReceived, ref(changeReceiver))));
	MOCK_EXPECT(changeReceiver.delegateChangeReceived).once();
	regressor->setIsActive(false);
}


BOOST_FIXTURE_TEST_CASE(TestThatEmptyStringsGetReturnedIfNoRegressorFormulaHasBeenDefined, test::SignalTestsApplicationContext) {
	using namespace boost;


	shared_ptr<regressor_t> regressor = shared_ptr<regressor_t>(new regressor_t(0, false));
	BOOST_REQUIRE_EQUAL("", regressor->getFormula().toStdString());
	BOOST_REQUIRE_EQUAL("", regressor->getRFormula().toStdString());
	BOOST_REQUIRE_EQUAL("", regressor->getInfluenceFactorName().toStdString());
	//BOOST_CHECK_THROW(regressor->getFormula(), std::exception);
	//BOOST_CHECK_THROW(regressor->getRFormula(), std::exception);
}
