/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * ReportEscapeFilterTests.cpp
 *
 *  Created on: Aug 4, 2014
 *      Author: gerberma
 */

#define BOOST_TEST_MODULE ReportEscapeFilterTests


#include <boost/test/unit_test.hpp>
#include <QFile>
#include "ReportContexts.hpp"

BOOST_AUTO_TEST_CASE(TestEscape) {
	QFile file;
	model::report::StreamReportContext ctx(&file, "foo");

	/*
	 * escape sequence for markdown: \`*_{}[]()>#+-.!
	 */
	BOOST_REQUIRE_EQUAL("\\\\", ctx.escape("\\").toStdString());
	BOOST_REQUIRE_EQUAL("\\`", ctx.escape("`").toStdString());
	BOOST_REQUIRE_EQUAL("\\*", ctx.escape("*").toStdString());
	BOOST_REQUIRE_EQUAL("\\_", ctx.escape("_").toStdString());
	BOOST_REQUIRE_EQUAL("\\{", ctx.escape("{").toStdString());
	BOOST_REQUIRE_EQUAL("\\}", ctx.escape("}").toStdString());
	BOOST_REQUIRE_EQUAL("\\[", ctx.escape("[").toStdString());
	BOOST_REQUIRE_EQUAL("\\]", ctx.escape("]").toStdString());
	BOOST_REQUIRE_EQUAL("\\(", ctx.escape("(").toStdString());
	BOOST_REQUIRE_EQUAL("\\)", ctx.escape(")").toStdString());
	BOOST_REQUIRE_EQUAL("\\>", ctx.escape(">").toStdString());
	BOOST_REQUIRE_EQUAL("\\#", ctx.escape("#").toStdString());
	BOOST_REQUIRE_EQUAL("\\+", ctx.escape("+").toStdString());
	BOOST_REQUIRE_EQUAL("\\-", ctx.escape("-").toStdString());
	BOOST_REQUIRE_EQUAL("\\.", ctx.escape(".").toStdString());
	BOOST_REQUIRE_EQUAL("\\!", ctx.escape("!").toStdString());
	BOOST_REQUIRE_EQUAL("\\^", ctx.escape("^").toStdString());
#ifdef _WIN32
	BOOST_REQUIRE_EQUAL("\n\n", ctx.escape("\n").toStdString());
#endif
}
