/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelAnalysisToolsTimeSeriesOverviewControllerSubscriptionManagerTests.cpp
 *
 *  Created on: Mar 12, 2014
 *      Author: gerberma
 */

#define BOOST_TEST_MODULE DataModelAnalysisToolsTimeSeriesOverviewControllerSubscriptionManagerTests

#include <boost/test/unit_test.hpp>
#include <turtle/mock.hpp>
#include <boost/bind.hpp>
#include <boost/any.hpp>
#include <QString>
#include "subscription/DataModelAnalysisToolsTimeSeriesOverviewControllerSubscriptionManager.hpp"
#include "app/GenericApplicationContext.hpp"
#include "tests/helper/GenericApplicationContextTestHelper.hpp"

using namespace app::context;
using namespace test::helper;

MOCK_CLASS(TimeSeriesOverview) {
};

struct ConfigurationUi {
	typedef communication::Signal<void(TimeSeriesOverview *, int)>  __ValueColumnChanged_signal_t;
	communication::Signal<void(TimeSeriesOverview *, int)>  ValueColumnChangedSignal;
	boost::signals2::connection subscribeForValueColumnChanged(const typename communication::Signal<void(TimeSeriesOverview *, int)>::slot_type &subscriber) {
		return ValueColumnChangedSignal.connect(subscriber);
	}
};

MOCK_CLASS(OverviewUi) {
	typedef ConfigurationUi configuration_widget_t;
};

MOCK_CLASS(Controller) {
	void valueColumnChanged(TimeSeriesOverview *t, int index) {
		delegateValueColumnChanged(t, index);
	}
	MOCK_METHOD_EXT(delegateValueColumnChanged, 2, void(TimeSeriesOverview *, int), delegateValueColumnChanged);
};


typedef subscription::DataModelAnalysisToolsTimeSeriesOverviewControllerSubscriptionManager<OverviewUi> SubscriptionManager_t;

struct ApplicationContext {
	ApplicationContext():
		GAC_DEFAULT_CTOR_INIT(SubscriptionManager_t)
	{;}

	TimeSeriesOverview timeSeriesOverview;
	ConfigurationUi configurationUi;
	OverviewUi overviewUi;
	Controller controller;
	GAC_SCOPE(SubscriptionManager_t)
};

BOOST_FIXTURE_TEST_CASE(TestSubscribeConfigurationUi, ApplicationContext) {
	MOCK_EXPECT(controller.delegateValueColumnChanged).once().with(&timeSeriesOverview, 1);
	FETCH_FROM_CTX(SubscriptionManager_t)->subscribe(&controller, &configurationUi);

	configurationUi.ValueColumnChangedSignal(&timeSeriesOverview, 1);
}




