/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
//#define BOOST_TEST_MAIN
#define BOOST_TEST_MODULE RegressionModelSubscriptionTests

#include <boost/test/unit_test.hpp>
#include <turtle/mock.hpp>
#include <boost/bind.hpp>
#include <boost/any.hpp>
#include <QString>
#include "subscription/RegressionModelSubscriptionManager.hpp"
#include "app/GenericApplicationContext.hpp"
#include "tests/helper/GenericApplicationContextTestHelper.hpp"

using namespace app::context;
using namespace test::helper;

MOCK_CLASS(AnalysisToolDescription) {
};

MOCK_CLASS(Analysis) {
};

MOCK_CLASS(RegressionModel) {
};

MOCK_CLASS(RegressionModelDefinition) {
};

MOCK_CLASS(RegressorFormulaBuilder) {
};

MOCK_CLASS(MethodDescription) {
};

MOCK_CLASS(AnalysisToolWidget) {
};

MOCK_CLASS(CategoryWidget) {
	COM_SUBSCRIPTION_FUNCTION(AddAnalysisTool, communication::Signal<void(AnalysisToolDescription *, boost::function<void(AnalysisToolWidget *)>)>);
	void addAnalysisTool(AnalysisToolWidget *widget) {

	}
	MOCK_METHOD_EXT(delegateAddAnalysisTool, 1, void(AnalysisToolWidget *), delegateAddAnalysisTool);
};

MOCK_CLASS(OverviewUi) {
};

MOCK_CLASS(InfluenceFactorSelectionUi) {
};

MOCK_CLASS(RegressionFormulaUi) {
};

MOCK_CLASS(RegressionExecuteUi) {
};

MOCK_CLASS(Wizard) {
	COM_SUBSCRIPTION_FUNCTION(SetName, communication::Signal<void(RegressionModel *, QString)>);
	COM_SUBSCRIPTION_FUNCTION(SetDescription, communication::Signal<void(RegressionModel *, QString)>);
	COM_SUBSCRIPTION_FUNCTION(FromDateChanged, communication::IndexChangedSignal<RegressionModel *>);
	COM_SUBSCRIPTION_FUNCTION(ToDateChanged, communication::IndexChangedSignal<RegressionModel *>);
	COM_SUBSCRIPTION_FUNCTION(BehaviorIndicatorChanged, communication::IndexChangedSignal<RegressionModel *>);
	//COM_SUBSCRIPTION_FUNCTION(SetWaterLevelRegressionFormula, communication::Signal<void(RegressionModel *, QString)>);
	COM_SUBSCRIPTION_FUNCTION(CreateWaterLevelRegressor, communication::Signal<void(RegressionModel *)>);
	COM_SUBSCRIPTION_FUNCTION(CreateTemperatureRegressor, communication::Signal<void(RegressionModel *)>);
	COM_SUBSCRIPTION_FUNCTION(WaterLevelRegressorFormulaEdited, communication::Signal<void(RegressionModel *, int, QString)>);
	COM_SUBSCRIPTION_FUNCTION(TemperatureRegressorFormulaEdited, communication::Signal<void(RegressionModel *, int, QString)>);
	COM_SUBSCRIPTION_FUNCTION(RemoveRegressor, communication::Signal<void(RegressionModel *, int)>);
	COM_SUBSCRIPTION_FUNCTION(AddRegressionFormulaMethodInstance, communication::Signal<void(RegressionModel *, MethodDescription *)>);
};

MOCK_CLASS(RegressionModelDefinitionEditUi) {
	typedef RegressionModelDefinition ModelDefinition_t;
	COM_SUBSCRIPTION_FUNCTION(AddRegressor, communication::Signal<void(ModelDefinition_t *, QString)>);
};

MOCK_CLASS(RegressorFormulaBuilderEditUi) {
	typedef RegressorFormulaBuilder RegressorFormulaBuilder_t;
	COM_SUBSCRIPTION_FUNCTION(SetIdentifier, communication::Signal<void(RegressorFormulaBuilder *, QString)>);
	COM_SUBSCRIPTION_FUNCTION(SetFormula, communication::Signal<void(RegressorFormulaBuilder *, QString)>);
	COM_SUBSCRIPTION_FUNCTION(SetAssignmentCategory, communication::Signal<void(RegressorFormulaBuilder *, QString)>);
	COM_SUBSCRIPTION_FUNCTION(SetFormulaDefinition, communication::Signal<void(RegressorFormulaBuilder *, QString)>);
};

MOCK_CLASS(Factory) {
};

MOCK_CLASS(Controller) {
	void setName(RegressionModel *rm, QString name) {delegateSetName(rm, name);}
	MOCK_METHOD_EXT(delegateSetName, 2, void(RegressionModel *, QString), delegateSetName);

	void setDescription(RegressionModel *rm, QString description) {delegateSetDescription(rm, description);}
	MOCK_METHOD_EXT(delegateSetDescription, 2, void(RegressionModel *, QString), delegateSetDescription);

	void fromDateChanged(RegressionModel *regressionModel, int dateColumnIndex) {delegateFromDateChanged(regressionModel, dateColumnIndex);}
	MOCK_METHOD_EXT(delegateFromDateChanged, 2, void(RegressionModel *, int), delegateFromDateChanged);

	void toDateChanged(RegressionModel *regressionModel, int dateColumnIndex) {delegateToDateChanged(regressionModel, dateColumnIndex);}
	MOCK_METHOD_EXT(delegateToDateChanged, 2, void(RegressionModel *, int), delegateToDateChanged);

	void behaviorIndicatorChanged(RegressionModel *regressionModel, int bIndex){delegateBehaviorIndicatorChanged(regressionModel, bIndex);}
	MOCK_METHOD_EXT(delegateBehaviorIndicatorChanged, 2, void(RegressionModel *, int), delegateBehaviorIndicatorChanged);

	/*
	void setWaterLevelFormula(RegressionModel *regressionModel, QString formula){delegateSetWaterLevelFormula(regressionModel, formula);}
	MOCK_METHOD_EXT(delegateSetWaterLevelFormula, 2, void(RegressionModel *, QString), delegateSetWaterLevelFormula);
	*/

	void createWaterLevelRegressor(RegressionModel *regressionModel){delegateCreateWaterLevelRegressor(regressionModel);}
	MOCK_METHOD_EXT(delegateCreateWaterLevelRegressor, 1, void(RegressionModel *), delegateCreateWaterLevelRegressor);

	void createTemperatureRegressor(RegressionModel *regressionModel){delegateCreateTemperatureRegressor(regressionModel);}
	MOCK_METHOD_EXT(delegateCreateTemperatureRegressor, 1, void(RegressionModel *), delegateCreateTemperatureRegressor);

	void waterLevelRegressorEditFinished(RegressionModel *regressionModel, int regressorIndex, QString formula){delegateWaterLevelRegressorEditFinished(regressionModel, regressorIndex, formula);}
	MOCK_METHOD_EXT(delegateWaterLevelRegressorEditFinished, 3, void(RegressionModel *, int, QString), delegateWaterLevelRegressorEditFinished);

	void temperatureRegressorEditFinished(RegressionModel *regressionModel, int regressorIndex, QString formula){delegateTemperatureRegressorEditFinished(regressionModel, regressorIndex, formula);}
	MOCK_METHOD_EXT(delegateTemperatureRegressorEditFinished, 3, void(RegressionModel *, int, QString), delegateTemperatureRegressorEditFinished);

	void removeRegressor(RegressionModel *regressionModel, int regressorIndex){delegateRemoveRegressor(regressionModel, regressorIndex);}
	MOCK_METHOD_EXT(delegateRemoveRegressor, 2, void(RegressionModel *, int), delegateRemoveRegressor);

	template<typename ModelDefinition, typename Factory, typename SubscriptionManager>
	void addRegressor(ModelDefinition *modelDefinition, QString formulaBuilderIdentifier, Factory *factory, SubscriptionManager *subscriptionManager) {
		delegateAddRegressor(modelDefinition, formulaBuilderIdentifier, subscriptionManager);
	}
	MOCK_METHOD_EXT(delegateAddRegressor, 3, void(RegressionModelDefinition *, QString, void *), delegateAddRegressor);
};

MOCK_CLASS(AnalysisToolsController) {
	typedef AnalysisToolWidget base_ui_t;
	typedef AnalysisToolDescription analysis_tool_description_t;

	void addAnalysisTool(Analysis *analysis, RegressionModel *regressionModel, AnalysisToolDescription *analysisToolDescription, boost::any f) {
		delegateAddAnalysisTool(analysis, regressionModel, analysisToolDescription, f);
	}

	MOCK_METHOD_EXT(delegateAddAnalysisTool, 4, void(Analysis *, RegressionModel *, AnalysisToolDescription *, boost::any), delegateAddAnalysisTool);
};

MOCK_CLASS(RegressionFormulaMethodsController) {
	void addMethodsInstance(RegressionModel *regressionModel, MethodDescription *methodDescription){
		delegateAddMethodsInstance(regressionModel, methodDescription);
	}

	MOCK_METHOD_EXT(delegateAddMethodsInstance, 2, void(RegressionModel *, MethodDescription *), delegateAddMethodsInstance);
};

typedef subscription::RegressionModelSubscriptionManager<
		OverviewUi,
		RegressionFormulaUi,
		RegressionExecuteUi,
		Wizard,
		InfluenceFactorSelectionUi,
		RegressionModelDefinitionEditUi,
		RegressorFormulaBuilderEditUi,
		Factory,
		Controller,
		AnalysisToolsController,
		RegressionFormulaMethodsController> RegressionModel_Subscription_Manager_t;


class ApplicationContext {
public:
	ApplicationContext() :
		GAC_PARTIAL_CTOR_INIT_WITH_FACTORY(RegressionModel_Subscription_Manager_t, boost::bind(&ApplicationContext::factoryMethod, this))
	{}

	RegressionModel_Subscription_Manager_t *factoryMethod() {
		return new RegressionModel_Subscription_Manager_t(&factory, &controller, &analysisToolsController, &methodsController);
	}

public:
	Factory factory;
	Controller controller;
	RegressionModel regressionModel;
	Wizard wizard;
	RegressionModelDefinitionEditUi regressionModelDefinitionEditUi;
	RegressorFormulaBuilderEditUi regressorFormulaBuilderEditUi;
	CategoryWidget anyCategoryWidget;
	Analysis analysis;
	AnalysisToolsController analysisToolsController;
	AnalysisToolDescription analysisToolDescription;
	AnalysisToolWidget analysisToolWidget;
	MethodDescription methodDescription;
	RegressionModelDefinition regressionModelDefinition;
	RegressionFormulaMethodsController methodsController;
private:
	GAC_SCOPE(RegressionModel_Subscription_Manager_t)
};


BOOST_FIXTURE_TEST_CASE(SubscribeWizardSetName, ApplicationContext)
{
	QString name("name");

	MOCK_EXPECT(controller.delegateSetName).once().with(&regressionModel, name);
	FETCH_FROM_CTX(RegressionModel_Subscription_Manager_t)->subscribe(&wizard);
	wizard.SetNameSignal(&regressionModel, name);
}

BOOST_FIXTURE_TEST_CASE(SubscribeWizardSetDescription, ApplicationContext)
{
	QString description("description");

	MOCK_EXPECT(controller.delegateSetDescription).once().with(&regressionModel, description);
	FETCH_FROM_CTX(RegressionModel_Subscription_Manager_t)->subscribe(&wizard);
	wizard.SetDescriptionSignal(&regressionModel, description);
}

BOOST_FIXTURE_TEST_CASE(SubscribeWizardFromDateChanged, ApplicationContext)
{
	MOCK_EXPECT(controller.delegateFromDateChanged).once().with(&regressionModel, 1);
	FETCH_FROM_CTX(RegressionModel_Subscription_Manager_t)->subscribe(&wizard);
	wizard.FromDateChangedSignal(&regressionModel, 1);
}

BOOST_FIXTURE_TEST_CASE(SubscribeWizardToDateChanged, ApplicationContext)
{
	MOCK_EXPECT(controller.delegateToDateChanged).once().with(&regressionModel, 1);
	FETCH_FROM_CTX(RegressionModel_Subscription_Manager_t)->subscribe(&wizard);
	wizard.ToDateChangedSignal(&regressionModel, 1);
}

BOOST_FIXTURE_TEST_CASE(SubscribeWizardBehaviorIndicatorChanged, ApplicationContext)
{
	MOCK_EXPECT(controller.delegateBehaviorIndicatorChanged).once().with(&regressionModel, 1);
	FETCH_FROM_CTX(RegressionModel_Subscription_Manager_t)->subscribe(&wizard);
	wizard.BehaviorIndicatorChangedSignal(&regressionModel, 1);
}

BOOST_FIXTURE_TEST_CASE(SubscribeDelegateToRegressionFormulaMethodController, ApplicationContext)
{
	MOCK_EXPECT(methodsController.delegateAddMethodsInstance).once().with(&regressionModel, &methodDescription);
	FETCH_FROM_CTX(RegressionModel_Subscription_Manager_t)->subscribe(&wizard);
	wizard.AddRegressionFormulaMethodInstanceSignal(&regressionModel, &methodDescription);
}

BOOST_FIXTURE_TEST_CASE(SubscribeDelegatesDelegatesToAnalysisToolsController, ApplicationContext)
{
	FETCH_FROM_CTX(RegressionModel_Subscription_Manager_t)->subscribeDelegates(&anyCategoryWidget, &analysis, &regressionModel);
}

BOOST_FIXTURE_TEST_CASE(SubscribeAddRegressor, ApplicationContext) {
	MOCK_EXPECT(controller.delegateAddRegressor).once().with(&regressionModelDefinition, "identifier", FETCH_FROM_CTX(RegressionModel_Subscription_Manager_t));
	FETCH_FROM_CTX(RegressionModel_Subscription_Manager_t)->subscribe(&regressionModelDefinitionEditUi);
	regressionModelDefinitionEditUi.AddRegressorSignal(&regressionModelDefinition, "identifier");
}
