/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * AnalysisUiManager.hpp
 *
 *  Created on: Nov 29, 2013
 *      Author: gerberma
 */

#ifndef ANALYSISUIMANAGER_HPP_
#define ANALYSISUIMANAGER_HPP_

#include <boost/shared_ptr.hpp>
#include "memory/OwnerhshipPolicies.hpp"
#include "UiManager.hpp"

namespace ui {


template<
	typename Analysis,
	typename OverviewUi,
	typename WizardUi,
	typename InfoDialogUi,
	class AnalysisUiFactory,
	template <class, class = memory::DeleteOnDeleteChildPolicy> class OwnershipPolicy
>
class AnalysisUiManager {
public:
	typedef OverviewUi overview_ui_type;
	typedef WizardUi wizard_ui_type;
	typedef InfoDialogUi info_dialog_ui_type;
public:
	AnalysisUiManager(AnalysisUiFactory *analysisUifactory);
	virtual ~AnalysisUiManager();

	OverviewUi *getOverviewUiFor(boost::shared_ptr<Analysis> analysis);
	void destroyOverviewUiFor(boost::shared_ptr<Analysis> analysis);

	WizardUi *getWizardFor(boost::shared_ptr<Analysis> analysis);
	void destroyWizardFor(boost::shared_ptr<Analysis> analysis);

	InfoDialogUi *getInfoDialogUiFor(boost::shared_ptr<Analysis> analysis) {
		return infoDialogUiManager.getFor(analysis, analysisUifactory);
	}

	void destroyInfoDialogUiFor(boost::shared_ptr<Analysis> analysis) {
		infoDialogUiManager.destroyFor(analysis);
	}

private:
	UiManager<Analysis, WizardUi, OwnershipPolicy<WizardUi> > wizardUiManager;
	UiManager<Analysis, OverviewUi, OwnershipPolicy<OverviewUi> > overviewUiManager;
	UiManager<Analysis, InfoDialogUi, OwnershipPolicy<InfoDialogUi> > infoDialogUiManager;
	AnalysisUiFactory *analysisUifactory;
};


template<typename Analysis, typename OverviewUi, typename WizardUi, typename InfoDialogUi,
		class AnalysisUiFactory,
		template<class, class> class OwnershipPolicy>
inline AnalysisUiManager<Analysis, OverviewUi, WizardUi, InfoDialogUi, AnalysisUiFactory, OwnershipPolicy>::AnalysisUiManager(AnalysisUiFactory *analysisUifactory)
	:analysisUifactory(analysisUifactory)
{
}

template<typename Analysis, typename OverviewUi, typename WizardUi, typename InfoDialogUi,
		class AnalysisUiFactory,
		template<class, class> class OwnershipPolicy>
inline AnalysisUiManager<Analysis, OverviewUi, WizardUi, InfoDialogUi, AnalysisUiFactory, OwnershipPolicy>::~AnalysisUiManager() {
}

template<typename Analysis, typename OverviewUi, typename WizardUi, typename InfoDialogUi,
		class AnalysisUiFactory,
		template<class, class> class OwnershipPolicy>
inline OverviewUi* AnalysisUiManager<Analysis, OverviewUi, WizardUi, InfoDialogUi, AnalysisUiFactory, OwnershipPolicy>::getOverviewUiFor(boost::shared_ptr<Analysis> analysis) {
	return overviewUiManager.getFor(analysis, analysisUifactory);
}


template<typename Analysis, typename OverviewUi, typename WizardUi, typename InfoDialogUi,
		class AnalysisUiFactory,
		template<class, class> class OwnershipPolicy>
inline void AnalysisUiManager<Analysis, OverviewUi, WizardUi, InfoDialogUi, AnalysisUiFactory, OwnershipPolicy>::destroyOverviewUiFor(boost::shared_ptr<Analysis> analysis) {
	overviewUiManager.destroyFor(analysis);
}


template<typename Analysis, typename OverviewUi, typename WizardUi, typename InfoDialogUi,
		class AnalysisUiFactory,
		template<class, class> class OwnershipPolicy>
inline WizardUi* AnalysisUiManager<Analysis, OverviewUi, WizardUi, InfoDialogUi, AnalysisUiFactory, OwnershipPolicy>::getWizardFor(boost::shared_ptr<Analysis> analysis) {
	return wizardUiManager.getFor(analysis, analysisUifactory);
}

template<typename Analysis, typename OverviewUi, typename WizardUi, typename InfoDialogUi,
		class AnalysisUiFactory,
		template<class, class> class OwnershipPolicy>
inline void AnalysisUiManager<Analysis, OverviewUi, WizardUi, InfoDialogUi, AnalysisUiFactory, OwnershipPolicy>::destroyWizardFor(boost::shared_ptr<Analysis> analysis) {
	wizardUiManager.destroyFor(analysis);
}

} /* namespace ui */


#endif /* ANALYSISUIMANAGER_HPP_ */
