/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelAnalysisTimeSeriesOverviewUiManager.hpp
 *
 *  Created on: Mar 11, 2014
 *      Author: gerberma
 */

#ifndef DATAMODELANALYSISTIMESERIESOVERVIEWUIMANAGER_HPP_
#define DATAMODELANALYSISTIMESERIESOVERVIEWUIMANAGER_HPP_

#include <boost/shared_ptr.hpp>
#include "UiManager.hpp"

namespace ui {

template
<
	typename T,
	typename OverviewUi,
	class UiFactory,
	template <class, class = memory::DeleteOnDeleteChildPolicy> class OwnershipPolicy
>
class DataModelAnalysisTimeSeriesOverviewUiManager {
public:
	typedef typename OverviewUi::base_ui_type base_ui_type;
	typedef OverviewUi overview_ui_t;

	DataModelAnalysisTimeSeriesOverviewUiManager(UiFactory *uiFactory):uiFactory(uiFactory){;}
	virtual ~DataModelAnalysisTimeSeriesOverviewUiManager(){;}

	OverviewUi *getOverviewUiFor(boost::shared_ptr<T> t) {
		bool newCreated = false;
		OverviewUi *overviewUi = overviewUiManager.getFor(t, uiFactory, newCreated);
		if(newCreated == false) {
			return overviewUi;
		}

		overviewUi->setConfigurationWidget(getConfigurationUiFor(t));

		overviewUi->init();

		return overviewUi;
	}

	typename OverviewUi::configuration_widget_t *getConfigurationUiFor(boost::shared_ptr<T> t) {
		return configurationUiManager.getFor(t, uiFactory);
	}

	void destroyOverviewUiFor(boost::shared_ptr<T> t) {
		overviewUiManager.destroyFor(t);
	}

private:
	UiFactory *uiFactory;
	UiManager<T, OverviewUi, OwnershipPolicy<OverviewUi> > overviewUiManager;
	UiManager<T, typename OverviewUi::configuration_widget_t, OwnershipPolicy<typename OverviewUi::configuration_widget_t, memory::IgnoreOnDeleteChildPolicy> > configurationUiManager;
};

}  // namespace ui


#endif /* DATAMODELANALYSISTIMESERIESOVERVIEWUIMANAGER_HPP_ */
