/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelAnalysisToolsUiManager.hpp
 *
 *  Created on: Jan 22, 2014
 *      Author: gerberma
 */

#ifndef DATAMODELANALYSISTOOLSUIMANAGER_HPP_
#define DATAMODELANALYSISTOOLSUIMANAGER_HPP_

#include <boost/shared_ptr.hpp>
#include "memory/OwnerhshipPolicies.hpp"
#include "UiManager.hpp"

namespace ui {

template<
	typename AnalysisTool,
	typename AnalysisToolUi,
	class UiFactory,
	template <class, class = memory::DeleteOnDeleteChildPolicy> class OwnershipPolicy
>
class DataModelAnalysisToolsUiManager {
public:
	typedef AnalysisToolUi analysis_tool_ui_type;
public:
	//TODO: we need to keep a reference to the DataModelUiManager to get the overview ui for a given DataModel
	DataModelAnalysisToolsUiManager(UiFactory *uiFactory);
	virtual ~DataModelAnalysisToolsUiManager();

	//TODO: also provide either the DataModel to lookup in the DataModelUiManager or the Analysis (from which we can get the DataModel internally)
	//Also TODO: provide an AnalysisToolUiFactory which is able to create a ui (of type gui::AnalysisToolWidget) based on the analysisToolDescription)
	//Instead of providing the dataModel, we can provide the analysis tool from which we can get the data model
	template<typename DataModel>
	AnalysisToolUi *getAnalysisToolUiFor(boost::shared_ptr<DataModel> dataModel, boost::shared_ptr<AnalysisTool> analysisTool);

	void addAnalysisToolUi(AnalysisToolUi *analysisToolUi);

private:
	UiFactory *uiFactory;
	UiManager<AnalysisTool, AnalysisToolUi, OwnershipPolicy<AnalysisToolUi> > analysisToolUiManager;
};

template<
	typename AnalysisTool,
	typename AnalysisToolUi,
	class UiFactory,
	template<class, class> class OwnershipPolicy
>
inline DataModelAnalysisToolsUiManager<AnalysisTool, AnalysisToolUi, UiFactory, OwnershipPolicy>::DataModelAnalysisToolsUiManager(UiFactory *uiFactory)
	:uiFactory(uiFactory)
{
}

template<
	typename AnalysisTool,
	typename AnalysisToolUi,
	class UiFactory,
	template<class, class> class OwnershipPolicy
>
inline DataModelAnalysisToolsUiManager<AnalysisTool, AnalysisToolUi, UiFactory, OwnershipPolicy>::~DataModelAnalysisToolsUiManager() {
}

template<
	typename AnalysisTool,
	typename AnalysisToolUi,
	class UiFactory,
	template<class, class> class OwnershipPolicy
>
template<typename DataModel>
inline AnalysisToolUi* DataModelAnalysisToolsUiManager<AnalysisTool, AnalysisToolUi, UiFactory, OwnershipPolicy>::getAnalysisToolUiFor(boost::shared_ptr<DataModel> dataModel, boost::shared_ptr<AnalysisTool> analysisTool) {
	return analysisToolUiManager.getFor(analysisTool, uiFactory);
}

template<
	typename AnalysisToolDescription,
	typename AnalysisToolUi,
	class UiFactory,
	template<class, class> class OwnershipPolicy
>
inline void DataModelAnalysisToolsUiManager<AnalysisToolDescription, AnalysisToolUi, UiFactory, OwnershipPolicy>::addAnalysisToolUi(AnalysisToolUi* analysisToolUi) {
}

} /* namespace ui */

#endif /* DATAMODELANALYSISTOOLSUIMANAGER_HPP_ */
