/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelOverviewStatisticsUiManager.hpp
 *
 *  Created on: Jan 31, 2014
 *      Author: gerberma
 */

#ifndef DATAMODELOVERVIEWSTATISTICSUIMANAGER_HPP_
#define DATAMODELOVERVIEWSTATISTICSUIMANAGER_HPP_

#include <boost/shared_ptr.hpp>
#include "memory/OwnerhshipPolicies.hpp"
#include "UiManager.hpp"

namespace ui {

template<
	typename OverviewStatistics,
	typename OverviewUi,
	class UiFactory,
	template <class, class = memory::DeleteOnDeleteChildPolicy> class OwnershipPolicy
>
class DataModelOverviewStatisticsUiManager {
public:
	typedef OverviewUi overview_ui_type;
	typedef typename OverviewUi::base_ui_type base_ui_type;

public:
	DataModelOverviewStatisticsUiManager(UiFactory *uiFactory)
		:uiFactory(uiFactory)
	{
	}
	virtual ~DataModelOverviewStatisticsUiManager() {}

	OverviewUi *getOverviewUiFor(boost::shared_ptr<OverviewStatistics> overviewStatistics) {
		return overviewUiManager.getFor(overviewStatistics, uiFactory);
	}

	void destroyOverviewUiFor(boost::shared_ptr<OverviewStatistics> overviewStatistics) {
		overviewUiManager.destroyFor(overviewStatistics);
	}

private:
	UiFactory *uiFactory;
	UiManager<OverviewStatistics, OverviewUi, OwnershipPolicy<OverviewUi> > overviewUiManager;
};

} /* namespace ui */


#endif /* DATAMODELOVERVIEWSTATISTICSUIMANAGER_HPP_ */
