/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * DataModelUiManager.hpp
 *
 *  Created on: Dec 11, 2013
 *      Author: gerberma
 */

#ifndef DATAMODELUIMANAGER_HPP_
#define DATAMODELUIMANAGER_HPP_

#include <boost/shared_ptr.hpp>
#include "memory/OwnerhshipPolicies.hpp"
#include "UiManager.hpp"

namespace ui {

template<
	typename DataModel,
	typename OverviewUi,
	typename WizardUi,
	typename UriDialogUi,
	typename WorkingAreaContainerUi,
	class UiFactory,
	template <class, class = memory::DeleteOnDeleteChildPolicy> class OwnershipPolicy
>
class DataModelUiManager {
public:
	typedef OverviewUi overview_ui_type;
	typedef WizardUi wizard_ui_type;
	typedef UriDialogUi uri_dialog_ui_type;
	typedef WorkingAreaContainerUi working_are_ui_type;

public:
	DataModelUiManager(UiFactory *uiFactory, WorkingAreaContainerUi *workingAreaContainerUi);
	virtual ~DataModelUiManager();

	WizardUi *getWizardFor(boost::shared_ptr<DataModel> sdataModel);
	void destroyWizardFor(boost::shared_ptr<DataModel> dataModel);

	UriDialogUi *getUriDialogUiFor(boost::shared_ptr<DataModel> dataModel);
	void destroyUriDialogUiFor(boost::shared_ptr<DataModel> dataModel);

	OverviewUi *getOverviewUiFor(boost::shared_ptr<DataModel> dataModel, bool &newCreated);
	void destroyOverviewUiFor(boost::shared_ptr<DataModel> dataModel);

	typename OverviewUi::OverviewCategoryWidget_t *getLeftSideOverviewCategoryWidgetFor(boost::shared_ptr<DataModel> dataModel) {
		return overviewUiManager.getFor(dataModel, uiFactory)->getLeftSideOverviewCategoryWidget();
	}

	typename OverviewUi::OverviewCategoryWidget_t *getRightSideOverviewCategoryWidgetFor(boost::shared_ptr<DataModel> dataModel) {
		return overviewUiManager.getFor(dataModel, uiFactory)->getRightSideOverviewCategoryWidget();
	}

	WorkingAreaContainerUi *getWorkingAreaContainerUi();

private:
	UiFactory *uiFactory;
	UiManager<DataModel, OverviewUi, OwnershipPolicy<OverviewUi> > overviewUiManager;
	UiManager<DataModel, WizardUi, OwnershipPolicy<WizardUi> > wizardUiManager;
	UiManager<DataModel, UriDialogUi, OwnershipPolicy<UriDialogUi> > uriDialogUiManager;
	WorkingAreaContainerUi *workingAreaContainerUi;
};



template<
	typename DataModel,
	typename OverviewUi,
	typename WizardUi,
	typename UriDialogUi,
	typename WorkingAreaContainerUi,
	class UiFactory,
	template<class, class> class OwnershipPolicy
>
inline DataModelUiManager<DataModel, OverviewUi, WizardUi, UriDialogUi, WorkingAreaContainerUi, UiFactory, OwnershipPolicy>::DataModelUiManager(UiFactory* uiFactory, WorkingAreaContainerUi *workingAreaContainerUi)
	:uiFactory(uiFactory), workingAreaContainerUi(workingAreaContainerUi)
{
}

template<
	typename DataModel,
	typename OverviewUi,
	typename WizardUi,
	typename UriDialogUi,
	typename WorkingAreaContainerUi,
	class UiFactory,
	template<class, class> class OwnershipPolicy
>
inline DataModelUiManager<DataModel, OverviewUi, WizardUi, UriDialogUi, WorkingAreaContainerUi, UiFactory, OwnershipPolicy>::~DataModelUiManager() {
}


template<
	typename DataModel,
	typename OverviewUi,
	typename WizardUi,
	typename UriDialogUi,
	typename WorkingAreaContainerUi,
	class UiFactory,
	template<class, class> class OwnershipPolicy
>
inline OverviewUi* DataModelUiManager<DataModel, OverviewUi, WizardUi, UriDialogUi, WorkingAreaContainerUi, UiFactory, OwnershipPolicy>::getOverviewUiFor(boost::shared_ptr<DataModel> dataModel, bool &newCreated) {
	return overviewUiManager.getFor(dataModel, uiFactory, newCreated);
}

template<
	typename DataModel,
	typename OverviewUi,
	typename WizardUi,
	typename UriDialogUi,
	typename WorkingAreaContainerUi,
	class UiFactory,
	template<class, class> class OwnershipPolicy
>
inline void DataModelUiManager<DataModel, OverviewUi, WizardUi, UriDialogUi, WorkingAreaContainerUi, UiFactory, OwnershipPolicy>::destroyOverviewUiFor(boost::shared_ptr<DataModel> dataModel) {
	overviewUiManager.destroyFor(dataModel);
}


template<
	typename DataModel,
	typename OverviewUi,
	typename WizardUi,
	typename UriDialogUi,
	typename WorkingAreaContainerUi,
	class UiFactory,
	template<class, class> class OwnershipPolicy
>
inline WorkingAreaContainerUi *DataModelUiManager<DataModel, OverviewUi, WizardUi, UriDialogUi, WorkingAreaContainerUi, UiFactory, OwnershipPolicy>::getWorkingAreaContainerUi() {
	return workingAreaContainerUi;
}


template<
	typename DataModel,
	typename OverviewUi,
	typename WizardUi,
	typename UriDialogUi,
	typename WorkingAreaContainerUi,
	class UiFactory,
	template<class, class> class OwnershipPolicy
>
inline WizardUi* DataModelUiManager<DataModel, OverviewUi, WizardUi, UriDialogUi, WorkingAreaContainerUi, UiFactory, OwnershipPolicy>::getWizardFor(boost::shared_ptr<DataModel> dataModel) {
	return wizardUiManager.getFor(dataModel, uiFactory);
}

template<
	typename DataModel,
	typename OverviewUi,
	typename WizardUi,
	typename UriDialogUi,
	typename WorkingAreaContainerUi,
	class UiFactory,
	template<class, class> class OwnershipPolicy
>
inline void DataModelUiManager<DataModel, OverviewUi, WizardUi, UriDialogUi, WorkingAreaContainerUi, UiFactory, OwnershipPolicy>::destroyWizardFor(boost::shared_ptr<DataModel> dataModel) {
	wizardUiManager.destroyFor(dataModel);
}

template<
	typename DataModel,
	typename OverviewUi,
	typename WizardUi,
	typename UriDialogUi,
	typename WorkingAreaContainerUi,
	class UiFactory,
	template<class, class> class OwnershipPolicy
>
inline UriDialogUi* DataModelUiManager<DataModel, OverviewUi, WizardUi, UriDialogUi, WorkingAreaContainerUi, UiFactory, OwnershipPolicy>::getUriDialogUiFor(boost::shared_ptr<DataModel> dataModel) {
	return uriDialogUiManager.getFor(dataModel, uiFactory);
}

template<
	typename DataModel,
	typename OverviewUi,
	typename WizardUi,
	typename UriDialogUi,
	typename WorkingAreaContainerUi,
	class UiFactory,
	template<class, class> class OwnershipPolicy
>
inline void DataModelUiManager<DataModel, OverviewUi, WizardUi, UriDialogUi, WorkingAreaContainerUi, UiFactory, OwnershipPolicy>::destroyUriDialogUiFor(boost::shared_ptr<DataModel> dataModel) {
	uriDialogUiManager.destroyFor(dataModel);
}


} /* namespace ui */


#endif /* DATAMODELUIMANAGER_HPP_ */
