/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * GenericAnalysisToolUiManager.hpp
 *
 *  Created on: Mar 28, 2014
 *      Author: gerberma
 */

#ifndef GENERICANALYSISTOOLUIMANAGER_HPP_
#define GENERICANALYSISTOOLUIMANAGER_HPP_

#include <boost/shared_ptr.hpp>
#include <QDebug>

namespace ui {

template
<
	typename T,
	typename OverviewUi,
	class UiFactory,
	template <class, class = memory::DeleteOnDeleteChildPolicy> class OwnershipPolicy
>
class GenericAnalysisToolUiManager {
public:
	typedef OverviewUi overview_ui_t;
	typedef typename OverviewUi::base_ui_type base_ui_type;
public:
	GenericAnalysisToolUiManager(UiFactory *uiFactory):uiFactory(uiFactory){;}
	virtual ~GenericAnalysisToolUiManager(){;}

	OverviewUi *getOverviewUiFor(boost::shared_ptr<T> t) {
		qDebug() << "1: " << __FUNCTION__ ;
		bool newCreated = false;
		OverviewUi *overviewUi = overviewUiManager.getFor(t, uiFactory, newCreated);

		if(newCreated == false) {
			qDebug() << "2: " << __FUNCTION__ ;
			return overviewUi;
		}

		overviewUi->setTargetAnalysisToolWidget(getTargetAnalysisToolUiFor(t));

		overviewUi->init();

		return overviewUi;
	}

	typename OverviewUi::TargetAnalysisToolWidget_t *getTargetAnalysisToolUiFor(boost::shared_ptr<T> t) {
		return targetAnalysisToolWidgetUiManager.getFor(t, uiFactory);
	}

	void destroyOverviewUiFor(boost::shared_ptr<T> t) {
		qDebug() << __FUNCTION__ ;

		destroyTargetAnalysisToolUiFor(t);
		overviewUiManager.destroyFor(t);
	}

	void destroyTargetAnalysisToolUiFor(boost::shared_ptr<T> t) {
		targetAnalysisToolWidgetUiManager.destroyFor(t);
	}

private:
	UiFactory *uiFactory;
	UiManager<T, OverviewUi, OwnershipPolicy<OverviewUi> > overviewUiManager;
	UiManager<T, typename OverviewUi::TargetAnalysisToolWidget_t, OwnershipPolicy<typename OverviewUi::TargetAnalysisToolWidget_t, memory::IgnoreOnDeleteChildPolicy> > targetAnalysisToolWidgetUiManager;
};

}  // namespace ui



#endif /* GENERICANALYSISTOOLUIMANAGER_HPP_ */
