/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * RawDataExportUiManager.hpp
 *
 *  Created on: 07.10.2014
 *      Author: gerberma
 */

#ifndef RAWDATAEXPORTUIMANAGER_HPP_
#define RAWDATAEXPORTUIMANAGER_HPP_

#include <boost/shared_ptr.hpp>
#include "memory/OwnerhshipPolicies.hpp"
#include "UiManager.hpp"

namespace ui {

template<
	typename RawPlotDataCollector,
	typename SaveDialogUi,
	typename UiFactory,
	template <class, class = memory::DeleteOnDeleteChildPolicy> class OwnershipPolicy
>
class RawDataExportUiManager {
public:
	typedef SaveDialogUi save_dialog_ui_t;
public:
	RawDataExportUiManager(UiFactory *uiFactory)
		:uiFactory(uiFactory)
	{;}

	SaveDialogUi *getSaveDialogUiFor(boost::shared_ptr<RawPlotDataCollector> rawPlotDataCollector) {
		return saveDialogUiManager.getFor(rawPlotDataCollector, uiFactory);
	}

	void destroySaveDialogUiFor(boost::shared_ptr<RawPlotDataCollector> rawPlotDataCollector) {
		saveDialogUiManager.destroyFor(rawPlotDataCollector);
	}

private:
	UiFactory *uiFactory;
	UiManager<RawPlotDataCollector, SaveDialogUi, OwnershipPolicy<SaveDialogUi> > saveDialogUiManager;
};

}  // namespace ui


#endif /* RAWDATAEXPORTUIMANAGER_HPP_ */
