/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * UiManager.hpp
 *
 *  Created on: Dec 11, 2013
 *      Author: gerberma
 */

#ifndef UIMANAGER_HPP_
#define UIMANAGER_HPP_

#include "UiManagerItem.hpp"
#include "UiManagerManagementItem.hpp"
#include <boost/bind.hpp>
#include <boost/function.hpp>
#include <boost/shared_ptr.hpp>

namespace ui {

namespace detail {
	template<typename Ui>
	void nullFunction(Ui *){;}
}

template<
	typename T,
	typename Ui,
	typename UiOwnershipPolicy
>
class UiManager {
public:

	template<typename UiFactory>
	Ui *getFor(boost::shared_ptr<T> t, UiFactory *uiFactory) {
		bool newUiCreated;
		return getFor(t, uiFactory, newUiCreated);
	}

	template<typename UiFactory>
	Ui *getFor(boost::shared_ptr<T> t, UiFactory *uiFactory, bool &newUiCreated) {
		return uiManagerManagement.getFor(t, boost::bind((Ui*(UiFactory::*)(boost::shared_ptr<T>))&UiFactory::create, uiFactory, _1), newUiCreated);
	}

	template<typename UiFactory, typename SubscriptionManager>
	Ui *getFor(boost::shared_ptr<T> t, UiFactory *uiFactory, SubscriptionManager *subscriptionManager) {
		bool newUiCreated = false;
		Ui *ui = uiManagerManagement.getFor(t, boost::bind((Ui*(UiFactory::*)(boost::shared_ptr<T>))&UiFactory::create, uiFactory, _1), newUiCreated);

		if(newUiCreated == true) {
			subscriptionManager->subscribe(ui);
		}

		return ui;
	}

	//NEW
	template<typename UiFactory>
	Ui *getFor(boost::shared_ptr<T> t, UiFactory *uiFactory, boost::function<void(Ui *)> subscriptionManager) {
		return getFor(t, boost::bind((Ui*(UiFactory::*)(boost::shared_ptr<T>))&UiFactory::create, uiFactory, _1), subscriptionManager);
	}

	Ui *getFor(boost::shared_ptr<T> t, boost::function<Ui*(boost::shared_ptr<T>)> uiFactory, boost::function<void(Ui *)> subscriptionManager = boost::bind(detail::nullFunction<Ui>,  _1)) {
		bool newUiCreated;
		return getFor(t, uiFactory, newUiCreated, subscriptionManager);
	}

	Ui *getFor(boost::shared_ptr<T> t, boost::function<Ui*(boost::shared_ptr<T>)> uiFactory, bool &newUiCreated, boost::function<void(Ui *)> subscriptionManager = boost::bind(detail::nullFunction<Ui>,  _1)) {
		Ui *ui = uiManagerManagement.getFor(t, uiFactory, newUiCreated);

		if(newUiCreated) {
			subscriptionManager(ui);
		}

		return ui;
	}

	bool containsFor(boost::shared_ptr<T> t) {
		return uiManagerManagement.containsFor(t);
	}

	void destroyFor(boost::shared_ptr<T> t) {
		uiManagerManagement.destroyFor(t);
	}

private:
	UiManagerManagementItem<UiManagerItem<T, Ui, UiOwnershipPolicy> > uiManagerManagement;
};

} /* namespace ui */

#endif /* UIMANAGER_HPP_ */
