/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * UiManagerItem.hpp
 *
 *  Created on: Dec 3, 2013
 *      Author: gerberma
 */

#ifndef UIMANAGERITEM_HPP_
#define UIMANAGERITEM_HPP_

#include <boost/shared_ptr.hpp>
#include <map>

namespace ui {

template<
	typename T,
	typename Ui,
	typename OwnershipPolicy
>
class UiManagerItem : public OwnershipPolicy {
public:
	typedef T item_t;
	typedef Ui ui_t;
public:

	Ui *get(boost::shared_ptr<T> t);
	void add(boost::shared_ptr<T> t, Ui *ui);
	void destroyFor(boost::shared_ptr<T> t);

private:
	std::map<boost::shared_ptr<T>, Ui *> container;
};


template<typename T, typename Ui, typename OwnershipPolicy>
inline Ui* UiManagerItem<T, Ui, OwnershipPolicy>::get(boost::shared_ptr<T> t) {
	if(container.count(t) == 1) {
		Ui *ui = container[t];
		if(ui != NULL && OwnershipPolicy::containsChild(ui)) {
			return ui;
		}
	}

	return NULL;
}


template<typename T, typename Ui, typename OwnershipPolicy>
inline void UiManagerItem<T, Ui, OwnershipPolicy>::add(boost::shared_ptr<T> t, Ui* ui) {
	container[t] = ui;
}

template<typename T, typename Ui, typename OwnershipPolicy>
inline void UiManagerItem<T, Ui, OwnershipPolicy>::destroyFor(boost::shared_ptr<T> t) {
	if(container.count(t) > 0) {
		Ui *ui = container[t];
		container.erase(t);
		//todo: notify all subscribers that the child gets deleted!
		OwnershipPolicy::deleteChild(ui);
	}
}

} /* namespace ui */

#endif /* UIMANAGERITEM_HPP_ */
