/*
    DamBASE - Dam Behaviour Analysis Software Environment
    Copyright (C) 2015   VAW, ETH Zurich, Gerber M., Buehlmann M.,
    Vetsch D.F., 2015
    http://www.dambase.ethz.ch

    This file is part of DamBASE

    DamBASE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DamBASE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * UiManagerManagementItem.hpp
 *
 *  Created on: Dec 3, 2013
 *      Author: gerberma
 */

#ifndef UIMANAGERMANAGEMENTITEM_HPP_
#define UIMANAGERMANAGEMENTITEM_HPP_

#include <boost/shared_ptr.hpp>
#include <boost/function.hpp>

namespace ui {

template<
	typename UiManagerItem
>
class UiManagerManagementItem {
public:
	typedef typename UiManagerItem::item_t item_t;
	typedef typename UiManagerItem::ui_t ui_t;
	typedef boost::function<ui_t*(boost::shared_ptr<item_t>)> FactoryFunction;
public:
	bool containsFor(boost::shared_ptr<typename UiManagerItem::item_t> item);
	typename UiManagerItem::ui_t *getFor(boost::shared_ptr<typename UiManagerItem::item_t> item, FactoryFunction factory, bool &newUiCreated);
	void destroyFor(boost::shared_ptr<typename UiManagerItem::item_t> item);
private:
	UiManagerItem container;
};

template<typename UiManagerItem>
inline bool UiManagerManagementItem<UiManagerItem>::containsFor(boost::shared_ptr<typename UiManagerItem::item_t> item) {
	return container.get(item) != NULL;
}

template<typename UiManagerItem>
inline typename UiManagerItem::ui_t* ui::UiManagerManagementItem<UiManagerItem>::getFor(boost::shared_ptr<typename UiManagerItem::item_t> item, FactoryFunction factory, bool &newUiCreated) {
	ui_t *ui = container.get(item);
	if(ui != NULL) {
		newUiCreated = false;
		return ui;
	}

	ui = factory(item); //analysisUifactory->template createFor<WizardUi, Analysis>(analysis);

	container.overtakeOwnership(ui);//OwnershipPolicy::overtakeOwnership(ui);

	container.add(item, ui);

	newUiCreated = true;

	return ui;
}


template<typename UiManagerItem>
inline void UiManagerManagementItem<UiManagerItem>::destroyFor(boost::shared_ptr<typename UiManagerItem::item_t> item) {
	container.destroyFor(item);
}

} /* namespace ui */

#endif /* UIMANAGERMANAGEMENTITEM_HPP_ */
